/*
 * Copyright (C) Jan 2013 Mellanox Technologies Ltd. All rights reserved.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING in the main directory of this source tree, or the
 * OpenIB.org BSD license below:
 *
 *     Redistribution and use in source and binary forms, with or
 *     without modification, are permitted provided that the following
 *     conditions are met:
 *
 *      - Redistributions of source code must retain the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer.
 *
 *      - Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/***
         *** This file was generated at "2017-03-19 12:09:18"
         *** by:
         ***    > /mswg/release/eat_me/last_release/adabe_plugins/adb2c/adb2pack.py --input adb/connectib/connectib.adb --file-prefix connectib --prefix connectib_
         ***/
#include "connectib_layouts.h"

void connectib_cmdif_ctx_special_qp_commands_uapp_pack(const struct connectib_cmdif_ctx_special_qp_commands_uapp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->uapp_qp_buf_id);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sw_wq_mkn);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_nim_indicies_allocated);

}

void connectib_cmdif_ctx_special_qp_commands_uapp_unpack(struct connectib_cmdif_ctx_special_qp_commands_uapp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->uapp_qp_buf_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 32;
	ptr_struct->sw_wq_mkn = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->num_nim_indicies_allocated = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_qp_commands_uapp_print(const struct connectib_cmdif_ctx_special_qp_commands_uapp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_qp_commands_uapp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uapp_qp_buf_id       : " UH_FMT "\n", ptr_struct->uapp_qp_buf_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_wq_mkn            : " U32H_FMT "\n", ptr_struct->sw_wq_mkn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_nim_indicies_allocated : " U32H_FMT "\n", ptr_struct->num_nim_indicies_allocated);

}

int connectib_cmdif_ctx_special_qp_commands_uapp_size(void)
{
	 return 12;
}

void connectib_cmdif_ctx_special_qp_commands_uapp_dump(const struct connectib_cmdif_ctx_special_qp_commands_uapp *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_qp_commands_uapp_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_qp_commands_underlay_pack(const struct connectib_cmdif_ctx_special_qp_commands_underlay *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ste_ix);

}

void connectib_cmdif_ctx_special_qp_commands_underlay_unpack(struct connectib_cmdif_ctx_special_qp_commands_underlay *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_qp_commands_underlay_print(const struct connectib_cmdif_ctx_special_qp_commands_underlay *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_qp_commands_underlay ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ste_ix               : " U32H_FMT "\n", ptr_struct->ste_ix);

}

int connectib_cmdif_ctx_special_qp_commands_underlay_size(void)
{
	 return 12;
}

void connectib_cmdif_ctx_special_qp_commands_underlay_dump(const struct connectib_cmdif_ctx_special_qp_commands_underlay *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_qp_commands_underlay_print(ptr_struct, file, 0);
}

void connectib_regular_qp_eth_pack(const struct connectib_regular_qp_eth *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->rq_head_qpn);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->lag_tx_port_affinity);

}

void connectib_regular_qp_eth_unpack(struct connectib_regular_qp_eth *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->rq_head_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->lag_tx_port_affinity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_regular_qp_eth_print(const struct connectib_regular_qp_eth *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_regular_qp_eth ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rq_head_qpn          : " UH_FMT "\n", ptr_struct->rq_head_qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lag_tx_port_affinity : " UH_FMT "\n", ptr_struct->lag_tx_port_affinity);

}

int connectib_regular_qp_eth_size(void)
{
	 return 4;
}

void connectib_regular_qp_eth_dump(const struct connectib_regular_qp_eth *ptr_struct, FILE* file)
{
	connectib_regular_qp_eth_print(ptr_struct, file, 0);
}

void connectib_regular_qp_ipoib_pack(const struct connectib_regular_qp_ipoib *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->underlay_qp_ste_ix);

}

void connectib_regular_qp_ipoib_unpack(struct connectib_regular_qp_ipoib *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->underlay_qp_ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_regular_qp_ipoib_print(const struct connectib_regular_qp_ipoib *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_regular_qp_ipoib ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "underlay_qp_ste_ix   : " U32H_FMT "\n", ptr_struct->underlay_qp_ste_ix);

}

int connectib_regular_qp_ipoib_size(void)
{
	 return 4;
}

void connectib_regular_qp_ipoib_dump(const struct connectib_regular_qp_ipoib *ptr_struct, FILE* file)
{
	connectib_regular_qp_ipoib_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_qp_commands_inner_pack(const union connectib_cmdif_ctx_special_qp_commands_inner *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_cmdif_ctx_special_qp_commands_uapp_pack(&(ptr_struct->uapp), ptr_buff);
}

void connectib_cmdif_ctx_special_qp_commands_inner_unpack(union connectib_cmdif_ctx_special_qp_commands_inner *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_cmdif_ctx_special_qp_commands_uapp_unpack(&(ptr_struct->uapp), ptr_buff);
}

void connectib_cmdif_ctx_special_qp_commands_inner_print(const union connectib_cmdif_ctx_special_qp_commands_inner *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_qp_commands_inner ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "underlay_qp:\n");
	connectib_cmdif_ctx_special_qp_commands_underlay_print(&(ptr_struct->underlay_qp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uapp:\n");
	connectib_cmdif_ctx_special_qp_commands_uapp_print(&(ptr_struct->uapp), file, indent_level + 1);

}

int connectib_cmdif_ctx_special_qp_commands_inner_size(void)
{
	 return 12;
}

void connectib_cmdif_ctx_special_qp_commands_inner_dump(const union connectib_cmdif_ctx_special_qp_commands_inner *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_qp_commands_inner_print(ptr_struct, file, 0);
}

void connectib_pmlp_code_style_lane_pack(const struct connectib_pmlp_code_style_lane *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module_num);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->module_lane);

}

void connectib_pmlp_code_style_lane_unpack(struct connectib_pmlp_code_style_lane *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->module_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 14;
	ptr_struct->module_lane = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_pmlp_code_style_lane_print(const struct connectib_pmlp_code_style_lane *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmlp_code_style_lane ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_num           : " UH_FMT "\n", ptr_struct->module_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_lane          : " UH_FMT "\n", ptr_struct->module_lane);

}

int connectib_pmlp_code_style_lane_size(void)
{
	 return 4;
}

void connectib_pmlp_code_style_lane_dump(const struct connectib_pmlp_code_style_lane *ptr_struct, FILE* file)
{
	connectib_pmlp_code_style_lane_print(ptr_struct, file, 0);
}

void connectib_dw_pack(const struct connectib_dw *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dw);

}

void connectib_dw_unpack(struct connectib_dw *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->dw = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_dw_print(const struct connectib_dw *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dw ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dw                   : " U32H_FMT "\n", ptr_struct->dw);

}

int connectib_dw_size(void)
{
	 return 4;
}

void connectib_dw_dump(const struct connectib_dw *ptr_struct, FILE* file)
{
	connectib_dw_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_type_nvlog_pack(const struct connectib_nv_tlv_type_nvlog *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->parameter_index);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sub_class);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->class_);

}

void connectib_nv_tlv_type_nvlog_unpack(struct connectib_nv_tlv_type_nvlog *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->parameter_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 8;
	ptr_struct->sub_class = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->class_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_nv_tlv_type_nvlog_print(const struct connectib_nv_tlv_type_nvlog *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_type_nvlog ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parameter_index      : " UH_FMT "\n", ptr_struct->parameter_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sub_class            : " UH_FMT "\n", ptr_struct->sub_class);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_               : " UH_FMT "\n", ptr_struct->class_);

}

int connectib_nv_tlv_type_nvlog_size(void)
{
	 return 4;
}

void connectib_nv_tlv_type_nvlog_dump(const struct connectib_nv_tlv_type_nvlog *ptr_struct, FILE* file)
{
	connectib_nv_tlv_type_nvlog_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_type_eswitch_pack(const struct connectib_nv_tlv_type_eswitch *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->parameter_index);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->port);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->host);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->class_);

}

void connectib_nv_tlv_type_eswitch_unpack(struct connectib_nv_tlv_type_eswitch *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->parameter_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 14;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 8;
	ptr_struct->host = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 0;
	ptr_struct->class_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_nv_tlv_type_eswitch_print(const struct connectib_nv_tlv_type_eswitch *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_type_eswitch ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parameter_index      : " UH_FMT "\n", ptr_struct->parameter_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host                 : " UH_FMT "\n", ptr_struct->host);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_               : " UH_FMT "\n", ptr_struct->class_);

}

int connectib_nv_tlv_type_eswitch_size(void)
{
	 return 4;
}

void connectib_nv_tlv_type_eswitch_dump(const struct connectib_nv_tlv_type_eswitch *ptr_struct, FILE* file)
{
	connectib_nv_tlv_type_eswitch_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_type_per_host_pack(const struct connectib_nv_tlv_type_per_host *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->parameter_index);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->function);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->host);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->class_);

}

void connectib_nv_tlv_type_per_host_unpack(struct connectib_nv_tlv_type_per_host *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	ptr_struct->parameter_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 14;
	ptr_struct->function = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->host = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 0;
	ptr_struct->class_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_nv_tlv_type_per_host_print(const struct connectib_nv_tlv_type_per_host *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_type_per_host ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parameter_index      : " UH_FMT "\n", ptr_struct->parameter_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "function             : " UH_FMT "\n", ptr_struct->function);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host                 : " UH_FMT "\n", ptr_struct->host);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_               : " UH_FMT "\n", ptr_struct->class_);

}

int connectib_nv_tlv_type_per_host_size(void)
{
	 return 4;
}

void connectib_nv_tlv_type_per_host_dump(const struct connectib_nv_tlv_type_per_host *ptr_struct, FILE* file)
{
	connectib_nv_tlv_type_per_host_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_type_bmc_pack(const struct connectib_nv_tlv_type_bmc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 13, (u_int32_t)ptr_struct->parameter_index);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->channel);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->host);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->class_);

}

void connectib_nv_tlv_type_bmc_unpack(struct connectib_nv_tlv_type_bmc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 19;
	ptr_struct->parameter_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 13);

	offset = 14;
	ptr_struct->channel = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 8;
	ptr_struct->host = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 0;
	ptr_struct->class_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_nv_tlv_type_bmc_print(const struct connectib_nv_tlv_type_bmc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_type_bmc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parameter_index      : " UH_FMT "\n", ptr_struct->parameter_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "channel              : " UH_FMT "\n", ptr_struct->channel);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host                 : " UH_FMT "\n", ptr_struct->host);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_               : " UH_FMT "\n", ptr_struct->class_);

}

int connectib_nv_tlv_type_bmc_size(void)
{
	 return 4;
}

void connectib_nv_tlv_type_bmc_dump(const struct connectib_nv_tlv_type_bmc *ptr_struct, FILE* file)
{
	connectib_nv_tlv_type_bmc_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_type_physical_port_pack(const struct connectib_nv_tlv_type_physical_port *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->parameter_index);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->class_);

}

void connectib_nv_tlv_type_physical_port_unpack(struct connectib_nv_tlv_type_physical_port *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->parameter_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 8;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->class_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_nv_tlv_type_physical_port_print(const struct connectib_nv_tlv_type_physical_port *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_type_physical_port ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parameter_index      : " UH_FMT "\n", ptr_struct->parameter_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_               : " UH_FMT "\n", ptr_struct->class_);

}

int connectib_nv_tlv_type_physical_port_size(void)
{
	 return 4;
}

void connectib_nv_tlv_type_physical_port_dump(const struct connectib_nv_tlv_type_physical_port *ptr_struct, FILE* file)
{
	connectib_nv_tlv_type_physical_port_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_type_global_pack(const struct connectib_nv_tlv_type_global *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->parameter_index);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->class_);

}

void connectib_nv_tlv_type_global_unpack(struct connectib_nv_tlv_type_global *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->parameter_index = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->class_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_nv_tlv_type_global_print(const struct connectib_nv_tlv_type_global *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_type_global ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parameter_index      : " UH_FMT "\n", ptr_struct->parameter_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_               : " UH_FMT "\n", ptr_struct->class_);

}

int connectib_nv_tlv_type_global_size(void)
{
	 return 4;
}

void connectib_nv_tlv_type_global_dump(const struct connectib_nv_tlv_type_global *ptr_struct, FILE* file)
{
	connectib_nv_tlv_type_global_print(ptr_struct, file, 0);
}

void connectib_regular_qp_sub_type_pack(const union connectib_regular_qp_sub_type *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_regular_qp_eth_pack(&(ptr_struct->eth), ptr_buff);
}

void connectib_regular_qp_sub_type_unpack(union connectib_regular_qp_sub_type *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_regular_qp_eth_unpack(&(ptr_struct->eth), ptr_buff);
}

void connectib_regular_qp_sub_type_print(const union connectib_regular_qp_sub_type *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_regular_qp_sub_type ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ipoib:\n");
	connectib_regular_qp_ipoib_print(&(ptr_struct->ipoib), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth:\n");
	connectib_regular_qp_eth_print(&(ptr_struct->eth), file, indent_level + 1);

}

int connectib_regular_qp_sub_type_size(void)
{
	 return 4;
}

void connectib_regular_qp_sub_type_dump(const union connectib_regular_qp_sub_type *ptr_struct, FILE* file)
{
	connectib_regular_qp_sub_type_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_qp_commands_other_pack(const struct connectib_cmdif_ctx_special_qp_commands_other *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->next_sqn_for_4page_allocate);

	offset = 32;
	connectib_cmdif_ctx_special_qp_commands_inner_pack(&(ptr_struct->inner), ptr_buff + offset/8);

}

void connectib_cmdif_ctx_special_qp_commands_other_unpack(struct connectib_cmdif_ctx_special_qp_commands_other *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->next_sqn_for_4page_allocate = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	connectib_cmdif_ctx_special_qp_commands_inner_unpack(&(ptr_struct->inner), ptr_buff + offset/8);

}

void connectib_cmdif_ctx_special_qp_commands_other_print(const struct connectib_cmdif_ctx_special_qp_commands_other *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_qp_commands_other ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_sqn_for_4page_allocate : " U32H_FMT "\n", ptr_struct->next_sqn_for_4page_allocate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner:\n");
	connectib_cmdif_ctx_special_qp_commands_inner_print(&(ptr_struct->inner), file, indent_level + 1);

}

int connectib_cmdif_ctx_special_qp_commands_other_size(void)
{
	 return 16;
}

void connectib_cmdif_ctx_special_qp_commands_other_dump(const struct connectib_cmdif_ctx_special_qp_commands_other *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_qp_commands_other_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_qp_commands_rts2rts_pack(const struct connectib_cmdif_ctx_special_qp_commands_rts2rts *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->new_cqn_rcv);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->new_srqn_rmpn);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->srq_cre_type);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->qp_params);

}

void connectib_cmdif_ctx_special_qp_commands_rts2rts_unpack(struct connectib_cmdif_ctx_special_qp_commands_rts2rts *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->new_cqn_rcv = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 40;
	ptr_struct->new_srqn_rmpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 36;
	ptr_struct->srq_cre_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->qp_params = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_qp_commands_rts2rts_print(const struct connectib_cmdif_ctx_special_qp_commands_rts2rts *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_qp_commands_rts2rts ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "new_cqn_rcv          : " UH_FMT "\n", ptr_struct->new_cqn_rcv);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "new_srqn_rmpn        : " UH_FMT "\n", ptr_struct->new_srqn_rmpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_cre_type         : " UH_FMT "\n", ptr_struct->srq_cre_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_params            : " U32H_FMT "\n", ptr_struct->qp_params);

}

int connectib_cmdif_ctx_special_qp_commands_rts2rts_size(void)
{
	 return 16;
}

void connectib_cmdif_ctx_special_qp_commands_rts2rts_dump(const struct connectib_cmdif_ctx_special_qp_commands_rts2rts *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_qp_commands_rts2rts_print(ptr_struct, file, 0);
}

void connectib_pmlp_code_style_pack(const struct connectib_pmlp_code_style *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->width);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 288, 1);
	connectib_pmlp_code_style_lane_pack(&(ptr_struct->local_lane[i]), ptr_buff + offset/8);
	}

}

void connectib_pmlp_code_style_unpack(struct connectib_pmlp_code_style *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->width = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 288, 1);
	connectib_pmlp_code_style_lane_unpack(&(ptr_struct->local_lane[i]), ptr_buff + offset/8);
	}

}

void connectib_pmlp_code_style_print(const struct connectib_pmlp_code_style *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmlp_code_style ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "width                : " UH_FMT "\n", ptr_struct->width);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_lane_%03d:\n", i);
	connectib_pmlp_code_style_lane_print(&(ptr_struct->local_lane[i]), file, indent_level + 1);
	}

}

int connectib_pmlp_code_style_size(void)
{
	 return 36;
}

void connectib_pmlp_code_style_dump(const struct connectib_pmlp_code_style *ptr_struct, FILE* file)
{
	connectib_pmlp_code_style_print(ptr_struct, file, 0);
}

void connectib_pmlp_prm_style_pack(const struct connectib_pmlp_prm_style *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->width);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module0);

	offset = 46;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lane0);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module1);

	offset = 78;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lane1);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module2);

	offset = 110;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lane2);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module3);

	offset = 142;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lane3);

}

void connectib_pmlp_prm_style_unpack(struct connectib_pmlp_prm_style *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->width = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 56;
	ptr_struct->module0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 46;
	ptr_struct->lane0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 88;
	ptr_struct->module1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 78;
	ptr_struct->lane1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 120;
	ptr_struct->module2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 110;
	ptr_struct->lane2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 152;
	ptr_struct->module3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 142;
	ptr_struct->lane3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_pmlp_prm_style_print(const struct connectib_pmlp_prm_style *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmlp_prm_style ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "width                : " UH_FMT "\n", ptr_struct->width);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module0              : " UH_FMT "\n", ptr_struct->module0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane0                : " UH_FMT "\n", ptr_struct->lane0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module1              : " UH_FMT "\n", ptr_struct->module1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane1                : " UH_FMT "\n", ptr_struct->lane1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module2              : " UH_FMT "\n", ptr_struct->module2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane2                : " UH_FMT "\n", ptr_struct->lane2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module3              : " UH_FMT "\n", ptr_struct->module3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane3                : " UH_FMT "\n", ptr_struct->lane3);

}

int connectib_pmlp_prm_style_size(void)
{
	 return 36;
}

void connectib_pmlp_prm_style_dump(const struct connectib_pmlp_prm_style *ptr_struct, FILE* file)
{
	connectib_pmlp_prm_style_print(ptr_struct, file, 0);
}

void connectib_register_access_data_reduced_pack(const struct connectib_register_access_data_reduced *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword[i]);
	}

}

void connectib_register_access_data_reduced_unpack(struct connectib_register_access_data_reduced *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	ptr_struct->dword[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_register_access_data_reduced_print(const struct connectib_register_access_data_reduced *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_access_data_reduced ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_%03d           : " U32H_FMT "\n", i, ptr_struct->dword[i]);
	}

}

int connectib_register_access_data_reduced_size(void)
{
	 return 16;
}

void connectib_register_access_data_reduced_dump(const struct connectib_register_access_data_reduced *ptr_struct, FILE* file)
{
	connectib_register_access_data_reduced_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_hdr_pack(const struct connectib_nv_tlv_hdr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->length);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->writer_host_id);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->writer_id);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->read_current);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->read_default);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rd_en);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ovr_en);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->header_type);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->priority);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->valid);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->type);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->crc16);

}

void connectib_nv_tlv_hdr_unpack(struct connectib_nv_tlv_hdr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 23;
	ptr_struct->length = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 20;
	ptr_struct->writer_host_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 16;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 11;
	ptr_struct->writer_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 9;
	ptr_struct->read_current = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->read_default = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->rd_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->ovr_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->header_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 2;
	ptr_struct->priority = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 0;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 32;
	ptr_struct->type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 80;
	ptr_struct->crc16 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_nv_tlv_hdr_print(const struct connectib_nv_tlv_hdr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_hdr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "length               : " UH_FMT "\n", ptr_struct->length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "writer_host_id       : " UH_FMT "\n", ptr_struct->writer_host_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "writer_id            : %s (" UH_FMT ")\n", (ptr_struct->writer_id == 0 ? ("NV_WRITER_ID_UNSPECIFIED") : ((ptr_struct->writer_id == 1 ? ("NV_WRITER_ID_CHASSIS_BMC") : ((ptr_struct->writer_id == 2 ? ("NV_WRITER_ID_MAD") : ((ptr_struct->writer_id == 3 ? ("NV_WRITER_ID_BMC") : ((ptr_struct->writer_id == 4 ? ("NV_WRITER_ID_CMD_IF") : ((ptr_struct->writer_id == 5 ? ("NV_WRITER_ID_ICMD") : ((ptr_struct->writer_id == 6 ? ("NV_WRITER_ID_ICMD_UEFI_HII") : ((ptr_struct->writer_id == 7 ? ("NV_WRITER_ID_ICMD_UEFI_CLP") : ((ptr_struct->writer_id == 8 ? ("NV_WRITER_ID_ICMD_FLEXBOOT") : ((ptr_struct->writer_id == 9 ? ("NV_WRITER_ID_ICMD_MLXCONFIG") : ((ptr_struct->writer_id == 10 ? ("NV_WRITER_ID_ICMD_USER1") : ((ptr_struct->writer_id == 11 ? ("NV_WRITER_ID_ICMD_USER2") : ((ptr_struct->writer_id == 31 ? ("NV_WRITER_ID_OTHER") : ("unknown")))))))))))))))))))))))))), ptr_struct->writer_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_current         : " UH_FMT "\n", ptr_struct->read_current);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_default         : " UH_FMT "\n", ptr_struct->read_default);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rd_en                : " UH_FMT "\n", ptr_struct->rd_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ovr_en               : " UH_FMT "\n", ptr_struct->ovr_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "header_type          : " UH_FMT "\n", ptr_struct->header_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "priority             : " UH_FMT "\n", ptr_struct->priority);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " U32H_FMT "\n", ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "crc16                : " UH_FMT "\n", ptr_struct->crc16);

}

int connectib_nv_tlv_hdr_size(void)
{
	 return 12;
}

void connectib_nv_tlv_hdr_dump(const struct connectib_nv_tlv_hdr *ptr_struct, FILE* file)
{
	connectib_nv_tlv_hdr_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_type_pack(const union connectib_nv_tlv_type *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_dw_pack(&(ptr_struct->as_dw), ptr_buff);
}

void connectib_nv_tlv_type_unpack(union connectib_nv_tlv_type *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_dw_unpack(&(ptr_struct->as_dw), ptr_buff);
}

void connectib_nv_tlv_type_print(const union connectib_nv_tlv_type *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_type ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "global:\n");
	connectib_nv_tlv_type_global_print(&(ptr_struct->global), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "physical_port:\n");
	connectib_nv_tlv_type_physical_port_print(&(ptr_struct->physical_port), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bmc:\n");
	connectib_nv_tlv_type_bmc_print(&(ptr_struct->bmc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "per_host:\n");
	connectib_nv_tlv_type_per_host_print(&(ptr_struct->per_host), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eswitch:\n");
	connectib_nv_tlv_type_eswitch_print(&(ptr_struct->eswitch), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvlog:\n");
	connectib_nv_tlv_type_nvlog_print(&(ptr_struct->nvlog), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "as_dw:\n");
	connectib_dw_print(&(ptr_struct->as_dw), file, indent_level + 1);

}

int connectib_nv_tlv_type_size(void)
{
	 return 4;
}

void connectib_nv_tlv_type_dump(const union connectib_nv_tlv_type *ptr_struct, FILE* file)
{
	connectib_nv_tlv_type_print(ptr_struct, file, 0);
}

void connectib_register_access_data_pack(const struct connectib_register_access_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword[i]);
	}

}

void connectib_register_access_data_unpack(struct connectib_register_access_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	ptr_struct->dword[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_register_access_data_print(const struct connectib_register_access_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_access_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_%03d           : " U32H_FMT "\n", i, ptr_struct->dword[i]);
	}

}

int connectib_register_access_data_size(void)
{
	 return 32;
}

void connectib_register_access_data_dump(const struct connectib_register_access_data *ptr_struct, FILE* file)
{
	connectib_register_access_data_print(ptr_struct, file, 0);
}

void connectib_mfba_header_pack(const struct connectib_mfba_header *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->fs);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->p);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->size);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->address);

}

void connectib_mfba_header_unpack(struct connectib_mfba_header *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->fs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 23;
	ptr_struct->p = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 55;
	ptr_struct->size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 72;
	ptr_struct->address = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_mfba_header_print(const struct connectib_mfba_header *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mfba_header ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fs                   : " UH_FMT "\n", ptr_struct->fs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "p                    : " UH_FMT "\n", ptr_struct->p);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address              : " UH_FMT "\n", ptr_struct->address);

}

int connectib_mfba_header_size(void)
{
	 return 12;
}

void connectib_mfba_header_dump(const struct connectib_mfba_header *ptr_struct, FILE* file)
{
	connectib_mfba_header_print(ptr_struct, file, 0);
}

void connectib_pmmp_sfp_protocol_override_layout_pack(const struct connectib_pmmp_sfp_protocol_override_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->tengig_ethernet_compliance_code);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cable_technology);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->bit_rate);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ethernet_compliance_code);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ext_ethernet_compliance_code);

}

void connectib_pmmp_sfp_protocol_override_layout_unpack(struct connectib_pmmp_sfp_protocol_override_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->tengig_ethernet_compliance_code = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->cable_technology = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->bit_rate = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 56;
	ptr_struct->ethernet_compliance_code = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->ext_ethernet_compliance_code = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_pmmp_sfp_protocol_override_layout_print(const struct connectib_pmmp_sfp_protocol_override_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmmp_sfp_protocol_override_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tengig_ethernet_compliance_code : " UH_FMT "\n", ptr_struct->tengig_ethernet_compliance_code);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cable_technology     : " UH_FMT "\n", ptr_struct->cable_technology);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bit_rate             : " UH_FMT "\n", ptr_struct->bit_rate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ethernet_compliance_code : " UH_FMT "\n", ptr_struct->ethernet_compliance_code);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ext_ethernet_compliance_code : " UH_FMT "\n", ptr_struct->ext_ethernet_compliance_code);

}

int connectib_pmmp_sfp_protocol_override_layout_size(void)
{
	 return 12;
}

void connectib_pmmp_sfp_protocol_override_layout_dump(const struct connectib_pmmp_sfp_protocol_override_layout *ptr_struct, FILE* file)
{
	connectib_pmmp_sfp_protocol_override_layout_print(ptr_struct, file, 0);
}

void connectib_pmmp_qsfp_protocol_override_layout_pack(const struct connectib_pmmp_qsfp_protocol_override_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ethernet_compliance_code);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cable_breakout);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->giga_ethernet_compliance_code);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ext_ethernet_compliance_code);

}

void connectib_pmmp_qsfp_protocol_override_layout_unpack(struct connectib_pmmp_qsfp_protocol_override_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->ethernet_compliance_code = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->cable_breakout = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 56;
	ptr_struct->giga_ethernet_compliance_code = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->ext_ethernet_compliance_code = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_pmmp_qsfp_protocol_override_layout_print(const struct connectib_pmmp_qsfp_protocol_override_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmmp_qsfp_protocol_override_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ethernet_compliance_code : " UH_FMT "\n", ptr_struct->ethernet_compliance_code);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cable_breakout       : " UH_FMT "\n", ptr_struct->cable_breakout);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "giga_ethernet_compliance_code : " UH_FMT "\n", ptr_struct->giga_ethernet_compliance_code);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ext_ethernet_compliance_code : " UH_FMT "\n", ptr_struct->ext_ethernet_compliance_code);

}

int connectib_pmmp_qsfp_protocol_override_layout_size(void)
{
	 return 12;
}

void connectib_pmmp_qsfp_protocol_override_layout_dump(const struct connectib_pmmp_qsfp_protocol_override_layout *ptr_struct, FILE* file)
{
	connectib_pmmp_qsfp_protocol_override_layout_print(ptr_struct, file, 0);
}

void connectib_pcie_timers_states_pack(const struct connectib_pcie_timers_states *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->life_time_counter_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->life_time_counter_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_to_boot_image_start);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_to_link_image);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->calibration_time);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_to_first_perst);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_to_detect_state);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_to_l0);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_to_crs_en);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_to_plastic_image_start);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_to_iron_image_start);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->perst_handler);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->times_in_l1);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->times_in_l23);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dl_down);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->config_cycle1usec);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->config_cycle2to7usec);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->config_cycle8to15usec);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->config_cycle16to63usec);

	offset = 608;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->config_cycle64usec);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->correctable_err_msg_sent);

	offset = 672;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->non_fatal_err_msg_sent);

	offset = 704;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fatal_err_msg_sent);

}

void connectib_pcie_timers_states_unpack(struct connectib_pcie_timers_states *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->life_time_counter_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->life_time_counter_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->time_to_boot_image_start = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->time_to_link_image = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->calibration_time = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->time_to_first_perst = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->time_to_detect_state = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->time_to_l0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->time_to_crs_en = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->time_to_plastic_image_start = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->time_to_iron_image_start = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->perst_handler = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->times_in_l1 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->times_in_l23 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->dl_down = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->config_cycle1usec = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 512;
	ptr_struct->config_cycle2to7usec = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->config_cycle8to15usec = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->config_cycle16to63usec = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 608;
	ptr_struct->config_cycle64usec = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 640;
	ptr_struct->correctable_err_msg_sent = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 672;
	ptr_struct->non_fatal_err_msg_sent = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 704;
	ptr_struct->fatal_err_msg_sent = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_pcie_timers_states_print(const struct connectib_pcie_timers_states *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_timers_states ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "life_time_counter_high : " U32H_FMT "\n", ptr_struct->life_time_counter_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "life_time_counter_low : " U32H_FMT "\n", ptr_struct->life_time_counter_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_to_boot_image_start : " U32H_FMT "\n", ptr_struct->time_to_boot_image_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_to_link_image   : " U32H_FMT "\n", ptr_struct->time_to_link_image);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "calibration_time     : " U32H_FMT "\n", ptr_struct->calibration_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_to_first_perst  : " U32H_FMT "\n", ptr_struct->time_to_first_perst);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_to_detect_state : " U32H_FMT "\n", ptr_struct->time_to_detect_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_to_l0           : " U32H_FMT "\n", ptr_struct->time_to_l0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_to_crs_en       : " U32H_FMT "\n", ptr_struct->time_to_crs_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_to_plastic_image_start : " U32H_FMT "\n", ptr_struct->time_to_plastic_image_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_to_iron_image_start : " U32H_FMT "\n", ptr_struct->time_to_iron_image_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "perst_handler        : " U32H_FMT "\n", ptr_struct->perst_handler);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "times_in_l1          : " U32H_FMT "\n", ptr_struct->times_in_l1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "times_in_l23         : " U32H_FMT "\n", ptr_struct->times_in_l23);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dl_down              : " U32H_FMT "\n", ptr_struct->dl_down);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "config_cycle1usec    : " U32H_FMT "\n", ptr_struct->config_cycle1usec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "config_cycle2to7usec : " U32H_FMT "\n", ptr_struct->config_cycle2to7usec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "config_cycle8to15usec : " U32H_FMT "\n", ptr_struct->config_cycle8to15usec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "config_cycle16to63usec : " U32H_FMT "\n", ptr_struct->config_cycle16to63usec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "config_cycle64usec   : " U32H_FMT "\n", ptr_struct->config_cycle64usec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "correctable_err_msg_sent : " U32H_FMT "\n", ptr_struct->correctable_err_msg_sent);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "non_fatal_err_msg_sent : " U32H_FMT "\n", ptr_struct->non_fatal_err_msg_sent);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fatal_err_msg_sent   : " U32H_FMT "\n", ptr_struct->fatal_err_msg_sent);

}

int connectib_pcie_timers_states_size(void)
{
	 return 248;
}

void connectib_pcie_timers_states_dump(const struct connectib_pcie_timers_states *ptr_struct, FILE* file)
{
	connectib_pcie_timers_states_print(ptr_struct, file, 0);
}

void connectib_pcie_lanes_counters_pack(const struct connectib_pcie_lanes_counters *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->life_time_counter_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->life_time_counter_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane0);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane1);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane2);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane3);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane4);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane5);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane6);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane7);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane8);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane9);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane10);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane11);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane12);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane13);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane14);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_counter_lane15);

}

void connectib_pcie_lanes_counters_unpack(struct connectib_pcie_lanes_counters *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->life_time_counter_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->life_time_counter_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->error_counter_lane0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->error_counter_lane1 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->error_counter_lane2 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->error_counter_lane3 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->error_counter_lane4 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->error_counter_lane5 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->error_counter_lane6 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->error_counter_lane7 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->error_counter_lane8 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->error_counter_lane9 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->error_counter_lane10 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->error_counter_lane11 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->error_counter_lane12 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->error_counter_lane13 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 512;
	ptr_struct->error_counter_lane14 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->error_counter_lane15 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_pcie_lanes_counters_print(const struct connectib_pcie_lanes_counters *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_lanes_counters ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "life_time_counter_high : " U32H_FMT "\n", ptr_struct->life_time_counter_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "life_time_counter_low : " U32H_FMT "\n", ptr_struct->life_time_counter_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane0  : " U32H_FMT "\n", ptr_struct->error_counter_lane0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane1  : " U32H_FMT "\n", ptr_struct->error_counter_lane1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane2  : " U32H_FMT "\n", ptr_struct->error_counter_lane2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane3  : " U32H_FMT "\n", ptr_struct->error_counter_lane3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane4  : " U32H_FMT "\n", ptr_struct->error_counter_lane4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane5  : " U32H_FMT "\n", ptr_struct->error_counter_lane5);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane6  : " U32H_FMT "\n", ptr_struct->error_counter_lane6);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane7  : " U32H_FMT "\n", ptr_struct->error_counter_lane7);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane8  : " U32H_FMT "\n", ptr_struct->error_counter_lane8);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane9  : " U32H_FMT "\n", ptr_struct->error_counter_lane9);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane10 : " U32H_FMT "\n", ptr_struct->error_counter_lane10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane11 : " U32H_FMT "\n", ptr_struct->error_counter_lane11);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane12 : " U32H_FMT "\n", ptr_struct->error_counter_lane12);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane13 : " U32H_FMT "\n", ptr_struct->error_counter_lane13);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane14 : " U32H_FMT "\n", ptr_struct->error_counter_lane14);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_counter_lane15 : " U32H_FMT "\n", ptr_struct->error_counter_lane15);

}

int connectib_pcie_lanes_counters_size(void)
{
	 return 248;
}

void connectib_pcie_lanes_counters_dump(const struct connectib_pcie_lanes_counters *ptr_struct, FILE* file)
{
	connectib_pcie_lanes_counters_print(ptr_struct, file, 0);
}

void connectib_pcie_perf_counters_pack(const struct connectib_pcie_perf_counters *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->life_time_counter_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->life_time_counter_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_errors);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_errors);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->l0_to_recovery_eieos);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->l0_to_recovery_ts);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->l0_to_recovery_framing);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->l0_to_recovery_retrain);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->crc_error_dllp);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->crc_error_tlp);

}

void connectib_pcie_perf_counters_unpack(struct connectib_pcie_perf_counters *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->life_time_counter_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->life_time_counter_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->rx_errors = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->tx_errors = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->l0_to_recovery_eieos = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->l0_to_recovery_ts = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->l0_to_recovery_framing = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->l0_to_recovery_retrain = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->crc_error_dllp = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->crc_error_tlp = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_pcie_perf_counters_print(const struct connectib_pcie_perf_counters *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_perf_counters ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "life_time_counter_high : " U32H_FMT "\n", ptr_struct->life_time_counter_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "life_time_counter_low : " U32H_FMT "\n", ptr_struct->life_time_counter_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_errors            : " U32H_FMT "\n", ptr_struct->rx_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_errors            : " U32H_FMT "\n", ptr_struct->tx_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l0_to_recovery_eieos : " U32H_FMT "\n", ptr_struct->l0_to_recovery_eieos);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l0_to_recovery_ts    : " U32H_FMT "\n", ptr_struct->l0_to_recovery_ts);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l0_to_recovery_framing : " U32H_FMT "\n", ptr_struct->l0_to_recovery_framing);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l0_to_recovery_retrain : " U32H_FMT "\n", ptr_struct->l0_to_recovery_retrain);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "crc_error_dllp       : " U32H_FMT "\n", ptr_struct->crc_error_dllp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "crc_error_tlp        : " U32H_FMT "\n", ptr_struct->crc_error_tlp);

}

int connectib_pcie_perf_counters_size(void)
{
	 return 200;
}

void connectib_pcie_perf_counters_dump(const struct connectib_pcie_perf_counters *ptr_struct, FILE* file)
{
	connectib_pcie_perf_counters_print(ptr_struct, file, 0);
}

void connectib_phys_layer_stat_cntrs_pack(const struct connectib_phys_layer_stat_cntrs *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_since_last_clear_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_since_last_clear_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_received_bits_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_received_bits_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_symbol_errors_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_symbol_errors_low);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_high);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_lane0_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_lane0_low);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_lane1_high);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_lane1_low);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_lane2_high);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_lane2_low);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_lane3_high);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phy_corrected_bits_lane3_low);

	offset = 796;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->raw_ber_coef);

	offset = 784;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->raw_ber_magnitude);

	offset = 828;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->effective_ber_coef);

	offset = 816;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->effective_ber_magnitude);

}

void connectib_phys_layer_stat_cntrs_unpack(struct connectib_phys_layer_stat_cntrs *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->time_since_last_clear_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->time_since_last_clear_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->phy_received_bits_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->phy_received_bits_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->phy_symbol_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->phy_symbol_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->phy_corrected_bits_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->phy_corrected_bits_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->phy_corrected_bits_lane0_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->phy_corrected_bits_lane0_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->phy_corrected_bits_lane1_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->phy_corrected_bits_lane1_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->phy_corrected_bits_lane2_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->phy_corrected_bits_lane2_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->phy_corrected_bits_lane3_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->phy_corrected_bits_lane3_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 796;
	ptr_struct->raw_ber_coef = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 784;
	ptr_struct->raw_ber_magnitude = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 828;
	ptr_struct->effective_ber_coef = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 816;
	ptr_struct->effective_ber_magnitude = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_phys_layer_stat_cntrs_print(const struct connectib_phys_layer_stat_cntrs *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_phys_layer_stat_cntrs ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_since_last_clear_high : " U32H_FMT "\n", ptr_struct->time_since_last_clear_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_since_last_clear_low : " U32H_FMT "\n", ptr_struct->time_since_last_clear_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_received_bits_high : " U32H_FMT "\n", ptr_struct->phy_received_bits_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_received_bits_low : " U32H_FMT "\n", ptr_struct->phy_received_bits_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_symbol_errors_high : " U32H_FMT "\n", ptr_struct->phy_symbol_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_symbol_errors_low : " U32H_FMT "\n", ptr_struct->phy_symbol_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_high : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_low : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_lane0_high : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_lane0_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_lane0_low : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_lane0_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_lane1_high : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_lane1_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_lane1_low : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_lane1_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_lane2_high : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_lane2_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_lane2_low : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_lane2_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_lane3_high : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_lane3_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_corrected_bits_lane3_low : " U32H_FMT "\n", ptr_struct->phy_corrected_bits_lane3_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw_ber_coef         : " UH_FMT "\n", ptr_struct->raw_ber_coef);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw_ber_magnitude    : " UH_FMT "\n", ptr_struct->raw_ber_magnitude);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "effective_ber_coef   : " UH_FMT "\n", ptr_struct->effective_ber_coef);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "effective_ber_magnitude : " UH_FMT "\n", ptr_struct->effective_ber_magnitude);

}

int connectib_phys_layer_stat_cntrs_size(void)
{
	 return 248;
}

void connectib_phys_layer_stat_cntrs_dump(const struct connectib_phys_layer_stat_cntrs *ptr_struct, FILE* file)
{
	connectib_phys_layer_stat_cntrs_print(ptr_struct, file, 0);
}

void connectib_eth_per_traffic_congestion_grp_data_layout_pack(const struct connectib_eth_per_traffic_congestion_grp_data_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->wred_discard_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->wred_discard_low);

}

void connectib_eth_per_traffic_congestion_grp_data_layout_unpack(struct connectib_eth_per_traffic_congestion_grp_data_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->wred_discard_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->wred_discard_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_eth_per_traffic_congestion_grp_data_layout_print(const struct connectib_eth_per_traffic_congestion_grp_data_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eth_per_traffic_congestion_grp_data_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wred_discard_high    : " U32H_FMT "\n", ptr_struct->wred_discard_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wred_discard_low     : " U32H_FMT "\n", ptr_struct->wred_discard_low);

}

int connectib_eth_per_traffic_congestion_grp_data_layout_size(void)
{
	 return 248;
}

void connectib_eth_per_traffic_congestion_grp_data_layout_dump(const struct connectib_eth_per_traffic_congestion_grp_data_layout *ptr_struct, FILE* file)
{
	connectib_eth_per_traffic_congestion_grp_data_layout_print(ptr_struct, file, 0);
}

void connectib_ib_port_counters_pack(const struct connectib_ib_port_counters *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->link_downed_counter);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->link_error_recovery_counter);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->symbol_error_counter);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->port_rcv_remote_physical_errors);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->port_rcv_errors);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->port_xmit_discards);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->port_rcv_switch_relay_errors);

	offset = 124;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->excessive_buffer_overrun_errors);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->local_link_integrity_errors);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_rcv_constraint_errors);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_xmit_constraint_errors);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vl15_dropped);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_xmit_data);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_rcv_data);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_xmit_pkts);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_rcv_pkts);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_xmit_wait);

}

void connectib_ib_port_counters_unpack(struct connectib_ib_port_counters *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->link_downed_counter = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->link_error_recovery_counter = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->symbol_error_counter = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	ptr_struct->port_rcv_remote_physical_errors = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->port_rcv_errors = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 80;
	ptr_struct->port_xmit_discards = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->port_rcv_switch_relay_errors = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 124;
	ptr_struct->excessive_buffer_overrun_errors = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 120;
	ptr_struct->local_link_integrity_errors = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 104;
	ptr_struct->port_rcv_constraint_errors = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->port_xmit_constraint_errors = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 144;
	ptr_struct->vl15_dropped = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 160;
	ptr_struct->port_xmit_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->port_rcv_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->port_xmit_pkts = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->port_rcv_pkts = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->port_xmit_wait = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_ib_port_counters_print(const struct connectib_ib_port_counters *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ib_port_counters ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_downed_counter  : " UH_FMT "\n", ptr_struct->link_downed_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_error_recovery_counter : " UH_FMT "\n", ptr_struct->link_error_recovery_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "symbol_error_counter : " UH_FMT "\n", ptr_struct->symbol_error_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_rcv_remote_physical_errors : " UH_FMT "\n", ptr_struct->port_rcv_remote_physical_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_rcv_errors      : " UH_FMT "\n", ptr_struct->port_rcv_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_xmit_discards   : " UH_FMT "\n", ptr_struct->port_xmit_discards);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_rcv_switch_relay_errors : " UH_FMT "\n", ptr_struct->port_rcv_switch_relay_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "excessive_buffer_overrun_errors : " UH_FMT "\n", ptr_struct->excessive_buffer_overrun_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_link_integrity_errors : " UH_FMT "\n", ptr_struct->local_link_integrity_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_rcv_constraint_errors : " UH_FMT "\n", ptr_struct->port_rcv_constraint_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_xmit_constraint_errors : " UH_FMT "\n", ptr_struct->port_xmit_constraint_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl15_dropped         : " UH_FMT "\n", ptr_struct->vl15_dropped);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_xmit_data       : " U32H_FMT "\n", ptr_struct->port_xmit_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_rcv_data        : " U32H_FMT "\n", ptr_struct->port_rcv_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_xmit_pkts       : " U32H_FMT "\n", ptr_struct->port_xmit_pkts);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_rcv_pkts        : " U32H_FMT "\n", ptr_struct->port_rcv_pkts);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_xmit_wait       : " U32H_FMT "\n", ptr_struct->port_xmit_wait);

}

int connectib_ib_port_counters_size(void)
{
	 return 248;
}

void connectib_ib_port_counters_dump(const struct connectib_ib_port_counters *ptr_struct, FILE* file)
{
	connectib_ib_port_counters_print(ptr_struct, file, 0);
}

void connectib_llr_counters_pack(const struct connectib_llr_counters *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_rx_cells_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_rx_cells_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_rx_error_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_rx_error_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_rx_crc_error_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_rx_crc_error_low);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_tx_cells_high);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_tx_cells_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_tx_ret_cells_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_tx_ret_cells_low);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_tx_ret_events_high);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->llr_tx_ret_events_low);

}

void connectib_llr_counters_unpack(struct connectib_llr_counters *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->llr_rx_cells_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->llr_rx_cells_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->llr_rx_error_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->llr_rx_error_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->llr_rx_crc_error_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->llr_rx_crc_error_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->llr_tx_cells_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->llr_tx_cells_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->llr_tx_ret_cells_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->llr_tx_ret_cells_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->llr_tx_ret_events_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->llr_tx_ret_events_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_llr_counters_print(const struct connectib_llr_counters *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_llr_counters ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_rx_cells_high    : " U32H_FMT "\n", ptr_struct->llr_rx_cells_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_rx_cells_low     : " U32H_FMT "\n", ptr_struct->llr_rx_cells_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_rx_error_high    : " U32H_FMT "\n", ptr_struct->llr_rx_error_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_rx_error_low     : " U32H_FMT "\n", ptr_struct->llr_rx_error_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_rx_crc_error_high : " U32H_FMT "\n", ptr_struct->llr_rx_crc_error_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_rx_crc_error_low : " U32H_FMT "\n", ptr_struct->llr_rx_crc_error_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_tx_cells_high    : " U32H_FMT "\n", ptr_struct->llr_tx_cells_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_tx_cells_low     : " U32H_FMT "\n", ptr_struct->llr_tx_cells_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_tx_ret_cells_high : " U32H_FMT "\n", ptr_struct->llr_tx_ret_cells_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_tx_ret_cells_low : " U32H_FMT "\n", ptr_struct->llr_tx_ret_cells_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_tx_ret_events_high : " U32H_FMT "\n", ptr_struct->llr_tx_ret_events_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_tx_ret_events_low : " U32H_FMT "\n", ptr_struct->llr_tx_ret_events_low);

}

int connectib_llr_counters_size(void)
{
	 return 248;
}

void connectib_llr_counters_dump(const struct connectib_llr_counters *ptr_struct, FILE* file)
{
	connectib_llr_counters_print(ptr_struct, file, 0);
}

void connectib_discard_cntrs_grp_data_pack(const struct connectib_discard_cntrs_grp_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_general_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_general_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_policy_engine_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_policy_engine_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_vlan_membership_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_vlan_membership_low);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_tag_frame_type_high);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_tag_frame_type_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_vlan_membership_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_vlan_membership_low);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->loopback_filter_high);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->loopback_filter_low);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_general_high);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_general_low);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_link_down_high);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_link_down_low);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_hoq_high);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_hoq_low);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_isolation_high);

	offset = 608;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_isolation_low);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_policy_engine_high);

	offset = 672;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_policy_engine_low);

	offset = 704;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_tx_link_down_high);

	offset = 736;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ingress_tx_link_down_low);

	offset = 768;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_stp_filter_high);

	offset = 800;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_stp_filter_low);

	offset = 832;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_hoq_stall_high);

	offset = 864;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->egress_hoq_stall_low);

}

void connectib_discard_cntrs_grp_data_unpack(struct connectib_discard_cntrs_grp_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->ingress_general_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->ingress_general_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->ingress_policy_engine_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->ingress_policy_engine_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->ingress_vlan_membership_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->ingress_vlan_membership_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->ingress_tag_frame_type_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->ingress_tag_frame_type_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->egress_vlan_membership_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->egress_vlan_membership_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->loopback_filter_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->loopback_filter_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->egress_general_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->egress_general_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->egress_link_down_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->egress_link_down_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 512;
	ptr_struct->egress_hoq_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->egress_hoq_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->port_isolation_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 608;
	ptr_struct->port_isolation_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 640;
	ptr_struct->egress_policy_engine_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 672;
	ptr_struct->egress_policy_engine_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 704;
	ptr_struct->ingress_tx_link_down_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 736;
	ptr_struct->ingress_tx_link_down_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 768;
	ptr_struct->egress_stp_filter_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 800;
	ptr_struct->egress_stp_filter_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 832;
	ptr_struct->egress_hoq_stall_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 864;
	ptr_struct->egress_hoq_stall_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_discard_cntrs_grp_data_print(const struct connectib_discard_cntrs_grp_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_discard_cntrs_grp_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_general_high : " U32H_FMT "\n", ptr_struct->ingress_general_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_general_low  : " U32H_FMT "\n", ptr_struct->ingress_general_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_policy_engine_high : " U32H_FMT "\n", ptr_struct->ingress_policy_engine_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_policy_engine_low : " U32H_FMT "\n", ptr_struct->ingress_policy_engine_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_vlan_membership_high : " U32H_FMT "\n", ptr_struct->ingress_vlan_membership_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_vlan_membership_low : " U32H_FMT "\n", ptr_struct->ingress_vlan_membership_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_tag_frame_type_high : " U32H_FMT "\n", ptr_struct->ingress_tag_frame_type_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_tag_frame_type_low : " U32H_FMT "\n", ptr_struct->ingress_tag_frame_type_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_vlan_membership_high : " U32H_FMT "\n", ptr_struct->egress_vlan_membership_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_vlan_membership_low : " U32H_FMT "\n", ptr_struct->egress_vlan_membership_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "loopback_filter_high : " U32H_FMT "\n", ptr_struct->loopback_filter_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "loopback_filter_low  : " U32H_FMT "\n", ptr_struct->loopback_filter_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_general_high  : " U32H_FMT "\n", ptr_struct->egress_general_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_general_low   : " U32H_FMT "\n", ptr_struct->egress_general_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_link_down_high : " U32H_FMT "\n", ptr_struct->egress_link_down_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_link_down_low : " U32H_FMT "\n", ptr_struct->egress_link_down_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_hoq_high      : " U32H_FMT "\n", ptr_struct->egress_hoq_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_hoq_low       : " U32H_FMT "\n", ptr_struct->egress_hoq_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_isolation_high  : " U32H_FMT "\n", ptr_struct->port_isolation_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_isolation_low   : " U32H_FMT "\n", ptr_struct->port_isolation_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_policy_engine_high : " U32H_FMT "\n", ptr_struct->egress_policy_engine_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_policy_engine_low : " U32H_FMT "\n", ptr_struct->egress_policy_engine_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_tx_link_down_high : " U32H_FMT "\n", ptr_struct->ingress_tx_link_down_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ingress_tx_link_down_low : " U32H_FMT "\n", ptr_struct->ingress_tx_link_down_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_stp_filter_high : " U32H_FMT "\n", ptr_struct->egress_stp_filter_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_stp_filter_low : " U32H_FMT "\n", ptr_struct->egress_stp_filter_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_hoq_stall_high : " U32H_FMT "\n", ptr_struct->egress_hoq_stall_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_hoq_stall_low : " U32H_FMT "\n", ptr_struct->egress_hoq_stall_low);

}

int connectib_discard_cntrs_grp_data_size(void)
{
	 return 248;
}

void connectib_discard_cntrs_grp_data_dump(const struct connectib_discard_cntrs_grp_data *ptr_struct, FILE* file)
{
	connectib_discard_cntrs_grp_data_print(ptr_struct, file, 0);
}

void connectib_phys_layer_cntrs_pack(const struct connectib_phys_layer_cntrs *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_since_last_clear_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_since_last_clear_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->symbol_errors_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->symbol_errors_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sync_headers_errors_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sync_headers_errors_low);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->edpl_bip_errors_lane0_high);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->edpl_bip_errors_lane0_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->edpl_bip_errors_lane1_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->edpl_bip_errors_lane1_low);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->edpl_bip_errors_lane2_high);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->edpl_bip_errors_lane2_low);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->edpl_bip_errors_lane3_high);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->edpl_bip_errors_lane3_low);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_corrected_blocks_lane0_high);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_corrected_blocks_lane0_low);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_corrected_blocks_lane1_high);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_corrected_blocks_lane1_low);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_corrected_blocks_lane2_high);

	offset = 608;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_corrected_blocks_lane2_low);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_corrected_blocks_lane3_high);

	offset = 672;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_corrected_blocks_lane3_low);

	offset = 704;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_uncorrectable_blocks_lane0_high);

	offset = 736;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_uncorrectable_blocks_lane0_low);

	offset = 768;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_uncorrectable_blocks_lane1_high);

	offset = 800;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_uncorrectable_blocks_lane1_low);

	offset = 832;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_uncorrectable_blocks_lane2_high);

	offset = 864;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_uncorrectable_blocks_lane2_low);

	offset = 896;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_uncorrectable_blocks_lane3_high);

	offset = 928;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_fec_uncorrectable_blocks_lane3_low);

	offset = 960;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_blocks_high);

	offset = 992;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_blocks_low);

	offset = 1024;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_uncorrectable_blocks_high);

	offset = 1056;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_uncorrectable_blocks_low);

	offset = 1088;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_no_errors_blocks_high);

	offset = 1120;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_no_errors_blocks_low);

	offset = 1152;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_single_error_blocks_high);

	offset = 1184;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_single_error_blocks_low);

	offset = 1216;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_total_high);

	offset = 1248;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_total_low);

	offset = 1280;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_lane0_high);

	offset = 1312;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_lane0_low);

	offset = 1344;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_lane1_high);

	offset = 1376;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_lane1_low);

	offset = 1408;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_lane2_high);

	offset = 1440;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_lane2_low);

	offset = 1472;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_lane3_high);

	offset = 1504;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_corrected_symbols_lane3_low);

	offset = 1536;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->link_down_events);

	offset = 1568;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->successful_recovery_events);

}

void connectib_phys_layer_cntrs_unpack(struct connectib_phys_layer_cntrs *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->time_since_last_clear_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->time_since_last_clear_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->symbol_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->symbol_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->sync_headers_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->sync_headers_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->edpl_bip_errors_lane0_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->edpl_bip_errors_lane0_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->edpl_bip_errors_lane1_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->edpl_bip_errors_lane1_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->edpl_bip_errors_lane2_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->edpl_bip_errors_lane2_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->edpl_bip_errors_lane3_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->edpl_bip_errors_lane3_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->fc_fec_corrected_blocks_lane0_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->fc_fec_corrected_blocks_lane0_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 512;
	ptr_struct->fc_fec_corrected_blocks_lane1_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->fc_fec_corrected_blocks_lane1_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->fc_fec_corrected_blocks_lane2_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 608;
	ptr_struct->fc_fec_corrected_blocks_lane2_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 640;
	ptr_struct->fc_fec_corrected_blocks_lane3_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 672;
	ptr_struct->fc_fec_corrected_blocks_lane3_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 704;
	ptr_struct->fc_fec_uncorrectable_blocks_lane0_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 736;
	ptr_struct->fc_fec_uncorrectable_blocks_lane0_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 768;
	ptr_struct->fc_fec_uncorrectable_blocks_lane1_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 800;
	ptr_struct->fc_fec_uncorrectable_blocks_lane1_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 832;
	ptr_struct->fc_fec_uncorrectable_blocks_lane2_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 864;
	ptr_struct->fc_fec_uncorrectable_blocks_lane2_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 896;
	ptr_struct->fc_fec_uncorrectable_blocks_lane3_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 928;
	ptr_struct->fc_fec_uncorrectable_blocks_lane3_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 960;
	ptr_struct->rs_fec_corrected_blocks_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 992;
	ptr_struct->rs_fec_corrected_blocks_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1024;
	ptr_struct->rs_fec_uncorrectable_blocks_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1056;
	ptr_struct->rs_fec_uncorrectable_blocks_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1088;
	ptr_struct->rs_fec_no_errors_blocks_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1120;
	ptr_struct->rs_fec_no_errors_blocks_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1152;
	ptr_struct->rs_fec_single_error_blocks_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1184;
	ptr_struct->rs_fec_single_error_blocks_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1216;
	ptr_struct->rs_fec_corrected_symbols_total_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1248;
	ptr_struct->rs_fec_corrected_symbols_total_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1280;
	ptr_struct->rs_fec_corrected_symbols_lane0_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1312;
	ptr_struct->rs_fec_corrected_symbols_lane0_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1344;
	ptr_struct->rs_fec_corrected_symbols_lane1_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1376;
	ptr_struct->rs_fec_corrected_symbols_lane1_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1408;
	ptr_struct->rs_fec_corrected_symbols_lane2_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1440;
	ptr_struct->rs_fec_corrected_symbols_lane2_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1472;
	ptr_struct->rs_fec_corrected_symbols_lane3_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1504;
	ptr_struct->rs_fec_corrected_symbols_lane3_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1536;
	ptr_struct->link_down_events = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1568;
	ptr_struct->successful_recovery_events = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_phys_layer_cntrs_print(const struct connectib_phys_layer_cntrs *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_phys_layer_cntrs ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_since_last_clear_high : " U32H_FMT "\n", ptr_struct->time_since_last_clear_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_since_last_clear_low : " U32H_FMT "\n", ptr_struct->time_since_last_clear_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "symbol_errors_high   : " U32H_FMT "\n", ptr_struct->symbol_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "symbol_errors_low    : " U32H_FMT "\n", ptr_struct->symbol_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sync_headers_errors_high : " U32H_FMT "\n", ptr_struct->sync_headers_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sync_headers_errors_low : " U32H_FMT "\n", ptr_struct->sync_headers_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edpl_bip_errors_lane0_high : " U32H_FMT "\n", ptr_struct->edpl_bip_errors_lane0_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edpl_bip_errors_lane0_low : " U32H_FMT "\n", ptr_struct->edpl_bip_errors_lane0_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edpl_bip_errors_lane1_high : " U32H_FMT "\n", ptr_struct->edpl_bip_errors_lane1_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edpl_bip_errors_lane1_low : " U32H_FMT "\n", ptr_struct->edpl_bip_errors_lane1_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edpl_bip_errors_lane2_high : " U32H_FMT "\n", ptr_struct->edpl_bip_errors_lane2_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edpl_bip_errors_lane2_low : " U32H_FMT "\n", ptr_struct->edpl_bip_errors_lane2_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edpl_bip_errors_lane3_high : " U32H_FMT "\n", ptr_struct->edpl_bip_errors_lane3_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edpl_bip_errors_lane3_low : " U32H_FMT "\n", ptr_struct->edpl_bip_errors_lane3_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_corrected_blocks_lane0_high : " U32H_FMT "\n", ptr_struct->fc_fec_corrected_blocks_lane0_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_corrected_blocks_lane0_low : " U32H_FMT "\n", ptr_struct->fc_fec_corrected_blocks_lane0_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_corrected_blocks_lane1_high : " U32H_FMT "\n", ptr_struct->fc_fec_corrected_blocks_lane1_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_corrected_blocks_lane1_low : " U32H_FMT "\n", ptr_struct->fc_fec_corrected_blocks_lane1_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_corrected_blocks_lane2_high : " U32H_FMT "\n", ptr_struct->fc_fec_corrected_blocks_lane2_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_corrected_blocks_lane2_low : " U32H_FMT "\n", ptr_struct->fc_fec_corrected_blocks_lane2_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_corrected_blocks_lane3_high : " U32H_FMT "\n", ptr_struct->fc_fec_corrected_blocks_lane3_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_corrected_blocks_lane3_low : " U32H_FMT "\n", ptr_struct->fc_fec_corrected_blocks_lane3_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_uncorrectable_blocks_lane0_high : " U32H_FMT "\n", ptr_struct->fc_fec_uncorrectable_blocks_lane0_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_uncorrectable_blocks_lane0_low : " U32H_FMT "\n", ptr_struct->fc_fec_uncorrectable_blocks_lane0_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_uncorrectable_blocks_lane1_high : " U32H_FMT "\n", ptr_struct->fc_fec_uncorrectable_blocks_lane1_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_uncorrectable_blocks_lane1_low : " U32H_FMT "\n", ptr_struct->fc_fec_uncorrectable_blocks_lane1_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_uncorrectable_blocks_lane2_high : " U32H_FMT "\n", ptr_struct->fc_fec_uncorrectable_blocks_lane2_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_uncorrectable_blocks_lane2_low : " U32H_FMT "\n", ptr_struct->fc_fec_uncorrectable_blocks_lane2_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_uncorrectable_blocks_lane3_high : " U32H_FMT "\n", ptr_struct->fc_fec_uncorrectable_blocks_lane3_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_fec_uncorrectable_blocks_lane3_low : " U32H_FMT "\n", ptr_struct->fc_fec_uncorrectable_blocks_lane3_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_blocks_high : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_blocks_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_blocks_low : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_blocks_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_uncorrectable_blocks_high : " U32H_FMT "\n", ptr_struct->rs_fec_uncorrectable_blocks_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_uncorrectable_blocks_low : " U32H_FMT "\n", ptr_struct->rs_fec_uncorrectable_blocks_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_no_errors_blocks_high : " U32H_FMT "\n", ptr_struct->rs_fec_no_errors_blocks_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_no_errors_blocks_low : " U32H_FMT "\n", ptr_struct->rs_fec_no_errors_blocks_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_single_error_blocks_high : " U32H_FMT "\n", ptr_struct->rs_fec_single_error_blocks_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_single_error_blocks_low : " U32H_FMT "\n", ptr_struct->rs_fec_single_error_blocks_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_total_high : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_total_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_total_low : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_total_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_lane0_high : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_lane0_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_lane0_low : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_lane0_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_lane1_high : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_lane1_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_lane1_low : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_lane1_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_lane2_high : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_lane2_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_lane2_low : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_lane2_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_lane3_high : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_lane3_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_corrected_symbols_lane3_low : " U32H_FMT "\n", ptr_struct->rs_fec_corrected_symbols_lane3_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_down_events     : " U32H_FMT "\n", ptr_struct->link_down_events);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "successful_recovery_events : " U32H_FMT "\n", ptr_struct->successful_recovery_events);

}

int connectib_phys_layer_cntrs_size(void)
{
	 return 200;
}

void connectib_phys_layer_cntrs_dump(const struct connectib_phys_layer_cntrs *ptr_struct, FILE* file)
{
	connectib_phys_layer_cntrs_print(ptr_struct, file, 0);
}

void connectib_correct_on_demand_layout_pack(const struct connectib_correct_on_demand_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rewind_counter_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rewind_counter_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data_rewinded_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data_rewinded_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pkt_rewinded_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pkt_rewinded_low);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->total_rcv_data_high);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->total_rcv_data_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->total_rcv_pkt_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->total_rcv_pkt_low);

}

void connectib_correct_on_demand_layout_unpack(struct connectib_correct_on_demand_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->rewind_counter_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->rewind_counter_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->data_rewinded_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->data_rewinded_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->pkt_rewinded_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->pkt_rewinded_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->total_rcv_data_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->total_rcv_data_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->total_rcv_pkt_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->total_rcv_pkt_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_correct_on_demand_layout_print(const struct connectib_correct_on_demand_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_correct_on_demand_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rewind_counter_high  : " U32H_FMT "\n", ptr_struct->rewind_counter_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rewind_counter_low   : " U32H_FMT "\n", ptr_struct->rewind_counter_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_rewinded_high   : " U32H_FMT "\n", ptr_struct->data_rewinded_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_rewinded_low    : " U32H_FMT "\n", ptr_struct->data_rewinded_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkt_rewinded_high    : " U32H_FMT "\n", ptr_struct->pkt_rewinded_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkt_rewinded_low     : " U32H_FMT "\n", ptr_struct->pkt_rewinded_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "total_rcv_data_high  : " U32H_FMT "\n", ptr_struct->total_rcv_data_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "total_rcv_data_low   : " U32H_FMT "\n", ptr_struct->total_rcv_data_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "total_rcv_pkt_high   : " U32H_FMT "\n", ptr_struct->total_rcv_pkt_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "total_rcv_pkt_low    : " U32H_FMT "\n", ptr_struct->total_rcv_pkt_low);

}

int connectib_correct_on_demand_layout_size(void)
{
	 return 56;
}

void connectib_correct_on_demand_layout_dump(const struct connectib_correct_on_demand_layout *ptr_struct, FILE* file)
{
	connectib_correct_on_demand_layout_print(ptr_struct, file, 0);
}

void connectib_eth_per_traffic_grp_data_layout_pack(const struct connectib_eth_per_traffic_grp_data_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->transmit_queue_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->transmit_queue_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->no_buffer_discard_uc_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->no_buffer_discard_uc_low);

}

void connectib_eth_per_traffic_grp_data_layout_unpack(struct connectib_eth_per_traffic_grp_data_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->transmit_queue_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->transmit_queue_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->no_buffer_discard_uc_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->no_buffer_discard_uc_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_eth_per_traffic_grp_data_layout_print(const struct connectib_eth_per_traffic_grp_data_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eth_per_traffic_grp_data_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transmit_queue_high  : " U32H_FMT "\n", ptr_struct->transmit_queue_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transmit_queue_low   : " U32H_FMT "\n", ptr_struct->transmit_queue_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_buffer_discard_uc_high : " U32H_FMT "\n", ptr_struct->no_buffer_discard_uc_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_buffer_discard_uc_low : " U32H_FMT "\n", ptr_struct->no_buffer_discard_uc_low);

}

int connectib_eth_per_traffic_grp_data_layout_size(void)
{
	 return 248;
}

void connectib_eth_per_traffic_grp_data_layout_dump(const struct connectib_eth_per_traffic_grp_data_layout *ptr_struct, FILE* file)
{
	connectib_eth_per_traffic_grp_data_layout_print(ptr_struct, file, 0);
}

void connectib_eth_per_prio_grp_data_layout_pack(const struct connectib_eth_per_prio_grp_data_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_octets_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_octets_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_frames_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_frames_low);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_octets_high);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_octets_low);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_frames_high);

	offset = 608;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_frames_low);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_pause_high);

	offset = 672;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_pause_low);

	offset = 704;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_pause_duration_high);

	offset = 736;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_pause_duration_low);

	offset = 768;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_pause_high);

	offset = 800;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_pause_low);

	offset = 832;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_pause_duration_high);

	offset = 864;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_pause_duration_low);

	offset = 896;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_pause_transition_high);

	offset = 928;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_pause_transition_low);

	offset = 960;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_discards_high);

	offset = 992;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_discards_low);

	offset = 1024;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->device_stall_minor_watermark_cnt_high);

	offset = 1056;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->device_stall_minor_watermark_cnt_low);

	offset = 1088;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->device_stall_critical_watermark_cnt_high);

	offset = 1120;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->device_stall_critical_watermark_cnt_low);

}

void connectib_eth_per_prio_grp_data_layout_unpack(struct connectib_eth_per_prio_grp_data_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->rx_octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->rx_octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->rx_frames_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->rx_frames_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->tx_octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->tx_octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->tx_frames_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 608;
	ptr_struct->tx_frames_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 640;
	ptr_struct->rx_pause_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 672;
	ptr_struct->rx_pause_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 704;
	ptr_struct->rx_pause_duration_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 736;
	ptr_struct->rx_pause_duration_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 768;
	ptr_struct->tx_pause_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 800;
	ptr_struct->tx_pause_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 832;
	ptr_struct->tx_pause_duration_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 864;
	ptr_struct->tx_pause_duration_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 896;
	ptr_struct->rx_pause_transition_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 928;
	ptr_struct->rx_pause_transition_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 960;
	ptr_struct->rx_discards_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 992;
	ptr_struct->rx_discards_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1024;
	ptr_struct->device_stall_minor_watermark_cnt_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1056;
	ptr_struct->device_stall_minor_watermark_cnt_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1088;
	ptr_struct->device_stall_critical_watermark_cnt_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1120;
	ptr_struct->device_stall_critical_watermark_cnt_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_eth_per_prio_grp_data_layout_print(const struct connectib_eth_per_prio_grp_data_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eth_per_prio_grp_data_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_octets_high       : " U32H_FMT "\n", ptr_struct->rx_octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_octets_low        : " U32H_FMT "\n", ptr_struct->rx_octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_frames_high       : " U32H_FMT "\n", ptr_struct->rx_frames_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_frames_low        : " U32H_FMT "\n", ptr_struct->rx_frames_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_octets_high       : " U32H_FMT "\n", ptr_struct->tx_octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_octets_low        : " U32H_FMT "\n", ptr_struct->tx_octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_frames_high       : " U32H_FMT "\n", ptr_struct->tx_frames_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_frames_low        : " U32H_FMT "\n", ptr_struct->tx_frames_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_pause_high        : " U32H_FMT "\n", ptr_struct->rx_pause_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_pause_low         : " U32H_FMT "\n", ptr_struct->rx_pause_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_pause_duration_high : " U32H_FMT "\n", ptr_struct->rx_pause_duration_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_pause_duration_low : " U32H_FMT "\n", ptr_struct->rx_pause_duration_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_pause_high        : " U32H_FMT "\n", ptr_struct->tx_pause_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_pause_low         : " U32H_FMT "\n", ptr_struct->tx_pause_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_pause_duration_high : " U32H_FMT "\n", ptr_struct->tx_pause_duration_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_pause_duration_low : " U32H_FMT "\n", ptr_struct->tx_pause_duration_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_pause_transition_high : " U32H_FMT "\n", ptr_struct->rx_pause_transition_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_pause_transition_low : " U32H_FMT "\n", ptr_struct->rx_pause_transition_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_discards_high     : " U32H_FMT "\n", ptr_struct->rx_discards_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_discards_low      : " U32H_FMT "\n", ptr_struct->rx_discards_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_stall_minor_watermark_cnt_high : " U32H_FMT "\n", ptr_struct->device_stall_minor_watermark_cnt_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_stall_minor_watermark_cnt_low : " U32H_FMT "\n", ptr_struct->device_stall_minor_watermark_cnt_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_stall_critical_watermark_cnt_high : " U32H_FMT "\n", ptr_struct->device_stall_critical_watermark_cnt_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_stall_critical_watermark_cnt_low : " U32H_FMT "\n", ptr_struct->device_stall_critical_watermark_cnt_low);

}

int connectib_eth_per_prio_grp_data_layout_size(void)
{
	 return 248;
}

void connectib_eth_per_prio_grp_data_layout_dump(const struct connectib_eth_per_prio_grp_data_layout *ptr_struct, FILE* file)
{
	connectib_eth_per_prio_grp_data_layout_print(ptr_struct, file, 0);
}

void connectib_eth_extended_cntrs_grp_data_layout_pack(const struct connectib_eth_extended_cntrs_grp_data_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_transmit_wait_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_transmit_wait_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ecn_marked_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ecn_marked_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->no_buffer_discard_mc_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->no_buffer_discard_mc_low);

}

void connectib_eth_extended_cntrs_grp_data_layout_unpack(struct connectib_eth_extended_cntrs_grp_data_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->port_transmit_wait_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->port_transmit_wait_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->ecn_marked_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->ecn_marked_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->no_buffer_discard_mc_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->no_buffer_discard_mc_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_eth_extended_cntrs_grp_data_layout_print(const struct connectib_eth_extended_cntrs_grp_data_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eth_extended_cntrs_grp_data_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_transmit_wait_high : " U32H_FMT "\n", ptr_struct->port_transmit_wait_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_transmit_wait_low : " U32H_FMT "\n", ptr_struct->port_transmit_wait_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecn_marked_high      : " U32H_FMT "\n", ptr_struct->ecn_marked_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecn_marked_low       : " U32H_FMT "\n", ptr_struct->ecn_marked_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_buffer_discard_mc_high : " U32H_FMT "\n", ptr_struct->no_buffer_discard_mc_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_buffer_discard_mc_low : " U32H_FMT "\n", ptr_struct->no_buffer_discard_mc_low);

}

int connectib_eth_extended_cntrs_grp_data_layout_size(void)
{
	 return 248;
}

void connectib_eth_extended_cntrs_grp_data_layout_dump(const struct connectib_eth_extended_cntrs_grp_data_layout *ptr_struct, FILE* file)
{
	connectib_eth_extended_cntrs_grp_data_layout_print(ptr_struct, file, 0);
}

void connectib_eth_3635_cntrs_grp_data_layout_pack(const struct connectib_eth_3635_cntrs_grp_data_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_alignment_errors_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_alignment_errors_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_fcs_errors_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_fcs_errors_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_single_collision_frames_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_single_collision_frames_low);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_multiple_collision_frames_high);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_multiple_collision_frames_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_sqe_test_errors_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_sqe_test_errors_low);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_deferred_transmissions_high);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_deferred_transmissions_low);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_late_collisions_high);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_late_collisions_low);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_excessive_collisions_high);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_excessive_collisions_low);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_internal_mac_transmit_errors_high);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_internal_mac_transmit_errors_low);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_carrier_sense_errors_high);

	offset = 608;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_carrier_sense_errors_low);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_frame_too_longs_high);

	offset = 672;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_frame_too_longs_low);

	offset = 704;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_internal_mac_receive_errors_high);

	offset = 736;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_internal_mac_receive_errors_low);

	offset = 768;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_symbol_errors_high);

	offset = 800;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3stats_symbol_errors_low);

	offset = 832;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3control_in_unknown_opcodes_high);

	offset = 864;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3control_in_unknown_opcodes_low);

	offset = 896;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3in_pause_frames_high);

	offset = 928;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3in_pause_frames_low);

	offset = 960;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3out_pause_frames_high);

	offset = 992;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dot3out_pause_frames_low);

}

void connectib_eth_3635_cntrs_grp_data_layout_unpack(struct connectib_eth_3635_cntrs_grp_data_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->dot3stats_alignment_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->dot3stats_alignment_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->dot3stats_fcs_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->dot3stats_fcs_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->dot3stats_single_collision_frames_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->dot3stats_single_collision_frames_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->dot3stats_multiple_collision_frames_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->dot3stats_multiple_collision_frames_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->dot3stats_sqe_test_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->dot3stats_sqe_test_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->dot3stats_deferred_transmissions_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->dot3stats_deferred_transmissions_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->dot3stats_late_collisions_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->dot3stats_late_collisions_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->dot3stats_excessive_collisions_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->dot3stats_excessive_collisions_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 512;
	ptr_struct->dot3stats_internal_mac_transmit_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->dot3stats_internal_mac_transmit_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->dot3stats_carrier_sense_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 608;
	ptr_struct->dot3stats_carrier_sense_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 640;
	ptr_struct->dot3stats_frame_too_longs_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 672;
	ptr_struct->dot3stats_frame_too_longs_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 704;
	ptr_struct->dot3stats_internal_mac_receive_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 736;
	ptr_struct->dot3stats_internal_mac_receive_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 768;
	ptr_struct->dot3stats_symbol_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 800;
	ptr_struct->dot3stats_symbol_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 832;
	ptr_struct->dot3control_in_unknown_opcodes_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 864;
	ptr_struct->dot3control_in_unknown_opcodes_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 896;
	ptr_struct->dot3in_pause_frames_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 928;
	ptr_struct->dot3in_pause_frames_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 960;
	ptr_struct->dot3out_pause_frames_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 992;
	ptr_struct->dot3out_pause_frames_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_eth_3635_cntrs_grp_data_layout_print(const struct connectib_eth_3635_cntrs_grp_data_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eth_3635_cntrs_grp_data_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_alignment_errors_high : " U32H_FMT "\n", ptr_struct->dot3stats_alignment_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_alignment_errors_low : " U32H_FMT "\n", ptr_struct->dot3stats_alignment_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_fcs_errors_high : " U32H_FMT "\n", ptr_struct->dot3stats_fcs_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_fcs_errors_low : " U32H_FMT "\n", ptr_struct->dot3stats_fcs_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_single_collision_frames_high : " U32H_FMT "\n", ptr_struct->dot3stats_single_collision_frames_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_single_collision_frames_low : " U32H_FMT "\n", ptr_struct->dot3stats_single_collision_frames_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_multiple_collision_frames_high : " U32H_FMT "\n", ptr_struct->dot3stats_multiple_collision_frames_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_multiple_collision_frames_low : " U32H_FMT "\n", ptr_struct->dot3stats_multiple_collision_frames_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_sqe_test_errors_high : " U32H_FMT "\n", ptr_struct->dot3stats_sqe_test_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_sqe_test_errors_low : " U32H_FMT "\n", ptr_struct->dot3stats_sqe_test_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_deferred_transmissions_high : " U32H_FMT "\n", ptr_struct->dot3stats_deferred_transmissions_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_deferred_transmissions_low : " U32H_FMT "\n", ptr_struct->dot3stats_deferred_transmissions_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_late_collisions_high : " U32H_FMT "\n", ptr_struct->dot3stats_late_collisions_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_late_collisions_low : " U32H_FMT "\n", ptr_struct->dot3stats_late_collisions_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_excessive_collisions_high : " U32H_FMT "\n", ptr_struct->dot3stats_excessive_collisions_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_excessive_collisions_low : " U32H_FMT "\n", ptr_struct->dot3stats_excessive_collisions_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_internal_mac_transmit_errors_high : " U32H_FMT "\n", ptr_struct->dot3stats_internal_mac_transmit_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_internal_mac_transmit_errors_low : " U32H_FMT "\n", ptr_struct->dot3stats_internal_mac_transmit_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_carrier_sense_errors_high : " U32H_FMT "\n", ptr_struct->dot3stats_carrier_sense_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_carrier_sense_errors_low : " U32H_FMT "\n", ptr_struct->dot3stats_carrier_sense_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_frame_too_longs_high : " U32H_FMT "\n", ptr_struct->dot3stats_frame_too_longs_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_frame_too_longs_low : " U32H_FMT "\n", ptr_struct->dot3stats_frame_too_longs_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_internal_mac_receive_errors_high : " U32H_FMT "\n", ptr_struct->dot3stats_internal_mac_receive_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_internal_mac_receive_errors_low : " U32H_FMT "\n", ptr_struct->dot3stats_internal_mac_receive_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_symbol_errors_high : " U32H_FMT "\n", ptr_struct->dot3stats_symbol_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3stats_symbol_errors_low : " U32H_FMT "\n", ptr_struct->dot3stats_symbol_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3control_in_unknown_opcodes_high : " U32H_FMT "\n", ptr_struct->dot3control_in_unknown_opcodes_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3control_in_unknown_opcodes_low : " U32H_FMT "\n", ptr_struct->dot3control_in_unknown_opcodes_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3in_pause_frames_high : " U32H_FMT "\n", ptr_struct->dot3in_pause_frames_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3in_pause_frames_low : " U32H_FMT "\n", ptr_struct->dot3in_pause_frames_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3out_pause_frames_high : " U32H_FMT "\n", ptr_struct->dot3out_pause_frames_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dot3out_pause_frames_low : " U32H_FMT "\n", ptr_struct->dot3out_pause_frames_low);

}

int connectib_eth_3635_cntrs_grp_data_layout_size(void)
{
	 return 248;
}

void connectib_eth_3635_cntrs_grp_data_layout_dump(const struct connectib_eth_3635_cntrs_grp_data_layout *ptr_struct, FILE* file)
{
	connectib_eth_3635_cntrs_grp_data_layout_print(ptr_struct, file, 0);
}

void connectib_eth_2819_cntrs_grp_data_layout_pack(const struct connectib_eth_2819_cntrs_grp_data_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_drop_events_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_drop_events_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_octets_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_octets_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts_low);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_broadcast_pkts_high);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_broadcast_pkts_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_multicast_pkts_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_multicast_pkts_low);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_crc_align_errors_high);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_crc_align_errors_low);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_undersize_pkts_high);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_undersize_pkts_low);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_oversize_pkts_high);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_oversize_pkts_low);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_fragments_high);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_fragments_low);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_jabbers_high);

	offset = 608;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_jabbers_low);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_collisions_high);

	offset = 672;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_collisions_low);

	offset = 704;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts64octets_high);

	offset = 736;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts64octets_low);

	offset = 768;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts65to127octets_high);

	offset = 800;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts65to127octets_low);

	offset = 832;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts128to255octets_high);

	offset = 864;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts128to255octets_low);

	offset = 896;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts256to511octets_high);

	offset = 928;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts256to511octets_low);

	offset = 960;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts512to1023octets_high);

	offset = 992;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts512to1023octets_low);

	offset = 1024;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts1024to1518octets_high);

	offset = 1056;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts1024to1518octets_low);

	offset = 1088;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts1519to2047octets_high);

	offset = 1120;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts1519to2047octets_low);

	offset = 1152;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts2048to4095octets_high);

	offset = 1184;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts2048to4095octets_low);

	offset = 1216;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts4096to8191octets_high);

	offset = 1248;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts4096to8191octets_low);

	offset = 1280;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts8192to10239octets_high);

	offset = 1312;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ether_stats_pkts8192to10239octets_low);

}

void connectib_eth_2819_cntrs_grp_data_layout_unpack(struct connectib_eth_2819_cntrs_grp_data_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->ether_stats_drop_events_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->ether_stats_drop_events_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->ether_stats_octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->ether_stats_octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->ether_stats_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->ether_stats_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->ether_stats_broadcast_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->ether_stats_broadcast_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->ether_stats_multicast_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->ether_stats_multicast_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->ether_stats_crc_align_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->ether_stats_crc_align_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->ether_stats_undersize_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->ether_stats_undersize_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->ether_stats_oversize_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->ether_stats_oversize_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 512;
	ptr_struct->ether_stats_fragments_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->ether_stats_fragments_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->ether_stats_jabbers_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 608;
	ptr_struct->ether_stats_jabbers_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 640;
	ptr_struct->ether_stats_collisions_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 672;
	ptr_struct->ether_stats_collisions_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 704;
	ptr_struct->ether_stats_pkts64octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 736;
	ptr_struct->ether_stats_pkts64octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 768;
	ptr_struct->ether_stats_pkts65to127octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 800;
	ptr_struct->ether_stats_pkts65to127octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 832;
	ptr_struct->ether_stats_pkts128to255octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 864;
	ptr_struct->ether_stats_pkts128to255octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 896;
	ptr_struct->ether_stats_pkts256to511octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 928;
	ptr_struct->ether_stats_pkts256to511octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 960;
	ptr_struct->ether_stats_pkts512to1023octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 992;
	ptr_struct->ether_stats_pkts512to1023octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1024;
	ptr_struct->ether_stats_pkts1024to1518octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1056;
	ptr_struct->ether_stats_pkts1024to1518octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1088;
	ptr_struct->ether_stats_pkts1519to2047octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1120;
	ptr_struct->ether_stats_pkts1519to2047octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1152;
	ptr_struct->ether_stats_pkts2048to4095octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1184;
	ptr_struct->ether_stats_pkts2048to4095octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1216;
	ptr_struct->ether_stats_pkts4096to8191octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1248;
	ptr_struct->ether_stats_pkts4096to8191octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1280;
	ptr_struct->ether_stats_pkts8192to10239octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1312;
	ptr_struct->ether_stats_pkts8192to10239octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_eth_2819_cntrs_grp_data_layout_print(const struct connectib_eth_2819_cntrs_grp_data_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eth_2819_cntrs_grp_data_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_drop_events_high : " U32H_FMT "\n", ptr_struct->ether_stats_drop_events_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_drop_events_low : " U32H_FMT "\n", ptr_struct->ether_stats_drop_events_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_broadcast_pkts_high : " U32H_FMT "\n", ptr_struct->ether_stats_broadcast_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_broadcast_pkts_low : " U32H_FMT "\n", ptr_struct->ether_stats_broadcast_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_multicast_pkts_high : " U32H_FMT "\n", ptr_struct->ether_stats_multicast_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_multicast_pkts_low : " U32H_FMT "\n", ptr_struct->ether_stats_multicast_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_crc_align_errors_high : " U32H_FMT "\n", ptr_struct->ether_stats_crc_align_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_crc_align_errors_low : " U32H_FMT "\n", ptr_struct->ether_stats_crc_align_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_undersize_pkts_high : " U32H_FMT "\n", ptr_struct->ether_stats_undersize_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_undersize_pkts_low : " U32H_FMT "\n", ptr_struct->ether_stats_undersize_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_oversize_pkts_high : " U32H_FMT "\n", ptr_struct->ether_stats_oversize_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_oversize_pkts_low : " U32H_FMT "\n", ptr_struct->ether_stats_oversize_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_fragments_high : " U32H_FMT "\n", ptr_struct->ether_stats_fragments_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_fragments_low : " U32H_FMT "\n", ptr_struct->ether_stats_fragments_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_jabbers_high : " U32H_FMT "\n", ptr_struct->ether_stats_jabbers_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_jabbers_low : " U32H_FMT "\n", ptr_struct->ether_stats_jabbers_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_collisions_high : " U32H_FMT "\n", ptr_struct->ether_stats_collisions_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_collisions_low : " U32H_FMT "\n", ptr_struct->ether_stats_collisions_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts64octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts64octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts64octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts64octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts65to127octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts65to127octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts65to127octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts65to127octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts128to255octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts128to255octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts128to255octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts128to255octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts256to511octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts256to511octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts256to511octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts256to511octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts512to1023octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts512to1023octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts512to1023octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts512to1023octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts1024to1518octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts1024to1518octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts1024to1518octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts1024to1518octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts1519to2047octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts1519to2047octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts1519to2047octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts1519to2047octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts2048to4095octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts2048to4095octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts2048to4095octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts2048to4095octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts4096to8191octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts4096to8191octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts4096to8191octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts4096to8191octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts8192to10239octets_high : " U32H_FMT "\n", ptr_struct->ether_stats_pkts8192to10239octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ether_stats_pkts8192to10239octets_low : " U32H_FMT "\n", ptr_struct->ether_stats_pkts8192to10239octets_low);

}

int connectib_eth_2819_cntrs_grp_data_layout_size(void)
{
	 return 248;
}

void connectib_eth_2819_cntrs_grp_data_layout_dump(const struct connectib_eth_2819_cntrs_grp_data_layout *ptr_struct, FILE* file)
{
	connectib_eth_2819_cntrs_grp_data_layout_print(ptr_struct, file, 0);
}

void connectib_eth_2863_cntrs_grp_data_layout_pack(const struct connectib_eth_2863_cntrs_grp_data_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_octets_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_octets_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_ucast_pkts_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_ucast_pkts_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_discards_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_discards_low);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_errors_high);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_errors_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_unknown_protos_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_unknown_protos_low);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_octets_high);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_octets_low);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_ucast_pkts_high);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_ucast_pkts_low);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_discards_high);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_discards_low);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_errors_high);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_errors_low);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_multicast_pkts_high);

	offset = 608;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_multicast_pkts_low);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_broadcast_pkts_high);

	offset = 672;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_in_broadcast_pkts_low);

	offset = 704;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_multicast_pkts_high);

	offset = 736;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_multicast_pkts_low);

	offset = 768;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_broadcast_pkts_high);

	offset = 800;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->if_out_broadcast_pkts_low);

}

void connectib_eth_2863_cntrs_grp_data_layout_unpack(struct connectib_eth_2863_cntrs_grp_data_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->if_in_octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->if_in_octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->if_in_ucast_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->if_in_ucast_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->if_in_discards_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->if_in_discards_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->if_in_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->if_in_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->if_in_unknown_protos_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->if_in_unknown_protos_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->if_out_octets_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->if_out_octets_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->if_out_ucast_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->if_out_ucast_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->if_out_discards_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->if_out_discards_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 512;
	ptr_struct->if_out_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->if_out_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->if_in_multicast_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 608;
	ptr_struct->if_in_multicast_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 640;
	ptr_struct->if_in_broadcast_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 672;
	ptr_struct->if_in_broadcast_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 704;
	ptr_struct->if_out_multicast_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 736;
	ptr_struct->if_out_multicast_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 768;
	ptr_struct->if_out_broadcast_pkts_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 800;
	ptr_struct->if_out_broadcast_pkts_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_eth_2863_cntrs_grp_data_layout_print(const struct connectib_eth_2863_cntrs_grp_data_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eth_2863_cntrs_grp_data_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_octets_high    : " U32H_FMT "\n", ptr_struct->if_in_octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_octets_low     : " U32H_FMT "\n", ptr_struct->if_in_octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_ucast_pkts_high : " U32H_FMT "\n", ptr_struct->if_in_ucast_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_ucast_pkts_low : " U32H_FMT "\n", ptr_struct->if_in_ucast_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_discards_high  : " U32H_FMT "\n", ptr_struct->if_in_discards_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_discards_low   : " U32H_FMT "\n", ptr_struct->if_in_discards_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_errors_high    : " U32H_FMT "\n", ptr_struct->if_in_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_errors_low     : " U32H_FMT "\n", ptr_struct->if_in_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_unknown_protos_high : " U32H_FMT "\n", ptr_struct->if_in_unknown_protos_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_unknown_protos_low : " U32H_FMT "\n", ptr_struct->if_in_unknown_protos_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_octets_high   : " U32H_FMT "\n", ptr_struct->if_out_octets_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_octets_low    : " U32H_FMT "\n", ptr_struct->if_out_octets_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_ucast_pkts_high : " U32H_FMT "\n", ptr_struct->if_out_ucast_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_ucast_pkts_low : " U32H_FMT "\n", ptr_struct->if_out_ucast_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_discards_high : " U32H_FMT "\n", ptr_struct->if_out_discards_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_discards_low  : " U32H_FMT "\n", ptr_struct->if_out_discards_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_errors_high   : " U32H_FMT "\n", ptr_struct->if_out_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_errors_low    : " U32H_FMT "\n", ptr_struct->if_out_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_multicast_pkts_high : " U32H_FMT "\n", ptr_struct->if_in_multicast_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_multicast_pkts_low : " U32H_FMT "\n", ptr_struct->if_in_multicast_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_broadcast_pkts_high : " U32H_FMT "\n", ptr_struct->if_in_broadcast_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_in_broadcast_pkts_low : " U32H_FMT "\n", ptr_struct->if_in_broadcast_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_multicast_pkts_high : " U32H_FMT "\n", ptr_struct->if_out_multicast_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_multicast_pkts_low : " U32H_FMT "\n", ptr_struct->if_out_multicast_pkts_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_broadcast_pkts_high : " U32H_FMT "\n", ptr_struct->if_out_broadcast_pkts_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "if_out_broadcast_pkts_low : " U32H_FMT "\n", ptr_struct->if_out_broadcast_pkts_low);

}

int connectib_eth_2863_cntrs_grp_data_layout_size(void)
{
	 return 248;
}

void connectib_eth_2863_cntrs_grp_data_layout_dump(const struct connectib_eth_2863_cntrs_grp_data_layout *ptr_struct, FILE* file)
{
	connectib_eth_2863_cntrs_grp_data_layout_print(ptr_struct, file, 0);
}

void connectib_eth_802_3_cntrs_grp_data_layout_pack(const struct connectib_eth_802_3_cntrs_grp_data_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_frames_transmitted_ok_high);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_frames_transmitted_ok_low);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_frames_received_ok_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_frames_received_ok_low);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_frame_check_sequence_errors_high);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_frame_check_sequence_errors_low);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_alignment_errors_high);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_alignment_errors_low);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_octets_transmitted_ok_high);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_octets_transmitted_ok_low);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_octets_received_ok_high);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_octets_received_ok_low);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_multicast_frames_xmitted_ok_high);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_multicast_frames_xmitted_ok_low);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_broadcast_frames_xmitted_ok_high);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_broadcast_frames_xmitted_ok_low);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_multicast_frames_received_ok_high);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_multicast_frames_received_ok_low);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_broadcast_frames_recieved_ok_high);

	offset = 608;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_broadcast_frames_recieved_ok_low);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_in_range_length_errors_high);

	offset = 672;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_in_range_length_errors_low);

	offset = 704;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_out_of_range_length_field_high);

	offset = 736;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_out_of_range_length_field_low);

	offset = 768;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_frame_too_long_errors_high);

	offset = 800;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_frame_too_long_errors_low);

	offset = 832;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_symbol_error_during_carrier_high);

	offset = 864;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_symbol_error_during_carrier_low);

	offset = 896;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_mac_control_frames_transmitted_high);

	offset = 928;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_mac_control_frames_transmitted_low);

	offset = 960;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_mac_control_frames_received_high);

	offset = 992;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_mac_control_frames_received_low);

	offset = 1024;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_unsupported_opcodes_received_high);

	offset = 1056;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_unsupported_opcodes_received_low);

	offset = 1088;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_pause_mac_ctrl_frames_received_high);

	offset = 1120;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_pause_mac_ctrl_frames_received_low);

	offset = 1152;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_pause_mac_ctrl_frames_transmitted_high);

	offset = 1184;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->a_pause_mac_ctrl_frames_transmitted_low);

}

void connectib_eth_802_3_cntrs_grp_data_layout_unpack(struct connectib_eth_802_3_cntrs_grp_data_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->a_frames_transmitted_ok_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->a_frames_transmitted_ok_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->a_frames_received_ok_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->a_frames_received_ok_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->a_frame_check_sequence_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->a_frame_check_sequence_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->a_alignment_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->a_alignment_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->a_octets_transmitted_ok_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->a_octets_transmitted_ok_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->a_octets_received_ok_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->a_octets_received_ok_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->a_multicast_frames_xmitted_ok_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->a_multicast_frames_xmitted_ok_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->a_broadcast_frames_xmitted_ok_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->a_broadcast_frames_xmitted_ok_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 512;
	ptr_struct->a_multicast_frames_received_ok_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->a_multicast_frames_received_ok_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->a_broadcast_frames_recieved_ok_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 608;
	ptr_struct->a_broadcast_frames_recieved_ok_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 640;
	ptr_struct->a_in_range_length_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 672;
	ptr_struct->a_in_range_length_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 704;
	ptr_struct->a_out_of_range_length_field_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 736;
	ptr_struct->a_out_of_range_length_field_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 768;
	ptr_struct->a_frame_too_long_errors_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 800;
	ptr_struct->a_frame_too_long_errors_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 832;
	ptr_struct->a_symbol_error_during_carrier_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 864;
	ptr_struct->a_symbol_error_during_carrier_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 896;
	ptr_struct->a_mac_control_frames_transmitted_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 928;
	ptr_struct->a_mac_control_frames_transmitted_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 960;
	ptr_struct->a_mac_control_frames_received_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 992;
	ptr_struct->a_mac_control_frames_received_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1024;
	ptr_struct->a_unsupported_opcodes_received_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1056;
	ptr_struct->a_unsupported_opcodes_received_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1088;
	ptr_struct->a_pause_mac_ctrl_frames_received_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1120;
	ptr_struct->a_pause_mac_ctrl_frames_received_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1152;
	ptr_struct->a_pause_mac_ctrl_frames_transmitted_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1184;
	ptr_struct->a_pause_mac_ctrl_frames_transmitted_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_eth_802_3_cntrs_grp_data_layout_print(const struct connectib_eth_802_3_cntrs_grp_data_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eth_802_3_cntrs_grp_data_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_frames_transmitted_ok_high : " U32H_FMT "\n", ptr_struct->a_frames_transmitted_ok_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_frames_transmitted_ok_low : " U32H_FMT "\n", ptr_struct->a_frames_transmitted_ok_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_frames_received_ok_high : " U32H_FMT "\n", ptr_struct->a_frames_received_ok_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_frames_received_ok_low : " U32H_FMT "\n", ptr_struct->a_frames_received_ok_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_frame_check_sequence_errors_high : " U32H_FMT "\n", ptr_struct->a_frame_check_sequence_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_frame_check_sequence_errors_low : " U32H_FMT "\n", ptr_struct->a_frame_check_sequence_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_alignment_errors_high : " U32H_FMT "\n", ptr_struct->a_alignment_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_alignment_errors_low : " U32H_FMT "\n", ptr_struct->a_alignment_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_octets_transmitted_ok_high : " U32H_FMT "\n", ptr_struct->a_octets_transmitted_ok_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_octets_transmitted_ok_low : " U32H_FMT "\n", ptr_struct->a_octets_transmitted_ok_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_octets_received_ok_high : " U32H_FMT "\n", ptr_struct->a_octets_received_ok_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_octets_received_ok_low : " U32H_FMT "\n", ptr_struct->a_octets_received_ok_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_multicast_frames_xmitted_ok_high : " U32H_FMT "\n", ptr_struct->a_multicast_frames_xmitted_ok_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_multicast_frames_xmitted_ok_low : " U32H_FMT "\n", ptr_struct->a_multicast_frames_xmitted_ok_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_broadcast_frames_xmitted_ok_high : " U32H_FMT "\n", ptr_struct->a_broadcast_frames_xmitted_ok_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_broadcast_frames_xmitted_ok_low : " U32H_FMT "\n", ptr_struct->a_broadcast_frames_xmitted_ok_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_multicast_frames_received_ok_high : " U32H_FMT "\n", ptr_struct->a_multicast_frames_received_ok_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_multicast_frames_received_ok_low : " U32H_FMT "\n", ptr_struct->a_multicast_frames_received_ok_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_broadcast_frames_recieved_ok_high : " U32H_FMT "\n", ptr_struct->a_broadcast_frames_recieved_ok_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_broadcast_frames_recieved_ok_low : " U32H_FMT "\n", ptr_struct->a_broadcast_frames_recieved_ok_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_in_range_length_errors_high : " U32H_FMT "\n", ptr_struct->a_in_range_length_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_in_range_length_errors_low : " U32H_FMT "\n", ptr_struct->a_in_range_length_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_out_of_range_length_field_high : " U32H_FMT "\n", ptr_struct->a_out_of_range_length_field_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_out_of_range_length_field_low : " U32H_FMT "\n", ptr_struct->a_out_of_range_length_field_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_frame_too_long_errors_high : " U32H_FMT "\n", ptr_struct->a_frame_too_long_errors_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_frame_too_long_errors_low : " U32H_FMT "\n", ptr_struct->a_frame_too_long_errors_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_symbol_error_during_carrier_high : " U32H_FMT "\n", ptr_struct->a_symbol_error_during_carrier_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_symbol_error_during_carrier_low : " U32H_FMT "\n", ptr_struct->a_symbol_error_during_carrier_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_mac_control_frames_transmitted_high : " U32H_FMT "\n", ptr_struct->a_mac_control_frames_transmitted_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_mac_control_frames_transmitted_low : " U32H_FMT "\n", ptr_struct->a_mac_control_frames_transmitted_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_mac_control_frames_received_high : " U32H_FMT "\n", ptr_struct->a_mac_control_frames_received_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_mac_control_frames_received_low : " U32H_FMT "\n", ptr_struct->a_mac_control_frames_received_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_unsupported_opcodes_received_high : " U32H_FMT "\n", ptr_struct->a_unsupported_opcodes_received_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_unsupported_opcodes_received_low : " U32H_FMT "\n", ptr_struct->a_unsupported_opcodes_received_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_pause_mac_ctrl_frames_received_high : " U32H_FMT "\n", ptr_struct->a_pause_mac_ctrl_frames_received_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_pause_mac_ctrl_frames_received_low : " U32H_FMT "\n", ptr_struct->a_pause_mac_ctrl_frames_received_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_pause_mac_ctrl_frames_transmitted_high : " U32H_FMT "\n", ptr_struct->a_pause_mac_ctrl_frames_transmitted_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a_pause_mac_ctrl_frames_transmitted_low : " U32H_FMT "\n", ptr_struct->a_pause_mac_ctrl_frames_transmitted_low);

}

int connectib_eth_802_3_cntrs_grp_data_layout_size(void)
{
	 return 248;
}

void connectib_eth_802_3_cntrs_grp_data_layout_dump(const struct connectib_eth_802_3_cntrs_grp_data_layout *ptr_struct, FILE* file)
{
	connectib_eth_802_3_cntrs_grp_data_layout_print(ptr_struct, file, 0);
}

void connectib_dct_qp_pack(const struct connectib_dct_qp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->dc_access_key_violation_count_state);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dc_access_key_violation_count);

}

void connectib_dct_qp_unpack(struct connectib_dct_qp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	ptr_struct->dc_access_key_violation_count_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 64;
	ptr_struct->dc_access_key_violation_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_dct_qp_print(const struct connectib_dct_qp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dct_qp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_access_key_violation_count_state : " UH_FMT "\n", ptr_struct->dc_access_key_violation_count_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_access_key_violation_count : " U32H_FMT "\n", ptr_struct->dc_access_key_violation_count);

}

int connectib_dct_qp_size(void)
{
	 return 16;
}

void connectib_dct_qp_dump(const struct connectib_dct_qp *ptr_struct, FILE* file)
{
	connectib_dct_qp_print(ptr_struct, file, 0);
}

void connectib_dcr_qp_pack(const struct connectib_dcr_qp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->gb_first_psn);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->gb_last_psn);

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_gb);

	offset = 38;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->drtl_cause_dcr_cleanup);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->reconnect_ack_psn);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dcr_heartbeat_clr_timestamp);

}

void connectib_dcr_qp_unpack(struct connectib_dcr_qp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->gb_first_psn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 40;
	ptr_struct->gb_last_psn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 39;
	ptr_struct->is_gb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 38;
	ptr_struct->drtl_cause_dcr_cleanup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->reconnect_ack_psn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->dcr_heartbeat_clr_timestamp = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_dcr_qp_print(const struct connectib_dcr_qp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dcr_qp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gb_first_psn         : " UH_FMT "\n", ptr_struct->gb_first_psn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gb_last_psn          : " UH_FMT "\n", ptr_struct->gb_last_psn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_gb                : " UH_FMT "\n", ptr_struct->is_gb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "drtl_cause_dcr_cleanup : " UH_FMT "\n", ptr_struct->drtl_cause_dcr_cleanup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reconnect_ack_psn    : " UH_FMT "\n", ptr_struct->reconnect_ack_psn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcr_heartbeat_clr_timestamp : " U32H_FMT "\n", ptr_struct->dcr_heartbeat_clr_timestamp);

}

int connectib_dcr_qp_size(void)
{
	 return 16;
}

void connectib_dcr_qp_dump(const struct connectib_dcr_qp *ptr_struct, FILE* file)
{
	connectib_dcr_qp_print(ptr_struct, file, 0);
}

void connectib_regular_qp_pack(const struct connectib_regular_qp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ds_exceeds_pi_retries);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->req_retrans_is_rdma_read);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->use_deth_sqpn);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->pm_state);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->xport_retry_counter);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->xport_retry_limit);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rnr_retry_counter);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rnr_retry_limit);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->sw_uar);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log_rtm);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sw_log_rtm);

	offset = 64;
	connectib_regular_qp_sub_type_pack(&(ptr_struct->sub_type_fields), ptr_buff + offset/8);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->ib_ft_root_id);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sw_xport_retry_limit);

}

void connectib_regular_qp_unpack(struct connectib_regular_qp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->ds_exceeds_pi_retries = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 27;
	ptr_struct->req_retrans_is_rdma_read = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->use_deth_sqpn = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->pm_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 16;
	ptr_struct->xport_retry_counter = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->xport_retry_limit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 4;
	ptr_struct->rnr_retry_counter = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->rnr_retry_limit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 40;
	ptr_struct->sw_uar = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 36;
	ptr_struct->log_rtm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->sw_log_rtm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 64;
	connectib_regular_qp_sub_type_unpack(&(ptr_struct->sub_type_fields), ptr_buff + offset/8);

	offset = 104;
	ptr_struct->ib_ft_root_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->sw_xport_retry_limit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_regular_qp_print(const struct connectib_regular_qp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_regular_qp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ds_exceeds_pi_retries : " UH_FMT "\n", ptr_struct->ds_exceeds_pi_retries);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "req_retrans_is_rdma_read : " UH_FMT "\n", ptr_struct->req_retrans_is_rdma_read);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "use_deth_sqpn        : " UH_FMT "\n", ptr_struct->use_deth_sqpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pm_state             : " UH_FMT "\n", ptr_struct->pm_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xport_retry_counter  : " UH_FMT "\n", ptr_struct->xport_retry_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xport_retry_limit    : " UH_FMT "\n", ptr_struct->xport_retry_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rnr_retry_counter    : " UH_FMT "\n", ptr_struct->rnr_retry_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rnr_retry_limit      : " UH_FMT "\n", ptr_struct->rnr_retry_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_uar               : " UH_FMT "\n", ptr_struct->sw_uar);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_rtm              : " UH_FMT "\n", ptr_struct->log_rtm);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_log_rtm           : " UH_FMT "\n", ptr_struct->sw_log_rtm);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sub_type_fields:\n");
	connectib_regular_qp_sub_type_print(&(ptr_struct->sub_type_fields), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_ft_root_id        : " UH_FMT "\n", ptr_struct->ib_ft_root_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_xport_retry_limit : " UH_FMT "\n", ptr_struct->sw_xport_retry_limit);

}

int connectib_regular_qp_size(void)
{
	 return 16;
}

void connectib_regular_qp_dump(const struct connectib_regular_qp *ptr_struct, FILE* file)
{
	connectib_regular_qp_print(ptr_struct, file, 0);
}

void connectib_schedint_cause_pack(const struct connectib_schedint_cause *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->clear_all_causes);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->schedint_destroy_qp);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->schedint_qp_flush);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->schedint_apm);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->schedint_duplicate);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->schedint_sl_diff);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->schedint_pp_diff);

}

void connectib_schedint_cause_unpack(struct connectib_schedint_cause *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	ptr_struct->clear_all_causes = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->schedint_destroy_qp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->schedint_qp_flush = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->schedint_apm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->schedint_duplicate = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->schedint_sl_diff = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->schedint_pp_diff = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_schedint_cause_print(const struct connectib_schedint_cause *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_schedint_cause ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "clear_all_causes     : " UH_FMT "\n", ptr_struct->clear_all_causes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "schedint_destroy_qp  : " UH_FMT "\n", ptr_struct->schedint_destroy_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "schedint_qp_flush    : " UH_FMT "\n", ptr_struct->schedint_qp_flush);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "schedint_apm         : " UH_FMT "\n", ptr_struct->schedint_apm);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "schedint_duplicate   : " UH_FMT "\n", ptr_struct->schedint_duplicate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "schedint_sl_diff     : " UH_FMT "\n", ptr_struct->schedint_sl_diff);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "schedint_pp_diff     : " UH_FMT "\n", ptr_struct->schedint_pp_diff);

}

int connectib_schedint_cause_size(void)
{
	 return 1;
}

void connectib_schedint_cause_dump(const struct connectib_schedint_cause *ptr_struct, FILE* file)
{
	connectib_schedint_cause_print(ptr_struct, file, 0);
}

void connectib_fw_qpc_ib_pack(const struct connectib_fw_qpc_ib *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->oldest_non_acked_psn);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->invalid_user_index);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->max_read_atomic_reduced);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->ulp_stateless_offload_mode);

}

void connectib_fw_qpc_ib_unpack(struct connectib_fw_qpc_ib *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->oldest_non_acked_psn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 7;
	ptr_struct->invalid_user_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->max_read_atomic_reduced = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->ulp_stateless_offload_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_fw_qpc_ib_print(const struct connectib_fw_qpc_ib *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_qpc_ib ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "oldest_non_acked_psn : " UH_FMT "\n", ptr_struct->oldest_non_acked_psn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "invalid_user_index   : " UH_FMT "\n", ptr_struct->invalid_user_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_read_atomic_reduced : " UH_FMT "\n", ptr_struct->max_read_atomic_reduced);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ulp_stateless_offload_mode : " UH_FMT "\n", ptr_struct->ulp_stateless_offload_mode);

}

int connectib_fw_qpc_ib_size(void)
{
	 return 4;
}

void connectib_fw_qpc_ib_dump(const struct connectib_fw_qpc_ib *ptr_struct, FILE* file)
{
	connectib_fw_qpc_ib_print(ptr_struct, file, 0);
}

void connectib_fw_qpc_eth_pack(const struct connectib_fw_qpc_eth *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->tis_num);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sq_in_lb_block);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reg_umr);

}

void connectib_fw_qpc_eth_unpack(struct connectib_fw_qpc_eth *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->tis_num = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 7;
	ptr_struct->sq_in_lb_block = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->reg_umr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_fw_qpc_eth_print(const struct connectib_fw_qpc_eth *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_qpc_eth ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tis_num              : " UH_FMT "\n", ptr_struct->tis_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq_in_lb_block       : " UH_FMT "\n", ptr_struct->sq_in_lb_block);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg_umr              : " UH_FMT "\n", ptr_struct->reg_umr);

}

int connectib_fw_qpc_eth_size(void)
{
	 return 4;
}

void connectib_fw_qpc_eth_dump(const struct connectib_fw_qpc_eth *ptr_struct, FILE* file)
{
	connectib_fw_qpc_eth_print(ptr_struct, file, 0);
}

void connectib_hw_msix_context_data0_pack(const struct connectib_hw_msix_context_data0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data);

}

void connectib_hw_msix_context_data0_unpack(struct connectib_hw_msix_context_data0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_msix_context_data0_print(const struct connectib_hw_msix_context_data0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_msix_context_data0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data                 : " U32H_FMT "\n", ptr_struct->data);

}

int connectib_hw_msix_context_data0_size(void)
{
	 return 4;
}

void connectib_hw_msix_context_data0_dump(const struct connectib_hw_msix_context_data0 *ptr_struct, FILE* file)
{
	connectib_hw_msix_context_data0_print(ptr_struct, file, 0);
}

void connectib_hw_ste_tag_generic_headers_pack(const struct connectib_hw_ste_tag_generic_headers *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword_select_valid);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword_select);

}

void connectib_hw_ste_tag_generic_headers_unpack(struct connectib_hw_ste_tag_generic_headers *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->dword_select_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->dword_select = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_ste_tag_generic_headers_print(const struct connectib_hw_ste_tag_generic_headers *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste_tag_generic_headers ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_select_valid   : " UH_FMT "\n", ptr_struct->dword_select_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_select         : " U32H_FMT "\n", ptr_struct->dword_select);

}

int connectib_hw_ste_tag_generic_headers_size(void)
{
	 return 16;
}

void connectib_hw_ste_tag_generic_headers_dump(const struct connectib_hw_ste_tag_generic_headers *ptr_struct, FILE* file)
{
	connectib_hw_ste_tag_generic_headers_print(ptr_struct, file, 0);
}

void connectib_hw_ste_tag_port_pack(const struct connectib_hw_ste_tag_port *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->grh);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bth);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->qp_type);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_sniffer);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_lb);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->func_lb);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->multicast);

}

void connectib_hw_ste_tag_port_unpack(struct connectib_hw_ste_tag_port *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 19;
	ptr_struct->grh = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->bth = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->qp_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 3;
	ptr_struct->sx_sniffer = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->force_lb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->func_lb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->multicast = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_ste_tag_port_print(const struct connectib_hw_ste_tag_port *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste_tag_port ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh                  : " UH_FMT "\n", ptr_struct->grh);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bth                  : " UH_FMT "\n", ptr_struct->bth);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_type              : " UH_FMT "\n", ptr_struct->qp_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_sniffer           : " UH_FMT "\n", ptr_struct->sx_sniffer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_lb             : " UH_FMT "\n", ptr_struct->force_lb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "func_lb              : " UH_FMT "\n", ptr_struct->func_lb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "multicast            : " UH_FMT "\n", ptr_struct->multicast);

}

int connectib_hw_ste_tag_port_size(void)
{
	 return 16;
}

void connectib_hw_ste_tag_port_dump(const struct connectib_hw_ste_tag_port *ptr_struct, FILE* file)
{
	connectib_hw_ste_tag_port_print(ptr_struct, file, 0);
}

void connectib_hw_ste_tag_source_gvmi_qp_pack(const struct connectib_hw_ste_tag_source_gvmi_qp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->source_gvmi);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->source_qp);

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->source_is_requestor);

}

void connectib_hw_ste_tag_source_gvmi_qp_unpack(struct connectib_hw_ste_tag_source_gvmi_qp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->source_gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->source_qp = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 39;
	ptr_struct->source_is_requestor = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_ste_tag_source_gvmi_qp_print(const struct connectib_hw_ste_tag_source_gvmi_qp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste_tag_source_gvmi_qp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "source_gvmi          : " UH_FMT "\n", ptr_struct->source_gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "source_qp            : " UH_FMT "\n", ptr_struct->source_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "source_is_requestor  : " UH_FMT "\n", ptr_struct->source_is_requestor);

}

int connectib_hw_ste_tag_source_gvmi_qp_size(void)
{
	 return 16;
}

void connectib_hw_ste_tag_source_gvmi_qp_dump(const struct connectib_hw_ste_tag_source_gvmi_qp *ptr_struct, FILE* file)
{
	connectib_hw_ste_tag_source_gvmi_qp_print(ptr_struct, file, 0);
}

void connectib_hw_ste_tag_ib_l4_pack(const struct connectib_hw_ste_tag_ib_l4 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->dest_qp);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pkey);

}

void connectib_hw_ste_tag_ib_l4_unpack(struct connectib_hw_ste_tag_ib_l4 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->dest_qp = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 48;
	ptr_struct->pkey = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_ste_tag_ib_l4_print(const struct connectib_hw_ste_tag_ib_l4 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste_tag_ib_l4 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dest_qp              : " UH_FMT "\n", ptr_struct->dest_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey                 : " UH_FMT "\n", ptr_struct->pkey);

}

int connectib_hw_ste_tag_ib_l4_size(void)
{
	 return 16;
}

void connectib_hw_ste_tag_ib_l4_dump(const struct connectib_hw_ste_tag_ib_l4 *ptr_struct, FILE* file)
{
	connectib_hw_ste_tag_ib_l4_print(ptr_struct, file, 0);
}

void connectib_hw_ste_tag_ib_l3_pack(const struct connectib_hw_ste_tag_ib_l3 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dgid_127_96);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dgid_95_64);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dgid_63_32);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dgid_31_0);

}

void connectib_hw_ste_tag_ib_l3_unpack(struct connectib_hw_ste_tag_ib_l3 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->dgid_127_96 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->dgid_95_64 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->dgid_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->dgid_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_ste_tag_ib_l3_print(const struct connectib_hw_ste_tag_ib_l3 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste_tag_ib_l3 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dgid_127_96          : " U32H_FMT "\n", ptr_struct->dgid_127_96);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dgid_95_64           : " U32H_FMT "\n", ptr_struct->dgid_95_64);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dgid_63_32           : " U32H_FMT "\n", ptr_struct->dgid_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dgid_31_0            : " U32H_FMT "\n", ptr_struct->dgid_31_0);

}

int connectib_hw_ste_tag_ib_l3_size(void)
{
	 return 16;
}

void connectib_hw_ste_tag_ib_l3_dump(const struct connectib_hw_ste_tag_ib_l3 *ptr_struct, FILE* file)
{
	connectib_hw_ste_tag_ib_l3_print(ptr_struct, file, 0);
}

void connectib_hw_ste_tag_ib_l3_extended_pack(const struct connectib_hw_ste_tag_ib_l3_extended *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->flow_label);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->traffic_class);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pkey);

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dceth_complete);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->deth_complete);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bth_complete);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ip_version);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->qkey_dc_access_key_63_32);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->qkey_dc_access_key_31_0);

}

void connectib_hw_ste_tag_ib_l3_extended_unpack(struct connectib_hw_ste_tag_ib_l3_extended *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	ptr_struct->flow_label = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

	offset = 8;
	ptr_struct->sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->traffic_class = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->pkey = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 35;
	ptr_struct->dceth_complete = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 34;
	ptr_struct->deth_complete = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 33;
	ptr_struct->bth_complete = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->ip_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->qkey_dc_access_key_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->qkey_dc_access_key_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_ste_tag_ib_l3_extended_print(const struct connectib_hw_ste_tag_ib_l3_extended *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste_tag_ib_l3_extended ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_label           : " UH_FMT "\n", ptr_struct->flow_label);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl                   : " UH_FMT "\n", ptr_struct->sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "traffic_class        : " UH_FMT "\n", ptr_struct->traffic_class);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey                 : " UH_FMT "\n", ptr_struct->pkey);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dceth_complete       : " UH_FMT "\n", ptr_struct->dceth_complete);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "deth_complete        : " UH_FMT "\n", ptr_struct->deth_complete);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bth_complete         : " UH_FMT "\n", ptr_struct->bth_complete);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ip_version           : " UH_FMT "\n", ptr_struct->ip_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qkey_dc_access_key_63_32 : " U32H_FMT "\n", ptr_struct->qkey_dc_access_key_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qkey_dc_access_key_31_0 : " U32H_FMT "\n", ptr_struct->qkey_dc_access_key_31_0);

}

int connectib_hw_ste_tag_ib_l3_extended_size(void)
{
	 return 16;
}

void connectib_hw_ste_tag_ib_l3_extended_dump(const struct connectib_hw_ste_tag_ib_l3_extended *ptr_struct, FILE* file)
{
	connectib_hw_ste_tag_ib_l3_extended_print(ptr_struct, file, 0);
}

void connectib_hw_ste_tag_ib_l2_pack(const struct connectib_hw_ste_tag_ib_l2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->dlid);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->grh);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bth);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->qp_type);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->multicast);

	offset = 76;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->flow_label);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rwh_ethertype);

}

void connectib_hw_ste_tag_ib_l2_unpack(struct connectib_hw_ste_tag_ib_l2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->dlid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 15;
	ptr_struct->grh = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->bth = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->qp_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 8;
	ptr_struct->sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->multicast = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 76;
	ptr_struct->flow_label = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

	offset = 112;
	ptr_struct->rwh_ethertype = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_ste_tag_ib_l2_print(const struct connectib_hw_ste_tag_ib_l2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste_tag_ib_l2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dlid                 : " UH_FMT "\n", ptr_struct->dlid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh                  : " UH_FMT "\n", ptr_struct->grh);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bth                  : " UH_FMT "\n", ptr_struct->bth);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_type              : " UH_FMT "\n", ptr_struct->qp_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl                   : " UH_FMT "\n", ptr_struct->sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "multicast            : " UH_FMT "\n", ptr_struct->multicast);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_label           : " UH_FMT "\n", ptr_struct->flow_label);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwh_ethertype        : " UH_FMT "\n", ptr_struct->rwh_ethertype);

}

int connectib_hw_ste_tag_ib_l2_size(void)
{
	 return 16;
}

void connectib_hw_ste_tag_ib_l2_dump(const struct connectib_hw_ste_tag_ib_l2 *ptr_struct, FILE* file)
{
	connectib_hw_ste_tag_ib_l2_print(ptr_struct, file, 0);
}

void connectib_hw_ste_tag_bulk_pack(const struct connectib_hw_ste_tag_bulk *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tag_127_96);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tag_95_64);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tag_63_32);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tag_31_0);

}

void connectib_hw_ste_tag_bulk_unpack(struct connectib_hw_ste_tag_bulk *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->tag_127_96 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->tag_95_64 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->tag_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->tag_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_ste_tag_bulk_print(const struct connectib_hw_ste_tag_bulk *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste_tag_bulk ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tag_127_96           : " U32H_FMT "\n", ptr_struct->tag_127_96);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tag_95_64            : " U32H_FMT "\n", ptr_struct->tag_95_64);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tag_63_32            : " U32H_FMT "\n", ptr_struct->tag_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tag_31_0             : " U32H_FMT "\n", ptr_struct->tag_31_0);

}

int connectib_hw_ste_tag_bulk_size(void)
{
	 return 16;
}

void connectib_hw_ste_tag_bulk_dump(const struct connectib_hw_ste_tag_bulk *ptr_struct, FILE* file)
{
	connectib_hw_ste_tag_bulk_print(ptr_struct, file, 0);
}

void connectib_hw_transaction_pdf_pack(const struct connectib_hw_transaction_pdf *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->invert_data_bits);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->swap_data_bits);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->seed);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->swap_crc_bytes);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->swap_crc_bits);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->invert_crc);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sig_zones_covered);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sig_type);

}

void connectib_hw_transaction_pdf_unpack(struct connectib_hw_transaction_pdf *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	ptr_struct->invert_data_bits = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->swap_data_bits = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->seed = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->swap_crc_bytes = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->swap_crc_bits = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->invert_crc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->sig_zones_covered = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->sig_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_transaction_pdf_print(const struct connectib_hw_transaction_pdf *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_transaction_pdf ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "invert_data_bits     : " UH_FMT "\n", ptr_struct->invert_data_bits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data_bits       : " UH_FMT "\n", ptr_struct->swap_data_bits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "seed                 : " UH_FMT "\n", ptr_struct->seed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_crc_bytes       : " UH_FMT "\n", ptr_struct->swap_crc_bytes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_crc_bits        : " UH_FMT "\n", ptr_struct->swap_crc_bits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "invert_crc           : " UH_FMT "\n", ptr_struct->invert_crc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sig_zones_covered    : " UH_FMT "\n", ptr_struct->sig_zones_covered);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sig_type             : " UH_FMT "\n", ptr_struct->sig_type);

}

int connectib_hw_transaction_pdf_size(void)
{
	 return 8;
}

void connectib_hw_transaction_pdf_dump(const struct connectib_hw_transaction_pdf *ptr_struct, FILE* file)
{
	connectib_hw_transaction_pdf_print(ptr_struct, file, 0);
}

void connectib_hw_pdf_decoder_pack(const struct connectib_hw_pdf_decoder *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->invert_data_bits);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->swap_data_bits);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->seed);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->swap_crc_bytes);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->swap_crc_bits);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->invert_crc);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->regular);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sig_type);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->dif_app_tag_bit_check_en);

	offset = 46;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->dif_guard_check_type);

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->dif_learn_mode);

	offset = 43;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dif_learn_app_tag);

	offset = 42;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dif_learn_ref_tag);

	offset = 41;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dif_inc_ref_tag);

}

void connectib_hw_pdf_decoder_unpack(struct connectib_hw_pdf_decoder *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	ptr_struct->invert_data_bits = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->swap_data_bits = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->seed = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->swap_crc_bytes = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->swap_crc_bits = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->invert_crc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->regular = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->sig_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->dif_app_tag_bit_check_en = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 46;
	ptr_struct->dif_guard_check_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 44;
	ptr_struct->dif_learn_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 43;
	ptr_struct->dif_learn_app_tag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 42;
	ptr_struct->dif_learn_ref_tag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 41;
	ptr_struct->dif_inc_ref_tag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_pdf_decoder_print(const struct connectib_hw_pdf_decoder *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_pdf_decoder ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "invert_data_bits     : " UH_FMT "\n", ptr_struct->invert_data_bits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data_bits       : " UH_FMT "\n", ptr_struct->swap_data_bits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "seed                 : " UH_FMT "\n", ptr_struct->seed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_crc_bytes       : " UH_FMT "\n", ptr_struct->swap_crc_bytes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_crc_bits        : " UH_FMT "\n", ptr_struct->swap_crc_bits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "invert_crc           : " UH_FMT "\n", ptr_struct->invert_crc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "regular              : " UH_FMT "\n", ptr_struct->regular);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sig_type             : " UH_FMT "\n", ptr_struct->sig_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dif_app_tag_bit_check_en : " UH_FMT "\n", ptr_struct->dif_app_tag_bit_check_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dif_guard_check_type : " UH_FMT "\n", ptr_struct->dif_guard_check_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dif_learn_mode       : " UH_FMT "\n", ptr_struct->dif_learn_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dif_learn_app_tag    : " UH_FMT "\n", ptr_struct->dif_learn_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dif_learn_ref_tag    : " UH_FMT "\n", ptr_struct->dif_learn_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dif_inc_ref_tag      : " UH_FMT "\n", ptr_struct->dif_inc_ref_tag);

}

int connectib_hw_pdf_decoder_size(void)
{
	 return 8;
}

void connectib_hw_pdf_decoder_dump(const struct connectib_hw_pdf_decoder *ptr_struct, FILE* file)
{
	connectib_hw_pdf_decoder_print(ptr_struct, file, 0);
}

void connectib_serdes_conf_tx_set_ini_pack(const struct connectib_serdes_conf_tx_set_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->ob_preemp_pre);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pre_polarity);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->ob_preemp_post);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->post_polarity);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->ob_preemp_main);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->main_polarity);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->preemp_mode);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ob_bias);

}

void connectib_serdes_conf_tx_set_ini_unpack(struct connectib_serdes_conf_tx_set_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	ptr_struct->ob_preemp_pre = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 24;
	ptr_struct->pre_polarity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->ob_preemp_post = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 16;
	ptr_struct->post_polarity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->ob_preemp_main = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 8;
	ptr_struct->main_polarity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->preemp_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 2;
	ptr_struct->ob_bias = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_serdes_conf_tx_set_ini_print(const struct connectib_serdes_conf_tx_set_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_conf_tx_set_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ob_preemp_pre        : " UH_FMT "\n", ptr_struct->ob_preemp_pre);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pre_polarity         : " UH_FMT "\n", ptr_struct->pre_polarity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ob_preemp_post       : " UH_FMT "\n", ptr_struct->ob_preemp_post);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "post_polarity        : " UH_FMT "\n", ptr_struct->post_polarity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ob_preemp_main       : " UH_FMT "\n", ptr_struct->ob_preemp_main);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "main_polarity        : " UH_FMT "\n", ptr_struct->main_polarity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "preemp_mode          : " UH_FMT "\n", ptr_struct->preemp_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ob_bias              : " UH_FMT "\n", ptr_struct->ob_bias);

}

int connectib_serdes_conf_tx_set_ini_size(void)
{
	 return 4;
}

void connectib_serdes_conf_tx_set_ini_dump(const struct connectib_serdes_conf_tx_set_ini *ptr_struct, FILE* file)
{
	connectib_serdes_conf_tx_set_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_conf_rx_set_ini_pack(const struct connectib_serdes_conf_rx_set_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->slicer_ind_en);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->slicer1_enable);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->slicer2_enable);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap0);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap1);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap2);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap3);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap4);

}

void connectib_serdes_conf_rx_set_ini_unpack(struct connectib_serdes_conf_rx_set_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->slicer_ind_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 20;
	ptr_struct->slicer1_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 16;
	ptr_struct->slicer2_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 8;
	ptr_struct->ffe_tap0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->ffe_tap1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 56;
	ptr_struct->ffe_tap2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->ffe_tap3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 40;
	ptr_struct->ffe_tap4 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_serdes_conf_rx_set_ini_print(const struct connectib_serdes_conf_rx_set_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_conf_rx_set_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slicer_ind_en        : " UH_FMT "\n", ptr_struct->slicer_ind_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slicer1_enable       : " UH_FMT "\n", ptr_struct->slicer1_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slicer2_enable       : " UH_FMT "\n", ptr_struct->slicer2_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap0             : " UH_FMT "\n", ptr_struct->ffe_tap0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap1             : " UH_FMT "\n", ptr_struct->ffe_tap1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap2             : " UH_FMT "\n", ptr_struct->ffe_tap2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap3             : " UH_FMT "\n", ptr_struct->ffe_tap3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap4             : " UH_FMT "\n", ptr_struct->ffe_tap4);

}

int connectib_serdes_conf_rx_set_ini_size(void)
{
	 return 8;
}

void connectib_serdes_conf_rx_set_ini_dump(const struct connectib_serdes_conf_rx_set_ini *ptr_struct, FILE* file)
{
	connectib_serdes_conf_rx_set_ini_print(ptr_struct, file, 0);
}

void connectib_aba_index_pack(const struct connectib_aba_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index0_start);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index0_end);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index0);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index1_start);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index1_end);

	offset = 58;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index1);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index2_start);

	offset = 46;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index2_end);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index2);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index3_start);

	offset = 90;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index3_end);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index3);

	offset = 78;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index4_start);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index4_end);

	offset = 66;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index4);

	offset = 122;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index5_start);

	offset = 116;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index5_end);

	offset = 110;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->aba_index5);

}

void connectib_aba_index_unpack(struct connectib_aba_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->aba_index0_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 20;
	ptr_struct->aba_index0_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 14;
	ptr_struct->aba_index0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 8;
	ptr_struct->aba_index1_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 2;
	ptr_struct->aba_index1_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 58;
	ptr_struct->aba_index1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 52;
	ptr_struct->aba_index2_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 46;
	ptr_struct->aba_index2_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 40;
	ptr_struct->aba_index2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 34;
	ptr_struct->aba_index3_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 90;
	ptr_struct->aba_index3_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 84;
	ptr_struct->aba_index3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 78;
	ptr_struct->aba_index4_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 72;
	ptr_struct->aba_index4_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 66;
	ptr_struct->aba_index4 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 122;
	ptr_struct->aba_index5_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 116;
	ptr_struct->aba_index5_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 110;
	ptr_struct->aba_index5 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

}

void connectib_aba_index_print(const struct connectib_aba_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_aba_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index0_start     : " UH_FMT "\n", ptr_struct->aba_index0_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index0_end       : " UH_FMT "\n", ptr_struct->aba_index0_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index0           : " UH_FMT "\n", ptr_struct->aba_index0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index1_start     : " UH_FMT "\n", ptr_struct->aba_index1_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index1_end       : " UH_FMT "\n", ptr_struct->aba_index1_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index1           : " UH_FMT "\n", ptr_struct->aba_index1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index2_start     : " UH_FMT "\n", ptr_struct->aba_index2_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index2_end       : " UH_FMT "\n", ptr_struct->aba_index2_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index2           : " UH_FMT "\n", ptr_struct->aba_index2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index3_start     : " UH_FMT "\n", ptr_struct->aba_index3_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index3_end       : " UH_FMT "\n", ptr_struct->aba_index3_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index3           : " UH_FMT "\n", ptr_struct->aba_index3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index4_start     : " UH_FMT "\n", ptr_struct->aba_index4_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index4_end       : " UH_FMT "\n", ptr_struct->aba_index4_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index4           : " UH_FMT "\n", ptr_struct->aba_index4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index5_start     : " UH_FMT "\n", ptr_struct->aba_index5_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index5_end       : " UH_FMT "\n", ptr_struct->aba_index5_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_index5           : " UH_FMT "\n", ptr_struct->aba_index5);

}

int connectib_aba_index_size(void)
{
	 return 16;
}

void connectib_aba_index_dump(const struct connectib_aba_index *ptr_struct, FILE* file)
{
	connectib_aba_index_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_qp_commands_specific_pack(const union connectib_cmdif_ctx_special_qp_commands_specific *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_cmdif_ctx_special_qp_commands_other_pack(&(ptr_struct->other), ptr_buff);
}

void connectib_cmdif_ctx_special_qp_commands_specific_unpack(union connectib_cmdif_ctx_special_qp_commands_specific *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_cmdif_ctx_special_qp_commands_other_unpack(&(ptr_struct->other), ptr_buff);
}

void connectib_cmdif_ctx_special_qp_commands_specific_print(const union connectib_cmdif_ctx_special_qp_commands_specific *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_qp_commands_specific ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rts2rts:\n");
	connectib_cmdif_ctx_special_qp_commands_rts2rts_print(&(ptr_struct->rts2rts), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "other:\n");
	connectib_cmdif_ctx_special_qp_commands_other_print(&(ptr_struct->other), file, indent_level + 1);

}

int connectib_cmdif_ctx_special_qp_commands_specific_size(void)
{
	 return 16;
}

void connectib_cmdif_ctx_special_qp_commands_specific_dump(const union connectib_cmdif_ctx_special_qp_commands_specific *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_qp_commands_specific_print(ptr_struct, file, 0);
}

void connectib_pmcr_reg_pack(const struct connectib_pmcr_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pmcr_reg_unpack(struct connectib_pmcr_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pmcr_reg_print(const struct connectib_pmcr_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmcr_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_pmcr_reg_size(void)
{
	 return 24;
}

void connectib_pmcr_reg_dump(const struct connectib_pmcr_reg *ptr_struct, FILE* file)
{
	connectib_pmcr_reg_print(ptr_struct, file, 0);
}

void connectib_mpein_reg_pack(const struct connectib_mpein_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pcie_index);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->capability_mask);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->link_speed_enabled);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->link_width_enabled);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->link_speed_active);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->link_width_active);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->lane0_physical_position);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->num_of_vfs);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->num_of_pfs);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->bdf0);

	offset = 223;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lane_reversal);

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_type);

	offset = 196;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_payload_size);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_read_request_size);

	offset = 304;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_state);

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->device_status);

	offset = 336;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->receiver_detect_result);

}

void connectib_mpein_reg_unpack(struct connectib_mpein_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->pcie_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->capability_mask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 80;
	ptr_struct->link_speed_enabled = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 72;
	ptr_struct->link_width_enabled = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 112;
	ptr_struct->link_speed_active = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 104;
	ptr_struct->link_width_active = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->lane0_physical_position = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 144;
	ptr_struct->num_of_vfs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 128;
	ptr_struct->num_of_pfs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 160;
	ptr_struct->bdf0 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 223;
	ptr_struct->lane_reversal = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 208;
	ptr_struct->port_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 196;
	ptr_struct->max_payload_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->max_read_request_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 304;
	ptr_struct->port_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 288;
	ptr_struct->device_status = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 336;
	ptr_struct->receiver_detect_result = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_mpein_reg_print(const struct connectib_mpein_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mpein_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_index           : " UH_FMT "\n", ptr_struct->pcie_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capability_mask      : " U32H_FMT "\n", ptr_struct->capability_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_speed_enabled   : " UH_FMT "\n", ptr_struct->link_speed_enabled);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_width_enabled   : " UH_FMT "\n", ptr_struct->link_width_enabled);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_speed_active    : " UH_FMT "\n", ptr_struct->link_speed_active);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_width_active    : " UH_FMT "\n", ptr_struct->link_width_active);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane0_physical_position : " UH_FMT "\n", ptr_struct->lane0_physical_position);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_vfs           : " UH_FMT "\n", ptr_struct->num_of_vfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_pfs           : " UH_FMT "\n", ptr_struct->num_of_pfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bdf0                 : " UH_FMT "\n", ptr_struct->bdf0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane_reversal        : " UH_FMT "\n", ptr_struct->lane_reversal);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_type            : " UH_FMT "\n", ptr_struct->port_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_payload_size     : " UH_FMT "\n", ptr_struct->max_payload_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_read_request_size : " UH_FMT "\n", ptr_struct->max_read_request_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_state           : " UH_FMT "\n", ptr_struct->port_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_status        : " UH_FMT "\n", ptr_struct->device_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "receiver_detect_result : " UH_FMT "\n", ptr_struct->receiver_detect_result);

}

int connectib_mpein_reg_size(void)
{
	 return 44;
}

void connectib_mpein_reg_dump(const struct connectib_mpein_reg *ptr_struct, FILE* file)
{
	connectib_mpein_reg_print(ptr_struct, file, 0);
}

void connectib_mvts_pack(const struct connectib_mvts *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->check_timestamp_flag);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->running_flag);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->clear_all_ts_flag);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->fw_ver_major);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->fw_ver_subminor);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->fw_ver_minor);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ts_day);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ts_month);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ts_year);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ts_seconds);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ts_minutes);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ts_hour);

}

void connectib_mvts_unpack(struct connectib_mvts *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	ptr_struct->check_timestamp_flag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->running_flag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->clear_all_ts_flag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->fw_ver_major = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 80;
	ptr_struct->fw_ver_subminor = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->fw_ver_minor = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 120;
	ptr_struct->ts_day = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 112;
	ptr_struct->ts_month = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->ts_year = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 144;
	ptr_struct->ts_seconds = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->ts_minutes = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 128;
	ptr_struct->ts_hour = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_mvts_print(const struct connectib_mvts *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mvts ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "check_timestamp_flag : " UH_FMT "\n", ptr_struct->check_timestamp_flag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "running_flag         : " UH_FMT "\n", ptr_struct->running_flag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "clear_all_ts_flag    : " UH_FMT "\n", ptr_struct->clear_all_ts_flag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ver_major         : " UH_FMT "\n", ptr_struct->fw_ver_major);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ver_subminor      : " UH_FMT "\n", ptr_struct->fw_ver_subminor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ver_minor         : " UH_FMT "\n", ptr_struct->fw_ver_minor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts_day               : " UH_FMT "\n", ptr_struct->ts_day);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts_month             : " UH_FMT "\n", ptr_struct->ts_month);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts_year              : " UH_FMT "\n", ptr_struct->ts_year);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts_seconds           : " UH_FMT "\n", ptr_struct->ts_seconds);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts_minutes           : " UH_FMT "\n", ptr_struct->ts_minutes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts_hour              : " UH_FMT "\n", ptr_struct->ts_hour);

}

int connectib_mvts_size(void)
{
	 return 20;
}

void connectib_mvts_dump(const struct connectib_mvts *ptr_struct, FILE* file)
{
	connectib_mvts_print(ptr_struct, file, 0);
}

void connectib_pmlp_pack(const union connectib_pmlp *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_pmlp_code_style_pack(&(ptr_struct->code_style), ptr_buff);
}

void connectib_pmlp_unpack(union connectib_pmlp *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_pmlp_code_style_unpack(&(ptr_struct->code_style), ptr_buff);
}

void connectib_pmlp_print(const union connectib_pmlp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmlp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prm_style:\n");
	connectib_pmlp_prm_style_print(&(ptr_struct->prm_style), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "code_style:\n");
	connectib_pmlp_code_style_print(&(ptr_struct->code_style), file, indent_level + 1);

}

int connectib_pmlp_size(void)
{
	 return 36;
}

void connectib_pmlp_dump(const union connectib_pmlp *ptr_struct, FILE* file)
{
	connectib_pmlp_print(ptr_struct, file, 0);
}

void connectib_pplm_reg_pack(const struct connectib_pplm_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->active_port_profile);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->static_port_profile);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_profile_mode);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->fec_mode_active);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->retransmission_active);

	offset = 156;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_cap_10g_40g);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_cap_25g);

	offset = 148;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_cap_50g);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_cap_100g);

	offset = 140;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_cap_56g);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_correction_bypass_cap);

	offset = 188;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_admin_10g_40g);

	offset = 184;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_admin_25g);

	offset = 180;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_admin_50g);

	offset = 176;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_admin_100g);

	offset = 172;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fec_override_admin_56g);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rs_fec_correction_bypass_admin);

}

void connectib_pplm_reg_unpack(struct connectib_pplm_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 80;
	ptr_struct->active_port_profile = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 72;
	ptr_struct->static_port_profile = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->port_profile_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->fec_mode_active = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->retransmission_active = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 156;
	ptr_struct->fec_override_cap_10g_40g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 152;
	ptr_struct->fec_override_cap_25g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 148;
	ptr_struct->fec_override_cap_50g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 144;
	ptr_struct->fec_override_cap_100g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 140;
	ptr_struct->fec_override_cap_56g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->rs_fec_correction_bypass_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 188;
	ptr_struct->fec_override_admin_10g_40g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 184;
	ptr_struct->fec_override_admin_25g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 180;
	ptr_struct->fec_override_admin_50g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 176;
	ptr_struct->fec_override_admin_100g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 172;
	ptr_struct->fec_override_admin_56g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->rs_fec_correction_bypass_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_pplm_reg_print(const struct connectib_pplm_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pplm_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "active_port_profile  : " UH_FMT "\n", ptr_struct->active_port_profile);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "static_port_profile  : " UH_FMT "\n", ptr_struct->static_port_profile);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_profile_mode    : " UH_FMT "\n", ptr_struct->port_profile_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_mode_active      : " UH_FMT "\n", ptr_struct->fec_mode_active);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "retransmission_active : " UH_FMT "\n", ptr_struct->retransmission_active);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_cap_10g_40g : " UH_FMT "\n", ptr_struct->fec_override_cap_10g_40g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_cap_25g : " UH_FMT "\n", ptr_struct->fec_override_cap_25g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_cap_50g : " UH_FMT "\n", ptr_struct->fec_override_cap_50g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_cap_100g : " UH_FMT "\n", ptr_struct->fec_override_cap_100g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_cap_56g : " UH_FMT "\n", ptr_struct->fec_override_cap_56g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_correction_bypass_cap : " UH_FMT "\n", ptr_struct->rs_fec_correction_bypass_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_admin_10g_40g : " UH_FMT "\n", ptr_struct->fec_override_admin_10g_40g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_admin_25g : " UH_FMT "\n", ptr_struct->fec_override_admin_25g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_admin_50g : " UH_FMT "\n", ptr_struct->fec_override_admin_50g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_admin_100g : " UH_FMT "\n", ptr_struct->fec_override_admin_100g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_override_admin_56g : " UH_FMT "\n", ptr_struct->fec_override_admin_56g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rs_fec_correction_bypass_admin : " UH_FMT "\n", ptr_struct->rs_fec_correction_bypass_admin);

}

int connectib_pplm_reg_size(void)
{
	 return 24;
}

void connectib_pplm_reg_dump(const struct connectib_pplm_reg *ptr_struct, FILE* file)
{
	connectib_pplm_reg_print(ptr_struct, file, 0);
}

void connectib_plpc_reg_pack(const struct connectib_plpc_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->proto_mask);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->profile_id);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lane_speed);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->fec_mode_policy);

	offset = 87;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lpbf);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->fec_mode_capability);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->retransmission_capability);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->fec_mode_support_admin);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->retransmission_support_admin);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->fec_mode_request_admin);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->retransmission_request_admin);

}

void connectib_plpc_reg_unpack(struct connectib_plpc_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 20;
	ptr_struct->proto_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 4;
	ptr_struct->profile_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 48;
	ptr_struct->lane_speed = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 88;
	ptr_struct->fec_mode_policy = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 87;
	ptr_struct->lpbf = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 104;
	ptr_struct->fec_mode_capability = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->retransmission_capability = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->fec_mode_support_admin = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 128;
	ptr_struct->retransmission_support_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 168;
	ptr_struct->fec_mode_request_admin = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 160;
	ptr_struct->retransmission_request_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_plpc_reg_print(const struct connectib_plpc_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_plpc_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "proto_mask           : " UH_FMT "\n", ptr_struct->proto_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "profile_id           : " UH_FMT "\n", ptr_struct->profile_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane_speed           : " UH_FMT "\n", ptr_struct->lane_speed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_mode_policy      : " UH_FMT "\n", ptr_struct->fec_mode_policy);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lpbf                 : " UH_FMT "\n", ptr_struct->lpbf);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_mode_capability  : " UH_FMT "\n", ptr_struct->fec_mode_capability);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "retransmission_capability : " UH_FMT "\n", ptr_struct->retransmission_capability);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_mode_support_admin : " UH_FMT "\n", ptr_struct->fec_mode_support_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "retransmission_support_admin : " UH_FMT "\n", ptr_struct->retransmission_support_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_mode_request_admin : " UH_FMT "\n", ptr_struct->fec_mode_request_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "retransmission_request_admin : " UH_FMT "\n", ptr_struct->retransmission_request_admin);

}

int connectib_plpc_reg_size(void)
{
	 return 40;
}

void connectib_plpc_reg_dump(const struct connectib_plpc_reg *ptr_struct, FILE* file)
{
	connectib_plpc_reg_print(ptr_struct, file, 0);
}

void connectib_pplr_reg_pack(const struct connectib_pplr_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pplr_reg_unpack(struct connectib_pplr_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pplr_reg_print(const struct connectib_pplr_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pplr_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_pplr_reg_size(void)
{
	 return 8;
}

void connectib_pplr_reg_dump(const struct connectib_pplr_reg *ptr_struct, FILE* file)
{
	connectib_pplr_reg_print(ptr_struct, file, 0);
}

void connectib_project_nvgn_pack(const struct connectib_project_nvgn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->nv_pointer);

	offset = 128;
	connectib_nv_tlv_hdr_pack(&(ptr_struct->tlv_header), ptr_buff + offset/8);

	offset = 224;
	connectib_register_access_data_reduced_pack(&(ptr_struct->tlv_data), ptr_buff + offset/8);

}

void connectib_project_nvgn_unpack(struct connectib_project_nvgn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->nv_pointer = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	connectib_nv_tlv_hdr_unpack(&(ptr_struct->tlv_header), ptr_buff + offset/8);

	offset = 224;
	connectib_register_access_data_reduced_unpack(&(ptr_struct->tlv_data), ptr_buff + offset/8);

}

void connectib_project_nvgn_print(const struct connectib_project_nvgn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_project_nvgn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nv_pointer           : " U32H_FMT "\n", ptr_struct->nv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tlv_header:\n");
	connectib_nv_tlv_hdr_print(&(ptr_struct->tlv_header), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tlv_data:\n");
	connectib_register_access_data_reduced_print(&(ptr_struct->tlv_data), file, indent_level + 1);

}

int connectib_project_nvgn_size(void)
{
	 return 44;
}

void connectib_project_nvgn_dump(const struct connectib_project_nvgn *ptr_struct, FILE* file)
{
	connectib_project_nvgn_print(ptr_struct, file, 0);
}

void connectib_project_nvia_pack(const struct connectib_project_nvia *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->target);

}

void connectib_project_nvia_unpack(struct connectib_project_nvia *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	ptr_struct->target = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_project_nvia_print(const struct connectib_project_nvia *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_project_nvia ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "target               : " UH_FMT "\n", ptr_struct->target);

}

int connectib_project_nvia_size(void)
{
	 return 8;
}

void connectib_project_nvia_dump(const struct connectib_project_nvia *ptr_struct, FILE* file)
{
	connectib_project_nvia_print(ptr_struct, file, 0);
}

void connectib_project_nvgc_pack(const struct connectib_project_nvgc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->nvda_read_factory_settings);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->nvda_read_current_settings);

}

void connectib_project_nvgc_unpack(struct connectib_project_nvgc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->nvda_read_factory_settings = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->nvda_read_current_settings = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_project_nvgc_print(const struct connectib_project_nvgc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_project_nvgc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvda_read_factory_settings : " UH_FMT "\n", ptr_struct->nvda_read_factory_settings);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvda_read_current_settings : " UH_FMT "\n", ptr_struct->nvda_read_current_settings);

}

int connectib_project_nvgc_size(void)
{
	 return 16;
}

void connectib_project_nvgc_dump(const struct connectib_project_nvgc *ptr_struct, FILE* file)
{
	connectib_project_nvgc_print(ptr_struct, file, 0);
}

void connectib_project_nvqc_pack(const struct connectib_project_nvqc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_tlv_type_pack(&(ptr_struct->type), ptr_buff + offset/8);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->support_rd);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->support_wr);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

}

void connectib_project_nvqc_unpack(struct connectib_project_nvqc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_tlv_type_unpack(&(ptr_struct->type), ptr_buff + offset/8);

	offset = 63;
	ptr_struct->support_rd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 62;
	ptr_struct->support_wr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 56;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_project_nvqc_print(const struct connectib_project_nvqc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_project_nvqc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type:\n");
	connectib_nv_tlv_type_print(&(ptr_struct->type), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "support_rd           : " UH_FMT "\n", ptr_struct->support_rd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "support_wr           : " UH_FMT "\n", ptr_struct->support_wr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

}

int connectib_project_nvqc_size(void)
{
	 return 8;
}

void connectib_project_nvqc_dump(const struct connectib_project_nvqc *ptr_struct, FILE* file)
{
	connectib_project_nvqc_print(ptr_struct, file, 0);
}

void connectib_project_nvda_pack(const struct connectib_project_nvda *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_tlv_hdr_pack(&(ptr_struct->tlv_header), ptr_buff + offset/8);

	offset = 96;
	connectib_register_access_data_pack(&(ptr_struct->nvda_data), ptr_buff + offset/8);

}

void connectib_project_nvda_unpack(struct connectib_project_nvda *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_tlv_hdr_unpack(&(ptr_struct->tlv_header), ptr_buff + offset/8);

	offset = 96;
	connectib_register_access_data_unpack(&(ptr_struct->nvda_data), ptr_buff + offset/8);

}

void connectib_project_nvda_print(const struct connectib_project_nvda *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_project_nvda ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tlv_header:\n");
	connectib_nv_tlv_hdr_print(&(ptr_struct->tlv_header), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvda_data:\n");
	connectib_register_access_data_print(&(ptr_struct->nvda_data), file, indent_level + 1);

}

int connectib_project_nvda_size(void)
{
	 return 44;
}

void connectib_project_nvda_dump(const struct connectib_project_nvda *ptr_struct, FILE* file)
{
	connectib_project_nvda_print(ptr_struct, file, 0);
}

void connectib_pmtu_reg_pack(const struct connectib_pmtu_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_mtu);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->admin_mtu);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->oper_mtu);

}

void connectib_pmtu_reg_unpack(struct connectib_pmtu_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->max_mtu = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->admin_mtu = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->oper_mtu = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_pmtu_reg_print(const struct connectib_pmtu_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmtu_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_mtu              : " UH_FMT "\n", ptr_struct->max_mtu);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "admin_mtu            : " UH_FMT "\n", ptr_struct->admin_mtu);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "oper_mtu             : " UH_FMT "\n", ptr_struct->oper_mtu);

}

int connectib_pmtu_reg_size(void)
{
	 return 16;
}

void connectib_pmtu_reg_dump(const struct connectib_pmtu_reg *ptr_struct, FILE* file)
{
	connectib_pmtu_reg_print(ptr_struct, file, 0);
}

void connectib_pamp_reg_pack(const struct connectib_pamp_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pamp_reg_unpack(struct connectib_pamp_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pamp_reg_print(const struct connectib_pamp_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pamp_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_pamp_reg_size(void)
{
	 return 44;
}

void connectib_pamp_reg_dump(const struct connectib_pamp_reg *ptr_struct, FILE* file)
{
	connectib_pamp_reg_print(ptr_struct, file, 0);
}

void connectib_ppaos_reg_pack(const struct connectib_ppaos_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_ppaos_reg_unpack(struct connectib_ppaos_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_ppaos_reg_print(const struct connectib_ppaos_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ppaos_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_ppaos_reg_size(void)
{
	 return 16;
}

void connectib_ppaos_reg_dump(const struct connectib_ppaos_reg *ptr_struct, FILE* file)
{
	connectib_ppaos_reg_print(ptr_struct, file, 0);
}

void connectib_pprt_reg_pack(const struct connectib_pprt_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pprt_reg_unpack(struct connectib_pprt_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pprt_reg_print(const struct connectib_pprt_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pprt_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_pprt_reg_size(void)
{
	 return 32;
}

void connectib_pprt_reg_dump(const struct connectib_pprt_reg *ptr_struct, FILE* file)
{
	connectib_pprt_reg_print(ptr_struct, file, 0);
}

void connectib_pptt_reg_pack(const struct connectib_pptt_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pptt_reg_unpack(struct connectib_pptt_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pptt_reg_print(const struct connectib_pptt_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pptt_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_pptt_reg_size(void)
{
	 return 28;
}

void connectib_pptt_reg_dump(const struct connectib_pptt_reg *ptr_struct, FILE* file)
{
	connectib_pptt_reg_print(ptr_struct, file, 0);
}

void connectib_slrg_reg_pack(const struct connectib_slrg_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_slrg_reg_unpack(struct connectib_slrg_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_slrg_reg_print(const struct connectib_slrg_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_slrg_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_slrg_reg_size(void)
{
	 return 40;
}

void connectib_slrg_reg_dump(const struct connectib_slrg_reg *ptr_struct, FILE* file)
{
	connectib_slrg_reg_print(ptr_struct, file, 0);
}

void connectib_ptas_reg_pack(const struct connectib_ptas_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_ptas_reg_unpack(struct connectib_ptas_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_ptas_reg_print(const struct connectib_ptas_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ptas_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_ptas_reg_size(void)
{
	 return 44;
}

void connectib_ptas_reg_dump(const struct connectib_ptas_reg *ptr_struct, FILE* file)
{
	connectib_ptas_reg_print(ptr_struct, file, 0);
}

void connectib_sltp_reg_pack(const struct connectib_sltp_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_sltp_reg_unpack(struct connectib_sltp_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_sltp_reg_print(const struct connectib_sltp_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sltp_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_sltp_reg_size(void)
{
	 return 20;
}

void connectib_sltp_reg_dump(const struct connectib_sltp_reg *ptr_struct, FILE* file)
{
	connectib_sltp_reg_print(ptr_struct, file, 0);
}

void connectib_slrp_reg_pack(const struct connectib_slrp_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_slrp_reg_unpack(struct connectib_slrp_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_slrp_reg_print(const struct connectib_slrp_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_slrp_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_slrp_reg_size(void)
{
	 return 40;
}

void connectib_slrp_reg_dump(const struct connectib_slrp_reg *ptr_struct, FILE* file)
{
	connectib_slrp_reg_print(ptr_struct, file, 0);
}

void connectib_paos_reg_pack(const struct connectib_paos_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->oper_status);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->admin_status);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->swid);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->e);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ee);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ase);

}

void connectib_paos_reg_unpack(struct connectib_paos_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->oper_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 20;
	ptr_struct->admin_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->swid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 62;
	ptr_struct->e = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 33;
	ptr_struct->ee = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->ase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_paos_reg_print(const struct connectib_paos_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_paos_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "oper_status          : " UH_FMT "\n", ptr_struct->oper_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "admin_status         : " UH_FMT "\n", ptr_struct->admin_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swid                 : " UH_FMT "\n", ptr_struct->swid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e                    : " UH_FMT "\n", ptr_struct->e);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ee                   : " UH_FMT "\n", ptr_struct->ee);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ase                  : " UH_FMT "\n", ptr_struct->ase);

}

int connectib_paos_reg_size(void)
{
	 return 16;
}

void connectib_paos_reg_dump(const struct connectib_paos_reg *ptr_struct, FILE* file)
{
	connectib_paos_reg_print(ptr_struct, file, 0);
}

void connectib_mjtag_pack(const struct connectib_mjtag *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->size);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sequence_number);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->cmd);

	for (i = 0; i < 10; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 352, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword[i]);
	}

}

void connectib_mjtag_unpack(struct connectib_mjtag *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 4;
	ptr_struct->sequence_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->cmd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	for (i = 0; i < 10; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 352, 1);
	ptr_struct->dword[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_mjtag_print(const struct connectib_mjtag *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mjtag ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sequence_number      : " UH_FMT "\n", ptr_struct->sequence_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmd                  : " UH_FMT "\n", ptr_struct->cmd);

	for (i = 0; i < 10; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_%03d           : " U32H_FMT "\n", i, ptr_struct->dword[i]);
	}

}

int connectib_mjtag_size(void)
{
	 return 44;
}

void connectib_mjtag_dump(const struct connectib_mjtag *ptr_struct, FILE* file)
{
	connectib_mjtag_print(ptr_struct, file, 0);
}

void connectib_mqis_reg_pack(const struct connectib_mqis_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->info_type);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->info_length);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->read_length);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->read_offset);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 160, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->info_string[i]);
	}

}

void connectib_mqis_reg_unpack(struct connectib_mqis_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->info_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->info_length = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 80;
	ptr_struct->read_length = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->read_offset = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 160, 1);
	ptr_struct->info_string[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_mqis_reg_print(const struct connectib_mqis_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mqis_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "info_type            : " UH_FMT "\n", ptr_struct->info_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "info_length          : " UH_FMT "\n", ptr_struct->info_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_length          : " UH_FMT "\n", ptr_struct->read_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_offset          : " UH_FMT "\n", ptr_struct->read_offset);

	for (i = 0; i < 1; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "info_string_%03d     : " U32H_FMT "\n", i, ptr_struct->info_string[i]);
	}

}

int connectib_mqis_reg_size(void)
{
	 return 20;
}

void connectib_mqis_reg_dump(const struct connectib_mqis_reg *ptr_struct, FILE* file)
{
	connectib_mqis_reg_print(ptr_struct, file, 0);
}

void connectib_mcda_reg_pack(const struct connectib_mcda_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->update_handle);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->offset);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->size);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 160, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

}

void connectib_mcda_reg_unpack(struct connectib_mcda_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->update_handle = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 32;
	ptr_struct->offset = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 80;
	ptr_struct->size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 160, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_mcda_reg_print(const struct connectib_mcda_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mcda_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "update_handle        : " UH_FMT "\n", ptr_struct->update_handle);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset               : " U32H_FMT "\n", ptr_struct->offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	for (i = 0; i < 1; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

}

int connectib_mcda_reg_size(void)
{
	 return 20;
}

void connectib_mcda_reg_dump(const struct connectib_mcda_reg *ptr_struct, FILE* file)
{
	connectib_mcda_reg_print(ptr_struct, file, 0);
}

void connectib_mcc_reg_pack(const struct connectib_mcc_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->instruction);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->time_elapsed_since_last_cmd);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->component_index);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->update_handle);

	offset = 124;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->control_state);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->error_code);

	offset = 105;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->control_progress);

	offset = 100;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->handle_owner_host_id);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->handle_owner_type);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->component_size);

}

void connectib_mcc_reg_unpack(struct connectib_mcc_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->instruction = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 4;
	ptr_struct->time_elapsed_since_last_cmd = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 48;
	ptr_struct->component_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 72;
	ptr_struct->update_handle = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 124;
	ptr_struct->control_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 112;
	ptr_struct->error_code = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 105;
	ptr_struct->control_progress = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 100;
	ptr_struct->handle_owner_host_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->handle_owner_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->component_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_mcc_reg_print(const struct connectib_mcc_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mcc_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "instruction          : " UH_FMT "\n", ptr_struct->instruction);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_elapsed_since_last_cmd : " UH_FMT "\n", ptr_struct->time_elapsed_since_last_cmd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "component_index      : " UH_FMT "\n", ptr_struct->component_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "update_handle        : " UH_FMT "\n", ptr_struct->update_handle);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "control_state        : " UH_FMT "\n", ptr_struct->control_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_code           : " UH_FMT "\n", ptr_struct->error_code);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "control_progress     : " UH_FMT "\n", ptr_struct->control_progress);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "handle_owner_host_id : " UH_FMT "\n", ptr_struct->handle_owner_host_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "handle_owner_type    : " UH_FMT "\n", ptr_struct->handle_owner_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "component_size       : " U32H_FMT "\n", ptr_struct->component_size);

}

int connectib_mcc_reg_size(void)
{
	 return 32;
}

void connectib_mcc_reg_dump(const struct connectib_mcc_reg *ptr_struct, FILE* file)
{
	connectib_mcc_reg_print(ptr_struct, file, 0);
}

void connectib_mcqi_reg_pack(const struct connectib_mcqi_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->component_index);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->read_pending_component);

	offset = 91;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->info_type);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->info_size);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->offset);

	offset = 176;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->data_size);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(192, 32, i, 224, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

}

void connectib_mcqi_reg_unpack(struct connectib_mcqi_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->component_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->read_pending_component = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 91;
	ptr_struct->info_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 96;
	ptr_struct->info_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->offset = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 176;
	ptr_struct->data_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(192, 32, i, 224, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_mcqi_reg_print(const struct connectib_mcqi_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mcqi_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "component_index      : " UH_FMT "\n", ptr_struct->component_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_pending_component : " UH_FMT "\n", ptr_struct->read_pending_component);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "info_type            : " UH_FMT "\n", ptr_struct->info_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "info_size            : " U32H_FMT "\n", ptr_struct->info_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset               : " U32H_FMT "\n", ptr_struct->offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_size            : " UH_FMT "\n", ptr_struct->data_size);

	for (i = 0; i < 1; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

}

int connectib_mcqi_reg_size(void)
{
	 return 28;
}

void connectib_mcqi_reg_dump(const struct connectib_mcqi_reg *ptr_struct, FILE* file)
{
	connectib_mcqi_reg_print(ptr_struct, file, 0);
}

void connectib_mcqs_reg_pack(const struct connectib_mcqs_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->component_index);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->last_index_flag);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->identifier);

	offset = 92;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->component_update_state);

	offset = 87;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->component_status);

	offset = 100;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->last_update_state_changer_host_id);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->last_update_state_changer_type);

}

void connectib_mcqs_reg_unpack(struct connectib_mcqs_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->component_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->last_index_flag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->identifier = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 92;
	ptr_struct->component_update_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 87;
	ptr_struct->component_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 100;
	ptr_struct->last_update_state_changer_host_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->last_update_state_changer_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_mcqs_reg_print(const struct connectib_mcqs_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mcqs_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "component_index      : " UH_FMT "\n", ptr_struct->component_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_index_flag      : " UH_FMT "\n", ptr_struct->last_index_flag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "identifier           : " UH_FMT "\n", ptr_struct->identifier);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "component_update_state : " UH_FMT "\n", ptr_struct->component_update_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "component_status     : " UH_FMT "\n", ptr_struct->component_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_update_state_changer_host_id : " UH_FMT "\n", ptr_struct->last_update_state_changer_host_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_update_state_changer_type : " UH_FMT "\n", ptr_struct->last_update_state_changer_type);

}

int connectib_mcqs_reg_size(void)
{
	 return 16;
}

void connectib_mcqs_reg_dump(const struct connectib_mcqs_reg *ptr_struct, FILE* file)
{
	connectib_mcqs_reg_print(ptr_struct, file, 0);
}

void connectib_mfbe_pack(const struct connectib_mfbe *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->fs);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->p);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bulk_64kb_erase);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bulk_32kb_erase);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->address);

}

void connectib_mfbe_unpack(struct connectib_mfbe *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->fs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 23;
	ptr_struct->p = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->bulk_64kb_erase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->bulk_32kb_erase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->address = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_mfbe_print(const struct connectib_mfbe *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mfbe ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fs                   : " UH_FMT "\n", ptr_struct->fs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "p                    : " UH_FMT "\n", ptr_struct->p);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bulk_64kb_erase      : " UH_FMT "\n", ptr_struct->bulk_64kb_erase);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bulk_32kb_erase      : " UH_FMT "\n", ptr_struct->bulk_32kb_erase);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address              : " UH_FMT "\n", ptr_struct->address);

}

int connectib_mfbe_size(void)
{
	 return 12;
}

void connectib_mfbe_dump(const struct connectib_mfbe *ptr_struct, FILE* file)
{
	connectib_mfbe_print(ptr_struct, file, 0);
}

void connectib_project_mfba_pack(const struct connectib_project_mfba *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mfba_header_pack(&(ptr_struct->mfba_header), ptr_buff + offset/8);

	offset = 96;
	connectib_register_access_data_pack(&(ptr_struct->mfba_data), ptr_buff + offset/8);

}

void connectib_project_mfba_unpack(struct connectib_project_mfba *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mfba_header_unpack(&(ptr_struct->mfba_header), ptr_buff + offset/8);

	offset = 96;
	connectib_register_access_data_unpack(&(ptr_struct->mfba_data), ptr_buff + offset/8);

}

void connectib_project_mfba_print(const struct connectib_project_mfba *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_project_mfba ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfba_header:\n");
	connectib_mfba_header_print(&(ptr_struct->mfba_header), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfba_data:\n");
	connectib_register_access_data_print(&(ptr_struct->mfba_data), file, indent_level + 1);

}

int connectib_project_mfba_size(void)
{
	 return 44;
}

void connectib_project_mfba_dump(const struct connectib_project_mfba *ptr_struct, FILE* file)
{
	connectib_project_mfba_print(ptr_struct, file, 0);
}

void connectib_register_misc_counters_pack(const struct connectib_register_misc_counters *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->counter_select);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ecc);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ldb_silent_drop);

}

void connectib_register_misc_counters_unpack(struct connectib_register_misc_counters *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	ptr_struct->counter_select = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->ecc = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->ldb_silent_drop = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_register_misc_counters_print(const struct connectib_register_misc_counters *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_misc_counters ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "counter_select       : " U32H_FMT "\n", ptr_struct->counter_select);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecc                  : " U32H_FMT "\n", ptr_struct->ecc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ldb_silent_drop      : " U32H_FMT "\n", ptr_struct->ldb_silent_drop);

}

int connectib_register_misc_counters_size(void)
{
	 return 32;
}

void connectib_register_misc_counters_dump(const struct connectib_register_misc_counters *ptr_struct, FILE* file)
{
	connectib_register_misc_counters_print(ptr_struct, file, 0);
}

void connectib_mtmp_pack(const struct connectib_mtmp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->sensor_index);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->internal_diodes_query);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->temperature);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_temperture);

	offset = 65;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mtr);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mte);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->temperature_threshold_hi);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->tee);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->temperature_threshold_lo);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->name_hi);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->name_lo);

}

void connectib_mtmp_unpack(struct connectib_mtmp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	ptr_struct->sensor_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 0;
	ptr_struct->internal_diodes_query = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->temperature = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 80;
	ptr_struct->max_temperture = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 65;
	ptr_struct->mtr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->mte = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 112;
	ptr_struct->temperature_threshold_hi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->tee = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 144;
	ptr_struct->temperature_threshold_lo = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 192;
	ptr_struct->name_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->name_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_mtmp_print(const struct connectib_mtmp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mtmp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sensor_index         : " UH_FMT "\n", ptr_struct->sensor_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "internal_diodes_query : " UH_FMT "\n", ptr_struct->internal_diodes_query);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "temperature          : " UH_FMT "\n", ptr_struct->temperature);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_temperture       : " UH_FMT "\n", ptr_struct->max_temperture);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtr                  : " UH_FMT "\n", ptr_struct->mtr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mte                  : " UH_FMT "\n", ptr_struct->mte);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "temperature_threshold_hi : " UH_FMT "\n", ptr_struct->temperature_threshold_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tee                  : " UH_FMT "\n", ptr_struct->tee);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "temperature_threshold_lo : " UH_FMT "\n", ptr_struct->temperature_threshold_lo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "name_hi              : " U32H_FMT "\n", ptr_struct->name_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "name_lo              : " U32H_FMT "\n", ptr_struct->name_lo);

}

int connectib_mtmp_size(void)
{
	 return 32;
}

void connectib_mtmp_dump(const struct connectib_mtmp *ptr_struct, FILE* file)
{
	connectib_mtmp_print(ptr_struct, file, 0);
}

void connectib_register_mfrl_pack(const struct connectib_register_mfrl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->reset_level);

}

void connectib_register_mfrl_unpack(struct connectib_register_mfrl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 56;
	ptr_struct->reset_level = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_register_mfrl_print(const struct connectib_register_mfrl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_mfrl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reset_level          : " UH_FMT "\n", ptr_struct->reset_level);

}

int connectib_register_mfrl_size(void)
{
	 return 16;
}

void connectib_register_mfrl_dump(const struct connectib_register_mfrl *ptr_struct, FILE* file)
{
	connectib_register_mfrl_print(ptr_struct, file, 0);
}

void connectib_register_mfai_pack(const struct connectib_register_mfai *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->address);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->permanent);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->use_address);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->use_image_id);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->image_id);

}

void connectib_register_mfai_unpack(struct connectib_register_mfai *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->address = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 2;
	ptr_struct->permanent = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->use_address = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->use_image_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 56;
	ptr_struct->image_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_register_mfai_print(const struct connectib_register_mfai *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_mfai ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address              : " UH_FMT "\n", ptr_struct->address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "permanent            : " UH_FMT "\n", ptr_struct->permanent);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "use_address          : " UH_FMT "\n", ptr_struct->use_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "use_image_id         : " UH_FMT "\n", ptr_struct->use_image_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "image_id             : " UH_FMT "\n", ptr_struct->image_id);

}

int connectib_register_mfai_size(void)
{
	 return 16;
}

void connectib_register_mfai_dump(const struct connectib_register_mfai *ptr_struct, FILE* file)
{
	connectib_register_mfai_print(ptr_struct, file, 0);
}

void connectib_mfpa_pack(const struct connectib_mfpa *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->fs);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->p);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->boot_address);

	offset = 156;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->flash_num);

	offset = 143;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sub_sector_wrp_en);

	offset = 142;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sector_wrp_en);

	offset = 130;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bulk_64kb_erase_en);

	offset = 129;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bulk_32kb_erase_en);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wip);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->jedec_id);

	offset = 214;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->sector_size);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->block_allignment);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->block_size);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->capability_mask);

}

void connectib_mfpa_unpack(struct connectib_mfpa *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->fs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 23;
	ptr_struct->p = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 40;
	ptr_struct->boot_address = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 156;
	ptr_struct->flash_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 143;
	ptr_struct->sub_sector_wrp_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 142;
	ptr_struct->sector_wrp_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 130;
	ptr_struct->bulk_64kb_erase_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 129;
	ptr_struct->bulk_32kb_erase_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 128;
	ptr_struct->wip = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->jedec_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 214;
	ptr_struct->sector_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 200;
	ptr_struct->block_allignment = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 192;
	ptr_struct->block_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 224;
	ptr_struct->capability_mask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_mfpa_print(const struct connectib_mfpa *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mfpa ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fs                   : " UH_FMT "\n", ptr_struct->fs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "p                    : " UH_FMT "\n", ptr_struct->p);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "boot_address         : " UH_FMT "\n", ptr_struct->boot_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_num            : " UH_FMT "\n", ptr_struct->flash_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sub_sector_wrp_en    : " UH_FMT "\n", ptr_struct->sub_sector_wrp_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sector_wrp_en        : " UH_FMT "\n", ptr_struct->sector_wrp_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bulk_64kb_erase_en   : " UH_FMT "\n", ptr_struct->bulk_64kb_erase_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bulk_32kb_erase_en   : " UH_FMT "\n", ptr_struct->bulk_32kb_erase_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wip                  : " UH_FMT "\n", ptr_struct->wip);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "jedec_id             : " UH_FMT "\n", ptr_struct->jedec_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sector_size          : " UH_FMT "\n", ptr_struct->sector_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "block_allignment     : " UH_FMT "\n", ptr_struct->block_allignment);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "block_size           : " UH_FMT "\n", ptr_struct->block_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capability_mask      : " U32H_FMT "\n", ptr_struct->capability_mask);

}

int connectib_mfpa_size(void)
{
	 return 32;
}

void connectib_mfpa_dump(const struct connectib_mfpa *ptr_struct, FILE* file)
{
	connectib_mfpa_print(ptr_struct, file, 0);
}

void connectib_fpga_shell_caps_pack(const struct connectib_fpga_shell_caps *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->total_rcv_credits);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_num_qps);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rc);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->uc);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ud);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dc);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rre);

	offset = 54;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rwe);

	offset = 53;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rae);

	offset = 46;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->qp_type);

	offset = 90;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log_ddr_size);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_fpga_qp_msg_size);

}

void connectib_fpga_shell_caps_unpack(struct connectib_fpga_shell_caps *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->total_rcv_credits = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->max_num_qps = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 63;
	ptr_struct->rc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 62;
	ptr_struct->uc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 61;
	ptr_struct->ud = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 60;
	ptr_struct->dc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 55;
	ptr_struct->rre = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 54;
	ptr_struct->rwe = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 53;
	ptr_struct->rae = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 46;
	ptr_struct->qp_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 90;
	ptr_struct->log_ddr_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 96;
	ptr_struct->max_fpga_qp_msg_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_fpga_shell_caps_print(const struct connectib_fpga_shell_caps *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fpga_shell_caps ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "total_rcv_credits    : " UH_FMT "\n", ptr_struct->total_rcv_credits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_qps          : " UH_FMT "\n", ptr_struct->max_num_qps);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rc                   : " UH_FMT "\n", ptr_struct->rc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uc                   : " UH_FMT "\n", ptr_struct->uc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ud                   : " UH_FMT "\n", ptr_struct->ud);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc                   : " UH_FMT "\n", ptr_struct->dc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rre                  : " UH_FMT "\n", ptr_struct->rre);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwe                  : " UH_FMT "\n", ptr_struct->rwe);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rae                  : " UH_FMT "\n", ptr_struct->rae);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_type              : %s (" UH_FMT ")\n", (ptr_struct->qp_type == 1 ? ("Shell_qp") : ((ptr_struct->qp_type == 2 ? ("Sandbox_qp") : ("unknown")))), ptr_struct->qp_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_ddr_size         : " UH_FMT "\n", ptr_struct->log_ddr_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_fpga_qp_msg_size : " U32H_FMT "\n", ptr_struct->max_fpga_qp_msg_size);

}

int connectib_fpga_shell_caps_size(void)
{
	 return 64;
}

void connectib_fpga_shell_caps_dump(const struct connectib_fpga_shell_caps *ptr_struct, FILE* file)
{
	connectib_fpga_shell_caps_print(ptr_struct, file, 0);
}

void connectib_pmmp_sfp_protocol_override_layout_auto_pack(const union connectib_pmmp_sfp_protocol_override_layout_auto *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_pmmp_sfp_protocol_override_layout_pack(&(ptr_struct->pmmp_sfp_protocol_override_layout), ptr_buff);
}

void connectib_pmmp_sfp_protocol_override_layout_auto_unpack(union connectib_pmmp_sfp_protocol_override_layout_auto *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_pmmp_sfp_protocol_override_layout_unpack(&(ptr_struct->pmmp_sfp_protocol_override_layout), ptr_buff);
}

void connectib_pmmp_sfp_protocol_override_layout_auto_print(const union connectib_pmmp_sfp_protocol_override_layout_auto *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmmp_sfp_protocol_override_layout_auto ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pmmp_sfp_protocol_override_layout:\n");
	connectib_pmmp_sfp_protocol_override_layout_print(&(ptr_struct->pmmp_sfp_protocol_override_layout), file, indent_level + 1);

}

int connectib_pmmp_sfp_protocol_override_layout_auto_size(void)
{
	 return 12;
}

void connectib_pmmp_sfp_protocol_override_layout_auto_dump(const union connectib_pmmp_sfp_protocol_override_layout_auto *ptr_struct, FILE* file)
{
	connectib_pmmp_sfp_protocol_override_layout_auto_print(ptr_struct, file, 0);
}

void connectib_pmmp_qsfp_protocol_override_layout_auto_pack(const union connectib_pmmp_qsfp_protocol_override_layout_auto *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_pmmp_qsfp_protocol_override_layout_pack(&(ptr_struct->pmmp_qsfp_protocol_override_layout), ptr_buff);
}

void connectib_pmmp_qsfp_protocol_override_layout_auto_unpack(union connectib_pmmp_qsfp_protocol_override_layout_auto *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_pmmp_qsfp_protocol_override_layout_unpack(&(ptr_struct->pmmp_qsfp_protocol_override_layout), ptr_buff);
}

void connectib_pmmp_qsfp_protocol_override_layout_auto_print(const union connectib_pmmp_qsfp_protocol_override_layout_auto *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmmp_qsfp_protocol_override_layout_auto ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pmmp_qsfp_protocol_override_layout:\n");
	connectib_pmmp_qsfp_protocol_override_layout_print(&(ptr_struct->pmmp_qsfp_protocol_override_layout), file, indent_level + 1);

}

int connectib_pmmp_qsfp_protocol_override_layout_auto_size(void)
{
	 return 12;
}

void connectib_pmmp_qsfp_protocol_override_layout_auto_dump(const union connectib_pmmp_qsfp_protocol_override_layout_auto *ptr_struct, FILE* file)
{
	connectib_pmmp_qsfp_protocol_override_layout_auto_print(ptr_struct, file, 0);
}

void connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto_pack(const union connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_pcie_timers_states_pack(&(ptr_struct->pcie_timers_states), ptr_buff);
}

void connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto_unpack(union connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_pcie_timers_states_unpack(&(ptr_struct->pcie_timers_states), ptr_buff);
}

void connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto_print(const union connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_perf_counters:\n");
	connectib_pcie_perf_counters_print(&(ptr_struct->pcie_perf_counters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_lanes_counters:\n");
	connectib_pcie_lanes_counters_print(&(ptr_struct->pcie_lanes_counters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_timers_states:\n");
	connectib_pcie_timers_states_print(&(ptr_struct->pcie_timers_states), file, indent_level + 1);

}

int connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto_size(void)
{
	 return 248;
}

void connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto_dump(const union connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto *ptr_struct, FILE* file)
{
	connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto_print(ptr_struct, file, 0);
}

void connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto_pack(const union connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_phys_layer_stat_cntrs_pack(&(ptr_struct->phys_layer_stat_cntrs), ptr_buff);
}

void connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto_unpack(union connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_phys_layer_stat_cntrs_unpack(&(ptr_struct->phys_layer_stat_cntrs), ptr_buff);
}

void connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto_print(const union connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_802_3_cntrs_grp_data_layout:\n");
	connectib_eth_802_3_cntrs_grp_data_layout_print(&(ptr_struct->eth_802_3_cntrs_grp_data_layout), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_2863_cntrs_grp_data_layout:\n");
	connectib_eth_2863_cntrs_grp_data_layout_print(&(ptr_struct->eth_2863_cntrs_grp_data_layout), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_2819_cntrs_grp_data_layout:\n");
	connectib_eth_2819_cntrs_grp_data_layout_print(&(ptr_struct->eth_2819_cntrs_grp_data_layout), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_3635_cntrs_grp_data_layout:\n");
	connectib_eth_3635_cntrs_grp_data_layout_print(&(ptr_struct->eth_3635_cntrs_grp_data_layout), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_extended_cntrs_grp_data_layout:\n");
	connectib_eth_extended_cntrs_grp_data_layout_print(&(ptr_struct->eth_extended_cntrs_grp_data_layout), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_per_prio_grp_data_layout:\n");
	connectib_eth_per_prio_grp_data_layout_print(&(ptr_struct->eth_per_prio_grp_data_layout), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_per_traffic_grp_data_layout:\n");
	connectib_eth_per_traffic_grp_data_layout_print(&(ptr_struct->eth_per_traffic_grp_data_layout), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "correct_on_demand:\n");
	connectib_correct_on_demand_layout_print(&(ptr_struct->correct_on_demand), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phys_layer_cntrs:\n");
	connectib_phys_layer_cntrs_print(&(ptr_struct->phys_layer_cntrs), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "discard:\n");
	connectib_discard_cntrs_grp_data_print(&(ptr_struct->discard), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_counters:\n");
	connectib_llr_counters_print(&(ptr_struct->llr_counters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_port_counters:\n");
	connectib_ib_port_counters_print(&(ptr_struct->ib_port_counters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_per_traffic_congestion_grp_data_layout:\n");
	connectib_eth_per_traffic_congestion_grp_data_layout_print(&(ptr_struct->eth_per_traffic_congestion_grp_data_layout), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phys_layer_stat_cntrs:\n");
	connectib_phys_layer_stat_cntrs_print(&(ptr_struct->phys_layer_stat_cntrs), file, indent_level + 1);

}

int connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto_size(void)
{
	 return 248;
}

void connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto_dump(const union connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto *ptr_struct, FILE* file)
{
	connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto_print(ptr_struct, file, 0);
}

void connectib_access_register_unit_perf_counters_data_line_pack(const struct connectib_access_register_unit_perf_counters_data_line *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->value);

}

void connectib_access_register_unit_perf_counters_data_line_unpack(struct connectib_access_register_unit_perf_counters_data_line *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->value = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_access_register_unit_perf_counters_data_line_print(const struct connectib_access_register_unit_perf_counters_data_line *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_access_register_unit_perf_counters_data_line ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "value                : " U32H_FMT "\n", ptr_struct->value);

}

int connectib_access_register_unit_perf_counters_data_line_size(void)
{
	 return 4;
}

void connectib_access_register_unit_perf_counters_data_line_dump(const struct connectib_access_register_unit_perf_counters_data_line *ptr_struct, FILE* file)
{
	connectib_access_register_unit_perf_counters_data_line_print(ptr_struct, file, 0);
}

void connectib_access_register_unit_perf_counters_cfg_line_pack(const struct connectib_access_register_unit_perf_counters_cfg_line *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->select);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cmd);

}

void connectib_access_register_unit_perf_counters_cfg_line_unpack(struct connectib_access_register_unit_perf_counters_cfg_line *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->select = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->cmd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_access_register_unit_perf_counters_cfg_line_print(const struct connectib_access_register_unit_perf_counters_cfg_line *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_access_register_unit_perf_counters_cfg_line ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "select               : " UH_FMT "\n", ptr_struct->select);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmd                  : " UH_FMT "\n", ptr_struct->cmd);

}

int connectib_access_register_unit_perf_counters_cfg_line_size(void)
{
	 return 4;
}

void connectib_access_register_unit_perf_counters_cfg_line_dump(const struct connectib_access_register_unit_perf_counters_cfg_line *ptr_struct, FILE* file)
{
	connectib_access_register_unit_perf_counters_cfg_line_print(ptr_struct, file, 0);
}

void connectib_node_description_pack(const struct connectib_node_description *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->d[i]);
	}

}

void connectib_node_description_unpack(struct connectib_node_description *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	ptr_struct->d[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_node_description_print(const struct connectib_node_description *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_node_description ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "d_%03d               : " U32H_FMT "\n", i, ptr_struct->d[i]);
	}

}

int connectib_node_description_size(void)
{
	 return 64;
}

void connectib_node_description_dump(const struct connectib_node_description *ptr_struct, FILE* file)
{
	connectib_node_description_print(ptr_struct, file, 0);
}

void connectib_application_prio_entry_pack(const struct connectib_application_prio_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->protocol_id);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->sel);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->priority);

}

void connectib_application_prio_entry_unpack(struct connectib_application_prio_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->protocol_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 13;
	ptr_struct->sel = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 8;
	ptr_struct->priority = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_application_prio_entry_print(const struct connectib_application_prio_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_application_prio_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "protocol_id          : " UH_FMT "\n", ptr_struct->protocol_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sel                  : " UH_FMT "\n", ptr_struct->sel);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "priority             : " UH_FMT "\n", ptr_struct->priority);

}

int connectib_application_prio_entry_size(void)
{
	 return 4;
}

void connectib_application_prio_entry_dump(const struct connectib_application_prio_entry *ptr_struct, FILE* file)
{
	connectib_application_prio_entry_print(ptr_struct, file, 0);
}

void connectib_tc_qos_configuration_pack(const struct connectib_tc_qos_configuration *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->bw_allocation);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->group);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->r);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->b);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->g);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->max_bw_value);

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_bw_units);

}

void connectib_tc_qos_configuration_unpack(struct connectib_tc_qos_configuration *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	ptr_struct->bw_allocation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 12;
	ptr_struct->group = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 2;
	ptr_struct->r = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->b = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 56;
	ptr_struct->max_bw_value = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 44;
	ptr_struct->max_bw_units = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_tc_qos_configuration_print(const struct connectib_tc_qos_configuration *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_tc_qos_configuration ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bw_allocation        : " UH_FMT "\n", ptr_struct->bw_allocation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "group                : " UH_FMT "\n", ptr_struct->group);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "r                    : " UH_FMT "\n", ptr_struct->r);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "b                    : " UH_FMT "\n", ptr_struct->b);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "g                    : " UH_FMT "\n", ptr_struct->g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_bw_value         : " UH_FMT "\n", ptr_struct->max_bw_value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_bw_units         : " UH_FMT "\n", ptr_struct->max_bw_units);

}

int connectib_tc_qos_configuration_size(void)
{
	 return 8;
}

void connectib_tc_qos_configuration_dump(const struct connectib_tc_qos_configuration *ptr_struct, FILE* file)
{
	connectib_tc_qos_configuration_print(ptr_struct, file, 0);
}

void connectib_fw_ref_count_pack(const struct connectib_fw_ref_count *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 28, (u_int32_t)ptr_struct->ref_counter);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ref_valid_cre_type);

}

void connectib_fw_ref_count_unpack(struct connectib_fw_ref_count *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 4;
	ptr_struct->ref_counter = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 28);

	offset = 0;
	ptr_struct->ref_valid_cre_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_fw_ref_count_print(const struct connectib_fw_ref_count *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_ref_count ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ref_counter          : " UH_FMT "\n", ptr_struct->ref_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ref_valid_cre_type   : " UH_FMT "\n", ptr_struct->ref_valid_cre_type);

}

int connectib_fw_ref_count_size(void)
{
	 return 4;
}

void connectib_fw_ref_count_dump(const struct connectib_fw_ref_count *ptr_struct, FILE* file)
{
	connectib_fw_ref_count_print(ptr_struct, file, 0);
}

void connectib_mac_address_layout_pack(const struct connectib_mac_address_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->mac_addr_47_32);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mac_addr_31_0);

}

void connectib_mac_address_layout_unpack(struct connectib_mac_address_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->mac_addr_47_32 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->mac_addr_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_mac_address_layout_print(const struct connectib_mac_address_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mac_address_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mac_addr_47_32       : " UH_FMT "\n", ptr_struct->mac_addr_47_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mac_addr_31_0        : " U32H_FMT "\n", ptr_struct->mac_addr_31_0);

}

int connectib_mac_address_layout_size(void)
{
	 return 8;
}

void connectib_mac_address_layout_dump(const struct connectib_mac_address_layout *ptr_struct, FILE* file)
{
	connectib_mac_address_layout_print(ptr_struct, file, 0);
}

void connectib_tir_dmfs_pack(const struct connectib_tir_dmfs *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tir_ste_ix);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hash_filter_base_ix);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hash_entries_base_ix);

}

void connectib_tir_dmfs_unpack(struct connectib_tir_dmfs *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	ptr_struct->tir_ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->hash_filter_base_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->hash_entries_base_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_tir_dmfs_print(const struct connectib_tir_dmfs *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_tir_dmfs ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tir_ste_ix           : " U32H_FMT "\n", ptr_struct->tir_ste_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hash_filter_base_ix  : " U32H_FMT "\n", ptr_struct->hash_filter_base_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hash_entries_base_ix : " U32H_FMT "\n", ptr_struct->hash_entries_base_ix);

}

int connectib_tir_dmfs_size(void)
{
	 return 16;
}

void connectib_tir_dmfs_dump(const struct connectib_tir_dmfs *ptr_struct, FILE* file)
{
	connectib_tir_dmfs_print(ptr_struct, file, 0);
}

void connectib_rx_hash_field_select_pack(const struct connectib_rx_hash_field_select *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 30, (u_int32_t)ptr_struct->selected_fields);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->l4_prot_type);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->l3_prot_type);

}

void connectib_rx_hash_field_select_unpack(struct connectib_rx_hash_field_select *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	ptr_struct->selected_fields = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 30);

	offset = 1;
	ptr_struct->l4_prot_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->l3_prot_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_rx_hash_field_select_print(const struct connectib_rx_hash_field_select *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rx_hash_field_select ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "selected_fields      : %s (" UH_FMT ")\n", (ptr_struct->selected_fields == 0 ? ("SRC_IP") : ((ptr_struct->selected_fields == 1 ? ("DST_IP") : ((ptr_struct->selected_fields == 2 ? ("L4_SPORT") : ((ptr_struct->selected_fields == 3 ? ("L4_DPORT") : ((ptr_struct->selected_fields == 4 ? ("IPSEC_SPI") : ("unknown")))))))))), ptr_struct->selected_fields);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l4_prot_type         : " UH_FMT "\n", ptr_struct->l4_prot_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l3_prot_type         : " UH_FMT "\n", ptr_struct->l3_prot_type);

}

int connectib_rx_hash_field_select_size(void)
{
	 return 4;
}

void connectib_rx_hash_field_select_dump(const struct connectib_rx_hash_field_select *ptr_struct, FILE* file)
{
	connectib_rx_hash_field_select_print(ptr_struct, file, 0);
}

void connectib_fw_qpc_special_pack(const union connectib_fw_qpc_special *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_dct_qp_pack(&(ptr_struct->dct_qp), ptr_buff);
}

void connectib_fw_qpc_special_unpack(union connectib_fw_qpc_special *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_dct_qp_unpack(&(ptr_struct->dct_qp), ptr_buff);
}

void connectib_fw_qpc_special_print(const union connectib_fw_qpc_special *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_qpc_special ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "regular_qp:\n");
	connectib_regular_qp_print(&(ptr_struct->regular_qp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcr_qp:\n");
	connectib_dcr_qp_print(&(ptr_struct->dcr_qp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dct_qp:\n");
	connectib_dct_qp_print(&(ptr_struct->dct_qp), file, indent_level + 1);

}

int connectib_fw_qpc_special_size(void)
{
	 return 16;
}

void connectib_fw_qpc_special_dump(const union connectib_fw_qpc_special *ptr_struct, FILE* file)
{
	connectib_fw_qpc_special_print(ptr_struct, file, 0);
}

void connectib_fw_qpc_q_pack(const struct connectib_fw_qpc_q *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->qp_state);

	offset = 16;
	connectib_schedint_cause_pack(&(ptr_struct->schedint_cause), ptr_buff + offset/8);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sxdc_valid);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qpc_valid);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->dct_cleanup_state);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qp_in_linked_list);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cmdif_awaiting);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dup_sx_flush_kill_qp_arb_disabled);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->broken_wq_buffer);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->broken_cq);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->catas_event_generated);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->last_wqe_event_generated);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->async_event_generated);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->flushed_in_error);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sl_diff_flow_done);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->all_connections_closed_event_req);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_flushed_in_error);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->packet_pacing_rate_limit_index);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rx_heartbeat_odp_ext);

	offset = 54;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rx_heartbeat_gnrl_ext);

	offset = 53;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rx_heartbeat_retry_cntr_ext);

	offset = 51;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_heartbeat_odp_ext);

	offset = 50;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_heartbeat_gnrl_ext);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vendor_syndrome);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->hw_syndrome);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->trap_reason);

	offset = 122;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->connect_sent_to_ctrl_dct);

	offset = 121;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->needed_events_generated);

	offset = 116;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->retransmission_state);

	offset = 115;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rx_fault_event_fired);

	offset = 113;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_fault_event_fired);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->trap_data_odp_gvmi);

}

void connectib_fw_qpc_q_unpack(struct connectib_fw_qpc_q *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->qp_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	connectib_schedint_cause_unpack(&(ptr_struct->schedint_cause), ptr_buff + offset/8);

	offset = 15;
	ptr_struct->sxdc_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->qpc_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->dct_cleanup_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 11;
	ptr_struct->qp_in_linked_list = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->cmdif_awaiting = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->dup_sx_flush_kill_qp_arb_disabled = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->broken_wq_buffer = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->broken_cq = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->catas_event_generated = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->last_wqe_event_generated = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->async_event_generated = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->flushed_in_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->sl_diff_flow_done = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->all_connections_closed_event_req = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->disable_flushed_in_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 56;
	ptr_struct->packet_pacing_rate_limit_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 55;
	ptr_struct->rx_heartbeat_odp_ext = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 54;
	ptr_struct->rx_heartbeat_gnrl_ext = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 53;
	ptr_struct->rx_heartbeat_retry_cntr_ext = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 51;
	ptr_struct->sx_heartbeat_odp_ext = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 50;
	ptr_struct->sx_heartbeat_gnrl_ext = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->vendor_syndrome = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->hw_syndrome = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 123;
	ptr_struct->trap_reason = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 122;
	ptr_struct->connect_sent_to_ctrl_dct = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 121;
	ptr_struct->needed_events_generated = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 116;
	ptr_struct->retransmission_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 115;
	ptr_struct->rx_fault_event_fired = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 113;
	ptr_struct->sx_fault_event_fired = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->trap_data_odp_gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_fw_qpc_q_print(const struct connectib_fw_qpc_q *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_qpc_q ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_state             : " UH_FMT "\n", ptr_struct->qp_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "schedint_cause:\n");
	connectib_schedint_cause_print(&(ptr_struct->schedint_cause), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sxdc_valid           : " UH_FMT "\n", ptr_struct->sxdc_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpc_valid            : " UH_FMT "\n", ptr_struct->qpc_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dct_cleanup_state    : " UH_FMT "\n", ptr_struct->dct_cleanup_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_in_linked_list    : " UH_FMT "\n", ptr_struct->qp_in_linked_list);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmdif_awaiting       : " UH_FMT "\n", ptr_struct->cmdif_awaiting);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dup_sx_flush_kill_qp_arb_disabled : " UH_FMT "\n", ptr_struct->dup_sx_flush_kill_qp_arb_disabled);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "broken_wq_buffer     : " UH_FMT "\n", ptr_struct->broken_wq_buffer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "broken_cq            : " UH_FMT "\n", ptr_struct->broken_cq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "catas_event_generated : " UH_FMT "\n", ptr_struct->catas_event_generated);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_wqe_event_generated : " UH_FMT "\n", ptr_struct->last_wqe_event_generated);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "async_event_generated : " UH_FMT "\n", ptr_struct->async_event_generated);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flushed_in_error     : " UH_FMT "\n", ptr_struct->flushed_in_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl_diff_flow_done    : " UH_FMT "\n", ptr_struct->sl_diff_flow_done);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "all_connections_closed_event_req : " UH_FMT "\n", ptr_struct->all_connections_closed_event_req);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_flushed_in_error : " UH_FMT "\n", ptr_struct->disable_flushed_in_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_pacing_rate_limit_index : " UH_FMT "\n", ptr_struct->packet_pacing_rate_limit_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_heartbeat_odp_ext : " UH_FMT "\n", ptr_struct->rx_heartbeat_odp_ext);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_heartbeat_gnrl_ext : " UH_FMT "\n", ptr_struct->rx_heartbeat_gnrl_ext);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_heartbeat_retry_cntr_ext : " UH_FMT "\n", ptr_struct->rx_heartbeat_retry_cntr_ext);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_heartbeat_odp_ext : " UH_FMT "\n", ptr_struct->sx_heartbeat_odp_ext);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_heartbeat_gnrl_ext : " UH_FMT "\n", ptr_struct->sx_heartbeat_gnrl_ext);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vendor_syndrome      : " UH_FMT "\n", ptr_struct->vendor_syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_syndrome          : " UH_FMT "\n", ptr_struct->hw_syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trap_reason          : %s (" UH_FMT ")\n", (ptr_struct->trap_reason == 0 ? ("NORMAL") : ((ptr_struct->trap_reason == 1 ? ("SET_LIMIT") : ((ptr_struct->trap_reason == 2 ? ("DCR_OVERFLOW") : ((ptr_struct->trap_reason == 3 ? ("DCR_DUP_WA") : ((ptr_struct->trap_reason == 4 ? ("DCI_RECONNECT") : ("unknown")))))))))), ptr_struct->trap_reason);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "connect_sent_to_ctrl_dct : " UH_FMT "\n", ptr_struct->connect_sent_to_ctrl_dct);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "needed_events_generated : " UH_FMT "\n", ptr_struct->needed_events_generated);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "retransmission_state : " UH_FMT "\n", ptr_struct->retransmission_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_fault_event_fired : " UH_FMT "\n", ptr_struct->rx_fault_event_fired);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_fault_event_fired : " UH_FMT "\n", ptr_struct->sx_fault_event_fired);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trap_data_odp_gvmi   : " UH_FMT "\n", ptr_struct->trap_data_odp_gvmi);

}

int connectib_fw_qpc_q_size(void)
{
	 return 16;
}

void connectib_fw_qpc_q_dump(const struct connectib_fw_qpc_q *ptr_struct, FILE* file)
{
	connectib_fw_qpc_q_print(ptr_struct, file, 0);
}

void connectib_fw_qpc_ts_special_pack(const union connectib_fw_qpc_ts_special *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_fw_qpc_ib_pack(&(ptr_struct->ib_qp), ptr_buff);
}

void connectib_fw_qpc_ts_special_unpack(union connectib_fw_qpc_ts_special *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_fw_qpc_ib_unpack(&(ptr_struct->ib_qp), ptr_buff);
}

void connectib_fw_qpc_ts_special_print(const union connectib_fw_qpc_ts_special *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_qpc_ts_special ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_qp:\n");
	connectib_fw_qpc_eth_print(&(ptr_struct->eth_qp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_qp:\n");
	connectib_fw_qpc_ib_print(&(ptr_struct->ib_qp), file, indent_level + 1);

}

int connectib_fw_qpc_ts_special_size(void)
{
	 return 4;
}

void connectib_fw_qpc_ts_special_dump(const union connectib_fw_qpc_ts_special *ptr_struct, FILE* file)
{
	connectib_fw_qpc_ts_special_print(ptr_struct, file, 0);
}

void connectib_fw_xrqc_pack(const struct connectib_fw_xrqc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_offloads);

	offset = 59;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->nsq_log_page_size);

	offset = 51;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->ncq_log_page_size);

	offset = 43;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->stg_log_page_size);

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->memory_page_size);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->tgt_num);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->send_cq_num);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ioccsize);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->log_match_list_size);

}

void connectib_fw_xrqc_unpack(struct connectib_fw_xrqc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->num_offloads = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 59;
	ptr_struct->nsq_log_page_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 51;
	ptr_struct->ncq_log_page_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 43;
	ptr_struct->stg_log_page_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 35;
	ptr_struct->memory_page_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 72;
	ptr_struct->tgt_num = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 112;
	ptr_struct->send_cq_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 144;
	ptr_struct->ioccsize = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 128;
	ptr_struct->log_match_list_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_fw_xrqc_print(const struct connectib_fw_xrqc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_xrqc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_offloads         : " U32H_FMT "\n", ptr_struct->num_offloads);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nsq_log_page_size    : " UH_FMT "\n", ptr_struct->nsq_log_page_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncq_log_page_size    : " UH_FMT "\n", ptr_struct->ncq_log_page_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "stg_log_page_size    : " UH_FMT "\n", ptr_struct->stg_log_page_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "memory_page_size     : " UH_FMT "\n", ptr_struct->memory_page_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tgt_num              : " UH_FMT "\n", ptr_struct->tgt_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "send_cq_num          : " UH_FMT "\n", ptr_struct->send_cq_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ioccsize             : " UH_FMT "\n", ptr_struct->ioccsize);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_match_list_size  : " UH_FMT "\n", ptr_struct->log_match_list_size);

}

int connectib_fw_xrqc_size(void)
{
	 return 32;
}

void connectib_fw_xrqc_dump(const struct connectib_fw_xrqc *ptr_struct, FILE* file)
{
	connectib_fw_xrqc_print(ptr_struct, file, 0);
}

void connectib_hw_msix_context_data_pack(const union connectib_hw_msix_context_data *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_msix_context_data0_pack(&(ptr_struct->regular), ptr_buff);
}

void connectib_hw_msix_context_data_unpack(union connectib_hw_msix_context_data *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_msix_context_data0_unpack(&(ptr_struct->regular), ptr_buff);
}

void connectib_hw_msix_context_data_print(const union connectib_hw_msix_context_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_msix_context_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "regular:\n");
	connectib_hw_msix_context_data0_print(&(ptr_struct->regular), file, indent_level + 1);

}

int connectib_hw_msix_context_data_size(void)
{
	 return 4;
}

void connectib_hw_msix_context_data_dump(const union connectib_hw_msix_context_data *ptr_struct, FILE* file)
{
	connectib_hw_msix_context_data_print(ptr_struct, file, 0);
}

void connectib_steering_qp_entry_pack(const struct connectib_steering_qp_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qp_number);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_responder);

}

void connectib_steering_qp_entry_unpack(struct connectib_steering_qp_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->qp_number = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->force_responder = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_steering_qp_entry_print(const struct connectib_steering_qp_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_steering_qp_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_number            : " UH_FMT "\n", ptr_struct->qp_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_responder      : " UH_FMT "\n", ptr_struct->force_responder);

}

int connectib_steering_qp_entry_size(void)
{
	 return 4;
}

void connectib_steering_qp_entry_dump(const struct connectib_steering_qp_entry *ptr_struct, FILE* file)
{
	connectib_steering_qp_entry_print(ptr_struct, file, 0);
}

void connectib_hw_ste_tag_pack(const union connectib_hw_ste_tag *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_ste_tag_generic_headers_pack(&(ptr_struct->generic_headers), ptr_buff);
}

void connectib_hw_ste_tag_unpack(union connectib_hw_ste_tag *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_ste_tag_generic_headers_unpack(&(ptr_struct->generic_headers), ptr_buff);
}

void connectib_hw_ste_tag_print(const union connectib_hw_ste_tag *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste_tag ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bulk:\n");
	connectib_hw_ste_tag_bulk_print(&(ptr_struct->bulk), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibl2:\n");
	connectib_hw_ste_tag_ib_l2_print(&(ptr_struct->ibl2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibl3_extended:\n");
	connectib_hw_ste_tag_ib_l3_extended_print(&(ptr_struct->ibl3_extended), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibl3:\n");
	connectib_hw_ste_tag_ib_l3_print(&(ptr_struct->ibl3), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibl4:\n");
	connectib_hw_ste_tag_ib_l4_print(&(ptr_struct->ibl4), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "source_gvmi_qp:\n");
	connectib_hw_ste_tag_source_gvmi_qp_print(&(ptr_struct->source_gvmi_qp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port:\n");
	connectib_hw_ste_tag_port_print(&(ptr_struct->port), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "generic_headers:\n");
	connectib_hw_ste_tag_generic_headers_print(&(ptr_struct->generic_headers), file, indent_level + 1);

}

int connectib_hw_ste_tag_size(void)
{
	 return 16;
}

void connectib_hw_ste_tag_dump(const union connectib_hw_ste_tag *ptr_struct, FILE* file)
{
	connectib_hw_ste_tag_print(ptr_struct, file, 0);
}

void connectib_hw_pdf_entry_pack(const union connectib_hw_pdf_entry *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_transaction_pdf_pack(&(ptr_struct->transaction_pdf), ptr_buff);
}

void connectib_hw_pdf_entry_unpack(union connectib_hw_pdf_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_transaction_pdf_unpack(&(ptr_struct->transaction_pdf), ptr_buff);
}

void connectib_hw_pdf_entry_print(const union connectib_hw_pdf_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_pdf_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pdf_decoder:\n");
	connectib_hw_pdf_decoder_print(&(ptr_struct->pdf_decoder), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transaction_pdf:\n");
	connectib_hw_transaction_pdf_print(&(ptr_struct->transaction_pdf), file, indent_level + 1);

}

int connectib_hw_pdf_entry_size(void)
{
	 return 8;
}

void connectib_hw_pdf_entry_dump(const union connectib_hw_pdf_entry *ptr_struct, FILE* file)
{
	connectib_hw_pdf_entry_print(ptr_struct, file, 0);
}

void connectib_hw_dct_dc_access_key_63_32_pack(const struct connectib_hw_dct_dc_access_key_63_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_dc_access_key_63_32_unpack(struct connectib_hw_dct_dc_access_key_63_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_dct_dc_access_key_63_32_print(const struct connectib_hw_dct_dc_access_key_63_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_dc_access_key_63_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_dc_access_key_63_32_size(void)
{
	 return 64;
}

void connectib_hw_dct_dc_access_key_63_32_dump(const struct connectib_hw_dct_dc_access_key_63_32 *ptr_struct, FILE* file)
{
	connectib_hw_dct_dc_access_key_63_32_print(ptr_struct, file, 0);
}

void connectib_hw_dct_dc_access_key_31_0_pack(const struct connectib_hw_dct_dc_access_key_31_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_dc_access_key_31_0_unpack(struct connectib_hw_dct_dc_access_key_31_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_dct_dc_access_key_31_0_print(const struct connectib_hw_dct_dc_access_key_31_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_dc_access_key_31_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_dc_access_key_31_0_size(void)
{
	 return 64;
}

void connectib_hw_dct_dc_access_key_31_0_dump(const struct connectib_hw_dct_dc_access_key_31_0 *ptr_struct, FILE* file)
{
	connectib_hw_dct_dc_access_key_31_0_print(ptr_struct, file, 0);
}

void connectib_hw_dct_tclass_pack(const struct connectib_hw_dct_tclass *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_tclass_unpack(struct connectib_hw_dct_tclass *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_dct_tclass_print(const struct connectib_hw_dct_tclass *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_tclass ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_tclass_size(void)
{
	 return 64;
}

void connectib_hw_dct_tclass_dump(const struct connectib_hw_dct_tclass *ptr_struct, FILE* file)
{
	connectib_hw_dct_tclass_print(ptr_struct, file, 0);
}

void connectib_hw_dct_context_number_pack(const struct connectib_hw_dct_context_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 424;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_context_number_unpack(struct connectib_hw_dct_context_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 424;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_dct_context_number_print(const struct connectib_hw_dct_context_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_context_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_context_number_size(void)
{
	 return 64;
}

void connectib_hw_dct_context_number_dump(const struct connectib_hw_dct_context_number *ptr_struct, FILE* file)
{
	connectib_hw_dct_context_number_print(ptr_struct, file, 0);
}

void connectib_hw_dct_hop_limit_pack(const struct connectib_hw_dct_hop_limit *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_hop_limit_unpack(struct connectib_hw_dct_hop_limit *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_dct_hop_limit_print(const struct connectib_hw_dct_hop_limit *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_hop_limit ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_hop_limit_size(void)
{
	 return 64;
}

void connectib_hw_dct_hop_limit_dump(const struct connectib_hw_dct_hop_limit *ptr_struct, FILE* file)
{
	connectib_hw_dct_hop_limit_print(ptr_struct, file, 0);
}

void connectib_hw_dct_cq_number_pack(const struct connectib_hw_dct_cq_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 392;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_cq_number_unpack(struct connectib_hw_dct_cq_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 392;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_dct_cq_number_print(const struct connectib_hw_dct_cq_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_cq_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_cq_number_size(void)
{
	 return 64;
}

void connectib_hw_dct_cq_number_dump(const struct connectib_hw_dct_cq_number *ptr_struct, FILE* file)
{
	connectib_hw_dct_cq_number_print(ptr_struct, file, 0);
}

void connectib_hw_dct_ts_pack(const struct connectib_hw_dct_ts *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_ts_unpack(struct connectib_hw_dct_ts *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_dct_ts_print(const struct connectib_hw_dct_ts *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_ts ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_ts_size(void)
{
	 return 64;
}

void connectib_hw_dct_ts_dump(const struct connectib_hw_dct_ts *ptr_struct, FILE* file)
{
	connectib_hw_dct_ts_print(ptr_struct, file, 0);
}

void connectib_hw_dct_ipd_pack(const struct connectib_hw_dct_ipd *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 356;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_ipd_unpack(struct connectib_hw_dct_ipd *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 356;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_dct_ipd_print(const struct connectib_hw_dct_ipd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_ipd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_ipd_size(void)
{
	 return 64;
}

void connectib_hw_dct_ipd_dump(const struct connectib_hw_dct_ipd *ptr_struct, FILE* file)
{
	connectib_hw_dct_ipd_print(ptr_struct, file, 0);
}

void connectib_hw_dct_my_gid_mac_index_pack(const struct connectib_hw_dct_my_gid_mac_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_my_gid_mac_index_unpack(struct connectib_hw_dct_my_gid_mac_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_dct_my_gid_mac_index_print(const struct connectib_hw_dct_my_gid_mac_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_my_gid_mac_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_my_gid_mac_index_size(void)
{
	 return 64;
}

void connectib_hw_dct_my_gid_mac_index_dump(const struct connectib_hw_dct_my_gid_mac_index *ptr_struct, FILE* file)
{
	connectib_hw_dct_my_gid_mac_index_print(ptr_struct, file, 0);
}

void connectib_hw_dct_extended_atomic_pack(const struct connectib_hw_dct_extended_atomic *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 368;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_extended_atomic_unpack(struct connectib_hw_dct_extended_atomic *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 368;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_dct_extended_atomic_print(const struct connectib_hw_dct_extended_atomic *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_extended_atomic ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_extended_atomic_size(void)
{
	 return 64;
}

void connectib_hw_dct_extended_atomic_dump(const struct connectib_hw_dct_extended_atomic *ptr_struct, FILE* file)
{
	connectib_hw_dct_extended_atomic_print(ptr_struct, file, 0);
}

void connectib_hw_dct_pkey_table_index_pack(const struct connectib_hw_dct_pkey_table_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 372;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_pkey_table_index_unpack(struct connectib_hw_dct_pkey_table_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 372;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

}

void connectib_hw_dct_pkey_table_index_print(const struct connectib_hw_dct_pkey_table_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_pkey_table_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_pkey_table_index_size(void)
{
	 return 64;
}

void connectib_hw_dct_pkey_table_index_dump(const struct connectib_hw_dct_pkey_table_index *ptr_struct, FILE* file)
{
	connectib_hw_dct_pkey_table_index_print(ptr_struct, file, 0);
}

void connectib_hw_dct_counter_index_pack(const struct connectib_hw_dct_counter_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 320;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_counter_index_unpack(struct connectib_hw_dct_counter_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 320;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_dct_counter_index_print(const struct connectib_hw_dct_counter_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_counter_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_counter_index_size(void)
{
	 return 64;
}

void connectib_hw_dct_counter_index_dump(const struct connectib_hw_dct_counter_index *ptr_struct, FILE* file)
{
	connectib_hw_dct_counter_index_print(ptr_struct, file, 0);
}

void connectib_hw_dct_srq_number_pack(const struct connectib_hw_dct_srq_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 328;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_srq_number_unpack(struct connectib_hw_dct_srq_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 328;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_dct_srq_number_print(const struct connectib_hw_dct_srq_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_srq_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_srq_number_size(void)
{
	 return 64;
}

void connectib_hw_dct_srq_number_dump(const struct connectib_hw_dct_srq_number *ptr_struct, FILE* file)
{
	connectib_hw_dct_srq_number_print(ptr_struct, file, 0);
}

void connectib_hw_dct_atomic_like_write_pack(const struct connectib_hw_dct_atomic_like_write *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_atomic_like_write_unpack(struct connectib_hw_dct_atomic_like_write *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 288;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_atomic_like_write_print(const struct connectib_hw_dct_atomic_like_write *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_atomic_like_write ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_atomic_like_write_size(void)
{
	 return 64;
}

void connectib_hw_dct_atomic_like_write_dump(const struct connectib_hw_dct_atomic_like_write *ptr_struct, FILE* file)
{
	connectib_hw_dct_atomic_like_write_print(ptr_struct, file, 0);
}

void connectib_hw_dct_my_lid_pack(const struct connectib_hw_dct_my_lid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 289;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_my_lid_unpack(struct connectib_hw_dct_my_lid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 289;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

}

void connectib_hw_dct_my_lid_print(const struct connectib_hw_dct_my_lid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_my_lid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_my_lid_size(void)
{
	 return 64;
}

void connectib_hw_dct_my_lid_dump(const struct connectib_hw_dct_my_lid *ptr_struct, FILE* file)
{
	connectib_hw_dct_my_lid_print(ptr_struct, file, 0);
}

void connectib_hw_dct_qp_valid_pack(const struct connectib_hw_dct_qp_valid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 296;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_qp_valid_unpack(struct connectib_hw_dct_qp_valid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 296;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_qp_valid_print(const struct connectib_hw_dct_qp_valid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_qp_valid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_qp_valid_size(void)
{
	 return 64;
}

void connectib_hw_dct_qp_valid_dump(const struct connectib_hw_dct_qp_valid *ptr_struct, FILE* file)
{
	connectib_hw_dct_qp_valid_print(ptr_struct, file, 0);
}

void connectib_hw_dct_max_read_atomic_pack(const struct connectib_hw_dct_max_read_atomic *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 297;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_max_read_atomic_unpack(struct connectib_hw_dct_max_read_atomic *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 297;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_dct_max_read_atomic_print(const struct connectib_hw_dct_max_read_atomic *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_max_read_atomic ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_max_read_atomic_size(void)
{
	 return 64;
}

void connectib_hw_dct_max_read_atomic_dump(const struct connectib_hw_dct_max_read_atomic *ptr_struct, FILE* file)
{
	connectib_hw_dct_max_read_atomic_print(ptr_struct, file, 0);
}

void connectib_hw_dct_flow_label_pack(const struct connectib_hw_dct_flow_label *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 300;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_flow_label_unpack(struct connectib_hw_dct_flow_label *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 300;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

}

void connectib_hw_dct_flow_label_print(const struct connectib_hw_dct_flow_label *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_flow_label ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_flow_label_size(void)
{
	 return 64;
}

void connectib_hw_dct_flow_label_dump(const struct connectib_hw_dct_flow_label *ptr_struct, FILE* file)
{
	connectib_hw_dct_flow_label_print(ptr_struct, file, 0);
}

void connectib_hw_dct_grh_bit_pack(const struct connectib_hw_dct_grh_bit *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 260;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_grh_bit_unpack(struct connectib_hw_dct_grh_bit *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 260;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_grh_bit_print(const struct connectib_hw_dct_grh_bit *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_grh_bit ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_grh_bit_size(void)
{
	 return 64;
}

void connectib_hw_dct_grh_bit_dump(const struct connectib_hw_dct_grh_bit *ptr_struct, FILE* file)
{
	connectib_hw_dct_grh_bit_print(ptr_struct, file, 0);
}

void connectib_hw_dct_port_number_pack(const struct connectib_hw_dct_port_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 261;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_port_number_unpack(struct connectib_hw_dct_port_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 261;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_port_number_print(const struct connectib_hw_dct_port_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_port_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_port_number_size(void)
{
	 return 64;
}

void connectib_hw_dct_port_number_dump(const struct connectib_hw_dct_port_number *ptr_struct, FILE* file)
{
	connectib_hw_dct_port_number_print(ptr_struct, file, 0);
}

void connectib_hw_dct_adaptive_routing_pack(const struct connectib_hw_dct_adaptive_routing *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 262;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_adaptive_routing_unpack(struct connectib_hw_dct_adaptive_routing *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 262;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_adaptive_routing_print(const struct connectib_hw_dct_adaptive_routing *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_adaptive_routing ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_adaptive_routing_size(void)
{
	 return 64;
}

void connectib_hw_dct_adaptive_routing_dump(const struct connectib_hw_dct_adaptive_routing *ptr_struct, FILE* file)
{
	connectib_hw_dct_adaptive_routing_print(ptr_struct, file, 0);
}

void connectib_hw_dct_tcu_trap_pack(const struct connectib_hw_dct_tcu_trap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 263;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_tcu_trap_unpack(struct connectib_hw_dct_tcu_trap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 263;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_tcu_trap_print(const struct connectib_hw_dct_tcu_trap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_tcu_trap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_tcu_trap_size(void)
{
	 return 64;
}

void connectib_hw_dct_tcu_trap_dump(const struct connectib_hw_dct_tcu_trap *ptr_struct, FILE* file)
{
	connectib_hw_dct_tcu_trap_print(ptr_struct, file, 0);
}

void connectib_hw_dct_tcu_drop_pack(const struct connectib_hw_dct_tcu_drop *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 264;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_tcu_drop_unpack(struct connectib_hw_dct_tcu_drop *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 264;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_tcu_drop_print(const struct connectib_hw_dct_tcu_drop *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_tcu_drop ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_tcu_drop_size(void)
{
	 return 64;
}

void connectib_hw_dct_tcu_drop_dump(const struct connectib_hw_dct_tcu_drop *ptr_struct, FILE* file)
{
	connectib_hw_dct_tcu_drop_print(ptr_struct, file, 0);
}

void connectib_hw_dct_dc_access_key_log_num_byte_pack(const struct connectib_hw_dct_dc_access_key_log_num_byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 265;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_dc_access_key_log_num_byte_unpack(struct connectib_hw_dct_dc_access_key_log_num_byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 265;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_dct_dc_access_key_log_num_byte_print(const struct connectib_hw_dct_dc_access_key_log_num_byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_dc_access_key_log_num_byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_dc_access_key_log_num_byte_size(void)
{
	 return 64;
}

void connectib_hw_dct_dc_access_key_log_num_byte_dump(const struct connectib_hw_dct_dc_access_key_log_num_byte *ptr_struct, FILE* file)
{
	connectib_hw_dct_dc_access_key_log_num_byte_print(ptr_struct, file, 0);
}

void connectib_hw_dct_fast_path_pack(const struct connectib_hw_dct_fast_path *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 268;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_fast_path_unpack(struct connectib_hw_dct_fast_path *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 268;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_fast_path_print(const struct connectib_hw_dct_fast_path *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_fast_path ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_fast_path_size(void)
{
	 return 64;
}

void connectib_hw_dct_fast_path_dump(const struct connectib_hw_dct_fast_path *ptr_struct, FILE* file)
{
	connectib_hw_dct_fast_path_print(ptr_struct, file, 0);
}

void connectib_hw_dct_mtu_pack(const struct connectib_hw_dct_mtu *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 269;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_mtu_unpack(struct connectib_hw_dct_mtu *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 269;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_dct_mtu_print(const struct connectib_hw_dct_mtu *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_mtu ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_mtu_size(void)
{
	 return 64;
}

void connectib_hw_dct_mtu_dump(const struct connectib_hw_dct_mtu *ptr_struct, FILE* file)
{
	connectib_hw_dct_mtu_print(ptr_struct, file, 0);
}

void connectib_hw_dct_read_enable_pack(const struct connectib_hw_dct_read_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_read_enable_unpack(struct connectib_hw_dct_read_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 272;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_read_enable_print(const struct connectib_hw_dct_read_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_read_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_read_enable_size(void)
{
	 return 64;
}

void connectib_hw_dct_read_enable_dump(const struct connectib_hw_dct_read_enable *ptr_struct, FILE* file)
{
	connectib_hw_dct_read_enable_print(ptr_struct, file, 0);
}

void connectib_hw_dct_send_with_invalidate_enable_pack(const struct connectib_hw_dct_send_with_invalidate_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 273;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_send_with_invalidate_enable_unpack(struct connectib_hw_dct_send_with_invalidate_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 273;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_send_with_invalidate_enable_print(const struct connectib_hw_dct_send_with_invalidate_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_send_with_invalidate_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_send_with_invalidate_enable_size(void)
{
	 return 64;
}

void connectib_hw_dct_send_with_invalidate_enable_dump(const struct connectib_hw_dct_send_with_invalidate_enable *ptr_struct, FILE* file)
{
	connectib_hw_dct_send_with_invalidate_enable_print(ptr_struct, file, 0);
}

void connectib_hw_dct_ignore_pkey_check_pack(const struct connectib_hw_dct_ignore_pkey_check *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 274;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_ignore_pkey_check_unpack(struct connectib_hw_dct_ignore_pkey_check *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 274;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_ignore_pkey_check_print(const struct connectib_hw_dct_ignore_pkey_check *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_ignore_pkey_check ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_ignore_pkey_check_size(void)
{
	 return 64;
}

void connectib_hw_dct_ignore_pkey_check_dump(const struct connectib_hw_dct_ignore_pkey_check *ptr_struct, FILE* file)
{
	connectib_hw_dct_ignore_pkey_check_print(ptr_struct, file, 0);
}

void connectib_hw_dct_max_message_size_pack(const struct connectib_hw_dct_max_message_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 275;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_max_message_size_unpack(struct connectib_hw_dct_max_message_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 275;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_dct_max_message_size_print(const struct connectib_hw_dct_max_message_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_max_message_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_max_message_size_size(void)
{
	 return 64;
}

void connectib_hw_dct_max_message_size_dump(const struct connectib_hw_dct_max_message_size *ptr_struct, FILE* file)
{
	connectib_hw_dct_max_message_size_print(ptr_struct, file, 0);
}

void connectib_hw_dct_inline_scatter_pack(const struct connectib_hw_dct_inline_scatter *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 280;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_inline_scatter_unpack(struct connectib_hw_dct_inline_scatter *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 280;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_dct_inline_scatter_print(const struct connectib_hw_dct_inline_scatter *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_inline_scatter ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_inline_scatter_size(void)
{
	 return 64;
}

void connectib_hw_dct_inline_scatter_dump(const struct connectib_hw_dct_inline_scatter *ptr_struct, FILE* file)
{
	connectib_hw_dct_inline_scatter_print(ptr_struct, file, 0);
}

void connectib_hw_dct_write_enable_pack(const struct connectib_hw_dct_write_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 282;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_write_enable_unpack(struct connectib_hw_dct_write_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 282;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_dct_write_enable_print(const struct connectib_hw_dct_write_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_write_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_write_enable_size(void)
{
	 return 64;
}

void connectib_hw_dct_write_enable_dump(const struct connectib_hw_dct_write_enable *ptr_struct, FILE* file)
{
	connectib_hw_dct_write_enable_print(ptr_struct, file, 0);
}

void connectib_hw_dct_rnr_timer_pack(const struct connectib_hw_dct_rnr_timer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 283;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_rnr_timer_unpack(struct connectib_hw_dct_rnr_timer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 283;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_dct_rnr_timer_print(const struct connectib_hw_dct_rnr_timer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_rnr_timer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_rnr_timer_size(void)
{
	 return 64;
}

void connectib_hw_dct_rnr_timer_dump(const struct connectib_hw_dct_rnr_timer *ptr_struct, FILE* file)
{
	connectib_hw_dct_rnr_timer_print(ptr_struct, file, 0);
}

void connectib_hw_dct_complete_to_64B_eyald_pack(const struct connectib_hw_dct_complete_to_64B_eyald *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_dct_complete_to_64B_eyald_unpack(struct connectib_hw_dct_complete_to_64B_eyald *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_dct_complete_to_64B_eyald_print(const struct connectib_hw_dct_complete_to_64B_eyald *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct_complete_to_64B_eyald ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_dct_complete_to_64B_eyald_size(void)
{
	 return 64;
}

void connectib_hw_dct_complete_to_64B_eyald_dump(const struct connectib_hw_dct_complete_to_64B_eyald *ptr_struct, FILE* file)
{
	connectib_hw_dct_complete_to_64B_eyald_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_qp_valid_pack(const struct connectib_hw_r_qpc_qp_valid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_qp_valid_unpack(struct connectib_hw_r_qpc_qp_valid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_qp_valid_print(const struct connectib_hw_r_qpc_qp_valid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_qp_valid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_qp_valid_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_qp_valid_dump(const struct connectib_hw_r_qpc_qp_valid *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_qp_valid_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_grh_to_cqe_pack(const struct connectib_hw_r_qpc_grh_to_cqe *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_grh_to_cqe_unpack(struct connectib_hw_r_qpc_grh_to_cqe *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_grh_to_cqe_print(const struct connectib_hw_r_qpc_grh_to_cqe *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_grh_to_cqe ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_grh_to_cqe_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_grh_to_cqe_dump(const struct connectib_hw_r_qpc_grh_to_cqe *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_grh_to_cqe_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_drop_my_self_multicast_pack(const struct connectib_hw_r_qpc_drop_my_self_multicast *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_drop_my_self_multicast_unpack(struct connectib_hw_r_qpc_drop_my_self_multicast *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_drop_my_self_multicast_print(const struct connectib_hw_r_qpc_drop_my_self_multicast *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_drop_my_self_multicast ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_drop_my_self_multicast_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_drop_my_self_multicast_dump(const struct connectib_hw_r_qpc_drop_my_self_multicast *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_drop_my_self_multicast_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_ack_type_pack(const struct connectib_hw_r_qpc_ack_type *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_ack_type_unpack(struct connectib_hw_r_qpc_ack_type *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_r_qpc_ack_type_print(const struct connectib_hw_r_qpc_ack_type *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_ack_type ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_ack_type_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_ack_type_dump(const struct connectib_hw_r_qpc_ack_type *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_ack_type_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_e2e_credits_enabled_pack(const struct connectib_hw_r_qpc_e2e_credits_enabled *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_e2e_credits_enabled_unpack(struct connectib_hw_r_qpc_e2e_credits_enabled *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 4;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_e2e_credits_enabled_print(const struct connectib_hw_r_qpc_e2e_credits_enabled *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_e2e_credits_enabled ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_e2e_credits_enabled_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_e2e_credits_enabled_dump(const struct connectib_hw_r_qpc_e2e_credits_enabled *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_e2e_credits_enabled_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_no_direct_access_pack(const struct connectib_hw_r_qpc_no_direct_access *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_no_direct_access_unpack(struct connectib_hw_r_qpc_no_direct_access *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_no_direct_access_print(const struct connectib_hw_r_qpc_no_direct_access *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_no_direct_access ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_no_direct_access_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_no_direct_access_dump(const struct connectib_hw_r_qpc_no_direct_access *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_no_direct_access_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_post_receive_slave_enable_pack(const struct connectib_hw_r_qpc_post_receive_slave_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_post_receive_slave_enable_unpack(struct connectib_hw_r_qpc_post_receive_slave_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 6;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_post_receive_slave_enable_print(const struct connectib_hw_r_qpc_post_receive_slave_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_post_receive_slave_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_post_receive_slave_enable_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_post_receive_slave_enable_dump(const struct connectib_hw_r_qpc_post_receive_slave_enable *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_post_receive_slave_enable_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_atomic_like_write_pack(const struct connectib_hw_r_qpc_atomic_like_write *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_atomic_like_write_unpack(struct connectib_hw_r_qpc_atomic_like_write *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_atomic_like_write_print(const struct connectib_hw_r_qpc_atomic_like_write *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_atomic_like_write ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_atomic_like_write_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_atomic_like_write_dump(const struct connectib_hw_r_qpc_atomic_like_write *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_atomic_like_write_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_write_enable_pack(const struct connectib_hw_r_qpc_write_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_write_enable_unpack(struct connectib_hw_r_qpc_write_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_write_enable_print(const struct connectib_hw_r_qpc_write_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_write_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_write_enable_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_write_enable_dump(const struct connectib_hw_r_qpc_write_enable *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_write_enable_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_max_read_atomic_pack(const struct connectib_hw_r_qpc_max_read_atomic *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_max_read_atomic_unpack(struct connectib_hw_r_qpc_max_read_atomic *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 9;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_r_qpc_max_read_atomic_print(const struct connectib_hw_r_qpc_max_read_atomic *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_max_read_atomic ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_max_read_atomic_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_max_read_atomic_dump(const struct connectib_hw_r_qpc_max_read_atomic *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_max_read_atomic_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_extended_atomic_pack(const struct connectib_hw_r_qpc_extended_atomic *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_extended_atomic_unpack(struct connectib_hw_r_qpc_extended_atomic *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_r_qpc_extended_atomic_print(const struct connectib_hw_r_qpc_extended_atomic *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_extended_atomic ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_extended_atomic_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_extended_atomic_dump(const struct connectib_hw_r_qpc_extended_atomic *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_extended_atomic_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_ignore_pkey_check_pack(const struct connectib_hw_r_qpc_ignore_pkey_check *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_ignore_pkey_check_unpack(struct connectib_hw_r_qpc_ignore_pkey_check *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_ignore_pkey_check_print(const struct connectib_hw_r_qpc_ignore_pkey_check *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_ignore_pkey_check ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_ignore_pkey_check_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_ignore_pkey_check_dump(const struct connectib_hw_r_qpc_ignore_pkey_check *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_ignore_pkey_check_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_send_with_invalidate_enable_pack(const struct connectib_hw_r_qpc_send_with_invalidate_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_send_with_invalidate_enable_unpack(struct connectib_hw_r_qpc_send_with_invalidate_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 17;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_send_with_invalidate_enable_print(const struct connectib_hw_r_qpc_send_with_invalidate_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_send_with_invalidate_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_send_with_invalidate_enable_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_send_with_invalidate_enable_dump(const struct connectib_hw_r_qpc_send_with_invalidate_enable *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_send_with_invalidate_enable_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_read_enable_pack(const struct connectib_hw_r_qpc_read_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_read_enable_unpack(struct connectib_hw_r_qpc_read_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 18;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_read_enable_print(const struct connectib_hw_r_qpc_read_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_read_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_read_enable_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_read_enable_dump(const struct connectib_hw_r_qpc_read_enable *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_read_enable_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_fast_path_pack(const struct connectib_hw_r_qpc_fast_path *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_fast_path_unpack(struct connectib_hw_r_qpc_fast_path *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 20;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_fast_path_print(const struct connectib_hw_r_qpc_fast_path *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_fast_path ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_fast_path_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_fast_path_dump(const struct connectib_hw_r_qpc_fast_path *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_fast_path_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_check_source_uc_lb_pack(const struct connectib_hw_r_qpc_check_source_uc_lb *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_check_source_uc_lb_unpack(struct connectib_hw_r_qpc_check_source_uc_lb *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 21;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_check_source_uc_lb_print(const struct connectib_hw_r_qpc_check_source_uc_lb *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_check_source_uc_lb ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_check_source_uc_lb_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_check_source_uc_lb_dump(const struct connectib_hw_r_qpc_check_source_uc_lb *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_check_source_uc_lb_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_check_source_mc_lb_pack(const struct connectib_hw_r_qpc_check_source_mc_lb *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_check_source_mc_lb_unpack(struct connectib_hw_r_qpc_check_source_mc_lb *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_check_source_mc_lb_print(const struct connectib_hw_r_qpc_check_source_mc_lb *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_check_source_mc_lb ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_check_source_mc_lb_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_check_source_mc_lb_dump(const struct connectib_hw_r_qpc_check_source_mc_lb *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_check_source_mc_lb_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_check_source_gvmi_pack(const struct connectib_hw_r_qpc_check_source_gvmi *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_check_source_gvmi_unpack(struct connectib_hw_r_qpc_check_source_gvmi *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 23;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_check_source_gvmi_print(const struct connectib_hw_r_qpc_check_source_gvmi *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_check_source_gvmi ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_check_source_gvmi_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_check_source_gvmi_dump(const struct connectib_hw_r_qpc_check_source_gvmi *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_check_source_gvmi_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_inline_scatter_pack(const struct connectib_hw_r_qpc_inline_scatter *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_inline_scatter_unpack(struct connectib_hw_r_qpc_inline_scatter *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_r_qpc_inline_scatter_print(const struct connectib_hw_r_qpc_inline_scatter *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_inline_scatter ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_inline_scatter_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_inline_scatter_dump(const struct connectib_hw_r_qpc_inline_scatter *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_inline_scatter_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rwq_type_pack(const struct connectib_hw_r_qpc_rwq_type *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rwq_type_unpack(struct connectib_hw_r_qpc_rwq_type *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_r_qpc_rwq_type_print(const struct connectib_hw_r_qpc_rwq_type *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rwq_type ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rwq_type_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rwq_type_dump(const struct connectib_hw_r_qpc_rwq_type *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rwq_type_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_last_opcode_pack(const struct connectib_hw_r_qpc_last_opcode *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_last_opcode_unpack(struct connectib_hw_r_qpc_last_opcode *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_last_opcode_print(const struct connectib_hw_r_qpc_last_opcode *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_last_opcode ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_last_opcode_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_last_opcode_dump(const struct connectib_hw_r_qpc_last_opcode *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_last_opcode_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_fw_link_type_pack(const struct connectib_hw_r_qpc_fw_link_type *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_fw_link_type_unpack(struct connectib_hw_r_qpc_fw_link_type *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_fw_link_type_print(const struct connectib_hw_r_qpc_fw_link_type *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_fw_link_type ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_fw_link_type_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_fw_link_type_dump(const struct connectib_hw_r_qpc_fw_link_type *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_fw_link_type_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_check_source_uc_wire_pack(const struct connectib_hw_r_qpc_check_source_uc_wire *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_check_source_uc_wire_unpack(struct connectib_hw_r_qpc_check_source_uc_wire *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_check_source_uc_wire_print(const struct connectib_hw_r_qpc_check_source_uc_wire *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_check_source_uc_wire ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_check_source_uc_wire_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_check_source_uc_wire_dump(const struct connectib_hw_r_qpc_check_source_uc_wire *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_check_source_uc_wire_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_write_rkey_pack(const struct connectib_hw_r_qpc_write_rkey *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_write_rkey_unpack(struct connectib_hw_r_qpc_write_rkey *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_r_qpc_write_rkey_print(const struct connectib_hw_r_qpc_write_rkey *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_write_rkey ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_write_rkey_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_write_rkey_dump(const struct connectib_hw_r_qpc_write_rkey *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_write_rkey_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rx_heartbeat_pack(const struct connectib_hw_r_qpc_rx_heartbeat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rx_heartbeat_unpack(struct connectib_hw_r_qpc_rx_heartbeat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_rx_heartbeat_print(const struct connectib_hw_r_qpc_rx_heartbeat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rx_heartbeat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rx_heartbeat_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rx_heartbeat_dump(const struct connectib_hw_r_qpc_rx_heartbeat *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rx_heartbeat_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_e2e_credits_prefetch_threshold_pack(const struct connectib_hw_r_qpc_e2e_credits_prefetch_threshold *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 65;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_e2e_credits_prefetch_threshold_unpack(struct connectib_hw_r_qpc_e2e_credits_prefetch_threshold *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 65;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_r_qpc_e2e_credits_prefetch_threshold_print(const struct connectib_hw_r_qpc_e2e_credits_prefetch_threshold *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_e2e_credits_prefetch_threshold ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_e2e_credits_prefetch_threshold_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_e2e_credits_prefetch_threshold_dump(const struct connectib_hw_r_qpc_e2e_credits_prefetch_threshold *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_e2e_credits_prefetch_threshold_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_e2e_fetch_enable_pack(const struct connectib_hw_r_qpc_e2e_fetch_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 68;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_e2e_fetch_enable_unpack(struct connectib_hw_r_qpc_e2e_fetch_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 68;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_e2e_fetch_enable_print(const struct connectib_hw_r_qpc_e2e_fetch_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_e2e_fetch_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_e2e_fetch_enable_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_e2e_fetch_enable_dump(const struct connectib_hw_r_qpc_e2e_fetch_enable *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_e2e_fetch_enable_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_tcu_drop_pack(const struct connectib_hw_r_qpc_tcu_drop *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 69;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_tcu_drop_unpack(struct connectib_hw_r_qpc_tcu_drop *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 69;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_tcu_drop_print(const struct connectib_hw_r_qpc_tcu_drop *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_tcu_drop ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_tcu_drop_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_tcu_drop_dump(const struct connectib_hw_r_qpc_tcu_drop *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_tcu_drop_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_can_use_reserved_lkey_pack(const struct connectib_hw_r_qpc_can_use_reserved_lkey *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 70;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_can_use_reserved_lkey_unpack(struct connectib_hw_r_qpc_can_use_reserved_lkey *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 70;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_can_use_reserved_lkey_print(const struct connectib_hw_r_qpc_can_use_reserved_lkey *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_can_use_reserved_lkey ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_can_use_reserved_lkey_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_can_use_reserved_lkey_dump(const struct connectib_hw_r_qpc_can_use_reserved_lkey *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_can_use_reserved_lkey_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_tcu_trap_pack(const struct connectib_hw_r_qpc_tcu_trap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 71;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_tcu_trap_unpack(struct connectib_hw_r_qpc_tcu_trap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 71;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_tcu_trap_print(const struct connectib_hw_r_qpc_tcu_trap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_tcu_trap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_tcu_trap_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_tcu_trap_dump(const struct connectib_hw_r_qpc_tcu_trap *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_tcu_trap_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rnr_timer_pack(const struct connectib_hw_r_qpc_rnr_timer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 75;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rnr_timer_unpack(struct connectib_hw_r_qpc_rnr_timer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 75;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_r_qpc_rnr_timer_print(const struct connectib_hw_r_qpc_rnr_timer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rnr_timer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rnr_timer_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rnr_timer_dump(const struct connectib_hw_r_qpc_rnr_timer *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rnr_timer_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rwqe_limit_event_pack(const struct connectib_hw_r_qpc_rwqe_limit_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rwqe_limit_event_unpack(struct connectib_hw_r_qpc_rwqe_limit_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 80;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_r_qpc_rwqe_limit_event_print(const struct connectib_hw_r_qpc_rwqe_limit_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rwqe_limit_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rwqe_limit_event_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rwqe_limit_event_dump(const struct connectib_hw_r_qpc_rwqe_limit_event *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rwqe_limit_event_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rwqe_consumer_index_pack(const struct connectib_hw_r_qpc_rwqe_consumer_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rwqe_consumer_index_unpack(struct connectib_hw_r_qpc_rwqe_consumer_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_r_qpc_rwqe_consumer_index_print(const struct connectib_hw_r_qpc_rwqe_consumer_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rwqe_consumer_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rwqe_consumer_index_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rwqe_consumer_index_dump(const struct connectib_hw_r_qpc_rwqe_consumer_index *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rwqe_consumer_index_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rcyclic_producer_pack(const struct connectib_hw_r_qpc_rcyclic_producer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rcyclic_producer_unpack(struct connectib_hw_r_qpc_rcyclic_producer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_r_qpc_rcyclic_producer_print(const struct connectib_hw_r_qpc_rcyclic_producer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rcyclic_producer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rcyclic_producer_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rcyclic_producer_dump(const struct connectib_hw_r_qpc_rcyclic_producer *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rcyclic_producer_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rwqe_producer_index_pack(const struct connectib_hw_r_qpc_rwqe_producer_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rwqe_producer_index_unpack(struct connectib_hw_r_qpc_rwqe_producer_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 112;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_r_qpc_rwqe_producer_index_print(const struct connectib_hw_r_qpc_rwqe_producer_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rwqe_producer_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rwqe_producer_index_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rwqe_producer_index_dump(const struct connectib_hw_r_qpc_rwqe_producer_index *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rwqe_producer_index_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_qp_state_pack(const struct connectib_hw_r_qpc_qp_state *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_qp_state_unpack(struct connectib_hw_r_qpc_qp_state *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 144;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_r_qpc_qp_state_print(const struct connectib_hw_r_qpc_qp_state *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_qp_state ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_qp_state_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_qp_state_dump(const struct connectib_hw_r_qpc_qp_state *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_qp_state_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rdb_write_pointer_pack(const struct connectib_hw_r_qpc_rdb_write_pointer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rdb_write_pointer_unpack(struct connectib_hw_r_qpc_rdb_write_pointer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_r_qpc_rdb_write_pointer_print(const struct connectib_hw_r_qpc_rdb_write_pointer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rdb_write_pointer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rdb_write_pointer_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rdb_write_pointer_dump(const struct connectib_hw_r_qpc_rdb_write_pointer *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rdb_write_pointer_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_first_non_acked_psn_pack(const struct connectib_hw_r_qpc_first_non_acked_psn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_first_non_acked_psn_unpack(struct connectib_hw_r_qpc_first_non_acked_psn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 168;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_first_non_acked_psn_print(const struct connectib_hw_r_qpc_first_non_acked_psn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_first_non_acked_psn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_first_non_acked_psn_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_first_non_acked_psn_dump(const struct connectib_hw_r_qpc_first_non_acked_psn *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_first_non_acked_psn_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rcyclic_consumer_pack(const struct connectib_hw_r_qpc_rcyclic_consumer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rcyclic_consumer_unpack(struct connectib_hw_r_qpc_rcyclic_consumer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 192;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_r_qpc_rcyclic_consumer_print(const struct connectib_hw_r_qpc_rcyclic_consumer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rcyclic_consumer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rcyclic_consumer_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rcyclic_consumer_dump(const struct connectib_hw_r_qpc_rcyclic_consumer *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rcyclic_consumer_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_msn_pack(const struct connectib_hw_r_qpc_msn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_msn_unpack(struct connectib_hw_r_qpc_msn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_msn_print(const struct connectib_hw_r_qpc_msn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_msn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_msn_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_msn_dump(const struct connectib_hw_r_qpc_msn *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_msn_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_dct_number_pack(const struct connectib_hw_r_qpc_dct_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_dct_number_unpack(struct connectib_hw_r_qpc_dct_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 232;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_dct_number_print(const struct connectib_hw_r_qpc_dct_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_dct_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_dct_number_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_dct_number_dump(const struct connectib_hw_r_qpc_dct_number *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_dct_number_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_cq_number_pack(const struct connectib_hw_r_qpc_cq_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_cq_number_unpack(struct connectib_hw_r_qpc_cq_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 232;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_cq_number_print(const struct connectib_hw_r_qpc_cq_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_cq_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_cq_number_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_cq_number_dump(const struct connectib_hw_r_qpc_cq_number *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_cq_number_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_expected_psn_pack(const struct connectib_hw_r_qpc_expected_psn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_expected_psn_unpack(struct connectib_hw_r_qpc_expected_psn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 256;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_expected_psn_print(const struct connectib_hw_r_qpc_expected_psn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_expected_psn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_expected_psn_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_expected_psn_dump(const struct connectib_hw_r_qpc_expected_psn *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_expected_psn_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_flush_syndrom_pack(const struct connectib_hw_r_qpc_flush_syndrom *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 280;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_flush_syndrom_unpack(struct connectib_hw_r_qpc_flush_syndrom *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 280;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_r_qpc_flush_syndrom_print(const struct connectib_hw_r_qpc_flush_syndrom *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_flush_syndrom ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_flush_syndrom_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_flush_syndrom_dump(const struct connectib_hw_r_qpc_flush_syndrom *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_flush_syndrom_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_counter_index_pack(const struct connectib_hw_r_qpc_counter_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_counter_index_unpack(struct connectib_hw_r_qpc_counter_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 288;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_r_qpc_counter_index_print(const struct connectib_hw_r_qpc_counter_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_counter_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_counter_index_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_counter_index_dump(const struct connectib_hw_r_qpc_counter_index *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_counter_index_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_always_zero_pack(const struct connectib_hw_r_qpc_always_zero *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 296;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_always_zero_unpack(struct connectib_hw_r_qpc_always_zero *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 296;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_always_zero_print(const struct connectib_hw_r_qpc_always_zero *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_always_zero ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_always_zero_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_always_zero_dump(const struct connectib_hw_r_qpc_always_zero *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_always_zero_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rdb_read_pointer_pack(const struct connectib_hw_r_qpc_rdb_read_pointer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 312;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rdb_read_pointer_unpack(struct connectib_hw_r_qpc_rdb_read_pointer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 312;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_r_qpc_rdb_read_pointer_print(const struct connectib_hw_r_qpc_rdb_read_pointer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rdb_read_pointer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rdb_read_pointer_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rdb_read_pointer_dump(const struct connectib_hw_r_qpc_rdb_read_pointer *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rdb_read_pointer_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_rx_length_committed_pack(const struct connectib_hw_r_qpc_rx_length_committed *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 320;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_rx_length_committed_unpack(struct connectib_hw_r_qpc_rx_length_committed *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 320;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_rx_length_committed_print(const struct connectib_hw_r_qpc_rx_length_committed *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_rx_length_committed ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_rx_length_committed_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_rx_length_committed_dump(const struct connectib_hw_r_qpc_rx_length_committed *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_rx_length_committed_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_ts_pack(const struct connectib_hw_r_qpc_ts *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_ts_unpack(struct connectib_hw_r_qpc_ts *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_r_qpc_ts_print(const struct connectib_hw_r_qpc_ts *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_ts ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_ts_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_ts_dump(const struct connectib_hw_r_qpc_ts *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_ts_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_sx_heartbeat_pack(const struct connectib_hw_r_qpc_sx_heartbeat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 356;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_sx_heartbeat_unpack(struct connectib_hw_r_qpc_sx_heartbeat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 356;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_sx_heartbeat_print(const struct connectib_hw_r_qpc_sx_heartbeat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_sx_heartbeat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_sx_heartbeat_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_sx_heartbeat_dump(const struct connectib_hw_r_qpc_sx_heartbeat *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_sx_heartbeat_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_sqn_pack(const struct connectib_hw_r_qpc_sqn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 357;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_sqn_unpack(struct connectib_hw_r_qpc_sqn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 357;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

}

void connectib_hw_r_qpc_sqn_print(const struct connectib_hw_r_qpc_sqn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_sqn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_sqn_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_sqn_dump(const struct connectib_hw_r_qpc_sqn *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_sqn_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_dct_gvmi_pack(const struct connectib_hw_r_qpc_dct_gvmi *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 368;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_dct_gvmi_unpack(struct connectib_hw_r_qpc_dct_gvmi *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 368;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_r_qpc_dct_gvmi_print(const struct connectib_hw_r_qpc_dct_gvmi *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_dct_gvmi ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_dct_gvmi_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_dct_gvmi_dump(const struct connectib_hw_r_qpc_dct_gvmi *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_dct_gvmi_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_no_wqe_drop_count_pack(const struct connectib_hw_r_qpc_no_wqe_drop_count *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 368;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_no_wqe_drop_count_unpack(struct connectib_hw_r_qpc_no_wqe_drop_count *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 368;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_r_qpc_no_wqe_drop_count_print(const struct connectib_hw_r_qpc_no_wqe_drop_count *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_no_wqe_drop_count ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_no_wqe_drop_count_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_no_wqe_drop_count_dump(const struct connectib_hw_r_qpc_no_wqe_drop_count *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_no_wqe_drop_count_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_write_va_63_32_pack(const struct connectib_hw_r_qpc_write_va_63_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_write_va_63_32_unpack(struct connectib_hw_r_qpc_write_va_63_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_r_qpc_write_va_63_32_print(const struct connectib_hw_r_qpc_write_va_63_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_write_va_63_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_write_va_63_32_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_write_va_63_32_dump(const struct connectib_hw_r_qpc_write_va_63_32 *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_write_va_63_32_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_write_va_31_0_pack(const struct connectib_hw_r_qpc_write_va_31_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_write_va_31_0_unpack(struct connectib_hw_r_qpc_write_va_31_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_r_qpc_write_va_31_0_print(const struct connectib_hw_r_qpc_write_va_31_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_write_va_31_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_write_va_31_0_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_write_va_31_0_dump(const struct connectib_hw_r_qpc_write_va_31_0 *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_write_va_31_0_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_srq_number_pack(const struct connectib_hw_r_qpc_srq_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 456;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_srq_number_unpack(struct connectib_hw_r_qpc_srq_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 456;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_srq_number_print(const struct connectib_hw_r_qpc_srq_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_srq_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_srq_number_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_srq_number_dump(const struct connectib_hw_r_qpc_srq_number *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_srq_number_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_next_dcr_pack(const struct connectib_hw_r_qpc_next_dcr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 456;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_next_dcr_unpack(struct connectib_hw_r_qpc_next_dcr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 456;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_next_dcr_print(const struct connectib_hw_r_qpc_next_dcr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_next_dcr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_next_dcr_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_next_dcr_dump(const struct connectib_hw_r_qpc_next_dcr *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_next_dcr_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_mpi_uar_pack(const struct connectib_hw_r_qpc_mpi_uar *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_mpi_uar_unpack(struct connectib_hw_r_qpc_mpi_uar *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_mpi_uar_print(const struct connectib_hw_r_qpc_mpi_uar *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_mpi_uar ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_mpi_uar_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_mpi_uar_dump(const struct connectib_hw_r_qpc_mpi_uar *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_mpi_uar_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_responder_session_id_pack(const struct connectib_hw_r_qpc_responder_session_id *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_responder_session_id_unpack(struct connectib_hw_r_qpc_responder_session_id *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_responder_session_id_print(const struct connectib_hw_r_qpc_responder_session_id *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_responder_session_id ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_responder_session_id_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_responder_session_id_dump(const struct connectib_hw_r_qpc_responder_session_id *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_responder_session_id_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_xrc_domain_pack(const struct connectib_hw_r_qpc_xrc_domain *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_xrc_domain_unpack(struct connectib_hw_r_qpc_xrc_domain *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_r_qpc_xrc_domain_print(const struct connectib_hw_r_qpc_xrc_domain *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_xrc_domain ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_xrc_domain_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_xrc_domain_dump(const struct connectib_hw_r_qpc_xrc_domain *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_xrc_domain_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_dcr_connected_pack(const struct connectib_hw_r_qpc_dcr_connected *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 509;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_dcr_connected_unpack(struct connectib_hw_r_qpc_dcr_connected *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 509;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_dcr_connected_print(const struct connectib_hw_r_qpc_dcr_connected *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_dcr_connected ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_dcr_connected_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_dcr_connected_dump(const struct connectib_hw_r_qpc_dcr_connected *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_dcr_connected_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_e2e_pa_63_32_pack(const struct connectib_hw_r_qpc_e2e_pa_63_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_e2e_pa_63_32_unpack(struct connectib_hw_r_qpc_e2e_pa_63_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_r_qpc_e2e_pa_63_32_print(const struct connectib_hw_r_qpc_e2e_pa_63_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_e2e_pa_63_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_e2e_pa_63_32_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_e2e_pa_63_32_dump(const struct connectib_hw_r_qpc_e2e_pa_63_32 *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_e2e_pa_63_32_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_e2e_pa_31_2_pack(const struct connectib_hw_r_qpc_e2e_pa_31_2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 30, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_e2e_pa_31_2_unpack(struct connectib_hw_r_qpc_e2e_pa_31_2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 30);

}

void connectib_hw_r_qpc_e2e_pa_31_2_print(const struct connectib_hw_r_qpc_e2e_pa_31_2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_e2e_pa_31_2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_e2e_pa_31_2_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_e2e_pa_31_2_dump(const struct connectib_hw_r_qpc_e2e_pa_31_2 *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_e2e_pa_31_2_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_check_source_mc_wire_pack(const struct connectib_hw_r_qpc_check_source_mc_wire *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 510;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_check_source_mc_wire_unpack(struct connectib_hw_r_qpc_check_source_mc_wire *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 510;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_r_qpc_check_source_mc_wire_print(const struct connectib_hw_r_qpc_check_source_mc_wire *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_check_source_mc_wire ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_check_source_mc_wire_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_check_source_mc_wire_dump(const struct connectib_hw_r_qpc_check_source_mc_wire *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_check_source_mc_wire_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_connection_state_pack(const struct connectib_hw_r_qpc_connection_state *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 510;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_r_qpc_connection_state_unpack(struct connectib_hw_r_qpc_connection_state *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 510;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_r_qpc_connection_state_print(const struct connectib_hw_r_qpc_connection_state *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc_connection_state ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_r_qpc_connection_state_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_connection_state_dump(const struct connectib_hw_r_qpc_connection_state *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_connection_state_print(ptr_struct, file, 0);
}

void connectib_pf_vf_vpd_pack(const struct connectib_pf_vf_vpd *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 15, (u_int32_t)ptr_struct->vpd_address);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->F);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vpd_data);

}

void connectib_pf_vf_vpd_unpack(struct connectib_pf_vf_vpd *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->vpd_address = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 15);

	offset = 0;
	ptr_struct->F = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->vpd_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_pf_vf_vpd_print(const struct connectib_pf_vf_vpd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pf_vf_vpd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_address          : " UH_FMT "\n", ptr_struct->vpd_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "F                    : " UH_FMT "\n", ptr_struct->F);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_data             : " U32H_FMT "\n", ptr_struct->vpd_data);

}

int connectib_pf_vf_vpd_size(void)
{
	 return 8;
}

void connectib_pf_vf_vpd_dump(const struct connectib_pf_vf_vpd *ptr_struct, FILE* file)
{
	connectib_pf_vf_vpd_print(ptr_struct, file, 0);
}

void connectib_pf_vf_sriov_pack(const struct connectib_pf_vf_sriov *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_enable);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_migration_en);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_migration_int_en);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_mse);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ari_capable_hierarchy);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_system_page_size);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->num_vfs);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->not_in_use);

}

void connectib_pf_vf_sriov_unpack(struct connectib_pf_vf_sriov *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->vf_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->vf_migration_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->vf_migration_int_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->vf_mse = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->ari_capable_hierarchy = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->log_system_page_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 6;
	ptr_struct->num_vfs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 32;
	ptr_struct->not_in_use = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_pf_vf_sriov_print(const struct connectib_pf_vf_sriov *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pf_vf_sriov ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_enable            : " UH_FMT "\n", ptr_struct->vf_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_migration_en      : " UH_FMT "\n", ptr_struct->vf_migration_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_migration_int_en  : " UH_FMT "\n", ptr_struct->vf_migration_int_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_mse               : " UH_FMT "\n", ptr_struct->vf_mse);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ari_capable_hierarchy : " UH_FMT "\n", ptr_struct->ari_capable_hierarchy);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_system_page_size : " UH_FMT "\n", ptr_struct->log_system_page_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_vfs              : " UH_FMT "\n", ptr_struct->num_vfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "not_in_use           : " U32H_FMT "\n", ptr_struct->not_in_use);

}

int connectib_pf_vf_sriov_size(void)
{
	 return 8;
}

void connectib_pf_vf_sriov_dump(const struct connectib_pf_vf_sriov *ptr_struct, FILE* file)
{
	connectib_pf_vf_sriov_print(ptr_struct, file, 0);
}

void connectib_serdes_conf_preset_tx_ini_pack(const struct connectib_serdes_conf_preset_tx_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	connectib_serdes_conf_tx_set_ini_pack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_preset_tx_ini_unpack(struct connectib_serdes_conf_preset_tx_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	connectib_serdes_conf_tx_set_ini_unpack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_preset_tx_ini_print(const struct connectib_serdes_conf_preset_tx_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_conf_preset_tx_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_%03d:\n", i);
	connectib_serdes_conf_tx_set_ini_print(&(ptr_struct->set[i]), file, indent_level + 1);
	}

}

int connectib_serdes_conf_preset_tx_ini_size(void)
{
	 return 64;
}

void connectib_serdes_conf_preset_tx_ini_dump(const struct connectib_serdes_conf_preset_tx_ini *ptr_struct, FILE* file)
{
	connectib_serdes_conf_preset_tx_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_active_tx_ini_pack(const struct connectib_serdes_active_tx_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	connectib_serdes_conf_tx_set_ini_pack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_active_tx_ini_unpack(struct connectib_serdes_active_tx_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	connectib_serdes_conf_tx_set_ini_unpack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_active_tx_ini_print(const struct connectib_serdes_active_tx_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_active_tx_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_%03d:\n", i);
	connectib_serdes_conf_tx_set_ini_print(&(ptr_struct->set[i]), file, indent_level + 1);
	}

}

int connectib_serdes_active_tx_ini_size(void)
{
	 return 16;
}

void connectib_serdes_active_tx_ini_dump(const struct connectib_serdes_active_tx_ini *ptr_struct, FILE* file)
{
	connectib_serdes_active_tx_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_active_rx_ini_pack(const struct connectib_serdes_active_rx_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 256, 1);
	connectib_serdes_conf_rx_set_ini_pack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_active_rx_ini_unpack(struct connectib_serdes_active_rx_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 256, 1);
	connectib_serdes_conf_rx_set_ini_unpack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_active_rx_ini_print(const struct connectib_serdes_active_rx_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_active_rx_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_%03d:\n", i);
	connectib_serdes_conf_rx_set_ini_print(&(ptr_struct->set[i]), file, indent_level + 1);
	}

}

int connectib_serdes_active_rx_ini_size(void)
{
	 return 32;
}

void connectib_serdes_active_rx_ini_dump(const struct connectib_serdes_active_rx_ini *ptr_struct, FILE* file)
{
	connectib_serdes_active_rx_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_force_tx_ini_pack(const struct connectib_serdes_force_tx_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 10; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 320, 1);
	connectib_serdes_conf_tx_set_ini_pack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_force_tx_ini_unpack(struct connectib_serdes_force_tx_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 10; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 320, 1);
	connectib_serdes_conf_tx_set_ini_unpack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_force_tx_ini_print(const struct connectib_serdes_force_tx_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_force_tx_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 10; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_%03d:\n", i);
	connectib_serdes_conf_tx_set_ini_print(&(ptr_struct->set[i]), file, indent_level + 1);
	}

}

int connectib_serdes_force_tx_ini_size(void)
{
	 return 40;
}

void connectib_serdes_force_tx_ini_dump(const struct connectib_serdes_force_tx_ini *ptr_struct, FILE* file)
{
	connectib_serdes_force_tx_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_force_rx_ini_pack(const struct connectib_serdes_force_rx_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 10; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 640, 1);
	connectib_serdes_conf_rx_set_ini_pack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_force_rx_ini_unpack(struct connectib_serdes_force_rx_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 10; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 640, 1);
	connectib_serdes_conf_rx_set_ini_unpack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_force_rx_ini_print(const struct connectib_serdes_force_rx_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_force_rx_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 10; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_%03d:\n", i);
	connectib_serdes_conf_rx_set_ini_print(&(ptr_struct->set[i]), file, indent_level + 1);
	}

}

int connectib_serdes_force_rx_ini_size(void)
{
	 return 80;
}

void connectib_serdes_force_rx_ini_dump(const struct connectib_serdes_force_rx_ini *ptr_struct, FILE* file)
{
	connectib_serdes_force_rx_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_conf_aba_tx_ini_pack(const struct connectib_serdes_conf_aba_tx_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	connectib_serdes_conf_tx_set_ini_pack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_aba_tx_ini_unpack(struct connectib_serdes_conf_aba_tx_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	connectib_serdes_conf_tx_set_ini_unpack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_aba_tx_ini_print(const struct connectib_serdes_conf_aba_tx_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_conf_aba_tx_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_%03d:\n", i);
	connectib_serdes_conf_tx_set_ini_print(&(ptr_struct->set[i]), file, indent_level + 1);
	}

}

int connectib_serdes_conf_aba_tx_ini_size(void)
{
	 return 128;
}

void connectib_serdes_conf_aba_tx_ini_dump(const struct connectib_serdes_conf_aba_tx_ini *ptr_struct, FILE* file)
{
	connectib_serdes_conf_aba_tx_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_conf_aba_rx_ini_pack(const struct connectib_serdes_conf_aba_rx_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 2048, 1);
	connectib_serdes_conf_rx_set_ini_pack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_aba_rx_ini_unpack(struct connectib_serdes_conf_aba_rx_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 2048, 1);
	connectib_serdes_conf_rx_set_ini_unpack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_aba_rx_ini_print(const struct connectib_serdes_conf_aba_rx_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_conf_aba_rx_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_%03d:\n", i);
	connectib_serdes_conf_rx_set_ini_print(&(ptr_struct->set[i]), file, indent_level + 1);
	}

}

int connectib_serdes_conf_aba_rx_ini_size(void)
{
	 return 256;
}

void connectib_serdes_conf_aba_rx_ini_dump(const struct connectib_serdes_conf_aba_rx_ini *ptr_struct, FILE* file)
{
	connectib_serdes_conf_aba_rx_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_conf_autoneg_tx_ini_pack(const struct connectib_serdes_conf_autoneg_tx_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	connectib_serdes_conf_tx_set_ini_pack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_autoneg_tx_ini_unpack(struct connectib_serdes_conf_autoneg_tx_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	connectib_serdes_conf_tx_set_ini_unpack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_autoneg_tx_ini_print(const struct connectib_serdes_conf_autoneg_tx_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_conf_autoneg_tx_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_%03d:\n", i);
	connectib_serdes_conf_tx_set_ini_print(&(ptr_struct->set[i]), file, indent_level + 1);
	}

}

int connectib_serdes_conf_autoneg_tx_ini_size(void)
{
	 return 128;
}

void connectib_serdes_conf_autoneg_tx_ini_dump(const struct connectib_serdes_conf_autoneg_tx_ini *ptr_struct, FILE* file)
{
	connectib_serdes_conf_autoneg_tx_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_conf_autoneg_rx_ini_pack(const struct connectib_serdes_conf_autoneg_rx_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 2048, 1);
	connectib_serdes_conf_rx_set_ini_pack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_autoneg_rx_ini_unpack(struct connectib_serdes_conf_autoneg_rx_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 2048, 1);
	connectib_serdes_conf_rx_set_ini_unpack(&(ptr_struct->set[i]), ptr_buff + offset/8);
	}

}

void connectib_serdes_conf_autoneg_rx_ini_print(const struct connectib_serdes_conf_autoneg_rx_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_conf_autoneg_rx_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_%03d:\n", i);
	connectib_serdes_conf_rx_set_ini_print(&(ptr_struct->set[i]), file, indent_level + 1);
	}

}

int connectib_serdes_conf_autoneg_rx_ini_size(void)
{
	 return 256;
}

void connectib_serdes_conf_autoneg_rx_ini_dump(const struct connectib_serdes_conf_autoneg_rx_ini *ptr_struct, FILE* file)
{
	connectib_serdes_conf_autoneg_rx_ini_print(ptr_struct, file, 0);
}

void connectib_aba_indexes_pack(const struct connectib_aba_indexes *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_aba_index_pack(&(ptr_struct->encode_64_66), ptr_buff + offset/8);

	offset = 128;
	connectib_aba_index_pack(&(ptr_struct->encode_8_10), ptr_buff + offset/8);

	offset = 256;
	connectib_aba_index_pack(&(ptr_struct->encode_8_10_legacy), ptr_buff + offset/8);

}

void connectib_aba_indexes_unpack(struct connectib_aba_indexes *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_aba_index_unpack(&(ptr_struct->encode_64_66), ptr_buff + offset/8);

	offset = 128;
	connectib_aba_index_unpack(&(ptr_struct->encode_8_10), ptr_buff + offset/8);

	offset = 256;
	connectib_aba_index_unpack(&(ptr_struct->encode_8_10_legacy), ptr_buff + offset/8);

}

void connectib_aba_indexes_print(const struct connectib_aba_indexes *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_aba_indexes ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encode_64_66:\n");
	connectib_aba_index_print(&(ptr_struct->encode_64_66), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encode_8_10:\n");
	connectib_aba_index_print(&(ptr_struct->encode_8_10), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encode_8_10_legacy:\n");
	connectib_aba_index_print(&(ptr_struct->encode_8_10_legacy), file, indent_level + 1);

}

int connectib_aba_indexes_size(void)
{
	 return 48;
}

void connectib_aba_indexes_dump(const struct connectib_aba_indexes *ptr_struct, FILE* file)
{
	connectib_aba_indexes_print(ptr_struct, file, 0);
}

void connectib_serdes_param_mask_per_proto_ini_pack(const struct connectib_serdes_param_mask_per_proto_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx);

}

void connectib_serdes_param_mask_per_proto_ini_unpack(struct connectib_serdes_param_mask_per_proto_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->rx = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_serdes_param_mask_per_proto_ini_print(const struct connectib_serdes_param_mask_per_proto_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_param_mask_per_proto_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx                   : " U32H_FMT "\n", ptr_struct->rx);

}

int connectib_serdes_param_mask_per_proto_ini_size(void)
{
	 return 4;
}

void connectib_serdes_param_mask_per_proto_ini_dump(const struct connectib_serdes_param_mask_per_proto_ini *ptr_struct, FILE* file)
{
	connectib_serdes_param_mask_per_proto_ini_print(ptr_struct, file, 0);
}

void connectib_pf_array_default_ones_pack(const struct connectib_pf_array_default_ones *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(31, 1, i, 64, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ix[i]);
	}

}

void connectib_pf_array_default_ones_unpack(struct connectib_pf_array_default_ones *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(31, 1, i, 64, 1);
	ptr_struct->ix[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);
	}

}

void connectib_pf_array_default_ones_print(const struct connectib_pf_array_default_ones *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pf_array_default_ones ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ix_%03d              : " UH_FMT "\n", i, ptr_struct->ix[i]);
	}

}

int connectib_pf_array_default_ones_size(void)
{
	 return 8;
}

void connectib_pf_array_default_ones_dump(const struct connectib_pf_array_default_ones *ptr_struct, FILE* file)
{
	connectib_pf_array_default_ones_print(ptr_struct, file, 0);
}

void connectib_nv_ets_tcN_config_reg_pack(const struct connectib_nv_ets_tcN_config_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->bw_allocation);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->group);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->max_bw_value);

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_bw_units);

}

void connectib_nv_ets_tcN_config_reg_unpack(struct connectib_nv_ets_tcN_config_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	ptr_struct->bw_allocation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 12;
	ptr_struct->group = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 56;
	ptr_struct->max_bw_value = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 44;
	ptr_struct->max_bw_units = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_nv_ets_tcN_config_reg_print(const struct connectib_nv_ets_tcN_config_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_ets_tcN_config_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bw_allocation        : " UH_FMT "\n", ptr_struct->bw_allocation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "group                : " UH_FMT "\n", ptr_struct->group);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_bw_value         : " UH_FMT "\n", ptr_struct->max_bw_value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_bw_units         : %s (" UH_FMT ")\n", (ptr_struct->max_bw_units == 4 ? ("GBPS") : ("unknown")), ptr_struct->max_bw_units);

}

int connectib_nv_ets_tcN_config_reg_size(void)
{
	 return 8;
}

void connectib_nv_ets_tcN_config_reg_dump(const struct connectib_nv_ets_tcN_config_reg *ptr_struct, FILE* file)
{
	connectib_nv_ets_tcN_config_reg_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_fpga_cap_pack(const struct connectib_cmdif_ctx_fpga_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->state);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mailbox_write_offset);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mailbox_write_size);

}

void connectib_cmdif_ctx_fpga_cap_unpack(struct connectib_cmdif_ctx_fpga_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->state = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->mailbox_write_offset = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->mailbox_write_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_fpga_cap_print(const struct connectib_cmdif_ctx_fpga_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_fpga_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "state                : " U32H_FMT "\n", ptr_struct->state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mailbox_write_offset : " U32H_FMT "\n", ptr_struct->mailbox_write_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mailbox_write_size   : " U32H_FMT "\n", ptr_struct->mailbox_write_size);

}

int connectib_cmdif_ctx_fpga_cap_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_fpga_cap_dump(const struct connectib_cmdif_ctx_fpga_cap *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_fpga_cap_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_fpga_access_reg_pack(const struct connectib_cmdif_ctx_fpga_access_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->state);

}

void connectib_cmdif_ctx_fpga_access_reg_unpack(struct connectib_cmdif_ctx_fpga_access_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->state = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_fpga_access_reg_print(const struct connectib_cmdif_ctx_fpga_access_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_fpga_access_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "state                : " U32H_FMT "\n", ptr_struct->state);

}

int connectib_cmdif_ctx_fpga_access_reg_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_fpga_access_reg_dump(const struct connectib_cmdif_ctx_fpga_access_reg *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_fpga_access_reg_print(ptr_struct, file, 0);
}

void connectib_cmdif_create_lag_pack(const struct connectib_cmdif_create_lag *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_remap_affinity_1);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_remap_affinity_2);

}

void connectib_cmdif_create_lag_unpack(struct connectib_cmdif_create_lag *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->tx_remap_affinity_1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 24;
	ptr_struct->tx_remap_affinity_2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_create_lag_print(const struct connectib_cmdif_create_lag *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_create_lag ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_remap_affinity_1  : " UH_FMT "\n", ptr_struct->tx_remap_affinity_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_remap_affinity_2  : " UH_FMT "\n", ptr_struct->tx_remap_affinity_2);

}

int connectib_cmdif_create_lag_size(void)
{
	 return 32;
}

void connectib_cmdif_create_lag_dump(const struct connectib_cmdif_create_lag *ptr_struct, FILE* file)
{
	connectib_cmdif_create_lag_print(ptr_struct, file, 0);
}

void connectib_cmdif_modify_sq_pack(const struct connectib_cmdif_modify_sq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->qp_flush_syndrom);

}

void connectib_cmdif_modify_sq_unpack(struct connectib_cmdif_modify_sq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->qp_flush_syndrom = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_modify_sq_print(const struct connectib_cmdif_modify_sq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_modify_sq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_flush_syndrom     : " U32H_FMT "\n", ptr_struct->qp_flush_syndrom);

}

int connectib_cmdif_modify_sq_size(void)
{
	 return 32;
}

void connectib_cmdif_modify_sq_dump(const struct connectib_cmdif_modify_sq *ptr_struct, FILE* file)
{
	connectib_cmdif_modify_sq_print(ptr_struct, file, 0);
}

void connectib_cmdif_modify_rq_pack(const struct connectib_cmdif_modify_rq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->qp_flush_syndrom);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lwm);

}

void connectib_cmdif_modify_rq_unpack(struct connectib_cmdif_modify_rq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->qp_flush_syndrom = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->lwm = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_modify_rq_print(const struct connectib_cmdif_modify_rq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_modify_rq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_flush_syndrom     : " U32H_FMT "\n", ptr_struct->qp_flush_syndrom);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lwm                  : " U32H_FMT "\n", ptr_struct->lwm);

}

int connectib_cmdif_modify_rq_size(void)
{
	 return 32;
}

void connectib_cmdif_modify_rq_dump(const struct connectib_cmdif_modify_rq *ptr_struct, FILE* file)
{
	connectib_cmdif_modify_rq_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_l2_table_pack(const struct connectib_cmdif_ctx_l2_table *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hw_rx_ix);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hw_sx_ix);

}

void connectib_cmdif_ctx_l2_table_unpack(struct connectib_cmdif_ctx_l2_table *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->hw_rx_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->hw_sx_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_l2_table_print(const struct connectib_cmdif_ctx_l2_table *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_l2_table ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_rx_ix             : " U32H_FMT "\n", ptr_struct->hw_rx_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_sx_ix             : " U32H_FMT "\n", ptr_struct->hw_sx_ix);

}

int connectib_cmdif_ctx_l2_table_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_l2_table_dump(const struct connectib_cmdif_ctx_l2_table *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_l2_table_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_dmfs_ft_pack(const struct connectib_cmdif_ctx_dmfs_ft *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vport_num);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->log_size);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ft_in_level0);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ft_is_root);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->next_table_id);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->table_miss_mode);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lag_demux);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->decap_en);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->encap_en);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->table_type);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_head_ix);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sx_head_ix);

}

void connectib_cmdif_ctx_dmfs_ft_unpack(struct connectib_cmdif_ctx_dmfs_ft *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->vport_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 8;
	ptr_struct->log_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 2;
	ptr_struct->ft_in_level0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->ft_is_root = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 40;
	ptr_struct->next_table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 36;
	ptr_struct->table_miss_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 34;
	ptr_struct->lag_demux = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 33;
	ptr_struct->decap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->encap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->table_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->rx_head_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->sx_head_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_dmfs_ft_print(const struct connectib_cmdif_ctx_dmfs_ft *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_dmfs_ft ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_num            : " UH_FMT "\n", ptr_struct->vport_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_size             : " UH_FMT "\n", ptr_struct->log_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_in_level0         : " UH_FMT "\n", ptr_struct->ft_in_level0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_is_root           : " UH_FMT "\n", ptr_struct->ft_is_root);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_table_id        : " UH_FMT "\n", ptr_struct->next_table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_miss_mode      : " UH_FMT "\n", ptr_struct->table_miss_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lag_demux            : " UH_FMT "\n", ptr_struct->lag_demux);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "decap_en             : " UH_FMT "\n", ptr_struct->decap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encap_en             : " UH_FMT "\n", ptr_struct->encap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_type           : " UH_FMT "\n", ptr_struct->table_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_head_ix           : " U32H_FMT "\n", ptr_struct->rx_head_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_head_ix           : " U32H_FMT "\n", ptr_struct->sx_head_ix);

}

int connectib_cmdif_ctx_dmfs_ft_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_dmfs_ft_dump(const struct connectib_cmdif_ctx_dmfs_ft *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_dmfs_ft_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_dmfs_fg_pack(const struct connectib_cmdif_ctx_dmfs_fg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vport_num);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->table_type);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vst_root_hash_meta_ix);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_end_ste_ix);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sx_end_ste_ix);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_root_hash_meta_ix);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sx_root_hash_meta_ix);

}

void connectib_cmdif_ctx_dmfs_fg_unpack(struct connectib_cmdif_ctx_dmfs_fg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->vport_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 72;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->table_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->vst_root_hash_meta_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->rx_end_ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->sx_end_ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->rx_root_hash_meta_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->sx_root_hash_meta_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_dmfs_fg_print(const struct connectib_cmdif_ctx_dmfs_fg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_dmfs_fg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_num            : " UH_FMT "\n", ptr_struct->vport_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_type           : " UH_FMT "\n", ptr_struct->table_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vst_root_hash_meta_ix : " U32H_FMT "\n", ptr_struct->vst_root_hash_meta_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_end_ste_ix        : " U32H_FMT "\n", ptr_struct->rx_end_ste_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_end_ste_ix        : " U32H_FMT "\n", ptr_struct->sx_end_ste_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_root_hash_meta_ix : " U32H_FMT "\n", ptr_struct->rx_root_hash_meta_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_root_hash_meta_ix : " U32H_FMT "\n", ptr_struct->sx_root_hash_meta_ix);

}

int connectib_cmdif_ctx_dmfs_fg_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_dmfs_fg_dump(const struct connectib_cmdif_ctx_dmfs_fg *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_dmfs_fg_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_dmfs_fte_pack(const struct connectib_cmdif_ctx_dmfs_fte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vport_num);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->copied_items_cntr);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->next_hierarchy_to_alloc);

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fte_use_iterator);

	offset = 38;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->src_port_ext_match_exists);

	offset = 37;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->modify_flow_list);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->external_dest_vport);

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->internal_dest_vport);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->uc_match_exists);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mc_match_exists);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fte_add_remove_);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->table_type);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->iterator_ix);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fte_rx_ix);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fte_sx_ix);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fte_rx2_ix);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fte_sx2_ix);

}

void connectib_cmdif_ctx_dmfs_fte_unpack(struct connectib_cmdif_ctx_dmfs_fte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->vport_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	ptr_struct->copied_items_cntr = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->next_hierarchy_to_alloc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 39;
	ptr_struct->fte_use_iterator = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 38;
	ptr_struct->src_port_ext_match_exists = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 37;
	ptr_struct->modify_flow_list = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 36;
	ptr_struct->external_dest_vport = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 35;
	ptr_struct->internal_dest_vport = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 34;
	ptr_struct->uc_match_exists = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 33;
	ptr_struct->mc_match_exists = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->fte_add_remove_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->table_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->iterator_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->fte_rx_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->fte_sx_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->fte_rx2_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->fte_sx2_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_dmfs_fte_print(const struct connectib_cmdif_ctx_dmfs_fte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_dmfs_fte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_num            : " UH_FMT "\n", ptr_struct->vport_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "copied_items_cntr    : " UH_FMT "\n", ptr_struct->copied_items_cntr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_hierarchy_to_alloc : " UH_FMT "\n", ptr_struct->next_hierarchy_to_alloc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fte_use_iterator     : " UH_FMT "\n", ptr_struct->fte_use_iterator);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "src_port_ext_match_exists : " UH_FMT "\n", ptr_struct->src_port_ext_match_exists);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modify_flow_list     : " UH_FMT "\n", ptr_struct->modify_flow_list);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "external_dest_vport  : " UH_FMT "\n", ptr_struct->external_dest_vport);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "internal_dest_vport  : " UH_FMT "\n", ptr_struct->internal_dest_vport);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uc_match_exists      : " UH_FMT "\n", ptr_struct->uc_match_exists);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mc_match_exists      : " UH_FMT "\n", ptr_struct->mc_match_exists);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fte_add_remove_      : " UH_FMT "\n", ptr_struct->fte_add_remove_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_type           : " UH_FMT "\n", ptr_struct->table_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iterator_ix          : " U32H_FMT "\n", ptr_struct->iterator_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fte_rx_ix            : " U32H_FMT "\n", ptr_struct->fte_rx_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fte_sx_ix            : " U32H_FMT "\n", ptr_struct->fte_sx_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fte_rx2_ix           : " U32H_FMT "\n", ptr_struct->fte_rx2_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fte_sx2_ix           : " U32H_FMT "\n", ptr_struct->fte_sx2_ix);

}

int connectib_cmdif_ctx_dmfs_fte_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_dmfs_fte_dump(const struct connectib_cmdif_ctx_dmfs_fte *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_dmfs_fte_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_create_mtt_pack(const struct connectib_cmdif_ctx_create_mtt *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->mtt_ix);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_mtts);

}

void connectib_cmdif_ctx_create_mtt_unpack(struct connectib_cmdif_ctx_create_mtt *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->mtt_ix = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->num_mtts = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_create_mtt_print(const struct connectib_cmdif_ctx_create_mtt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_create_mtt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_ix               : " U64H_FMT "\n", ptr_struct->mtt_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_mtts             : " U32H_FMT "\n", ptr_struct->num_mtts);

}

int connectib_cmdif_ctx_create_mtt_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_create_mtt_dump(const struct connectib_cmdif_ctx_create_mtt *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_create_mtt_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_psv_pack(const struct connectib_cmdif_ctx_psv *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_psv);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_alloced_psv);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pd);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 256, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->psv_ix[i]);
	}

}

void connectib_cmdif_ctx_psv_unpack(struct connectib_cmdif_ctx_psv *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->num_psv = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->num_alloced_psv = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->pd = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 256, 1);
	ptr_struct->psv_ix[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_cmdif_ctx_psv_print(const struct connectib_cmdif_ctx_psv *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_psv ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_psv              : " U32H_FMT "\n", ptr_struct->num_psv);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_alloced_psv      : " U32H_FMT "\n", ptr_struct->num_alloced_psv);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pd                   : " U32H_FMT "\n", ptr_struct->pd);

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "psv_ix_%03d          : " U32H_FMT "\n", i, ptr_struct->psv_ix[i]);
	}

}

int connectib_cmdif_ctx_psv_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_psv_dump(const struct connectib_cmdif_ctx_psv *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_psv_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_mcg_pack(const struct connectib_cmdif_ctx_mcg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_mcg_index);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sx_mcg_index);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->gid_msb);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->gid_lsb);

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->member_count);

	offset = 206;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->ulp);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->iterator_ix);

}

void connectib_cmdif_ctx_mcg_unpack(struct connectib_cmdif_ctx_mcg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->rx_mcg_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->sx_mcg_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->gid_msb = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 128;
	ptr_struct->gid_lsb = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 208;
	ptr_struct->member_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 206;
	ptr_struct->ulp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 224;
	ptr_struct->iterator_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_mcg_print(const struct connectib_cmdif_ctx_mcg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_mcg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_mcg_index         : " U32H_FMT "\n", ptr_struct->rx_mcg_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_mcg_index         : " U32H_FMT "\n", ptr_struct->sx_mcg_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gid_msb              : " U64H_FMT "\n", ptr_struct->gid_msb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gid_lsb              : " U64H_FMT "\n", ptr_struct->gid_lsb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "member_count         : " UH_FMT "\n", ptr_struct->member_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ulp                  : " UH_FMT "\n", ptr_struct->ulp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iterator_ix          : " U32H_FMT "\n", ptr_struct->iterator_ix);

}

int connectib_cmdif_ctx_mcg_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_mcg_dump(const struct connectib_cmdif_ctx_mcg *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_mcg_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_sniffer_pack(const struct connectib_cmdif_ctx_sniffer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port_0_1);

}

void connectib_cmdif_ctx_sniffer_unpack(struct connectib_cmdif_ctx_sniffer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->port_0_1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_cmdif_ctx_sniffer_print(const struct connectib_cmdif_ctx_sniffer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_sniffer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_0_1             : " UH_FMT "\n", ptr_struct->port_0_1);

}

int connectib_cmdif_ctx_sniffer_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_sniffer_dump(const struct connectib_cmdif_ctx_sniffer *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_sniffer_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_modify_rmp_pack(const struct connectib_cmdif_ctx_special_modify_rmp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->bitmask);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lwm);

}

void connectib_cmdif_ctx_special_modify_rmp_unpack(struct connectib_cmdif_ctx_special_modify_rmp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->bitmask = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->lwm = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_modify_rmp_print(const struct connectib_cmdif_ctx_special_modify_rmp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_modify_rmp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bitmask              : " U64H_FMT "\n", ptr_struct->bitmask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lwm                  : " U32H_FMT "\n", ptr_struct->lwm);

}

int connectib_cmdif_ctx_special_modify_rmp_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_modify_rmp_dump(const struct connectib_cmdif_ctx_special_modify_rmp *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_modify_rmp_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_arm_rq_pack(const struct connectib_cmdif_ctx_special_arm_rq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lwm);

}

void connectib_cmdif_ctx_special_arm_rq_unpack(struct connectib_cmdif_ctx_special_arm_rq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->lwm = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_arm_rq_print(const struct connectib_cmdif_ctx_special_arm_rq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_arm_rq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lwm                  : " U32H_FMT "\n", ptr_struct->lwm);

}

int connectib_cmdif_ctx_special_arm_rq_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_arm_rq_dump(const struct connectib_cmdif_ctx_special_arm_rq *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_arm_rq_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_mkey_pack(const struct connectib_cmdif_ctx_special_mkey *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->hw_addr);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->fw_addr);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->octoword_actual_size_mtt);

}

void connectib_cmdif_ctx_special_mkey_unpack(struct connectib_cmdif_ctx_special_mkey *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->hw_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->fw_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 128;
	ptr_struct->octoword_actual_size_mtt = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_mkey_print(const struct connectib_cmdif_ctx_special_mkey *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_mkey ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_addr              : " U64H_FMT "\n", ptr_struct->hw_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_addr              : " U64H_FMT "\n", ptr_struct->fw_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "octoword_actual_size_mtt : " U32H_FMT "\n", ptr_struct->octoword_actual_size_mtt);

}

int connectib_cmdif_ctx_special_mkey_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_mkey_dump(const struct connectib_cmdif_ctx_special_mkey *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_mkey_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_modify_cq_pack(const struct connectib_cmdif_ctx_special_modify_cq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->mtt_ix);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->producer_index);

	offset = 127;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cqe_128b_size);

	offset = 122;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log2_size);

	offset = 117;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_page_size);

	offset = 111;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->cq_page_offset);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fence_slice);

	offset = 184;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->old_sw_eqn);

	offset = 183;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->modify_fields_c_eqn);

}

void connectib_cmdif_ctx_special_modify_cq_unpack(struct connectib_cmdif_ctx_special_modify_cq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->mtt_ix = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 72;
	ptr_struct->producer_index = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 127;
	ptr_struct->cqe_128b_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 122;
	ptr_struct->log2_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 117;
	ptr_struct->log_page_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 111;
	ptr_struct->cq_page_offset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 128;
	ptr_struct->fence_slice = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 184;
	ptr_struct->old_sw_eqn = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 183;
	ptr_struct->modify_fields_c_eqn = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_cmdif_ctx_special_modify_cq_print(const struct connectib_cmdif_ctx_special_modify_cq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_modify_cq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_ix               : " U64H_FMT "\n", ptr_struct->mtt_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "producer_index       : " UH_FMT "\n", ptr_struct->producer_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_128b_size        : " UH_FMT "\n", ptr_struct->cqe_128b_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_size            : " UH_FMT "\n", ptr_struct->log2_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_page_size        : " UH_FMT "\n", ptr_struct->log_page_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_page_offset       : " UH_FMT "\n", ptr_struct->cq_page_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fence_slice          : " U32H_FMT "\n", ptr_struct->fence_slice);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "old_sw_eqn           : " UH_FMT "\n", ptr_struct->old_sw_eqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modify_fields_c_eqn  : " UH_FMT "\n", ptr_struct->modify_fields_c_eqn);

}

int connectib_cmdif_ctx_special_modify_cq_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_modify_cq_dump(const struct connectib_cmdif_ctx_special_modify_cq *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_modify_cq_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_create_cq_pack(const struct connectib_cmdif_ctx_special_create_cq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->fw_addr);

}

void connectib_cmdif_ctx_special_create_cq_unpack(struct connectib_cmdif_ctx_special_create_cq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->fw_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_cmdif_ctx_special_create_cq_print(const struct connectib_cmdif_ctx_special_create_cq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_create_cq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_addr              : " U64H_FMT "\n", ptr_struct->fw_addr);

}

int connectib_cmdif_ctx_special_create_cq_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_create_cq_dump(const struct connectib_cmdif_ctx_special_create_cq *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_create_cq_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_create_eq_pack(const struct connectib_cmdif_ctx_special_create_eq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->fw_eqc_addr);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->mtt_ix_pas);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->event_bitmask);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hw_eqn);

	offset = 248;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->offset_in_page);

	offset = 242;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log_page_size);

	offset = 239;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_pa_mode);

}

void connectib_cmdif_ctx_special_create_eq_unpack(struct connectib_cmdif_ctx_special_create_eq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->fw_eqc_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->mtt_ix_pas = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 128;
	ptr_struct->event_bitmask = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 192;
	ptr_struct->hw_eqn = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 248;
	ptr_struct->offset_in_page = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 242;
	ptr_struct->log_page_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 239;
	ptr_struct->is_pa_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_cmdif_ctx_special_create_eq_print(const struct connectib_cmdif_ctx_special_create_eq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_create_eq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_eqc_addr          : " U64H_FMT "\n", ptr_struct->fw_eqc_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_ix_pas           : " U64H_FMT "\n", ptr_struct->mtt_ix_pas);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_bitmask        : " U64H_FMT "\n", ptr_struct->event_bitmask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_eqn               : " U32H_FMT "\n", ptr_struct->hw_eqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset_in_page       : " UH_FMT "\n", ptr_struct->offset_in_page);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_page_size        : " UH_FMT "\n", ptr_struct->log_page_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_pa_mode           : " UH_FMT "\n", ptr_struct->is_pa_mode);

}

int connectib_cmdif_ctx_special_create_eq_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_create_eq_dump(const struct connectib_cmdif_ctx_special_create_eq *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_create_eq_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_rqt_pack(const struct connectib_cmdif_ctx_rqt *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rqt_fw_data_ix);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rqt_hw_data_ix);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->rss_indirection_table_base_qpn);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->rss_qpn_size);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_rqs_to_dec);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rqt_max_size);

	offset = 176;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rqt_actual_size);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rqt_actual_size_old);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rqt_fw_data_ix_old);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rqt_hw_data_ix_old);

}

void connectib_cmdif_ctx_rqt_unpack(struct connectib_cmdif_ctx_rqt *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->rqt_fw_data_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->rqt_hw_data_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->rss_indirection_table_base_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->rss_qpn_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 96;
	ptr_struct->num_rqs_to_dec = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->rqt_max_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 176;
	ptr_struct->rqt_actual_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 160;
	ptr_struct->rqt_actual_size_old = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 192;
	ptr_struct->rqt_fw_data_ix_old = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->rqt_hw_data_ix_old = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_rqt_print(const struct connectib_cmdif_ctx_rqt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_rqt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_fw_data_ix       : " U32H_FMT "\n", ptr_struct->rqt_fw_data_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_hw_data_ix       : " U32H_FMT "\n", ptr_struct->rqt_hw_data_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rss_indirection_table_base_qpn : " UH_FMT "\n", ptr_struct->rss_indirection_table_base_qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rss_qpn_size         : " UH_FMT "\n", ptr_struct->rss_qpn_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_rqs_to_dec       : " U32H_FMT "\n", ptr_struct->num_rqs_to_dec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_max_size         : " UH_FMT "\n", ptr_struct->rqt_max_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_actual_size      : " UH_FMT "\n", ptr_struct->rqt_actual_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_actual_size_old  : " UH_FMT "\n", ptr_struct->rqt_actual_size_old);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_fw_data_ix_old   : " U32H_FMT "\n", ptr_struct->rqt_fw_data_ix_old);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_hw_data_ix_old   : " U32H_FMT "\n", ptr_struct->rqt_hw_data_ix_old);

}

int connectib_cmdif_ctx_rqt_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_rqt_dump(const struct connectib_cmdif_ctx_rqt *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_rqt_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_create_xrq_pack(const struct connectib_cmdif_ctx_special_create_xrq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->curr_mtt_offset);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->fw_srqn);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->tgt_num);

	offset = 68;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->offload);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->topology);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->nvme_sq_mkn);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->nvme_cq_mkn);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->uapp_data_mkn);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->uapp_ctrl_mkn);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_nim_indicies_allocated);

}

void connectib_cmdif_ctx_special_create_xrq_unpack(struct connectib_cmdif_ctx_special_create_xrq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->curr_mtt_offset = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->fw_srqn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 72;
	ptr_struct->tgt_num = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 68;
	ptr_struct->offload = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->topology = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 104;
	ptr_struct->nvme_sq_mkn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 136;
	ptr_struct->nvme_cq_mkn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 168;
	ptr_struct->uapp_data_mkn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 200;
	ptr_struct->uapp_ctrl_mkn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 224;
	ptr_struct->num_nim_indicies_allocated = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_create_xrq_print(const struct connectib_cmdif_ctx_special_create_xrq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_create_xrq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "curr_mtt_offset      : " UH_FMT "\n", ptr_struct->curr_mtt_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_srqn              : " UH_FMT "\n", ptr_struct->fw_srqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tgt_num              : " UH_FMT "\n", ptr_struct->tgt_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offload              : " UH_FMT "\n", ptr_struct->offload);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "topology             : " UH_FMT "\n", ptr_struct->topology);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvme_sq_mkn          : " UH_FMT "\n", ptr_struct->nvme_sq_mkn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvme_cq_mkn          : " UH_FMT "\n", ptr_struct->nvme_cq_mkn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uapp_data_mkn        : " UH_FMT "\n", ptr_struct->uapp_data_mkn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uapp_ctrl_mkn        : " UH_FMT "\n", ptr_struct->uapp_ctrl_mkn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_nim_indicies_allocated : " U32H_FMT "\n", ptr_struct->num_nim_indicies_allocated);

}

int connectib_cmdif_ctx_special_create_xrq_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_create_xrq_dump(const struct connectib_cmdif_ctx_special_create_xrq *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_create_xrq_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_create_srq_pack(const struct connectib_cmdif_ctx_special_create_srq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->fw_addr);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->srqi);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cont_srq);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pas_offset);

}

void connectib_cmdif_ctx_special_create_srq_unpack(struct connectib_cmdif_ctx_special_create_srq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->fw_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 72;
	ptr_struct->srqi = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->cont_srq = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->pas_offset = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_create_srq_print(const struct connectib_cmdif_ctx_special_create_srq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_create_srq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_addr              : " U64H_FMT "\n", ptr_struct->fw_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srqi                 : " UH_FMT "\n", ptr_struct->srqi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cont_srq             : " UH_FMT "\n", ptr_struct->cont_srq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pas_offset           : " U32H_FMT "\n", ptr_struct->pas_offset);

}

int connectib_cmdif_ctx_special_create_srq_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_create_srq_dump(const struct connectib_cmdif_ctx_special_create_srq *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_create_srq_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_create_rq_pack(const struct connectib_cmdif_ctx_special_create_rq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->fw_qpc_addr);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->no_lro_qpn);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_max_qp);

	offset = 116;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->wq_type);

}

void connectib_cmdif_ctx_special_create_rq_unpack(struct connectib_cmdif_ctx_special_create_rq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->fw_qpc_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->no_lro_qpn = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 123;
	ptr_struct->log_max_qp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 116;
	ptr_struct->wq_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_create_rq_print(const struct connectib_cmdif_ctx_special_create_rq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_create_rq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_qpc_addr          : " U64H_FMT "\n", ptr_struct->fw_qpc_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_lro_qpn           : " U32H_FMT "\n", ptr_struct->no_lro_qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_qp           : " UH_FMT "\n", ptr_struct->log_max_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wq_type              : " UH_FMT "\n", ptr_struct->wq_type);

}

int connectib_cmdif_ctx_special_create_rq_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_create_rq_dump(const struct connectib_cmdif_ctx_special_create_rq *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_create_rq_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_qp_commands_pack(const struct connectib_cmdif_ctx_special_qp_commands *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->fw_qpc_addr);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->qpi);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_max_qp);

	offset = 122;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_dci);

	offset = 121;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_enhanced_ipoib);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_xrq);

	offset = 117;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->offload_xrq_type);

	offset = 116;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_tag_matching_xrq);

	offset = 128;
	connectib_cmdif_ctx_special_qp_commands_specific_pack(&(ptr_struct->specific), ptr_buff + offset/8);

}

void connectib_cmdif_ctx_special_qp_commands_unpack(struct connectib_cmdif_ctx_special_qp_commands *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->fw_qpc_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->qpi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 123;
	ptr_struct->log_max_qp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 122;
	ptr_struct->is_dci = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 121;
	ptr_struct->is_enhanced_ipoib = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 120;
	ptr_struct->is_xrq = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 117;
	ptr_struct->offload_xrq_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 116;
	ptr_struct->is_tag_matching_xrq = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 128;
	connectib_cmdif_ctx_special_qp_commands_specific_unpack(&(ptr_struct->specific), ptr_buff + offset/8);

}

void connectib_cmdif_ctx_special_qp_commands_print(const struct connectib_cmdif_ctx_special_qp_commands *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_qp_commands ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_qpc_addr          : " U64H_FMT "\n", ptr_struct->fw_qpc_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpi                  : " U32H_FMT "\n", ptr_struct->qpi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_qp           : " UH_FMT "\n", ptr_struct->log_max_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_dci               : " UH_FMT "\n", ptr_struct->is_dci);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_enhanced_ipoib    : " UH_FMT "\n", ptr_struct->is_enhanced_ipoib);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_xrq               : " UH_FMT "\n", ptr_struct->is_xrq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offload_xrq_type     : " UH_FMT "\n", ptr_struct->offload_xrq_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_tag_matching_xrq  : " UH_FMT "\n", ptr_struct->is_tag_matching_xrq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "specific:\n");
	connectib_cmdif_ctx_special_qp_commands_specific_print(&(ptr_struct->specific), file, indent_level + 1);

}

int connectib_cmdif_ctx_special_qp_commands_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_qp_commands_dump(const struct connectib_cmdif_ctx_special_qp_commands *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_qp_commands_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_modify_tis_pack(const struct connectib_cmdif_ctx_special_modify_tis *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->transport_domain);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->underlay_qpn);

}

void connectib_cmdif_ctx_special_modify_tis_unpack(struct connectib_cmdif_ctx_special_modify_tis *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->transport_domain = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 40;
	ptr_struct->underlay_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_cmdif_ctx_special_modify_tis_print(const struct connectib_cmdif_ctx_special_modify_tis *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_modify_tis ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transport_domain     : " UH_FMT "\n", ptr_struct->transport_domain);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "underlay_qpn         : " UH_FMT "\n", ptr_struct->underlay_qpn);

}

int connectib_cmdif_ctx_special_modify_tis_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_modify_tis_dump(const struct connectib_cmdif_ctx_special_modify_tis *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_modify_tis_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_modify_tir_pack(const struct connectib_cmdif_ctx_special_modify_tir *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->bitmask);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tir_ste_ix);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hash_filter_base_ix);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hash_entries_base_ix);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->rqt_number);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->old_rqt_number);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->inline_rqn);

	offset = 196;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->disp_type);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->transport_domain);

}

void connectib_cmdif_ctx_special_modify_tir_unpack(struct connectib_cmdif_ctx_special_modify_tir *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->bitmask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->tir_ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->hash_filter_base_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->hash_entries_base_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 136;
	ptr_struct->rqt_number = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 168;
	ptr_struct->old_rqt_number = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 200;
	ptr_struct->inline_rqn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 196;
	ptr_struct->disp_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 232;
	ptr_struct->transport_domain = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_cmdif_ctx_special_modify_tir_print(const struct connectib_cmdif_ctx_special_modify_tir *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_modify_tir ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bitmask              : " U32H_FMT "\n", ptr_struct->bitmask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tir_ste_ix           : " U32H_FMT "\n", ptr_struct->tir_ste_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hash_filter_base_ix  : " U32H_FMT "\n", ptr_struct->hash_filter_base_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hash_entries_base_ix : " U32H_FMT "\n", ptr_struct->hash_entries_base_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_number           : " UH_FMT "\n", ptr_struct->rqt_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "old_rqt_number       : " UH_FMT "\n", ptr_struct->old_rqt_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inline_rqn           : " UH_FMT "\n", ptr_struct->inline_rqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disp_type            : " UH_FMT "\n", ptr_struct->disp_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transport_domain     : " UH_FMT "\n", ptr_struct->transport_domain);

}

int connectib_cmdif_ctx_special_modify_tir_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_modify_tir_dump(const struct connectib_cmdif_ctx_special_modify_tir *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_modify_tir_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_create_pack(const struct connectib_cmdif_ctx_special_create *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_mapped_mtts);

}

void connectib_cmdif_ctx_special_create_unpack(struct connectib_cmdif_ctx_special_create *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->num_mapped_mtts = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmdif_ctx_special_create_print(const struct connectib_cmdif_ctx_special_create *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special_create ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_mapped_mtts      : " U32H_FMT "\n", ptr_struct->num_mapped_mtts);

}

int connectib_cmdif_ctx_special_create_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_create_dump(const struct connectib_cmdif_ctx_special_create *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_create_print(ptr_struct, file, 0);
}

void connectib_flush_type_bitmask_pack(const struct connectib_flush_type_bitmask *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword);

}

void connectib_flush_type_bitmask_unpack(struct connectib_flush_type_bitmask *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->dword = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_flush_type_bitmask_print(const struct connectib_flush_type_bitmask *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_flush_type_bitmask ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword                : " U32H_FMT "\n", ptr_struct->dword);

}

int connectib_flush_type_bitmask_size(void)
{
	 return 4;
}

void connectib_flush_type_bitmask_dump(const struct connectib_flush_type_bitmask *ptr_struct, FILE* file)
{
	connectib_flush_type_bitmask_print(ptr_struct, file, 0);
}

void connectib_flush_type_bits_pack(const struct connectib_flush_type_bits *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mkey_flush);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qp_s_flush);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qp_r_flush);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rx_flush);

}

void connectib_flush_type_bits_unpack(struct connectib_flush_type_bits *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->mkey_flush = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->qp_s_flush = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->qp_r_flush = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->rx_flush = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_flush_type_bits_print(const struct connectib_flush_type_bits *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_flush_type_bits ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mkey_flush           : " UH_FMT "\n", ptr_struct->mkey_flush);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_s_flush           : " UH_FMT "\n", ptr_struct->qp_s_flush);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_r_flush           : " UH_FMT "\n", ptr_struct->qp_r_flush);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_flush             : " UH_FMT "\n", ptr_struct->rx_flush);

}

int connectib_flush_type_bits_size(void)
{
	 return 4;
}

void connectib_flush_type_bits_dump(const struct connectib_flush_type_bits *ptr_struct, FILE* file)
{
	connectib_flush_type_bits_print(ptr_struct, file, 0);
}

void connectib_reg_access_reg_tlv_data_pack(const union connectib_reg_access_reg_tlv_data *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_pmcr_reg_pack(&(ptr_struct->pmcr), ptr_buff);
}

void connectib_reg_access_reg_tlv_data_unpack(union connectib_reg_access_reg_tlv_data *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_pmcr_reg_unpack(&(ptr_struct->pmcr), ptr_buff);
}

void connectib_reg_access_reg_tlv_data_print(const union connectib_reg_access_reg_tlv_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_reg_access_reg_tlv_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfpa:\n");
	connectib_mfpa_print(&(ptr_struct->mfpa), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfai:\n");
	connectib_register_mfai_print(&(ptr_struct->mfai), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfrl:\n");
	connectib_register_mfrl_print(&(ptr_struct->mfrl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtmp:\n");
	connectib_mtmp_print(&(ptr_struct->mtmp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "misc_counters:\n");
	connectib_register_misc_counters_print(&(ptr_struct->misc_counters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfba:\n");
	connectib_project_mfba_print(&(ptr_struct->mfba), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfbe:\n");
	connectib_mfbe_print(&(ptr_struct->mfbe), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mcqs:\n");
	connectib_mcqs_reg_print(&(ptr_struct->mcqs), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mcqi:\n");
	connectib_mcqi_reg_print(&(ptr_struct->mcqi), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mcc:\n");
	connectib_mcc_reg_print(&(ptr_struct->mcc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mcda:\n");
	connectib_mcda_reg_print(&(ptr_struct->mcda), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mqis:\n");
	connectib_mqis_reg_print(&(ptr_struct->mqis), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mjtag:\n");
	connectib_mjtag_print(&(ptr_struct->mjtag), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "paos:\n");
	connectib_paos_reg_print(&(ptr_struct->paos), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slrp:\n");
	connectib_slrp_reg_print(&(ptr_struct->slrp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sltp:\n");
	connectib_sltp_reg_print(&(ptr_struct->sltp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ptas:\n");
	connectib_ptas_reg_print(&(ptr_struct->ptas), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slrg:\n");
	connectib_slrg_reg_print(&(ptr_struct->slrg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pptt:\n");
	connectib_pptt_reg_print(&(ptr_struct->pptt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pprt:\n");
	connectib_pprt_reg_print(&(ptr_struct->pprt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ppaos:\n");
	connectib_ppaos_reg_print(&(ptr_struct->ppaos), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ppamp:\n");
	connectib_pamp_reg_print(&(ptr_struct->ppamp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pmtu:\n");
	connectib_pmtu_reg_print(&(ptr_struct->pmtu), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvda:\n");
	connectib_project_nvda_print(&(ptr_struct->nvda), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvdi:\n");
	connectib_nv_tlv_hdr_print(&(ptr_struct->nvdi), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvqc:\n");
	connectib_project_nvqc_print(&(ptr_struct->nvqc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvgc:\n");
	connectib_project_nvgc_print(&(ptr_struct->nvgc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvia:\n");
	connectib_project_nvia_print(&(ptr_struct->nvia), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nvgn:\n");
	connectib_project_nvgn_print(&(ptr_struct->nvgn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pplr:\n");
	connectib_pplr_reg_print(&(ptr_struct->pplr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "plpc:\n");
	connectib_plpc_reg_print(&(ptr_struct->plpc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pplm:\n");
	connectib_pplm_reg_print(&(ptr_struct->pplm), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pmlp:\n");
	connectib_pmlp_print(&(ptr_struct->pmlp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mvts:\n");
	connectib_mvts_print(&(ptr_struct->mvts), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mpein:\n");
	connectib_mpein_reg_print(&(ptr_struct->mpein), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pmcr:\n");
	connectib_pmcr_reg_print(&(ptr_struct->pmcr), file, indent_level + 1);

}

int connectib_reg_access_reg_tlv_data_size(void)
{
	 return 44;
}

void connectib_reg_access_reg_tlv_data_dump(const union connectib_reg_access_reg_tlv_data *ptr_struct, FILE* file)
{
	connectib_reg_access_reg_tlv_data_print(ptr_struct, file, 0);
}

void connectib_pmaos_reg_pack(const struct connectib_pmaos_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->oper_status);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->admin_status);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->e);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_type);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ee);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ase);

}

void connectib_pmaos_reg_unpack(struct connectib_pmaos_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->oper_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 20;
	ptr_struct->admin_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 8;
	ptr_struct->module = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 62;
	ptr_struct->e = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 52;
	ptr_struct->error_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 33;
	ptr_struct->ee = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->ase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_pmaos_reg_print(const struct connectib_pmaos_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmaos_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "oper_status          : " UH_FMT "\n", ptr_struct->oper_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "admin_status         : " UH_FMT "\n", ptr_struct->admin_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module               : " UH_FMT "\n", ptr_struct->module);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e                    : " UH_FMT "\n", ptr_struct->e);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_type           : " UH_FMT "\n", ptr_struct->error_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ee                   : " UH_FMT "\n", ptr_struct->ee);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ase                  : " UH_FMT "\n", ptr_struct->ase);

}

int connectib_pmaos_reg_size(void)
{
	 return 16;
}

void connectib_pmaos_reg_dump(const struct connectib_pmaos_reg *ptr_struct, FILE* file)
{
	connectib_pmaos_reg_print(ptr_struct, file, 0);
}

void connectib_mtppse_reg_pack(const struct connectib_mtppse_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pin);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->event_generation_mode);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->event_arm);

}

void connectib_mtppse_reg_unpack(struct connectib_mtppse_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->pin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 60;
	ptr_struct->event_generation_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->event_arm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_mtppse_reg_print(const struct connectib_mtppse_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mtppse_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pin                  : " UH_FMT "\n", ptr_struct->pin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_generation_mode : " UH_FMT "\n", ptr_struct->event_generation_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_arm            : " UH_FMT "\n", ptr_struct->event_arm);

}

int connectib_mtppse_reg_size(void)
{
	 return 16;
}

void connectib_mtppse_reg_dump(const struct connectib_mtppse_reg *ptr_struct, FILE* file)
{
	connectib_mtppse_reg_print(ptr_struct, file, 0);
}

void connectib_mtpps_reg_pack(const struct connectib_mtpps_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_max_num_of_pps_out_pins);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_max_num_of_pps_in_pins);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_number_of_pps_pins);

	offset = 92;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_pin_0_mode);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_pin_1_mode);

	offset = 76;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_pin_2_mode);

	offset = 68;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_pin_3_mode);

	offset = 124;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_pin_4_mode);

	offset = 116;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_pin_5_mode);

	offset = 108;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_pin_6_mode);

	offset = 100;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cap_pin_7_mode);

	offset = 280;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pin);

	offset = 276;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pin_mode);

	offset = 268;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pattern);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_stamp_hi);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_stamp_lo);

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->out_periodic_adjustment);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->out_pulse_duration);

}

void connectib_mtpps_reg_unpack(struct connectib_mtpps_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->cap_max_num_of_pps_out_pins = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 20;
	ptr_struct->cap_max_num_of_pps_in_pins = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 12;
	ptr_struct->cap_number_of_pps_pins = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 92;
	ptr_struct->cap_pin_0_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 84;
	ptr_struct->cap_pin_1_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 76;
	ptr_struct->cap_pin_2_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 68;
	ptr_struct->cap_pin_3_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 124;
	ptr_struct->cap_pin_4_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 116;
	ptr_struct->cap_pin_5_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 108;
	ptr_struct->cap_pin_6_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 100;
	ptr_struct->cap_pin_7_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 280;
	ptr_struct->pin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 276;
	ptr_struct->pin_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 268;
	ptr_struct->pattern = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 320;
	ptr_struct->time_stamp_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->time_stamp_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 400;
	ptr_struct->out_periodic_adjustment = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 384;
	ptr_struct->out_pulse_duration = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_mtpps_reg_print(const struct connectib_mtpps_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mtpps_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_max_num_of_pps_out_pins : " UH_FMT "\n", ptr_struct->cap_max_num_of_pps_out_pins);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_max_num_of_pps_in_pins : " UH_FMT "\n", ptr_struct->cap_max_num_of_pps_in_pins);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_number_of_pps_pins : " UH_FMT "\n", ptr_struct->cap_number_of_pps_pins);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_pin_0_mode       : " UH_FMT "\n", ptr_struct->cap_pin_0_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_pin_1_mode       : " UH_FMT "\n", ptr_struct->cap_pin_1_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_pin_2_mode       : " UH_FMT "\n", ptr_struct->cap_pin_2_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_pin_3_mode       : " UH_FMT "\n", ptr_struct->cap_pin_3_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_pin_4_mode       : " UH_FMT "\n", ptr_struct->cap_pin_4_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_pin_5_mode       : " UH_FMT "\n", ptr_struct->cap_pin_5_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_pin_6_mode       : " UH_FMT "\n", ptr_struct->cap_pin_6_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_pin_7_mode       : " UH_FMT "\n", ptr_struct->cap_pin_7_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pin                  : " UH_FMT "\n", ptr_struct->pin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pin_mode             : " UH_FMT "\n", ptr_struct->pin_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pattern              : " UH_FMT "\n", ptr_struct->pattern);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable               : " UH_FMT "\n", ptr_struct->enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_stamp_hi        : " U32H_FMT "\n", ptr_struct->time_stamp_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_stamp_lo        : " U32H_FMT "\n", ptr_struct->time_stamp_lo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out_periodic_adjustment : " UH_FMT "\n", ptr_struct->out_periodic_adjustment);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out_pulse_duration   : " UH_FMT "\n", ptr_struct->out_pulse_duration);

}

int connectib_mtpps_reg_size(void)
{
	 return 64;
}

void connectib_mtpps_reg_dump(const struct connectib_mtpps_reg *ptr_struct, FILE* file)
{
	connectib_mtpps_reg_print(ptr_struct, file, 0);
}

void connectib_fpga_ctrl_pack(const struct connectib_fpga_ctrl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->status);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->operation);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->flash_select_oper);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->flash_select_admin);

}

void connectib_fpga_ctrl_unpack(struct connectib_fpga_ctrl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->operation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 56;
	ptr_struct->flash_select_oper = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 40;
	ptr_struct->flash_select_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_fpga_ctrl_print(const struct connectib_fpga_ctrl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fpga_ctrl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : %s (" UH_FMT ")\n", (ptr_struct->status == 0 ? ("Success") : ((ptr_struct->status == 1 ? ("Failure") : ((ptr_struct->status == 2 ? ("In_progress") : ("unknown")))))), ptr_struct->status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "operation            : %s (" UH_FMT ")\n", (ptr_struct->operation == 1 ? ("LOAD") : ((ptr_struct->operation == 2 ? ("RESET") : ((ptr_struct->operation == 3 ? ("FLASH_SELECT") : ((ptr_struct->operation == 4 ? ("Sandbox_Bypass_On") : ((ptr_struct->operation == 5 ? ("Sandbox_Bypass_Off") : ("unknown")))))))))), ptr_struct->operation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_select_oper    : %s (" UH_FMT ")\n", (ptr_struct->flash_select_oper == 0 ? ("Factory_default") : ((ptr_struct->flash_select_oper == 1 ? ("User") : ("unknown")))), ptr_struct->flash_select_oper);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_select_admin   : %s (" UH_FMT ")\n", (ptr_struct->flash_select_admin == 0 ? ("Factory_default") : ((ptr_struct->flash_select_admin == 1 ? ("User") : ("unknown")))), ptr_struct->flash_select_admin);

}

int connectib_fpga_ctrl_size(void)
{
	 return 16;
}

void connectib_fpga_ctrl_dump(const struct connectib_fpga_ctrl *ptr_struct, FILE* file)
{
	connectib_fpga_ctrl_print(ptr_struct, file, 0);
}

void connectib_fpga_cap_pack(const struct connectib_fpga_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->fpga_device);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->fpga_id);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->register_file_ver);

	offset = 78;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->access_reg_modify_mode);

	offset = 70;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->access_reg_query_mode);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fpga_ctrl_modify);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->image_version);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->image_date);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->image_time);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->shell_version);

	offset = 384;
	connectib_fpga_shell_caps_pack(&(ptr_struct->shell_caps), ptr_buff + offset/8);

	offset = 904;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->ieee_vendor_id);

	offset = 944;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->sandbox_product_id);

	offset = 928;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->sandbox_product_version);

	offset = 960;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sandbox_basic_caps);

	offset = 1008;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->sandbox_extended_caps_len);

	offset = 1024;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sandbox_extended_caps_addr_h);

	offset = 1056;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sandbox_extended_caps_addr_l);

	offset = 1088;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fpga_ddr_start_addr_h);

	offset = 1120;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fpga_ddr_start_addr_l);

	offset = 1152;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fpga_cr_space_start_addr_h);

	offset = 1184;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fpga_cr_space_start_addr_l);

	offset = 1216;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fpga_ddr_size);

	offset = 1248;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fpga_cr_space_size);

}

void connectib_fpga_cap_unpack(struct connectib_fpga_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->fpga_device = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->fpga_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->register_file_ver = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 78;
	ptr_struct->access_reg_modify_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 70;
	ptr_struct->access_reg_query_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 64;
	ptr_struct->fpga_ctrl_modify = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 128;
	ptr_struct->image_version = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->image_date = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->image_time = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->shell_version = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	connectib_fpga_shell_caps_unpack(&(ptr_struct->shell_caps), ptr_buff + offset/8);

	offset = 904;
	ptr_struct->ieee_vendor_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 944;
	ptr_struct->sandbox_product_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 928;
	ptr_struct->sandbox_product_version = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 960;
	ptr_struct->sandbox_basic_caps = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1008;
	ptr_struct->sandbox_extended_caps_len = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 1024;
	ptr_struct->sandbox_extended_caps_addr_h = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1056;
	ptr_struct->sandbox_extended_caps_addr_l = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1088;
	ptr_struct->fpga_ddr_start_addr_h = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1120;
	ptr_struct->fpga_ddr_start_addr_l = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1152;
	ptr_struct->fpga_cr_space_start_addr_h = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1184;
	ptr_struct->fpga_cr_space_start_addr_l = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1216;
	ptr_struct->fpga_ddr_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1248;
	ptr_struct->fpga_cr_space_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_fpga_cap_print(const struct connectib_fpga_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fpga_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_device          : %s (" UH_FMT ")\n", (ptr_struct->fpga_device == 0 ? ("KU040") : ((ptr_struct->fpga_device == 1 ? ("KU060") : ("unknown")))), ptr_struct->fpga_device);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_id              : %s (" UH_FMT ")\n", (ptr_struct->fpga_id == 0 ? ("unknown") : ((ptr_struct->fpga_id == 1 ? ("Newton_X") : ("unknown")))), ptr_struct->fpga_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "register_file_ver    : " U32H_FMT "\n", ptr_struct->register_file_ver);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_reg_modify_mode : %s (" UH_FMT ")\n", (ptr_struct->access_reg_modify_mode == 0 ? ("Not_allowed") : ((ptr_struct->access_reg_modify_mode == 1 ? ("All_range_allowed") : ("unknown")))), ptr_struct->access_reg_modify_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_reg_query_mode : %s (" UH_FMT ")\n", (ptr_struct->access_reg_query_mode == 0 ? ("Not_allowed") : ((ptr_struct->access_reg_query_mode == 1 ? ("All_range_allowed") : ("unknown")))), ptr_struct->access_reg_query_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_ctrl_modify     : " UH_FMT "\n", ptr_struct->fpga_ctrl_modify);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "image_version        : " U32H_FMT "\n", ptr_struct->image_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "image_date           : " U32H_FMT "\n", ptr_struct->image_date);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "image_time           : " U32H_FMT "\n", ptr_struct->image_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "shell_version        : " U32H_FMT "\n", ptr_struct->shell_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "shell_caps:\n");
	connectib_fpga_shell_caps_print(&(ptr_struct->shell_caps), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ieee_vendor_id       : " UH_FMT "\n", ptr_struct->ieee_vendor_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sandbox_product_id   : %s (" UH_FMT ")\n", (ptr_struct->sandbox_product_id == 1 ? ("example") : ((ptr_struct->sandbox_product_id == 2 ? ("IPsec") : ((ptr_struct->sandbox_product_id == 3 ? ("TLS") : ("unknown")))))), ptr_struct->sandbox_product_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sandbox_product_version : " UH_FMT "\n", ptr_struct->sandbox_product_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sandbox_basic_caps   : " U32H_FMT "\n", ptr_struct->sandbox_basic_caps);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sandbox_extended_caps_len : " UH_FMT "\n", ptr_struct->sandbox_extended_caps_len);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sandbox_extended_caps_addr_h : " U32H_FMT "\n", ptr_struct->sandbox_extended_caps_addr_h);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sandbox_extended_caps_addr_l : " U32H_FMT "\n", ptr_struct->sandbox_extended_caps_addr_l);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_ddr_start_addr_h : " U32H_FMT "\n", ptr_struct->fpga_ddr_start_addr_h);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_ddr_start_addr_l : " U32H_FMT "\n", ptr_struct->fpga_ddr_start_addr_l);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_cr_space_start_addr_h : " U32H_FMT "\n", ptr_struct->fpga_cr_space_start_addr_h);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_cr_space_start_addr_l : " U32H_FMT "\n", ptr_struct->fpga_cr_space_start_addr_l);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_ddr_size        : " U32H_FMT "\n", ptr_struct->fpga_ddr_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_cr_space_size   : " U32H_FMT "\n", ptr_struct->fpga_cr_space_size);

}

int connectib_fpga_cap_size(void)
{
	 return 256;
}

void connectib_fpga_cap_dump(const struct connectib_fpga_cap *ptr_struct, FILE* file)
{
	connectib_fpga_cap_print(ptr_struct, file, 0);
}

void connectib_fpga_access_reg_pack(const struct connectib_fpga_access_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->size);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->address_h);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->address_l);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(152, 8, i, 160, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->data[i]);
	}

}

void connectib_fpga_access_reg_unpack(struct connectib_fpga_access_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 48;
	ptr_struct->size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->address_h = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->address_l = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(152, 8, i, 160, 1);
	ptr_struct->data[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

}

void connectib_fpga_access_reg_print(const struct connectib_fpga_access_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fpga_access_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address_h            : " U32H_FMT "\n", ptr_struct->address_h);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address_l            : " U32H_FMT "\n", ptr_struct->address_l);

	for (i = 0; i < 1; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " UH_FMT "\n", i, ptr_struct->data[i]);
	}

}

int connectib_fpga_access_reg_size(void)
{
	 return 20;
}

void connectib_fpga_access_reg_dump(const struct connectib_fpga_access_reg *ptr_struct, FILE* file)
{
	connectib_fpga_access_reg_print(ptr_struct, file, 0);
}

void connectib_pmmp_reg_pack(const struct connectib_pmmp_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->eeprom_override);

	offset = 64;
	connectib_pmmp_qsfp_protocol_override_layout_auto_pack(&(ptr_struct->qsfp_cable_protocol_technology), ptr_buff + offset/8);

	offset = 160;
	connectib_pmmp_sfp_protocol_override_layout_auto_pack(&(ptr_struct->sfp_cable_protocol_technology), ptr_buff + offset/8);

	offset = 280;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cable_length);

	offset = 312;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->attenuation_5g);

	offset = 304;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->attenuation_7g);

	offset = 296;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->attenuation_12g);

	offset = 344;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module_identifier);

}

void connectib_pmmp_reg_unpack(struct connectib_pmmp_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->module = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 56;
	ptr_struct->eeprom_override = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	connectib_pmmp_qsfp_protocol_override_layout_auto_unpack(&(ptr_struct->qsfp_cable_protocol_technology), ptr_buff + offset/8);

	offset = 160;
	connectib_pmmp_sfp_protocol_override_layout_auto_unpack(&(ptr_struct->sfp_cable_protocol_technology), ptr_buff + offset/8);

	offset = 280;
	ptr_struct->cable_length = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 312;
	ptr_struct->attenuation_5g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 304;
	ptr_struct->attenuation_7g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 296;
	ptr_struct->attenuation_12g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 344;
	ptr_struct->module_identifier = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_pmmp_reg_print(const struct connectib_pmmp_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmmp_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module               : " UH_FMT "\n", ptr_struct->module);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eeprom_override      : " UH_FMT "\n", ptr_struct->eeprom_override);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qsfp_cable_protocol_technology:\n");
	connectib_pmmp_qsfp_protocol_override_layout_auto_print(&(ptr_struct->qsfp_cable_protocol_technology), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sfp_cable_protocol_technology:\n");
	connectib_pmmp_sfp_protocol_override_layout_auto_print(&(ptr_struct->sfp_cable_protocol_technology), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cable_length         : " UH_FMT "\n", ptr_struct->cable_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "attenuation_5g       : " UH_FMT "\n", ptr_struct->attenuation_5g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "attenuation_7g       : " UH_FMT "\n", ptr_struct->attenuation_7g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "attenuation_12g      : " UH_FMT "\n", ptr_struct->attenuation_12g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_identifier    : " UH_FMT "\n", ptr_struct->module_identifier);

}

int connectib_pmmp_reg_size(void)
{
	 return 44;
}

void connectib_pmmp_reg_dump(const struct connectib_pmmp_reg *ptr_struct, FILE* file)
{
	connectib_pmmp_reg_print(ptr_struct, file, 0);
}

void connectib_pcmr_reg_pack(const struct connectib_pcmr_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pcs_cap);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fcs_cap);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sht_cap);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mtu_cap);

	offset = 59;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->type_cap);

	offset = 58;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->len_cap);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fc_cap);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cp_cap);

	offset = 95;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pcs_chk);

	offset = 94;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fcs_chk);

	offset = 93;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sht_chk);

	offset = 92;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mtu_chk);

	offset = 91;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->type_chk);

	offset = 90;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->len_chk);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fc_chk);

	offset = 87;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cp_chk);

}

void connectib_pcmr_reg_unpack(struct connectib_pcmr_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 63;
	ptr_struct->pcs_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 62;
	ptr_struct->fcs_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 61;
	ptr_struct->sht_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 60;
	ptr_struct->mtu_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 59;
	ptr_struct->type_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 58;
	ptr_struct->len_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 56;
	ptr_struct->fc_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 55;
	ptr_struct->cp_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 95;
	ptr_struct->pcs_chk = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 94;
	ptr_struct->fcs_chk = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 93;
	ptr_struct->sht_chk = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 92;
	ptr_struct->mtu_chk = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 91;
	ptr_struct->type_chk = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 90;
	ptr_struct->len_chk = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 88;
	ptr_struct->fc_chk = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 87;
	ptr_struct->cp_chk = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_pcmr_reg_print(const struct connectib_pcmr_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcmr_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcs_cap              : " UH_FMT "\n", ptr_struct->pcs_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fcs_cap              : " UH_FMT "\n", ptr_struct->fcs_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sht_cap              : " UH_FMT "\n", ptr_struct->sht_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtu_cap              : " UH_FMT "\n", ptr_struct->mtu_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type_cap             : " UH_FMT "\n", ptr_struct->type_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "len_cap              : " UH_FMT "\n", ptr_struct->len_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_cap               : " UH_FMT "\n", ptr_struct->fc_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cp_cap               : " UH_FMT "\n", ptr_struct->cp_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcs_chk              : " UH_FMT "\n", ptr_struct->pcs_chk);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fcs_chk              : " UH_FMT "\n", ptr_struct->fcs_chk);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sht_chk              : " UH_FMT "\n", ptr_struct->sht_chk);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtu_chk              : " UH_FMT "\n", ptr_struct->mtu_chk);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type_chk             : " UH_FMT "\n", ptr_struct->type_chk);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "len_chk              : " UH_FMT "\n", ptr_struct->len_chk);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_chk               : " UH_FMT "\n", ptr_struct->fc_chk);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cp_chk               : " UH_FMT "\n", ptr_struct->cp_chk);

}

int connectib_pcmr_reg_size(void)
{
	 return 12;
}

void connectib_pcmr_reg_dump(const struct connectib_pcmr_reg *ptr_struct, FILE* file)
{
	connectib_pcmr_reg_print(ptr_struct, file, 0);
}

void connectib_mcion_reg_pack(const struct connectib_mcion_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module_status_bits);

}

void connectib_mcion_reg_unpack(struct connectib_mcion_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->module = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 56;
	ptr_struct->module_status_bits = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_mcion_reg_print(const struct connectib_mcion_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mcion_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module               : " UH_FMT "\n", ptr_struct->module);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_status_bits   : " UH_FMT "\n", ptr_struct->module_status_bits);

}

int connectib_mcion_reg_size(void)
{
	 return 24;
}

void connectib_mcion_reg_dump(const struct connectib_mcion_reg *ptr_struct, FILE* file)
{
	connectib_mcion_reg_print(ptr_struct, file, 0);
}

void connectib_pddr_reg_pack(const struct connectib_pddr_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pddr_reg_unpack(struct connectib_pddr_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_pddr_reg_print(const struct connectib_pddr_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pddr_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_pddr_reg_size(void)
{
	 return 256;
}

void connectib_pddr_reg_dump(const struct connectib_pddr_reg *ptr_struct, FILE* file)
{
	connectib_pddr_reg_print(ptr_struct, file, 0);
}

void connectib_mpcnt_reg_pack(const struct connectib_mpcnt_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->grp);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pcie_index);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->clr);

	offset = 64;
	connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto_pack(&(ptr_struct->counter_set), ptr_buff + offset/8);

}

void connectib_mpcnt_reg_unpack(struct connectib_mpcnt_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->grp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 8;
	ptr_struct->pcie_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->clr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto_unpack(&(ptr_struct->counter_set), ptr_buff + offset/8);

}

void connectib_mpcnt_reg_print(const struct connectib_mpcnt_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mpcnt_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grp                  : " UH_FMT "\n", ptr_struct->grp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_index           : " UH_FMT "\n", ptr_struct->pcie_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "clr                  : " UH_FMT "\n", ptr_struct->clr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "counter_set:\n");
	connectib_pcie_perf_counters_pcie_lanes_counters_pcie_timers_states_auto_print(&(ptr_struct->counter_set), file, indent_level + 1);

}

int connectib_mpcnt_reg_size(void)
{
	 return 256;
}

void connectib_mpcnt_reg_dump(const struct connectib_mpcnt_reg *ptr_struct, FILE* file)
{
	connectib_mpcnt_reg_print(ptr_struct, file, 0);
}

void connectib_mcia_reg_pack(const struct connectib_mcia_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->status);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->l);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->device_address);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->page_number);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->i2c_device_address);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->size);

	offset = 66;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->passwd_cap);

	offset = 65;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->passwd_clr);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->passwd_v);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->password);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword[i]);
	}

}

void connectib_mcia_reg_unpack(struct connectib_mcia_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->module = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->l = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->device_address = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->page_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->i2c_device_address = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 80;
	ptr_struct->size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 66;
	ptr_struct->passwd_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 65;
	ptr_struct->passwd_clr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->passwd_v = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->password = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 512, 1);
	ptr_struct->dword[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_mcia_reg_print(const struct connectib_mcia_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mcia_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " UH_FMT "\n", ptr_struct->status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module               : " UH_FMT "\n", ptr_struct->module);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l                    : " UH_FMT "\n", ptr_struct->l);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_address       : " UH_FMT "\n", ptr_struct->device_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "page_number          : " UH_FMT "\n", ptr_struct->page_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_device_address   : " UH_FMT "\n", ptr_struct->i2c_device_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "passwd_cap           : " UH_FMT "\n", ptr_struct->passwd_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "passwd_clr           : " UH_FMT "\n", ptr_struct->passwd_clr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "passwd_v             : " UH_FMT "\n", ptr_struct->passwd_v);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "password             : " U32H_FMT "\n", ptr_struct->password);

	for (i = 0; i < 12; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_%03d           : " U32H_FMT "\n", i, ptr_struct->dword[i]);
	}

}

int connectib_mcia_reg_size(void)
{
	 return 64;
}

void connectib_mcia_reg_dump(const struct connectib_mcia_reg *ptr_struct, FILE* file)
{
	connectib_mcia_reg_print(ptr_struct, file, 0);
}

void connectib_ppad_reg_pack(const struct connectib_ppad_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->mac_47_32);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->single_base_mac);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mac_31_0);

}

void connectib_ppad_reg_unpack(struct connectib_ppad_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->mac_47_32 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->single_base_mac = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->mac_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_ppad_reg_print(const struct connectib_ppad_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ppad_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mac_47_32            : " UH_FMT "\n", ptr_struct->mac_47_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "single_base_mac      : " UH_FMT "\n", ptr_struct->single_base_mac);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mac_31_0             : " U32H_FMT "\n", ptr_struct->mac_31_0);

}

int connectib_ppad_reg_size(void)
{
	 return 16;
}

void connectib_ppad_reg_dump(const struct connectib_ppad_reg *ptr_struct, FILE* file)
{
	connectib_ppad_reg_print(ptr_struct, file, 0);
}

void connectib_pvlc_reg_pack(const struct connectib_pvlc_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl_hw_cap);

	offset = 92;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl_admin);

	offset = 124;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl_operational);

}

void connectib_pvlc_reg_unpack(struct connectib_pvlc_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 60;
	ptr_struct->vl_hw_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 92;
	ptr_struct->vl_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 124;
	ptr_struct->vl_operational = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_pvlc_reg_print(const struct connectib_pvlc_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pvlc_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl_hw_cap            : " UH_FMT "\n", ptr_struct->vl_hw_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl_admin             : " UH_FMT "\n", ptr_struct->vl_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl_operational       : " UH_FMT "\n", ptr_struct->vl_operational);

}

int connectib_pvlc_reg_size(void)
{
	 return 16;
}

void connectib_pvlc_reg_dump(const struct connectib_pvlc_reg *ptr_struct, FILE* file)
{
	connectib_pvlc_reg_print(ptr_struct, file, 0);
}

void connectib_ppcnt_reg_pack(const struct connectib_ppcnt_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->grp);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->pnat);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->swid);

	offset = 59;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->prio_tc);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->clr);

	offset = 64;
	connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto_pack(&(ptr_struct->counter_set), ptr_buff + offset/8);

}

void connectib_ppcnt_reg_unpack(struct connectib_ppcnt_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->grp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 16;
	ptr_struct->pnat = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->swid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 59;
	ptr_struct->prio_tc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 32;
	ptr_struct->clr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto_unpack(&(ptr_struct->counter_set), ptr_buff + offset/8);

}

void connectib_ppcnt_reg_print(const struct connectib_ppcnt_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ppcnt_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grp                  : " UH_FMT "\n", ptr_struct->grp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pnat                 : " UH_FMT "\n", ptr_struct->pnat);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swid                 : " UH_FMT "\n", ptr_struct->swid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prio_tc              : " UH_FMT "\n", ptr_struct->prio_tc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "clr                  : " UH_FMT "\n", ptr_struct->clr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "counter_set:\n");
	connectib_eth_802_3_cntrs_grp_data_layout_eth_2863_cntrs_grp_data_layout_eth_2819_cntrs_grp_data_layout_eth_3635_cntrs_grp_data_layout_eth_extended_cntrs_grp_data_layout_eth_per_prio_grp_data_layout_eth_per_traffic_grp_data_layout_phys_layer_cntrs_auto_print(&(ptr_struct->counter_set), file, indent_level + 1);

}

int connectib_ppcnt_reg_size(void)
{
	 return 256;
}

void connectib_ppcnt_reg_dump(const struct connectib_ppcnt_reg *ptr_struct, FILE* file)
{
	connectib_ppcnt_reg_print(ptr_struct, file, 0);
}

void connectib_ptys_reg_pack(const struct connectib_ptys_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->proto_mask);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_tx_aba_param);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->an_disable_cap);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->an_disable_admin);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved_high);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->data_rate_oper);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->an_status);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_proto_capability);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->eth_proto_capability);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ib_proto_capability);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ib_link_width_capability);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_proto_admin);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->eth_proto_admin);

	offset = 240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ib_proto_admin);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ib_link_width_admin);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fc_proto_oper);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->eth_proto_oper);

	offset = 336;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ib_proto_oper);

	offset = 320;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ib_link_width_oper);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->eth_proto_lp_advertise);

}

void connectib_ptys_reg_unpack(struct connectib_ptys_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	ptr_struct->proto_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 7;
	ptr_struct->force_tx_aba_param = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->an_disable_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->an_disable_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->reserved_high = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->data_rate_oper = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->an_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->fc_proto_capability = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->eth_proto_capability = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 144;
	ptr_struct->ib_proto_capability = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 128;
	ptr_struct->ib_link_width_capability = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 160;
	ptr_struct->fc_proto_admin = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->eth_proto_admin = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 240;
	ptr_struct->ib_proto_admin = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 224;
	ptr_struct->ib_link_width_admin = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 256;
	ptr_struct->fc_proto_oper = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->eth_proto_oper = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 336;
	ptr_struct->ib_proto_oper = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 320;
	ptr_struct->ib_link_width_oper = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 384;
	ptr_struct->eth_proto_lp_advertise = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_ptys_reg_print(const struct connectib_ptys_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ptys_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "proto_mask           : %s (" UH_FMT ")\n", (ptr_struct->proto_mask == 1 ? ("InfiniBand") : ((ptr_struct->proto_mask == 2 ? ("FC") : ((ptr_struct->proto_mask == 4 ? ("Ethernet") : ("unknown")))))), ptr_struct->proto_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_tx_aba_param   : " UH_FMT "\n", ptr_struct->force_tx_aba_param);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "an_disable_cap       : " UH_FMT "\n", ptr_struct->an_disable_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "an_disable_admin     : " UH_FMT "\n", ptr_struct->an_disable_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved_high        : " UH_FMT "\n", ptr_struct->reserved_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_rate_oper       : " UH_FMT "\n", ptr_struct->data_rate_oper);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "an_status            : " UH_FMT "\n", ptr_struct->an_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_proto_capability  : " U32H_FMT "\n", ptr_struct->fc_proto_capability);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_proto_capability : " U32H_FMT "\n", ptr_struct->eth_proto_capability);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_proto_capability  : " UH_FMT "\n", ptr_struct->ib_proto_capability);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_link_width_capability : " UH_FMT "\n", ptr_struct->ib_link_width_capability);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_proto_admin       : " U32H_FMT "\n", ptr_struct->fc_proto_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_proto_admin      : " U32H_FMT "\n", ptr_struct->eth_proto_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_proto_admin       : " UH_FMT "\n", ptr_struct->ib_proto_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_link_width_admin  : " UH_FMT "\n", ptr_struct->ib_link_width_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_proto_oper        : " U32H_FMT "\n", ptr_struct->fc_proto_oper);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_proto_oper       : " U32H_FMT "\n", ptr_struct->eth_proto_oper);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_proto_oper        : " UH_FMT "\n", ptr_struct->ib_proto_oper);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_link_width_oper   : " UH_FMT "\n", ptr_struct->ib_link_width_oper);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_proto_lp_advertise : " U32H_FMT "\n", ptr_struct->eth_proto_lp_advertise);

}

int connectib_ptys_reg_size(void)
{
	 return 64;
}

void connectib_ptys_reg_dump(const struct connectib_ptys_reg *ptr_struct, FILE* file)
{
	connectib_ptys_reg_print(ptr_struct, file, 0);
}

void connectib_pmlp_reg_pack(const struct connectib_pmlp_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->width);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rxtx);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lane0_module_mapping);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lane1_module_mapping);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lane2_module_mapping);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lane3_module_mapping);

}

void connectib_pmlp_reg_unpack(struct connectib_pmlp_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->width = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->rxtx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->lane0_module_mapping = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->lane1_module_mapping = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->lane2_module_mapping = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->lane3_module_mapping = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_pmlp_reg_print(const struct connectib_pmlp_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pmlp_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "width                : " UH_FMT "\n", ptr_struct->width);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rxtx                 : " UH_FMT "\n", ptr_struct->rxtx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane0_module_mapping : " U32H_FMT "\n", ptr_struct->lane0_module_mapping);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane1_module_mapping : " U32H_FMT "\n", ptr_struct->lane1_module_mapping);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane2_module_mapping : " U32H_FMT "\n", ptr_struct->lane2_module_mapping);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane3_module_mapping : " U32H_FMT "\n", ptr_struct->lane3_module_mapping);

}

int connectib_pmlp_reg_size(void)
{
	 return 64;
}

void connectib_pmlp_reg_dump(const struct connectib_pmlp_reg *ptr_struct, FILE* file)
{
	connectib_pmlp_reg_print(ptr_struct, file, 0);
}

void connectib_register_mfbe_pack(const struct connectib_register_mfbe *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->fs);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->p);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bulk_64kb_erase);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bulk_32kb_erase);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->address);

}

void connectib_register_mfbe_unpack(struct connectib_register_mfbe *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->fs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 23;
	ptr_struct->p = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->bulk_64kb_erase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->bulk_32kb_erase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->address = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_register_mfbe_print(const struct connectib_register_mfbe *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_mfbe ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fs                   : " UH_FMT "\n", ptr_struct->fs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "p                    : " UH_FMT "\n", ptr_struct->p);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bulk_64kb_erase      : " UH_FMT "\n", ptr_struct->bulk_64kb_erase);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bulk_32kb_erase      : " UH_FMT "\n", ptr_struct->bulk_32kb_erase);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address              : " UH_FMT "\n", ptr_struct->address);

}

int connectib_register_mfbe_size(void)
{
	 return 12;
}

void connectib_register_mfbe_dump(const struct connectib_register_mfbe *ptr_struct, FILE* file)
{
	connectib_register_mfbe_print(ptr_struct, file, 0);
}

void connectib_register_mfba_pack(const struct connectib_register_mfba *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->fs);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->p);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->size);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->address);

	offset = 96;
	connectib_register_access_data_pack(&(ptr_struct->data), ptr_buff + offset/8);

}

void connectib_register_mfba_unpack(struct connectib_register_mfba *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->fs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 23;
	ptr_struct->p = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 55;
	ptr_struct->size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 72;
	ptr_struct->address = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	connectib_register_access_data_unpack(&(ptr_struct->data), ptr_buff + offset/8);

}

void connectib_register_mfba_print(const struct connectib_register_mfba *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_mfba ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fs                   : " UH_FMT "\n", ptr_struct->fs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "p                    : " UH_FMT "\n", ptr_struct->p);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address              : " UH_FMT "\n", ptr_struct->address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data:\n");
	connectib_register_access_data_print(&(ptr_struct->data), file, indent_level + 1);

}

int connectib_register_mfba_size(void)
{
	 return 44;
}

void connectib_register_mfba_dump(const struct connectib_register_mfba *ptr_struct, FILE* file)
{
	connectib_register_mfba_print(ptr_struct, file, 0);
}

void connectib_register_mfpa_pack(const struct connectib_register_mfpa *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->fs);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->p);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->boot_address);

	offset = 156;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->flash_num);

	offset = 130;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bulk_64kb_erase_en);

	offset = 129;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bulk_32kb_erase_en);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wip);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->jedec_id);

	offset = 214;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->sector_size);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->block_allignment);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->capability_mask);

}

void connectib_register_mfpa_unpack(struct connectib_register_mfpa *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->fs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 23;
	ptr_struct->p = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 40;
	ptr_struct->boot_address = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 156;
	ptr_struct->flash_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 130;
	ptr_struct->bulk_64kb_erase_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 129;
	ptr_struct->bulk_32kb_erase_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 128;
	ptr_struct->wip = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->jedec_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 214;
	ptr_struct->sector_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 200;
	ptr_struct->block_allignment = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 224;
	ptr_struct->capability_mask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_register_mfpa_print(const struct connectib_register_mfpa *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_mfpa ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fs                   : " UH_FMT "\n", ptr_struct->fs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "p                    : " UH_FMT "\n", ptr_struct->p);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "boot_address         : " UH_FMT "\n", ptr_struct->boot_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_num            : " UH_FMT "\n", ptr_struct->flash_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bulk_64kb_erase_en   : " UH_FMT "\n", ptr_struct->bulk_64kb_erase_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bulk_32kb_erase_en   : " UH_FMT "\n", ptr_struct->bulk_32kb_erase_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wip                  : " UH_FMT "\n", ptr_struct->wip);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "jedec_id             : " UH_FMT "\n", ptr_struct->jedec_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sector_size          : " UH_FMT "\n", ptr_struct->sector_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "block_allignment     : " UH_FMT "\n", ptr_struct->block_allignment);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capability_mask      : " U32H_FMT "\n", ptr_struct->capability_mask);

}

int connectib_register_mfpa_size(void)
{
	 return 32;
}

void connectib_register_mfpa_dump(const struct connectib_register_mfpa *ptr_struct, FILE* file)
{
	connectib_register_mfpa_print(ptr_struct, file, 0);
}

void connectib_db_threshold_register_layout_pack(const struct connectib_db_threshold_register_layout *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->threshold);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->size);

}

void connectib_db_threshold_register_layout_unpack(struct connectib_db_threshold_register_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->threshold = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	ptr_struct->size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_db_threshold_register_layout_print(const struct connectib_db_threshold_register_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_db_threshold_register_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "threshold            : " UH_FMT "\n", ptr_struct->threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

}

int connectib_db_threshold_register_layout_size(void)
{
	 return 16;
}

void connectib_db_threshold_register_layout_dump(const struct connectib_db_threshold_register_layout *ptr_struct, FILE* file)
{
	connectib_db_threshold_register_layout_print(ptr_struct, file, 0);
}

void connectib_register_pplr_pack(const struct connectib_register_pplr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->exl);

}

void connectib_register_pplr_unpack(struct connectib_register_pplr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 61;
	ptr_struct->exl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_register_pplr_print(const struct connectib_register_pplr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_pplr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "exl                  : " UH_FMT "\n", ptr_struct->exl);

}

int connectib_register_pplr_size(void)
{
	 return 16;
}

void connectib_register_pplr_dump(const struct connectib_register_pplr *ptr_struct, FILE* file)
{
	connectib_register_pplr_print(ptr_struct, file, 0);
}

void connectib_register_host_endianness_pack(const struct connectib_register_host_endianness *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->he);

}

void connectib_register_host_endianness_unpack(struct connectib_register_host_endianness *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->he = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_register_host_endianness_print(const struct connectib_register_host_endianness *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_host_endianness ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "he                   : %s (" UH_FMT ")\n", (ptr_struct->he == 0 ? ("LITTLE_ENDIAN") : ((ptr_struct->he == 1 ? ("BIG_ENDIAN") : ("unknown")))), ptr_struct->he);

}

int connectib_register_host_endianness_size(void)
{
	 return 16;
}

void connectib_register_host_endianness_dump(const struct connectib_register_host_endianness *ptr_struct, FILE* file)
{
	connectib_register_host_endianness_print(ptr_struct, file, 0);
}

void connectib_register_fp_sl_map_pack(const struct connectib_register_fp_sl_map *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->fp_sl_indication);

}

void connectib_register_fp_sl_map_unpack(struct connectib_register_fp_sl_map *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->fp_sl_indication = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_register_fp_sl_map_print(const struct connectib_register_fp_sl_map *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_fp_sl_map ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fp_sl_indication     : " UH_FMT "\n", ptr_struct->fp_sl_indication);

}

int connectib_register_fp_sl_map_size(void)
{
	 return 16;
}

void connectib_register_fp_sl_map_dump(const struct connectib_register_fp_sl_map *ptr_struct, FILE* file)
{
	connectib_register_fp_sl_map_print(ptr_struct, file, 0);
}

void connectib_register_power_settings_pack(const struct connectib_register_power_settings *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->power_settings_level);

}

void connectib_register_power_settings_unpack(struct connectib_register_power_settings *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->power_settings_level = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_register_power_settings_print(const struct connectib_register_power_settings *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_power_settings ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "power_settings_level : " UH_FMT "\n", ptr_struct->power_settings_level);

}

int connectib_register_power_settings_size(void)
{
	 return 16;
}

void connectib_register_power_settings_dump(const struct connectib_register_power_settings *ptr_struct, FILE* file)
{
	connectib_register_power_settings_print(ptr_struct, file, 0);
}

void connectib_register_loopback_control_pack(const struct connectib_register_loopback_control *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->no_lb);

}

void connectib_register_loopback_control_unpack(struct connectib_register_loopback_control *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->no_lb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_register_loopback_control_print(const struct connectib_register_loopback_control *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_loopback_control ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_lb                : " UH_FMT "\n", ptr_struct->no_lb);

}

int connectib_register_loopback_control_size(void)
{
	 return 16;
}

void connectib_register_loopback_control_dump(const struct connectib_register_loopback_control *ptr_struct, FILE* file)
{
	connectib_register_loopback_control_print(ptr_struct, file, 0);
}

void connectib_pfcc_reg_pack(const struct connectib_pfcc_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->shl_opr);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->shl_cap);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->pnat);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cap_remote_admin);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cap_local_admin);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->dcbx_operation_type);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->prio_mask_rx);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->prio_mask_tx);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ppan);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cbftx);

	offset = 87;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fctx_disabled);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pfctx);

	offset = 65;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aptx);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pptx);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cbfrx);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pfcrx);

	offset = 97;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aprx);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pprx);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->device_stall_critical_watermark);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->device_stall_minor_watermark);

}

void connectib_pfcc_reg_unpack(struct connectib_pfcc_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->shl_opr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->shl_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->pnat = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->cap_remote_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->cap_local_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->dcbx_operation_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 56;
	ptr_struct->prio_mask_rx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 40;
	ptr_struct->prio_mask_tx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->ppan = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 88;
	ptr_struct->cbftx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 87;
	ptr_struct->fctx_disabled = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->pfctx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 65;
	ptr_struct->aptx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->pptx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 120;
	ptr_struct->cbfrx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->pfcrx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 97;
	ptr_struct->aprx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->pprx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 144;
	ptr_struct->device_stall_critical_watermark = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 128;
	ptr_struct->device_stall_minor_watermark = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_pfcc_reg_print(const struct connectib_pfcc_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pfcc_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "shl_opr              : " UH_FMT "\n", ptr_struct->shl_opr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "shl_cap              : " UH_FMT "\n", ptr_struct->shl_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pnat                 : " UH_FMT "\n", ptr_struct->pnat);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_remote_admin     : " UH_FMT "\n", ptr_struct->cap_remote_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_local_admin      : " UH_FMT "\n", ptr_struct->cap_local_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcbx_operation_type  : " UH_FMT "\n", ptr_struct->dcbx_operation_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prio_mask_rx         : " UH_FMT "\n", ptr_struct->prio_mask_rx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prio_mask_tx         : " UH_FMT "\n", ptr_struct->prio_mask_tx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ppan                 : %s (" UH_FMT ")\n", (ptr_struct->ppan == 0 ? ("disabled") : ((ptr_struct->ppan == 1 ? ("enabled") : ("unknown")))), ptr_struct->ppan);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cbftx                : " UH_FMT "\n", ptr_struct->cbftx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fctx_disabled        : " UH_FMT "\n", ptr_struct->fctx_disabled);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pfctx                : " UH_FMT "\n", ptr_struct->pfctx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aptx                 : " UH_FMT "\n", ptr_struct->aptx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pptx                 : " UH_FMT "\n", ptr_struct->pptx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cbfrx                : " UH_FMT "\n", ptr_struct->cbfrx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pfcrx                : " UH_FMT "\n", ptr_struct->pfcrx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aprx                 : " UH_FMT "\n", ptr_struct->aprx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pprx                 : " UH_FMT "\n", ptr_struct->pprx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_stall_critical_watermark : " UH_FMT "\n", ptr_struct->device_stall_critical_watermark);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_stall_minor_watermark : " UH_FMT "\n", ptr_struct->device_stall_minor_watermark);

}

int connectib_pfcc_reg_size(void)
{
	 return 32;
}

void connectib_pfcc_reg_dump(const struct connectib_pfcc_reg *ptr_struct, FILE* file)
{
	connectib_pfcc_reg_print(ptr_struct, file, 0);
}

void connectib_access_register_unit_port_capabilities_pack(const struct connectib_access_register_unit_port_capabilities *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->reserved_1);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->reserved_0);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_capabilities_127_96);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_capabilities_95_64);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_capabilities_63_32);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_capabilities_31_0);

}

void connectib_access_register_unit_port_capabilities_unpack(struct connectib_access_register_unit_port_capabilities *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->reserved_1 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->reserved_0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->port_capabilities_127_96 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->port_capabilities_95_64 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->port_capabilities_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->port_capabilities_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_access_register_unit_port_capabilities_print(const struct connectib_access_register_unit_port_capabilities *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_access_register_unit_port_capabilities ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved_1           : " UH_FMT "\n", ptr_struct->reserved_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved_0           : " UH_FMT "\n", ptr_struct->reserved_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_capabilities_127_96 : " U32H_FMT "\n", ptr_struct->port_capabilities_127_96);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_capabilities_95_64 : " U32H_FMT "\n", ptr_struct->port_capabilities_95_64);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_capabilities_63_32 : " U32H_FMT "\n", ptr_struct->port_capabilities_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_capabilities_31_0 : " U32H_FMT "\n", ptr_struct->port_capabilities_31_0);

}

int connectib_access_register_unit_port_capabilities_size(void)
{
	 return 20;
}

void connectib_access_register_unit_port_capabilities_dump(const struct connectib_access_register_unit_port_capabilities *ptr_struct, FILE* file)
{
	connectib_access_register_unit_port_capabilities_print(ptr_struct, file, 0);
}

void connectib_register_unit_perf_counters_global_ctrl_pack(const struct connectib_register_unit_perf_counters_global_ctrl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cmd);

}

void connectib_register_unit_perf_counters_global_ctrl_unpack(struct connectib_register_unit_perf_counters_global_ctrl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->cmd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_register_unit_perf_counters_global_ctrl_print(const struct connectib_register_unit_perf_counters_global_ctrl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_unit_perf_counters_global_ctrl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmd                  : " UH_FMT "\n", ptr_struct->cmd);

}

int connectib_register_unit_perf_counters_global_ctrl_size(void)
{
	 return 16;
}

void connectib_register_unit_perf_counters_global_ctrl_dump(const struct connectib_register_unit_perf_counters_global_ctrl *ptr_struct, FILE* file)
{
	connectib_register_unit_perf_counters_global_ctrl_print(ptr_struct, file, 0);
}

void connectib_access_register_unit_perf_counters_data_pack(const struct connectib_access_register_unit_perf_counters_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	connectib_access_register_unit_perf_counters_data_line_pack(&(ptr_struct->counter[i]), ptr_buff + offset/8);
	}

}

void connectib_access_register_unit_perf_counters_data_unpack(struct connectib_access_register_unit_perf_counters_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	connectib_access_register_unit_perf_counters_data_line_unpack(&(ptr_struct->counter[i]), ptr_buff + offset/8);
	}

}

void connectib_access_register_unit_perf_counters_data_print(const struct connectib_access_register_unit_perf_counters_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_access_register_unit_perf_counters_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "counter_%03d:\n", i);
	connectib_access_register_unit_perf_counters_data_line_print(&(ptr_struct->counter[i]), file, indent_level + 1);
	}

}

int connectib_access_register_unit_perf_counters_data_size(void)
{
	 return 32;
}

void connectib_access_register_unit_perf_counters_data_dump(const struct connectib_access_register_unit_perf_counters_data *ptr_struct, FILE* file)
{
	connectib_access_register_unit_perf_counters_data_print(ptr_struct, file, 0);
}

void connectib_access_register_unit_perf_counters_cfg_pack(const struct connectib_access_register_unit_perf_counters_cfg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	connectib_access_register_unit_perf_counters_cfg_line_pack(&(ptr_struct->counter[i]), ptr_buff + offset/8);
	}

}

void connectib_access_register_unit_perf_counters_cfg_unpack(struct connectib_access_register_unit_perf_counters_cfg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	connectib_access_register_unit_perf_counters_cfg_line_unpack(&(ptr_struct->counter[i]), ptr_buff + offset/8);
	}

}

void connectib_access_register_unit_perf_counters_cfg_print(const struct connectib_access_register_unit_perf_counters_cfg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_access_register_unit_perf_counters_cfg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "counter_%03d:\n", i);
	connectib_access_register_unit_perf_counters_cfg_line_print(&(ptr_struct->counter[i]), file, indent_level + 1);
	}

}

int connectib_access_register_unit_perf_counters_cfg_size(void)
{
	 return 32;
}

void connectib_access_register_unit_perf_counters_cfg_dump(const struct connectib_access_register_unit_perf_counters_cfg *ptr_struct, FILE* file)
{
	connectib_access_register_unit_perf_counters_cfg_print(ptr_struct, file, 0);
}

void connectib_register_set_node_pack(const struct connectib_register_set_node *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_node_description_pack(&(ptr_struct->node_description), ptr_buff + offset/8);

}

void connectib_register_set_node_unpack(struct connectib_register_set_node *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_node_description_unpack(&(ptr_struct->node_description), ptr_buff + offset/8);

}

void connectib_register_set_node_print(const struct connectib_register_set_node *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_register_set_node ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "node_description:\n");
	connectib_node_description_print(&(ptr_struct->node_description), file, indent_level + 1);

}

int connectib_register_set_node_size(void)
{
	 return 64;
}

void connectib_register_set_node_dump(const struct connectib_register_set_node *ptr_struct, FILE* file)
{
	connectib_register_set_node_print(ptr_struct, file, 0);
}

void connectib_dcbx_app_reg_pack(const struct connectib_dcbx_app_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_number);

	offset = 58;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->num_app_prio);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 160, 1);
	connectib_application_prio_entry_pack(&(ptr_struct->app_prio[i]), ptr_buff + offset/8);
	}

}

void connectib_dcbx_app_reg_unpack(struct connectib_dcbx_app_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->port_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 58;
	ptr_struct->num_app_prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	for (i = 0; i < 1; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 160, 1);
	connectib_application_prio_entry_unpack(&(ptr_struct->app_prio[i]), ptr_buff + offset/8);
	}

}

void connectib_dcbx_app_reg_print(const struct connectib_dcbx_app_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dcbx_app_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number          : " UH_FMT "\n", ptr_struct->port_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_app_prio         : " UH_FMT "\n", ptr_struct->num_app_prio);

	for (i = 0; i < 1; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "app_prio_%03d:\n", i);
	connectib_application_prio_entry_print(&(ptr_struct->app_prio[i]), file, indent_level + 1);
	}

}

int connectib_dcbx_app_reg_size(void)
{
	 return 20;
}

void connectib_dcbx_app_reg_dump(const struct connectib_dcbx_app_reg *ptr_struct, FILE* file)
{
	connectib_dcbx_app_reg_print(ptr_struct, file, 0);
}

void connectib_dcbx_param_reg_pack(const struct connectib_dcbx_param_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->max_application_table_size);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_number);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dcbx_standby_cap);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dcbx_ieee_cap);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dcbx_cee_cap);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->version_admin);

	offset = 53;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->version_oper);

	offset = 92;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_tc_admin);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_tc_oper);

	offset = 76;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pfc_cap_admin);

	offset = 68;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pfc_cap_oper);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->willing_admin);

	offset = 124;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->remote_num_of_tc);

	offset = 100;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->remote_pfc_cap);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->remote_willing);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->error);

}

void connectib_dcbx_param_reg_unpack(struct connectib_dcbx_param_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->max_application_table_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 8;
	ptr_struct->port_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 2;
	ptr_struct->dcbx_standby_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->dcbx_ieee_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->dcbx_cee_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 61;
	ptr_struct->version_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 53;
	ptr_struct->version_oper = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 92;
	ptr_struct->num_of_tc_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 84;
	ptr_struct->num_of_tc_oper = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 76;
	ptr_struct->pfc_cap_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 68;
	ptr_struct->pfc_cap_oper = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->willing_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 124;
	ptr_struct->remote_num_of_tc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 100;
	ptr_struct->remote_pfc_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->remote_willing = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 152;
	ptr_struct->error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_dcbx_param_reg_print(const struct connectib_dcbx_param_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dcbx_param_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_application_table_size : " UH_FMT "\n", ptr_struct->max_application_table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number          : " UH_FMT "\n", ptr_struct->port_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcbx_standby_cap     : " UH_FMT "\n", ptr_struct->dcbx_standby_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcbx_ieee_cap        : " UH_FMT "\n", ptr_struct->dcbx_ieee_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcbx_cee_cap         : " UH_FMT "\n", ptr_struct->dcbx_cee_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version_admin        : %s (" UH_FMT ")\n", (ptr_struct->version_admin == 0 ? ("disabled") : ((ptr_struct->version_admin == 1 ? ("CEE") : ((ptr_struct->version_admin == 2 ? ("IEEE") : ((ptr_struct->version_admin == 3 ? ("CEE_and_IEEE") : ("unknown")))))))), ptr_struct->version_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version_oper         : %s (" UH_FMT ")\n", (ptr_struct->version_oper == 0 ? ("disabled") : ((ptr_struct->version_oper == 1 ? ("CEE_enabled") : ((ptr_struct->version_oper == 2 ? ("IEEE_enabled") : ("unknown")))))), ptr_struct->version_oper);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_tc_admin      : " UH_FMT "\n", ptr_struct->num_of_tc_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_tc_oper       : " UH_FMT "\n", ptr_struct->num_of_tc_oper);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pfc_cap_admin        : " UH_FMT "\n", ptr_struct->pfc_cap_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pfc_cap_oper         : " UH_FMT "\n", ptr_struct->pfc_cap_oper);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "willing_admin        : %s (" UH_FMT ")\n", (ptr_struct->willing_admin == 0 ? ("non_willing") : ((ptr_struct->willing_admin == 1 ? ("willing") : ("unknown")))), ptr_struct->willing_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "remote_num_of_tc     : " UH_FMT "\n", ptr_struct->remote_num_of_tc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "remote_pfc_cap       : " UH_FMT "\n", ptr_struct->remote_pfc_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "remote_willing       : " UH_FMT "\n", ptr_struct->remote_willing);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error                : " UH_FMT "\n", ptr_struct->error);

}

int connectib_dcbx_param_reg_size(void)
{
	 return 64;
}

void connectib_dcbx_param_reg_dump(const struct connectib_dcbx_param_reg *ptr_struct, FILE* file)
{
	connectib_dcbx_param_reg_print(ptr_struct, file, 0);
}

void connectib_mlcr_pack(const struct connectib_mlcr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->led_type);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cap_local_or_uid_only);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->beacon_duration);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->beacon_remain);

}

void connectib_mlcr_unpack(struct connectib_mlcr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->led_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 27;
	ptr_struct->cap_local_or_uid_only = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->beacon_duration = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 80;
	ptr_struct->beacon_remain = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_mlcr_print(const struct connectib_mlcr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mlcr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "led_type             : " UH_FMT "\n", ptr_struct->led_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_local_or_uid_only : " UH_FMT "\n", ptr_struct->cap_local_or_uid_only);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "beacon_duration      : " UH_FMT "\n", ptr_struct->beacon_duration);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "beacon_remain        : " UH_FMT "\n", ptr_struct->beacon_remain);

}

int connectib_mlcr_size(void)
{
	 return 12;
}

void connectib_mlcr_dump(const struct connectib_mlcr *ptr_struct, FILE* file)
{
	connectib_mlcr_print(ptr_struct, file, 0);
}

void connectib_qhll_pack(const struct connectib_qhll *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 59;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->hll_time);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hll_mask);

	offset = 93;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->stall_cnt);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->stall_en);

}

void connectib_qhll_unpack(struct connectib_qhll *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 59;
	ptr_struct->hll_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 32;
	ptr_struct->hll_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 93;
	ptr_struct->stall_cnt = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 64;
	ptr_struct->stall_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_qhll_print(const struct connectib_qhll *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_qhll ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hll_time             : " UH_FMT "\n", ptr_struct->hll_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hll_mask             : " UH_FMT "\n", ptr_struct->hll_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "stall_cnt            : " UH_FMT "\n", ptr_struct->stall_cnt);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "stall_en             : " UH_FMT "\n", ptr_struct->stall_en);

}

int connectib_qhll_size(void)
{
	 return 12;
}

void connectib_qhll_dump(const struct connectib_qhll *ptr_struct, FILE* file)
{
	connectib_qhll_print(ptr_struct, file, 0);
}

void connectib_qtct_reg_pack(const struct connectib_qtct_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->prio);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_number);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cap_remote_admin);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cap_local_admin);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->operation_type);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->tclass);

}

void connectib_qtct_reg_unpack(struct connectib_qtct_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	ptr_struct->prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 8;
	ptr_struct->port_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->cap_remote_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->cap_local_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->operation_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 61;
	ptr_struct->tclass = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_qtct_reg_print(const struct connectib_qtct_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_qtct_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prio                 : " UH_FMT "\n", ptr_struct->prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number          : " UH_FMT "\n", ptr_struct->port_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_remote_admin     : " UH_FMT "\n", ptr_struct->cap_remote_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_local_admin      : " UH_FMT "\n", ptr_struct->cap_local_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "operation_type       : %s (" UH_FMT ")\n", (ptr_struct->operation_type == 0 ? ("local_operative_parameters") : ((ptr_struct->operation_type == 1 ? ("local_admin_parameters") : ((ptr_struct->operation_type == 2 ? ("remote_admin_parameters") : ("unknown")))))), ptr_struct->operation_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tclass               : " UH_FMT "\n", ptr_struct->tclass);

}

int connectib_qtct_reg_size(void)
{
	 return 8;
}

void connectib_qtct_reg_dump(const struct connectib_qtct_reg *ptr_struct, FILE* file)
{
	connectib_qtct_reg_print(ptr_struct, file, 0);
}

void connectib_qpdp_reg_pack(const struct connectib_qpdp_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_number);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->pprio);

}

void connectib_qpdp_reg_unpack(struct connectib_qpdp_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->port_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 61;
	ptr_struct->pprio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_qpdp_reg_print(const struct connectib_qpdp_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_qpdp_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number          : " UH_FMT "\n", ptr_struct->port_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pprio                : " UH_FMT "\n", ptr_struct->pprio);

}

int connectib_qpdp_reg_size(void)
{
	 return 8;
}

void connectib_qpdp_reg_dump(const struct connectib_qpdp_reg *ptr_struct, FILE* file)
{
	connectib_qpdp_reg_print(ptr_struct, file, 0);
}

void connectib_qetcr_reg_pack(const struct connectib_qetcr_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_number);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cap_remote_admin);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cap_local_admin);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->operation_type);

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(64, 64, i, 640, 1);
	connectib_tc_qos_configuration_pack(&(ptr_struct->tc[i]), ptr_buff + offset/8);
	}

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->global_configuration);

}

void connectib_qetcr_reg_unpack(struct connectib_qetcr_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->port_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->cap_remote_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->cap_local_admin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->operation_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(64, 64, i, 640, 1);
	connectib_tc_qos_configuration_unpack(&(ptr_struct->tc[i]), ptr_buff + offset/8);
	}

	offset = 576;
	ptr_struct->global_configuration = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_qetcr_reg_print(const struct connectib_qetcr_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_qetcr_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number          : " UH_FMT "\n", ptr_struct->port_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_remote_admin     : " UH_FMT "\n", ptr_struct->cap_remote_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_local_admin      : " UH_FMT "\n", ptr_struct->cap_local_admin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "operation_type       : " UH_FMT "\n", ptr_struct->operation_type);

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tc_%03d:\n", i);
	connectib_tc_qos_configuration_print(&(ptr_struct->tc[i]), file, indent_level + 1);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "global_configuration : " U64H_FMT "\n", ptr_struct->global_configuration);

}

int connectib_qetcr_reg_size(void)
{
	 return 80;
}

void connectib_qetcr_reg_dump(const struct connectib_qetcr_reg *ptr_struct, FILE* file)
{
	connectib_qetcr_reg_print(ptr_struct, file, 0);
}

void connectib_pcam_reg_pack(const struct connectib_pcam_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->access_reg_group);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->feature_group);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 640, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_access_reg_cap_mask[i]);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(320, 32, i, 640, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->feature_cap_mask[i]);
	}

}

void connectib_pcam_reg_unpack(struct connectib_pcam_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->access_reg_group = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->feature_group = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 640, 1);
	ptr_struct->port_access_reg_cap_mask[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(320, 32, i, 640, 1);
	ptr_struct->feature_cap_mask[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_pcam_reg_print(const struct connectib_pcam_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcam_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_reg_group     : " UH_FMT "\n", ptr_struct->access_reg_group);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "feature_group        : " UH_FMT "\n", ptr_struct->feature_group);

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_access_reg_cap_mask_%03d : " U32H_FMT "\n", i, ptr_struct->port_access_reg_cap_mask[i]);
	}

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "feature_cap_mask_%03d : " U32H_FMT "\n", i, ptr_struct->feature_cap_mask[i]);
	}

}

int connectib_pcam_reg_size(void)
{
	 return 80;
}

void connectib_pcam_reg_dump(const struct connectib_pcam_reg *ptr_struct, FILE* file)
{
	connectib_pcam_reg_print(ptr_struct, file, 0);
}

void connectib_mcam_reg_pack(const struct connectib_mcam_reg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->access_reg_group);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->feature_group);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 576, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mng_access_reg_cap_mask[i]);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(320, 32, i, 576, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mng_feature_cap_mask[i]);
	}

}

void connectib_mcam_reg_unpack(struct connectib_mcam_reg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->access_reg_group = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->feature_group = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 576, 1);
	ptr_struct->mng_access_reg_cap_mask[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(320, 32, i, 576, 1);
	ptr_struct->mng_feature_cap_mask[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_mcam_reg_print(const struct connectib_mcam_reg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mcam_reg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_reg_group     : " UH_FMT "\n", ptr_struct->access_reg_group);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "feature_group        : " UH_FMT "\n", ptr_struct->feature_group);

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mng_access_reg_cap_mask_%03d : " U32H_FMT "\n", i, ptr_struct->mng_access_reg_cap_mask[i]);
	}

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mng_feature_cap_mask_%03d : " U32H_FMT "\n", i, ptr_struct->mng_feature_cap_mask[i]);
	}

}

int connectib_mcam_reg_size(void)
{
	 return 72;
}

void connectib_mcam_reg_dump(const struct connectib_mcam_reg *ptr_struct, FILE* file)
{
	connectib_mcam_reg_print(ptr_struct, file, 0);
}

void connectib_ib_gid_pack(const struct connectib_ib_gid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword[i]);
	}

}

void connectib_ib_gid_unpack(struct connectib_ib_gid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	ptr_struct->dword[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_ib_gid_print(const struct connectib_ib_gid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ib_gid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_%03d           : " U32H_FMT "\n", i, ptr_struct->dword[i]);
	}

}

int connectib_ib_gid_size(void)
{
	 return 16;
}

void connectib_ib_gid_dump(const struct connectib_ib_gid *ptr_struct, FILE* file)
{
	connectib_ib_gid_print(ptr_struct, file, 0);
}

void connectib_dmfs_encap_meta_pack(const struct connectib_dmfs_encap_meta *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->header_type);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->encap_size);

}

void connectib_dmfs_encap_meta_unpack(struct connectib_dmfs_encap_meta *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 61;
	ptr_struct->header_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 48;
	ptr_struct->encap_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

}

void connectib_dmfs_encap_meta_print(const struct connectib_dmfs_encap_meta *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dmfs_encap_meta ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "header_type          : " UH_FMT "\n", ptr_struct->header_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encap_size           : " UH_FMT "\n", ptr_struct->encap_size);

}

int connectib_dmfs_encap_meta_size(void)
{
	 return 8;
}

void connectib_dmfs_encap_meta_dump(const struct connectib_dmfs_encap_meta *ptr_struct, FILE* file)
{
	connectib_dmfs_encap_meta_print(ptr_struct, file, 0);
}

void connectib_flow_counter_fw_ctx_pack(const struct connectib_flow_counter_fw_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->src_gvmi);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->ft_type);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ft_type_valid);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->mac);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->next_flow_counter);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->prev_flow_counter);

}

void connectib_flow_counter_fw_ctx_unpack(struct connectib_flow_counter_fw_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 48;
	ptr_struct->src_gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 33;
	ptr_struct->ft_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 32;
	ptr_struct->ft_type_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->mac = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 128;
	ptr_struct->next_flow_counter = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->prev_flow_counter = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_flow_counter_fw_ctx_print(const struct connectib_flow_counter_fw_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_flow_counter_fw_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "src_gvmi             : " UH_FMT "\n", ptr_struct->src_gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_type              : " UH_FMT "\n", ptr_struct->ft_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_type_valid        : " UH_FMT "\n", ptr_struct->ft_type_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mac                  : " U64H_FMT "\n", ptr_struct->mac);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_flow_counter    : " U32H_FMT "\n", ptr_struct->next_flow_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prev_flow_counter    : " U32H_FMT "\n", ptr_struct->prev_flow_counter);

}

int connectib_flow_counter_fw_ctx_size(void)
{
	 return 32;
}

void connectib_flow_counter_fw_ctx_dump(const struct connectib_flow_counter_fw_ctx *ptr_struct, FILE* file)
{
	connectib_flow_counter_fw_ctx_print(ptr_struct, file, 0);
}

void connectib_l2_table_entry_fw_ctx_pack(const struct connectib_l2_table_entry_fw_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mac_address_layout_pack(&(ptr_struct->mac), ptr_buff + offset/8);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->vlan);

	offset = 83;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vlan_valid);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hw_rx_ix);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hw_sx_ix);

}

void connectib_l2_table_entry_fw_ctx_unpack(struct connectib_l2_table_entry_fw_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mac_address_layout_unpack(&(ptr_struct->mac), ptr_buff + offset/8);

	offset = 84;
	ptr_struct->vlan = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 83;
	ptr_struct->vlan_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->hw_rx_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->hw_sx_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_l2_table_entry_fw_ctx_print(const struct connectib_l2_table_entry_fw_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_l2_table_entry_fw_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mac:\n");
	connectib_mac_address_layout_print(&(ptr_struct->mac), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vlan                 : " UH_FMT "\n", ptr_struct->vlan);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vlan_valid           : " UH_FMT "\n", ptr_struct->vlan_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_rx_ix             : " U32H_FMT "\n", ptr_struct->hw_rx_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_sx_ix             : " U32H_FMT "\n", ptr_struct->hw_sx_ix);

}

int connectib_l2_table_entry_fw_ctx_size(void)
{
	 return 32;
}

void connectib_l2_table_entry_fw_ctx_dump(const struct connectib_l2_table_entry_fw_ctx *ptr_struct, FILE* file)
{
	connectib_l2_table_entry_fw_ctx_print(ptr_struct, file, 0);
}

void connectib_dmfs_fte_meta_pack(const struct connectib_dmfs_fte_meta *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->external_dest_vport);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->internal_dest_vport);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->uc_match_exists);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mc_match_exists);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->src_port_ext_match_exists);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fte_use_iterator);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dest_iter_ix);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vst_dealloc_hash_ix);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vst_dealloc_ste_ix);

}

void connectib_dmfs_fte_meta_unpack(struct connectib_dmfs_fte_meta *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->external_dest_vport = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->internal_dest_vport = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->uc_match_exists = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->mc_match_exists = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->src_port_ext_match_exists = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->fte_use_iterator = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->dest_iter_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->vst_dealloc_hash_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->vst_dealloc_ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_dmfs_fte_meta_print(const struct connectib_dmfs_fte_meta *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dmfs_fte_meta ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "external_dest_vport  : " UH_FMT "\n", ptr_struct->external_dest_vport);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "internal_dest_vport  : " UH_FMT "\n", ptr_struct->internal_dest_vport);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uc_match_exists      : " UH_FMT "\n", ptr_struct->uc_match_exists);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mc_match_exists      : " UH_FMT "\n", ptr_struct->mc_match_exists);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "src_port_ext_match_exists : " UH_FMT "\n", ptr_struct->src_port_ext_match_exists);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fte_use_iterator     : " UH_FMT "\n", ptr_struct->fte_use_iterator);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dest_iter_ix         : " U32H_FMT "\n", ptr_struct->dest_iter_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vst_dealloc_hash_ix  : " U32H_FMT "\n", ptr_struct->vst_dealloc_hash_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vst_dealloc_ste_ix   : " U32H_FMT "\n", ptr_struct->vst_dealloc_ste_ix);

}

int connectib_dmfs_fte_meta_size(void)
{
	 return 64;
}

void connectib_dmfs_fte_meta_dump(const struct connectib_dmfs_fte_meta *ptr_struct, FILE* file)
{
	connectib_dmfs_fte_meta_print(ptr_struct, file, 0);
}

void connectib_dmfs_ft_meta_pack(const struct connectib_dmfs_ft_meta *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->table_type);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->log2_table_size);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->level);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ft_is_root);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->total_ref_count);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vport_num);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->prev_ft);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->next_ft);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->groups_list_head);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->groups_member_count);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ft_rx_head_ix);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ft_sx_head_ix);

	offset = 264;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->next_table_id);

	offset = 260;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->table_miss_mode);

	offset = 258;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lag_demux);

	offset = 257;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->decap_en);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->encap_en);

	offset = 304;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->fgs_in_rx_hw_count);

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->fgs_in_sx_hw_count);

}

void connectib_dmfs_ft_meta_unpack(struct connectib_dmfs_ft_meta *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->table_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->log2_table_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->level = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 1;
	ptr_struct->ft_is_root = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->total_ref_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->vport_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->prev_ft = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->next_ft = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->groups_list_head = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->groups_member_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->ft_rx_head_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->ft_sx_head_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 264;
	ptr_struct->next_table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 260;
	ptr_struct->table_miss_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 258;
	ptr_struct->lag_demux = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 257;
	ptr_struct->decap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 256;
	ptr_struct->encap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 304;
	ptr_struct->fgs_in_rx_hw_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 288;
	ptr_struct->fgs_in_sx_hw_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_dmfs_ft_meta_print(const struct connectib_dmfs_ft_meta *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dmfs_ft_meta ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_type           : " UH_FMT "\n", ptr_struct->table_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_table_size      : " UH_FMT "\n", ptr_struct->log2_table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "level                : " UH_FMT "\n", ptr_struct->level);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_is_root           : " UH_FMT "\n", ptr_struct->ft_is_root);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "total_ref_count      : " UH_FMT "\n", ptr_struct->total_ref_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_num            : " UH_FMT "\n", ptr_struct->vport_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prev_ft              : " U32H_FMT "\n", ptr_struct->prev_ft);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_ft              : " U32H_FMT "\n", ptr_struct->next_ft);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "groups_list_head     : " U32H_FMT "\n", ptr_struct->groups_list_head);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "groups_member_count  : " U32H_FMT "\n", ptr_struct->groups_member_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_rx_head_ix        : " U32H_FMT "\n", ptr_struct->ft_rx_head_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_sx_head_ix        : " U32H_FMT "\n", ptr_struct->ft_sx_head_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_table_id        : " UH_FMT "\n", ptr_struct->next_table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_miss_mode      : " UH_FMT "\n", ptr_struct->table_miss_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lag_demux            : " UH_FMT "\n", ptr_struct->lag_demux);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "decap_en             : " UH_FMT "\n", ptr_struct->decap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encap_en             : " UH_FMT "\n", ptr_struct->encap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fgs_in_rx_hw_count   : " UH_FMT "\n", ptr_struct->fgs_in_rx_hw_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fgs_in_sx_hw_count   : " UH_FMT "\n", ptr_struct->fgs_in_sx_hw_count);

}

int connectib_dmfs_ft_meta_size(void)
{
	 return 64;
}

void connectib_dmfs_ft_meta_dump(const struct connectib_dmfs_ft_meta *ptr_struct, FILE* file)
{
	connectib_dmfs_ft_meta_print(ptr_struct, file, 0);
}

void connectib_dmfs_fg_meta_pack(const struct connectib_dmfs_fg_meta *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->table_type);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->match_criteria_enable);

	offset = 87;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vlan_match_exist);

	offset = 86;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->in_rx_hw);

	offset = 85;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->in_sx_hw);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vport_num);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->start_flow_index);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->end_flow_index);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->prev_group_id);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->next_group_id);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vst_root_hash_meta_ix);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_root_hash_meta_ix);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sx_root_hash_meta_ix);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_end_ste_ix);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sx_end_ste_ix);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->unused_ftes_in_rx_count);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->unused_ftes_in_sx_count);

}

void connectib_dmfs_fg_meta_unpack(struct connectib_dmfs_fg_meta *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 56;
	ptr_struct->table_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 88;
	ptr_struct->match_criteria_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 87;
	ptr_struct->vlan_match_exist = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 86;
	ptr_struct->in_rx_hw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 85;
	ptr_struct->in_sx_hw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->vport_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->start_flow_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->end_flow_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->prev_group_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->next_group_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->vst_root_hash_meta_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->rx_root_hash_meta_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->sx_root_hash_meta_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->rx_end_ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->sx_end_ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->unused_ftes_in_rx_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->unused_ftes_in_sx_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_dmfs_fg_meta_print(const struct connectib_dmfs_fg_meta *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dmfs_fg_meta ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_type           : " UH_FMT "\n", ptr_struct->table_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "match_criteria_enable : " UH_FMT "\n", ptr_struct->match_criteria_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vlan_match_exist     : " UH_FMT "\n", ptr_struct->vlan_match_exist);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in_rx_hw             : " UH_FMT "\n", ptr_struct->in_rx_hw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in_sx_hw             : " UH_FMT "\n", ptr_struct->in_sx_hw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_num            : " UH_FMT "\n", ptr_struct->vport_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "start_flow_index     : " U32H_FMT "\n", ptr_struct->start_flow_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "end_flow_index       : " U32H_FMT "\n", ptr_struct->end_flow_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prev_group_id        : " U32H_FMT "\n", ptr_struct->prev_group_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_group_id        : " U32H_FMT "\n", ptr_struct->next_group_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vst_root_hash_meta_ix : " U32H_FMT "\n", ptr_struct->vst_root_hash_meta_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_root_hash_meta_ix : " U32H_FMT "\n", ptr_struct->rx_root_hash_meta_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_root_hash_meta_ix : " U32H_FMT "\n", ptr_struct->sx_root_hash_meta_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_end_ste_ix        : " U32H_FMT "\n", ptr_struct->rx_end_ste_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_end_ste_ix        : " U32H_FMT "\n", ptr_struct->sx_end_ste_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unused_ftes_in_rx_count : " U32H_FMT "\n", ptr_struct->unused_ftes_in_rx_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unused_ftes_in_sx_count : " U32H_FMT "\n", ptr_struct->unused_ftes_in_sx_count);

}

int connectib_dmfs_fg_meta_size(void)
{
	 return 64;
}

void connectib_dmfs_fg_meta_dump(const struct connectib_dmfs_fg_meta *ptr_struct, FILE* file)
{
	connectib_dmfs_fg_meta_print(ptr_struct, file, 0);
}

void connectib_mkey_dump_pack(const struct connectib_mkey_dump *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ready_4_dump);

}

void connectib_mkey_dump_unpack(struct connectib_mkey_dump *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->ready_4_dump = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_mkey_dump_print(const struct connectib_mkey_dump *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mkey_dump ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ready_4_dump         : " UH_FMT "\n", ptr_struct->ready_4_dump);

}

int connectib_mkey_dump_size(void)
{
	 return 4;
}

void connectib_mkey_dump_dump(const struct connectib_mkey_dump *ptr_struct, FILE* file)
{
	connectib_mkey_dump_print(ptr_struct, file, 0);
}

void connectib_fw_tis_pack(const struct connectib_fw_tis *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lag_tx_port_affinity);

	offset = 59;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->strict_lag_tx_port_affinity);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->prio);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->sqn_head);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->transport_domain);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->underlay_qpn);

}

void connectib_fw_tis_unpack(struct connectib_fw_tis *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 60;
	ptr_struct->lag_tx_port_affinity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 59;
	ptr_struct->strict_lag_tx_port_affinity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->sqn_head = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 104;
	ptr_struct->transport_domain = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 136;
	ptr_struct->underlay_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_fw_tis_print(const struct connectib_fw_tis *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_tis ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lag_tx_port_affinity : " UH_FMT "\n", ptr_struct->lag_tx_port_affinity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "strict_lag_tx_port_affinity : " UH_FMT "\n", ptr_struct->strict_lag_tx_port_affinity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prio                 : " UH_FMT "\n", ptr_struct->prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sqn_head             : " UH_FMT "\n", ptr_struct->sqn_head);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transport_domain     : " UH_FMT "\n", ptr_struct->transport_domain);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "underlay_qpn         : " UH_FMT "\n", ptr_struct->underlay_qpn);

}

int connectib_fw_tis_size(void)
{
	 return 64;
}

void connectib_fw_tis_dump(const struct connectib_fw_tis *ptr_struct, FILE* file)
{
	connectib_fw_tis_print(ptr_struct, file, 0);
}

void connectib_fw_tir_pack(const struct connectib_fw_tir *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->inline_rqn);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->disp_type);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->lro_max_ip_payload_size);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lro_enable_mask);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lro_timeout_period_usecs);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->rqt_number);

	offset = 100;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_hash_fn);

	offset = 99;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->tunneled_offload_en);

	offset = 97;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rx_hash_symmetric);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->steering_start_pipe);

	offset = 128;
	connectib_rx_hash_field_select_pack(&(ptr_struct->rx_hash_field_selector_outer), ptr_buff + offset/8);

	offset = 160;
	connectib_rx_hash_field_select_pack(&(ptr_struct->rx_hash_field_selector_inner), ptr_buff + offset/8);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->transport_domain);

	offset = 198;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->self_lb_block);

	offset = 384;
	connectib_tir_dmfs_pack(&(ptr_struct->dmfs), ptr_buff + offset/8);

}

void connectib_fw_tir_unpack(struct connectib_fw_tir *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 40;
	ptr_struct->inline_rqn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 36;
	ptr_struct->disp_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 88;
	ptr_struct->lro_max_ip_payload_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 84;
	ptr_struct->lro_enable_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->lro_timeout_period_usecs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 104;
	ptr_struct->rqt_number = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 100;
	ptr_struct->rx_hash_fn = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 99;
	ptr_struct->tunneled_offload_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 97;
	ptr_struct->rx_hash_symmetric = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->steering_start_pipe = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 128;
	connectib_rx_hash_field_select_unpack(&(ptr_struct->rx_hash_field_selector_outer), ptr_buff + offset/8);

	offset = 160;
	connectib_rx_hash_field_select_unpack(&(ptr_struct->rx_hash_field_selector_inner), ptr_buff + offset/8);

	offset = 200;
	ptr_struct->transport_domain = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 198;
	ptr_struct->self_lb_block = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 384;
	connectib_tir_dmfs_unpack(&(ptr_struct->dmfs), ptr_buff + offset/8);

}

void connectib_fw_tir_print(const struct connectib_fw_tir *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_tir ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inline_rqn           : " UH_FMT "\n", ptr_struct->inline_rqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disp_type            : " UH_FMT "\n", ptr_struct->disp_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_max_ip_payload_size : " UH_FMT "\n", ptr_struct->lro_max_ip_payload_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_enable_mask      : " UH_FMT "\n", ptr_struct->lro_enable_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_timeout_period_usecs : " UH_FMT "\n", ptr_struct->lro_timeout_period_usecs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_number           : " UH_FMT "\n", ptr_struct->rqt_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_hash_fn           : " UH_FMT "\n", ptr_struct->rx_hash_fn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tunneled_offload_en  : " UH_FMT "\n", ptr_struct->tunneled_offload_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_hash_symmetric    : " UH_FMT "\n", ptr_struct->rx_hash_symmetric);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_start_pipe  : " UH_FMT "\n", ptr_struct->steering_start_pipe);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_hash_field_selector_outer:\n");
	connectib_rx_hash_field_select_print(&(ptr_struct->rx_hash_field_selector_outer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_hash_field_selector_inner:\n");
	connectib_rx_hash_field_select_print(&(ptr_struct->rx_hash_field_selector_inner), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transport_domain     : " UH_FMT "\n", ptr_struct->transport_domain);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "self_lb_block        : " UH_FMT "\n", ptr_struct->self_lb_block);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmfs:\n");
	connectib_tir_dmfs_print(&(ptr_struct->dmfs), file, indent_level + 1);

}

int connectib_fw_tir_size(void)
{
	 return 64;
}

void connectib_fw_tir_dump(const struct connectib_fw_tir *ptr_struct, FILE* file)
{
	connectib_fw_tir_print(ptr_struct, file, 0);
}

void connectib_fw_qpc_pack(const struct connectib_fw_qpc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sw_st);

	offset = 53;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->sw_rwq_type);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->no_sq);

	offset = 50;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->sw_rq_type);

	offset = 49;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qp_dual_write);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ack_fence_req);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ack_slice_to_fence);

	offset = 37;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->min_wqe_inline_mode);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->allow_multi_pkt_send_wqe);

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_connected_to_nvme_xrq);

	offset = 91;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->sw_timer_granularity);

	offset = 86;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->current_timer_granularity);

	offset = 85;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->one_side_flush);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->apm_path_validation_failed);

	offset = 83;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cd_slave_send);

	offset = 82;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cd_slave_receive);

	offset = 81;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cd_master);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sqd_on_sigerr);

	offset = 76;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl_diff_new_sl);

	offset = 73;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->fw_max_read_atomic);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->hca_sq_owner);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cmdif_entry_ix);

	offset = 96;
	connectib_fw_qpc_ts_special_pack(&(ptr_struct->fw_qpc_ts_special), ptr_buff + offset/8);

	offset = 128;
	connectib_fw_qpc_q_pack(&(ptr_struct->responder), ptr_buff + offset/8);

	offset = 256;
	connectib_fw_qpc_q_pack(&(ptr_struct->requestor), ptr_buff + offset/8);

	offset = 384;
	connectib_fw_qpc_special_pack(&(ptr_struct->fw_qpc_special), ptr_buff + offset/8);

}

void connectib_fw_qpc_unpack(struct connectib_fw_qpc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 56;
	ptr_struct->sw_st = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 53;
	ptr_struct->sw_rwq_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 52;
	ptr_struct->no_sq = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 50;
	ptr_struct->sw_rq_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 49;
	ptr_struct->qp_dual_write = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->ack_fence_req = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 40;
	ptr_struct->ack_slice_to_fence = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 37;
	ptr_struct->min_wqe_inline_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 36;
	ptr_struct->allow_multi_pkt_send_wqe = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 35;
	ptr_struct->is_connected_to_nvme_xrq = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 91;
	ptr_struct->sw_timer_granularity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 86;
	ptr_struct->current_timer_granularity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 85;
	ptr_struct->one_side_flush = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 84;
	ptr_struct->apm_path_validation_failed = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 83;
	ptr_struct->cd_slave_send = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 82;
	ptr_struct->cd_slave_receive = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 81;
	ptr_struct->cd_master = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->sqd_on_sigerr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 76;
	ptr_struct->sl_diff_new_sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 73;
	ptr_struct->fw_max_read_atomic = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 72;
	ptr_struct->hca_sq_owner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->cmdif_entry_ix = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	connectib_fw_qpc_ts_special_unpack(&(ptr_struct->fw_qpc_ts_special), ptr_buff + offset/8);

	offset = 128;
	connectib_fw_qpc_q_unpack(&(ptr_struct->responder), ptr_buff + offset/8);

	offset = 256;
	connectib_fw_qpc_q_unpack(&(ptr_struct->requestor), ptr_buff + offset/8);

	offset = 384;
	connectib_fw_qpc_special_unpack(&(ptr_struct->fw_qpc_special), ptr_buff + offset/8);

}

void connectib_fw_qpc_print(const struct connectib_fw_qpc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_qpc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_st                : " UH_FMT "\n", ptr_struct->sw_st);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_rwq_type          : " UH_FMT "\n", ptr_struct->sw_rwq_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_sq                : " UH_FMT "\n", ptr_struct->no_sq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_rq_type           : " UH_FMT "\n", ptr_struct->sw_rq_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_dual_write        : " UH_FMT "\n", ptr_struct->qp_dual_write);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ack_fence_req        : " UH_FMT "\n", ptr_struct->ack_fence_req);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ack_slice_to_fence   : " UH_FMT "\n", ptr_struct->ack_slice_to_fence);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "min_wqe_inline_mode  : " UH_FMT "\n", ptr_struct->min_wqe_inline_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allow_multi_pkt_send_wqe : " UH_FMT "\n", ptr_struct->allow_multi_pkt_send_wqe);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_connected_to_nvme_xrq : " UH_FMT "\n", ptr_struct->is_connected_to_nvme_xrq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_timer_granularity : " UH_FMT "\n", ptr_struct->sw_timer_granularity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "current_timer_granularity : " UH_FMT "\n", ptr_struct->current_timer_granularity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "one_side_flush       : " UH_FMT "\n", ptr_struct->one_side_flush);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "apm_path_validation_failed : " UH_FMT "\n", ptr_struct->apm_path_validation_failed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cd_slave_send        : " UH_FMT "\n", ptr_struct->cd_slave_send);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cd_slave_receive     : " UH_FMT "\n", ptr_struct->cd_slave_receive);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cd_master            : " UH_FMT "\n", ptr_struct->cd_master);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sqd_on_sigerr        : " UH_FMT "\n", ptr_struct->sqd_on_sigerr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl_diff_new_sl       : " UH_FMT "\n", ptr_struct->sl_diff_new_sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_max_read_atomic   : " UH_FMT "\n", ptr_struct->fw_max_read_atomic);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hca_sq_owner         : " UH_FMT "\n", ptr_struct->hca_sq_owner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmdif_entry_ix       : " UH_FMT "\n", ptr_struct->cmdif_entry_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_qpc_ts_special:\n");
	connectib_fw_qpc_ts_special_print(&(ptr_struct->fw_qpc_ts_special), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "responder:\n");
	connectib_fw_qpc_q_print(&(ptr_struct->responder), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "requestor:\n");
	connectib_fw_qpc_q_print(&(ptr_struct->requestor), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_qpc_special:\n");
	connectib_fw_qpc_special_print(&(ptr_struct->fw_qpc_special), file, indent_level + 1);

}

int connectib_fw_qpc_size(void)
{
	 return 64;
}

void connectib_fw_qpc_dump(const struct connectib_fw_qpc *ptr_struct, FILE* file)
{
	connectib_fw_qpc_print(ptr_struct, file, 0);
}

void connectib_fw_srqc_pack(const struct connectib_fw_srqc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->catas_event_generated);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->basic_cyclic_rcv_wqe);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->invalid_user_index);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->end_padding_mode);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->trap_reason);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->trap_data);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->no_lro_qpn);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->memory_rq_state);

	offset = 256;
	connectib_fw_xrqc_pack(&(ptr_struct->xrq), ptr_buff + offset/8);

}

void connectib_fw_srqc_unpack(struct connectib_fw_srqc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 63;
	ptr_struct->catas_event_generated = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 62;
	ptr_struct->basic_cyclic_rcv_wqe = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 61;
	ptr_struct->invalid_user_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 60;
	ptr_struct->end_padding_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->trap_reason = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->trap_data = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 136;
	ptr_struct->no_lro_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 128;
	ptr_struct->memory_rq_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 256;
	connectib_fw_xrqc_unpack(&(ptr_struct->xrq), ptr_buff + offset/8);

}

void connectib_fw_srqc_print(const struct connectib_fw_srqc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_srqc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "catas_event_generated : " UH_FMT "\n", ptr_struct->catas_event_generated);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "basic_cyclic_rcv_wqe : " UH_FMT "\n", ptr_struct->basic_cyclic_rcv_wqe);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "invalid_user_index   : " UH_FMT "\n", ptr_struct->invalid_user_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "end_padding_mode     : " UH_FMT "\n", ptr_struct->end_padding_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trap_reason          : %s (" UH_FMT ")\n", (ptr_struct->trap_reason == 1 ? ("NOP") : ((ptr_struct->trap_reason == 2 ? ("SET_LIMIT") : ("unknown")))), ptr_struct->trap_reason);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trap_data            : " UH_FMT "\n", ptr_struct->trap_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_lro_qpn           : " UH_FMT "\n", ptr_struct->no_lro_qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "memory_rq_state      : " UH_FMT "\n", ptr_struct->memory_rq_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xrq:\n");
	connectib_fw_xrqc_print(&(ptr_struct->xrq), file, indent_level + 1);

}

int connectib_fw_srqc_size(void)
{
	 return 64;
}

void connectib_fw_srqc_dump(const struct connectib_fw_srqc *ptr_struct, FILE* file)
{
	connectib_fw_srqc_print(ptr_struct, file, 0);
}

void connectib_sw_ads_pack(const struct connectib_sw_ads *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pkey_index);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->free_ar);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fl);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rlid);

	offset = 41;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->mlid);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->grh);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hop_limit);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->stat_rate);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log_rtm);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->src_addr_index);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->ack_timeout);

	offset = 108;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->flow_label);

	offset = 100;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->tclass);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(152, 8, i, 352, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->rgid_rip[i]);
	}

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->udp_sport);

	offset = 266;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->dscp);

	offset = 264;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->ecn);

	offset = 263;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->f_eth_prio);

	offset = 261;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->f_ecn);

	offset = 260;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->f_dscp);

	offset = 304;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rmac_47_32);

	offset = 296;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port);

	offset = 292;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl);

	offset = 289;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->eth_prio);

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dei_cfi_reserved_from_prm_041);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rmac_31_0);

}

void connectib_sw_ads_unpack(struct connectib_sw_ads *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->pkey_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 1;
	ptr_struct->free_ar = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->fl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->rlid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 41;
	ptr_struct->mlid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 40;
	ptr_struct->grh = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 88;
	ptr_struct->hop_limit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 84;
	ptr_struct->stat_rate = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 80;
	ptr_struct->log_rtm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->src_addr_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->ack_timeout = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 108;
	ptr_struct->flow_label = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

	offset = 100;
	ptr_struct->tclass = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(152, 8, i, 352, 1);
	ptr_struct->rgid_rip[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	offset = 272;
	ptr_struct->udp_sport = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 266;
	ptr_struct->dscp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 264;
	ptr_struct->ecn = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 263;
	ptr_struct->f_eth_prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 261;
	ptr_struct->f_ecn = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 260;
	ptr_struct->f_dscp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 304;
	ptr_struct->rmac_47_32 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 296;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 292;
	ptr_struct->sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 289;
	ptr_struct->eth_prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 288;
	ptr_struct->dei_cfi_reserved_from_prm_041 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 320;
	ptr_struct->rmac_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_sw_ads_print(const struct connectib_sw_ads *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_ads ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey_index           : " UH_FMT "\n", ptr_struct->pkey_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "free_ar              : " UH_FMT "\n", ptr_struct->free_ar);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fl                   : " UH_FMT "\n", ptr_struct->fl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rlid                 : " UH_FMT "\n", ptr_struct->rlid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlid                 : " UH_FMT "\n", ptr_struct->mlid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh                  : " UH_FMT "\n", ptr_struct->grh);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hop_limit            : " UH_FMT "\n", ptr_struct->hop_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "stat_rate            : %s (" UH_FMT ")\n", (ptr_struct->stat_rate == 0 ? ("no_limit") : ((ptr_struct->stat_rate == 7 ? ("Gbps_2_5") : ((ptr_struct->stat_rate == 8 ? ("Gbps_10") : ((ptr_struct->stat_rate == 9 ? ("Gbps_30") : ((ptr_struct->stat_rate == 10 ? ("Gbps_5") : ((ptr_struct->stat_rate == 11 ? ("Gbps_20") : ((ptr_struct->stat_rate == 12 ? ("Gbps_40") : ((ptr_struct->stat_rate == 13 ? ("Gbps_60") : ((ptr_struct->stat_rate == 14 ? ("Gbps_80") : ((ptr_struct->stat_rate == 15 ? ("Gbps_120") : ("unknown")))))))))))))))))))), ptr_struct->stat_rate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_rtm              : " UH_FMT "\n", ptr_struct->log_rtm);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "src_addr_index       : " UH_FMT "\n", ptr_struct->src_addr_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ack_timeout          : " UH_FMT "\n", ptr_struct->ack_timeout);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_label           : " UH_FMT "\n", ptr_struct->flow_label);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tclass               : " UH_FMT "\n", ptr_struct->tclass);

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rgid_rip_%03d        : " UH_FMT "\n", i, ptr_struct->rgid_rip[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "udp_sport            : " UH_FMT "\n", ptr_struct->udp_sport);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dscp                 : " UH_FMT "\n", ptr_struct->dscp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecn                  : " UH_FMT "\n", ptr_struct->ecn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "f_eth_prio           : " UH_FMT "\n", ptr_struct->f_eth_prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "f_ecn                : " UH_FMT "\n", ptr_struct->f_ecn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "f_dscp               : " UH_FMT "\n", ptr_struct->f_dscp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rmac_47_32           : " UH_FMT "\n", ptr_struct->rmac_47_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl                   : " UH_FMT "\n", ptr_struct->sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_prio             : " UH_FMT "\n", ptr_struct->eth_prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dei_cfi_reserved_from_prm_041 : " UH_FMT "\n", ptr_struct->dei_cfi_reserved_from_prm_041);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rmac_31_0            : " U32H_FMT "\n", ptr_struct->rmac_31_0);

}

int connectib_sw_ads_size(void)
{
	 return 44;
}

void connectib_sw_ads_dump(const struct connectib_sw_ads *ptr_struct, FILE* file)
{
	connectib_sw_ads_print(ptr_struct, file, 0);
}

void connectib_hw_msix_ext_context_pack(const struct connectib_hw_msix_ext_context *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mask_3_0);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->disable_3_0);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->need);

	offset = 76;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->timestamp_51_32);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->min_wait);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->timestamp_31_0);

}

void connectib_hw_msix_ext_context_unpack(struct connectib_hw_msix_ext_context *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->mask_3_0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 24;
	ptr_struct->disable_3_0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 23;
	ptr_struct->need = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 76;
	ptr_struct->timestamp_51_32 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

	offset = 64;
	ptr_struct->min_wait = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 96;
	ptr_struct->timestamp_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_msix_ext_context_print(const struct connectib_hw_msix_ext_context *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_msix_ext_context ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mask_3_0             : " UH_FMT "\n", ptr_struct->mask_3_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_3_0          : " UH_FMT "\n", ptr_struct->disable_3_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "need                 : " UH_FMT "\n", ptr_struct->need);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_51_32      : " UH_FMT "\n", ptr_struct->timestamp_51_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "min_wait             : " UH_FMT "\n", ptr_struct->min_wait);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_31_0       : " U32H_FMT "\n", ptr_struct->timestamp_31_0);

}

int connectib_hw_msix_ext_context_size(void)
{
	 return 16;
}

void connectib_hw_msix_ext_context_dump(const struct connectib_hw_msix_ext_context *ptr_struct, FILE* file)
{
	connectib_hw_msix_ext_context_print(ptr_struct, file, 0);
}

void connectib_hw_msix_context_pack(const struct connectib_hw_msix_context *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 30, (u_int32_t)ptr_struct->addr_31_2_);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->addr_63_32_);

	offset = 64;
	connectib_hw_msix_context_data_pack(&(ptr_struct->data), ptr_buff + offset/8);

	offset = 127;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->spec_mask);

}

void connectib_hw_msix_context_unpack(struct connectib_hw_msix_context *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->addr_31_2_ = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 30);

	offset = 32;
	ptr_struct->addr_63_32_ = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	connectib_hw_msix_context_data_unpack(&(ptr_struct->data), ptr_buff + offset/8);

	offset = 127;
	ptr_struct->spec_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_msix_context_print(const struct connectib_hw_msix_context *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_msix_context ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr_31_2_           : " UH_FMT "\n", ptr_struct->addr_31_2_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr_63_32_          : " U32H_FMT "\n", ptr_struct->addr_63_32_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data:\n");
	connectib_hw_msix_context_data_print(&(ptr_struct->data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spec_mask            : " UH_FMT "\n", ptr_struct->spec_mask);

}

int connectib_hw_msix_context_size(void)
{
	 return 16;
}

void connectib_hw_msix_context_dump(const struct connectib_hw_msix_context *ptr_struct, FILE* file)
{
	connectib_hw_msix_context_print(ptr_struct, file, 0);
}

void connectib_hw_rdb_atomic_data_pack(const struct connectib_hw_rdb_atomic_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->atomic_data[i]);
	}

}

void connectib_hw_rdb_atomic_data_unpack(struct connectib_hw_rdb_atomic_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	ptr_struct->atomic_data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_hw_rdb_atomic_data_print(const struct connectib_hw_rdb_atomic_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_rdb_atomic_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "atomic_data_%03d     : " U32H_FMT "\n", i, ptr_struct->atomic_data[i]);
	}

}

int connectib_hw_rdb_atomic_data_size(void)
{
	 return 32;
}

void connectib_hw_rdb_atomic_data_dump(const struct connectib_hw_rdb_atomic_data *ptr_struct, FILE* file)
{
	connectib_hw_rdb_atomic_data_print(ptr_struct, file, 0);
}

void connectib_hw_mtt2_pack(const struct connectib_hw_mtt2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hi);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lo);

}

void connectib_hw_mtt2_unpack(struct connectib_hw_mtt2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_mtt2_print(const struct connectib_hw_mtt2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_mtt2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hi                   : " U32H_FMT "\n", ptr_struct->hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lo                   : " U32H_FMT "\n", ptr_struct->lo);

}

int connectib_hw_mtt2_size(void)
{
	 return 8;
}

void connectib_hw_mtt2_dump(const struct connectib_hw_mtt2 *ptr_struct, FILE* file)
{
	connectib_hw_mtt2_print(ptr_struct, file, 0);
}

void connectib_hw_mtt_pack(const struct connectib_hw_mtt *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pas_63_32);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rd_en);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wr_en);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 29, (u_int32_t)ptr_struct->pas_31_3);

}

void connectib_hw_mtt_unpack(struct connectib_hw_mtt *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->pas_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 63;
	ptr_struct->rd_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 62;
	ptr_struct->wr_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->pas_31_3 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 29);

}

void connectib_hw_mtt_print(const struct connectib_hw_mtt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_mtt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pas_63_32            : " U32H_FMT "\n", ptr_struct->pas_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rd_en                : " UH_FMT "\n", ptr_struct->rd_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wr_en                : " UH_FMT "\n", ptr_struct->wr_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pas_31_3             : " UH_FMT "\n", ptr_struct->pas_31_3);

}

int connectib_hw_mtt_size(void)
{
	 return 8;
}

void connectib_hw_mtt_dump(const struct connectib_hw_mtt *ptr_struct, FILE* file)
{
	connectib_hw_mtt_print(ptr_struct, file, 0);
}

void connectib_steering_qp_list_pack(const struct connectib_steering_qp_list *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry0), ptr_buff + offset/8);

	offset = 32;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry1), ptr_buff + offset/8);

	offset = 64;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry2), ptr_buff + offset/8);

	offset = 96;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry3), ptr_buff + offset/8);

	offset = 128;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry4), ptr_buff + offset/8);

	offset = 160;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry5), ptr_buff + offset/8);

	offset = 192;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry6), ptr_buff + offset/8);

	offset = 224;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry7), ptr_buff + offset/8);

	offset = 256;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry8), ptr_buff + offset/8);

	offset = 288;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry9), ptr_buff + offset/8);

	offset = 320;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry10), ptr_buff + offset/8);

	offset = 352;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry11), ptr_buff + offset/8);

	offset = 384;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry12), ptr_buff + offset/8);

	offset = 416;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry13), ptr_buff + offset/8);

	offset = 448;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry14), ptr_buff + offset/8);

	offset = 480;
	connectib_steering_qp_entry_pack(&(ptr_struct->steering_qp_entry15), ptr_buff + offset/8);

}

void connectib_steering_qp_list_unpack(struct connectib_steering_qp_list *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry0), ptr_buff + offset/8);

	offset = 32;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry1), ptr_buff + offset/8);

	offset = 64;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry2), ptr_buff + offset/8);

	offset = 96;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry3), ptr_buff + offset/8);

	offset = 128;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry4), ptr_buff + offset/8);

	offset = 160;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry5), ptr_buff + offset/8);

	offset = 192;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry6), ptr_buff + offset/8);

	offset = 224;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry7), ptr_buff + offset/8);

	offset = 256;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry8), ptr_buff + offset/8);

	offset = 288;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry9), ptr_buff + offset/8);

	offset = 320;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry10), ptr_buff + offset/8);

	offset = 352;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry11), ptr_buff + offset/8);

	offset = 384;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry12), ptr_buff + offset/8);

	offset = 416;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry13), ptr_buff + offset/8);

	offset = 448;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry14), ptr_buff + offset/8);

	offset = 480;
	connectib_steering_qp_entry_unpack(&(ptr_struct->steering_qp_entry15), ptr_buff + offset/8);

}

void connectib_steering_qp_list_print(const struct connectib_steering_qp_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_steering_qp_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry0:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry1:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry2:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry3:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry3), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry4:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry4), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry5:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry5), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry6:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry6), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry7:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry7), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry8:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry9:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry9), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry10:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry10), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry11:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry11), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry12:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry12), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry13:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry13), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry14:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry14), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_entry15:\n");
	connectib_steering_qp_entry_print(&(ptr_struct->steering_qp_entry15), file, indent_level + 1);

}

int connectib_steering_qp_list_size(void)
{
	 return 64;
}

void connectib_steering_qp_list_dump(const struct connectib_steering_qp_list *ptr_struct, FILE* file)
{
	connectib_steering_qp_list_print(ptr_struct, file, 0);
}

void connectib_hw_ste_pack(const struct connectib_hw_ste *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->member_count);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rx_packet_qp);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rx_fw_int);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->rx_cmd);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pport_byte_counters_trigger);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pport_packet_counters_trigger);

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vport_byte_counters_trigger);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vport_packet_counters_trigger);

	offset = 37;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_sniffer);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_func_lb);

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_wire);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_fw_int);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->sx_cmd);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->fw_id);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->my_lookup_type);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hit_lookup_type_hash_field_mask);

	offset = 114;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log2_hit_entry_size);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hit_base_address_63_32);

	offset = 184;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hit_lookup_type);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->hit_base_address_31_8);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->miss_address_63_32);

	offset = 250;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log2_hit_table_size);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 26, (u_int32_t)ptr_struct->miss_address_31_6);

	offset = 256;
	connectib_hw_ste_tag_pack(&(ptr_struct->tag_data), ptr_buff + offset/8);

	offset = 384;
	connectib_hw_ste_tag_pack(&(ptr_struct->tag_mask), ptr_buff + offset/8);

}

void connectib_hw_ste_unpack(struct connectib_hw_ste *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->member_count = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 3;
	ptr_struct->rx_packet_qp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->rx_fw_int = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->rx_cmd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 56;
	ptr_struct->pport_byte_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->pport_packet_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 44;
	ptr_struct->vport_byte_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 40;
	ptr_struct->vport_packet_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 37;
	ptr_struct->sx_sniffer = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 36;
	ptr_struct->sx_func_lb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 35;
	ptr_struct->sx_wire = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 34;
	ptr_struct->sx_fw_int = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->sx_cmd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 72;
	ptr_struct->fw_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->my_lookup_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 120;
	ptr_struct->hit_lookup_type_hash_field_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 114;
	ptr_struct->log2_hit_entry_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 96;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 128;
	ptr_struct->hit_base_address_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 184;
	ptr_struct->hit_lookup_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 160;
	ptr_struct->hit_base_address_31_8 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 192;
	ptr_struct->miss_address_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 250;
	ptr_struct->log2_hit_table_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 224;
	ptr_struct->miss_address_31_6 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 26);

	offset = 256;
	connectib_hw_ste_tag_unpack(&(ptr_struct->tag_data), ptr_buff + offset/8);

	offset = 384;
	connectib_hw_ste_tag_unpack(&(ptr_struct->tag_mask), ptr_buff + offset/8);

}

void connectib_hw_ste_print(const struct connectib_hw_ste *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_ste ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "member_count         : " UH_FMT "\n", ptr_struct->member_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_packet_qp         : " UH_FMT "\n", ptr_struct->rx_packet_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_fw_int            : " UH_FMT "\n", ptr_struct->rx_fw_int);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_cmd               : " UH_FMT "\n", ptr_struct->rx_cmd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pport_byte_counters_trigger : " UH_FMT "\n", ptr_struct->pport_byte_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pport_packet_counters_trigger : " UH_FMT "\n", ptr_struct->pport_packet_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_byte_counters_trigger : " UH_FMT "\n", ptr_struct->vport_byte_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_packet_counters_trigger : " UH_FMT "\n", ptr_struct->vport_packet_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_sniffer           : " UH_FMT "\n", ptr_struct->sx_sniffer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_func_lb           : " UH_FMT "\n", ptr_struct->sx_func_lb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_wire              : " UH_FMT "\n", ptr_struct->sx_wire);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_fw_int            : " UH_FMT "\n", ptr_struct->sx_fw_int);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_cmd               : " UH_FMT "\n", ptr_struct->sx_cmd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_id                : " UH_FMT "\n", ptr_struct->fw_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "my_lookup_type       : " UH_FMT "\n", ptr_struct->my_lookup_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hit_lookup_type_hash_field_mask : " UH_FMT "\n", ptr_struct->hit_lookup_type_hash_field_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_hit_entry_size  : " UH_FMT "\n", ptr_struct->log2_hit_entry_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hit_base_address_63_32 : " U32H_FMT "\n", ptr_struct->hit_base_address_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hit_lookup_type      : " UH_FMT "\n", ptr_struct->hit_lookup_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hit_base_address_31_8 : " UH_FMT "\n", ptr_struct->hit_base_address_31_8);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "miss_address_63_32   : " U32H_FMT "\n", ptr_struct->miss_address_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_hit_table_size  : " UH_FMT "\n", ptr_struct->log2_hit_table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "miss_address_31_6    : " UH_FMT "\n", ptr_struct->miss_address_31_6);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tag_data:\n");
	connectib_hw_ste_tag_print(&(ptr_struct->tag_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tag_mask:\n");
	connectib_hw_ste_tag_print(&(ptr_struct->tag_mask), file, indent_level + 1);

}

int connectib_hw_ste_size(void)
{
	 return 64;
}

void connectib_hw_ste_dump(const struct connectib_hw_ste *ptr_struct, FILE* file)
{
	connectib_hw_ste_print(ptr_struct, file, 0);
}

void connectib_hw_timer_pack(const struct connectib_hw_timer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->timer_granularity);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->timer_state);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->timer_valid);

}

void connectib_hw_timer_unpack(struct connectib_hw_timer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	ptr_struct->timer_granularity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 1;
	ptr_struct->timer_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 0;
	ptr_struct->timer_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_timer_print(const struct connectib_hw_timer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_timer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timer_granularity    : " UH_FMT "\n", ptr_struct->timer_granularity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timer_state          : " UH_FMT "\n", ptr_struct->timer_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timer_valid          : " UH_FMT "\n", ptr_struct->timer_valid);

}

int connectib_hw_timer_size(void)
{
	 return 1;
}

void connectib_hw_timer_dump(const struct connectib_hw_timer *ptr_struct, FILE* file)
{
	connectib_hw_timer_print(ptr_struct, file, 0);
}

void connectib_sxdc_requestor_in_sq_pack(const struct connectib_sxdc_requestor_in_sq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_requestor_in_sq_unpack(struct connectib_sxdc_requestor_in_sq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 39;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_sxdc_requestor_in_sq_print(const struct connectib_sxdc_requestor_in_sq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_requestor_in_sq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_requestor_in_sq_size(void)
{
	 return 8;
}

void connectib_sxdc_requestor_in_sq_dump(const struct connectib_sxdc_requestor_in_sq *ptr_struct, FILE* file)
{
	connectib_sxdc_requestor_in_sq_print(ptr_struct, file, 0);
}

void connectib_sxdc_responder_in_sq_pack(const struct connectib_sxdc_responder_in_sq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 38;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_responder_in_sq_unpack(struct connectib_sxdc_responder_in_sq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 38;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_sxdc_responder_in_sq_print(const struct connectib_sxdc_responder_in_sq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_responder_in_sq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_responder_in_sq_size(void)
{
	 return 8;
}

void connectib_sxdc_responder_in_sq_dump(const struct connectib_sxdc_responder_in_sq *ptr_struct, FILE* file)
{
	connectib_sxdc_responder_in_sq_print(ptr_struct, file, 0);
}

void connectib_sxdc_scheduling_interrupt_pack(const struct connectib_sxdc_scheduling_interrupt *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 37;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_scheduling_interrupt_unpack(struct connectib_sxdc_scheduling_interrupt *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 37;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_sxdc_scheduling_interrupt_print(const struct connectib_sxdc_scheduling_interrupt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_scheduling_interrupt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_scheduling_interrupt_size(void)
{
	 return 8;
}

void connectib_sxdc_scheduling_interrupt_dump(const struct connectib_sxdc_scheduling_interrupt *ptr_struct, FILE* file)
{
	connectib_sxdc_scheduling_interrupt_print(ptr_struct, file, 0);
}

void connectib_sxdc_doorbell_int_pack(const struct connectib_sxdc_doorbell_int *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_doorbell_int_unpack(struct connectib_sxdc_doorbell_int *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 35;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_sxdc_doorbell_int_print(const struct connectib_sxdc_doorbell_int *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_doorbell_int ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_doorbell_int_size(void)
{
	 return 8;
}

void connectib_sxdc_doorbell_int_dump(const struct connectib_sxdc_doorbell_int *ptr_struct, FILE* file)
{
	connectib_sxdc_doorbell_int_print(ptr_struct, file, 0);
}

void connectib_sxdc_fast_path_pack(const struct connectib_sxdc_fast_path *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_fast_path_unpack(struct connectib_sxdc_fast_path *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 34;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_sxdc_fast_path_print(const struct connectib_sxdc_fast_path *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_fast_path ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_fast_path_size(void)
{
	 return 8;
}

void connectib_sxdc_fast_path_dump(const struct connectib_sxdc_fast_path *ptr_struct, FILE* file)
{
	connectib_sxdc_fast_path_print(ptr_struct, file, 0);
}

void connectib_sxdc_external_db_en_pack(const struct connectib_sxdc_external_db_en *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_external_db_en_unpack(struct connectib_sxdc_external_db_en *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 33;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_sxdc_external_db_en_print(const struct connectib_sxdc_external_db_en *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_external_db_en ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_external_db_en_size(void)
{
	 return 8;
}

void connectib_sxdc_external_db_en_dump(const struct connectib_sxdc_external_db_en *ptr_struct, FILE* file)
{
	connectib_sxdc_external_db_en_print(ptr_struct, file, 0);
}

void connectib_sxdc_valid_pack(const struct connectib_sxdc_valid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_valid_unpack(struct connectib_sxdc_valid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_sxdc_valid_print(const struct connectib_sxdc_valid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_valid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_valid_size(void)
{
	 return 8;
}

void connectib_sxdc_valid_dump(const struct connectib_sxdc_valid *ptr_struct, FILE* file)
{
	connectib_sxdc_valid_print(ptr_struct, file, 0);
}

void connectib_sxdc_sq_pack(const struct connectib_sxdc_sq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_sq_unpack(struct connectib_sxdc_sq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_sxdc_sq_print(const struct connectib_sxdc_sq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_sq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_sq_size(void)
{
	 return 8;
}

void connectib_sxdc_sq_dump(const struct connectib_sxdc_sq *ptr_struct, FILE* file)
{
	connectib_sxdc_sq_print(ptr_struct, file, 0);
}

void connectib_sxdc_swqe_producer_index_pack(const struct connectib_sxdc_swqe_producer_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_swqe_producer_index_unpack(struct connectib_sxdc_swqe_producer_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_sxdc_swqe_producer_index_print(const struct connectib_sxdc_swqe_producer_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_swqe_producer_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_swqe_producer_index_size(void)
{
	 return 8;
}

void connectib_sxdc_swqe_producer_index_dump(const struct connectib_sxdc_swqe_producer_index *ptr_struct, FILE* file)
{
	connectib_sxdc_swqe_producer_index_print(ptr_struct, file, 0);
}

void connectib_sxdc_uar_pack(const struct connectib_sxdc_uar *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_sxdc_uar_unpack(struct connectib_sxdc_uar *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 40;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_sxdc_uar_print(const struct connectib_sxdc_uar *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sxdc_uar ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_sxdc_uar_size(void)
{
	 return 8;
}

void connectib_sxdc_uar_dump(const struct connectib_sxdc_uar *ptr_struct, FILE* file)
{
	connectib_sxdc_uar_print(ptr_struct, file, 0);
}

void connectib_hw_bsf_entry_w_non_repeating_m_non_repeating_pack(const struct connectib_hw_bsf_entry_w_non_repeating_m_non_repeating *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_check_gen);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_init);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->m_pad);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_check_gen);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_init);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->w_pad);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->byte_check_en);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aligned);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->size);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->raw_size);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_b_psv_pointer);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_b_pdf_ptr);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_b_psv_pointer);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_b_pdf_ptr);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->prologue_size);

	offset = 135;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_end);

	offset = 134;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_start);

	offset = 131;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_end);

	offset = 130;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_start);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->epilogue_size);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_t_psv_pointer);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_t_pdf_ptr);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_t_psv_pointer);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_t_pdf_ptr);

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->w_dif_app_tag);

	offset = 257;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_refresh_dif_params);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_in_line_vld);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->w_dif_ref_tag);

	offset = 320;
	connectib_hw_pdf_entry_pack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->m_dif_app_tag);

	offset = 385;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_refresh_dif_params);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_in_line_vld);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->m_dif_ref_tag);

	offset = 448;
	connectib_hw_pdf_entry_pack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_w_non_repeating_m_non_repeating_unpack(struct connectib_hw_bsf_entry_w_non_repeating_m_non_repeating *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->m_check_gen = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->m_init = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->m_pad = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 23;
	ptr_struct->w_check_gen = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->w_init = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->w_pad = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 8;
	ptr_struct->byte_check_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->aligned = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 32;
	ptr_struct->raw_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->w_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->w_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->m_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->m_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->prologue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 135;
	ptr_struct->m_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 134;
	ptr_struct->m_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 131;
	ptr_struct->w_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 130;
	ptr_struct->w_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->epilogue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 200;
	ptr_struct->w_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 192;
	ptr_struct->w_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 232;
	ptr_struct->m_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 224;
	ptr_struct->m_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 272;
	ptr_struct->w_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 257;
	ptr_struct->w_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 256;
	ptr_struct->w_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 288;
	ptr_struct->w_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	ptr_struct->m_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 385;
	ptr_struct->m_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 384;
	ptr_struct->m_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 416;
	ptr_struct->m_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_w_non_repeating_m_non_repeating_print(const struct connectib_hw_bsf_entry_w_non_repeating_m_non_repeating *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_bsf_entry_w_non_repeating_m_non_repeating ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_check_gen          : " UH_FMT "\n", ptr_struct->m_check_gen);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_init               : " UH_FMT "\n", ptr_struct->m_init);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_pad                : " UH_FMT "\n", ptr_struct->m_pad);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_check_gen          : " UH_FMT "\n", ptr_struct->w_check_gen);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_init               : " UH_FMT "\n", ptr_struct->w_init);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_pad                : " UH_FMT "\n", ptr_struct->w_pad);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_check_en        : " UH_FMT "\n", ptr_struct->byte_check_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aligned              : " UH_FMT "\n", ptr_struct->aligned);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw_size             : " U32H_FMT "\n", ptr_struct->raw_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_psv_pointer      : " UH_FMT "\n", ptr_struct->w_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_psv_pointer      : " UH_FMT "\n", ptr_struct->m_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prologue_size        : " UH_FMT "\n", ptr_struct->prologue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_end              : " UH_FMT "\n", ptr_struct->m_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_start            : " UH_FMT "\n", ptr_struct->m_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_end              : " UH_FMT "\n", ptr_struct->w_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_start            : " UH_FMT "\n", ptr_struct->w_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "epilogue_size        : " UH_FMT "\n", ptr_struct->epilogue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_psv_pointer      : " UH_FMT "\n", ptr_struct->w_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_psv_pointer      : " UH_FMT "\n", ptr_struct->m_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_app_tag        : " UH_FMT "\n", ptr_struct->w_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_refresh_dif_params : " UH_FMT "\n", ptr_struct->w_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_vld        : " UH_FMT "\n", ptr_struct->w_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->w_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->w_in_line_pdf), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_app_tag        : " UH_FMT "\n", ptr_struct->m_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_refresh_dif_params : " UH_FMT "\n", ptr_struct->m_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_vld        : " UH_FMT "\n", ptr_struct->m_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->m_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->m_in_line_pdf), file, indent_level + 1);

}

int connectib_hw_bsf_entry_w_non_repeating_m_non_repeating_size(void)
{
	 return 64;
}

void connectib_hw_bsf_entry_w_non_repeating_m_non_repeating_dump(const struct connectib_hw_bsf_entry_w_non_repeating_m_non_repeating *ptr_struct, FILE* file)
{
	connectib_hw_bsf_entry_w_non_repeating_m_non_repeating_print(ptr_struct, file, 0);
}

void connectib_hw_bsf_entry_w_non_repeating_m_repeating_pack(const struct connectib_hw_bsf_entry_w_non_repeating_m_repeating *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_block_size_selector);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_check_gen);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_init);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->w_pad);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->byte_check_en);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aligned);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->size);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->raw_size);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_b_psv_pointer);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_b_pdf_ptr);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_b_psv_pointer);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_b_pdf_ptr);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->prologue_size);

	offset = 135;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_end);

	offset = 134;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_start);

	offset = 131;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_end);

	offset = 130;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_start);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->epilogue_size);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_t_psv_pointer);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_t_pdf_ptr);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_t_psv_pointer);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_t_pdf_ptr);

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->w_dif_app_tag);

	offset = 257;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_refresh_dif_params);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_in_line_vld);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->w_dif_ref_tag);

	offset = 320;
	connectib_hw_pdf_entry_pack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->m_dif_app_tag);

	offset = 385;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_refresh_dif_params);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_in_line_vld);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->m_dif_ref_tag);

	offset = 448;
	connectib_hw_pdf_entry_pack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_w_non_repeating_m_repeating_unpack(struct connectib_hw_bsf_entry_w_non_repeating_m_repeating *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->m_block_size_selector = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 23;
	ptr_struct->w_check_gen = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->w_init = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->w_pad = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 8;
	ptr_struct->byte_check_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->aligned = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 32;
	ptr_struct->raw_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->w_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->w_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->m_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->m_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->prologue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 135;
	ptr_struct->m_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 134;
	ptr_struct->m_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 131;
	ptr_struct->w_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 130;
	ptr_struct->w_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->epilogue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 200;
	ptr_struct->w_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 192;
	ptr_struct->w_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 232;
	ptr_struct->m_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 224;
	ptr_struct->m_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 272;
	ptr_struct->w_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 257;
	ptr_struct->w_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 256;
	ptr_struct->w_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 288;
	ptr_struct->w_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	ptr_struct->m_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 385;
	ptr_struct->m_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 384;
	ptr_struct->m_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 416;
	ptr_struct->m_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_w_non_repeating_m_repeating_print(const struct connectib_hw_bsf_entry_w_non_repeating_m_repeating *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_bsf_entry_w_non_repeating_m_repeating ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_block_size_selector : " UH_FMT "\n", ptr_struct->m_block_size_selector);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_check_gen          : " UH_FMT "\n", ptr_struct->w_check_gen);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_init               : " UH_FMT "\n", ptr_struct->w_init);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_pad                : " UH_FMT "\n", ptr_struct->w_pad);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_check_en        : " UH_FMT "\n", ptr_struct->byte_check_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aligned              : " UH_FMT "\n", ptr_struct->aligned);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw_size             : " U32H_FMT "\n", ptr_struct->raw_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_psv_pointer      : " UH_FMT "\n", ptr_struct->w_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_psv_pointer      : " UH_FMT "\n", ptr_struct->m_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prologue_size        : " UH_FMT "\n", ptr_struct->prologue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_end              : " UH_FMT "\n", ptr_struct->m_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_start            : " UH_FMT "\n", ptr_struct->m_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_end              : " UH_FMT "\n", ptr_struct->w_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_start            : " UH_FMT "\n", ptr_struct->w_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "epilogue_size        : " UH_FMT "\n", ptr_struct->epilogue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_psv_pointer      : " UH_FMT "\n", ptr_struct->w_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_psv_pointer      : " UH_FMT "\n", ptr_struct->m_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_app_tag        : " UH_FMT "\n", ptr_struct->w_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_refresh_dif_params : " UH_FMT "\n", ptr_struct->w_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_vld        : " UH_FMT "\n", ptr_struct->w_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->w_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->w_in_line_pdf), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_app_tag        : " UH_FMT "\n", ptr_struct->m_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_refresh_dif_params : " UH_FMT "\n", ptr_struct->m_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_vld        : " UH_FMT "\n", ptr_struct->m_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->m_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->m_in_line_pdf), file, indent_level + 1);

}

int connectib_hw_bsf_entry_w_non_repeating_m_repeating_size(void)
{
	 return 64;
}

void connectib_hw_bsf_entry_w_non_repeating_m_repeating_dump(const struct connectib_hw_bsf_entry_w_non_repeating_m_repeating *ptr_struct, FILE* file)
{
	connectib_hw_bsf_entry_w_non_repeating_m_repeating_print(ptr_struct, file, 0);
}

void connectib_hw_bsf_entry_w_repeating_m_non_repeating_pack(const struct connectib_hw_bsf_entry_w_repeating_m_non_repeating *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_check_gen);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_init);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->m_pad);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_block_size_selector);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->byte_check_en);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aligned);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->size);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->raw_size);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_b_psv_pointer);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_b_pdf_ptr);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_b_psv_pointer);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_b_pdf_ptr);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->prologue_size);

	offset = 135;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_end);

	offset = 134;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_start);

	offset = 131;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_end);

	offset = 130;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_start);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->epilogue_size);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_t_psv_pointer);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_t_pdf_ptr);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_t_psv_pointer);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_t_pdf_ptr);

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->w_dif_app_tag);

	offset = 257;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_refresh_dif_params);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_in_line_vld);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->w_dif_ref_tag);

	offset = 320;
	connectib_hw_pdf_entry_pack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->m_dif_app_tag);

	offset = 385;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_refresh_dif_params);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_in_line_vld);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->m_dif_ref_tag);

	offset = 448;
	connectib_hw_pdf_entry_pack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_w_repeating_m_non_repeating_unpack(struct connectib_hw_bsf_entry_w_repeating_m_non_repeating *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->m_check_gen = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->m_init = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->m_pad = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 16;
	ptr_struct->w_block_size_selector = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->byte_check_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->aligned = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 32;
	ptr_struct->raw_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->w_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->w_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->m_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->m_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->prologue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 135;
	ptr_struct->m_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 134;
	ptr_struct->m_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 131;
	ptr_struct->w_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 130;
	ptr_struct->w_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->epilogue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 200;
	ptr_struct->w_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 192;
	ptr_struct->w_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 232;
	ptr_struct->m_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 224;
	ptr_struct->m_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 272;
	ptr_struct->w_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 257;
	ptr_struct->w_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 256;
	ptr_struct->w_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 288;
	ptr_struct->w_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	ptr_struct->m_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 385;
	ptr_struct->m_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 384;
	ptr_struct->m_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 416;
	ptr_struct->m_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_w_repeating_m_non_repeating_print(const struct connectib_hw_bsf_entry_w_repeating_m_non_repeating *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_bsf_entry_w_repeating_m_non_repeating ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_check_gen          : " UH_FMT "\n", ptr_struct->m_check_gen);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_init               : " UH_FMT "\n", ptr_struct->m_init);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_pad                : " UH_FMT "\n", ptr_struct->m_pad);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_block_size_selector : " UH_FMT "\n", ptr_struct->w_block_size_selector);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_check_en        : " UH_FMT "\n", ptr_struct->byte_check_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aligned              : " UH_FMT "\n", ptr_struct->aligned);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw_size             : " U32H_FMT "\n", ptr_struct->raw_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_psv_pointer      : " UH_FMT "\n", ptr_struct->w_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_psv_pointer      : " UH_FMT "\n", ptr_struct->m_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prologue_size        : " UH_FMT "\n", ptr_struct->prologue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_end              : " UH_FMT "\n", ptr_struct->m_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_start            : " UH_FMT "\n", ptr_struct->m_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_end              : " UH_FMT "\n", ptr_struct->w_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_start            : " UH_FMT "\n", ptr_struct->w_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "epilogue_size        : " UH_FMT "\n", ptr_struct->epilogue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_psv_pointer      : " UH_FMT "\n", ptr_struct->w_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_psv_pointer      : " UH_FMT "\n", ptr_struct->m_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_app_tag        : " UH_FMT "\n", ptr_struct->w_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_refresh_dif_params : " UH_FMT "\n", ptr_struct->w_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_vld        : " UH_FMT "\n", ptr_struct->w_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->w_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->w_in_line_pdf), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_app_tag        : " UH_FMT "\n", ptr_struct->m_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_refresh_dif_params : " UH_FMT "\n", ptr_struct->m_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_vld        : " UH_FMT "\n", ptr_struct->m_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->m_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->m_in_line_pdf), file, indent_level + 1);

}

int connectib_hw_bsf_entry_w_repeating_m_non_repeating_size(void)
{
	 return 64;
}

void connectib_hw_bsf_entry_w_repeating_m_non_repeating_dump(const struct connectib_hw_bsf_entry_w_repeating_m_non_repeating *ptr_struct, FILE* file)
{
	connectib_hw_bsf_entry_w_repeating_m_non_repeating_print(ptr_struct, file, 0);
}

void connectib_hw_bsf_entry_w_repeating_m_repeating_pack(const struct connectib_hw_bsf_entry_w_repeating_m_repeating *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_block_size_selector);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_block_size_selector);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->byte_check_en);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aligned);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->size);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->raw_size);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_b_psv_pointer);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_b_pdf_ptr);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_b_psv_pointer);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_b_pdf_ptr);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->prologue_size);

	offset = 135;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_end);

	offset = 134;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_start);

	offset = 131;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_end);

	offset = 130;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_start);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->epilogue_size);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_t_psv_pointer);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_t_pdf_ptr);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_t_psv_pointer);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_t_pdf_ptr);

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->w_dif_app_tag);

	offset = 257;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_refresh_dif_params);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_in_line_vld);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->w_dif_ref_tag);

	offset = 320;
	connectib_hw_pdf_entry_pack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->m_dif_app_tag);

	offset = 385;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_refresh_dif_params);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_in_line_vld);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->m_dif_ref_tag);

	offset = 448;
	connectib_hw_pdf_entry_pack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_w_repeating_m_repeating_unpack(struct connectib_hw_bsf_entry_w_repeating_m_repeating *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->m_block_size_selector = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->w_block_size_selector = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->byte_check_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->aligned = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 32;
	ptr_struct->raw_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->w_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->w_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->m_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->m_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->prologue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 135;
	ptr_struct->m_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 134;
	ptr_struct->m_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 131;
	ptr_struct->w_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 130;
	ptr_struct->w_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->epilogue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 200;
	ptr_struct->w_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 192;
	ptr_struct->w_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 232;
	ptr_struct->m_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 224;
	ptr_struct->m_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 272;
	ptr_struct->w_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 257;
	ptr_struct->w_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 256;
	ptr_struct->w_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 288;
	ptr_struct->w_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	ptr_struct->m_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 385;
	ptr_struct->m_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 384;
	ptr_struct->m_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 416;
	ptr_struct->m_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_w_repeating_m_repeating_print(const struct connectib_hw_bsf_entry_w_repeating_m_repeating *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_bsf_entry_w_repeating_m_repeating ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_block_size_selector : " UH_FMT "\n", ptr_struct->m_block_size_selector);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_block_size_selector : " UH_FMT "\n", ptr_struct->w_block_size_selector);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_check_en        : " UH_FMT "\n", ptr_struct->byte_check_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aligned              : " UH_FMT "\n", ptr_struct->aligned);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw_size             : " U32H_FMT "\n", ptr_struct->raw_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_psv_pointer      : " UH_FMT "\n", ptr_struct->w_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_psv_pointer      : " UH_FMT "\n", ptr_struct->m_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prologue_size        : " UH_FMT "\n", ptr_struct->prologue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_end              : " UH_FMT "\n", ptr_struct->m_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_start            : " UH_FMT "\n", ptr_struct->m_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_end              : " UH_FMT "\n", ptr_struct->w_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_start            : " UH_FMT "\n", ptr_struct->w_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "epilogue_size        : " UH_FMT "\n", ptr_struct->epilogue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_psv_pointer      : " UH_FMT "\n", ptr_struct->w_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_psv_pointer      : " UH_FMT "\n", ptr_struct->m_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_app_tag        : " UH_FMT "\n", ptr_struct->w_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_refresh_dif_params : " UH_FMT "\n", ptr_struct->w_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_vld        : " UH_FMT "\n", ptr_struct->w_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->w_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->w_in_line_pdf), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_app_tag        : " UH_FMT "\n", ptr_struct->m_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_refresh_dif_params : " UH_FMT "\n", ptr_struct->m_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_vld        : " UH_FMT "\n", ptr_struct->m_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->m_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->m_in_line_pdf), file, indent_level + 1);

}

int connectib_hw_bsf_entry_w_repeating_m_repeating_size(void)
{
	 return 64;
}

void connectib_hw_bsf_entry_w_repeating_m_repeating_dump(const struct connectib_hw_bsf_entry_w_repeating_m_repeating *ptr_struct, FILE* file)
{
	connectib_hw_bsf_entry_w_repeating_m_repeating_print(ptr_struct, file, 0);
}

void connectib_hw_bsf_entry_same_block_non_repeating_pack(const struct connectib_hw_bsf_entry_same_block_non_repeating *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_check_gen);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_init);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->m_pad);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->copy_byte_mask);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->byte_check_en);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aligned);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->size);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->raw_size);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_b_psv_pointer);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_b_pdf_ptr);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_b_psv_pointer);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_b_pdf_ptr);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->prologue_size);

	offset = 135;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_end);

	offset = 134;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_start);

	offset = 131;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_end);

	offset = 130;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_start);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->epilogue_size);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_t_psv_pointer);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_t_pdf_ptr);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_t_psv_pointer);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_t_pdf_ptr);

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->w_dif_app_tag);

	offset = 257;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_refresh_dif_params);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_in_line_vld);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->w_dif_ref_tag);

	offset = 320;
	connectib_hw_pdf_entry_pack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->m_dif_app_tag);

	offset = 385;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_refresh_dif_params);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_in_line_vld);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->m_dif_ref_tag);

	offset = 448;
	connectib_hw_pdf_entry_pack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_same_block_non_repeating_unpack(struct connectib_hw_bsf_entry_same_block_non_repeating *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->m_check_gen = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->m_init = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->m_pad = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 16;
	ptr_struct->copy_byte_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->byte_check_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->aligned = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 32;
	ptr_struct->raw_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->w_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->w_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->m_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->m_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->prologue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 135;
	ptr_struct->m_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 134;
	ptr_struct->m_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 131;
	ptr_struct->w_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 130;
	ptr_struct->w_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->epilogue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 200;
	ptr_struct->w_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 192;
	ptr_struct->w_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 232;
	ptr_struct->m_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 224;
	ptr_struct->m_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 272;
	ptr_struct->w_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 257;
	ptr_struct->w_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 256;
	ptr_struct->w_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 288;
	ptr_struct->w_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	ptr_struct->m_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 385;
	ptr_struct->m_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 384;
	ptr_struct->m_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 416;
	ptr_struct->m_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_same_block_non_repeating_print(const struct connectib_hw_bsf_entry_same_block_non_repeating *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_bsf_entry_same_block_non_repeating ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_check_gen          : " UH_FMT "\n", ptr_struct->m_check_gen);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_init               : " UH_FMT "\n", ptr_struct->m_init);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_pad                : " UH_FMT "\n", ptr_struct->m_pad);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "copy_byte_mask       : " UH_FMT "\n", ptr_struct->copy_byte_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_check_en        : " UH_FMT "\n", ptr_struct->byte_check_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aligned              : " UH_FMT "\n", ptr_struct->aligned);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw_size             : " U32H_FMT "\n", ptr_struct->raw_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_psv_pointer      : " UH_FMT "\n", ptr_struct->w_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_psv_pointer      : " UH_FMT "\n", ptr_struct->m_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prologue_size        : " UH_FMT "\n", ptr_struct->prologue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_end              : " UH_FMT "\n", ptr_struct->m_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_start            : " UH_FMT "\n", ptr_struct->m_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_end              : " UH_FMT "\n", ptr_struct->w_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_start            : " UH_FMT "\n", ptr_struct->w_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "epilogue_size        : " UH_FMT "\n", ptr_struct->epilogue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_psv_pointer      : " UH_FMT "\n", ptr_struct->w_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_psv_pointer      : " UH_FMT "\n", ptr_struct->m_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_app_tag        : " UH_FMT "\n", ptr_struct->w_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_refresh_dif_params : " UH_FMT "\n", ptr_struct->w_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_vld        : " UH_FMT "\n", ptr_struct->w_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->w_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->w_in_line_pdf), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_app_tag        : " UH_FMT "\n", ptr_struct->m_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_refresh_dif_params : " UH_FMT "\n", ptr_struct->m_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_vld        : " UH_FMT "\n", ptr_struct->m_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->m_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->m_in_line_pdf), file, indent_level + 1);

}

int connectib_hw_bsf_entry_same_block_non_repeating_size(void)
{
	 return 64;
}

void connectib_hw_bsf_entry_same_block_non_repeating_dump(const struct connectib_hw_bsf_entry_same_block_non_repeating *ptr_struct, FILE* file)
{
	connectib_hw_bsf_entry_same_block_non_repeating_print(ptr_struct, file, 0);
}

void connectib_hw_bsf_entry_same_block_repeating_pack(const struct connectib_hw_bsf_entry_same_block_repeating *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_block_size_selector);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->copy_byte_mask);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->byte_check_en);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aligned);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->size);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->raw_size);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_b_psv_pointer);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_b_pdf_ptr);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_b_psv_pointer);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_b_pdf_ptr);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->prologue_size);

	offset = 135;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_end);

	offset = 134;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_t_start);

	offset = 131;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_end);

	offset = 130;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_t_start);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->epilogue_size);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->w_t_psv_pointer);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->w_t_pdf_ptr);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->m_t_psv_pointer);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->m_t_pdf_ptr);

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->w_dif_app_tag);

	offset = 257;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_refresh_dif_params);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->w_in_line_vld);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->w_dif_ref_tag);

	offset = 320;
	connectib_hw_pdf_entry_pack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->m_dif_app_tag);

	offset = 385;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_refresh_dif_params);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->m_in_line_vld);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->m_dif_ref_tag);

	offset = 448;
	connectib_hw_pdf_entry_pack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_same_block_repeating_unpack(struct connectib_hw_bsf_entry_same_block_repeating *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->m_block_size_selector = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->copy_byte_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->byte_check_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 3;
	ptr_struct->aligned = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 32;
	ptr_struct->raw_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->w_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->w_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->m_b_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->m_b_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->prologue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 135;
	ptr_struct->m_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 134;
	ptr_struct->m_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 131;
	ptr_struct->w_t_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 130;
	ptr_struct->w_t_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->epilogue_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 200;
	ptr_struct->w_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 192;
	ptr_struct->w_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 232;
	ptr_struct->m_t_psv_pointer = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 224;
	ptr_struct->m_t_pdf_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 272;
	ptr_struct->w_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 257;
	ptr_struct->w_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 256;
	ptr_struct->w_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 288;
	ptr_struct->w_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->w_in_line_pdf), ptr_buff + offset/8);

	offset = 400;
	ptr_struct->m_dif_app_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 385;
	ptr_struct->m_refresh_dif_params = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 384;
	ptr_struct->m_in_line_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 416;
	ptr_struct->m_dif_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	connectib_hw_pdf_entry_unpack(&(ptr_struct->m_in_line_pdf), ptr_buff + offset/8);

}

void connectib_hw_bsf_entry_same_block_repeating_print(const struct connectib_hw_bsf_entry_same_block_repeating *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_bsf_entry_same_block_repeating ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_block_size_selector : " UH_FMT "\n", ptr_struct->m_block_size_selector);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "copy_byte_mask       : " UH_FMT "\n", ptr_struct->copy_byte_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_check_en        : " UH_FMT "\n", ptr_struct->byte_check_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aligned              : " UH_FMT "\n", ptr_struct->aligned);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw_size             : " U32H_FMT "\n", ptr_struct->raw_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_psv_pointer      : " UH_FMT "\n", ptr_struct->w_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_psv_pointer      : " UH_FMT "\n", ptr_struct->m_b_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_b_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_b_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prologue_size        : " UH_FMT "\n", ptr_struct->prologue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_end              : " UH_FMT "\n", ptr_struct->m_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_start            : " UH_FMT "\n", ptr_struct->m_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_end              : " UH_FMT "\n", ptr_struct->w_t_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_start            : " UH_FMT "\n", ptr_struct->w_t_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "epilogue_size        : " UH_FMT "\n", ptr_struct->epilogue_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_psv_pointer      : " UH_FMT "\n", ptr_struct->w_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->w_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_psv_pointer      : " UH_FMT "\n", ptr_struct->m_t_psv_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_t_pdf_ptr          : " UH_FMT "\n", ptr_struct->m_t_pdf_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_app_tag        : " UH_FMT "\n", ptr_struct->w_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_refresh_dif_params : " UH_FMT "\n", ptr_struct->w_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_vld        : " UH_FMT "\n", ptr_struct->w_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->w_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->w_in_line_pdf), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_app_tag        : " UH_FMT "\n", ptr_struct->m_dif_app_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_refresh_dif_params : " UH_FMT "\n", ptr_struct->m_refresh_dif_params);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_vld        : " UH_FMT "\n", ptr_struct->m_in_line_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_dif_ref_tag        : " U32H_FMT "\n", ptr_struct->m_dif_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_in_line_pdf:\n");
	connectib_hw_pdf_entry_print(&(ptr_struct->m_in_line_pdf), file, indent_level + 1);

}

int connectib_hw_bsf_entry_same_block_repeating_size(void)
{
	 return 64;
}

void connectib_hw_bsf_entry_same_block_repeating_dump(const struct connectib_hw_bsf_entry_same_block_repeating *ptr_struct, FILE* file)
{
	connectib_hw_bsf_entry_same_block_repeating_print(ptr_struct, file, 0);
}

void connectib_hw_dct_pack(const union connectib_hw_dct *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_dct_dc_access_key_63_32_pack(&(ptr_struct->dc_access_key_63_32), ptr_buff);
}

void connectib_hw_dct_unpack(union connectib_hw_dct *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_dct_dc_access_key_63_32_unpack(&(ptr_struct->dc_access_key_63_32), ptr_buff);
}

void connectib_hw_dct_print(const union connectib_hw_dct *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_dct ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "complete_to_64B_eyald:\n");
	connectib_hw_dct_complete_to_64B_eyald_print(&(ptr_struct->complete_to_64B_eyald), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rnr_timer:\n");
	connectib_hw_dct_rnr_timer_print(&(ptr_struct->rnr_timer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_enable:\n");
	connectib_hw_dct_write_enable_print(&(ptr_struct->write_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inline_scatter:\n");
	connectib_hw_dct_inline_scatter_print(&(ptr_struct->inline_scatter), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_message_size:\n");
	connectib_hw_dct_max_message_size_print(&(ptr_struct->max_message_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ignore_pkey_check:\n");
	connectib_hw_dct_ignore_pkey_check_print(&(ptr_struct->ignore_pkey_check), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "send_with_invalidate_enable:\n");
	connectib_hw_dct_send_with_invalidate_enable_print(&(ptr_struct->send_with_invalidate_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_enable:\n");
	connectib_hw_dct_read_enable_print(&(ptr_struct->read_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtu:\n");
	connectib_hw_dct_mtu_print(&(ptr_struct->mtu), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fast_path:\n");
	connectib_hw_dct_fast_path_print(&(ptr_struct->fast_path), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_access_key_log_num_byte:\n");
	connectib_hw_dct_dc_access_key_log_num_byte_print(&(ptr_struct->dc_access_key_log_num_byte), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcu_drop:\n");
	connectib_hw_dct_tcu_drop_print(&(ptr_struct->tcu_drop), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcu_trap:\n");
	connectib_hw_dct_tcu_trap_print(&(ptr_struct->tcu_trap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "adaptive_routing:\n");
	connectib_hw_dct_adaptive_routing_print(&(ptr_struct->adaptive_routing), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number:\n");
	connectib_hw_dct_port_number_print(&(ptr_struct->port_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh_bit:\n");
	connectib_hw_dct_grh_bit_print(&(ptr_struct->grh_bit), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_label:\n");
	connectib_hw_dct_flow_label_print(&(ptr_struct->flow_label), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_read_atomic:\n");
	connectib_hw_dct_max_read_atomic_print(&(ptr_struct->max_read_atomic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_valid:\n");
	connectib_hw_dct_qp_valid_print(&(ptr_struct->qp_valid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "my_lid:\n");
	connectib_hw_dct_my_lid_print(&(ptr_struct->my_lid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "atomic_like_write:\n");
	connectib_hw_dct_atomic_like_write_print(&(ptr_struct->atomic_like_write), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_number:\n");
	connectib_hw_dct_srq_number_print(&(ptr_struct->srq_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "counter_index:\n");
	connectib_hw_dct_counter_index_print(&(ptr_struct->counter_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey_table_index:\n");
	connectib_hw_dct_pkey_table_index_print(&(ptr_struct->pkey_table_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_atomic:\n");
	connectib_hw_dct_extended_atomic_print(&(ptr_struct->extended_atomic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "my_gid_mac_index:\n");
	connectib_hw_dct_my_gid_mac_index_print(&(ptr_struct->my_gid_mac_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ipd:\n");
	connectib_hw_dct_ipd_print(&(ptr_struct->ipd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts:\n");
	connectib_hw_dct_ts_print(&(ptr_struct->ts), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_number:\n");
	connectib_hw_dct_cq_number_print(&(ptr_struct->cq_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hop_limit:\n");
	connectib_hw_dct_hop_limit_print(&(ptr_struct->hop_limit), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "context_number:\n");
	connectib_hw_dct_context_number_print(&(ptr_struct->context_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tclass:\n");
	connectib_hw_dct_tclass_print(&(ptr_struct->tclass), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_access_key_31_0:\n");
	connectib_hw_dct_dc_access_key_31_0_print(&(ptr_struct->dc_access_key_31_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_access_key_63_32:\n");
	connectib_hw_dct_dc_access_key_63_32_print(&(ptr_struct->dc_access_key_63_32), file, indent_level + 1);

}

int connectib_hw_dct_size(void)
{
	 return 64;
}

void connectib_hw_dct_dump(const union connectib_hw_dct *ptr_struct, FILE* file)
{
	connectib_hw_dct_print(ptr_struct, file, 0);
}

void connectib_hw_r_qpc_pack(const union connectib_hw_r_qpc *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_r_qpc_qp_valid_pack(&(ptr_struct->qp_valid), ptr_buff);
}

void connectib_hw_r_qpc_unpack(union connectib_hw_r_qpc *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_r_qpc_qp_valid_unpack(&(ptr_struct->qp_valid), ptr_buff);
}

void connectib_hw_r_qpc_print(const union connectib_hw_r_qpc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_r_qpc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "connection_state:\n");
	connectib_hw_r_qpc_connection_state_print(&(ptr_struct->connection_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "check_source_mc_wire:\n");
	connectib_hw_r_qpc_check_source_mc_wire_print(&(ptr_struct->check_source_mc_wire), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_pa_31_2:\n");
	connectib_hw_r_qpc_e2e_pa_31_2_print(&(ptr_struct->e2e_pa_31_2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_pa_63_32:\n");
	connectib_hw_r_qpc_e2e_pa_63_32_print(&(ptr_struct->e2e_pa_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcr_connected:\n");
	connectib_hw_r_qpc_dcr_connected_print(&(ptr_struct->dcr_connected), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xrc_domain:\n");
	connectib_hw_r_qpc_xrc_domain_print(&(ptr_struct->xrc_domain), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "responder_session_id:\n");
	connectib_hw_r_qpc_responder_session_id_print(&(ptr_struct->responder_session_id), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mpi_uar:\n");
	connectib_hw_r_qpc_mpi_uar_print(&(ptr_struct->mpi_uar), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_dcr:\n");
	connectib_hw_r_qpc_next_dcr_print(&(ptr_struct->next_dcr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_number:\n");
	connectib_hw_r_qpc_srq_number_print(&(ptr_struct->srq_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_va_31_0:\n");
	connectib_hw_r_qpc_write_va_31_0_print(&(ptr_struct->write_va_31_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_va_63_32:\n");
	connectib_hw_r_qpc_write_va_63_32_print(&(ptr_struct->write_va_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_wqe_drop_count:\n");
	connectib_hw_r_qpc_no_wqe_drop_count_print(&(ptr_struct->no_wqe_drop_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dct_gvmi:\n");
	connectib_hw_r_qpc_dct_gvmi_print(&(ptr_struct->dct_gvmi), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sqn:\n");
	connectib_hw_r_qpc_sqn_print(&(ptr_struct->sqn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_heartbeat:\n");
	connectib_hw_r_qpc_sx_heartbeat_print(&(ptr_struct->sx_heartbeat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts:\n");
	connectib_hw_r_qpc_ts_print(&(ptr_struct->ts), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_length_committed:\n");
	connectib_hw_r_qpc_rx_length_committed_print(&(ptr_struct->rx_length_committed), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdb_read_pointer:\n");
	connectib_hw_r_qpc_rdb_read_pointer_print(&(ptr_struct->rdb_read_pointer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "always_zero:\n");
	connectib_hw_r_qpc_always_zero_print(&(ptr_struct->always_zero), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "counter_index:\n");
	connectib_hw_r_qpc_counter_index_print(&(ptr_struct->counter_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flush_syndrom:\n");
	connectib_hw_r_qpc_flush_syndrom_print(&(ptr_struct->flush_syndrom), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "expected_psn:\n");
	connectib_hw_r_qpc_expected_psn_print(&(ptr_struct->expected_psn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_number:\n");
	connectib_hw_r_qpc_cq_number_print(&(ptr_struct->cq_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dct_number:\n");
	connectib_hw_r_qpc_dct_number_print(&(ptr_struct->dct_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msn:\n");
	connectib_hw_r_qpc_msn_print(&(ptr_struct->msn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rcyclic_consumer:\n");
	connectib_hw_r_qpc_rcyclic_consumer_print(&(ptr_struct->rcyclic_consumer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_non_acked_psn:\n");
	connectib_hw_r_qpc_first_non_acked_psn_print(&(ptr_struct->first_non_acked_psn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdb_write_pointer:\n");
	connectib_hw_r_qpc_rdb_write_pointer_print(&(ptr_struct->rdb_write_pointer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_state:\n");
	connectib_hw_r_qpc_qp_state_print(&(ptr_struct->qp_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_producer_index:\n");
	connectib_hw_r_qpc_rwqe_producer_index_print(&(ptr_struct->rwqe_producer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rcyclic_producer:\n");
	connectib_hw_r_qpc_rcyclic_producer_print(&(ptr_struct->rcyclic_producer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_consumer_index:\n");
	connectib_hw_r_qpc_rwqe_consumer_index_print(&(ptr_struct->rwqe_consumer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_limit_event:\n");
	connectib_hw_r_qpc_rwqe_limit_event_print(&(ptr_struct->rwqe_limit_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rnr_timer:\n");
	connectib_hw_r_qpc_rnr_timer_print(&(ptr_struct->rnr_timer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcu_trap:\n");
	connectib_hw_r_qpc_tcu_trap_print(&(ptr_struct->tcu_trap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "can_use_reserved_lkey:\n");
	connectib_hw_r_qpc_can_use_reserved_lkey_print(&(ptr_struct->can_use_reserved_lkey), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcu_drop:\n");
	connectib_hw_r_qpc_tcu_drop_print(&(ptr_struct->tcu_drop), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_fetch_enable:\n");
	connectib_hw_r_qpc_e2e_fetch_enable_print(&(ptr_struct->e2e_fetch_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_credits_prefetch_threshold:\n");
	connectib_hw_r_qpc_e2e_credits_prefetch_threshold_print(&(ptr_struct->e2e_credits_prefetch_threshold), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_heartbeat:\n");
	connectib_hw_r_qpc_rx_heartbeat_print(&(ptr_struct->rx_heartbeat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_rkey:\n");
	connectib_hw_r_qpc_write_rkey_print(&(ptr_struct->write_rkey), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "check_source_uc_wire:\n");
	connectib_hw_r_qpc_check_source_uc_wire_print(&(ptr_struct->check_source_uc_wire), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_link_type:\n");
	connectib_hw_r_qpc_fw_link_type_print(&(ptr_struct->fw_link_type), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_opcode:\n");
	connectib_hw_r_qpc_last_opcode_print(&(ptr_struct->last_opcode), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwq_type:\n");
	connectib_hw_r_qpc_rwq_type_print(&(ptr_struct->rwq_type), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inline_scatter:\n");
	connectib_hw_r_qpc_inline_scatter_print(&(ptr_struct->inline_scatter), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "check_source_gvmi:\n");
	connectib_hw_r_qpc_check_source_gvmi_print(&(ptr_struct->check_source_gvmi), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "check_source_mc_lb:\n");
	connectib_hw_r_qpc_check_source_mc_lb_print(&(ptr_struct->check_source_mc_lb), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "check_source_uc_lb:\n");
	connectib_hw_r_qpc_check_source_uc_lb_print(&(ptr_struct->check_source_uc_lb), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fast_path:\n");
	connectib_hw_r_qpc_fast_path_print(&(ptr_struct->fast_path), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_enable:\n");
	connectib_hw_r_qpc_read_enable_print(&(ptr_struct->read_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "send_with_invalidate_enable:\n");
	connectib_hw_r_qpc_send_with_invalidate_enable_print(&(ptr_struct->send_with_invalidate_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ignore_pkey_check:\n");
	connectib_hw_r_qpc_ignore_pkey_check_print(&(ptr_struct->ignore_pkey_check), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_atomic:\n");
	connectib_hw_r_qpc_extended_atomic_print(&(ptr_struct->extended_atomic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_read_atomic:\n");
	connectib_hw_r_qpc_max_read_atomic_print(&(ptr_struct->max_read_atomic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_enable:\n");
	connectib_hw_r_qpc_write_enable_print(&(ptr_struct->write_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "atomic_like_write:\n");
	connectib_hw_r_qpc_atomic_like_write_print(&(ptr_struct->atomic_like_write), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "post_receive_slave_enable:\n");
	connectib_hw_r_qpc_post_receive_slave_enable_print(&(ptr_struct->post_receive_slave_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_direct_access:\n");
	connectib_hw_r_qpc_no_direct_access_print(&(ptr_struct->no_direct_access), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_credits_enabled:\n");
	connectib_hw_r_qpc_e2e_credits_enabled_print(&(ptr_struct->e2e_credits_enabled), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ack_type:\n");
	connectib_hw_r_qpc_ack_type_print(&(ptr_struct->ack_type), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "drop_my_self_multicast:\n");
	connectib_hw_r_qpc_drop_my_self_multicast_print(&(ptr_struct->drop_my_self_multicast), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh_to_cqe:\n");
	connectib_hw_r_qpc_grh_to_cqe_print(&(ptr_struct->grh_to_cqe), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_valid:\n");
	connectib_hw_r_qpc_qp_valid_print(&(ptr_struct->qp_valid), file, indent_level + 1);

}

int connectib_hw_r_qpc_size(void)
{
	 return 64;
}

void connectib_hw_r_qpc_dump(const union connectib_hw_r_qpc *ptr_struct, FILE* file)
{
	connectib_hw_r_qpc_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_packet_allowed_pack(const struct connectib_hw_s_qpc_packet_allowed *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_packet_allowed_unpack(struct connectib_hw_s_qpc_packet_allowed *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_packet_allowed_print(const struct connectib_hw_s_qpc_packet_allowed *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_packet_allowed ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_packet_allowed_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_packet_allowed_dump(const struct connectib_hw_s_qpc_packet_allowed *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_packet_allowed_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_qp_valid_pack(const struct connectib_hw_s_qpc_qp_valid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_qp_valid_unpack(struct connectib_hw_s_qpc_qp_valid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_qp_valid_print(const struct connectib_hw_s_qpc_qp_valid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_qp_valid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_qp_valid_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_qp_valid_dump(const struct connectib_hw_s_qpc_qp_valid *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_qp_valid_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_uncompleted_non_wire_wqe_pack(const struct connectib_hw_s_qpc_uncompleted_non_wire_wqe *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_uncompleted_non_wire_wqe_unpack(struct connectib_hw_s_qpc_uncompleted_non_wire_wqe *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_uncompleted_non_wire_wqe_print(const struct connectib_hw_s_qpc_uncompleted_non_wire_wqe *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_uncompleted_non_wire_wqe ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_uncompleted_non_wire_wqe_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_uncompleted_non_wire_wqe_dump(const struct connectib_hw_s_qpc_uncompleted_non_wire_wqe *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_uncompleted_non_wire_wqe_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_dc_full_handshake_pack(const struct connectib_hw_s_qpc_dc_full_handshake *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_dc_full_handshake_unpack(struct connectib_hw_s_qpc_dc_full_handshake *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_dc_full_handshake_print(const struct connectib_hw_s_qpc_dc_full_handshake *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_dc_full_handshake ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_dc_full_handshake_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_dc_full_handshake_dump(const struct connectib_hw_s_qpc_dc_full_handshake *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_dc_full_handshake_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_e2e_credits_enabled_pack(const struct connectib_hw_s_qpc_e2e_credits_enabled *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_e2e_credits_enabled_unpack(struct connectib_hw_s_qpc_e2e_credits_enabled *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 4;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_e2e_credits_enabled_print(const struct connectib_hw_s_qpc_e2e_credits_enabled *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_e2e_credits_enabled ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_e2e_credits_enabled_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_e2e_credits_enabled_dump(const struct connectib_hw_s_qpc_e2e_credits_enabled *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_e2e_credits_enabled_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_no_direct_access_pack(const struct connectib_hw_s_qpc_no_direct_access *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_no_direct_access_unpack(struct connectib_hw_s_qpc_no_direct_access *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_no_direct_access_print(const struct connectib_hw_s_qpc_no_direct_access *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_no_direct_access ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_no_direct_access_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_no_direct_access_dump(const struct connectib_hw_s_qpc_no_direct_access *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_no_direct_access_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_umr_permission_select_pack(const struct connectib_hw_s_qpc_umr_permission_select *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_umr_permission_select_unpack(struct connectib_hw_s_qpc_umr_permission_select *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 6;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_s_qpc_umr_permission_select_print(const struct connectib_hw_s_qpc_umr_permission_select *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_umr_permission_select ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_umr_permission_select_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_umr_permission_select_dump(const struct connectib_hw_s_qpc_umr_permission_select *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_umr_permission_select_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_max_read_atomic_pack(const struct connectib_hw_s_qpc_max_read_atomic *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_max_read_atomic_unpack(struct connectib_hw_s_qpc_max_read_atomic *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 9;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_s_qpc_max_read_atomic_print(const struct connectib_hw_s_qpc_max_read_atomic *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_max_read_atomic ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_max_read_atomic_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_max_read_atomic_dump(const struct connectib_hw_s_qpc_max_read_atomic *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_max_read_atomic_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_extended_atomic_pack(const struct connectib_hw_s_qpc_extended_atomic *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_extended_atomic_unpack(struct connectib_hw_s_qpc_extended_atomic *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_s_qpc_extended_atomic_print(const struct connectib_hw_s_qpc_extended_atomic *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_extended_atomic ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_extended_atomic_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_extended_atomic_dump(const struct connectib_hw_s_qpc_extended_atomic *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_extended_atomic_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_ignore_pkey_check_pack(const struct connectib_hw_s_qpc_ignore_pkey_check *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_ignore_pkey_check_unpack(struct connectib_hw_s_qpc_ignore_pkey_check *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_ignore_pkey_check_print(const struct connectib_hw_s_qpc_ignore_pkey_check *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_ignore_pkey_check ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_ignore_pkey_check_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_ignore_pkey_check_dump(const struct connectib_hw_s_qpc_ignore_pkey_check *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_ignore_pkey_check_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_rnr_nak_retry_counter_pack(const struct connectib_hw_s_qpc_rnr_nak_retry_counter *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_rnr_nak_retry_counter_unpack(struct connectib_hw_s_qpc_rnr_nak_retry_counter *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 17;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_s_qpc_rnr_nak_retry_counter_print(const struct connectib_hw_s_qpc_rnr_nak_retry_counter *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_rnr_nak_retry_counter ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_rnr_nak_retry_counter_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_rnr_nak_retry_counter_dump(const struct connectib_hw_s_qpc_rnr_nak_retry_counter *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_rnr_nak_retry_counter_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_fast_path_pack(const struct connectib_hw_s_qpc_fast_path *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_fast_path_unpack(struct connectib_hw_s_qpc_fast_path *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 20;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_fast_path_print(const struct connectib_hw_s_qpc_fast_path *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_fast_path ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_fast_path_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_fast_path_dump(const struct connectib_hw_s_qpc_fast_path *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_fast_path_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_rnr_nak_retry_limit_pack(const struct connectib_hw_s_qpc_rnr_nak_retry_limit *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_rnr_nak_retry_limit_unpack(struct connectib_hw_s_qpc_rnr_nak_retry_limit *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 21;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_s_qpc_rnr_nak_retry_limit_print(const struct connectib_hw_s_qpc_rnr_nak_retry_limit *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_rnr_nak_retry_limit ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_rnr_nak_retry_limit_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_rnr_nak_retry_limit_dump(const struct connectib_hw_s_qpc_rnr_nak_retry_limit *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_rnr_nak_retry_limit_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_inline_scatter_pack(const struct connectib_hw_s_qpc_inline_scatter *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_inline_scatter_unpack(struct connectib_hw_s_qpc_inline_scatter *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_s_qpc_inline_scatter_print(const struct connectib_hw_s_qpc_inline_scatter *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_inline_scatter ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_inline_scatter_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_inline_scatter_dump(const struct connectib_hw_s_qpc_inline_scatter *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_inline_scatter_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_ldb_int_pack(const struct connectib_hw_s_qpc_ldb_int *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_ldb_int_unpack(struct connectib_hw_s_qpc_ldb_int *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_ldb_int_print(const struct connectib_hw_s_qpc_ldb_int *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_ldb_int ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_ldb_int_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_ldb_int_dump(const struct connectib_hw_s_qpc_ldb_int *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_ldb_int_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_rx_heartbeat_pack(const struct connectib_hw_s_qpc_rx_heartbeat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_rx_heartbeat_unpack(struct connectib_hw_s_qpc_rx_heartbeat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_rx_heartbeat_print(const struct connectib_hw_s_qpc_rx_heartbeat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_rx_heartbeat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_rx_heartbeat_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_rx_heartbeat_dump(const struct connectib_hw_s_qpc_rx_heartbeat *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_rx_heartbeat_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_frwr_enable_pack(const struct connectib_hw_s_qpc_frwr_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 66;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_frwr_enable_unpack(struct connectib_hw_s_qpc_frwr_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 66;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_frwr_enable_print(const struct connectib_hw_s_qpc_frwr_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_frwr_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_frwr_enable_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_frwr_enable_dump(const struct connectib_hw_s_qpc_frwr_enable *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_frwr_enable_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_umr_enable_pack(const struct connectib_hw_s_qpc_umr_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 67;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_umr_enable_unpack(struct connectib_hw_s_qpc_umr_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 67;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_umr_enable_print(const struct connectib_hw_s_qpc_umr_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_umr_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_umr_enable_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_umr_enable_dump(const struct connectib_hw_s_qpc_umr_enable *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_umr_enable_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_tcu_drop_pack(const struct connectib_hw_s_qpc_tcu_drop *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 69;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_tcu_drop_unpack(struct connectib_hw_s_qpc_tcu_drop *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 69;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_tcu_drop_print(const struct connectib_hw_s_qpc_tcu_drop *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_tcu_drop ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_tcu_drop_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_tcu_drop_dump(const struct connectib_hw_s_qpc_tcu_drop *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_tcu_drop_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_can_use_reserved_lkey_pack(const struct connectib_hw_s_qpc_can_use_reserved_lkey *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 70;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_can_use_reserved_lkey_unpack(struct connectib_hw_s_qpc_can_use_reserved_lkey *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 70;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_can_use_reserved_lkey_print(const struct connectib_hw_s_qpc_can_use_reserved_lkey *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_can_use_reserved_lkey ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_can_use_reserved_lkey_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_can_use_reserved_lkey_dump(const struct connectib_hw_s_qpc_can_use_reserved_lkey *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_can_use_reserved_lkey_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_tcu_trap_pack(const struct connectib_hw_s_qpc_tcu_trap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 71;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_tcu_trap_unpack(struct connectib_hw_s_qpc_tcu_trap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 71;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_tcu_trap_print(const struct connectib_hw_s_qpc_tcu_trap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_tcu_trap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_tcu_trap_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_tcu_trap_dump(const struct connectib_hw_s_qpc_tcu_trap *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_tcu_trap_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_ack_req_frequency_pack(const struct connectib_hw_s_qpc_ack_req_frequency *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_ack_req_frequency_unpack(struct connectib_hw_s_qpc_ack_req_frequency *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 72;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_s_qpc_ack_req_frequency_print(const struct connectib_hw_s_qpc_ack_req_frequency *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_ack_req_frequency ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_ack_req_frequency_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_ack_req_frequency_dump(const struct connectib_hw_s_qpc_ack_req_frequency *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_ack_req_frequency_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_limited_qp_pack(const struct connectib_hw_s_qpc_limited_qp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 76;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_limited_qp_unpack(struct connectib_hw_s_qpc_limited_qp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 76;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_s_qpc_limited_qp_print(const struct connectib_hw_s_qpc_limited_qp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_limited_qp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_limited_qp_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_limited_qp_dump(const struct connectib_hw_s_qpc_limited_qp *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_limited_qp_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_dc_state_pack(const struct connectib_hw_s_qpc_dc_state *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 94;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_dc_state_unpack(struct connectib_hw_s_qpc_dc_state *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 94;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_s_qpc_dc_state_print(const struct connectib_hw_s_qpc_dc_state *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_dc_state ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_dc_state_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_dc_state_dump(const struct connectib_hw_s_qpc_dc_state *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_dc_state_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_currently_completing_wqe_index_pack(const struct connectib_hw_s_qpc_currently_completing_wqe_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_currently_completing_wqe_index_unpack(struct connectib_hw_s_qpc_currently_completing_wqe_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_s_qpc_currently_completing_wqe_index_print(const struct connectib_hw_s_qpc_currently_completing_wqe_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_currently_completing_wqe_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_currently_completing_wqe_index_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_currently_completing_wqe_index_dump(const struct connectib_hw_s_qpc_currently_completing_wqe_index *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_currently_completing_wqe_index_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_swqe_consumer_index_pack(const struct connectib_hw_s_qpc_swqe_consumer_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_swqe_consumer_index_unpack(struct connectib_hw_s_qpc_swqe_consumer_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 128;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_s_qpc_swqe_consumer_index_print(const struct connectib_hw_s_qpc_swqe_consumer_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_swqe_consumer_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_swqe_consumer_index_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_swqe_consumer_index_dump(const struct connectib_hw_s_qpc_swqe_consumer_index *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_swqe_consumer_index_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_qp_state_pack(const struct connectib_hw_s_qpc_qp_state *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_qp_state_unpack(struct connectib_hw_s_qpc_qp_state *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 144;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_s_qpc_qp_state_print(const struct connectib_hw_s_qpc_qp_state *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_qp_state ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_qp_state_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_qp_state_dump(const struct connectib_hw_s_qpc_qp_state *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_qp_state_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_psn_since_ackreq_pack(const struct connectib_hw_s_qpc_psn_since_ackreq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_psn_since_ackreq_unpack(struct connectib_hw_s_qpc_psn_since_ackreq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_s_qpc_psn_since_ackreq_print(const struct connectib_hw_s_qpc_psn_since_ackreq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_psn_since_ackreq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_psn_since_ackreq_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_psn_since_ackreq_dump(const struct connectib_hw_s_qpc_psn_since_ackreq *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_psn_since_ackreq_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_oldest_non_acked_psn_pack(const struct connectib_hw_s_qpc_oldest_non_acked_psn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_oldest_non_acked_psn_unpack(struct connectib_hw_s_qpc_oldest_non_acked_psn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 168;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_s_qpc_oldest_non_acked_psn_print(const struct connectib_hw_s_qpc_oldest_non_acked_psn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_oldest_non_acked_psn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_oldest_non_acked_psn_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_oldest_non_acked_psn_dump(const struct connectib_hw_s_qpc_oldest_non_acked_psn *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_oldest_non_acked_psn_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_deth_sqpn_pack(const struct connectib_hw_s_qpc_deth_sqpn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_deth_sqpn_unpack(struct connectib_hw_s_qpc_deth_sqpn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 168;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_s_qpc_deth_sqpn_print(const struct connectib_hw_s_qpc_deth_sqpn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_deth_sqpn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_deth_sqpn_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_deth_sqpn_dump(const struct connectib_hw_s_qpc_deth_sqpn *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_deth_sqpn_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_responder_session_id_pack(const struct connectib_hw_s_qpc_responder_session_id *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_responder_session_id_unpack(struct connectib_hw_s_qpc_responder_session_id *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_s_qpc_responder_session_id_print(const struct connectib_hw_s_qpc_responder_session_id *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_responder_session_id ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_responder_session_id_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_responder_session_id_dump(const struct connectib_hw_s_qpc_responder_session_id *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_responder_session_id_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_cq_number_pack(const struct connectib_hw_s_qpc_cq_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_cq_number_unpack(struct connectib_hw_s_qpc_cq_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 232;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_s_qpc_cq_number_print(const struct connectib_hw_s_qpc_cq_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_cq_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_cq_number_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_cq_number_dump(const struct connectib_hw_s_qpc_cq_number *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_cq_number_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_next_to_send_psn_pack(const struct connectib_hw_s_qpc_next_to_send_psn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_next_to_send_psn_unpack(struct connectib_hw_s_qpc_next_to_send_psn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 256;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_s_qpc_next_to_send_psn_print(const struct connectib_hw_s_qpc_next_to_send_psn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_next_to_send_psn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_next_to_send_psn_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_next_to_send_psn_dump(const struct connectib_hw_s_qpc_next_to_send_psn *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_next_to_send_psn_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_flush_syndrom_pack(const struct connectib_hw_s_qpc_flush_syndrom *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 280;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_flush_syndrom_unpack(struct connectib_hw_s_qpc_flush_syndrom *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 280;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_s_qpc_flush_syndrom_print(const struct connectib_hw_s_qpc_flush_syndrom *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_flush_syndrom ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_flush_syndrom_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_flush_syndrom_dump(const struct connectib_hw_s_qpc_flush_syndrom *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_flush_syndrom_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_counter_index_pack(const struct connectib_hw_s_qpc_counter_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_counter_index_unpack(struct connectib_hw_s_qpc_counter_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 288;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_s_qpc_counter_index_print(const struct connectib_hw_s_qpc_counter_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_counter_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_counter_index_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_counter_index_dump(const struct connectib_hw_s_qpc_counter_index *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_counter_index_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_ssn_pack(const struct connectib_hw_s_qpc_ssn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 296;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_ssn_unpack(struct connectib_hw_s_qpc_ssn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 296;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_s_qpc_ssn_print(const struct connectib_hw_s_qpc_ssn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_ssn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_ssn_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_ssn_dump(const struct connectib_hw_s_qpc_ssn *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_ssn_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_to_sequence_error_retry_counter_pack(const struct connectib_hw_s_qpc_to_sequence_error_retry_counter *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 329;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_to_sequence_error_retry_counter_unpack(struct connectib_hw_s_qpc_to_sequence_error_retry_counter *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 329;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_s_qpc_to_sequence_error_retry_counter_print(const struct connectib_hw_s_qpc_to_sequence_error_retry_counter *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_to_sequence_error_retry_counter ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_to_sequence_error_retry_counter_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_to_sequence_error_retry_counter_dump(const struct connectib_hw_s_qpc_to_sequence_error_retry_counter *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_to_sequence_error_retry_counter_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_reverse_sl_pack(const struct connectib_hw_s_qpc_reverse_sl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 332;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_reverse_sl_unpack(struct connectib_hw_s_qpc_reverse_sl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 332;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_s_qpc_reverse_sl_print(const struct connectib_hw_s_qpc_reverse_sl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_reverse_sl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_reverse_sl_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_reverse_sl_dump(const struct connectib_hw_s_qpc_reverse_sl *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_reverse_sl_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_force_grh_tclass_pack(const struct connectib_hw_s_qpc_force_grh_tclass *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 336;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_force_grh_tclass_unpack(struct connectib_hw_s_qpc_force_grh_tclass *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 336;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_force_grh_tclass_print(const struct connectib_hw_s_qpc_force_grh_tclass *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_force_grh_tclass ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_force_grh_tclass_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_force_grh_tclass_dump(const struct connectib_hw_s_qpc_force_grh_tclass *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_force_grh_tclass_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_ts_pack(const struct connectib_hw_s_qpc_ts *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_ts_unpack(struct connectib_hw_s_qpc_ts *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_s_qpc_ts_print(const struct connectib_hw_s_qpc_ts *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_ts ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_ts_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_ts_dump(const struct connectib_hw_s_qpc_ts *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_ts_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_sx_heartbeat_pack(const struct connectib_hw_s_qpc_sx_heartbeat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 356;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_sx_heartbeat_unpack(struct connectib_hw_s_qpc_sx_heartbeat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 356;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_sx_heartbeat_print(const struct connectib_hw_s_qpc_sx_heartbeat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_sx_heartbeat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_sx_heartbeat_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_sx_heartbeat_dump(const struct connectib_hw_s_qpc_sx_heartbeat *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_sx_heartbeat_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_to_sequence_error_retry_limit_pack(const struct connectib_hw_s_qpc_to_sequence_error_retry_limit *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 357;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_to_sequence_error_retry_limit_unpack(struct connectib_hw_s_qpc_to_sequence_error_retry_limit *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 357;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_s_qpc_to_sequence_error_retry_limit_print(const struct connectib_hw_s_qpc_to_sequence_error_retry_limit *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_to_sequence_error_retry_limit ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_to_sequence_error_retry_limit_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_to_sequence_error_retry_limit_dump(const struct connectib_hw_s_qpc_to_sequence_error_retry_limit *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_to_sequence_error_retry_limit_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_force_my_gid_index_pack(const struct connectib_hw_s_qpc_force_my_gid_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_force_my_gid_index_unpack(struct connectib_hw_s_qpc_force_my_gid_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_force_my_gid_index_print(const struct connectib_hw_s_qpc_force_my_gid_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_force_my_gid_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_force_my_gid_index_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_force_my_gid_index_dump(const struct connectib_hw_s_qpc_force_my_gid_index *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_force_my_gid_index_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_sx_gather_commit_pack(const struct connectib_hw_s_qpc_sx_gather_commit *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 361;
	adb2c_push_bits_to_buff(ptr_buff, offset, 23, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_sx_gather_commit_unpack(struct connectib_hw_s_qpc_sx_gather_commit *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 361;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 23);

}

void connectib_hw_s_qpc_sx_gather_commit_print(const struct connectib_hw_s_qpc_sx_gather_commit *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_sx_gather_commit ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_sx_gather_commit_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_sx_gather_commit_dump(const struct connectib_hw_s_qpc_sx_gather_commit *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_sx_gather_commit_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_lsn_pack(const struct connectib_hw_s_qpc_lsn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 392;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_lsn_unpack(struct connectib_hw_s_qpc_lsn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 392;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_s_qpc_lsn_print(const struct connectib_hw_s_qpc_lsn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_lsn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_lsn_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_lsn_dump(const struct connectib_hw_s_qpc_lsn *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_lsn_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_dc_access_key_63_32_pack(const struct connectib_hw_s_qpc_dc_access_key_63_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_dc_access_key_63_32_unpack(struct connectib_hw_s_qpc_dc_access_key_63_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_s_qpc_dc_access_key_63_32_print(const struct connectib_hw_s_qpc_dc_access_key_63_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_dc_access_key_63_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_dc_access_key_63_32_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_dc_access_key_63_32_dump(const struct connectib_hw_s_qpc_dc_access_key_63_32 *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_dc_access_key_63_32_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_dc_access_key_31_0_pack(const struct connectib_hw_s_qpc_dc_access_key_31_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_dc_access_key_31_0_unpack(struct connectib_hw_s_qpc_dc_access_key_31_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_s_qpc_dc_access_key_31_0_print(const struct connectib_hw_s_qpc_dc_access_key_31_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_dc_access_key_31_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_dc_access_key_31_0_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_dc_access_key_31_0_dump(const struct connectib_hw_s_qpc_dc_access_key_31_0 *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_dc_access_key_31_0_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_force_grh_pack(const struct connectib_hw_s_qpc_force_grh *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_force_grh_unpack(struct connectib_hw_s_qpc_force_grh *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_force_grh_print(const struct connectib_hw_s_qpc_force_grh *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_force_grh ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_force_grh_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_force_grh_dump(const struct connectib_hw_s_qpc_force_grh *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_force_grh_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_vl15_pack(const struct connectib_hw_s_qpc_vl15 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 449;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_vl15_unpack(struct connectib_hw_s_qpc_vl15 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 449;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_vl15_print(const struct connectib_hw_s_qpc_vl15 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_vl15 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_vl15_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_vl15_dump(const struct connectib_hw_s_qpc_vl15 *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_vl15_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_timer_enable_pack(const struct connectib_hw_s_qpc_timer_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 449;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_timer_enable_unpack(struct connectib_hw_s_qpc_timer_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 449;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_s_qpc_timer_enable_print(const struct connectib_hw_s_qpc_timer_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_timer_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_timer_enable_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_timer_enable_dump(const struct connectib_hw_s_qpc_timer_enable *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_timer_enable_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_currently_completing_ds_pack(const struct connectib_hw_s_qpc_currently_completing_ds *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 450;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_currently_completing_ds_unpack(struct connectib_hw_s_qpc_currently_completing_ds *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 450;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

}

void connectib_hw_s_qpc_currently_completing_ds_print(const struct connectib_hw_s_qpc_currently_completing_ds *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_currently_completing_ds ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_currently_completing_ds_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_currently_completing_ds_dump(const struct connectib_hw_s_qpc_currently_completing_ds *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_currently_completing_ds_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_first_psn_of_message_currently_completing_pack(const struct connectib_hw_s_qpc_first_psn_of_message_currently_completing *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 456;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_first_psn_of_message_currently_completing_unpack(struct connectib_hw_s_qpc_first_psn_of_message_currently_completing *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 456;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_s_qpc_first_psn_of_message_currently_completing_print(const struct connectib_hw_s_qpc_first_psn_of_message_currently_completing *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_first_psn_of_message_currently_completing ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_first_psn_of_message_currently_completing_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_first_psn_of_message_currently_completing_dump(const struct connectib_hw_s_qpc_first_psn_of_message_currently_completing *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_first_psn_of_message_currently_completing_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_number_outstanding_reads_pack(const struct connectib_hw_s_qpc_number_outstanding_reads *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_number_outstanding_reads_unpack(struct connectib_hw_s_qpc_number_outstanding_reads *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_s_qpc_number_outstanding_reads_print(const struct connectib_hw_s_qpc_number_outstanding_reads *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_number_outstanding_reads ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_number_outstanding_reads_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_number_outstanding_reads_dump(const struct connectib_hw_s_qpc_number_outstanding_reads *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_number_outstanding_reads_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_first_unsent_psn_pack(const struct connectib_hw_s_qpc_first_unsent_psn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 488;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_s_qpc_first_unsent_psn_unpack(struct connectib_hw_s_qpc_first_unsent_psn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 488;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_s_qpc_first_unsent_psn_print(const struct connectib_hw_s_qpc_first_unsent_psn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc_first_unsent_psn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_s_qpc_first_unsent_psn_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_first_unsent_psn_dump(const struct connectib_hw_s_qpc_first_unsent_psn *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_first_unsent_psn_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_common_valid_pack(const struct connectib_hw_c_qpc_common_valid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_common_valid_unpack(struct connectib_hw_c_qpc_common_valid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_c_qpc_common_valid_print(const struct connectib_hw_c_qpc_common_valid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_common_valid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_common_valid_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_common_valid_dump(const struct connectib_hw_c_qpc_common_valid *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_common_valid_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_wq_signature_pack(const struct connectib_hw_c_qpc_wq_signature *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_wq_signature_unpack(struct connectib_hw_c_qpc_wq_signature *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_c_qpc_wq_signature_print(const struct connectib_hw_c_qpc_wq_signature *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_wq_signature ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_wq_signature_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_wq_signature_dump(const struct connectib_hw_c_qpc_wq_signature *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_wq_signature_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_dual_write_pack(const struct connectib_hw_c_qpc_dual_write *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_dual_write_unpack(struct connectib_hw_c_qpc_dual_write *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_c_qpc_dual_write_print(const struct connectib_hw_c_qpc_dual_write *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_dual_write ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_dual_write_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_dual_write_dump(const struct connectib_hw_c_qpc_dual_write *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_dual_write_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_log_page_size_pack(const struct connectib_hw_c_qpc_log_page_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_log_page_size_unpack(struct connectib_hw_c_qpc_log_page_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_c_qpc_log_page_size_print(const struct connectib_hw_c_qpc_log_page_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_log_page_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_log_page_size_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_log_page_size_dump(const struct connectib_hw_c_qpc_log_page_size *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_log_page_size_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_rwq_type_pack(const struct connectib_hw_c_qpc_rwq_type *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_rwq_type_unpack(struct connectib_hw_c_qpc_rwq_type *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_c_qpc_rwq_type_print(const struct connectib_hw_c_qpc_rwq_type *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_rwq_type ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_rwq_type_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_rwq_type_dump(const struct connectib_hw_c_qpc_rwq_type *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_rwq_type_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_force_loopback_pack(const struct connectib_hw_c_qpc_force_loopback *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_force_loopback_unpack(struct connectib_hw_c_qpc_force_loopback *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_c_qpc_force_loopback_print(const struct connectib_hw_c_qpc_force_loopback *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_force_loopback ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_force_loopback_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_force_loopback_dump(const struct connectib_hw_c_qpc_force_loopback *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_force_loopback_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_ipd_pack(const struct connectib_hw_c_qpc_ipd *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_ipd_unpack(struct connectib_hw_c_qpc_ipd *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 44;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_ipd_print(const struct connectib_hw_c_qpc_ipd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_ipd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_ipd_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_ipd_dump(const struct connectib_hw_c_qpc_ipd *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_ipd_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_rwqe_stride_pack(const struct connectib_hw_c_qpc_rwqe_stride *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 49;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_rwqe_stride_unpack(struct connectib_hw_c_qpc_rwqe_stride *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 49;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_c_qpc_rwqe_stride_print(const struct connectib_hw_c_qpc_rwqe_stride *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_rwqe_stride ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_rwqe_stride_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_rwqe_stride_dump(const struct connectib_hw_c_qpc_rwqe_stride *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_rwqe_stride_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_rcyclic_size_pack(const struct connectib_hw_c_qpc_rcyclic_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 51;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_rcyclic_size_unpack(struct connectib_hw_c_qpc_rcyclic_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 51;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_c_qpc_rcyclic_size_print(const struct connectib_hw_c_qpc_rcyclic_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_rcyclic_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_rcyclic_size_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_rcyclic_size_dump(const struct connectib_hw_c_qpc_rcyclic_size *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_rcyclic_size_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_rwqe_buffer_size_pack(const struct connectib_hw_c_qpc_rwqe_buffer_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_rwqe_buffer_size_unpack(struct connectib_hw_c_qpc_rwqe_buffer_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 52;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_rwqe_buffer_size_print(const struct connectib_hw_c_qpc_rwqe_buffer_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_rwqe_buffer_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_rwqe_buffer_size_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_rwqe_buffer_size_dump(const struct connectib_hw_c_qpc_rwqe_buffer_size *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_rwqe_buffer_size_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_tclass_pack(const struct connectib_hw_c_qpc_tclass *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_tclass_unpack(struct connectib_hw_c_qpc_tclass *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 56;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_c_qpc_tclass_print(const struct connectib_hw_c_qpc_tclass *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_tclass ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_tclass_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_tclass_dump(const struct connectib_hw_c_qpc_tclass *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_tclass_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_hop_limit_pack(const struct connectib_hw_c_qpc_hop_limit *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_hop_limit_unpack(struct connectib_hw_c_qpc_hop_limit *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_c_qpc_hop_limit_print(const struct connectib_hw_c_qpc_hop_limit *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_hop_limit ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_hop_limit_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_hop_limit_dump(const struct connectib_hw_c_qpc_hop_limit *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_hop_limit_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_peer_qp_pack(const struct connectib_hw_c_qpc_peer_qp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_peer_qp_unpack(struct connectib_hw_c_qpc_peer_qp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 72;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_c_qpc_peer_qp_print(const struct connectib_hw_c_qpc_peer_qp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_peer_qp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_peer_qp_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_peer_qp_dump(const struct connectib_hw_c_qpc_peer_qp *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_peer_qp_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_rx_migreq_pack(const struct connectib_hw_c_qpc_rx_migreq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_rx_migreq_unpack(struct connectib_hw_c_qpc_rx_migreq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_c_qpc_rx_migreq_print(const struct connectib_hw_c_qpc_rx_migreq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_rx_migreq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_rx_migreq_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_rx_migreq_dump(const struct connectib_hw_c_qpc_rx_migreq *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_rx_migreq_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_sx_migreq_pack(const struct connectib_hw_c_qpc_sx_migreq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 97;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_sx_migreq_unpack(struct connectib_hw_c_qpc_sx_migreq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 97;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_c_qpc_sx_migreq_print(const struct connectib_hw_c_qpc_sx_migreq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_sx_migreq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_sx_migreq_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_sx_migreq_dump(const struct connectib_hw_c_qpc_sx_migreq *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_sx_migreq_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_wqe_page_offset_pack(const struct connectib_hw_c_qpc_wqe_page_offset *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 98;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_wqe_page_offset_unpack(struct connectib_hw_c_qpc_wqe_page_offset *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 98;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

}

void connectib_hw_c_qpc_wqe_page_offset_print(const struct connectib_hw_c_qpc_wqe_page_offset *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_wqe_page_offset ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_wqe_page_offset_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_wqe_page_offset_dump(const struct connectib_hw_c_qpc_wqe_page_offset *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_wqe_page_offset_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_context_number_pack(const struct connectib_hw_c_qpc_context_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_context_number_unpack(struct connectib_hw_c_qpc_context_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 104;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_c_qpc_context_number_print(const struct connectib_hw_c_qpc_context_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_context_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_context_number_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_context_number_dump(const struct connectib_hw_c_qpc_context_number *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_context_number_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_pkey_table_index_pack(const struct connectib_hw_c_qpc_pkey_table_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_pkey_table_index_unpack(struct connectib_hw_c_qpc_pkey_table_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 128;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

}

void connectib_hw_c_qpc_pkey_table_index_print(const struct connectib_hw_c_qpc_pkey_table_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_pkey_table_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_pkey_table_index_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_pkey_table_index_dump(const struct connectib_hw_c_qpc_pkey_table_index *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_pkey_table_index_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_flow_label_pack(const struct connectib_hw_c_qpc_flow_label *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 140;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_flow_label_unpack(struct connectib_hw_c_qpc_flow_label *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 140;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

}

void connectib_hw_c_qpc_flow_label_print(const struct connectib_hw_c_qpc_flow_label *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_flow_label ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_flow_label_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_flow_label_dump(const struct connectib_hw_c_qpc_flow_label *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_flow_label_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_qkey_pack(const struct connectib_hw_c_qpc_qkey *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_qkey_unpack(struct connectib_hw_c_qpc_qkey *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_qkey_print(const struct connectib_hw_c_qpc_qkey *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_qkey ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_qkey_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_qkey_dump(const struct connectib_hw_c_qpc_qkey *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_qkey_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_mtu_pack(const struct connectib_hw_c_qpc_mtu *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_mtu_unpack(struct connectib_hw_c_qpc_mtu *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 192;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_c_qpc_mtu_print(const struct connectib_hw_c_qpc_mtu *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_mtu ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_mtu_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_mtu_dump(const struct connectib_hw_c_qpc_mtu *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_mtu_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_max_message_size_pack(const struct connectib_hw_c_qpc_max_message_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 195;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_max_message_size_unpack(struct connectib_hw_c_qpc_max_message_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 195;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_c_qpc_max_message_size_print(const struct connectib_hw_c_qpc_max_message_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_max_message_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_max_message_size_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_max_message_size_dump(const struct connectib_hw_c_qpc_max_message_size *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_max_message_size_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_my_gid_mac_index_pack(const struct connectib_hw_c_qpc_my_gid_mac_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_my_gid_mac_index_unpack(struct connectib_hw_c_qpc_my_gid_mac_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_c_qpc_my_gid_mac_index_print(const struct connectib_hw_c_qpc_my_gid_mac_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_my_gid_mac_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_my_gid_mac_index_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_my_gid_mac_index_dump(const struct connectib_hw_c_qpc_my_gid_mac_index *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_my_gid_mac_index_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_my_lid_pack(const struct connectib_hw_c_qpc_my_lid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 233;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_my_lid_unpack(struct connectib_hw_c_qpc_my_lid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 233;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

}

void connectib_hw_c_qpc_my_lid_print(const struct connectib_hw_c_qpc_my_lid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_my_lid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_my_lid_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_my_lid_dump(const struct connectib_hw_c_qpc_my_lid *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_my_lid_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_peer_lid_pack(const struct connectib_hw_c_qpc_peer_lid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_peer_lid_unpack(struct connectib_hw_c_qpc_peer_lid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 240;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_c_qpc_peer_lid_print(const struct connectib_hw_c_qpc_peer_lid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_peer_lid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_peer_lid_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_peer_lid_dump(const struct connectib_hw_c_qpc_peer_lid *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_peer_lid_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_swqe_buffer_size_pack(const struct connectib_hw_c_qpc_swqe_buffer_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 260;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_swqe_buffer_size_unpack(struct connectib_hw_c_qpc_swqe_buffer_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 260;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_swqe_buffer_size_print(const struct connectib_hw_c_qpc_swqe_buffer_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_swqe_buffer_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_swqe_buffer_size_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_swqe_buffer_size_dump(const struct connectib_hw_c_qpc_swqe_buffer_size *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_swqe_buffer_size_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_adaptive_routing_pack(const struct connectib_hw_c_qpc_adaptive_routing *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 265;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_adaptive_routing_unpack(struct connectib_hw_c_qpc_adaptive_routing *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 265;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_c_qpc_adaptive_routing_print(const struct connectib_hw_c_qpc_adaptive_routing *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_adaptive_routing ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_adaptive_routing_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_adaptive_routing_dump(const struct connectib_hw_c_qpc_adaptive_routing *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_adaptive_routing_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_port_number_pack(const struct connectib_hw_c_qpc_port_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 266;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_port_number_unpack(struct connectib_hw_c_qpc_port_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 266;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_c_qpc_port_number_print(const struct connectib_hw_c_qpc_port_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_port_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_port_number_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_port_number_dump(const struct connectib_hw_c_qpc_port_number *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_port_number_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_grh_bit_pack(const struct connectib_hw_c_qpc_grh_bit *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 267;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_grh_bit_unpack(struct connectib_hw_c_qpc_grh_bit *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 267;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_c_qpc_grh_bit_print(const struct connectib_hw_c_qpc_grh_bit *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_grh_bit ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_grh_bit_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_grh_bit_dump(const struct connectib_hw_c_qpc_grh_bit *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_grh_bit_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_sl_pack(const struct connectib_hw_c_qpc_sl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 268;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_sl_unpack(struct connectib_hw_c_qpc_sl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 268;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_sl_print(const struct connectib_hw_c_qpc_sl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_sl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_sl_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_sl_dump(const struct connectib_hw_c_qpc_sl *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_sl_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_mtt_pointer_39_37_pack(const struct connectib_hw_c_qpc_mtt_pointer_39_37 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 317;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_mtt_pointer_39_37_unpack(struct connectib_hw_c_qpc_mtt_pointer_39_37 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 317;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_c_qpc_mtt_pointer_39_37_print(const struct connectib_hw_c_qpc_mtt_pointer_39_37 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_mtt_pointer_39_37 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_mtt_pointer_39_37_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_mtt_pointer_39_37_dump(const struct connectib_hw_c_qpc_mtt_pointer_39_37 *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_mtt_pointer_39_37_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_mtt_pointer_36_5_pack(const struct connectib_hw_c_qpc_mtt_pointer_36_5 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_mtt_pointer_36_5_unpack(struct connectib_hw_c_qpc_mtt_pointer_36_5 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 320;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_mtt_pointer_36_5_print(const struct connectib_hw_c_qpc_mtt_pointer_36_5 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_mtt_pointer_36_5 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_mtt_pointer_36_5_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_mtt_pointer_36_5_dump(const struct connectib_hw_c_qpc_mtt_pointer_36_5 *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_mtt_pointer_36_5_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_mtt_pointer_4_0_pack(const struct connectib_hw_c_qpc_mtt_pointer_4_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_mtt_pointer_4_0_unpack(struct connectib_hw_c_qpc_mtt_pointer_4_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_c_qpc_mtt_pointer_4_0_print(const struct connectib_hw_c_qpc_mtt_pointer_4_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_mtt_pointer_4_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_mtt_pointer_4_0_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_mtt_pointer_4_0_dump(const struct connectib_hw_c_qpc_mtt_pointer_4_0 *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_mtt_pointer_4_0_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_pd_pack(const struct connectib_hw_c_qpc_pd *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_pd_unpack(struct connectib_hw_c_qpc_pd *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_c_qpc_pd_print(const struct connectib_hw_c_qpc_pd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_pd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_pd_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_pd_dump(const struct connectib_hw_c_qpc_pd *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_pd_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_peer_qp2_pack(const struct connectib_hw_c_qpc_peer_qp2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 456;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_peer_qp2_unpack(struct connectib_hw_c_qpc_peer_qp2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 456;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_c_qpc_peer_qp2_print(const struct connectib_hw_c_qpc_peer_qp2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_peer_qp2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_peer_qp2_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_peer_qp2_dump(const struct connectib_hw_c_qpc_peer_qp2 *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_peer_qp2_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_peer_lid2_pack(const struct connectib_hw_c_qpc_peer_lid2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_peer_lid2_unpack(struct connectib_hw_c_qpc_peer_lid2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 496;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_c_qpc_peer_lid2_print(const struct connectib_hw_c_qpc_peer_lid2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_peer_lid2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_peer_lid2_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_peer_lid2_dump(const struct connectib_hw_c_qpc_peer_lid2 *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_peer_lid2_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_peer_gid_127_96_pack(const struct connectib_hw_c_qpc_peer_gid_127_96 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_peer_gid_127_96_unpack(struct connectib_hw_c_qpc_peer_gid_127_96 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_peer_gid_127_96_print(const struct connectib_hw_c_qpc_peer_gid_127_96 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_peer_gid_127_96 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_peer_gid_127_96_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_peer_gid_127_96_dump(const struct connectib_hw_c_qpc_peer_gid_127_96 *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_peer_gid_127_96_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_peer_gid_95_64_pack(const struct connectib_hw_c_qpc_peer_gid_95_64 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_peer_gid_95_64_unpack(struct connectib_hw_c_qpc_peer_gid_95_64 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_peer_gid_95_64_print(const struct connectib_hw_c_qpc_peer_gid_95_64 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_peer_gid_95_64 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_peer_gid_95_64_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_peer_gid_95_64_dump(const struct connectib_hw_c_qpc_peer_gid_95_64 *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_peer_gid_95_64_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_peer_gid_63_32_pack(const struct connectib_hw_c_qpc_peer_gid_63_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_peer_gid_63_32_unpack(struct connectib_hw_c_qpc_peer_gid_63_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_peer_gid_63_32_print(const struct connectib_hw_c_qpc_peer_gid_63_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_peer_gid_63_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_peer_gid_63_32_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_peer_gid_63_32_dump(const struct connectib_hw_c_qpc_peer_gid_63_32 *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_peer_gid_63_32_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_peer_gid_31_0_pack(const struct connectib_hw_c_qpc_peer_gid_31_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_c_qpc_peer_gid_31_0_unpack(struct connectib_hw_c_qpc_peer_gid_31_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_c_qpc_peer_gid_31_0_print(const struct connectib_hw_c_qpc_peer_gid_31_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc_peer_gid_31_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_c_qpc_peer_gid_31_0_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_peer_gid_31_0_dump(const struct connectib_hw_c_qpc_peer_gid_31_0 *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_peer_gid_31_0_print(ptr_struct, file, 0);
}

void connectib_e2e_pa_33_2_pack(const struct connectib_e2e_pa_33_2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_e2e_pa_33_2_unpack(struct connectib_e2e_pa_33_2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_e2e_pa_33_2_print(const struct connectib_e2e_pa_33_2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_e2e_pa_33_2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_e2e_pa_33_2_size(void)
{
	 return 64;
}

void connectib_e2e_pa_33_2_dump(const struct connectib_e2e_pa_33_2 *ptr_struct, FILE* file)
{
	connectib_e2e_pa_33_2_print(ptr_struct, file, 0);
}

void connectib_wq_signature_pack(const struct connectib_wq_signature *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_wq_signature_unpack(struct connectib_wq_signature *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_wq_signature_print(const struct connectib_wq_signature *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wq_signature ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_wq_signature_size(void)
{
	 return 64;
}

void connectib_wq_signature_dump(const struct connectib_wq_signature *ptr_struct, FILE* file)
{
	connectib_wq_signature_print(ptr_struct, file, 0);
}

void connectib_srq_valid_pack(const struct connectib_srq_valid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 449;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_srq_valid_unpack(struct connectib_srq_valid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 449;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_srq_valid_print(const struct connectib_srq_valid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_srq_valid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_srq_valid_size(void)
{
	 return 64;
}

void connectib_srq_valid_dump(const struct connectib_srq_valid *ptr_struct, FILE* file)
{
	connectib_srq_valid_print(ptr_struct, file, 0);
}

void connectib_e2e_pa_63_34_pack(const struct connectib_e2e_pa_63_34 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 450;
	adb2c_push_bits_to_buff(ptr_buff, offset, 30, (u_int32_t)ptr_struct->val);

}

void connectib_e2e_pa_63_34_unpack(struct connectib_e2e_pa_63_34 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 450;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 30);

}

void connectib_e2e_pa_63_34_print(const struct connectib_e2e_pa_63_34 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_e2e_pa_63_34 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_e2e_pa_63_34_size(void)
{
	 return 64;
}

void connectib_e2e_pa_63_34_dump(const struct connectib_e2e_pa_63_34 *ptr_struct, FILE* file)
{
	connectib_e2e_pa_63_34_print(ptr_struct, file, 0);
}

void connectib_mtt_pointer_31_0_pack(const struct connectib_mtt_pointer_31_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_mtt_pointer_31_0_unpack(struct connectib_mtt_pointer_31_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_mtt_pointer_31_0_print(const struct connectib_mtt_pointer_31_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mtt_pointer_31_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_mtt_pointer_31_0_size(void)
{
	 return 64;
}

void connectib_mtt_pointer_31_0_dump(const struct connectib_mtt_pointer_31_0 *ptr_struct, FILE* file)
{
	connectib_mtt_pointer_31_0_print(ptr_struct, file, 0);
}

void connectib_rwqe_limit_event_pack(const struct connectib_rwqe_limit_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_rwqe_limit_event_unpack(struct connectib_rwqe_limit_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_rwqe_limit_event_print(const struct connectib_rwqe_limit_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rwqe_limit_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_rwqe_limit_event_size(void)
{
	 return 64;
}

void connectib_rwqe_limit_event_dump(const struct connectib_rwqe_limit_event *ptr_struct, FILE* file)
{
	connectib_rwqe_limit_event_print(ptr_struct, file, 0);
}

void connectib_e2e_fetch_enable_pack(const struct connectib_e2e_fetch_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_e2e_fetch_enable_unpack(struct connectib_e2e_fetch_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 400;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_e2e_fetch_enable_print(const struct connectib_e2e_fetch_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_e2e_fetch_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_e2e_fetch_enable_size(void)
{
	 return 64;
}

void connectib_e2e_fetch_enable_dump(const struct connectib_e2e_fetch_enable *ptr_struct, FILE* file)
{
	connectib_e2e_fetch_enable_print(ptr_struct, file, 0);
}

void connectib_rx_heartbeat_pack(const struct connectib_rx_heartbeat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 401;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_rx_heartbeat_unpack(struct connectib_rx_heartbeat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 401;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_rx_heartbeat_print(const struct connectib_rx_heartbeat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rx_heartbeat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_rx_heartbeat_size(void)
{
	 return 64;
}

void connectib_rx_heartbeat_dump(const struct connectib_rx_heartbeat *ptr_struct, FILE* file)
{
	connectib_rx_heartbeat_print(ptr_struct, file, 0);
}

void connectib_wqe_page_offset_pack(const struct connectib_wqe_page_offset *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 402;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->val);

}

void connectib_wqe_page_offset_unpack(struct connectib_wqe_page_offset *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 402;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

}

void connectib_wqe_page_offset_print(const struct connectib_wqe_page_offset *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_page_offset ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_wqe_page_offset_size(void)
{
	 return 64;
}

void connectib_wqe_page_offset_dump(const struct connectib_wqe_page_offset *ptr_struct, FILE* file)
{
	connectib_wqe_page_offset_print(ptr_struct, file, 0);
}

void connectib_mtt_pointer_39_32_pack(const struct connectib_mtt_pointer_39_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 408;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_mtt_pointer_39_32_unpack(struct connectib_mtt_pointer_39_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 408;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_mtt_pointer_39_32_print(const struct connectib_mtt_pointer_39_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mtt_pointer_39_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_mtt_pointer_39_32_size(void)
{
	 return 64;
}

void connectib_mtt_pointer_39_32_dump(const struct connectib_mtt_pointer_39_32 *ptr_struct, FILE* file)
{
	connectib_mtt_pointer_39_32_print(ptr_struct, file, 0);
}

void connectib_tcu_trap_pack(const struct connectib_tcu_trap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_tcu_trap_unpack(struct connectib_tcu_trap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_tcu_trap_print(const struct connectib_tcu_trap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_tcu_trap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_tcu_trap_size(void)
{
	 return 64;
}

void connectib_tcu_trap_dump(const struct connectib_tcu_trap *ptr_struct, FILE* file)
{
	connectib_tcu_trap_print(ptr_struct, file, 0);
}

void connectib_can_use_reserved_lkey_pack(const struct connectib_can_use_reserved_lkey *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 353;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_can_use_reserved_lkey_unpack(struct connectib_can_use_reserved_lkey *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 353;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_can_use_reserved_lkey_print(const struct connectib_can_use_reserved_lkey *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_can_use_reserved_lkey ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_can_use_reserved_lkey_size(void)
{
	 return 64;
}

void connectib_can_use_reserved_lkey_dump(const struct connectib_can_use_reserved_lkey *ptr_struct, FILE* file)
{
	connectib_can_use_reserved_lkey_print(ptr_struct, file, 0);
}

void connectib_tcu_drop_pack(const struct connectib_tcu_drop *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 354;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_tcu_drop_unpack(struct connectib_tcu_drop *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 354;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_tcu_drop_print(const struct connectib_tcu_drop *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_tcu_drop ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_tcu_drop_size(void)
{
	 return 64;
}

void connectib_tcu_drop_dump(const struct connectib_tcu_drop *ptr_struct, FILE* file)
{
	connectib_tcu_drop_print(ptr_struct, file, 0);
}

void connectib_log_page_size_pack(const struct connectib_log_page_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 355;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_log_page_size_unpack(struct connectib_log_page_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 355;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_log_page_size_print(const struct connectib_log_page_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_log_page_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_log_page_size_size(void)
{
	 return 64;
}

void connectib_log_page_size_dump(const struct connectib_log_page_size *ptr_struct, FILE* file)
{
	connectib_log_page_size_print(ptr_struct, file, 0);
}

void connectib_context_number_pack(const struct connectib_context_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_context_number_unpack(struct connectib_context_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_context_number_print(const struct connectib_context_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_context_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_context_number_size(void)
{
	 return 64;
}

void connectib_context_number_dump(const struct connectib_context_number *ptr_struct, FILE* file)
{
	connectib_context_number_print(ptr_struct, file, 0);
}

void connectib_rwqe_stride_pack(const struct connectib_rwqe_stride *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 321;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_rwqe_stride_unpack(struct connectib_rwqe_stride *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 321;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_rwqe_stride_print(const struct connectib_rwqe_stride *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rwqe_stride ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_rwqe_stride_size(void)
{
	 return 64;
}

void connectib_rwqe_stride_dump(const struct connectib_rwqe_stride *ptr_struct, FILE* file)
{
	connectib_rwqe_stride_print(ptr_struct, file, 0);
}

void connectib_rwqe_buffer_size_pack(const struct connectib_rwqe_buffer_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 324;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_rwqe_buffer_size_unpack(struct connectib_rwqe_buffer_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 324;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_rwqe_buffer_size_print(const struct connectib_rwqe_buffer_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rwqe_buffer_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_rwqe_buffer_size_size(void)
{
	 return 64;
}

void connectib_rwqe_buffer_size_dump(const struct connectib_rwqe_buffer_size *ptr_struct, FILE* file)
{
	connectib_rwqe_buffer_size_print(ptr_struct, file, 0);
}

void connectib_cq_number_pack(const struct connectib_cq_number *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 328;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_cq_number_unpack(struct connectib_cq_number *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 328;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_cq_number_print(const struct connectib_cq_number *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cq_number ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_cq_number_size(void)
{
	 return 64;
}

void connectib_cq_number_dump(const struct connectib_cq_number *ptr_struct, FILE* file)
{
	connectib_cq_number_print(ptr_struct, file, 0);
}

void connectib_e2e_credits_prefetch_threshold_pack(const struct connectib_e2e_credits_prefetch_threshold *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 293;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_e2e_credits_prefetch_threshold_unpack(struct connectib_e2e_credits_prefetch_threshold *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 293;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_e2e_credits_prefetch_threshold_print(const struct connectib_e2e_credits_prefetch_threshold *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_e2e_credits_prefetch_threshold ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_e2e_credits_prefetch_threshold_size(void)
{
	 return 64;
}

void connectib_e2e_credits_prefetch_threshold_dump(const struct connectib_e2e_credits_prefetch_threshold *ptr_struct, FILE* file)
{
	connectib_e2e_credits_prefetch_threshold_print(ptr_struct, file, 0);
}

void connectib_pd_pack(const struct connectib_pd *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 296;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_pd_unpack(struct connectib_pd *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 296;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_pd_print(const struct connectib_pd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_pd_size(void)
{
	 return 64;
}

void connectib_pd_dump(const struct connectib_pd *ptr_struct, FILE* file)
{
	connectib_pd_print(ptr_struct, file, 0);
}

void connectib_xrc_domain_pack(const struct connectib_xrc_domain *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 264;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_xrc_domain_unpack(struct connectib_xrc_domain *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 264;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_xrc_domain_print(const struct connectib_xrc_domain *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_xrc_domain ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_xrc_domain_size(void)
{
	 return 64;
}

void connectib_xrc_domain_dump(const struct connectib_xrc_domain *ptr_struct, FILE* file)
{
	connectib_xrc_domain_print(ptr_struct, file, 0);
}

void connectib_e2e_producer_index_pack(const struct connectib_e2e_producer_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_e2e_producer_index_unpack(struct connectib_e2e_producer_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 224;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_e2e_producer_index_print(const struct connectib_e2e_producer_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_e2e_producer_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_e2e_producer_index_size(void)
{
	 return 64;
}

void connectib_e2e_producer_index_dump(const struct connectib_e2e_producer_index *ptr_struct, FILE* file)
{
	connectib_e2e_producer_index_print(ptr_struct, file, 0);
}

void connectib_rwqe_consumer_index_pack(const struct connectib_rwqe_consumer_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_rwqe_consumer_index_unpack(struct connectib_rwqe_consumer_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 240;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_rwqe_consumer_index_print(const struct connectib_rwqe_consumer_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rwqe_consumer_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_rwqe_consumer_index_size(void)
{
	 return 64;
}

void connectib_rwqe_consumer_index_dump(const struct connectib_rwqe_consumer_index *ptr_struct, FILE* file)
{
	connectib_rwqe_consumer_index_print(ptr_struct, file, 0);
}

void connectib_srq_fw_pack(const struct connectib_srq_fw *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 198;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->val);

}

void connectib_srq_fw_unpack(struct connectib_srq_fw *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 198;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

}

void connectib_srq_fw_print(const struct connectib_srq_fw *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_srq_fw ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_srq_fw_size(void)
{
	 return 64;
}

void connectib_srq_fw_dump(const struct connectib_srq_fw *ptr_struct, FILE* file)
{
	connectib_srq_fw_print(ptr_struct, file, 0);
}

void connectib_e2e_consumer_index_pack(const struct connectib_e2e_consumer_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_e2e_consumer_index_unpack(struct connectib_e2e_consumer_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 208;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_e2e_consumer_index_print(const struct connectib_e2e_consumer_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_e2e_consumer_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_e2e_consumer_index_size(void)
{
	 return 64;
}

void connectib_e2e_consumer_index_dump(const struct connectib_e2e_consumer_index *ptr_struct, FILE* file)
{
	connectib_e2e_consumer_index_print(ptr_struct, file, 0);
}

void connectib_pf_vf_cfg_space_pack(const struct connectib_pf_vf_cfg_space *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_pf_vf_sriov_pack(&(ptr_struct->sriov), ptr_buff + offset/8);

	offset = 64;
	connectib_pf_vf_vpd_pack(&(ptr_struct->vpd), ptr_buff + offset/8);

	offset = 150;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->hw_msix_vec0);

	offset = 149;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->msix_func_mask);

	offset = 148;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->msix_enable);

	offset = 147;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bus_master_en);

	offset = 146;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->memory_space);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->serr_status);

	offset = 143;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->serr_enable);

	offset = 142;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->perr_enable);

	offset = 141;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->int_disable);

	offset = 140;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->need_to_fire_flr_event);

	offset = 139;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->in_middle_of_flr);

	offset = 138;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->hide);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->power_state);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cache_line);

	offset = 184;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->interrupt_line);

	offset = 181;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->exp_rom_en);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 21, (u_int32_t)ptr_struct->exp_rom_addr_31_11);

	offset = 223;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_control_correctable_error_report_en);

	offset = 222;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_control_nonfatal_error_report_en);

	offset = 221;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_control_fatal_error_report_en);

	offset = 220;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_control_ur_error_report_en);

	offset = 219;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_control_enable_relaxed_ordering);

	offset = 216;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->device_control_max_payload_size);

	offset = 215;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_control_extended_tag_en);

	offset = 214;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_control_enable_no_snoop);

	offset = 211;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->device_control_max_read_request_size);

	offset = 210;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_status_correctable_error_detected);

	offset = 209;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_status_nonfatal_error_detected);

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_status_fatal_error_detected);

	offset = 207;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_status_ur_error_detected);

	offset = 206;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->command_register_memory_space);

	offset = 205;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->command_register_bus_master_en);

	offset = 204;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->command_register_perr_report_en);

	offset = 203;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->command_register_serr_report_en);

	offset = 202;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->command_register_interrupt_disable);

	offset = 201;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->status_register_interrupt_status);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->status_register_perr);

	offset = 199;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->status_register_serr);

	offset = 198;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->status_register_received_master_abort);

	offset = 197;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->status_register_received_target_abort);

	offset = 196;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->status_register_detected_parity_error);

	offset = 195;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->status_register_signaled_target_abort);

}

void connectib_pf_vf_cfg_space_unpack(struct connectib_pf_vf_cfg_space *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_pf_vf_sriov_unpack(&(ptr_struct->sriov), ptr_buff + offset/8);

	offset = 64;
	connectib_pf_vf_vpd_unpack(&(ptr_struct->vpd), ptr_buff + offset/8);

	offset = 150;
	ptr_struct->hw_msix_vec0 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 149;
	ptr_struct->msix_func_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 148;
	ptr_struct->msix_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 147;
	ptr_struct->bus_master_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 146;
	ptr_struct->memory_space = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 144;
	ptr_struct->serr_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 143;
	ptr_struct->serr_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 142;
	ptr_struct->perr_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 141;
	ptr_struct->int_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 140;
	ptr_struct->need_to_fire_flr_event = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 139;
	ptr_struct->in_middle_of_flr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 138;
	ptr_struct->hide = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 136;
	ptr_struct->power_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 128;
	ptr_struct->cache_line = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 184;
	ptr_struct->interrupt_line = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 181;
	ptr_struct->exp_rom_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 160;
	ptr_struct->exp_rom_addr_31_11 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 21);

	offset = 223;
	ptr_struct->device_control_correctable_error_report_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 222;
	ptr_struct->device_control_nonfatal_error_report_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 221;
	ptr_struct->device_control_fatal_error_report_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 220;
	ptr_struct->device_control_ur_error_report_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 219;
	ptr_struct->device_control_enable_relaxed_ordering = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 216;
	ptr_struct->device_control_max_payload_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 215;
	ptr_struct->device_control_extended_tag_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 214;
	ptr_struct->device_control_enable_no_snoop = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 211;
	ptr_struct->device_control_max_read_request_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 210;
	ptr_struct->device_status_correctable_error_detected = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 209;
	ptr_struct->device_status_nonfatal_error_detected = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 208;
	ptr_struct->device_status_fatal_error_detected = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 207;
	ptr_struct->device_status_ur_error_detected = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 206;
	ptr_struct->command_register_memory_space = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 205;
	ptr_struct->command_register_bus_master_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 204;
	ptr_struct->command_register_perr_report_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 203;
	ptr_struct->command_register_serr_report_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 202;
	ptr_struct->command_register_interrupt_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 201;
	ptr_struct->status_register_interrupt_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 200;
	ptr_struct->status_register_perr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 199;
	ptr_struct->status_register_serr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 198;
	ptr_struct->status_register_received_master_abort = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 197;
	ptr_struct->status_register_received_target_abort = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 196;
	ptr_struct->status_register_detected_parity_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 195;
	ptr_struct->status_register_signaled_target_abort = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_pf_vf_cfg_space_print(const struct connectib_pf_vf_cfg_space *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pf_vf_cfg_space ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sriov:\n");
	connectib_pf_vf_sriov_print(&(ptr_struct->sriov), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd:\n");
	connectib_pf_vf_vpd_print(&(ptr_struct->vpd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_msix_vec0         : " UH_FMT "\n", ptr_struct->hw_msix_vec0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msix_func_mask       : " UH_FMT "\n", ptr_struct->msix_func_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msix_enable          : " UH_FMT "\n", ptr_struct->msix_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bus_master_en        : " UH_FMT "\n", ptr_struct->bus_master_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "memory_space         : " UH_FMT "\n", ptr_struct->memory_space);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serr_status          : " UH_FMT "\n", ptr_struct->serr_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serr_enable          : " UH_FMT "\n", ptr_struct->serr_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "perr_enable          : " UH_FMT "\n", ptr_struct->perr_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "int_disable          : " UH_FMT "\n", ptr_struct->int_disable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "need_to_fire_flr_event : " UH_FMT "\n", ptr_struct->need_to_fire_flr_event);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in_middle_of_flr     : " UH_FMT "\n", ptr_struct->in_middle_of_flr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hide                 : " UH_FMT "\n", ptr_struct->hide);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "power_state          : " UH_FMT "\n", ptr_struct->power_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cache_line           : " UH_FMT "\n", ptr_struct->cache_line);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "interrupt_line       : " UH_FMT "\n", ptr_struct->interrupt_line);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "exp_rom_en           : " UH_FMT "\n", ptr_struct->exp_rom_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "exp_rom_addr_31_11   : " UH_FMT "\n", ptr_struct->exp_rom_addr_31_11);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_control_correctable_error_report_en : " UH_FMT "\n", ptr_struct->device_control_correctable_error_report_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_control_nonfatal_error_report_en : " UH_FMT "\n", ptr_struct->device_control_nonfatal_error_report_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_control_fatal_error_report_en : " UH_FMT "\n", ptr_struct->device_control_fatal_error_report_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_control_ur_error_report_en : " UH_FMT "\n", ptr_struct->device_control_ur_error_report_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_control_enable_relaxed_ordering : " UH_FMT "\n", ptr_struct->device_control_enable_relaxed_ordering);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_control_max_payload_size : " UH_FMT "\n", ptr_struct->device_control_max_payload_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_control_extended_tag_en : " UH_FMT "\n", ptr_struct->device_control_extended_tag_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_control_enable_no_snoop : " UH_FMT "\n", ptr_struct->device_control_enable_no_snoop);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_control_max_read_request_size : " UH_FMT "\n", ptr_struct->device_control_max_read_request_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_status_correctable_error_detected : " UH_FMT "\n", ptr_struct->device_status_correctable_error_detected);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_status_nonfatal_error_detected : " UH_FMT "\n", ptr_struct->device_status_nonfatal_error_detected);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_status_fatal_error_detected : " UH_FMT "\n", ptr_struct->device_status_fatal_error_detected);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_status_ur_error_detected : " UH_FMT "\n", ptr_struct->device_status_ur_error_detected);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "command_register_memory_space : " UH_FMT "\n", ptr_struct->command_register_memory_space);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "command_register_bus_master_en : " UH_FMT "\n", ptr_struct->command_register_bus_master_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "command_register_perr_report_en : " UH_FMT "\n", ptr_struct->command_register_perr_report_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "command_register_serr_report_en : " UH_FMT "\n", ptr_struct->command_register_serr_report_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "command_register_interrupt_disable : " UH_FMT "\n", ptr_struct->command_register_interrupt_disable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status_register_interrupt_status : " UH_FMT "\n", ptr_struct->status_register_interrupt_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status_register_perr : " UH_FMT "\n", ptr_struct->status_register_perr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status_register_serr : " UH_FMT "\n", ptr_struct->status_register_serr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status_register_received_master_abort : " UH_FMT "\n", ptr_struct->status_register_received_master_abort);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status_register_received_target_abort : " UH_FMT "\n", ptr_struct->status_register_received_target_abort);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status_register_detected_parity_error : " UH_FMT "\n", ptr_struct->status_register_detected_parity_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status_register_signaled_target_abort : " UH_FMT "\n", ptr_struct->status_register_signaled_target_abort);

}

int connectib_pf_vf_cfg_space_size(void)
{
	 return 28;
}

void connectib_pf_vf_cfg_space_dump(const struct connectib_pf_vf_cfg_space *ptr_struct, FILE* file)
{
	connectib_pf_vf_cfg_space_print(ptr_struct, file, 0);
}

void connectib_hw_init_info_pack(const struct connectib_hw_init_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->init_state);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->init_lock);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->owner_pf);

}

void connectib_hw_init_info_unpack(struct connectib_hw_init_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	ptr_struct->init_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 29;
	ptr_struct->init_lock = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->owner_pf = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_init_info_print(const struct connectib_hw_init_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_init_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "init_state           : " UH_FMT "\n", ptr_struct->init_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "init_lock            : " UH_FMT "\n", ptr_struct->init_lock);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "owner_pf             : " UH_FMT "\n", ptr_struct->owner_pf);

}

int connectib_hw_init_info_size(void)
{
	 return 4;
}

void connectib_hw_init_info_dump(const struct connectib_hw_init_info *ptr_struct, FILE* file)
{
	connectib_hw_init_info_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_moderation_timestamp_7_0_pack(const struct connectib_hw_cqc_moderation_timestamp_7_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_moderation_timestamp_7_0_unpack(struct connectib_hw_cqc_moderation_timestamp_7_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 480;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_cqc_moderation_timestamp_7_0_print(const struct connectib_hw_cqc_moderation_timestamp_7_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_moderation_timestamp_7_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_moderation_timestamp_7_0_size(void)
{
	 return 64;
}

void connectib_hw_cqc_moderation_timestamp_7_0_dump(const struct connectib_hw_cqc_moderation_timestamp_7_0 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_moderation_timestamp_7_0_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_uar_pack(const struct connectib_hw_cqc_uar *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 488;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_uar_unpack(struct connectib_hw_cqc_uar *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 488;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_cqc_uar_print(const struct connectib_hw_cqc_uar *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_uar ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_uar_size(void)
{
	 return 64;
}

void connectib_hw_cqc_uar_dump(const struct connectib_hw_cqc_uar *ptr_struct, FILE* file)
{
	connectib_hw_cqc_uar_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_moderation_timestamp_39_8_pack(const struct connectib_hw_cqc_moderation_timestamp_39_8 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_moderation_timestamp_39_8_unpack(struct connectib_hw_cqc_moderation_timestamp_39_8 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 448;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_cqc_moderation_timestamp_39_8_print(const struct connectib_hw_cqc_moderation_timestamp_39_8 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_moderation_timestamp_39_8 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_moderation_timestamp_39_8_size(void)
{
	 return 64;
}

void connectib_hw_cqc_moderation_timestamp_39_8_dump(const struct connectib_hw_cqc_moderation_timestamp_39_8 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_moderation_timestamp_39_8_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_pa1_31_12_pack(const struct connectib_hw_cqc_pa1_31_12 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_pa1_31_12_unpack(struct connectib_hw_cqc_pa1_31_12 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 416;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

}

void connectib_hw_cqc_pa1_31_12_print(const struct connectib_hw_cqc_pa1_31_12 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_pa1_31_12 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_pa1_31_12_size(void)
{
	 return 64;
}

void connectib_hw_cqc_pa1_31_12_dump(const struct connectib_hw_cqc_pa1_31_12 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_pa1_31_12_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_moderation_timestamp_51_40_pack(const struct connectib_hw_cqc_moderation_timestamp_51_40 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 436;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_moderation_timestamp_51_40_unpack(struct connectib_hw_cqc_moderation_timestamp_51_40 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 436;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

}

void connectib_hw_cqc_moderation_timestamp_51_40_print(const struct connectib_hw_cqc_moderation_timestamp_51_40 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_moderation_timestamp_51_40 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_moderation_timestamp_51_40_size(void)
{
	 return 64;
}

void connectib_hw_cqc_moderation_timestamp_51_40_dump(const struct connectib_hw_cqc_moderation_timestamp_51_40 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_moderation_timestamp_51_40_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_pa1_63_32_pack(const struct connectib_hw_cqc_pa1_63_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_pa1_63_32_unpack(struct connectib_hw_cqc_pa1_63_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 384;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_cqc_pa1_63_32_print(const struct connectib_hw_cqc_pa1_63_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_pa1_63_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_pa1_63_32_size(void)
{
	 return 64;
}

void connectib_hw_cqc_pa1_63_32_dump(const struct connectib_hw_cqc_pa1_63_32 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_pa1_63_32_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_mtt_pointer_4_0_pack(const struct connectib_hw_cqc_mtt_pointer_4_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_mtt_pointer_4_0_unpack(struct connectib_hw_cqc_mtt_pointer_4_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 352;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_cqc_mtt_pointer_4_0_print(const struct connectib_hw_cqc_mtt_pointer_4_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_mtt_pointer_4_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_mtt_pointer_4_0_size(void)
{
	 return 64;
}

void connectib_hw_cqc_mtt_pointer_4_0_dump(const struct connectib_hw_cqc_mtt_pointer_4_0 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_mtt_pointer_4_0_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_overrun_ignore_pack(const struct connectib_hw_cqc_overrun_ignore *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 357;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_overrun_ignore_unpack(struct connectib_hw_cqc_overrun_ignore *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 357;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_overrun_ignore_print(const struct connectib_hw_cqc_overrun_ignore *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_overrun_ignore ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_overrun_ignore_size(void)
{
	 return 64;
}

void connectib_hw_cqc_overrun_ignore_dump(const struct connectib_hw_cqc_overrun_ignore *ptr_struct, FILE* file)
{
	connectib_hw_cqc_overrun_ignore_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_cq_state_pack(const struct connectib_hw_cqc_cq_state *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 358;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_cq_state_unpack(struct connectib_hw_cqc_cq_state *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 358;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_cqc_cq_state_print(const struct connectib_hw_cqc_cq_state *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_cq_state ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_cq_state_size(void)
{
	 return 64;
}

void connectib_hw_cqc_cq_state_dump(const struct connectib_hw_cqc_cq_state *ptr_struct, FILE* file)
{
	connectib_hw_cqc_cq_state_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_consumer_index_pack(const struct connectib_hw_cqc_consumer_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_consumer_index_unpack(struct connectib_hw_cqc_consumer_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 360;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_cqc_consumer_index_print(const struct connectib_hw_cqc_consumer_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_consumer_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_consumer_index_size(void)
{
	 return 64;
}

void connectib_hw_cqc_consumer_index_dump(const struct connectib_hw_cqc_consumer_index *ptr_struct, FILE* file)
{
	connectib_hw_cqc_consumer_index_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_mtt_pointer_36_5_pack(const struct connectib_hw_cqc_mtt_pointer_36_5 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_mtt_pointer_36_5_unpack(struct connectib_hw_cqc_mtt_pointer_36_5 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 320;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_cqc_mtt_pointer_36_5_print(const struct connectib_hw_cqc_mtt_pointer_36_5 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_mtt_pointer_36_5 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_mtt_pointer_36_5_size(void)
{
	 return 64;
}

void connectib_hw_cqc_mtt_pointer_36_5_dump(const struct connectib_hw_cqc_mtt_pointer_36_5 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_mtt_pointer_36_5_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_cipa_31_3_pack(const struct connectib_hw_cqc_cipa_31_3 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 29, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_cipa_31_3_unpack(struct connectib_hw_cqc_cipa_31_3 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 288;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 29);

}

void connectib_hw_cqc_cipa_31_3_print(const struct connectib_hw_cqc_cipa_31_3 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_cipa_31_3 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_cipa_31_3_size(void)
{
	 return 64;
}

void connectib_hw_cqc_cipa_31_3_dump(const struct connectib_hw_cqc_cipa_31_3 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_cipa_31_3_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_mtt_pointer_39_37_pack(const struct connectib_hw_cqc_mtt_pointer_39_37 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 317;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_mtt_pointer_39_37_unpack(struct connectib_hw_cqc_mtt_pointer_39_37 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 317;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_hw_cqc_mtt_pointer_39_37_print(const struct connectib_hw_cqc_mtt_pointer_39_37 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_mtt_pointer_39_37 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_mtt_pointer_39_37_size(void)
{
	 return 64;
}

void connectib_hw_cqc_mtt_pointer_39_37_dump(const struct connectib_hw_cqc_mtt_pointer_39_37 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_mtt_pointer_39_37_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_cipa_63_32_pack(const struct connectib_hw_cqc_cipa_63_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_cipa_63_32_unpack(struct connectib_hw_cqc_cipa_63_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 256;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_cqc_cipa_63_32_print(const struct connectib_hw_cqc_cipa_63_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_cipa_63_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_cipa_63_32_size(void)
{
	 return 64;
}

void connectib_hw_cqc_cipa_63_32_dump(const struct connectib_hw_cqc_cipa_63_32 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_cipa_63_32_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_number_of_events_generated_pack(const struct connectib_hw_cqc_number_of_events_generated *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_number_of_events_generated_unpack(struct connectib_hw_cqc_number_of_events_generated *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 224;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_cqc_number_of_events_generated_print(const struct connectib_hw_cqc_number_of_events_generated *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_number_of_events_generated ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_number_of_events_generated_size(void)
{
	 return 64;
}

void connectib_hw_cqc_number_of_events_generated_dump(const struct connectib_hw_cqc_number_of_events_generated *ptr_struct, FILE* file)
{
	connectib_hw_cqc_number_of_events_generated_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_db_heartbeat_pack(const struct connectib_hw_cqc_db_heartbeat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 226;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_db_heartbeat_unpack(struct connectib_hw_cqc_db_heartbeat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 226;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_db_heartbeat_print(const struct connectib_hw_cqc_db_heartbeat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_db_heartbeat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_db_heartbeat_size(void)
{
	 return 64;
}

void connectib_hw_cqc_db_heartbeat_dump(const struct connectib_hw_cqc_db_heartbeat *ptr_struct, FILE* file)
{
	connectib_hw_cqc_db_heartbeat_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_log2_size_pack(const struct connectib_hw_cqc_log2_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 227;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_log2_size_unpack(struct connectib_hw_cqc_log2_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 227;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_cqc_log2_size_print(const struct connectib_hw_cqc_log2_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_log2_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_log2_size_size(void)
{
	 return 64;
}

void connectib_hw_cqc_log2_size_dump(const struct connectib_hw_cqc_log2_size *ptr_struct, FILE* file)
{
	connectib_hw_cqc_log2_size_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_db_ci_pack(const struct connectib_hw_cqc_db_ci *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_db_ci_unpack(struct connectib_hw_cqc_db_ci *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 232;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_cqc_db_ci_print(const struct connectib_hw_cqc_db_ci *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_db_ci ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_db_ci_size(void)
{
	 return 64;
}

void connectib_hw_cqc_db_ci_dump(const struct connectib_hw_cqc_db_ci *ptr_struct, FILE* file)
{
	connectib_hw_cqc_db_ci_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_moderation_min_wait_time_pack(const struct connectib_hw_cqc_moderation_min_wait_time *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_moderation_min_wait_time_unpack(struct connectib_hw_cqc_moderation_min_wait_time *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 192;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

}

void connectib_hw_cqc_moderation_min_wait_time_print(const struct connectib_hw_cqc_moderation_min_wait_time *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_moderation_min_wait_time ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_moderation_min_wait_time_size(void)
{
	 return 64;
}

void connectib_hw_cqc_moderation_min_wait_time_dump(const struct connectib_hw_cqc_moderation_min_wait_time *ptr_struct, FILE* file)
{
	connectib_hw_cqc_moderation_min_wait_time_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_eqn_msix_pack(const struct connectib_hw_cqc_eqn_msix *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 204;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_eqn_msix_unpack(struct connectib_hw_cqc_eqn_msix *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 204;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

}

void connectib_hw_cqc_eqn_msix_print(const struct connectib_hw_cqc_eqn_msix *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_eqn_msix ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_eqn_msix_size(void)
{
	 return 64;
}

void connectib_hw_cqc_eqn_msix_dump(const struct connectib_hw_cqc_eqn_msix *ptr_struct, FILE* file)
{
	connectib_hw_cqc_eqn_msix_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_moderation_cnt_limit_7_0_pack(const struct connectib_hw_cqc_moderation_cnt_limit_7_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_moderation_cnt_limit_7_0_unpack(struct connectib_hw_cqc_moderation_cnt_limit_7_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_cqc_moderation_cnt_limit_7_0_print(const struct connectib_hw_cqc_moderation_cnt_limit_7_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_moderation_cnt_limit_7_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_moderation_cnt_limit_7_0_size(void)
{
	 return 64;
}

void connectib_hw_cqc_moderation_cnt_limit_7_0_dump(const struct connectib_hw_cqc_moderation_cnt_limit_7_0 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_moderation_cnt_limit_7_0_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_producer_index_pack(const struct connectib_hw_cqc_producer_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_producer_index_unpack(struct connectib_hw_cqc_producer_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 168;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_cqc_producer_index_print(const struct connectib_hw_cqc_producer_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_producer_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_producer_index_size(void)
{
	 return 64;
}

void connectib_hw_cqc_producer_index_dump(const struct connectib_hw_cqc_producer_index *ptr_struct, FILE* file)
{
	connectib_hw_cqc_producer_index_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_last_notified_index_pack(const struct connectib_hw_cqc_last_notified_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_last_notified_index_unpack(struct connectib_hw_cqc_last_notified_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 128;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_cqc_last_notified_index_print(const struct connectib_hw_cqc_last_notified_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_last_notified_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_last_notified_index_size(void)
{
	 return 64;
}

void connectib_hw_cqc_last_notified_index_dump(const struct connectib_hw_cqc_last_notified_index *ptr_struct, FILE* file)
{
	connectib_hw_cqc_last_notified_index_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_moderation_cnt_limit_15_8_pack(const struct connectib_hw_cqc_moderation_cnt_limit_15_8 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_moderation_cnt_limit_15_8_unpack(struct connectib_hw_cqc_moderation_cnt_limit_15_8 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 152;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_cqc_moderation_cnt_limit_15_8_print(const struct connectib_hw_cqc_moderation_cnt_limit_15_8 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_moderation_cnt_limit_15_8 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_moderation_cnt_limit_15_8_size(void)
{
	 return 64;
}

void connectib_hw_cqc_moderation_cnt_limit_15_8_dump(const struct connectib_hw_cqc_moderation_cnt_limit_15_8 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_moderation_cnt_limit_15_8_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_pa0_31_12_pack(const struct connectib_hw_cqc_pa0_31_12 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_pa0_31_12_unpack(struct connectib_hw_cqc_pa0_31_12 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

}

void connectib_hw_cqc_pa0_31_12_print(const struct connectib_hw_cqc_pa0_31_12 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_pa0_31_12 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_pa0_31_12_size(void)
{
	 return 64;
}

void connectib_hw_cqc_pa0_31_12_dump(const struct connectib_hw_cqc_pa0_31_12 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_pa0_31_12_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_cq_trap_pack(const struct connectib_hw_cqc_cq_trap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 116;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_cq_trap_unpack(struct connectib_hw_cqc_cq_trap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 116;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_cq_trap_print(const struct connectib_hw_cqc_cq_trap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_cq_trap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_cq_trap_size(void)
{
	 return 64;
}

void connectib_hw_cqc_cq_trap_dump(const struct connectib_hw_cqc_cq_trap *ptr_struct, FILE* file)
{
	connectib_hw_cqc_cq_trap_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_shrink_pack(const struct connectib_hw_cqc_shrink *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 117;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_shrink_unpack(struct connectib_hw_cqc_shrink *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 117;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_shrink_print(const struct connectib_hw_cqc_shrink *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_shrink ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_shrink_size(void)
{
	 return 64;
}

void connectib_hw_cqc_shrink_dump(const struct connectib_hw_cqc_shrink *ptr_struct, FILE* file)
{
	connectib_hw_cqc_shrink_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_pa_mode_pack(const struct connectib_hw_cqc_pa_mode *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 118;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_pa_mode_unpack(struct connectib_hw_cqc_pa_mode *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 118;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_cqc_pa_mode_print(const struct connectib_hw_cqc_pa_mode *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_pa_mode ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_pa_mode_size(void)
{
	 return 64;
}

void connectib_hw_cqc_pa_mode_dump(const struct connectib_hw_cqc_pa_mode *ptr_struct, FILE* file)
{
	connectib_hw_cqc_pa_mode_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_cqe_heartbeat_pack(const struct connectib_hw_cqc_cqe_heartbeat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_cqe_heartbeat_unpack(struct connectib_hw_cqc_cqe_heartbeat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 120;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_cqe_heartbeat_print(const struct connectib_hw_cqc_cqe_heartbeat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_cqe_heartbeat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_cqe_heartbeat_size(void)
{
	 return 64;
}

void connectib_hw_cqc_cqe_heartbeat_dump(const struct connectib_hw_cqc_cqe_heartbeat *ptr_struct, FILE* file)
{
	connectib_hw_cqc_cqe_heartbeat_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_db_recovery_enable_pack(const struct connectib_hw_cqc_db_recovery_enable *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 121;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_db_recovery_enable_unpack(struct connectib_hw_cqc_db_recovery_enable *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 121;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_db_recovery_enable_print(const struct connectib_hw_cqc_db_recovery_enable *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_db_recovery_enable ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_db_recovery_enable_size(void)
{
	 return 64;
}

void connectib_hw_cqc_db_recovery_enable_dump(const struct connectib_hw_cqc_db_recovery_enable *ptr_struct, FILE* file)
{
	connectib_hw_cqc_db_recovery_enable_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_cq_page_offset_pack(const struct connectib_hw_cqc_cq_page_offset *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 122;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_cq_page_offset_unpack(struct connectib_hw_cqc_cq_page_offset *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 122;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

}

void connectib_hw_cqc_cq_page_offset_print(const struct connectib_hw_cqc_cq_page_offset *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_cq_page_offset ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_cq_page_offset_size(void)
{
	 return 64;
}

void connectib_hw_cqc_cq_page_offset_dump(const struct connectib_hw_cqc_cq_page_offset *ptr_struct, FILE* file)
{
	connectib_hw_cqc_cq_page_offset_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_pa0_63_32_pack(const struct connectib_hw_cqc_pa0_63_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_pa0_63_32_unpack(struct connectib_hw_cqc_pa0_63_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_cqc_pa0_63_32_print(const struct connectib_hw_cqc_pa0_63_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_pa0_63_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_pa0_63_32_size(void)
{
	 return 64;
}

void connectib_hw_cqc_pa0_63_32_dump(const struct connectib_hw_cqc_pa0_63_32 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_pa0_63_32_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_lookahead_size_7_0_pack(const struct connectib_hw_cqc_lookahead_size_7_0 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_lookahead_size_7_0_unpack(struct connectib_hw_cqc_lookahead_size_7_0 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_cqc_lookahead_size_7_0_print(const struct connectib_hw_cqc_lookahead_size_7_0 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_lookahead_size_7_0 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_lookahead_size_7_0_size(void)
{
	 return 64;
}

void connectib_hw_cqc_lookahead_size_7_0_dump(const struct connectib_hw_cqc_lookahead_size_7_0 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_lookahead_size_7_0_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_last_solicited_index_pack(const struct connectib_hw_cqc_last_solicited_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_last_solicited_index_unpack(struct connectib_hw_cqc_last_solicited_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 40;
	ptr_struct->val = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_hw_cqc_last_solicited_index_print(const struct connectib_hw_cqc_last_solicited_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_last_solicited_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_last_solicited_index_size(void)
{
	 return 64;
}

void connectib_hw_cqc_last_solicited_index_dump(const struct connectib_hw_cqc_last_solicited_index *ptr_struct, FILE* file)
{
	connectib_hw_cqc_last_solicited_index_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_last_notified_check_for_arm_en_pack(const struct connectib_hw_cqc_last_notified_check_for_arm_en *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_last_notified_check_for_arm_en_unpack(struct connectib_hw_cqc_last_notified_check_for_arm_en *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_last_notified_check_for_arm_en_print(const struct connectib_hw_cqc_last_notified_check_for_arm_en *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_last_notified_check_for_arm_en ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_last_notified_check_for_arm_en_size(void)
{
	 return 64;
}

void connectib_hw_cqc_last_notified_check_for_arm_en_dump(const struct connectib_hw_cqc_last_notified_check_for_arm_en *ptr_struct, FILE* file)
{
	connectib_hw_cqc_last_notified_check_for_arm_en_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_poll_cq_queue_in_error_state_pack(const struct connectib_hw_cqc_poll_cq_queue_in_error_state *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_poll_cq_queue_in_error_state_unpack(struct connectib_hw_cqc_poll_cq_queue_in_error_state *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_poll_cq_queue_in_error_state_print(const struct connectib_hw_cqc_poll_cq_queue_in_error_state *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_poll_cq_queue_in_error_state ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_poll_cq_queue_in_error_state_size(void)
{
	 return 64;
}

void connectib_hw_cqc_poll_cq_queue_in_error_state_dump(const struct connectib_hw_cqc_poll_cq_queue_in_error_state *ptr_struct, FILE* file)
{
	connectib_hw_cqc_poll_cq_queue_in_error_state_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue_pack(const struct connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue_unpack(struct connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue_print(const struct connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue_size(void)
{
	 return 64;
}

void connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue_dump(const struct connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue *ptr_struct, FILE* file)
{
	connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_log_page_size_pack(const struct connectib_hw_cqc_log_page_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_log_page_size_unpack(struct connectib_hw_cqc_log_page_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_hw_cqc_log_page_size_print(const struct connectib_hw_cqc_log_page_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_log_page_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_log_page_size_size(void)
{
	 return 64;
}

void connectib_hw_cqc_log_page_size_dump(const struct connectib_hw_cqc_log_page_size *ptr_struct, FILE* file)
{
	connectib_hw_cqc_log_page_size_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_pa_error_pack(const struct connectib_hw_cqc_pa_error *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_pa_error_unpack(struct connectib_hw_cqc_pa_error *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_hw_cqc_pa_error_print(const struct connectib_hw_cqc_pa_error *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_pa_error ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_pa_error_size(void)
{
	 return 64;
}

void connectib_hw_cqc_pa_error_dump(const struct connectib_hw_cqc_pa_error *ptr_struct, FILE* file)
{
	connectib_hw_cqc_pa_error_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_cqe_coalesce_checksum_msix_pack(const struct connectib_hw_cqc_cqe_coalesce_checksum_msix *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_cqe_coalesce_checksum_msix_unpack(struct connectib_hw_cqc_cqe_coalesce_checksum_msix *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 13;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_cqe_coalesce_checksum_msix_print(const struct connectib_hw_cqc_cqe_coalesce_checksum_msix *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_cqe_coalesce_checksum_msix ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_cqe_coalesce_checksum_msix_size(void)
{
	 return 64;
}

void connectib_hw_cqc_cqe_coalesce_checksum_msix_dump(const struct connectib_hw_cqc_cqe_coalesce_checksum_msix *ptr_struct, FILE* file)
{
	connectib_hw_cqc_cqe_coalesce_checksum_msix_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_cqe_128b_size_pack(const struct connectib_hw_cqc_cqe_128b_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_cqe_128b_size_unpack(struct connectib_hw_cqc_cqe_128b_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_cqe_128b_size_print(const struct connectib_hw_cqc_cqe_128b_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_cqe_128b_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_cqe_128b_size_size(void)
{
	 return 64;
}

void connectib_hw_cqc_cqe_128b_size_dump(const struct connectib_hw_cqc_cqe_128b_size *ptr_struct, FILE* file)
{
	connectib_hw_cqc_cqe_128b_size_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_cqc_valid_pack(const struct connectib_hw_cqc_cqc_valid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_cqc_valid_unpack(struct connectib_hw_cqc_cqc_valid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_cqc_cqc_valid_print(const struct connectib_hw_cqc_cqc_valid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_cqc_valid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_cqc_valid_size(void)
{
	 return 64;
}

void connectib_hw_cqc_cqc_valid_dump(const struct connectib_hw_cqc_cqc_valid *ptr_struct, FILE* file)
{
	connectib_hw_cqc_cqc_valid_print(ptr_struct, file, 0);
}

void connectib_hw_cqc_lookahead_size_23_8_pack(const struct connectib_hw_cqc_lookahead_size_23_8 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->val);

}

void connectib_hw_cqc_lookahead_size_23_8_unpack(struct connectib_hw_cqc_lookahead_size_23_8 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->val = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_cqc_lookahead_size_23_8_print(const struct connectib_hw_cqc_lookahead_size_23_8 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_cqc_lookahead_size_23_8 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " UH_FMT "\n", ptr_struct->val);

}

int connectib_hw_cqc_lookahead_size_23_8_size(void)
{
	 return 64;
}

void connectib_hw_cqc_lookahead_size_23_8_dump(const struct connectib_hw_cqc_lookahead_size_23_8 *ptr_struct, FILE* file)
{
	connectib_hw_cqc_lookahead_size_23_8_print(ptr_struct, file, 0);
}

void connectib_mlnx_enhanced_equal_info_ini_pack(const struct connectib_mlnx_enhanced_equal_info_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mlpn_100kr4_extended_time_sup);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mlpn_100cr4_extended_time_sup);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mlpn_40cr4_extended_time_sup);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mlpn_40kr4_extended_time_sup);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mlpn_kr_extended_time_sup);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->edr_rx_atten_sup);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->edr_rx_atten);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fdr_rx_atten_sup);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->fdr_rx_atten);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qdr_ddr_ts3_rev);

}

void connectib_mlnx_enhanced_equal_info_ini_unpack(struct connectib_mlnx_enhanced_equal_info_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->mlpn_100kr4_extended_time_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->mlpn_100cr4_extended_time_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->mlpn_40cr4_extended_time_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->mlpn_40kr4_extended_time_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->mlpn_kr_extended_time_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->edr_rx_atten_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->edr_rx_atten = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 20;
	ptr_struct->fdr_rx_atten_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 15;
	ptr_struct->fdr_rx_atten = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 14;
	ptr_struct->qdr_ddr_ts3_rev = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_mlnx_enhanced_equal_info_ini_print(const struct connectib_mlnx_enhanced_equal_info_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mlnx_enhanced_equal_info_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_100kr4_extended_time_sup : " UH_FMT "\n", ptr_struct->mlpn_100kr4_extended_time_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_100cr4_extended_time_sup : " UH_FMT "\n", ptr_struct->mlpn_100cr4_extended_time_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_40cr4_extended_time_sup : " UH_FMT "\n", ptr_struct->mlpn_40cr4_extended_time_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_40kr4_extended_time_sup : " UH_FMT "\n", ptr_struct->mlpn_40kr4_extended_time_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_kr_extended_time_sup : " UH_FMT "\n", ptr_struct->mlpn_kr_extended_time_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edr_rx_atten_sup     : " UH_FMT "\n", ptr_struct->edr_rx_atten_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "edr_rx_atten         : " UH_FMT "\n", ptr_struct->edr_rx_atten);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fdr_rx_atten_sup     : " UH_FMT "\n", ptr_struct->fdr_rx_atten_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fdr_rx_atten         : " UH_FMT "\n", ptr_struct->fdr_rx_atten);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qdr_ddr_ts3_rev      : " UH_FMT "\n", ptr_struct->qdr_ddr_ts3_rev);

}

int connectib_mlnx_enhanced_equal_info_ini_size(void)
{
	 return 4;
}

void connectib_mlnx_enhanced_equal_info_ini_dump(const struct connectib_mlnx_enhanced_equal_info_ini *ptr_struct, FILE* file)
{
	connectib_mlnx_enhanced_equal_info_ini_print(ptr_struct, file, 0);
}

void connectib_mlpn_database_all_protocol_bits_ini_pack(const struct connectib_mlpn_database_all_protocol_bits_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fdr_8_10);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mlx_kr2);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->eth_56gb);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fdr10);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->speed_reduction);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->width_reduction);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->eth_cbf);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ib_64b_credits);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_ib_edr);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_ib_fdr);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_ib_fdr10);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_ib_qdr);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_ib_ddr);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_ib_sdr);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_56_kr4);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_56_kx4);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_kr4);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_cr4);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_slr4);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_kr);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_cr);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_slr);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_xaui);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_sgmii);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fc_protocol);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->eth_protocol);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ib_protocol);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ib_64_66_spec_recovery);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ib_128b_llr);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ll_llr);

}

void connectib_mlpn_database_all_protocol_bits_ini_unpack(struct connectib_mlpn_database_all_protocol_bits_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->fdr_8_10 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->mlx_kr2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->eth_56gb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->fdr10 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->speed_reduction = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->width_reduction = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->eth_cbf = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->ib_64b_credits = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->llr_ib_edr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->llr_ib_fdr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->llr_ib_fdr10 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->llr_ib_qdr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->llr_ib_ddr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->llr_ib_sdr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->llr_56_kr4 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->llr_56_kx4 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 15;
	ptr_struct->llr_kr4 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->llr_cr4 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->llr_slr4 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->llr_kr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->llr_cr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->llr_slr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->llr_xaui = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->llr_sgmii = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->fc_protocol = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->eth_protocol = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->ib_protocol = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->ib_64_66_spec_recovery = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->ib_128b_llr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->ll_llr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_mlpn_database_all_protocol_bits_ini_print(const struct connectib_mlpn_database_all_protocol_bits_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mlpn_database_all_protocol_bits_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fdr_8_10             : " UH_FMT "\n", ptr_struct->fdr_8_10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlx_kr2              : " UH_FMT "\n", ptr_struct->mlx_kr2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_56gb             : " UH_FMT "\n", ptr_struct->eth_56gb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fdr10                : " UH_FMT "\n", ptr_struct->fdr10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "speed_reduction      : " UH_FMT "\n", ptr_struct->speed_reduction);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "width_reduction      : " UH_FMT "\n", ptr_struct->width_reduction);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_cbf              : " UH_FMT "\n", ptr_struct->eth_cbf);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_64b_credits       : " UH_FMT "\n", ptr_struct->ib_64b_credits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_ib_edr           : " UH_FMT "\n", ptr_struct->llr_ib_edr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_ib_fdr           : " UH_FMT "\n", ptr_struct->llr_ib_fdr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_ib_fdr10         : " UH_FMT "\n", ptr_struct->llr_ib_fdr10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_ib_qdr           : " UH_FMT "\n", ptr_struct->llr_ib_qdr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_ib_ddr           : " UH_FMT "\n", ptr_struct->llr_ib_ddr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_ib_sdr           : " UH_FMT "\n", ptr_struct->llr_ib_sdr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_56_kr4           : " UH_FMT "\n", ptr_struct->llr_56_kr4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_56_kx4           : " UH_FMT "\n", ptr_struct->llr_56_kx4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_kr4              : " UH_FMT "\n", ptr_struct->llr_kr4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_cr4              : " UH_FMT "\n", ptr_struct->llr_cr4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_slr4             : " UH_FMT "\n", ptr_struct->llr_slr4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_kr               : " UH_FMT "\n", ptr_struct->llr_kr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_cr               : " UH_FMT "\n", ptr_struct->llr_cr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_slr              : " UH_FMT "\n", ptr_struct->llr_slr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_xaui             : " UH_FMT "\n", ptr_struct->llr_xaui);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_sgmii            : " UH_FMT "\n", ptr_struct->llr_sgmii);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc_protocol          : " UH_FMT "\n", ptr_struct->fc_protocol);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_protocol         : " UH_FMT "\n", ptr_struct->eth_protocol);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_protocol          : " UH_FMT "\n", ptr_struct->ib_protocol);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_64_66_spec_recovery : " UH_FMT "\n", ptr_struct->ib_64_66_spec_recovery);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_128b_llr          : " UH_FMT "\n", ptr_struct->ib_128b_llr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ll_llr               : " UH_FMT "\n", ptr_struct->ll_llr);

}

int connectib_mlpn_database_all_protocol_bits_ini_size(void)
{
	 return 4;
}

void connectib_mlpn_database_all_protocol_bits_ini_dump(const struct connectib_mlpn_database_all_protocol_bits_ini *ptr_struct, FILE* file)
{
	connectib_mlpn_database_all_protocol_bits_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_preset_sets_ini_pack(const struct connectib_serdes_preset_sets_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_conf_preset_tx_ini_pack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_preset_sets_ini_unpack(struct connectib_serdes_preset_sets_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_conf_preset_tx_ini_unpack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_preset_sets_ini_print(const struct connectib_serdes_preset_sets_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_preset_sets_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx:\n");
	connectib_serdes_conf_preset_tx_ini_print(&(ptr_struct->tx), file, indent_level + 1);

}

int connectib_serdes_preset_sets_ini_size(void)
{
	 return 64;
}

void connectib_serdes_preset_sets_ini_dump(const struct connectib_serdes_preset_sets_ini *ptr_struct, FILE* file)
{
	connectib_serdes_preset_sets_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_active_sets_ini_pack(const struct connectib_serdes_active_sets_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_active_rx_ini_pack(&(ptr_struct->rx), ptr_buff + offset/8);

	offset = 256;
	connectib_serdes_active_tx_ini_pack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_active_sets_ini_unpack(struct connectib_serdes_active_sets_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_active_rx_ini_unpack(&(ptr_struct->rx), ptr_buff + offset/8);

	offset = 256;
	connectib_serdes_active_tx_ini_unpack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_active_sets_ini_print(const struct connectib_serdes_active_sets_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_active_sets_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx:\n");
	connectib_serdes_active_rx_ini_print(&(ptr_struct->rx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx:\n");
	connectib_serdes_active_tx_ini_print(&(ptr_struct->tx), file, indent_level + 1);

}

int connectib_serdes_active_sets_ini_size(void)
{
	 return 48;
}

void connectib_serdes_active_sets_ini_dump(const struct connectib_serdes_active_sets_ini *ptr_struct, FILE* file)
{
	connectib_serdes_active_sets_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_force_sets_ini_pack(const struct connectib_serdes_force_sets_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_force_rx_ini_pack(&(ptr_struct->rx), ptr_buff + offset/8);

	offset = 640;
	connectib_serdes_force_tx_ini_pack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_force_sets_ini_unpack(struct connectib_serdes_force_sets_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_force_rx_ini_unpack(&(ptr_struct->rx), ptr_buff + offset/8);

	offset = 640;
	connectib_serdes_force_tx_ini_unpack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_force_sets_ini_print(const struct connectib_serdes_force_sets_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_force_sets_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx:\n");
	connectib_serdes_force_rx_ini_print(&(ptr_struct->rx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx:\n");
	connectib_serdes_force_tx_ini_print(&(ptr_struct->tx), file, indent_level + 1);

}

int connectib_serdes_force_sets_ini_size(void)
{
	 return 120;
}

void connectib_serdes_force_sets_ini_dump(const struct connectib_serdes_force_sets_ini *ptr_struct, FILE* file)
{
	connectib_serdes_force_sets_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_aba_sets_ini_pack(const struct connectib_serdes_aba_sets_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_conf_aba_rx_ini_pack(&(ptr_struct->rx), ptr_buff + offset/8);

	offset = 2048;
	connectib_serdes_conf_aba_tx_ini_pack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_aba_sets_ini_unpack(struct connectib_serdes_aba_sets_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_conf_aba_rx_ini_unpack(&(ptr_struct->rx), ptr_buff + offset/8);

	offset = 2048;
	connectib_serdes_conf_aba_tx_ini_unpack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_aba_sets_ini_print(const struct connectib_serdes_aba_sets_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_aba_sets_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx:\n");
	connectib_serdes_conf_aba_rx_ini_print(&(ptr_struct->rx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx:\n");
	connectib_serdes_conf_aba_tx_ini_print(&(ptr_struct->tx), file, indent_level + 1);

}

int connectib_serdes_aba_sets_ini_size(void)
{
	 return 384;
}

void connectib_serdes_aba_sets_ini_dump(const struct connectib_serdes_aba_sets_ini *ptr_struct, FILE* file)
{
	connectib_serdes_aba_sets_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_autoneg_sets_ini_pack(const struct connectib_serdes_autoneg_sets_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_conf_autoneg_rx_ini_pack(&(ptr_struct->rx), ptr_buff + offset/8);

	offset = 2048;
	connectib_serdes_conf_autoneg_tx_ini_pack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_autoneg_sets_ini_unpack(struct connectib_serdes_autoneg_sets_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_conf_autoneg_rx_ini_unpack(&(ptr_struct->rx), ptr_buff + offset/8);

	offset = 2048;
	connectib_serdes_conf_autoneg_tx_ini_unpack(&(ptr_struct->tx), ptr_buff + offset/8);

}

void connectib_serdes_autoneg_sets_ini_print(const struct connectib_serdes_autoneg_sets_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_autoneg_sets_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx:\n");
	connectib_serdes_conf_autoneg_rx_ini_print(&(ptr_struct->rx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx:\n");
	connectib_serdes_conf_autoneg_tx_ini_print(&(ptr_struct->tx), file, indent_level + 1);

}

int connectib_serdes_autoneg_sets_ini_size(void)
{
	 return 384;
}

void connectib_serdes_autoneg_sets_ini_dump(const struct connectib_serdes_autoneg_sets_ini *ptr_struct, FILE* file)
{
	connectib_serdes_autoneg_sets_ini_print(ptr_struct, file, 0);
}

void connectib_spec1_3_preset_ini_pack(const struct connectib_spec1_3_preset_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->preset_tx_mask_internal);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->preset_tx_mask);

	offset = 59;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask0_start);

	offset = 51;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask0_end);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->tx_ABA_en_mask);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->aba_mask0);

	offset = 91;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask1_start);

	offset = 83;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask1_end);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->tx_ABA_mode_mask);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->aba_mask1);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask2_start);

	offset = 115;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask2_end);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->aba_mask2);

	offset = 155;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask3_start);

	offset = 147;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask3_end);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->aba_mask3);

	offset = 187;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask4_start);

	offset = 179;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask4_end);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->aba_mask4);

	offset = 219;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask5_start);

	offset = 211;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->aba_mask5_end);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->aba_mask5);

	offset = 224;
	connectib_aba_indexes_pack(&(ptr_struct->aba_indexes), ptr_buff + offset/8);

}

void connectib_spec1_3_preset_ini_unpack(struct connectib_spec1_3_preset_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->preset_tx_mask_internal = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->preset_tx_mask = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 59;
	ptr_struct->aba_mask0_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 51;
	ptr_struct->aba_mask0_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 48;
	ptr_struct->tx_ABA_en_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 32;
	ptr_struct->aba_mask0 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 91;
	ptr_struct->aba_mask1_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 83;
	ptr_struct->aba_mask1_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 80;
	ptr_struct->tx_ABA_mode_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 64;
	ptr_struct->aba_mask1 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 123;
	ptr_struct->aba_mask2_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 115;
	ptr_struct->aba_mask2_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 96;
	ptr_struct->aba_mask2 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 155;
	ptr_struct->aba_mask3_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 147;
	ptr_struct->aba_mask3_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 128;
	ptr_struct->aba_mask3 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 187;
	ptr_struct->aba_mask4_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 179;
	ptr_struct->aba_mask4_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 160;
	ptr_struct->aba_mask4 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 219;
	ptr_struct->aba_mask5_start = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 211;
	ptr_struct->aba_mask5_end = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 192;
	ptr_struct->aba_mask5 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 224;
	connectib_aba_indexes_unpack(&(ptr_struct->aba_indexes), ptr_buff + offset/8);

}

void connectib_spec1_3_preset_ini_print(const struct connectib_spec1_3_preset_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_spec1_3_preset_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "preset_tx_mask_internal : " UH_FMT "\n", ptr_struct->preset_tx_mask_internal);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "preset_tx_mask       : " UH_FMT "\n", ptr_struct->preset_tx_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask0_start      : " UH_FMT "\n", ptr_struct->aba_mask0_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask0_end        : " UH_FMT "\n", ptr_struct->aba_mask0_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_ABA_en_mask       : " UH_FMT "\n", ptr_struct->tx_ABA_en_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask0            : " UH_FMT "\n", ptr_struct->aba_mask0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask1_start      : " UH_FMT "\n", ptr_struct->aba_mask1_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask1_end        : " UH_FMT "\n", ptr_struct->aba_mask1_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_ABA_mode_mask     : " UH_FMT "\n", ptr_struct->tx_ABA_mode_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask1            : " UH_FMT "\n", ptr_struct->aba_mask1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask2_start      : " UH_FMT "\n", ptr_struct->aba_mask2_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask2_end        : " UH_FMT "\n", ptr_struct->aba_mask2_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask2            : " UH_FMT "\n", ptr_struct->aba_mask2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask3_start      : " UH_FMT "\n", ptr_struct->aba_mask3_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask3_end        : " UH_FMT "\n", ptr_struct->aba_mask3_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask3            : " UH_FMT "\n", ptr_struct->aba_mask3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask4_start      : " UH_FMT "\n", ptr_struct->aba_mask4_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask4_end        : " UH_FMT "\n", ptr_struct->aba_mask4_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask4            : " UH_FMT "\n", ptr_struct->aba_mask4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask5_start      : " UH_FMT "\n", ptr_struct->aba_mask5_start);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask5_end        : " UH_FMT "\n", ptr_struct->aba_mask5_end);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_mask5            : " UH_FMT "\n", ptr_struct->aba_mask5);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba_indexes:\n");
	connectib_aba_indexes_print(&(ptr_struct->aba_indexes), file, indent_level + 1);

}

int connectib_spec1_3_preset_ini_size(void)
{
	 return 76;
}

void connectib_spec1_3_preset_ini_dump(const struct connectib_spec1_3_preset_ini *ptr_struct, FILE* file)
{
	connectib_spec1_3_preset_ini_print(ptr_struct, file, 0);
}

void connectib_ib_ini_per_lcl_ini_pack(const struct connectib_ib_ini_per_lcl_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->spec1_2_ib_support);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->spec1_3_ib_support);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fec_request_fdr);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fec_request_fdr10);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->heartbeat_enable);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->force_speed_val);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_speed_en);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->force_width_val);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_width_en);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_qdr_is_fdr10_val);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_qdr_is_fdr10_en);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_llr_fdr_en);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_llr_fdr_val);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_llr_fdr10_en);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_llr_fdr10_val);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mlpn_en);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_cell_size);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->speed_reduction_force);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->width_reduction_force);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_fdr10_force);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_fdr_force);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->spec1_2_legacy_mode);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_disable_failsafe);

}

void connectib_ib_ini_per_lcl_ini_unpack(struct connectib_ib_ini_per_lcl_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->spec1_2_ib_support = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->spec1_3_ib_support = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->fec_request_fdr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->fec_request_fdr10 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->heartbeat_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->force_speed_val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 22;
	ptr_struct->force_speed_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->force_width_val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 17;
	ptr_struct->force_width_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->force_qdr_is_fdr10_val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 15;
	ptr_struct->force_qdr_is_fdr10_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->force_llr_fdr_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->force_llr_fdr_val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->force_llr_fdr10_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->force_llr_fdr10_val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->mlpn_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->llr_cell_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->speed_reduction_force = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->width_reduction_force = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->llr_fdr10_force = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->llr_fdr_force = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->spec1_2_legacy_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->llr_disable_failsafe = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_ib_ini_per_lcl_ini_print(const struct connectib_ib_ini_per_lcl_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ib_ini_per_lcl_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spec1_2_ib_support   : " UH_FMT "\n", ptr_struct->spec1_2_ib_support);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spec1_3_ib_support   : " UH_FMT "\n", ptr_struct->spec1_3_ib_support);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_request_fdr      : " UH_FMT "\n", ptr_struct->fec_request_fdr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_request_fdr10    : " UH_FMT "\n", ptr_struct->fec_request_fdr10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "heartbeat_enable     : " UH_FMT "\n", ptr_struct->heartbeat_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_speed_val      : " UH_FMT "\n", ptr_struct->force_speed_val);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_speed_en       : " UH_FMT "\n", ptr_struct->force_speed_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_width_val      : " UH_FMT "\n", ptr_struct->force_width_val);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_width_en       : " UH_FMT "\n", ptr_struct->force_width_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_qdr_is_fdr10_val : " UH_FMT "\n", ptr_struct->force_qdr_is_fdr10_val);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_qdr_is_fdr10_en : " UH_FMT "\n", ptr_struct->force_qdr_is_fdr10_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_llr_fdr_en     : " UH_FMT "\n", ptr_struct->force_llr_fdr_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_llr_fdr_val    : " UH_FMT "\n", ptr_struct->force_llr_fdr_val);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_llr_fdr10_en   : " UH_FMT "\n", ptr_struct->force_llr_fdr10_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_llr_fdr10_val  : " UH_FMT "\n", ptr_struct->force_llr_fdr10_val);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_en              : " UH_FMT "\n", ptr_struct->mlpn_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_cell_size        : " UH_FMT "\n", ptr_struct->llr_cell_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "speed_reduction_force : " UH_FMT "\n", ptr_struct->speed_reduction_force);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "width_reduction_force : " UH_FMT "\n", ptr_struct->width_reduction_force);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_fdr10_force      : " UH_FMT "\n", ptr_struct->llr_fdr10_force);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_fdr_force        : " UH_FMT "\n", ptr_struct->llr_fdr_force);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spec1_2_legacy_mode  : " UH_FMT "\n", ptr_struct->spec1_2_legacy_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_disable_failsafe : " UH_FMT "\n", ptr_struct->llr_disable_failsafe);

}

int connectib_ib_ini_per_lcl_ini_size(void)
{
	 return 4;
}

void connectib_ib_ini_per_lcl_ini_dump(const struct connectib_ib_ini_per_lcl_ini *ptr_struct, FILE* file)
{
	connectib_ib_ini_per_lcl_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_param_mask_ini_pack(const struct connectib_serdes_param_mask_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_param_mask_per_proto_ini_pack(&(ptr_struct->encode_8_10), ptr_buff + offset/8);

	offset = 32;
	connectib_serdes_param_mask_per_proto_ini_pack(&(ptr_struct->encode_64_66), ptr_buff + offset/8);

	offset = 64;
	connectib_serdes_param_mask_per_proto_ini_pack(&(ptr_struct->encode_kr), ptr_buff + offset/8);

	offset = 96;
	connectib_serdes_param_mask_per_proto_ini_pack(&(ptr_struct->fdr), ptr_buff + offset/8);

}

void connectib_serdes_param_mask_ini_unpack(struct connectib_serdes_param_mask_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_param_mask_per_proto_ini_unpack(&(ptr_struct->encode_8_10), ptr_buff + offset/8);

	offset = 32;
	connectib_serdes_param_mask_per_proto_ini_unpack(&(ptr_struct->encode_64_66), ptr_buff + offset/8);

	offset = 64;
	connectib_serdes_param_mask_per_proto_ini_unpack(&(ptr_struct->encode_kr), ptr_buff + offset/8);

	offset = 96;
	connectib_serdes_param_mask_per_proto_ini_unpack(&(ptr_struct->fdr), ptr_buff + offset/8);

}

void connectib_serdes_param_mask_ini_print(const struct connectib_serdes_param_mask_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_param_mask_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encode_8_10:\n");
	connectib_serdes_param_mask_per_proto_ini_print(&(ptr_struct->encode_8_10), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encode_64_66:\n");
	connectib_serdes_param_mask_per_proto_ini_print(&(ptr_struct->encode_64_66), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encode_kr:\n");
	connectib_serdes_param_mask_per_proto_ini_print(&(ptr_struct->encode_kr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fdr:\n");
	connectib_serdes_param_mask_per_proto_ini_print(&(ptr_struct->fdr), file, indent_level + 1);

}

int connectib_serdes_param_mask_ini_size(void)
{
	 return 16;
}

void connectib_serdes_param_mask_ini_dump(const struct connectib_serdes_param_mask_ini *ptr_struct, FILE* file)
{
	connectib_serdes_param_mask_ini_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_data_tpt_capabilities_pack(const struct connectib_nv_tlv_data_tpt_capabilities *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->log_max_payload_size_supported);

}

void connectib_nv_tlv_data_tpt_capabilities_unpack(struct connectib_nv_tlv_data_tpt_capabilities *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->log_max_payload_size_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_nv_tlv_data_tpt_capabilities_print(const struct connectib_nv_tlv_data_tpt_capabilities *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_data_tpt_capabilities ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_payload_size_supported : " UH_FMT "\n", ptr_struct->log_max_payload_size_supported);

}

int connectib_nv_tlv_data_tpt_capabilities_size(void)
{
	 return 4;
}

void connectib_nv_tlv_data_tpt_capabilities_dump(const struct connectib_nv_tlv_data_tpt_capabilities *ptr_struct, FILE* file)
{
	connectib_nv_tlv_data_tpt_capabilities_print(ptr_struct, file, 0);
}

void connectib_nv_tpt_conf_pack(const struct connectib_nv_tpt_conf *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log_max_payload_size);

}

void connectib_nv_tpt_conf_unpack(struct connectib_nv_tpt_conf *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->log_max_payload_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_nv_tpt_conf_print(const struct connectib_nv_tpt_conf *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tpt_conf ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_payload_size : %s (" UH_FMT ")\n", (ptr_struct->log_max_payload_size == 0 ? ("AUTOMATIC") : ((ptr_struct->log_max_payload_size == 12 ? ("_4KB") : ("unknown")))), ptr_struct->log_max_payload_size);

}

int connectib_nv_tpt_conf_size(void)
{
	 return 4;
}

void connectib_nv_tpt_conf_dump(const struct connectib_nv_tpt_conf *ptr_struct, FILE* file)
{
	connectib_nv_tpt_conf_print(ptr_struct, file, 0);
}

void connectib_nv_global_pci_cap_pack(const struct connectib_nv_global_pci_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->non_prefetchable_pf_bar_supported);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pf_bar_size_supported);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_bar_size_supported);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->num_pf_msix_supported);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->num_vf_msix_supported);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->num_pfs_supported);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->max_total_bar_valid);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->max_total_msix_valid);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->max_vfs_per_pf_valid);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_vfs_per_pf);

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_num_pfs);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fpp_support);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_qos_control_support);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sriov_support);

	offset = 90;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->max_log_pf_uar_bar_size);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->max_log_vf_uar_bar_size);

	offset = 74;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->max_num_pf_msix);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->max_num_vf_msix);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_total_msix);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_total_bar);

}

void connectib_nv_global_pci_cap_unpack(struct connectib_nv_global_pci_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->non_prefetchable_pf_bar_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->pf_bar_size_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->vf_bar_size_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->num_pf_msix_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->num_vf_msix_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->num_pfs_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->max_total_bar_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->max_total_msix_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->max_vfs_per_pf_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->max_vfs_per_pf = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 44;
	ptr_struct->max_num_pfs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 34;
	ptr_struct->fpp_support = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 33;
	ptr_struct->vf_qos_control_support = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->sriov_support = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 90;
	ptr_struct->max_log_pf_uar_bar_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 84;
	ptr_struct->max_log_vf_uar_bar_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 74;
	ptr_struct->max_num_pf_msix = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 64;
	ptr_struct->max_num_vf_msix = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 96;
	ptr_struct->max_total_msix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->max_total_bar = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_nv_global_pci_cap_print(const struct connectib_nv_global_pci_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_global_pci_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "non_prefetchable_pf_bar_supported : " UH_FMT "\n", ptr_struct->non_prefetchable_pf_bar_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pf_bar_size_supported : " UH_FMT "\n", ptr_struct->pf_bar_size_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_bar_size_supported : " UH_FMT "\n", ptr_struct->vf_bar_size_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_pf_msix_supported : " UH_FMT "\n", ptr_struct->num_pf_msix_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_vf_msix_supported : " UH_FMT "\n", ptr_struct->num_vf_msix_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_pfs_supported    : " UH_FMT "\n", ptr_struct->num_pfs_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_total_bar_valid  : " UH_FMT "\n", ptr_struct->max_total_bar_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_total_msix_valid : " UH_FMT "\n", ptr_struct->max_total_msix_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_vfs_per_pf_valid : " UH_FMT "\n", ptr_struct->max_vfs_per_pf_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_vfs_per_pf       : " UH_FMT "\n", ptr_struct->max_vfs_per_pf);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_pfs          : " UH_FMT "\n", ptr_struct->max_num_pfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpp_support          : " UH_FMT "\n", ptr_struct->fpp_support);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_qos_control_support : " UH_FMT "\n", ptr_struct->vf_qos_control_support);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sriov_support        : " UH_FMT "\n", ptr_struct->sriov_support);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_log_pf_uar_bar_size : " UH_FMT "\n", ptr_struct->max_log_pf_uar_bar_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_log_vf_uar_bar_size : " UH_FMT "\n", ptr_struct->max_log_vf_uar_bar_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_pf_msix      : " UH_FMT "\n", ptr_struct->max_num_pf_msix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_vf_msix      : " UH_FMT "\n", ptr_struct->max_num_vf_msix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_total_msix       : " U32H_FMT "\n", ptr_struct->max_total_msix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_total_bar        : " U32H_FMT "\n", ptr_struct->max_total_bar);

}

int connectib_nv_global_pci_cap_size(void)
{
	 return 20;
}

void connectib_nv_global_pci_cap_dump(const struct connectib_nv_global_pci_cap *ptr_struct, FILE* file)
{
	connectib_nv_global_pci_cap_print(ptr_struct, file, 0);
}

void connectib_nv_global_pci_conf_pack(const struct connectib_nv_global_pci_conf *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->toolpf_en);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->non_prefetchable_pf_bar);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->non_prefetchable_pf_bar_valid);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pf_bar_size_valid);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_bar_size_valid);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->num_pf_msix_valid);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->num_vf_msix_valid);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->num_pfs_valid);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fpp_valid);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->full_vf_qos_valid);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sriov_valid);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->total_vfs);

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_pfs);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fpp_en);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->full_vf_qos);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sriov_en);

	offset = 90;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log_pf_uar_bar_size);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log_vf_uar_bar_size);

	offset = 74;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->num_pf_msix);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->num_vf_msix);

}

void connectib_nv_global_pci_conf_unpack(struct connectib_nv_global_pci_conf *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->toolpf_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->non_prefetchable_pf_bar = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->non_prefetchable_pf_bar_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->pf_bar_size_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->vf_bar_size_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->num_pf_msix_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->num_vf_msix_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->num_pfs_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->fpp_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->full_vf_qos_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->sriov_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->total_vfs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 44;
	ptr_struct->num_pfs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 34;
	ptr_struct->fpp_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 33;
	ptr_struct->full_vf_qos = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->sriov_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 90;
	ptr_struct->log_pf_uar_bar_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 84;
	ptr_struct->log_vf_uar_bar_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 74;
	ptr_struct->num_pf_msix = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 64;
	ptr_struct->num_vf_msix = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

}

void connectib_nv_global_pci_conf_print(const struct connectib_nv_global_pci_conf *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_global_pci_conf ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "toolpf_en            : " UH_FMT "\n", ptr_struct->toolpf_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "non_prefetchable_pf_bar : " UH_FMT "\n", ptr_struct->non_prefetchable_pf_bar);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "non_prefetchable_pf_bar_valid : " UH_FMT "\n", ptr_struct->non_prefetchable_pf_bar_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pf_bar_size_valid    : " UH_FMT "\n", ptr_struct->pf_bar_size_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_bar_size_valid    : " UH_FMT "\n", ptr_struct->vf_bar_size_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_pf_msix_valid    : " UH_FMT "\n", ptr_struct->num_pf_msix_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_vf_msix_valid    : " UH_FMT "\n", ptr_struct->num_vf_msix_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_pfs_valid        : " UH_FMT "\n", ptr_struct->num_pfs_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpp_valid            : " UH_FMT "\n", ptr_struct->fpp_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "full_vf_qos_valid    : " UH_FMT "\n", ptr_struct->full_vf_qos_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sriov_valid          : " UH_FMT "\n", ptr_struct->sriov_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "total_vfs            : " UH_FMT "\n", ptr_struct->total_vfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_pfs              : " UH_FMT "\n", ptr_struct->num_pfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpp_en               : " UH_FMT "\n", ptr_struct->fpp_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "full_vf_qos          : " UH_FMT "\n", ptr_struct->full_vf_qos);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sriov_en             : " UH_FMT "\n", ptr_struct->sriov_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_pf_uar_bar_size  : " UH_FMT "\n", ptr_struct->log_pf_uar_bar_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_vf_uar_bar_size  : " UH_FMT "\n", ptr_struct->log_vf_uar_bar_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_pf_msix          : " UH_FMT "\n", ptr_struct->num_pf_msix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_vf_msix          : " UH_FMT "\n", ptr_struct->num_vf_msix);

}

int connectib_nv_global_pci_conf_size(void)
{
	 return 12;
}

void connectib_nv_global_pci_conf_dump(const struct connectib_nv_global_pci_conf *ptr_struct, FILE* file)
{
	connectib_nv_global_pci_conf_print(ptr_struct, file, 0);
}

void connectib_pcie_acs_ctrl_fields_pack(const struct connectib_pcie_acs_ctrl_fields *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_source_validation);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_translation_blocking);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_p2p_request_redirect);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_p2p_complition_redirect);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_upstream_forwarding);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_p2p_egress_control);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_direct_translated_p2p);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->egress_controll_vector_size);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_source_validation_enable);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_translation_blocking_enable);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_p2p_request_redirect_enable);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_p2p_complition_redirect_enable);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_p2p_upstream_forwarding_enable);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_p2p_egress_control_enable);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_direct_translated_p2p_enable);

}

void connectib_pcie_acs_ctrl_fields_unpack(struct connectib_pcie_acs_ctrl_fields *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->acs_source_validation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->acs_translation_blocking = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->acs_p2p_request_redirect = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->acs_p2p_complition_redirect = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->acs_upstream_forwarding = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->acs_p2p_egress_control = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->acs_direct_translated_p2p = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->egress_controll_vector_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 15;
	ptr_struct->acs_source_validation_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->acs_translation_blocking_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->acs_p2p_request_redirect_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->acs_p2p_complition_redirect_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->acs_p2p_upstream_forwarding_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->acs_p2p_egress_control_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->acs_direct_translated_p2p_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_pcie_acs_ctrl_fields_print(const struct connectib_pcie_acs_ctrl_fields *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_acs_ctrl_fields ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_source_validation : " UH_FMT "\n", ptr_struct->acs_source_validation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_translation_blocking : " UH_FMT "\n", ptr_struct->acs_translation_blocking);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_p2p_request_redirect : " UH_FMT "\n", ptr_struct->acs_p2p_request_redirect);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_p2p_complition_redirect : " UH_FMT "\n", ptr_struct->acs_p2p_complition_redirect);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_upstream_forwarding : " UH_FMT "\n", ptr_struct->acs_upstream_forwarding);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_p2p_egress_control : " UH_FMT "\n", ptr_struct->acs_p2p_egress_control);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_direct_translated_p2p : " UH_FMT "\n", ptr_struct->acs_direct_translated_p2p);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "egress_controll_vector_size : " UH_FMT "\n", ptr_struct->egress_controll_vector_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_source_validation_enable : " UH_FMT "\n", ptr_struct->acs_source_validation_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_translation_blocking_enable : " UH_FMT "\n", ptr_struct->acs_translation_blocking_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_p2p_request_redirect_enable : " UH_FMT "\n", ptr_struct->acs_p2p_request_redirect_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_p2p_complition_redirect_enable : " UH_FMT "\n", ptr_struct->acs_p2p_complition_redirect_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_p2p_upstream_forwarding_enable : " UH_FMT "\n", ptr_struct->acs_p2p_upstream_forwarding_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_p2p_egress_control_enable : " UH_FMT "\n", ptr_struct->acs_p2p_egress_control_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_direct_translated_p2p_enable : " UH_FMT "\n", ptr_struct->acs_direct_translated_p2p_enable);

}

int connectib_pcie_acs_ctrl_fields_size(void)
{
	 return 4;
}

void connectib_pcie_acs_ctrl_fields_dump(const struct connectib_pcie_acs_ctrl_fields *ptr_struct, FILE* file)
{
	connectib_pcie_acs_ctrl_fields_print(ptr_struct, file, 0);
}

void connectib_header_log_pack(const struct connectib_header_log *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hdr_127_96);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hdr_95_64);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hdr_63_32);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hdr_31_0);

}

void connectib_header_log_unpack(struct connectib_header_log *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->hdr_127_96 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->hdr_95_64 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->hdr_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->hdr_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_header_log_print(const struct connectib_header_log *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_header_log ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hdr_127_96           : " U32H_FMT "\n", ptr_struct->hdr_127_96);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hdr_95_64            : " U32H_FMT "\n", ptr_struct->hdr_95_64);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hdr_63_32            : " U32H_FMT "\n", ptr_struct->hdr_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hdr_31_0             : " U32H_FMT "\n", ptr_struct->hdr_31_0);

}

int connectib_header_log_size(void)
{
	 return 16;
}

void connectib_header_log_dump(const struct connectib_header_log *ptr_struct, FILE* file)
{
	connectib_header_log_print(ptr_struct, file, 0);
}

void connectib_advanced_error_cap_ctl_pack(const struct connectib_advanced_error_cap_ctl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->first_error_pointer);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ecrc_generation_cap);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ecrc_generation_enable);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ecrc_check_cap);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ecrc_check_enable);

}

void connectib_advanced_error_cap_ctl_unpack(struct connectib_advanced_error_cap_ctl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 27;
	ptr_struct->first_error_pointer = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 26;
	ptr_struct->ecrc_generation_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->ecrc_generation_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->ecrc_check_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->ecrc_check_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_advanced_error_cap_ctl_print(const struct connectib_advanced_error_cap_ctl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_advanced_error_cap_ctl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_error_pointer  : " UH_FMT "\n", ptr_struct->first_error_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecrc_generation_cap  : " UH_FMT "\n", ptr_struct->ecrc_generation_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecrc_generation_enable : " UH_FMT "\n", ptr_struct->ecrc_generation_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecrc_check_cap       : " UH_FMT "\n", ptr_struct->ecrc_check_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecrc_check_enable    : " UH_FMT "\n", ptr_struct->ecrc_check_enable);

}

int connectib_advanced_error_cap_ctl_size(void)
{
	 return 4;
}

void connectib_advanced_error_cap_ctl_dump(const struct connectib_advanced_error_cap_ctl *ptr_struct, FILE* file)
{
	connectib_advanced_error_cap_ctl_print(ptr_struct, file, 0);
}

void connectib_correctable_error_mask_pack(const struct connectib_correctable_error_mask *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->receiver_error_mask);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved1);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved2);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved3);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved4);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved5);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bad_tlp_mask);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bad_dllp_mask);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->replay_num_rollover_mask);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved9);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved10);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved11);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->replay_timer_timeout_mask);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->advisory_non_fatal_err_mask);

}

void connectib_correctable_error_mask_unpack(struct connectib_correctable_error_mask *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->receiver_error_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->reserved1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->reserved2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->reserved3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->reserved4 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->reserved5 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->bad_tlp_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->bad_dllp_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->replay_num_rollover_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->reserved9 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->reserved10 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->reserved11 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->replay_timer_timeout_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->advisory_non_fatal_err_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_correctable_error_mask_print(const struct connectib_correctable_error_mask *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_correctable_error_mask ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "receiver_error_mask  : " UH_FMT "\n", ptr_struct->receiver_error_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved1            : " UH_FMT "\n", ptr_struct->reserved1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved2            : " UH_FMT "\n", ptr_struct->reserved2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved3            : " UH_FMT "\n", ptr_struct->reserved3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved4            : " UH_FMT "\n", ptr_struct->reserved4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved5            : " UH_FMT "\n", ptr_struct->reserved5);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bad_tlp_mask         : " UH_FMT "\n", ptr_struct->bad_tlp_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bad_dllp_mask        : " UH_FMT "\n", ptr_struct->bad_dllp_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "replay_num_rollover_mask : " UH_FMT "\n", ptr_struct->replay_num_rollover_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved9            : " UH_FMT "\n", ptr_struct->reserved9);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved10           : " UH_FMT "\n", ptr_struct->reserved10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved11           : " UH_FMT "\n", ptr_struct->reserved11);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "replay_timer_timeout_mask : " UH_FMT "\n", ptr_struct->replay_timer_timeout_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "advisory_non_fatal_err_mask : " UH_FMT "\n", ptr_struct->advisory_non_fatal_err_mask);

}

int connectib_correctable_error_mask_size(void)
{
	 return 4;
}

void connectib_correctable_error_mask_dump(const struct connectib_correctable_error_mask *ptr_struct, FILE* file)
{
	connectib_correctable_error_mask_print(ptr_struct, file, 0);
}

void connectib_correctable_error_status_pack(const struct connectib_correctable_error_status *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->receiver_error_status);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bad_tlp_status);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bad_dllp_status);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->replay_num_rollover_status);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->replay_timer_timeout_status);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->advisory_nonftl_err_status);

}

void connectib_correctable_error_status_unpack(struct connectib_correctable_error_status *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->receiver_error_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->bad_tlp_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->bad_dllp_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->replay_num_rollover_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->replay_timer_timeout_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->advisory_nonftl_err_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_correctable_error_status_print(const struct connectib_correctable_error_status *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_correctable_error_status ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "receiver_error_status : " UH_FMT "\n", ptr_struct->receiver_error_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bad_tlp_status       : " UH_FMT "\n", ptr_struct->bad_tlp_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bad_dllp_status      : " UH_FMT "\n", ptr_struct->bad_dllp_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "replay_num_rollover_status : " UH_FMT "\n", ptr_struct->replay_num_rollover_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "replay_timer_timeout_status : " UH_FMT "\n", ptr_struct->replay_timer_timeout_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "advisory_nonftl_err_status : " UH_FMT "\n", ptr_struct->advisory_nonftl_err_status);

}

int connectib_correctable_error_status_size(void)
{
	 return 4;
}

void connectib_correctable_error_status_dump(const struct connectib_correctable_error_status *ptr_struct, FILE* file)
{
	connectib_correctable_error_status_print(ptr_struct, file, 0);
}

void connectib_uncorrectable_error_severity_pack(const struct connectib_uncorrectable_error_severity *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->training_error_severity);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved1);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved2);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved3);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->data_link_protocol_error);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved5);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved6);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved7);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved8);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved9);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved10);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved11);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->poisoned_tlp);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->flow_control_protocol_error);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->completion_timeout);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->completer_abort);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->unexpected_completion);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->receiver_overflow);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->malformed_tlp);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ecrc_error);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->unsupported_request_error);

}

void connectib_uncorrectable_error_severity_unpack(struct connectib_uncorrectable_error_severity *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->training_error_severity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->reserved1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->reserved2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->reserved3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->data_link_protocol_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->reserved5 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->reserved6 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->reserved7 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->reserved8 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->reserved9 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->reserved10 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->reserved11 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->poisoned_tlp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->flow_control_protocol_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->completion_timeout = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->completer_abort = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 15;
	ptr_struct->unexpected_completion = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->receiver_overflow = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->malformed_tlp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->ecrc_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->unsupported_request_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_uncorrectable_error_severity_print(const struct connectib_uncorrectable_error_severity *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_uncorrectable_error_severity ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "training_error_severity : " UH_FMT "\n", ptr_struct->training_error_severity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved1            : " UH_FMT "\n", ptr_struct->reserved1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved2            : " UH_FMT "\n", ptr_struct->reserved2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved3            : " UH_FMT "\n", ptr_struct->reserved3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_link_protocol_error : " UH_FMT "\n", ptr_struct->data_link_protocol_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved5            : " UH_FMT "\n", ptr_struct->reserved5);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved6            : " UH_FMT "\n", ptr_struct->reserved6);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved7            : " UH_FMT "\n", ptr_struct->reserved7);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved8            : " UH_FMT "\n", ptr_struct->reserved8);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved9            : " UH_FMT "\n", ptr_struct->reserved9);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved10           : " UH_FMT "\n", ptr_struct->reserved10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved11           : " UH_FMT "\n", ptr_struct->reserved11);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "poisoned_tlp         : " UH_FMT "\n", ptr_struct->poisoned_tlp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_control_protocol_error : " UH_FMT "\n", ptr_struct->flow_control_protocol_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completion_timeout   : " UH_FMT "\n", ptr_struct->completion_timeout);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completer_abort      : " UH_FMT "\n", ptr_struct->completer_abort);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unexpected_completion : " UH_FMT "\n", ptr_struct->unexpected_completion);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "receiver_overflow    : " UH_FMT "\n", ptr_struct->receiver_overflow);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "malformed_tlp        : " UH_FMT "\n", ptr_struct->malformed_tlp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecrc_error           : " UH_FMT "\n", ptr_struct->ecrc_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unsupported_request_error : " UH_FMT "\n", ptr_struct->unsupported_request_error);

}

int connectib_uncorrectable_error_severity_size(void)
{
	 return 4;
}

void connectib_uncorrectable_error_severity_dump(const struct connectib_uncorrectable_error_severity *ptr_struct, FILE* file)
{
	connectib_uncorrectable_error_severity_print(ptr_struct, file, 0);
}

void connectib_uncorrectable_error_mask_pack(const struct connectib_uncorrectable_error_mask *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->training_error_mask);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved1);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved2);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved3);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->data_link_protocol_error);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved5);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved6);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved7);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved8);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved9);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved10);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reserved11);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->poisoned_tlp);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->flow_control_protocol_error);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->completion_timeout);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->completer_abort);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->unexpected_completion);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->receiver_overflow);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->malformed_tlp);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ecrc_error);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->unsupported_request_error);

}

void connectib_uncorrectable_error_mask_unpack(struct connectib_uncorrectable_error_mask *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->training_error_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->reserved1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->reserved2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->reserved3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->data_link_protocol_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->reserved5 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->reserved6 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->reserved7 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->reserved8 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->reserved9 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->reserved10 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->reserved11 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->poisoned_tlp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->flow_control_protocol_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->completion_timeout = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->completer_abort = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 15;
	ptr_struct->unexpected_completion = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->receiver_overflow = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->malformed_tlp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->ecrc_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->unsupported_request_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_uncorrectable_error_mask_print(const struct connectib_uncorrectable_error_mask *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_uncorrectable_error_mask ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "training_error_mask  : " UH_FMT "\n", ptr_struct->training_error_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved1            : " UH_FMT "\n", ptr_struct->reserved1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved2            : " UH_FMT "\n", ptr_struct->reserved2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved3            : " UH_FMT "\n", ptr_struct->reserved3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_link_protocol_error : " UH_FMT "\n", ptr_struct->data_link_protocol_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved5            : " UH_FMT "\n", ptr_struct->reserved5);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved6            : " UH_FMT "\n", ptr_struct->reserved6);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved7            : " UH_FMT "\n", ptr_struct->reserved7);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved8            : " UH_FMT "\n", ptr_struct->reserved8);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved9            : " UH_FMT "\n", ptr_struct->reserved9);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved10           : " UH_FMT "\n", ptr_struct->reserved10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved11           : " UH_FMT "\n", ptr_struct->reserved11);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "poisoned_tlp         : " UH_FMT "\n", ptr_struct->poisoned_tlp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_control_protocol_error : " UH_FMT "\n", ptr_struct->flow_control_protocol_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completion_timeout   : " UH_FMT "\n", ptr_struct->completion_timeout);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completer_abort      : " UH_FMT "\n", ptr_struct->completer_abort);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unexpected_completion : " UH_FMT "\n", ptr_struct->unexpected_completion);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "receiver_overflow    : " UH_FMT "\n", ptr_struct->receiver_overflow);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "malformed_tlp        : " UH_FMT "\n", ptr_struct->malformed_tlp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecrc_error           : " UH_FMT "\n", ptr_struct->ecrc_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unsupported_request_error : " UH_FMT "\n", ptr_struct->unsupported_request_error);

}

int connectib_uncorrectable_error_mask_size(void)
{
	 return 4;
}

void connectib_uncorrectable_error_mask_dump(const struct connectib_uncorrectable_error_mask *ptr_struct, FILE* file)
{
	connectib_uncorrectable_error_mask_print(ptr_struct, file, 0);
}

void connectib_uncorrectable_error_status_pack(const struct connectib_uncorrectable_error_status *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->training_error_status);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->data_link_protocol_error_status);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->poisoned_tlp_status);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->flow_control_protocol_error_status);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->completion_timeout_status);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->completer_abort_status);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->unexpected_completion_status);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->receiver_overflow_status);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->malformed_tlp_status);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ecrc_error_status);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->unsupported_request_error_status);

}

void connectib_uncorrectable_error_status_unpack(struct connectib_uncorrectable_error_status *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->training_error_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->data_link_protocol_error_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->poisoned_tlp_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->flow_control_protocol_error_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->completion_timeout_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->completer_abort_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 15;
	ptr_struct->unexpected_completion_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->receiver_overflow_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->malformed_tlp_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->ecrc_error_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->unsupported_request_error_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_uncorrectable_error_status_print(const struct connectib_uncorrectable_error_status *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_uncorrectable_error_status ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "training_error_status : " UH_FMT "\n", ptr_struct->training_error_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_link_protocol_error_status : " UH_FMT "\n", ptr_struct->data_link_protocol_error_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "poisoned_tlp_status  : " UH_FMT "\n", ptr_struct->poisoned_tlp_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_control_protocol_error_status : " UH_FMT "\n", ptr_struct->flow_control_protocol_error_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completion_timeout_status : " UH_FMT "\n", ptr_struct->completion_timeout_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completer_abort_status : " UH_FMT "\n", ptr_struct->completer_abort_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unexpected_completion_status : " UH_FMT "\n", ptr_struct->unexpected_completion_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "receiver_overflow_status : " UH_FMT "\n", ptr_struct->receiver_overflow_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "malformed_tlp_status : " UH_FMT "\n", ptr_struct->malformed_tlp_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ecrc_error_status    : " UH_FMT "\n", ptr_struct->ecrc_error_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unsupported_request_error_status : " UH_FMT "\n", ptr_struct->unsupported_request_error_status);

}

int connectib_uncorrectable_error_status_size(void)
{
	 return 4;
}

void connectib_uncorrectable_error_status_dump(const struct connectib_uncorrectable_error_status *ptr_struct, FILE* file)
{
	connectib_uncorrectable_error_status_print(ptr_struct, file, 0);
}

void connectib_bar_high_pack(const struct connectib_bar_high *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->addr_63_32);

}

void connectib_bar_high_unpack(struct connectib_bar_high *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->addr_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_bar_high_print(const struct connectib_bar_high *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_bar_high ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr_63_32           : " U32H_FMT "\n", ptr_struct->addr_63_32);

}

int connectib_bar_high_size(void)
{
	 return 4;
}

void connectib_bar_high_dump(const struct connectib_bar_high *ptr_struct, FILE* file)
{
	connectib_bar_high_print(ptr_struct, file, 0);
}

void connectib_bar_low_pack(const struct connectib_bar_low *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->memory_or_io);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->type);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->prefetchable);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 28, (u_int32_t)ptr_struct->addr_31_4);

}

void connectib_bar_low_unpack(struct connectib_bar_low *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->memory_or_io = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 28;
	ptr_struct->prefetchable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->addr_31_4 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 28);

}

void connectib_bar_low_print(const struct connectib_bar_low *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_bar_low ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "memory_or_io         : " UH_FMT "\n", ptr_struct->memory_or_io);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " UH_FMT "\n", ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prefetchable         : " UH_FMT "\n", ptr_struct->prefetchable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr_31_4            : " UH_FMT "\n", ptr_struct->addr_31_4);

}

int connectib_bar_low_size(void)
{
	 return 4;
}

void connectib_bar_low_dump(const struct connectib_bar_low *ptr_struct, FILE* file)
{
	connectib_bar_low_print(ptr_struct, file, 0);
}

void connectib_sriov_status_pack(const struct connectib_sriov_status *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_mig_status);

}

void connectib_sriov_status_unpack(struct connectib_sriov_status *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->vf_mig_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_sriov_status_print(const struct connectib_sriov_status *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sriov_status ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_mig_status        : " UH_FMT "\n", ptr_struct->vf_mig_status);

}

int connectib_sriov_status_size(void)
{
	 return 2;
}

void connectib_sriov_status_dump(const struct connectib_sriov_status *ptr_struct, FILE* file)
{
	connectib_sriov_status_print(ptr_struct, file, 0);
}

void connectib_sriov_control_pack(const struct connectib_sriov_control *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_enable);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_migration_en);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_migration_int_en);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_mse);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ari_capable_hierarchy);

}

void connectib_sriov_control_unpack(struct connectib_sriov_control *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->vf_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->vf_migration_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->vf_migration_int_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->vf_mse = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->ari_capable_hierarchy = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_sriov_control_print(const struct connectib_sriov_control *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sriov_control ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_enable            : " UH_FMT "\n", ptr_struct->vf_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_migration_en      : " UH_FMT "\n", ptr_struct->vf_migration_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_migration_int_en  : " UH_FMT "\n", ptr_struct->vf_migration_int_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_mse               : " UH_FMT "\n", ptr_struct->vf_mse);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ari_capable_hierarchy : " UH_FMT "\n", ptr_struct->ari_capable_hierarchy);

}

int connectib_sriov_control_size(void)
{
	 return 2;
}

void connectib_sriov_control_dump(const struct connectib_sriov_control *ptr_struct, FILE* file)
{
	connectib_sriov_control_print(ptr_struct, file, 0);
}

void connectib_sriov_capabilities_pack(const struct connectib_sriov_capabilities *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_mig_capable);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ari_cap_hier_preserved);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->vf_mig_int_msg_num);

}

void connectib_sriov_capabilities_unpack(struct connectib_sriov_capabilities *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->vf_mig_capable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->ari_cap_hier_preserved = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->vf_mig_int_msg_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

}

void connectib_sriov_capabilities_print(const struct connectib_sriov_capabilities *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sriov_capabilities ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_mig_capable       : " UH_FMT "\n", ptr_struct->vf_mig_capable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ari_cap_hier_preserved : " UH_FMT "\n", ptr_struct->ari_cap_hier_preserved);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_mig_int_msg_num   : " UH_FMT "\n", ptr_struct->vf_mig_int_msg_num);

}

int connectib_sriov_capabilities_size(void)
{
	 return 4;
}

void connectib_sriov_capabilities_dump(const struct connectib_sriov_capabilities *ptr_struct, FILE* file)
{
	connectib_sriov_capabilities_print(ptr_struct, file, 0);
}

void connectib_ari_control_pack(const struct connectib_ari_control *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mfvc_en);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs_en);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->func_group);

}

void connectib_ari_control_unpack(struct connectib_ari_control *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->mfvc_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->acs_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->func_group = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_ari_control_print(const struct connectib_ari_control *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ari_control ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfvc_en              : " UH_FMT "\n", ptr_struct->mfvc_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_en               : " UH_FMT "\n", ptr_struct->acs_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "func_group           : " UH_FMT "\n", ptr_struct->func_group);

}

int connectib_ari_control_size(void)
{
	 return 2;
}

void connectib_ari_control_dump(const struct connectib_ari_control *ptr_struct, FILE* file)
{
	connectib_ari_control_print(ptr_struct, file, 0);
}

void connectib_ari_capabilities_pack(const struct connectib_ari_capabilities *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mfvc);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->acs);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->next_func_num);

}

void connectib_ari_capabilities_unpack(struct connectib_ari_capabilities *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->mfvc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->acs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->next_func_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_ari_capabilities_print(const struct connectib_ari_capabilities *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ari_capabilities ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfvc                 : " UH_FMT "\n", ptr_struct->mfvc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs                  : " UH_FMT "\n", ptr_struct->acs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_func_num        : " UH_FMT "\n", ptr_struct->next_func_num);

}

int connectib_ari_capabilities_size(void)
{
	 return 2;
}

void connectib_ari_capabilities_dump(const struct connectib_ari_capabilities *ptr_struct, FILE* file)
{
	connectib_ari_capabilities_print(ptr_struct, file, 0);
}

void connectib_lane_eql_ctl_pack(const struct connectib_lane_eql_ctl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->downstream_port_transmitter_preset0);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->downstream_port_receiver_preset_hint0);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->upstream_port_transmitter_preset0);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->upstream_port_receiver_preset_hint0);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->downstream_port_transmitter_preset1);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->downstream_port_receiver_preset_hint1);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->upstream_port_transmitter_preset1);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->upstream_port_receiver_preset_hint1);

}

void connectib_lane_eql_ctl_unpack(struct connectib_lane_eql_ctl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->downstream_port_transmitter_preset0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 25;
	ptr_struct->downstream_port_receiver_preset_hint0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 20;
	ptr_struct->upstream_port_transmitter_preset0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 17;
	ptr_struct->upstream_port_receiver_preset_hint0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 12;
	ptr_struct->downstream_port_transmitter_preset1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 9;
	ptr_struct->downstream_port_receiver_preset_hint1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 4;
	ptr_struct->upstream_port_transmitter_preset1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 1;
	ptr_struct->upstream_port_receiver_preset_hint1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_lane_eql_ctl_print(const struct connectib_lane_eql_ctl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_lane_eql_ctl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "downstream_port_transmitter_preset0 : " UH_FMT "\n", ptr_struct->downstream_port_transmitter_preset0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "downstream_port_receiver_preset_hint0 : " UH_FMT "\n", ptr_struct->downstream_port_receiver_preset_hint0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "upstream_port_transmitter_preset0 : " UH_FMT "\n", ptr_struct->upstream_port_transmitter_preset0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "upstream_port_receiver_preset_hint0 : " UH_FMT "\n", ptr_struct->upstream_port_receiver_preset_hint0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "downstream_port_transmitter_preset1 : " UH_FMT "\n", ptr_struct->downstream_port_transmitter_preset1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "downstream_port_receiver_preset_hint1 : " UH_FMT "\n", ptr_struct->downstream_port_receiver_preset_hint1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "upstream_port_transmitter_preset1 : " UH_FMT "\n", ptr_struct->upstream_port_transmitter_preset1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "upstream_port_receiver_preset_hint1 : " UH_FMT "\n", ptr_struct->upstream_port_receiver_preset_hint1);

}

int connectib_lane_eql_ctl_size(void)
{
	 return 4;
}

void connectib_lane_eql_ctl_dump(const struct connectib_lane_eql_ctl *ptr_struct, FILE* file)
{
	connectib_lane_eql_ctl_print(ptr_struct, file, 0);
}

void connectib_pciex_link_ctl3_pack(const struct connectib_pciex_link_ctl3 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->perform_equalization);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_equalization_req_int_en);

}

void connectib_pciex_link_ctl3_unpack(struct connectib_pciex_link_ctl3 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->perform_equalization = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->link_equalization_req_int_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_pciex_link_ctl3_print(const struct connectib_pciex_link_ctl3 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pciex_link_ctl3 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "perform_equalization : " UH_FMT "\n", ptr_struct->perform_equalization);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_equalization_req_int_en : " UH_FMT "\n", ptr_struct->link_equalization_req_int_en);

}

int connectib_pciex_link_ctl3_size(void)
{
	 return 4;
}

void connectib_pciex_link_ctl3_dump(const struct connectib_pciex_link_ctl3 *ptr_struct, FILE* file)
{
	connectib_pciex_link_ctl3_print(ptr_struct, file, 0);
}

void connectib_bridge_control_pack(const struct connectib_bridge_control *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->parity_error_response_enable);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->serr_enable);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->secondary_bus_reset);

}

void connectib_bridge_control_unpack(struct connectib_bridge_control *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->parity_error_response_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->serr_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->secondary_bus_reset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_bridge_control_print(const struct connectib_bridge_control *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_bridge_control ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parity_error_response_enable : " UH_FMT "\n", ptr_struct->parity_error_response_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serr_enable          : " UH_FMT "\n", ptr_struct->serr_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "secondary_bus_reset  : " UH_FMT "\n", ptr_struct->secondary_bus_reset);

}

int connectib_bridge_control_size(void)
{
	 return 2;
}

void connectib_bridge_control_dump(const struct connectib_bridge_control *ptr_struct, FILE* file)
{
	connectib_bridge_control_print(ptr_struct, file, 0);
}

void connectib_exp_rom_pack(const struct connectib_exp_rom *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->exp_rom_en);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 21, (u_int32_t)ptr_struct->addr_31_11);

}

void connectib_exp_rom_unpack(struct connectib_exp_rom *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->exp_rom_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->addr_31_11 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 21);

}

void connectib_exp_rom_print(const struct connectib_exp_rom *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_exp_rom ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "exp_rom_en           : " UH_FMT "\n", ptr_struct->exp_rom_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr_31_11           : " UH_FMT "\n", ptr_struct->addr_31_11);

}

int connectib_exp_rom_size(void)
{
	 return 4;
}

void connectib_exp_rom_dump(const struct connectib_exp_rom *ptr_struct, FILE* file)
{
	connectib_exp_rom_print(ptr_struct, file, 0);
}

void connectib_secondary_status_pack(const struct connectib_secondary_status *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->master_data_parity_error);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->signaled_target_abort);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->received_target_abort);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->received_master_abort);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->received_system_error);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->detected_parity_error);

}

void connectib_secondary_status_unpack(struct connectib_secondary_status *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	ptr_struct->master_data_parity_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->signaled_target_abort = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->received_target_abort = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->received_master_abort = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->received_system_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->detected_parity_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_secondary_status_print(const struct connectib_secondary_status *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_secondary_status ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "master_data_parity_error : " UH_FMT "\n", ptr_struct->master_data_parity_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signaled_target_abort : " UH_FMT "\n", ptr_struct->signaled_target_abort);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "received_target_abort : " UH_FMT "\n", ptr_struct->received_target_abort);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "received_master_abort : " UH_FMT "\n", ptr_struct->received_master_abort);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "received_system_error : " UH_FMT "\n", ptr_struct->received_system_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "detected_parity_error : " UH_FMT "\n", ptr_struct->detected_parity_error);

}

int connectib_secondary_status_size(void)
{
	 return 2;
}

void connectib_secondary_status_dump(const struct connectib_secondary_status *ptr_struct, FILE* file)
{
	connectib_secondary_status_print(ptr_struct, file, 0);
}

void connectib_header_type_pack(const struct connectib_header_type *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->header_layout);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->multi_function_device);

}

void connectib_header_type_unpack(struct connectib_header_type *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->header_layout = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 0;
	ptr_struct->multi_function_device = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_header_type_print(const struct connectib_header_type *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_header_type ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "header_layout        : " UH_FMT "\n", ptr_struct->header_layout);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "multi_function_device : " UH_FMT "\n", ptr_struct->multi_function_device);

}

int connectib_header_type_size(void)
{
	 return 1;
}

void connectib_header_type_dump(const struct connectib_header_type *ptr_struct, FILE* file)
{
	connectib_header_type_print(ptr_struct, file, 0);
}

void connectib_stat_reg1_pack(const struct connectib_stat_reg1 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->int_status);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->capabilities_list);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->master_data_perr);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->signaled_ta);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->received_ta);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->received_ma);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->signaled_serr);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->detected_perr);

}

void connectib_stat_reg1_unpack(struct connectib_stat_reg1 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	ptr_struct->int_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->capabilities_list = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->master_data_perr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->signaled_ta = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->received_ta = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->received_ma = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->signaled_serr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->detected_perr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_stat_reg1_print(const struct connectib_stat_reg1 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_stat_reg1 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "int_status           : " UH_FMT "\n", ptr_struct->int_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capabilities_list    : " UH_FMT "\n", ptr_struct->capabilities_list);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "master_data_perr     : " UH_FMT "\n", ptr_struct->master_data_perr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signaled_ta          : " UH_FMT "\n", ptr_struct->signaled_ta);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "received_ta          : " UH_FMT "\n", ptr_struct->received_ta);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "received_ma          : " UH_FMT "\n", ptr_struct->received_ma);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signaled_serr        : " UH_FMT "\n", ptr_struct->signaled_serr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "detected_perr        : " UH_FMT "\n", ptr_struct->detected_perr);

}

int connectib_stat_reg1_size(void)
{
	 return 2;
}

void connectib_stat_reg1_dump(const struct connectib_stat_reg1 *ptr_struct, FILE* file)
{
	connectib_stat_reg1_print(ptr_struct, file, 0);
}

void connectib_cmd_reg1_pack(const struct connectib_cmd_reg1 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->io_space);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->memory_space);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bus_master_en);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->perr_enable);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->serr_enable);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->int_disable);

}

void connectib_cmd_reg1_unpack(struct connectib_cmd_reg1 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->io_space = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->memory_space = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->bus_master_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->perr_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->serr_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->int_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_cmd_reg1_print(const struct connectib_cmd_reg1 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmd_reg1 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "io_space             : " UH_FMT "\n", ptr_struct->io_space);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "memory_space         : " UH_FMT "\n", ptr_struct->memory_space);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bus_master_en        : " UH_FMT "\n", ptr_struct->bus_master_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "perr_enable          : " UH_FMT "\n", ptr_struct->perr_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serr_enable          : " UH_FMT "\n", ptr_struct->serr_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "int_disable          : " UH_FMT "\n", ptr_struct->int_disable);

}

int connectib_cmd_reg1_size(void)
{
	 return 2;
}

void connectib_cmd_reg1_dump(const struct connectib_cmd_reg1 *ptr_struct, FILE* file)
{
	connectib_cmd_reg1_print(ptr_struct, file, 0);
}

void connectib_message_control_pack(const struct connectib_message_control *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->table_size);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->function_mask);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->msix_en);

}

void connectib_message_control_unpack(struct connectib_message_control *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	ptr_struct->table_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

	offset = 1;
	ptr_struct->function_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->msix_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_message_control_print(const struct connectib_message_control *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_message_control ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_size           : " UH_FMT "\n", ptr_struct->table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "function_mask        : " UH_FMT "\n", ptr_struct->function_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msix_en              : " UH_FMT "\n", ptr_struct->msix_en);

}

int connectib_message_control_size(void)
{
	 return 2;
}

void connectib_message_control_dump(const struct connectib_message_control *ptr_struct, FILE* file)
{
	connectib_message_control_print(ptr_struct, file, 0);
}

void connectib_link_stat2_pack(const struct connectib_link_stat2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->current_de_emphasis_level);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->equalization_complete);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->equalization_phase1_successful);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->equalization_phase2_successful);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->equalization_phase3_successful);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_equalization_request);

}

void connectib_link_stat2_unpack(struct connectib_link_stat2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->current_de_emphasis_level = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->equalization_complete = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->equalization_phase1_successful = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->equalization_phase2_successful = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->equalization_phase3_successful = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->link_equalization_request = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_link_stat2_print(const struct connectib_link_stat2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_link_stat2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "current_de_emphasis_level : " UH_FMT "\n", ptr_struct->current_de_emphasis_level);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "equalization_complete : " UH_FMT "\n", ptr_struct->equalization_complete);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "equalization_phase1_successful : " UH_FMT "\n", ptr_struct->equalization_phase1_successful);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "equalization_phase2_successful : " UH_FMT "\n", ptr_struct->equalization_phase2_successful);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "equalization_phase3_successful : " UH_FMT "\n", ptr_struct->equalization_phase3_successful);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_equalization_request : " UH_FMT "\n", ptr_struct->link_equalization_request);

}

int connectib_link_stat2_size(void)
{
	 return 2;
}

void connectib_link_stat2_dump(const struct connectib_link_stat2 *ptr_struct, FILE* file)
{
	connectib_link_stat2_print(ptr_struct, file, 0);
}

void connectib_link_ctl2_pack(const struct connectib_link_ctl2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->target_link_speed);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enter_compliance);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->hardware_autonomous_speed_disable);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->selectable_de_emphasis);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->transmit_margin);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->modified_compliance);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->compliance_sos);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->compliance_preset_de_emphasis);

}

void connectib_link_ctl2_unpack(struct connectib_link_ctl2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	ptr_struct->target_link_speed = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 11;
	ptr_struct->enter_compliance = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->hardware_autonomous_speed_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->selectable_de_emphasis = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->transmit_margin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 5;
	ptr_struct->modified_compliance = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->compliance_sos = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->compliance_preset_de_emphasis = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_link_ctl2_print(const struct connectib_link_ctl2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_link_ctl2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "target_link_speed    : " UH_FMT "\n", ptr_struct->target_link_speed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enter_compliance     : " UH_FMT "\n", ptr_struct->enter_compliance);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hardware_autonomous_speed_disable : " UH_FMT "\n", ptr_struct->hardware_autonomous_speed_disable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "selectable_de_emphasis : " UH_FMT "\n", ptr_struct->selectable_de_emphasis);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transmit_margin      : " UH_FMT "\n", ptr_struct->transmit_margin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modified_compliance  : " UH_FMT "\n", ptr_struct->modified_compliance);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compliance_sos       : " UH_FMT "\n", ptr_struct->compliance_sos);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compliance_preset_de_emphasis : " UH_FMT "\n", ptr_struct->compliance_preset_de_emphasis);

}

int connectib_link_ctl2_size(void)
{
	 return 2;
}

void connectib_link_ctl2_dump(const struct connectib_link_ctl2 *ptr_struct, FILE* file)
{
	connectib_link_ctl2_print(ptr_struct, file, 0);
}

void connectib_link_caps2_pack(const struct connectib_link_caps2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->supported_link_speed_vector);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cross_link_supported);

}

void connectib_link_caps2_unpack(struct connectib_link_caps2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->supported_link_speed_vector = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 23;
	ptr_struct->cross_link_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_link_caps2_print(const struct connectib_link_caps2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_link_caps2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "supported_link_speed_vector : " UH_FMT "\n", ptr_struct->supported_link_speed_vector);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cross_link_supported : " UH_FMT "\n", ptr_struct->cross_link_supported);

}

int connectib_link_caps2_size(void)
{
	 return 4;
}

void connectib_link_caps2_dump(const struct connectib_link_caps2 *ptr_struct, FILE* file)
{
	connectib_link_caps2_print(ptr_struct, file, 0);
}

void connectib_device_stat2_pack(const struct connectib_device_stat2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_device_stat2_unpack(struct connectib_device_stat2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_device_stat2_print(const struct connectib_device_stat2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_device_stat2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_device_stat2_size(void)
{
	 return 2;
}

void connectib_device_stat2_dump(const struct connectib_device_stat2 *ptr_struct, FILE* file)
{
	connectib_device_stat2_print(ptr_struct, file, 0);
}

void connectib_device_ctl2_pack(const struct connectib_device_ctl2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->completion_timeout_value);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->completion_timeout_disable);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ari_forwarding_enable);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->obff_enable);

}

void connectib_device_ctl2_unpack(struct connectib_device_ctl2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	ptr_struct->completion_timeout_value = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 11;
	ptr_struct->completion_timeout_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->ari_forwarding_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->obff_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_device_ctl2_print(const struct connectib_device_ctl2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_device_ctl2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completion_timeout_value : " UH_FMT "\n", ptr_struct->completion_timeout_value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completion_timeout_disable : " UH_FMT "\n", ptr_struct->completion_timeout_disable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ari_forwarding_enable : " UH_FMT "\n", ptr_struct->ari_forwarding_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "obff_enable          : " UH_FMT "\n", ptr_struct->obff_enable);

}

int connectib_device_ctl2_size(void)
{
	 return 2;
}

void connectib_device_ctl2_dump(const struct connectib_device_ctl2 *ptr_struct, FILE* file)
{
	connectib_device_ctl2_print(ptr_struct, file, 0);
}

void connectib_device_caps2_pack(const struct connectib_device_caps2 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->completion_timeout_ranges_supported);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->completion_timeout_disable_supported);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->ari_forwarding_supported);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->_10bit_tag_completer_supported);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->obff_supported);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->extended_fmt_field_supported);

}

void connectib_device_caps2_unpack(struct connectib_device_caps2 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->completion_timeout_ranges_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 27;
	ptr_struct->completion_timeout_disable_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->ari_forwarding_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 15;
	ptr_struct->_10bit_tag_completer_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->obff_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 11;
	ptr_struct->extended_fmt_field_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_device_caps2_print(const struct connectib_device_caps2 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_device_caps2 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completion_timeout_ranges_supported : " UH_FMT "\n", ptr_struct->completion_timeout_ranges_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "completion_timeout_disable_supported : " UH_FMT "\n", ptr_struct->completion_timeout_disable_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ari_forwarding_supported : " UH_FMT "\n", ptr_struct->ari_forwarding_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "_10bit_tag_completer_supported : " UH_FMT "\n", ptr_struct->_10bit_tag_completer_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "obff_supported       : " UH_FMT "\n", ptr_struct->obff_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_fmt_field_supported : " UH_FMT "\n", ptr_struct->extended_fmt_field_supported);

}

int connectib_device_caps2_size(void)
{
	 return 4;
}

void connectib_device_caps2_dump(const struct connectib_device_caps2 *ptr_struct, FILE* file)
{
	connectib_device_caps2_print(ptr_struct, file, 0);
}

void connectib_slot_caps_pack(const struct connectib_slot_caps *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->attention_button_present);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->power_controller_present);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mrl_sensor_present);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->attention_indicator_present);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->power_indicator_present);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->hot_plug_surprise);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->hot_plug_capable);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->slot_power_limit_value);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->slot_power_limit_scale);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->electromechanical_interlock_present);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->no_command_completed_supported);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 13, (u_int32_t)ptr_struct->physical_slot_number);

}

void connectib_slot_caps_unpack(struct connectib_slot_caps *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->attention_button_present = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->power_controller_present = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->mrl_sensor_present = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->attention_indicator_present = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->power_indicator_present = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->hot_plug_surprise = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->hot_plug_capable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->slot_power_limit_value = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 15;
	ptr_struct->slot_power_limit_scale = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 14;
	ptr_struct->electromechanical_interlock_present = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->no_command_completed_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->physical_slot_number = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 13);

}

void connectib_slot_caps_print(const struct connectib_slot_caps *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_slot_caps ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "attention_button_present : " UH_FMT "\n", ptr_struct->attention_button_present);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "power_controller_present : " UH_FMT "\n", ptr_struct->power_controller_present);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mrl_sensor_present   : " UH_FMT "\n", ptr_struct->mrl_sensor_present);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "attention_indicator_present : " UH_FMT "\n", ptr_struct->attention_indicator_present);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "power_indicator_present : " UH_FMT "\n", ptr_struct->power_indicator_present);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hot_plug_surprise    : " UH_FMT "\n", ptr_struct->hot_plug_surprise);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hot_plug_capable     : " UH_FMT "\n", ptr_struct->hot_plug_capable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slot_power_limit_value : " UH_FMT "\n", ptr_struct->slot_power_limit_value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slot_power_limit_scale : " UH_FMT "\n", ptr_struct->slot_power_limit_scale);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "electromechanical_interlock_present : " UH_FMT "\n", ptr_struct->electromechanical_interlock_present);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_command_completed_supported : " UH_FMT "\n", ptr_struct->no_command_completed_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "physical_slot_number : " UH_FMT "\n", ptr_struct->physical_slot_number);

}

int connectib_slot_caps_size(void)
{
	 return 4;
}

void connectib_slot_caps_dump(const struct connectib_slot_caps *ptr_struct, FILE* file)
{
	connectib_slot_caps_print(ptr_struct, file, 0);
}

void connectib_link_stat_pack(const struct connectib_link_stat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->link_speed);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->negotiated_link_width);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->training_error);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_training);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->slot_clock_cfg);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_bandwidth_mgmt_status);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_autonomous_bandwidth_status);

}

void connectib_link_stat_unpack(struct connectib_link_stat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	ptr_struct->link_speed = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 6;
	ptr_struct->negotiated_link_width = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 5;
	ptr_struct->training_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->link_training = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->slot_clock_cfg = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->link_bandwidth_mgmt_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->link_autonomous_bandwidth_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_link_stat_print(const struct connectib_link_stat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_link_stat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_speed           : " UH_FMT "\n", ptr_struct->link_speed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "negotiated_link_width : " UH_FMT "\n", ptr_struct->negotiated_link_width);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "training_error       : " UH_FMT "\n", ptr_struct->training_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_training        : " UH_FMT "\n", ptr_struct->link_training);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slot_clock_cfg       : " UH_FMT "\n", ptr_struct->slot_clock_cfg);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_bandwidth_mgmt_status : " UH_FMT "\n", ptr_struct->link_bandwidth_mgmt_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_autonomous_bandwidth_status : " UH_FMT "\n", ptr_struct->link_autonomous_bandwidth_status);

}

int connectib_link_stat_size(void)
{
	 return 2;
}

void connectib_link_stat_dump(const struct connectib_link_stat *ptr_struct, FILE* file)
{
	connectib_link_stat_print(ptr_struct, file, 0);
}

void connectib_link_ctl_pack(const struct connectib_link_ctl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->aspm_control);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rcb);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_disable);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->retrain_link);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->common_clock_configuration);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->extended_synch);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable_clock_power_management);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->hardware_autonomous_width_disable);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_bandwidth_mgmt_interrupt_en);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_autonomous_bandwidth_interrupt_en);

}

void connectib_link_ctl_unpack(struct connectib_link_ctl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	ptr_struct->aspm_control = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 12;
	ptr_struct->rcb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->link_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->retrain_link = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->common_clock_configuration = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->extended_synch = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->enable_clock_power_management = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->hardware_autonomous_width_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->link_bandwidth_mgmt_interrupt_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->link_autonomous_bandwidth_interrupt_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_link_ctl_print(const struct connectib_link_ctl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_link_ctl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aspm_control         : " UH_FMT "\n", ptr_struct->aspm_control);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rcb                  : " UH_FMT "\n", ptr_struct->rcb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_disable         : " UH_FMT "\n", ptr_struct->link_disable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "retrain_link         : " UH_FMT "\n", ptr_struct->retrain_link);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "common_clock_configuration : " UH_FMT "\n", ptr_struct->common_clock_configuration);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_synch       : " UH_FMT "\n", ptr_struct->extended_synch);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable_clock_power_management : " UH_FMT "\n", ptr_struct->enable_clock_power_management);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hardware_autonomous_width_disable : " UH_FMT "\n", ptr_struct->hardware_autonomous_width_disable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_bandwidth_mgmt_interrupt_en : " UH_FMT "\n", ptr_struct->link_bandwidth_mgmt_interrupt_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_autonomous_bandwidth_interrupt_en : " UH_FMT "\n", ptr_struct->link_autonomous_bandwidth_interrupt_en);

}

int connectib_link_ctl_size(void)
{
	 return 2;
}

void connectib_link_ctl_dump(const struct connectib_link_ctl *ptr_struct, FILE* file)
{
	connectib_link_ctl_print(ptr_struct, file, 0);
}

void connectib_link_caps_pack(const struct connectib_link_caps *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_link_speed);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->max_link_width);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->aspm_support);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->l0s_exit_latency);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->l1_exit_latency);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->clock_power_management);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->surprise_down_error_reporting_cap);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_active_reporting_cap);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_bw_notification_cap);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aspm_optionality_compliance);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_number);

}

void connectib_link_caps_unpack(struct connectib_link_caps *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->max_link_speed = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 22;
	ptr_struct->max_link_width = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 20;
	ptr_struct->aspm_support = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 17;
	ptr_struct->l0s_exit_latency = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 14;
	ptr_struct->l1_exit_latency = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 13;
	ptr_struct->clock_power_management = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->surprise_down_error_reporting_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->link_active_reporting_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->link_bw_notification_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->aspm_optionality_compliance = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->port_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_link_caps_print(const struct connectib_link_caps *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_link_caps ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_link_speed       : " UH_FMT "\n", ptr_struct->max_link_speed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_link_width       : " UH_FMT "\n", ptr_struct->max_link_width);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aspm_support         : " UH_FMT "\n", ptr_struct->aspm_support);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l0s_exit_latency     : " UH_FMT "\n", ptr_struct->l0s_exit_latency);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l1_exit_latency      : " UH_FMT "\n", ptr_struct->l1_exit_latency);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "clock_power_management : " UH_FMT "\n", ptr_struct->clock_power_management);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "surprise_down_error_reporting_cap : " UH_FMT "\n", ptr_struct->surprise_down_error_reporting_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_active_reporting_cap : " UH_FMT "\n", ptr_struct->link_active_reporting_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_bw_notification_cap : " UH_FMT "\n", ptr_struct->link_bw_notification_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aspm_optionality_compliance : " UH_FMT "\n", ptr_struct->aspm_optionality_compliance);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number          : " UH_FMT "\n", ptr_struct->port_number);

}

int connectib_link_caps_size(void)
{
	 return 4;
}

void connectib_link_caps_dump(const struct connectib_link_caps *ptr_struct, FILE* file)
{
	connectib_link_caps_print(ptr_struct, file, 0);
}

void connectib_device_stat_pack(const struct connectib_device_stat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->correctable_error);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->non_fatal_error);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fatal_error);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->unsupported_request);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aux_power);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->transactions_pending);

}

void connectib_device_stat_unpack(struct connectib_device_stat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->correctable_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->non_fatal_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->fatal_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->unsupported_request = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->aux_power = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->transactions_pending = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_device_stat_print(const struct connectib_device_stat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_device_stat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "correctable_error    : " UH_FMT "\n", ptr_struct->correctable_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "non_fatal_error      : " UH_FMT "\n", ptr_struct->non_fatal_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fatal_error          : " UH_FMT "\n", ptr_struct->fatal_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unsupported_request  : " UH_FMT "\n", ptr_struct->unsupported_request);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aux_power            : " UH_FMT "\n", ptr_struct->aux_power);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transactions_pending : " UH_FMT "\n", ptr_struct->transactions_pending);

}

int connectib_device_stat_size(void)
{
	 return 2;
}

void connectib_device_stat_dump(const struct connectib_device_stat *ptr_struct, FILE* file)
{
	connectib_device_stat_print(ptr_struct, file, 0);
}

void connectib_device_ctl_pack(const struct connectib_device_ctl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->correctable_error_report);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->non_fatal_error_report);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fatal_error_report);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->unsupported_request_report);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->relaxed_ordering);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->max_payload_size);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->extended_tag_field_en);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->phantom_funcs);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aux_power);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->no_snoop);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->max_read_request_size);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->initiate_function_level_reset);

}

void connectib_device_ctl_unpack(struct connectib_device_ctl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 15;
	ptr_struct->correctable_error_report = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->non_fatal_error_report = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->fatal_error_report = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->unsupported_request_report = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->relaxed_ordering = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->max_payload_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 7;
	ptr_struct->extended_tag_field_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->phantom_funcs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->aux_power = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->no_snoop = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->max_read_request_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 0;
	ptr_struct->initiate_function_level_reset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_device_ctl_print(const struct connectib_device_ctl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_device_ctl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "correctable_error_report : " UH_FMT "\n", ptr_struct->correctable_error_report);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "non_fatal_error_report : " UH_FMT "\n", ptr_struct->non_fatal_error_report);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fatal_error_report   : " UH_FMT "\n", ptr_struct->fatal_error_report);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unsupported_request_report : " UH_FMT "\n", ptr_struct->unsupported_request_report);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "relaxed_ordering     : " UH_FMT "\n", ptr_struct->relaxed_ordering);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_payload_size     : " UH_FMT "\n", ptr_struct->max_payload_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_tag_field_en : " UH_FMT "\n", ptr_struct->extended_tag_field_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phantom_funcs        : " UH_FMT "\n", ptr_struct->phantom_funcs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aux_power            : " UH_FMT "\n", ptr_struct->aux_power);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_snoop             : " UH_FMT "\n", ptr_struct->no_snoop);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_read_request_size : " UH_FMT "\n", ptr_struct->max_read_request_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "initiate_function_level_reset : " UH_FMT "\n", ptr_struct->initiate_function_level_reset);

}

int connectib_device_ctl_size(void)
{
	 return 2;
}

void connectib_device_ctl_dump(const struct connectib_device_ctl *ptr_struct, FILE* file)
{
	connectib_device_ctl_print(ptr_struct, file, 0);
}

void connectib_device_caps_pack(const struct connectib_device_caps *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->max_payload_size_supported);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->phantom_funcs);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->extended_tag_field_sup);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->l0s_acceptable_latency);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->l1_acceptable_latency);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->role_based_error_reporting);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->captured_slot_power_limit_value);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->captured_slot_power_limit_scale);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->function_level_reset_capability);

}

void connectib_device_caps_unpack(struct connectib_device_caps *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	ptr_struct->max_payload_size_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 27;
	ptr_struct->phantom_funcs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 26;
	ptr_struct->extended_tag_field_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->l0s_acceptable_latency = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 20;
	ptr_struct->l1_acceptable_latency = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 16;
	ptr_struct->role_based_error_reporting = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->captured_slot_power_limit_value = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 4;
	ptr_struct->captured_slot_power_limit_scale = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 3;
	ptr_struct->function_level_reset_capability = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_device_caps_print(const struct connectib_device_caps *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_device_caps ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_payload_size_supported : " UH_FMT "\n", ptr_struct->max_payload_size_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phantom_funcs        : " UH_FMT "\n", ptr_struct->phantom_funcs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_tag_field_sup : " UH_FMT "\n", ptr_struct->extended_tag_field_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l0s_acceptable_latency : " UH_FMT "\n", ptr_struct->l0s_acceptable_latency);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l1_acceptable_latency : " UH_FMT "\n", ptr_struct->l1_acceptable_latency);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "role_based_error_reporting : " UH_FMT "\n", ptr_struct->role_based_error_reporting);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "captured_slot_power_limit_value : " UH_FMT "\n", ptr_struct->captured_slot_power_limit_value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "captured_slot_power_limit_scale : " UH_FMT "\n", ptr_struct->captured_slot_power_limit_scale);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "function_level_reset_capability : " UH_FMT "\n", ptr_struct->function_level_reset_capability);

}

int connectib_device_caps_size(void)
{
	 return 4;
}

void connectib_device_caps_dump(const struct connectib_device_caps *ptr_struct, FILE* file)
{
	connectib_device_caps_print(ptr_struct, file, 0);
}

void connectib_pciex_cap_pack(const struct connectib_pciex_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->device_port_type);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->slot);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->int_message_num);

}

void connectib_pciex_cap_unpack(struct connectib_pciex_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 12;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 8;
	ptr_struct->device_port_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 7;
	ptr_struct->slot = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->int_message_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_pciex_cap_print(const struct connectib_pciex_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pciex_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_port_type     : " UH_FMT "\n", ptr_struct->device_port_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slot                 : " UH_FMT "\n", ptr_struct->slot);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "int_message_num      : " UH_FMT "\n", ptr_struct->int_message_num);

}

int connectib_pciex_cap_size(void)
{
	 return 2;
}

void connectib_pciex_cap_dump(const struct connectib_pciex_cap *ptr_struct, FILE* file)
{
	connectib_pciex_cap_print(ptr_struct, file, 0);
}

void connectib_external_port_pf_array_pack(const struct connectib_external_port_pf_array *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_pf_array_default_ones_pack(&(ptr_struct->port_owner), ptr_buff + offset/8);

	offset = 64;
	connectib_pf_array_default_ones_pack(&(ptr_struct->allow_rd_counters), ptr_buff + offset/8);

	offset = 128;
	connectib_pf_array_default_ones_pack(&(ptr_struct->reneg_on_change), ptr_buff + offset/8);

}

void connectib_external_port_pf_array_unpack(struct connectib_external_port_pf_array *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_pf_array_default_ones_unpack(&(ptr_struct->port_owner), ptr_buff + offset/8);

	offset = 64;
	connectib_pf_array_default_ones_unpack(&(ptr_struct->allow_rd_counters), ptr_buff + offset/8);

	offset = 128;
	connectib_pf_array_default_ones_unpack(&(ptr_struct->reneg_on_change), ptr_buff + offset/8);

}

void connectib_external_port_pf_array_print(const struct connectib_external_port_pf_array *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_external_port_pf_array ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_owner:\n");
	connectib_pf_array_default_ones_print(&(ptr_struct->port_owner), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allow_rd_counters:\n");
	connectib_pf_array_default_ones_print(&(ptr_struct->allow_rd_counters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reneg_on_change:\n");
	connectib_pf_array_default_ones_print(&(ptr_struct->reneg_on_change), file, indent_level + 1);

}

int connectib_external_port_pf_array_size(void)
{
	 return 24;
}

void connectib_external_port_pf_array_dump(const struct connectib_external_port_pf_array *ptr_struct, FILE* file)
{
	connectib_external_port_pf_array_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_data_curr_config_pack(const struct connectib_nv_tlv_data_curr_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->current_cfg_method);

}

void connectib_nv_tlv_data_curr_config_unpack(struct connectib_nv_tlv_data_curr_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->current_cfg_method = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_nv_tlv_data_curr_config_print(const struct connectib_nv_tlv_data_curr_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_data_curr_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "current_cfg_method   : " UH_FMT "\n", ptr_struct->current_cfg_method);

}

int connectib_nv_tlv_data_curr_config_size(void)
{
	 return 4;
}

void connectib_nv_tlv_data_curr_config_dump(const struct connectib_nv_tlv_data_curr_config *ptr_struct, FILE* file)
{
	connectib_nv_tlv_data_curr_config_print(ptr_struct, file, 0);
}

void connectib_nv_config_flex_nic_pack(const struct connectib_nv_config_flex_nic *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->fpga_image_bank);

}

void connectib_nv_config_flex_nic_unpack(struct connectib_nv_config_flex_nic *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->fpga_image_bank = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_nv_config_flex_nic_print(const struct connectib_nv_config_flex_nic *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_flex_nic ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_image_bank      : " UH_FMT "\n", ptr_struct->fpga_image_bank);

}

int connectib_nv_config_flex_nic_size(void)
{
	 return 4;
}

void connectib_nv_config_flex_nic_dump(const struct connectib_nv_config_flex_nic *ptr_struct, FILE* file)
{
	connectib_nv_config_flex_nic_print(ptr_struct, file, 0);
}

void connectib_nv_config_host_2_bmc_pack(const struct connectib_nv_config_host_2_bmc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->host_2_bmc_traffic_enable);

}

void connectib_nv_config_host_2_bmc_unpack(struct connectib_nv_config_host_2_bmc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->host_2_bmc_traffic_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_nv_config_host_2_bmc_print(const struct connectib_nv_config_host_2_bmc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_host_2_bmc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_2_bmc_traffic_enable : " UH_FMT "\n", ptr_struct->host_2_bmc_traffic_enable);

}

int connectib_nv_config_host_2_bmc_size(void)
{
	 return 4;
}

void connectib_nv_config_host_2_bmc_dump(const struct connectib_nv_config_host_2_bmc *ptr_struct, FILE* file)
{
	connectib_nv_config_host_2_bmc_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_tweaks_pack(const struct connectib_nv_tlv_tweaks *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->packet_pacing);

}

void connectib_nv_tlv_tweaks_unpack(struct connectib_nv_tlv_tweaks *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->packet_pacing = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_nv_tlv_tweaks_print(const struct connectib_nv_tlv_tweaks *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_tweaks ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_pacing        : " UH_FMT "\n", ptr_struct->packet_pacing);

}

int connectib_nv_tlv_tweaks_size(void)
{
	 return 4;
}

void connectib_nv_tlv_tweaks_dump(const struct connectib_nv_tlv_tweaks *ptr_struct, FILE* file)
{
	connectib_nv_tlv_tweaks_print(ptr_struct, file, 0);
}

void connectib_nv_sw_offload_conf_pack(const struct connectib_nv_sw_offload_conf *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->cqe_compression);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->one_qp_per_recovery);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lro_log_timeout0);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lro_log_timeout1);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lro_log_timeout2);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lro_log_timeout3);

}

void connectib_nv_sw_offload_conf_unpack(struct connectib_nv_sw_offload_conf *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	ptr_struct->cqe_compression = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 28;
	ptr_struct->one_qp_per_recovery = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 60;
	ptr_struct->lro_log_timeout0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 56;
	ptr_struct->lro_log_timeout1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 52;
	ptr_struct->lro_log_timeout2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 48;
	ptr_struct->lro_log_timeout3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_nv_sw_offload_conf_print(const struct connectib_nv_sw_offload_conf *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_sw_offload_conf ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_compression      : %s (" UH_FMT ")\n", (ptr_struct->cqe_compression == 0 ? ("BALANCED") : ((ptr_struct->cqe_compression == 1 ? ("AGGRESSIVE") : ("unknown")))), ptr_struct->cqe_compression);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "one_qp_per_recovery  : %s (" UH_FMT ")\n", (ptr_struct->one_qp_per_recovery == 0 ? ("BALANCED") : ((ptr_struct->one_qp_per_recovery == 1 ? ("AGGRESSIVE") : ("unknown")))), ptr_struct->one_qp_per_recovery);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_log_timeout0     : " UH_FMT "\n", ptr_struct->lro_log_timeout0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_log_timeout1     : " UH_FMT "\n", ptr_struct->lro_log_timeout1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_log_timeout2     : " UH_FMT "\n", ptr_struct->lro_log_timeout2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_log_timeout3     : " UH_FMT "\n", ptr_struct->lro_log_timeout3);

}

int connectib_nv_sw_offload_conf_size(void)
{
	 return 8;
}

void connectib_nv_sw_offload_conf_dump(const struct connectib_nv_sw_offload_conf *ptr_struct, FILE* file)
{
	connectib_nv_sw_offload_conf_print(ptr_struct, file, 0);
}

void connectib_nv_roce_1_5_conf_pack(const struct connectib_nv_roce_1_5_conf *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->roce_over_ip_next_protocol);

}

void connectib_nv_roce_1_5_conf_unpack(struct connectib_nv_roce_1_5_conf *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->roce_over_ip_next_protocol = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_nv_roce_1_5_conf_print(const struct connectib_nv_roce_1_5_conf *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_roce_1_5_conf ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "roce_over_ip_next_protocol : " UH_FMT "\n", ptr_struct->roce_over_ip_next_protocol);

}

int connectib_nv_roce_1_5_conf_size(void)
{
	 return 4;
}

void connectib_nv_roce_1_5_conf_dump(const struct connectib_nv_roce_1_5_conf *ptr_struct, FILE* file)
{
	connectib_nv_roce_1_5_conf_print(ptr_struct, file, 0);
}

void connectib_nv_qos_ets_tc_conf_pack(const struct connectib_nv_qos_ets_tc_conf *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(64, 64, i, 576, 1);
	connectib_nv_ets_tcN_config_reg_pack(&(ptr_struct->tc_configuration[i]), ptr_buff + offset/8);
	}

}

void connectib_nv_qos_ets_tc_conf_unpack(struct connectib_nv_qos_ets_tc_conf *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(64, 64, i, 576, 1);
	connectib_nv_ets_tcN_config_reg_unpack(&(ptr_struct->tc_configuration[i]), ptr_buff + offset/8);
	}

}

void connectib_nv_qos_ets_tc_conf_print(const struct connectib_nv_qos_ets_tc_conf *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_qos_ets_tc_conf ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tc_configuration_%03d:\n", i);
	connectib_nv_ets_tcN_config_reg_print(&(ptr_struct->tc_configuration[i]), file, indent_level + 1);
	}

}

int connectib_nv_qos_ets_tc_conf_size(void)
{
	 return 72;
}

void connectib_nv_qos_ets_tc_conf_dump(const struct connectib_nv_qos_ets_tc_conf *ptr_struct, FILE* file)
{
	connectib_nv_qos_ets_tc_conf_print(ptr_struct, file, 0);
}

void connectib_nv_qos_cap_pack(const struct connectib_nv_qos_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_num_of_vl);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_num_of_tc);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_num_pfc);

}

void connectib_nv_qos_cap_unpack(struct connectib_nv_qos_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->max_num_of_vl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 24;
	ptr_struct->max_num_of_tc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 20;
	ptr_struct->max_num_pfc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_nv_qos_cap_print(const struct connectib_nv_qos_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_qos_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_of_vl        : %s (" UH_FMT ")\n", (ptr_struct->max_num_of_vl == 1 ? ("_1_VL") : ((ptr_struct->max_num_of_vl == 2 ? ("_2_VLs") : ((ptr_struct->max_num_of_vl == 3 ? ("_4_VLs") : ((ptr_struct->max_num_of_vl == 4 ? ("_8_VLs") : ((ptr_struct->max_num_of_vl == 5 ? ("_15_VLs") : ("unknown")))))))))), ptr_struct->max_num_of_vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_of_tc        : " UH_FMT "\n", ptr_struct->max_num_of_tc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_pfc          : " UH_FMT "\n", ptr_struct->max_num_pfc);

}

int connectib_nv_qos_cap_size(void)
{
	 return 4;
}

void connectib_nv_qos_cap_dump(const struct connectib_nv_qos_cap *ptr_struct, FILE* file)
{
	connectib_nv_qos_cap_print(ptr_struct, file, 0);
}

void connectib_nv_qos_conf_pack(const struct connectib_nv_qos_conf *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_vl);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_tc);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_pfc);

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(60, 4, i, 96, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->priority_pfcc_pfc[i]);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(92, 4, i, 96, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->priority_tc[i]);
	}

}

void connectib_nv_qos_conf_unpack(struct connectib_nv_qos_conf *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->num_of_vl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 24;
	ptr_struct->num_of_tc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 20;
	ptr_struct->num_pfc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(60, 4, i, 96, 1);
	ptr_struct->priority_pfcc_pfc[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(92, 4, i, 96, 1);
	ptr_struct->priority_tc[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_nv_qos_conf_print(const struct connectib_nv_qos_conf *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_qos_conf ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_vl            : %s (" UH_FMT ")\n", (ptr_struct->num_of_vl == 1 ? ("_1_VL") : ((ptr_struct->num_of_vl == 2 ? ("_2_VLs") : ((ptr_struct->num_of_vl == 3 ? ("_4_VLs") : ((ptr_struct->num_of_vl == 4 ? ("_8_VLs") : ((ptr_struct->num_of_vl == 5 ? ("_15_VLs") : ("unknown")))))))))), ptr_struct->num_of_vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_tc            : %s (" UH_FMT ")\n", (ptr_struct->num_of_tc == 0 ? ("_8_TCs") : ((ptr_struct->num_of_tc == 1 ? ("_1_TC") : ((ptr_struct->num_of_tc == 2 ? ("_2_TCs") : ((ptr_struct->num_of_tc == 3 ? ("_3_TCs") : ((ptr_struct->num_of_tc == 4 ? ("_4_TCs") : ((ptr_struct->num_of_tc == 5 ? ("_5_TCs") : ((ptr_struct->num_of_tc == 6 ? ("_6_TCs") : ((ptr_struct->num_of_tc == 7 ? ("_7_TCs") : ("unknown")))))))))))))))), ptr_struct->num_of_tc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_pfc              : " UH_FMT "\n", ptr_struct->num_pfc);

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "priority_pfcc_pfc_%03d : %s (" UH_FMT ")\n", i, (ptr_struct->priority_pfcc_pfc[i] == 0 ? ("DISABLED") : ((ptr_struct->priority_pfcc_pfc[i] == 1 ? ("RX_ONLY") : ((ptr_struct->priority_pfcc_pfc[i] == 2 ? ("TX_ONLY") : ((ptr_struct->priority_pfcc_pfc[i] == 3 ? ("RX_TX") : ("unknown")))))))), ptr_struct->priority_pfcc_pfc[i]);
	}

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "priority_tc_%03d     : " UH_FMT "\n", i, ptr_struct->priority_tc[i]);
	}

}

int connectib_nv_qos_conf_size(void)
{
	 return 12;
}

void connectib_nv_qos_conf_dump(const struct connectib_nv_qos_conf *ptr_struct, FILE* file)
{
	connectib_nv_qos_conf_print(ptr_struct, file, 0);
}

void connectib_nv_lldp_nb_dcbx_pack(const struct connectib_nv_lldp_nb_dcbx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ieee_dcbx_en);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cee_dcbx_en);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dcbx_willing);

}

void connectib_nv_lldp_nb_dcbx_unpack(struct connectib_nv_lldp_nb_dcbx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->ieee_dcbx_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->cee_dcbx_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->dcbx_willing = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_nv_lldp_nb_dcbx_print(const struct connectib_nv_lldp_nb_dcbx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_lldp_nb_dcbx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ieee_dcbx_en         : " UH_FMT "\n", ptr_struct->ieee_dcbx_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cee_dcbx_en          : " UH_FMT "\n", ptr_struct->cee_dcbx_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcbx_willing         : " UH_FMT "\n", ptr_struct->dcbx_willing);

}

int connectib_nv_lldp_nb_dcbx_size(void)
{
	 return 4;
}

void connectib_nv_lldp_nb_dcbx_dump(const struct connectib_nv_lldp_nb_dcbx *ptr_struct, FILE* file)
{
	connectib_nv_lldp_nb_dcbx_print(ptr_struct, file, 0);
}

void connectib_nv_lldp_nb_conf_pack(const struct connectib_nv_lldp_nb_conf *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->lldp_msg_tx_interval);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lldp_nb_dcbx);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lldp_nb_rx_mode);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lldp_nb_tx_mode);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lldp_nb_rx_buffer_size);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lldp_nb_tx_buffer_size);

}

void connectib_nv_lldp_nb_conf_unpack(struct connectib_nv_lldp_nb_conf *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 20;
	ptr_struct->lldp_msg_tx_interval = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 4;
	ptr_struct->lldp_nb_dcbx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->lldp_nb_rx_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 0;
	ptr_struct->lldp_nb_tx_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 80;
	ptr_struct->lldp_nb_rx_buffer_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->lldp_nb_tx_buffer_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_nv_lldp_nb_conf_print(const struct connectib_nv_lldp_nb_conf *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_lldp_nb_conf ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_msg_tx_interval : " UH_FMT "\n", ptr_struct->lldp_msg_tx_interval);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_dcbx         : " UH_FMT "\n", ptr_struct->lldp_nb_dcbx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_rx_mode      : %s (" UH_FMT ")\n", (ptr_struct->lldp_nb_rx_mode == 0 ? ("OFF") : ((ptr_struct->lldp_nb_rx_mode == 1 ? ("MANDATORY") : ((ptr_struct->lldp_nb_rx_mode == 2 ? ("ALL") : ("unknown")))))), ptr_struct->lldp_nb_rx_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_tx_mode      : %s (" UH_FMT ")\n", (ptr_struct->lldp_nb_tx_mode == 0 ? ("OFF") : ((ptr_struct->lldp_nb_tx_mode == 1 ? ("MANDATORY") : ((ptr_struct->lldp_nb_tx_mode == 2 ? ("ALL") : ("unknown")))))), ptr_struct->lldp_nb_tx_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_rx_buffer_size : " UH_FMT "\n", ptr_struct->lldp_nb_rx_buffer_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_tx_buffer_size : " UH_FMT "\n", ptr_struct->lldp_nb_tx_buffer_size);

}

int connectib_nv_lldp_nb_conf_size(void)
{
	 return 12;
}

void connectib_nv_lldp_nb_conf_dump(const struct connectib_nv_lldp_nb_conf *ptr_struct, FILE* file)
{
	connectib_nv_lldp_nb_conf_print(ptr_struct, file, 0);
}

void connectib_nv_lldp_nb_cap_pack(const struct connectib_nv_lldp_nb_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lldp_nb_dcbx_en);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lldp_nb_rx_cap);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lldp_nb_tx_cap);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_lldp_nb_rx_buffer_size);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_lldp_nb_tx_buffer_size);

	offset = 89;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->max_num_of_tx_optional_lldp_tlv);

}

void connectib_nv_lldp_nb_cap_unpack(struct connectib_nv_lldp_nb_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 4;
	ptr_struct->lldp_nb_dcbx_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->lldp_nb_rx_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 0;
	ptr_struct->lldp_nb_tx_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 48;
	ptr_struct->max_lldp_nb_rx_buffer_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->max_lldp_nb_tx_buffer_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 89;
	ptr_struct->max_num_of_tx_optional_lldp_tlv = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

}

void connectib_nv_lldp_nb_cap_print(const struct connectib_nv_lldp_nb_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_lldp_nb_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_dcbx_en      : " UH_FMT "\n", ptr_struct->lldp_nb_dcbx_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_rx_cap       : %s (" UH_FMT ")\n", (ptr_struct->lldp_nb_rx_cap == 0 ? ("NA") : ((ptr_struct->lldp_nb_rx_cap == 1 ? ("MANDATORY") : ((ptr_struct->lldp_nb_rx_cap == 2 ? ("ALL") : ("unknown")))))), ptr_struct->lldp_nb_rx_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_tx_cap       : %s (" UH_FMT ")\n", (ptr_struct->lldp_nb_tx_cap == 0 ? ("NA") : ((ptr_struct->lldp_nb_tx_cap == 1 ? ("MANDATORY") : ((ptr_struct->lldp_nb_tx_cap == 2 ? ("ALL") : ((ptr_struct->lldp_nb_tx_cap == 3 ? ("reserved") : ("unknown")))))))), ptr_struct->lldp_nb_tx_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_lldp_nb_rx_buffer_size : " UH_FMT "\n", ptr_struct->max_lldp_nb_rx_buffer_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_lldp_nb_tx_buffer_size : " UH_FMT "\n", ptr_struct->max_lldp_nb_tx_buffer_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_of_tx_optional_lldp_tlv : " UH_FMT "\n", ptr_struct->max_num_of_tx_optional_lldp_tlv);

}

int connectib_nv_lldp_nb_cap_size(void)
{
	 return 12;
}

void connectib_nv_lldp_nb_cap_dump(const struct connectib_nv_lldp_nb_cap *ptr_struct, FILE* file)
{
	connectib_nv_lldp_nb_cap_print(ptr_struct, file, 0);
}

void connectib_nv_keep_link_up_pack(const struct connectib_nv_keep_link_up *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->keep_eth_link_up);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->keep_ib_link_up);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->keep_link_up_on_boot);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->keep_link_up_on_standby);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_clear_port_stat_on_loading);

}

void connectib_nv_keep_link_up_unpack(struct connectib_nv_keep_link_up *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->keep_eth_link_up = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->keep_ib_link_up = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->keep_link_up_on_boot = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->keep_link_up_on_standby = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->disable_clear_port_stat_on_loading = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_nv_keep_link_up_print(const struct connectib_nv_keep_link_up *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_keep_link_up ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "keep_eth_link_up     : " UH_FMT "\n", ptr_struct->keep_eth_link_up);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "keep_ib_link_up      : " UH_FMT "\n", ptr_struct->keep_ib_link_up);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "keep_link_up_on_boot : " UH_FMT "\n", ptr_struct->keep_link_up_on_boot);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "keep_link_up_on_standby : " UH_FMT "\n", ptr_struct->keep_link_up_on_standby);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_clear_port_stat_on_loading : " UH_FMT "\n", ptr_struct->disable_clear_port_stat_on_loading);

}

int connectib_nv_keep_link_up_size(void)
{
	 return 4;
}

void connectib_nv_keep_link_up_dump(const struct connectib_nv_keep_link_up *ptr_struct, FILE* file)
{
	connectib_nv_keep_link_up_print(ptr_struct, file, 0);
}

void connectib_nv_roce_cc_ecn_pack(const struct connectib_nv_roce_cc_ecn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->clamp_tgt_rate_after_time_inc);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->clamp_tgt_rate);

	offset = 47;
	adb2c_push_bits_to_buff(ptr_buff, offset, 17, (u_int32_t)ptr_struct->rpg_time_reset);

	offset = 81;
	adb2c_push_bits_to_buff(ptr_buff, offset, 15, (u_int32_t)ptr_struct->rpg_byte_reset);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->rpg_threshold);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rpg_max_rate);

	offset = 175;
	adb2c_push_bits_to_buff(ptr_buff, offset, 17, (u_int32_t)ptr_struct->rpg_ai_rate);

	offset = 207;
	adb2c_push_bits_to_buff(ptr_buff, offset, 17, (u_int32_t)ptr_struct->rpg_hai_rate);

	offset = 252;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rpg_gd);

	offset = 280;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->rpg_min_dec_fac);

	offset = 303;
	adb2c_push_bits_to_buff(ptr_buff, offset, 17, (u_int32_t)ptr_struct->rpg_min_rate);

	offset = 335;
	adb2c_push_bits_to_buff(ptr_buff, offset, 17, (u_int32_t)ptr_struct->rate_to_set_on_first_cnp);

	offset = 374;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->dce_tcp_g);

	offset = 399;
	adb2c_push_bits_to_buff(ptr_buff, offset, 17, (u_int32_t)ptr_struct->dce_tcp_rtt);

	offset = 431;
	adb2c_push_bits_to_buff(ptr_buff, offset, 17, (u_int32_t)ptr_struct->rate_reduce_monitor_period);

	offset = 470;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->initial_alpha_value);

	offset = 495;
	adb2c_push_bits_to_buff(ptr_buff, offset, 17, (u_int32_t)ptr_struct->min_time_between_cnps);

	offset = 541;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->cnp_802p_prio);

	offset = 530;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->cnp_dscp);

}

void connectib_nv_roce_cc_ecn_unpack(struct connectib_nv_roce_cc_ecn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->clamp_tgt_rate_after_time_inc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->clamp_tgt_rate = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 47;
	ptr_struct->rpg_time_reset = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 17);

	offset = 81;
	ptr_struct->rpg_byte_reset = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 15);

	offset = 123;
	ptr_struct->rpg_threshold = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 128;
	ptr_struct->rpg_max_rate = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 175;
	ptr_struct->rpg_ai_rate = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 17);

	offset = 207;
	ptr_struct->rpg_hai_rate = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 17);

	offset = 252;
	ptr_struct->rpg_gd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 280;
	ptr_struct->rpg_min_dec_fac = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 303;
	ptr_struct->rpg_min_rate = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 17);

	offset = 335;
	ptr_struct->rate_to_set_on_first_cnp = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 17);

	offset = 374;
	ptr_struct->dce_tcp_g = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 399;
	ptr_struct->dce_tcp_rtt = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 17);

	offset = 431;
	ptr_struct->rate_reduce_monitor_period = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 17);

	offset = 470;
	ptr_struct->initial_alpha_value = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 495;
	ptr_struct->min_time_between_cnps = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 17);

	offset = 541;
	ptr_struct->cnp_802p_prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 530;
	ptr_struct->cnp_dscp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

}

void connectib_nv_roce_cc_ecn_print(const struct connectib_nv_roce_cc_ecn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_roce_cc_ecn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "clamp_tgt_rate_after_time_inc : " UH_FMT "\n", ptr_struct->clamp_tgt_rate_after_time_inc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "clamp_tgt_rate       : " UH_FMT "\n", ptr_struct->clamp_tgt_rate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rpg_time_reset       : " UH_FMT "\n", ptr_struct->rpg_time_reset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rpg_byte_reset       : " UH_FMT "\n", ptr_struct->rpg_byte_reset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rpg_threshold        : " UH_FMT "\n", ptr_struct->rpg_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rpg_max_rate         : " U32H_FMT "\n", ptr_struct->rpg_max_rate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rpg_ai_rate          : " UH_FMT "\n", ptr_struct->rpg_ai_rate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rpg_hai_rate         : " UH_FMT "\n", ptr_struct->rpg_hai_rate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rpg_gd               : " UH_FMT "\n", ptr_struct->rpg_gd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rpg_min_dec_fac      : " UH_FMT "\n", ptr_struct->rpg_min_dec_fac);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rpg_min_rate         : " UH_FMT "\n", ptr_struct->rpg_min_rate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rate_to_set_on_first_cnp : " UH_FMT "\n", ptr_struct->rate_to_set_on_first_cnp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dce_tcp_g            : " UH_FMT "\n", ptr_struct->dce_tcp_g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dce_tcp_rtt          : " UH_FMT "\n", ptr_struct->dce_tcp_rtt);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rate_reduce_monitor_period : " UH_FMT "\n", ptr_struct->rate_reduce_monitor_period);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "initial_alpha_value  : " UH_FMT "\n", ptr_struct->initial_alpha_value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "min_time_between_cnps : " UH_FMT "\n", ptr_struct->min_time_between_cnps);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cnp_802p_prio        : " UH_FMT "\n", ptr_struct->cnp_802p_prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cnp_dscp             : " UH_FMT "\n", ptr_struct->cnp_dscp);

}

int connectib_nv_roce_cc_ecn_size(void)
{
	 return 68;
}

void connectib_nv_roce_cc_ecn_dump(const struct connectib_nv_roce_cc_ecn *ptr_struct, FILE* file)
{
	connectib_nv_roce_cc_ecn_print(ptr_struct, file, 0);
}

void connectib_nv_roce_cc_pack(const struct connectib_nv_roce_cc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->roce_cc_enable_pri);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->roce_cc_algorithm);

}

void connectib_nv_roce_cc_unpack(struct connectib_nv_roce_cc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->roce_cc_enable_pri = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->roce_cc_algorithm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_nv_roce_cc_print(const struct connectib_nv_roce_cc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_roce_cc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "roce_cc_enable_pri   : " UH_FMT "\n", ptr_struct->roce_cc_enable_pri);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "roce_cc_algorithm    : %s (" UH_FMT ")\n", (ptr_struct->roce_cc_algorithm == 0 ? ("ECN") : ((ptr_struct->roce_cc_algorithm == 1 ? ("QCN") : ("unknown")))), ptr_struct->roce_cc_algorithm);

}

int connectib_nv_roce_cc_size(void)
{
	 return 4;
}

void connectib_nv_roce_cc_dump(const struct connectib_nv_roce_cc *ptr_struct, FILE* file)
{
	connectib_nv_roce_cc_print(ptr_struct, file, 0);
}

void connectib_dword_val_pack(const struct connectib_dword_val *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->val);

}

void connectib_dword_val_unpack(struct connectib_dword_val *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_dword_val_print(const struct connectib_dword_val *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dword_val ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "val                  : " U32H_FMT "\n", ptr_struct->val);

}

int connectib_dword_val_size(void)
{
	 return 4;
}

void connectib_dword_val_dump(const struct connectib_dword_val *ptr_struct, FILE* file)
{
	connectib_dword_val_print(ptr_struct, file, 0);
}

void connectib_port_cap_mask1_fields_pack(const struct connectib_port_cap_mask1_fields *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_sm);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_notice_sup);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_trap_sup);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_optional_ipd_sup);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_automatic_migration_sup);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_sl_mapping_sup);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_mkey_nvram);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_pkey_nvram);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_led_info_sup);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_sm_disabled);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_system_image_guid_sup);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_pkey_switch_external_port_trap_sup);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_cable_info_supported);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_extended_speed_sup);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_capability_mask2_supported);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_communication_management_sup);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_snmp_tunneling_sup);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_reinit_sup);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_device_management_sup);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_vendor_class_sup);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_dr_notice_sup);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_capability_mask_notice_sup);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_boot_management_sup);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_link_round_trip_latency_sup);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_client_reregistration_sup);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_other_local_changes_notice_supported);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_link_speed_width_pairs_table_supported);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_vendor_specific_mads_table_supported);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_multicast_pkey_trap_suppression_supported);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_multicast_fdb_top_supported);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_hierarchy_info_supported);

}

void connectib_port_cap_mask1_fields_unpack(struct connectib_port_cap_mask1_fields *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	ptr_struct->is_sm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->is_notice_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->is_trap_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->is_optional_ipd_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->is_automatic_migration_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->is_sl_mapping_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->is_mkey_nvram = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->is_pkey_nvram = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->is_led_info_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->is_sm_disabled = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->is_system_image_guid_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->is_pkey_switch_external_port_trap_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->is_cable_info_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->is_extended_speed_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->is_capability_mask2_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 15;
	ptr_struct->is_communication_management_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->is_snmp_tunneling_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->is_reinit_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->is_device_management_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->is_vendor_class_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->is_dr_notice_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 9;
	ptr_struct->is_capability_mask_notice_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->is_boot_management_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->is_link_round_trip_latency_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->is_client_reregistration_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->is_other_local_changes_notice_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->is_link_speed_width_pairs_table_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->is_vendor_specific_mads_table_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->is_multicast_pkey_trap_suppression_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->is_multicast_fdb_top_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->is_hierarchy_info_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_port_cap_mask1_fields_print(const struct connectib_port_cap_mask1_fields *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_port_cap_mask1_fields ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_sm                : " UH_FMT "\n", ptr_struct->is_sm);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_notice_sup        : " UH_FMT "\n", ptr_struct->is_notice_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_trap_sup          : " UH_FMT "\n", ptr_struct->is_trap_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_optional_ipd_sup  : " UH_FMT "\n", ptr_struct->is_optional_ipd_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_automatic_migration_sup : " UH_FMT "\n", ptr_struct->is_automatic_migration_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_sl_mapping_sup    : " UH_FMT "\n", ptr_struct->is_sl_mapping_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_mkey_nvram        : " UH_FMT "\n", ptr_struct->is_mkey_nvram);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_pkey_nvram        : " UH_FMT "\n", ptr_struct->is_pkey_nvram);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_led_info_sup      : " UH_FMT "\n", ptr_struct->is_led_info_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_sm_disabled       : " UH_FMT "\n", ptr_struct->is_sm_disabled);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_system_image_guid_sup : " UH_FMT "\n", ptr_struct->is_system_image_guid_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_pkey_switch_external_port_trap_sup : " UH_FMT "\n", ptr_struct->is_pkey_switch_external_port_trap_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_cable_info_supported : " UH_FMT "\n", ptr_struct->is_cable_info_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_extended_speed_sup : " UH_FMT "\n", ptr_struct->is_extended_speed_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_capability_mask2_supported : " UH_FMT "\n", ptr_struct->is_capability_mask2_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_communication_management_sup : " UH_FMT "\n", ptr_struct->is_communication_management_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_snmp_tunneling_sup : " UH_FMT "\n", ptr_struct->is_snmp_tunneling_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_reinit_sup        : " UH_FMT "\n", ptr_struct->is_reinit_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_device_management_sup : " UH_FMT "\n", ptr_struct->is_device_management_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_vendor_class_sup  : " UH_FMT "\n", ptr_struct->is_vendor_class_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_dr_notice_sup     : " UH_FMT "\n", ptr_struct->is_dr_notice_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_capability_mask_notice_sup : " UH_FMT "\n", ptr_struct->is_capability_mask_notice_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_boot_management_sup : " UH_FMT "\n", ptr_struct->is_boot_management_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_link_round_trip_latency_sup : " UH_FMT "\n", ptr_struct->is_link_round_trip_latency_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_client_reregistration_sup : " UH_FMT "\n", ptr_struct->is_client_reregistration_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_other_local_changes_notice_supported : " UH_FMT "\n", ptr_struct->is_other_local_changes_notice_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_link_speed_width_pairs_table_supported : " UH_FMT "\n", ptr_struct->is_link_speed_width_pairs_table_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_vendor_specific_mads_table_supported : " UH_FMT "\n", ptr_struct->is_vendor_specific_mads_table_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_multicast_pkey_trap_suppression_supported : " UH_FMT "\n", ptr_struct->is_multicast_pkey_trap_suppression_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_multicast_fdb_top_supported : " UH_FMT "\n", ptr_struct->is_multicast_fdb_top_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_hierarchy_info_supported : " UH_FMT "\n", ptr_struct->is_hierarchy_info_supported);

}

int connectib_port_cap_mask1_fields_size(void)
{
	 return 4;
}

void connectib_port_cap_mask1_fields_dump(const struct connectib_port_cap_mask1_fields *ptr_struct, FILE* file)
{
	connectib_port_cap_mask1_fields_print(ptr_struct, file, 0);
}

void connectib_dcr_odp_event_entry_pack(const struct connectib_dcr_odp_event_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->dcrn);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

}

void connectib_dcr_odp_event_entry_unpack(struct connectib_dcr_odp_event_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->dcrn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_dcr_odp_event_entry_print(const struct connectib_dcr_odp_event_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dcr_odp_event_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcrn                 : " UH_FMT "\n", ptr_struct->dcrn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

}

int connectib_dcr_odp_event_entry_size(void)
{
	 return 4;
}

void connectib_dcr_odp_event_entry_dump(const struct connectib_dcr_odp_event_entry *ptr_struct, FILE* file)
{
	connectib_dcr_odp_event_entry_print(ptr_struct, file, 0);
}

void connectib_flow_counter_list_pack(const struct connectib_flow_counter_list *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->flow_counter_id);

}

void connectib_flow_counter_list_unpack(struct connectib_flow_counter_list *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->flow_counter_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_flow_counter_list_print(const struct connectib_flow_counter_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_flow_counter_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_counter_id      : " UH_FMT "\n", ptr_struct->flow_counter_id);

}

int connectib_flow_counter_list_size(void)
{
	 return 8;
}

void connectib_flow_counter_list_dump(const struct connectib_flow_counter_list *ptr_struct, FILE* file)
{
	connectib_flow_counter_list_print(ptr_struct, file, 0);
}

void connectib_dest_format_struct_pack(const struct connectib_dest_format_struct *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->destination_id);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->destination_type);

}

void connectib_dest_format_struct_unpack(struct connectib_dest_format_struct *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->destination_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->destination_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_dest_format_struct_print(const struct connectib_dest_format_struct *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dest_format_struct ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "destination_id       : " UH_FMT "\n", ptr_struct->destination_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "destination_type     : %s (" UH_FMT ")\n", (ptr_struct->destination_type == 0 ? ("VPORT") : ((ptr_struct->destination_type == 1 ? ("FLOW_TABLE") : ((ptr_struct->destination_type == 2 ? ("TIR") : ((ptr_struct->destination_type == 3 ? ("QP") : ("unknown")))))))), ptr_struct->destination_type);

}

int connectib_dest_format_struct_size(void)
{
	 return 8;
}

void connectib_dest_format_struct_dump(const struct connectib_dest_format_struct *ptr_struct, FILE* file)
{
	connectib_dest_format_struct_print(ptr_struct, file, 0);
}

void connectib_odp_hca_params_pack(const struct connectib_odp_hca_params *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->odp_rc);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->odp_uc);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->odp_ud);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->odp_xrc);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->odp_dc);

}

void connectib_odp_hca_params_unpack(struct connectib_odp_hca_params *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->odp_rc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->odp_uc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->odp_ud = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->odp_xrc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->odp_dc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_odp_hca_params_print(const struct connectib_odp_hca_params *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_odp_hca_params ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "odp_rc               : " UH_FMT "\n", ptr_struct->odp_rc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "odp_uc               : " UH_FMT "\n", ptr_struct->odp_uc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "odp_ud               : " UH_FMT "\n", ptr_struct->odp_ud);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "odp_xrc              : " UH_FMT "\n", ptr_struct->odp_xrc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "odp_dc               : " UH_FMT "\n", ptr_struct->odp_dc);

}

int connectib_odp_hca_params_size(void)
{
	 return 4;
}

void connectib_odp_hca_params_dump(const struct connectib_odp_hca_params *ptr_struct, FILE* file)
{
	connectib_odp_hca_params_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_special_pack(const union connectib_cmdif_ctx_special *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_cmdif_ctx_fpga_cap_pack(&(ptr_struct->fpga_cap), ptr_buff);
}

void connectib_cmdif_ctx_special_unpack(union connectib_cmdif_ctx_special *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_cmdif_ctx_fpga_cap_unpack(&(ptr_struct->fpga_cap), ptr_buff);
}

void connectib_cmdif_ctx_special_print(const union connectib_cmdif_ctx_special *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx_special ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "c:\n");
	connectib_cmdif_ctx_special_create_print(&(ptr_struct->c), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tir:\n");
	connectib_cmdif_ctx_special_modify_tir_print(&(ptr_struct->tir), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tis:\n");
	connectib_cmdif_ctx_special_modify_tis_print(&(ptr_struct->tis), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp:\n");
	connectib_cmdif_ctx_special_qp_commands_print(&(ptr_struct->qp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rq:\n");
	connectib_cmdif_ctx_special_create_rq_print(&(ptr_struct->rq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq:\n");
	connectib_cmdif_ctx_special_create_srq_print(&(ptr_struct->srq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xrq:\n");
	connectib_cmdif_ctx_special_create_xrq_print(&(ptr_struct->xrq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt:\n");
	connectib_cmdif_ctx_rqt_print(&(ptr_struct->rqt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eq:\n");
	connectib_cmdif_ctx_special_create_eq_print(&(ptr_struct->eq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "c_cq:\n");
	connectib_cmdif_ctx_special_create_cq_print(&(ptr_struct->c_cq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_cq:\n");
	connectib_cmdif_ctx_special_modify_cq_print(&(ptr_struct->m_cq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mkey:\n");
	connectib_cmdif_ctx_special_mkey_print(&(ptr_struct->mkey), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "arm_rq:\n");
	connectib_cmdif_ctx_special_arm_rq_print(&(ptr_struct->arm_rq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modify_rmp:\n");
	connectib_cmdif_ctx_special_modify_rmp_print(&(ptr_struct->modify_rmp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sniffer:\n");
	connectib_cmdif_ctx_sniffer_print(&(ptr_struct->sniffer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mcg:\n");
	connectib_cmdif_ctx_mcg_print(&(ptr_struct->mcg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "psv:\n");
	connectib_cmdif_ctx_psv_print(&(ptr_struct->psv), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt:\n");
	connectib_cmdif_ctx_create_mtt_print(&(ptr_struct->mtt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmfs_fte:\n");
	connectib_cmdif_ctx_dmfs_fte_print(&(ptr_struct->dmfs_fte), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmfs_fg:\n");
	connectib_cmdif_ctx_dmfs_fg_print(&(ptr_struct->dmfs_fg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmfs_ft:\n");
	connectib_cmdif_ctx_dmfs_ft_print(&(ptr_struct->dmfs_ft), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l2_table:\n");
	connectib_cmdif_ctx_l2_table_print(&(ptr_struct->l2_table), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modify_rq:\n");
	connectib_cmdif_modify_rq_print(&(ptr_struct->modify_rq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modify_sq:\n");
	connectib_cmdif_modify_sq_print(&(ptr_struct->modify_sq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lag:\n");
	connectib_cmdif_create_lag_print(&(ptr_struct->lag), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_access_reg:\n");
	connectib_cmdif_ctx_fpga_access_reg_print(&(ptr_struct->fpga_access_reg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_cap:\n");
	connectib_cmdif_ctx_fpga_cap_print(&(ptr_struct->fpga_cap), file, indent_level + 1);

}

int connectib_cmdif_ctx_special_size(void)
{
	 return 32;
}

void connectib_cmdif_ctx_special_dump(const union connectib_cmdif_ctx_special *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_special_print(ptr_struct, file, 0);
}

void connectib_flush_type_pack(const union connectib_flush_type *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_flush_type_bitmask_pack(&(ptr_struct->bitmask), ptr_buff);
}

void connectib_flush_type_unpack(union connectib_flush_type *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_flush_type_bitmask_unpack(&(ptr_struct->bitmask), ptr_buff);
}

void connectib_flush_type_print(const union connectib_flush_type *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_flush_type ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bits:\n");
	connectib_flush_type_bits_print(&(ptr_struct->bits), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bitmask:\n");
	connectib_flush_type_bitmask_print(&(ptr_struct->bitmask), file, indent_level + 1);

}

int connectib_flush_type_size(void)
{
	 return 4;
}

void connectib_flush_type_dump(const union connectib_flush_type *ptr_struct, FILE* file)
{
	connectib_flush_type_print(ptr_struct, file, 0);
}

void connectib_general_notification_delay_drop_event_pack(const struct connectib_general_notification_delay_drop_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->rq_user_index_delay_drop);

}

void connectib_general_notification_delay_drop_event_unpack(struct connectib_general_notification_delay_drop_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->rq_user_index_delay_drop = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_general_notification_delay_drop_event_print(const struct connectib_general_notification_delay_drop_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_general_notification_delay_drop_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rq_user_index_delay_drop : " UH_FMT "\n", ptr_struct->rq_user_index_delay_drop);

}

int connectib_general_notification_delay_drop_event_size(void)
{
	 return 28;
}

void connectib_general_notification_delay_drop_event_dump(const struct connectib_general_notification_delay_drop_event *ptr_struct, FILE* file)
{
	connectib_general_notification_delay_drop_event_print(ptr_struct, file, 0);
}

void connectib_pps_event_pack(const struct connectib_pps_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pin);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_stamp_h);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->time_stamp_l);

}

void connectib_pps_event_unpack(struct connectib_pps_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->pin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->time_stamp_h = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->time_stamp_l = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_pps_event_print(const struct connectib_pps_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pps_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pin                  : " UH_FMT "\n", ptr_struct->pin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_stamp_h         : " U32H_FMT "\n", ptr_struct->time_stamp_h);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_stamp_l         : " U32H_FMT "\n", ptr_struct->time_stamp_l);

}

int connectib_pps_event_size(void)
{
	 return 28;
}

void connectib_pps_event_dump(const struct connectib_pps_event *ptr_struct, FILE* file)
{
	connectib_pps_event_print(ptr_struct, file, 0);
}

void connectib_fpga_qp_error_event_pack(const struct connectib_fpga_qp_error_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->syndrome);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->fpga_qpn);

}

void connectib_fpga_qp_error_event_unpack(struct connectib_fpga_qp_error_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 88;
	ptr_struct->syndrome = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 200;
	ptr_struct->fpga_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_fpga_qp_error_event_print(const struct connectib_fpga_qp_error_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fpga_qp_error_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "syndrome             : %s (" UH_FMT ")\n", (ptr_struct->syndrome == 1 ? ("retry_counter_expired") : ((ptr_struct->syndrome == 2 ? ("RNR_expired") : ("unknown")))), ptr_struct->syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_qpn             : " UH_FMT "\n", ptr_struct->fpga_qpn);

}

int connectib_fpga_qp_error_event_size(void)
{
	 return 28;
}

void connectib_fpga_qp_error_event_dump(const struct connectib_fpga_qp_error_event *ptr_struct, FILE* file)
{
	connectib_fpga_qp_error_event_print(ptr_struct, file, 0);
}

void connectib_fpga_error_event_pack(const struct connectib_fpga_error_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->syndrome);

}

void connectib_fpga_error_event_unpack(struct connectib_fpga_error_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 88;
	ptr_struct->syndrome = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_fpga_error_event_print(const struct connectib_fpga_error_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fpga_error_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "syndrome             : %s (" UH_FMT ")\n", (ptr_struct->syndrome == 1 ? ("corrupted_ddr") : ((ptr_struct->syndrome == 2 ? ("flash_timeout") : ((ptr_struct->syndrome == 3 ? ("internal_link_error") : ((ptr_struct->syndrome == 4 ? ("watchdog_failure") : ((ptr_struct->syndrome == 5 ? ("I2C_failure") : ((ptr_struct->syndrome == 6 ? ("Image_changed") : ("unknown")))))))))))), ptr_struct->syndrome);

}

int connectib_fpga_error_event_size(void)
{
	 return 28;
}

void connectib_fpga_error_event_dump(const struct connectib_fpga_error_event *ptr_struct, FILE* file)
{
	connectib_fpga_error_event_print(ptr_struct, file, 0);
}

void connectib_nic_vport_change_event_pack(const struct connectib_nic_vport_change_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vport_num);

}

void connectib_nic_vport_change_event_unpack(struct connectib_nic_vport_change_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->vport_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_nic_vport_change_event_print(const struct connectib_nic_vport_change_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nic_vport_change_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_num            : " UH_FMT "\n", ptr_struct->vport_num);

}

int connectib_nic_vport_change_event_size(void)
{
	 return 28;
}

void connectib_nic_vport_change_event_dump(const struct connectib_nic_vport_change_event *ptr_struct, FILE* file)
{
	connectib_nic_vport_change_event_print(ptr_struct, file, 0);
}

void connectib_pages_req_event_pack(const struct connectib_pages_req_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->function_id);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_pages);

}

void connectib_pages_req_event_unpack(struct connectib_pages_req_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->function_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->num_pages = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_pages_req_event_print(const struct connectib_pages_req_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pages_req_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "function_id          : " UH_FMT "\n", ptr_struct->function_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_pages            : " U32H_FMT "\n", ptr_struct->num_pages);

}

int connectib_pages_req_event_size(void)
{
	 return 28;
}

void connectib_pages_req_event_dump(const struct connectib_pages_req_event *ptr_struct, FILE* file)
{
	connectib_pages_req_event_print(ptr_struct, file, 0);
}

void connectib_cmd_inter_comp_event_pack(const struct connectib_cmd_inter_comp_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->command_completion_vector);

}

void connectib_cmd_inter_comp_event_unpack(struct connectib_cmd_inter_comp_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->command_completion_vector = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_cmd_inter_comp_event_print(const struct connectib_cmd_inter_comp_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmd_inter_comp_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "command_completion_vector : " U32H_FMT "\n", ptr_struct->command_completion_vector);

}

int connectib_cmd_inter_comp_event_size(void)
{
	 return 28;
}

void connectib_cmd_inter_comp_event_dump(const struct connectib_cmd_inter_comp_event *ptr_struct, FILE* file)
{
	connectib_cmd_inter_comp_event_print(ptr_struct, file, 0);
}

void connectib_stall_vl_event_pack(const struct connectib_stall_vl_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port_num);

}

void connectib_stall_vl_event_unpack(struct connectib_stall_vl_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->vl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 24;
	ptr_struct->port_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_stall_vl_event_print(const struct connectib_stall_vl_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_stall_vl_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl                   : " UH_FMT "\n", ptr_struct->vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_num             : " UH_FMT "\n", ptr_struct->port_num);

}

int connectib_stall_vl_event_size(void)
{
	 return 24;
}

void connectib_stall_vl_event_dump(const struct connectib_stall_vl_event *ptr_struct, FILE* file)
{
	connectib_stall_vl_event_print(ptr_struct, file, 0);
}

void connectib_db_bf_congestion_event_pack(const struct connectib_db_bf_congestion_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->congestion_level);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->event_subtype);

}

void connectib_db_bf_congestion_event_unpack(struct connectib_db_bf_congestion_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->congestion_level = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->event_subtype = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_db_bf_congestion_event_print(const struct connectib_db_bf_congestion_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_db_bf_congestion_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "congestion_level     : " UH_FMT "\n", ptr_struct->congestion_level);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_subtype        : " UH_FMT "\n", ptr_struct->event_subtype);

}

int connectib_db_bf_congestion_event_size(void)
{
	 return 24;
}

void connectib_db_bf_congestion_event_dump(const struct connectib_db_bf_congestion_event *ptr_struct, FILE* file)
{
	connectib_db_bf_congestion_event_print(ptr_struct, file, 0);
}

void connectib_temp_warning_event_pack(const struct connectib_temp_warning_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sensor_warning[i]);
	}

}

void connectib_temp_warning_event_unpack(struct connectib_temp_warning_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	ptr_struct->sensor_warning[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_temp_warning_event_print(const struct connectib_temp_warning_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_temp_warning_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sensor_warning_%03d  : " U32H_FMT "\n", i, ptr_struct->sensor_warning[i]);
	}

}

int connectib_temp_warning_event_size(void)
{
	 return 16;
}

void connectib_temp_warning_event_dump(const struct connectib_temp_warning_event *ptr_struct, FILE* file)
{
	connectib_temp_warning_event_print(ptr_struct, file, 0);
}

void connectib_dcbx_change_event_pack(const struct connectib_dcbx_change_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->event_sub_type);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error);

}

void connectib_dcbx_change_event_unpack(struct connectib_dcbx_change_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->event_sub_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 20;
	ptr_struct->error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_dcbx_change_event_print(const struct connectib_dcbx_change_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dcbx_change_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_sub_type       : " UH_FMT "\n", ptr_struct->event_sub_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error                : " UH_FMT "\n", ptr_struct->error);

}

int connectib_dcbx_change_event_size(void)
{
	 return 28;
}

void connectib_dcbx_change_event_dump(const struct connectib_dcbx_change_event *ptr_struct, FILE* file)
{
	connectib_dcbx_change_event_print(ptr_struct, file, 0);
}

void connectib_port_module_event_pack(const struct connectib_port_module_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->module_status);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->error_type);

}

void connectib_port_module_event_unpack(struct connectib_port_module_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->module_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 8;
	ptr_struct->module = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 52;
	ptr_struct->error_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_port_module_event_print(const struct connectib_port_module_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_port_module_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_status        : " UH_FMT "\n", ptr_struct->module_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module               : " UH_FMT "\n", ptr_struct->module);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_type           : " UH_FMT "\n", ptr_struct->error_type);

}

int connectib_port_module_event_size(void)
{
	 return 28;
}

void connectib_port_module_event_dump(const struct connectib_port_module_event *ptr_struct, FILE* file)
{
	connectib_port_module_event_print(ptr_struct, file, 0);
}

void connectib_gpio_event_pack(const struct connectib_gpio_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gpio_event_hi);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gpio_event_lo);

}

void connectib_gpio_event_unpack(struct connectib_gpio_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 96;
	ptr_struct->gpio_event_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->gpio_event_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_gpio_event_print(const struct connectib_gpio_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_gpio_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpio_event_hi        : " U32H_FMT "\n", ptr_struct->gpio_event_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpio_event_lo        : " U32H_FMT "\n", ptr_struct->gpio_event_lo);

}

int connectib_gpio_event_size(void)
{
	 return 28;
}

void connectib_gpio_event_dump(const struct connectib_gpio_event *ptr_struct, FILE* file)
{
	connectib_gpio_event_print(ptr_struct, file, 0);
}

void connectib_port_state_change_event_pack(const struct connectib_port_state_change_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_num);

}

void connectib_port_state_change_event_unpack(struct connectib_port_state_change_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 64;
	ptr_struct->port_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_port_state_change_event_print(const struct connectib_port_state_change_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_port_state_change_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_num             : " UH_FMT "\n", ptr_struct->port_num);

}

int connectib_port_state_change_event_size(void)
{
	 return 28;
}

void connectib_port_state_change_event_dump(const struct connectib_port_state_change_event *ptr_struct, FILE* file)
{
	connectib_port_state_change_event_print(ptr_struct, file, 0);
}

void connectib_dropped_packet_logged_pack(const struct connectib_dropped_packet_logged *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_dropped_packet_logged_unpack(struct connectib_dropped_packet_logged *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_dropped_packet_logged_print(const struct connectib_dropped_packet_logged *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dropped_packet_logged ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_dropped_packet_logged_size(void)
{
	 return 28;
}

void connectib_dropped_packet_logged_dump(const struct connectib_dropped_packet_logged *ptr_struct, FILE* file)
{
	connectib_dropped_packet_logged_print(ptr_struct, file, 0);
}

void connectib_cq_error_pack(const struct connectib_cq_error *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->cqn);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->syndrome);

}

void connectib_cq_error_unpack(struct connectib_cq_error *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->cqn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 88;
	ptr_struct->syndrome = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_cq_error_print(const struct connectib_cq_error *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cq_error ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqn                  : " UH_FMT "\n", ptr_struct->cqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "syndrome             : %s (" UH_FMT ")\n", (ptr_struct->syndrome == 1 ? ("CQ_overrun") : ((ptr_struct->syndrome == 2 ? ("CQ_access_violation_error") : ("unknown")))), ptr_struct->syndrome);

}

int connectib_cq_error_size(void)
{
	 return 28;
}

void connectib_cq_error_dump(const struct connectib_cq_error *ptr_struct, FILE* file)
{
	connectib_cq_error_print(ptr_struct, file, 0);
}

void connectib_rdma_page_fault_event_pack(const struct connectib_rdma_page_fault_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->bytes_committed);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->r_key);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->packet_len);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rdma_op_len);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rdma_va_hi);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rdma_va_lo);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 199;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->requestor);

	offset = 198;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->write_read_);

	offset = 197;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rdma);

}

void connectib_rdma_page_fault_event_unpack(struct connectib_rdma_page_fault_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->bytes_committed = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->r_key = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 80;
	ptr_struct->packet_len = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->rdma_op_len = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->rdma_va_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->rdma_va_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 200;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 199;
	ptr_struct->requestor = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 198;
	ptr_struct->write_read_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 197;
	ptr_struct->rdma = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_rdma_page_fault_event_print(const struct connectib_rdma_page_fault_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rdma_page_fault_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bytes_committed      : " U32H_FMT "\n", ptr_struct->bytes_committed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "r_key                : " U32H_FMT "\n", ptr_struct->r_key);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_len           : " UH_FMT "\n", ptr_struct->packet_len);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdma_op_len          : " U32H_FMT "\n", ptr_struct->rdma_op_len);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdma_va_hi           : " U32H_FMT "\n", ptr_struct->rdma_va_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdma_va_lo           : " U32H_FMT "\n", ptr_struct->rdma_va_lo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "requestor            : " UH_FMT "\n", ptr_struct->requestor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_read_          : " UH_FMT "\n", ptr_struct->write_read_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdma                 : " UH_FMT "\n", ptr_struct->rdma);

}

int connectib_rdma_page_fault_event_size(void)
{
	 return 28;
}

void connectib_rdma_page_fault_event_dump(const struct connectib_rdma_page_fault_event *ptr_struct, FILE* file)
{
	connectib_rdma_page_fault_event_print(ptr_struct, file, 0);
}

void connectib_wqe_associated_page_fault_event_pack(const struct connectib_wqe_associated_page_fault_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->bytes_committed);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->wqe_index);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->len);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 199;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->requestor);

	offset = 198;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->write_read);

	offset = 197;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rdma);

}

void connectib_wqe_associated_page_fault_event_unpack(struct connectib_wqe_associated_page_fault_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->bytes_committed = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 48;
	ptr_struct->wqe_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 80;
	ptr_struct->len = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 200;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 199;
	ptr_struct->requestor = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 198;
	ptr_struct->write_read = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 197;
	ptr_struct->rdma = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_wqe_associated_page_fault_event_print(const struct connectib_wqe_associated_page_fault_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_associated_page_fault_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bytes_committed      : " U32H_FMT "\n", ptr_struct->bytes_committed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_index            : " UH_FMT "\n", ptr_struct->wqe_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "len                  : " UH_FMT "\n", ptr_struct->len);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "requestor            : " UH_FMT "\n", ptr_struct->requestor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_read           : " UH_FMT "\n", ptr_struct->write_read);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdma                 : " UH_FMT "\n", ptr_struct->rdma);

}

int connectib_wqe_associated_page_fault_event_size(void)
{
	 return 28;
}

void connectib_wqe_associated_page_fault_event_dump(const struct connectib_wqe_associated_page_fault_event *ptr_struct, FILE* file)
{
	connectib_wqe_associated_page_fault_event_print(ptr_struct, file, 0);
}

void connectib_qp_events_pack(const struct connectib_qp_events *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->type);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn_rqn_sqn);

}

void connectib_qp_events_unpack(struct connectib_qp_events *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 160;
	ptr_struct->type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 200;
	ptr_struct->qpn_rqn_sqn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_qp_events_print(const struct connectib_qp_events *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_qp_events ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : %s (" UH_FMT ")\n", (ptr_struct->type == 0 ? ("QP") : ((ptr_struct->type == 1 ? ("RQ") : ((ptr_struct->type == 2 ? ("SQ") : ("unknown")))))), ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn_rqn_sqn          : " UH_FMT "\n", ptr_struct->qpn_rqn_sqn);

}

int connectib_qp_events_size(void)
{
	 return 28;
}

void connectib_qp_events_dump(const struct connectib_qp_events *ptr_struct, FILE* file)
{
	connectib_qp_events_print(ptr_struct, file, 0);
}

void connectib_dct_events_pack(const struct connectib_dct_events *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->dctn);

}

void connectib_dct_events_unpack(struct connectib_dct_events *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	ptr_struct->dctn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_dct_events_print(const struct connectib_dct_events *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dct_events ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dctn                 : " UH_FMT "\n", ptr_struct->dctn);

}

int connectib_dct_events_size(void)
{
	 return 28;
}

void connectib_dct_events_dump(const struct connectib_dct_events *ptr_struct, FILE* file)
{
	connectib_dct_events_print(ptr_struct, file, 0);
}

void connectib_comp_event_pack(const struct connectib_comp_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->cqn);

}

void connectib_comp_event_unpack(struct connectib_comp_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 200;
	ptr_struct->cqn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_comp_event_print(const struct connectib_comp_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_comp_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqn                  : " UH_FMT "\n", ptr_struct->cqn);

}

int connectib_comp_event_size(void)
{
	 return 28;
}

void connectib_comp_event_dump(const struct connectib_comp_event *ptr_struct, FILE* file)
{
	connectib_comp_event_print(ptr_struct, file, 0);
}

void connectib_sw_eqe_flat_data_pack(const struct connectib_sw_eqe_flat_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 7; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 224, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

}

void connectib_sw_eqe_flat_data_unpack(struct connectib_sw_eqe_flat_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 7; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 224, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_sw_eqe_flat_data_print(const struct connectib_sw_eqe_flat_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_eqe_flat_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 7; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

}

int connectib_sw_eqe_flat_data_size(void)
{
	 return 28;
}

void connectib_sw_eqe_flat_data_dump(const struct connectib_sw_eqe_flat_data *ptr_struct, FILE* file)
{
	connectib_sw_eqe_flat_data_print(ptr_struct, file, 0);
}

void connectib_generic_flash_reg_access_data_pack(const struct connectib_generic_flash_reg_access_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reg_access_reg_tlv_data_pack(&(ptr_struct->reg_data), ptr_buff + offset/8);

	for (i = 0; i < 56; i++) {
	offset = adb2c_calc_array_field_address(352, 32, i, 2144, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->more_data[i]);
	}

}

void connectib_generic_flash_reg_access_data_unpack(struct connectib_generic_flash_reg_access_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reg_access_reg_tlv_data_unpack(&(ptr_struct->reg_data), ptr_buff + offset/8);

	for (i = 0; i < 56; i++) {
	offset = adb2c_calc_array_field_address(352, 32, i, 2144, 1);
	ptr_struct->more_data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_generic_flash_reg_access_data_print(const struct connectib_generic_flash_reg_access_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_generic_flash_reg_access_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg_data:\n");
	connectib_reg_access_reg_tlv_data_print(&(ptr_struct->reg_data), file, indent_level + 1);

	for (i = 0; i < 56; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "more_data_%03d       : " U32H_FMT "\n", i, ptr_struct->more_data[i]);
	}

}

int connectib_generic_flash_reg_access_data_size(void)
{
	 return 268;
}

void connectib_generic_flash_reg_access_data_dump(const struct connectib_generic_flash_reg_access_data *ptr_struct, FILE* file)
{
	connectib_generic_flash_reg_access_data_print(ptr_struct, file, 0);
}

void connectib_access_register_data_layout_pack(const union connectib_access_register_data_layout *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_pmaos_reg_pack(&(ptr_struct->access_register_pmaos), ptr_buff);
}

void connectib_access_register_data_layout_unpack(union connectib_access_register_data_layout *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_pmaos_reg_unpack(&(ptr_struct->access_register_pmaos), ptr_buff);
}

void connectib_access_register_data_layout_print(const union connectib_access_register_data_layout *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_access_register_data_layout ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mcam_reg:\n");
	connectib_mcam_reg_print(&(ptr_struct->mcam_reg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcam_reg:\n");
	connectib_pcam_reg_print(&(ptr_struct->pcam_reg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qetcr:\n");
	connectib_qetcr_reg_print(&(ptr_struct->qetcr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpdp:\n");
	connectib_qpdp_reg_print(&(ptr_struct->qpdp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qtct:\n");
	connectib_qtct_reg_print(&(ptr_struct->qtct), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qhll:\n");
	connectib_qhll_print(&(ptr_struct->qhll), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlcr:\n");
	connectib_mlcr_print(&(ptr_struct->mlcr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcbx:\n");
	connectib_dcbx_param_reg_print(&(ptr_struct->dcbx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcbx_app:\n");
	connectib_dcbx_app_reg_print(&(ptr_struct->dcbx_app), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "register_set_node:\n");
	connectib_register_set_node_print(&(ptr_struct->register_set_node), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "perf_counters_cfg:\n");
	connectib_access_register_unit_perf_counters_cfg_print(&(ptr_struct->perf_counters_cfg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "perf_counters_data:\n");
	connectib_access_register_unit_perf_counters_data_print(&(ptr_struct->perf_counters_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "perf_counters_global_ctrl:\n");
	connectib_register_unit_perf_counters_global_ctrl_print(&(ptr_struct->perf_counters_global_ctrl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_capabilities_reg:\n");
	connectib_access_register_unit_port_capabilities_print(&(ptr_struct->port_capabilities_reg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_flow_control_cfg_reg:\n");
	connectib_pfcc_reg_print(&(ptr_struct->port_flow_control_cfg_reg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "loopback_control_reg:\n");
	connectib_register_loopback_control_print(&(ptr_struct->loopback_control_reg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "power_settings_reg:\n");
	connectib_register_power_settings_print(&(ptr_struct->power_settings_reg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fp_sl_map_reg:\n");
	connectib_register_fp_sl_map_print(&(ptr_struct->fp_sl_map_reg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "register_host_endianness:\n");
	connectib_register_host_endianness_print(&(ptr_struct->register_host_endianness), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "register_pplr:\n");
	connectib_register_pplr_print(&(ptr_struct->register_pplr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "misc_counters:\n");
	connectib_register_misc_counters_print(&(ptr_struct->misc_counters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_threshold:\n");
	connectib_db_threshold_register_layout_print(&(ptr_struct->db_threshold), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_mfpa:\n");
	connectib_register_mfpa_print(&(ptr_struct->flash_mfpa), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_mfba:\n");
	connectib_register_mfba_print(&(ptr_struct->flash_mfba), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_mfbe:\n");
	connectib_register_mfbe_print(&(ptr_struct->flash_mfbe), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_paos:\n");
	connectib_paos_reg_print(&(ptr_struct->access_register_paos), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pmtu:\n");
	connectib_pmtu_reg_print(&(ptr_struct->access_register_pmtu), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pmlp:\n");
	connectib_pmlp_reg_print(&(ptr_struct->access_register_pmlp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ptys:\n");
	connectib_ptys_reg_print(&(ptr_struct->access_register_ptys), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ppcnt:\n");
	connectib_ppcnt_reg_print(&(ptr_struct->access_register_ppcnt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pvlc:\n");
	connectib_pvlc_reg_print(&(ptr_struct->access_register_pvlc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pamp:\n");
	connectib_pamp_reg_print(&(ptr_struct->access_register_pamp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ptas:\n");
	connectib_ptas_reg_print(&(ptr_struct->access_register_ptas), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_slrg:\n");
	connectib_slrg_reg_print(&(ptr_struct->access_register_slrg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_slrp:\n");
	connectib_slrp_reg_print(&(ptr_struct->access_register_slrp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_sltp:\n");
	connectib_sltp_reg_print(&(ptr_struct->access_register_sltp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pplr:\n");
	connectib_pplr_reg_print(&(ptr_struct->access_register_pplr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_plpc:\n");
	connectib_plpc_reg_print(&(ptr_struct->access_register_plpc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pplm:\n");
	connectib_pplm_reg_print(&(ptr_struct->access_register_pplm), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ppad:\n");
	connectib_ppad_reg_print(&(ptr_struct->access_register_ppad), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_mcia:\n");
	connectib_mcia_reg_print(&(ptr_struct->access_register_mcia), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_mpein:\n");
	connectib_mpein_reg_print(&(ptr_struct->access_register_mpein), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_mpcnt:\n");
	connectib_mpcnt_reg_print(&(ptr_struct->access_register_mpcnt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pddr:\n");
	connectib_pddr_reg_print(&(ptr_struct->access_register_pddr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_mcion:\n");
	connectib_mcion_reg_print(&(ptr_struct->access_register_mcion), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pptt:\n");
	connectib_pptt_reg_print(&(ptr_struct->access_register_pptt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pprt:\n");
	connectib_pprt_reg_print(&(ptr_struct->access_register_pprt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ppaos:\n");
	connectib_ppaos_reg_print(&(ptr_struct->access_register_ppaos), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pcmr:\n");
	connectib_pcmr_reg_print(&(ptr_struct->access_register_pcmr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pmmp:\n");
	connectib_pmmp_reg_print(&(ptr_struct->access_register_pmmp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pmcr:\n");
	connectib_pmcr_reg_print(&(ptr_struct->access_register_pmcr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_access_reg:\n");
	connectib_fpga_access_reg_print(&(ptr_struct->fpga_access_reg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_cap:\n");
	connectib_fpga_cap_print(&(ptr_struct->fpga_cap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_ctrl:\n");
	connectib_fpga_ctrl_print(&(ptr_struct->fpga_ctrl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_mtpps:\n");
	connectib_mtpps_reg_print(&(ptr_struct->access_register_mtpps), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_mtppse:\n");
	connectib_mtppse_reg_print(&(ptr_struct->access_register_mtppse), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pmaos:\n");
	connectib_pmaos_reg_print(&(ptr_struct->access_register_pmaos), file, indent_level + 1);

}

int connectib_access_register_data_layout_size(void)
{
	 return 264;
}

void connectib_access_register_data_layout_dump(const union connectib_access_register_data_layout *ptr_struct, FILE* file)
{
	connectib_access_register_data_layout_print(ptr_struct, file, 0);
}

void connectib_scheduling_context_pack(const struct connectib_scheduling_context *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->element_type);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->element_attributes);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->parent_element_id);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->bw_share);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_average_bw);

}

void connectib_scheduling_context_unpack(struct connectib_scheduling_context *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->element_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->element_attributes = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->parent_element_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->bw_share = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->max_average_bw = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_scheduling_context_print(const struct connectib_scheduling_context *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_scheduling_context ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "element_type         : %s (" UH_FMT ")\n", (ptr_struct->element_type == 0 ? ("TSAR") : ((ptr_struct->element_type == 1 ? ("VPORT") : ((ptr_struct->element_type == 2 ? ("VPORT_TC") : ((ptr_struct->element_type == 3 ? ("PARA_VPORT_TC") : ("unknown")))))))), ptr_struct->element_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "element_attributes   : " U32H_FMT "\n", ptr_struct->element_attributes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parent_element_id    : " U32H_FMT "\n", ptr_struct->parent_element_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bw_share             : " U32H_FMT "\n", ptr_struct->bw_share);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_average_bw       : " U32H_FMT "\n", ptr_struct->max_average_bw);

}

int connectib_scheduling_context_size(void)
{
	 return 64;
}

void connectib_scheduling_context_dump(const struct connectib_scheduling_context *ptr_struct, FILE* file)
{
	connectib_scheduling_context_print(ptr_struct, file, 0);
}

void connectib_fw_ipoib_meta_pack(const struct connectib_fw_ipoib_meta *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->q_key);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pkey_index);

}

void connectib_fw_ipoib_meta_unpack(struct connectib_fw_ipoib_meta *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->q_key = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 48;
	ptr_struct->pkey_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_fw_ipoib_meta_print(const struct connectib_fw_ipoib_meta *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_ipoib_meta ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "q_key                : " U32H_FMT "\n", ptr_struct->q_key);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey_index           : " UH_FMT "\n", ptr_struct->pkey_index);

}

int connectib_fw_ipoib_meta_size(void)
{
	 return 16;
}

void connectib_fw_ipoib_meta_dump(const struct connectib_fw_ipoib_meta *ptr_struct, FILE* file)
{
	connectib_fw_ipoib_meta_print(ptr_struct, file, 0);
}

void connectib_fw_q_counters_pack(const struct connectib_fw_q_counters *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->duplicate_request);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rnr_nack_retry_err);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->packet_seq_err);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->implied_nack_seq_err);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->local_ack_timeout_err);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->resp_rnr_nak);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->req_rnr_retries_exceeded);

}

void connectib_fw_q_counters_unpack(struct connectib_fw_q_counters *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 32;
	ptr_struct->duplicate_request = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->rnr_nack_retry_err = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->packet_seq_err = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->implied_nack_seq_err = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->local_ack_timeout_err = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->resp_rnr_nak = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->req_rnr_retries_exceeded = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_fw_q_counters_print(const struct connectib_fw_q_counters *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_q_counters ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "duplicate_request    : " U32H_FMT "\n", ptr_struct->duplicate_request);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rnr_nack_retry_err   : " U32H_FMT "\n", ptr_struct->rnr_nack_retry_err);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_seq_err       : " U32H_FMT "\n", ptr_struct->packet_seq_err);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "implied_nack_seq_err : " U32H_FMT "\n", ptr_struct->implied_nack_seq_err);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_ack_timeout_err : " U32H_FMT "\n", ptr_struct->local_ack_timeout_err);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "resp_rnr_nak         : " U32H_FMT "\n", ptr_struct->resp_rnr_nak);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "req_rnr_retries_exceeded : " U32H_FMT "\n", ptr_struct->req_rnr_retries_exceeded);

}

int connectib_fw_q_counters_size(void)
{
	 return 64;
}

void connectib_fw_q_counters_dump(const struct connectib_fw_q_counters *ptr_struct, FILE* file)
{
	connectib_fw_q_counters_print(ptr_struct, file, 0);
}

void connectib_fw_transport_domain_pack(const struct connectib_fw_transport_domain *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sq_list_size);

}

void connectib_fw_transport_domain_unpack(struct connectib_fw_transport_domain *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 32;
	ptr_struct->sq_list_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_fw_transport_domain_print(const struct connectib_fw_transport_domain *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_transport_domain ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq_list_size         : " U32H_FMT "\n", ptr_struct->sq_list_size);

}

int connectib_fw_transport_domain_size(void)
{
	 return 8;
}

void connectib_fw_transport_domain_dump(const struct connectib_fw_transport_domain *ptr_struct, FILE* file)
{
	connectib_fw_transport_domain_print(ptr_struct, file, 0);
}

void connectib_vqos_arb_ctx_pack(const struct connectib_vqos_arb_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->allowed);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->father_id);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->weight);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sum_of_weights);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->token_count);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cweight);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ctoken_count);

	offset = 159;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->i_am_first);

}

void connectib_vqos_arb_ctx_unpack(struct connectib_vqos_arb_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->allowed = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->father_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

	offset = 16;
	ptr_struct->weight = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 8;
	ptr_struct->sum_of_weights = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->token_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->cweight = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->ctoken_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 159;
	ptr_struct->i_am_first = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_vqos_arb_ctx_print(const struct connectib_vqos_arb_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_vqos_arb_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allowed              : " UH_FMT "\n", ptr_struct->allowed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "father_id            : " UH_FMT "\n", ptr_struct->father_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "weight               : " UH_FMT "\n", ptr_struct->weight);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sum_of_weights       : " UH_FMT "\n", ptr_struct->sum_of_weights);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "token_count          : " U32H_FMT "\n", ptr_struct->token_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cweight              : " U32H_FMT "\n", ptr_struct->cweight);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctoken_count         : " U32H_FMT "\n", ptr_struct->ctoken_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i_am_first           : " UH_FMT "\n", ptr_struct->i_am_first);

}

int connectib_vqos_arb_ctx_size(void)
{
	 return 32;
}

void connectib_vqos_arb_ctx_dump(const struct connectib_vqos_arb_ctx *ptr_struct, FILE* file)
{
	connectib_vqos_arb_ctx_print(ptr_struct, file, 0);
}

void connectib_dc_control_packet_desc_pack(const struct connectib_dc_control_packet_desc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->dlid);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->slid);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->source_qp);

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->dest_qp);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->psn);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pkey);

	offset = 140;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl);

	offset = 139;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port_id);

	offset = 138;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->has_grh);

	offset = 137;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_loopback);

	offset = 176;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->dct_gvmi);

	offset = 216;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->tclass);

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hop_limit);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->flow_label);

	offset = 256;
	connectib_ib_gid_pack(&(ptr_struct->dgid), ptr_buff + offset/8);

	offset = 384;
	connectib_ib_gid_pack(&(ptr_struct->sgid), ptr_buff + offset/8);

}

void connectib_dc_control_packet_desc_unpack(struct connectib_dc_control_packet_desc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->dlid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->slid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->source_qp = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 39;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->dest_qp = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 104;
	ptr_struct->psn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 144;
	ptr_struct->pkey = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 140;
	ptr_struct->sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 139;
	ptr_struct->port_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 138;
	ptr_struct->has_grh = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 137;
	ptr_struct->force_loopback = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 176;
	ptr_struct->dct_gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 216;
	ptr_struct->tclass = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 208;
	ptr_struct->hop_limit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 192;
	ptr_struct->flow_label = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 256;
	connectib_ib_gid_unpack(&(ptr_struct->dgid), ptr_buff + offset/8);

	offset = 384;
	connectib_ib_gid_unpack(&(ptr_struct->sgid), ptr_buff + offset/8);

}

void connectib_dc_control_packet_desc_print(const struct connectib_dc_control_packet_desc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dc_control_packet_desc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dlid                 : " UH_FMT "\n", ptr_struct->dlid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slid                 : " UH_FMT "\n", ptr_struct->slid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "source_qp            : " UH_FMT "\n", ptr_struct->source_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dest_qp              : " UH_FMT "\n", ptr_struct->dest_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "psn                  : " UH_FMT "\n", ptr_struct->psn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey                 : " UH_FMT "\n", ptr_struct->pkey);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl                   : " UH_FMT "\n", ptr_struct->sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_id              : " UH_FMT "\n", ptr_struct->port_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "has_grh              : " UH_FMT "\n", ptr_struct->has_grh);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_loopback       : " UH_FMT "\n", ptr_struct->force_loopback);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dct_gvmi             : " UH_FMT "\n", ptr_struct->dct_gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tclass               : " UH_FMT "\n", ptr_struct->tclass);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hop_limit            : " UH_FMT "\n", ptr_struct->hop_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_label           : " UH_FMT "\n", ptr_struct->flow_label);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dgid:\n");
	connectib_ib_gid_print(&(ptr_struct->dgid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sgid:\n");
	connectib_ib_gid_print(&(ptr_struct->sgid), file, indent_level + 1);

}

int connectib_dc_control_packet_desc_size(void)
{
	 return 64;
}

void connectib_dc_control_packet_desc_dump(const struct connectib_dc_control_packet_desc *ptr_struct, FILE* file)
{
	connectib_dc_control_packet_desc_print(ptr_struct, file, 0);
}

void connectib_dmfs_resources_pack(const union connectib_dmfs_resources *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_dmfs_encap_meta_pack(&(ptr_struct->encap_meta), ptr_buff);
}

void connectib_dmfs_resources_unpack(union connectib_dmfs_resources *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_dmfs_encap_meta_unpack(&(ptr_struct->encap_meta), ptr_buff);
}

void connectib_dmfs_resources_print(const union connectib_dmfs_resources *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dmfs_resources ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fg_meta:\n");
	connectib_dmfs_fg_meta_print(&(ptr_struct->fg_meta), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_meta:\n");
	connectib_dmfs_ft_meta_print(&(ptr_struct->ft_meta), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fte_meta:\n");
	connectib_dmfs_fte_meta_print(&(ptr_struct->fte_meta), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l2_table_entry_fw_ctx:\n");
	connectib_l2_table_entry_fw_ctx_print(&(ptr_struct->l2_table_entry_fw_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_counter_fw_ctx:\n");
	connectib_flow_counter_fw_ctx_print(&(ptr_struct->flow_counter_fw_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encap_meta:\n");
	connectib_dmfs_encap_meta_print(&(ptr_struct->encap_meta), file, indent_level + 1);

}

int connectib_dmfs_resources_size(void)
{
	 return 64;
}

void connectib_dmfs_resources_dump(const union connectib_dmfs_resources *ptr_struct, FILE* file)
{
	connectib_dmfs_resources_print(ptr_struct, file, 0);
}

void connectib_list_element_pack(const struct connectib_list_element *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 25, (u_int32_t)ptr_struct->next);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->list_ix);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->in_list);

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 25, (u_int32_t)ptr_struct->prev);

}

void connectib_list_element_unpack(struct connectib_list_element *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	ptr_struct->next = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 25);

	offset = 1;
	ptr_struct->list_ix = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 0;
	ptr_struct->in_list = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 39;
	ptr_struct->prev = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 25);

}

void connectib_list_element_print(const struct connectib_list_element *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_list_element ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next                 : " UH_FMT "\n", ptr_struct->next);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "list_ix              : " UH_FMT "\n", ptr_struct->list_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in_list              : " UH_FMT "\n", ptr_struct->in_list);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prev                 : " UH_FMT "\n", ptr_struct->prev);

}

int connectib_list_element_size(void)
{
	 return 8;
}

void connectib_list_element_dump(const struct connectib_list_element *ptr_struct, FILE* file)
{
	connectib_list_element_print(ptr_struct, file, 0);
}

void connectib_fw_rqt_pack(const struct connectib_fw_rqt *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rqt_max_size);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rqt_actual_size);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->lro_max_ip_payload_size);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->lro_enable_mask);

	offset = 83;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->tunneled_offload_en);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lro_timeout_period_usecs);

	offset = 126;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->rss_qpn_size);

	offset = 102;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->rss_indirection_table_base_qpn);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rqt_hw_data_ix);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rqt_fw_data_ix);

}

void connectib_fw_rqt_unpack(struct connectib_fw_rqt *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 48;
	ptr_struct->rqt_max_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->rqt_actual_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 88;
	ptr_struct->lro_max_ip_payload_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 84;
	ptr_struct->lro_enable_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 83;
	ptr_struct->tunneled_offload_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->lro_timeout_period_usecs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 126;
	ptr_struct->rss_qpn_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 102;
	ptr_struct->rss_indirection_table_base_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 128;
	ptr_struct->rqt_hw_data_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->rqt_fw_data_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_fw_rqt_print(const struct connectib_fw_rqt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_rqt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_max_size         : " UH_FMT "\n", ptr_struct->rqt_max_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_actual_size      : " UH_FMT "\n", ptr_struct->rqt_actual_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_max_ip_payload_size : " UH_FMT "\n", ptr_struct->lro_max_ip_payload_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_enable_mask      : " UH_FMT "\n", ptr_struct->lro_enable_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tunneled_offload_en  : " UH_FMT "\n", ptr_struct->tunneled_offload_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lro_timeout_period_usecs : " UH_FMT "\n", ptr_struct->lro_timeout_period_usecs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rss_qpn_size         : " UH_FMT "\n", ptr_struct->rss_qpn_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rss_indirection_table_base_qpn : " UH_FMT "\n", ptr_struct->rss_indirection_table_base_qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_hw_data_ix       : " U32H_FMT "\n", ptr_struct->rqt_hw_data_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqt_fw_data_ix       : " U32H_FMT "\n", ptr_struct->rqt_fw_data_ix);

}

int connectib_fw_rqt_size(void)
{
	 return 32;
}

void connectib_fw_rqt_dump(const struct connectib_fw_rqt *ptr_struct, FILE* file)
{
	connectib_fw_rqt_print(ptr_struct, file, 0);
}

void connectib_fw_cqc_pack(const struct connectib_fw_cqc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cq_status);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->trap_reason);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->in_cq_overrun_list);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->in_eq_overrun_list);

}

void connectib_fw_cqc_unpack(struct connectib_fw_cqc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 56;
	ptr_struct->cq_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 52;
	ptr_struct->trap_reason = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 33;
	ptr_struct->in_cq_overrun_list = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->in_eq_overrun_list = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_fw_cqc_print(const struct connectib_fw_cqc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_cqc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_status            : " UH_FMT "\n", ptr_struct->cq_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trap_reason          : " UH_FMT "\n", ptr_struct->trap_reason);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in_cq_overrun_list   : " UH_FMT "\n", ptr_struct->in_cq_overrun_list);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in_eq_overrun_list   : " UH_FMT "\n", ptr_struct->in_eq_overrun_list);

}

int connectib_fw_cqc_size(void)
{
	 return 16;
}

void connectib_fw_cqc_dump(const struct connectib_fw_cqc *ptr_struct, FILE* file)
{
	connectib_fw_cqc_print(ptr_struct, file, 0);
}

void connectib_fw_psv_pack(const struct connectib_fw_psv *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->pd);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fw_marked_sigerr);

}

void connectib_fw_psv_unpack(struct connectib_fw_psv *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->pd = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 7;
	ptr_struct->fw_marked_sigerr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_fw_psv_print(const struct connectib_fw_psv *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_psv ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pd                   : " UH_FMT "\n", ptr_struct->pd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_marked_sigerr     : " UH_FMT "\n", ptr_struct->fw_marked_sigerr);

}

int connectib_fw_psv_size(void)
{
	 return 4;
}

void connectib_fw_psv_dump(const struct connectib_fw_psv *ptr_struct, FILE* file)
{
	connectib_fw_psv_print(ptr_struct, file, 0);
}

void connectib_fw_mkc_pack(const struct connectib_fw_mkc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 31, (u_int32_t)ptr_struct->state);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sync_umr);

	offset = 64;
	connectib_mkey_dump_pack(&(ptr_struct->mkey_dump), ptr_buff + offset/8);

	offset = 127;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->next_num_of_sigerr);

	offset = 126;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pg_access);

}

void connectib_fw_mkc_unpack(struct connectib_fw_mkc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 33;
	ptr_struct->state = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 31);

	offset = 32;
	ptr_struct->sync_umr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	connectib_mkey_dump_unpack(&(ptr_struct->mkey_dump), ptr_buff + offset/8);

	offset = 127;
	ptr_struct->next_num_of_sigerr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 126;
	ptr_struct->pg_access = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_fw_mkc_print(const struct connectib_fw_mkc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_mkc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "state                : " UH_FMT "\n", ptr_struct->state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sync_umr             : " UH_FMT "\n", ptr_struct->sync_umr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mkey_dump:\n");
	connectib_mkey_dump_print(&(ptr_struct->mkey_dump), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_num_of_sigerr   : " UH_FMT "\n", ptr_struct->next_num_of_sigerr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pg_access            : " UH_FMT "\n", ptr_struct->pg_access);

}

int connectib_fw_mkc_size(void)
{
	 return 16;
}

void connectib_fw_mkc_dump(const struct connectib_fw_mkc *ptr_struct, FILE* file)
{
	connectib_fw_mkc_print(ptr_struct, file, 0);
}

void connectib_fw_cre_pack(const union connectib_fw_cre *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_fw_tis_pack(&(ptr_struct->fw_tis), ptr_buff);
}

void connectib_fw_cre_unpack(union connectib_fw_cre *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_fw_tis_unpack(&(ptr_struct->fw_tis), ptr_buff);
}

void connectib_fw_cre_print(const union connectib_fw_cre *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_cre ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_srqc:\n");
	connectib_fw_srqc_print(&(ptr_struct->fw_srqc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_qpc:\n");
	connectib_fw_qpc_print(&(ptr_struct->fw_qpc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_tir:\n");
	connectib_fw_tir_print(&(ptr_struct->fw_tir), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_tis:\n");
	connectib_fw_tis_print(&(ptr_struct->fw_tis), file, indent_level + 1);

}

int connectib_fw_cre_size(void)
{
	 return 64;
}

void connectib_fw_cre_dump(const union connectib_fw_cre *ptr_struct, FILE* file)
{
	connectib_fw_cre_print(ptr_struct, file, 0);
}

void connectib_fw_basic_res_ctx_pack(const struct connectib_fw_basic_res_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

}

void connectib_fw_basic_res_ctx_unpack(struct connectib_fw_basic_res_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

}

void connectib_fw_basic_res_ctx_print(const struct connectib_fw_basic_res_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_basic_res_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

}

int connectib_fw_basic_res_ctx_size(void)
{
	 return 4;
}

void connectib_fw_basic_res_ctx_dump(const struct connectib_fw_basic_res_ctx *ptr_struct, FILE* file)
{
	connectib_fw_basic_res_ctx_print(ptr_struct, file, 0);
}

void connectib_fw_alt_path_pack(const struct connectib_fw_alt_path *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_sw_ads_pack(&(ptr_struct->alt), ptr_buff + offset/8);

}

void connectib_fw_alt_path_unpack(struct connectib_fw_alt_path *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_sw_ads_unpack(&(ptr_struct->alt), ptr_buff + offset/8);

}

void connectib_fw_alt_path_print(const struct connectib_fw_alt_path *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_alt_path ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "alt:\n");
	connectib_sw_ads_print(&(ptr_struct->alt), file, indent_level + 1);

}

int connectib_fw_alt_path_size(void)
{
	 return 48;
}

void connectib_fw_alt_path_dump(const struct connectib_fw_alt_path *ptr_struct, FILE* file)
{
	connectib_fw_alt_path_print(ptr_struct, file, 0);
}

void connectib_fw_eqc_pack(const struct connectib_fw_eqc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_pack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->eq_status);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->trap_reason);

	offset = 43;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_page_size);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->needs_eq_probe);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->in_eq_overrun_list);

}

void connectib_fw_eqc_unpack(struct connectib_fw_eqc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_ref_count_unpack(&(ptr_struct->fw_ref_count), ptr_buff + offset/8);

	offset = 56;
	ptr_struct->eq_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 52;
	ptr_struct->trap_reason = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 43;
	ptr_struct->log_page_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 33;
	ptr_struct->needs_eq_probe = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->in_eq_overrun_list = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_fw_eqc_print(const struct connectib_fw_eqc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_eqc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ref_count:\n");
	connectib_fw_ref_count_print(&(ptr_struct->fw_ref_count), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eq_status            : " UH_FMT "\n", ptr_struct->eq_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trap_reason          : " UH_FMT "\n", ptr_struct->trap_reason);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_page_size        : " UH_FMT "\n", ptr_struct->log_page_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "needs_eq_probe       : " UH_FMT "\n", ptr_struct->needs_eq_probe);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in_eq_overrun_list   : " UH_FMT "\n", ptr_struct->in_eq_overrun_list);

}

int connectib_fw_eqc_size(void)
{
	 return 16;
}

void connectib_fw_eqc_dump(const struct connectib_fw_eqc *ptr_struct, FILE* file)
{
	connectib_fw_eqc_print(ptr_struct, file, 0);
}

void connectib_hw_qp_counter_pack(const struct connectib_hw_qp_counter *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pkt_nobuffer_drop);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pkt_good_unicast);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pkt_good_multicast);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pkt_dropped);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->msg_write_req);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->msg_rnr_nack);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->msg_read);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->msg_dct_connect);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->msg_atomic);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fw_reserved);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->bytes_rdma_frame);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->bytes_good_unicast);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->bytes_good_multicast);

}

void connectib_hw_qp_counter_unpack(struct connectib_hw_qp_counter *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->pkt_nobuffer_drop = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->pkt_good_unicast = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->pkt_good_multicast = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->pkt_dropped = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->msg_write_req = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->msg_rnr_nack = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->msg_read = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->msg_dct_connect = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->msg_atomic = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->fw_reserved = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->bytes_rdma_frame = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 384;
	ptr_struct->bytes_good_unicast = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 448;
	ptr_struct->bytes_good_multicast = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_qp_counter_print(const struct connectib_hw_qp_counter *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_qp_counter ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkt_nobuffer_drop    : " U32H_FMT "\n", ptr_struct->pkt_nobuffer_drop);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkt_good_unicast     : " U32H_FMT "\n", ptr_struct->pkt_good_unicast);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkt_good_multicast   : " U32H_FMT "\n", ptr_struct->pkt_good_multicast);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkt_dropped          : " U32H_FMT "\n", ptr_struct->pkt_dropped);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msg_write_req        : " U32H_FMT "\n", ptr_struct->msg_write_req);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msg_rnr_nack         : " U32H_FMT "\n", ptr_struct->msg_rnr_nack);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msg_read             : " U32H_FMT "\n", ptr_struct->msg_read);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msg_dct_connect      : " U32H_FMT "\n", ptr_struct->msg_dct_connect);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msg_atomic           : " U32H_FMT "\n", ptr_struct->msg_atomic);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_reserved          : " U32H_FMT "\n", ptr_struct->fw_reserved);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bytes_rdma_frame     : " U64H_FMT "\n", ptr_struct->bytes_rdma_frame);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bytes_good_unicast   : " U64H_FMT "\n", ptr_struct->bytes_good_unicast);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bytes_good_multicast : " U64H_FMT "\n", ptr_struct->bytes_good_multicast);

}

int connectib_hw_qp_counter_size(void)
{
	 return 64;
}

void connectib_hw_qp_counter_dump(const struct connectib_hw_qp_counter *ptr_struct, FILE* file)
{
	connectib_hw_qp_counter_print(ptr_struct, file, 0);
}

void connectib_hw_msix_pack(const struct connectib_hw_msix *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_hw_msix_context_pack(&(ptr_struct->msix_context), ptr_buff + offset/8);

	offset = 128;
	connectib_hw_msix_ext_context_pack(&(ptr_struct->msix_ext_context), ptr_buff + offset/8);

}

void connectib_hw_msix_unpack(struct connectib_hw_msix *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_hw_msix_context_unpack(&(ptr_struct->msix_context), ptr_buff + offset/8);

	offset = 128;
	connectib_hw_msix_ext_context_unpack(&(ptr_struct->msix_ext_context), ptr_buff + offset/8);

}

void connectib_hw_msix_print(const struct connectib_hw_msix *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_msix ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msix_context:\n");
	connectib_hw_msix_context_print(&(ptr_struct->msix_context), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msix_ext_context:\n");
	connectib_hw_msix_ext_context_print(&(ptr_struct->msix_ext_context), file, indent_level + 1);

}

int connectib_hw_msix_size(void)
{
	 return 32;
}

void connectib_hw_msix_dump(const struct connectib_hw_msix *ptr_struct, FILE* file)
{
	connectib_hw_msix_print(ptr_struct, file, 0);
}

void connectib_hw_rdb_pack(const struct connectib_hw_rdb *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->opcode);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->psn);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->r_key);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->address_63_32);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->address_31_0);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->byte_count);

	offset = 184;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->can_use_reserved_lkey);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->pd);

	offset = 213;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->protection_ffser);

	offset = 255;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->allow_excessiveness);

	offset = 256;
	connectib_hw_rdb_atomic_data_pack(&(ptr_struct->atomic_data), ptr_buff + offset/8);

}

void connectib_hw_rdb_unpack(struct connectib_hw_rdb *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->psn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 32;
	ptr_struct->r_key = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->address_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->address_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->byte_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 184;
	ptr_struct->can_use_reserved_lkey = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 160;
	ptr_struct->pd = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 213;
	ptr_struct->protection_ffser = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

	offset = 255;
	ptr_struct->allow_excessiveness = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 256;
	connectib_hw_rdb_atomic_data_unpack(&(ptr_struct->atomic_data), ptr_buff + offset/8);

}

void connectib_hw_rdb_print(const struct connectib_hw_rdb *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_rdb ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " UH_FMT "\n", ptr_struct->opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "psn                  : " UH_FMT "\n", ptr_struct->psn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "r_key                : " U32H_FMT "\n", ptr_struct->r_key);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address_63_32        : " U32H_FMT "\n", ptr_struct->address_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address_31_0         : " U32H_FMT "\n", ptr_struct->address_31_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_count           : " U32H_FMT "\n", ptr_struct->byte_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "can_use_reserved_lkey : " UH_FMT "\n", ptr_struct->can_use_reserved_lkey);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pd                   : " UH_FMT "\n", ptr_struct->pd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "protection_ffser     : " UH_FMT "\n", ptr_struct->protection_ffser);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allow_excessiveness  : " UH_FMT "\n", ptr_struct->allow_excessiveness);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "atomic_data:\n");
	connectib_hw_rdb_atomic_data_print(&(ptr_struct->atomic_data), file, indent_level + 1);

}

int connectib_hw_rdb_size(void)
{
	 return 64;
}

void connectib_hw_rdb_dump(const struct connectib_hw_rdb *ptr_struct, FILE* file)
{
	connectib_hw_rdb_print(ptr_struct, file, 0);
}

void connectib_icm_mtt_pack(const union connectib_icm_mtt *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_mtt2_pack(&(ptr_struct->as_dw), ptr_buff);
}

void connectib_icm_mtt_unpack(union connectib_icm_mtt *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_mtt2_unpack(&(ptr_struct->as_dw), ptr_buff);
}

void connectib_icm_mtt_print(const union connectib_icm_mtt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icm_mtt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw:\n");
	connectib_hw_mtt_print(&(ptr_struct->hw), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "as_dw:\n");
	connectib_hw_mtt2_print(&(ptr_struct->as_dw), file, indent_level + 1);

}

int connectib_icm_mtt_size(void)
{
	 return 8;
}

void connectib_icm_mtt_dump(const union connectib_icm_mtt *ptr_struct, FILE* file)
{
	connectib_icm_mtt_print(ptr_struct, file, 0);
}

void connectib_steering_entry_icmc_pack(const union connectib_steering_entry_icmc *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_steering_qp_list_pack(&(ptr_struct->steering_qp_list), ptr_buff);
}

void connectib_steering_entry_icmc_unpack(union connectib_steering_entry_icmc *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_steering_qp_list_unpack(&(ptr_struct->steering_qp_list), ptr_buff);
}

void connectib_steering_entry_icmc_print(const union connectib_steering_entry_icmc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_steering_entry_icmc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_ste:\n");
	connectib_hw_ste_print(&(ptr_struct->hw_ste), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_qp_list:\n");
	connectib_steering_qp_list_print(&(ptr_struct->steering_qp_list), file, indent_level + 1);

}

int connectib_steering_entry_icmc_size(void)
{
	 return 64;
}

void connectib_steering_entry_icmc_dump(const union connectib_steering_entry_icmc *ptr_struct, FILE* file)
{
	connectib_steering_entry_icmc_print(ptr_struct, file, 0);
}

void connectib_hw_pkey_pack(const struct connectib_hw_pkey *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pkey);

}

void connectib_hw_pkey_unpack(struct connectib_hw_pkey *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->pkey = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_pkey_print(const struct connectib_hw_pkey *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_pkey ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey                 : " UH_FMT "\n", ptr_struct->pkey);

}

int connectib_hw_pkey_size(void)
{
	 return 4;
}

void connectib_hw_pkey_dump(const struct connectib_hw_pkey *ptr_struct, FILE* file)
{
	connectib_hw_pkey_print(ptr_struct, file, 0);
}

void connectib_hw_timer_32_pack(const struct connectib_hw_timer_32 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	connectib_hw_timer_pack(&(ptr_struct->hw_timer), ptr_buff + offset/8);

}

void connectib_hw_timer_32_unpack(struct connectib_hw_timer_32 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	connectib_hw_timer_unpack(&(ptr_struct->hw_timer), ptr_buff + offset/8);

}

void connectib_hw_timer_32_print(const struct connectib_hw_timer_32 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_timer_32 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_timer:\n");
	connectib_hw_timer_print(&(ptr_struct->hw_timer), file, indent_level + 1);

}

int connectib_hw_timer_32_size(void)
{
	 return 4;
}

void connectib_hw_timer_32_dump(const struct connectib_hw_timer_32 *ptr_struct, FILE* file)
{
	connectib_hw_timer_32_print(ptr_struct, file, 0);
}

void connectib_hw_sq_qp_list_pack(const struct connectib_hw_sq_qp_list *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->global_gvmi);

}

void connectib_hw_sq_qp_list_unpack(struct connectib_hw_sq_qp_list *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->qpi = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->global_gvmi = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_sq_qp_list_print(const struct connectib_hw_sq_qp_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_sq_qp_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpi                  : " UH_FMT "\n", ptr_struct->qpi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "global_gvmi          : " UH_FMT "\n", ptr_struct->global_gvmi);

}

int connectib_hw_sq_qp_list_size(void)
{
	 return 4;
}

void connectib_hw_sq_qp_list_dump(const struct connectib_hw_sq_qp_list *ptr_struct, FILE* file)
{
	connectib_hw_sq_qp_list_print(ptr_struct, file, 0);
}

void connectib_hw_pipa_pack(const struct connectib_hw_pipa *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pa_63_32);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->recovery_en);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->always_zero);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 30, (u_int32_t)ptr_struct->pa_31_2);

}

void connectib_hw_pipa_unpack(struct connectib_hw_pipa *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->pa_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 63;
	ptr_struct->recovery_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 62;
	ptr_struct->always_zero = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->pa_31_2 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 30);

}

void connectib_hw_pipa_print(const struct connectib_hw_pipa *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_pipa ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa_63_32             : " U32H_FMT "\n", ptr_struct->pa_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "recovery_en          : " UH_FMT "\n", ptr_struct->recovery_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "always_zero          : " UH_FMT "\n", ptr_struct->always_zero);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa_31_2              : " UH_FMT "\n", ptr_struct->pa_31_2);

}

int connectib_hw_pipa_size(void)
{
	 return 8;
}

void connectib_hw_pipa_dump(const struct connectib_hw_pipa *ptr_struct, FILE* file)
{
	connectib_hw_pipa_print(ptr_struct, file, 0);
}

void connectib_hw_sxdc_pack(const union connectib_hw_sxdc *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_sxdc_requestor_in_sq_pack(&(ptr_struct->requestor_in_sq), ptr_buff);
}

void connectib_hw_sxdc_unpack(union connectib_hw_sxdc *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_sxdc_requestor_in_sq_unpack(&(ptr_struct->requestor_in_sq), ptr_buff);
}

void connectib_hw_sxdc_print(const union connectib_hw_sxdc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_sxdc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uar:\n");
	connectib_sxdc_uar_print(&(ptr_struct->uar), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swqe_producer_index:\n");
	connectib_sxdc_swqe_producer_index_print(&(ptr_struct->swqe_producer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq:\n");
	connectib_sxdc_sq_print(&(ptr_struct->sq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid:\n");
	connectib_sxdc_valid_print(&(ptr_struct->valid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "external_db_en:\n");
	connectib_sxdc_external_db_en_print(&(ptr_struct->external_db_en), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fast_path:\n");
	connectib_sxdc_fast_path_print(&(ptr_struct->fast_path), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "doorbell_int:\n");
	connectib_sxdc_doorbell_int_print(&(ptr_struct->doorbell_int), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "scheduling_interrupt:\n");
	connectib_sxdc_scheduling_interrupt_print(&(ptr_struct->scheduling_interrupt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "responder_in_sq:\n");
	connectib_sxdc_responder_in_sq_print(&(ptr_struct->responder_in_sq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "requestor_in_sq:\n");
	connectib_sxdc_requestor_in_sq_print(&(ptr_struct->requestor_in_sq), file, indent_level + 1);

}

int connectib_hw_sxdc_size(void)
{
	 return 8;
}

void connectib_hw_sxdc_dump(const union connectib_hw_sxdc *ptr_struct, FILE* file)
{
	connectib_hw_sxdc_print(ptr_struct, file, 0);
}

void connectib_hw_klm_entry_pack(const struct connectib_hw_klm_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->length);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->key);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->va);

}

void connectib_hw_klm_entry_unpack(struct connectib_hw_klm_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->length = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->key = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->va = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_klm_entry_print(const struct connectib_hw_klm_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_klm_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "length               : " U32H_FMT "\n", ptr_struct->length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "key                  : " U32H_FMT "\n", ptr_struct->key);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "va                   : " U64H_FMT "\n", ptr_struct->va);

}

int connectib_hw_klm_entry_size(void)
{
	 return 16;
}

void connectib_hw_klm_entry_dump(const struct connectib_hw_klm_entry *ptr_struct, FILE* file)
{
	connectib_hw_klm_entry_print(ptr_struct, file, 0);
}

void connectib_hw_sq_tokens_pack(const struct connectib_hw_sq_tokens *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ware_house_burst_size_23_8);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->responder_burst_size);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ware_house_burst_size_7_0);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->requestor_burst_size);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->gvmi_vl_weight);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->gvmi_vl_token_count);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->gvmi_vl_saturation);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->dwrr_weight);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->dwrr_token_count);

	offset = 228;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl);

	offset = 225;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->vlg);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port);

}

void connectib_hw_sq_tokens_unpack(struct connectib_hw_sq_tokens *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->ware_house_burst_size_23_8 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->responder_burst_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 32;
	ptr_struct->ware_house_burst_size_7_0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 72;
	ptr_struct->requestor_burst_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 104;
	ptr_struct->gvmi_vl_weight = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 136;
	ptr_struct->gvmi_vl_token_count = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 168;
	ptr_struct->gvmi_vl_saturation = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 200;
	ptr_struct->dwrr_weight = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 232;
	ptr_struct->dwrr_token_count = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 228;
	ptr_struct->vl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 225;
	ptr_struct->vlg = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 224;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_sq_tokens_print(const struct connectib_hw_sq_tokens *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_sq_tokens ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ware_house_burst_size_23_8 : " UH_FMT "\n", ptr_struct->ware_house_burst_size_23_8);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "responder_burst_size : " UH_FMT "\n", ptr_struct->responder_burst_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ware_house_burst_size_7_0 : " UH_FMT "\n", ptr_struct->ware_house_burst_size_7_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "requestor_burst_size : " UH_FMT "\n", ptr_struct->requestor_burst_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_vl_weight       : " UH_FMT "\n", ptr_struct->gvmi_vl_weight);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_vl_token_count  : " UH_FMT "\n", ptr_struct->gvmi_vl_token_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_vl_saturation   : " UH_FMT "\n", ptr_struct->gvmi_vl_saturation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dwrr_weight          : " UH_FMT "\n", ptr_struct->dwrr_weight);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dwrr_token_count     : " UH_FMT "\n", ptr_struct->dwrr_token_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl                   : " UH_FMT "\n", ptr_struct->vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vlg                  : " UH_FMT "\n", ptr_struct->vlg);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

}

int connectib_hw_sq_tokens_size(void)
{
	 return 32;
}

void connectib_hw_sq_tokens_dump(const struct connectib_hw_sq_tokens *ptr_struct, FILE* file)
{
	connectib_hw_sq_tokens_print(ptr_struct, file, 0);
}

void connectib_hw_gvmi_tokens_pack(const struct connectib_hw_gvmi_tokens *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 91;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->port1_avg_saturation);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->port1_high_vlg_req_token);

	offset = 83;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->port1_log2avg_time);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->port1_max_weight_exp);

	offset = 75;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->port1_log2max_time);

	offset = 74;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port0_has_token);

	offset = 73;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port0_vlg_req_vld);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port1_has_token);

	offset = 67;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->port1_max_saturation);

	offset = 66;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port1_vlg_req_vld);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->port0_avg_saturation);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->port0_avg_weight_exp);

	offset = 115;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->port0_log2avg_time);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->port0_high_vlg_req_token);

	offset = 107;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->port0_log2max_time);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->port0_max_weight_exp);

	offset = 99;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->port0_max_saturation);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->port1_avg_weight_exp);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->port1_max_token_count);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port1_max_weight_mantissa);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->port1_avg_token_count);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port1_avg_weight_mantissa);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->port0_max_token_count);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port0_max_weight_mantissa);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->port0_avg_token_count);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port0_avg_weight_mantissa);

}

void connectib_hw_gvmi_tokens_unpack(struct connectib_hw_gvmi_tokens *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 91;
	ptr_struct->port1_avg_saturation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 88;
	ptr_struct->port1_high_vlg_req_token = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 83;
	ptr_struct->port1_log2avg_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 80;
	ptr_struct->port1_max_weight_exp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 75;
	ptr_struct->port1_log2max_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 74;
	ptr_struct->port0_has_token = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 73;
	ptr_struct->port0_vlg_req_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->port1_has_token = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 67;
	ptr_struct->port1_max_saturation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 66;
	ptr_struct->port1_vlg_req_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 123;
	ptr_struct->port0_avg_saturation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 120;
	ptr_struct->port0_avg_weight_exp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 115;
	ptr_struct->port0_log2avg_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 112;
	ptr_struct->port0_high_vlg_req_token = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 107;
	ptr_struct->port0_log2max_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 104;
	ptr_struct->port0_max_weight_exp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 99;
	ptr_struct->port0_max_saturation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 96;
	ptr_struct->port1_avg_weight_exp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 136;
	ptr_struct->port1_max_token_count = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 128;
	ptr_struct->port1_max_weight_mantissa = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 168;
	ptr_struct->port1_avg_token_count = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 160;
	ptr_struct->port1_avg_weight_mantissa = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 200;
	ptr_struct->port0_max_token_count = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 192;
	ptr_struct->port0_max_weight_mantissa = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 232;
	ptr_struct->port0_avg_token_count = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 224;
	ptr_struct->port0_avg_weight_mantissa = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_gvmi_tokens_print(const struct connectib_hw_gvmi_tokens *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_gvmi_tokens ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_avg_saturation : " UH_FMT "\n", ptr_struct->port1_avg_saturation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_high_vlg_req_token : " UH_FMT "\n", ptr_struct->port1_high_vlg_req_token);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_log2avg_time   : " UH_FMT "\n", ptr_struct->port1_log2avg_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_max_weight_exp : " UH_FMT "\n", ptr_struct->port1_max_weight_exp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_log2max_time   : " UH_FMT "\n", ptr_struct->port1_log2max_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_has_token      : " UH_FMT "\n", ptr_struct->port0_has_token);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_vlg_req_vld    : " UH_FMT "\n", ptr_struct->port0_vlg_req_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_has_token      : " UH_FMT "\n", ptr_struct->port1_has_token);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_max_saturation : " UH_FMT "\n", ptr_struct->port1_max_saturation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_vlg_req_vld    : " UH_FMT "\n", ptr_struct->port1_vlg_req_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_avg_saturation : " UH_FMT "\n", ptr_struct->port0_avg_saturation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_avg_weight_exp : " UH_FMT "\n", ptr_struct->port0_avg_weight_exp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_log2avg_time   : " UH_FMT "\n", ptr_struct->port0_log2avg_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_high_vlg_req_token : " UH_FMT "\n", ptr_struct->port0_high_vlg_req_token);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_log2max_time   : " UH_FMT "\n", ptr_struct->port0_log2max_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_max_weight_exp : " UH_FMT "\n", ptr_struct->port0_max_weight_exp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_max_saturation : " UH_FMT "\n", ptr_struct->port0_max_saturation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_avg_weight_exp : " UH_FMT "\n", ptr_struct->port1_avg_weight_exp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_max_token_count : " UH_FMT "\n", ptr_struct->port1_max_token_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_max_weight_mantissa : " UH_FMT "\n", ptr_struct->port1_max_weight_mantissa);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_avg_token_count : " UH_FMT "\n", ptr_struct->port1_avg_token_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port1_avg_weight_mantissa : " UH_FMT "\n", ptr_struct->port1_avg_weight_mantissa);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_max_token_count : " UH_FMT "\n", ptr_struct->port0_max_token_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_max_weight_mantissa : " UH_FMT "\n", ptr_struct->port0_max_weight_mantissa);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_avg_token_count : " UH_FMT "\n", ptr_struct->port0_avg_token_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_avg_weight_mantissa : " UH_FMT "\n", ptr_struct->port0_avg_weight_mantissa);

}

int connectib_hw_gvmi_tokens_size(void)
{
	 return 32;
}

void connectib_hw_gvmi_tokens_dump(const struct connectib_hw_gvmi_tokens *ptr_struct, FILE* file)
{
	connectib_hw_gvmi_tokens_print(ptr_struct, file, 0);
}

void connectib_hw_sq_pointers_pack(const struct connectib_hw_sq_pointers *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->weight_requestor);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->sq_offset_11_6);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->weight_responder);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 38;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->token);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->sq_offset_5_0);

	offset = 71;
	adb2c_push_bits_to_buff(ptr_buff, offset, 25, (u_int32_t)ptr_struct->ware_house_counter_responder);

	offset = 103;
	adb2c_push_bits_to_buff(ptr_buff, offset, 25, (u_int32_t)ptr_struct->ware_house_counter_requestor);

	offset = 135;
	adb2c_push_bits_to_buff(ptr_buff, offset, 25, (u_int32_t)ptr_struct->push_pointer_responder);

	offset = 167;
	adb2c_push_bits_to_buff(ptr_buff, offset, 25, (u_int32_t)ptr_struct->push_pointer_requestor);

	offset = 166;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port);

	offset = 165;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->scheduling_interrupt);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl);

	offset = 199;
	adb2c_push_bits_to_buff(ptr_buff, offset, 25, (u_int32_t)ptr_struct->pop_pointer_responder);

	offset = 198;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->last_gnt_requestor);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log2_max_num_of_qp_responder);

	offset = 231;
	adb2c_push_bits_to_buff(ptr_buff, offset, 25, (u_int32_t)ptr_struct->pop_pointer_requestor);

	offset = 230;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->db_interrupt);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log2_max_num_of_qp_requestor);

}

void connectib_hw_sq_pointers_unpack(struct connectib_hw_sq_pointers *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	ptr_struct->weight_requestor = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 16;
	ptr_struct->sq_offset_11_6 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 6;
	ptr_struct->weight_responder = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 38;
	ptr_struct->token = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 32;
	ptr_struct->sq_offset_5_0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 71;
	ptr_struct->ware_house_counter_responder = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 25);

	offset = 103;
	ptr_struct->ware_house_counter_requestor = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 25);

	offset = 135;
	ptr_struct->push_pointer_responder = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 25);

	offset = 167;
	ptr_struct->push_pointer_requestor = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 25);

	offset = 166;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 165;
	ptr_struct->scheduling_interrupt = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 160;
	ptr_struct->vl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 199;
	ptr_struct->pop_pointer_responder = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 25);

	offset = 198;
	ptr_struct->last_gnt_requestor = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 192;
	ptr_struct->log2_max_num_of_qp_responder = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 231;
	ptr_struct->pop_pointer_requestor = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 25);

	offset = 230;
	ptr_struct->db_interrupt = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 224;
	ptr_struct->log2_max_num_of_qp_requestor = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

}

void connectib_hw_sq_pointers_print(const struct connectib_hw_sq_pointers *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_sq_pointers ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "weight_requestor     : " UH_FMT "\n", ptr_struct->weight_requestor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq_offset_11_6       : " UH_FMT "\n", ptr_struct->sq_offset_11_6);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "weight_responder     : " UH_FMT "\n", ptr_struct->weight_responder);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "token                : " UH_FMT "\n", ptr_struct->token);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq_offset_5_0        : " UH_FMT "\n", ptr_struct->sq_offset_5_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ware_house_counter_responder : " UH_FMT "\n", ptr_struct->ware_house_counter_responder);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ware_house_counter_requestor : " UH_FMT "\n", ptr_struct->ware_house_counter_requestor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "push_pointer_responder : " UH_FMT "\n", ptr_struct->push_pointer_responder);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "push_pointer_requestor : " UH_FMT "\n", ptr_struct->push_pointer_requestor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "scheduling_interrupt : " UH_FMT "\n", ptr_struct->scheduling_interrupt);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl                   : " UH_FMT "\n", ptr_struct->vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pop_pointer_responder : " UH_FMT "\n", ptr_struct->pop_pointer_responder);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_gnt_requestor   : " UH_FMT "\n", ptr_struct->last_gnt_requestor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_max_num_of_qp_responder : " UH_FMT "\n", ptr_struct->log2_max_num_of_qp_responder);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pop_pointer_requestor : " UH_FMT "\n", ptr_struct->pop_pointer_requestor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_interrupt         : " UH_FMT "\n", ptr_struct->db_interrupt);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_max_num_of_qp_requestor : " UH_FMT "\n", ptr_struct->log2_max_num_of_qp_requestor);

}

int connectib_hw_sq_pointers_size(void)
{
	 return 32;
}

void connectib_hw_sq_pointers_dump(const struct connectib_hw_sq_pointers *ptr_struct, FILE* file)
{
	connectib_hw_sq_pointers_print(ptr_struct, file, 0);
}

void connectib_hw_trasta_pack(const struct connectib_hw_trasta *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->va_offset_63_32);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->va_offset_31_0);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->klm_index);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->invalidation_counter);

	offset = 111;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

}

void connectib_hw_trasta_unpack(struct connectib_hw_trasta *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->va_offset_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->va_offset_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->klm_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 112;
	ptr_struct->invalidation_counter = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 111;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_hw_trasta_print(const struct connectib_hw_trasta *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_trasta ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "va_offset_63_32      : " U32H_FMT "\n", ptr_struct->va_offset_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "va_offset_31_0       : " U32H_FMT "\n", ptr_struct->va_offset_31_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "klm_index            : " U32H_FMT "\n", ptr_struct->klm_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "invalidation_counter : " UH_FMT "\n", ptr_struct->invalidation_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

}

int connectib_hw_trasta_size(void)
{
	 return 16;
}

void connectib_hw_trasta_dump(const struct connectib_hw_trasta *ptr_struct, FILE* file)
{
	connectib_hw_trasta_print(ptr_struct, file, 0);
}

void connectib_hw_trastb_pack(const struct connectib_hw_trastb *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->generation_count);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->bsf_offset);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->m_fbo);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->w_fbo);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->m_va);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->w_va);

}

void connectib_hw_trastb_unpack(struct connectib_hw_trastb *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->generation_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->bsf_offset = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->m_fbo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->w_fbo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->m_va = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 192;
	ptr_struct->w_va = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_hw_trastb_print(const struct connectib_hw_trastb *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_trastb ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "generation_count     : " UH_FMT "\n", ptr_struct->generation_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bsf_offset           : " U32H_FMT "\n", ptr_struct->bsf_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_fbo                : " U32H_FMT "\n", ptr_struct->m_fbo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_fbo                : " U32H_FMT "\n", ptr_struct->w_fbo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_va                 : " U64H_FMT "\n", ptr_struct->m_va);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "w_va                 : " U64H_FMT "\n", ptr_struct->w_va);

}

int connectib_hw_trastb_size(void)
{
	 return 32;
}

void connectib_hw_trastb_dump(const struct connectib_hw_trastb *ptr_struct, FILE* file)
{
	connectib_hw_trastb_print(ptr_struct, file, 0);
}

void connectib_hw_base_psv_pack(const struct connectib_hw_base_psv *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->pd);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->data_heart_beat);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wqe_heart_beat);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->status);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->syndrome);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->transient_signature);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->reference_tag);

}

void connectib_hw_base_psv_unpack(struct connectib_hw_base_psv *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->pd = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 2;
	ptr_struct->data_heart_beat = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->wqe_heart_beat = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->status = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->syndrome = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->transient_signature = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->reference_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_hw_base_psv_print(const struct connectib_hw_base_psv *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_base_psv ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pd                   : " UH_FMT "\n", ptr_struct->pd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_heart_beat      : " UH_FMT "\n", ptr_struct->data_heart_beat);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_heart_beat       : " UH_FMT "\n", ptr_struct->wqe_heart_beat);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " UH_FMT "\n", ptr_struct->status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "syndrome             : " UH_FMT "\n", ptr_struct->syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "transient_signature  : " U32H_FMT "\n", ptr_struct->transient_signature);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reference_tag        : " U32H_FMT "\n", ptr_struct->reference_tag);

}

int connectib_hw_base_psv_size(void)
{
	 return 16;
}

void connectib_hw_base_psv_dump(const struct connectib_hw_base_psv *ptr_struct, FILE* file)
{
	connectib_hw_base_psv_print(ptr_struct, file, 0);
}

void connectib_hw_bsf_entry_pack(const union connectib_hw_bsf_entry *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_bsf_entry_w_non_repeating_m_non_repeating_pack(&(ptr_struct->hw_bsf_entry_w_non_repeating_m_non_repeating), ptr_buff);
}

void connectib_hw_bsf_entry_unpack(union connectib_hw_bsf_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_bsf_entry_w_non_repeating_m_non_repeating_unpack(&(ptr_struct->hw_bsf_entry_w_non_repeating_m_non_repeating), ptr_buff);
}

void connectib_hw_bsf_entry_print(const union connectib_hw_bsf_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_bsf_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "same_block_repeating:\n");
	connectib_hw_bsf_entry_same_block_repeating_print(&(ptr_struct->same_block_repeating), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "same_block_non_repeating:\n");
	connectib_hw_bsf_entry_same_block_non_repeating_print(&(ptr_struct->same_block_non_repeating), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_bsf_entry_w_repeating_m_repeating:\n");
	connectib_hw_bsf_entry_w_repeating_m_repeating_print(&(ptr_struct->hw_bsf_entry_w_repeating_m_repeating), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_bsf_entry_w_repeating_m_non_repeating:\n");
	connectib_hw_bsf_entry_w_repeating_m_non_repeating_print(&(ptr_struct->hw_bsf_entry_w_repeating_m_non_repeating), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_bsf_entry_w_non_repeating_m_repeating:\n");
	connectib_hw_bsf_entry_w_non_repeating_m_repeating_print(&(ptr_struct->hw_bsf_entry_w_non_repeating_m_repeating), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_bsf_entry_w_non_repeating_m_non_repeating:\n");
	connectib_hw_bsf_entry_w_non_repeating_m_non_repeating_print(&(ptr_struct->hw_bsf_entry_w_non_repeating_m_non_repeating), file, indent_level + 1);

}

int connectib_hw_bsf_entry_size(void)
{
	 return 64;
}

void connectib_hw_bsf_entry_dump(const union connectib_hw_bsf_entry *ptr_struct, FILE* file)
{
	connectib_hw_bsf_entry_print(ptr_struct, file, 0);
}

void connectib_hw_port_info_pack(const struct connectib_hw_port_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->lmc);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->mad_filter);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lid);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->gid_prefix);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(272, 16, i, 512, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->dc_rsl2sq_map[i]);
	}

}

void connectib_hw_port_info_unpack(struct connectib_hw_port_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	ptr_struct->lmc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 2;
	ptr_struct->mad_filter = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 48;
	ptr_struct->lid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->gid_prefix = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(272, 16, i, 512, 1);
	ptr_struct->dc_rsl2sq_map[i] = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);
	}

}

void connectib_hw_port_info_print(const struct connectib_hw_port_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_port_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lmc                  : " UH_FMT "\n", ptr_struct->lmc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mad_filter           : " UH_FMT "\n", ptr_struct->mad_filter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lid                  : " UH_FMT "\n", ptr_struct->lid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gid_prefix           : " U64H_FMT "\n", ptr_struct->gid_prefix);

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_rsl2sq_map_%03d   : " UH_FMT "\n", i, ptr_struct->dc_rsl2sq_map[i]);
	}

}

int connectib_hw_port_info_size(void)
{
	 return 64;
}

void connectib_hw_port_info_dump(const struct connectib_hw_port_info *ptr_struct, FILE* file)
{
	connectib_hw_port_info_print(ptr_struct, file, 0);
}

void connectib_hw_qpc_r_dct_pack(const union connectib_hw_qpc_r_dct *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_dct_pack(&(ptr_struct->dct), ptr_buff);
}

void connectib_hw_qpc_r_dct_unpack(union connectib_hw_qpc_r_dct *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_dct_unpack(&(ptr_struct->dct), ptr_buff);
}

void connectib_hw_qpc_r_dct_print(const union connectib_hw_qpc_r_dct *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_qpc_r_dct ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpc:\n");
	connectib_hw_r_qpc_print(&(ptr_struct->qpc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dct:\n");
	connectib_hw_dct_print(&(ptr_struct->dct), file, indent_level + 1);

}

int connectib_hw_qpc_r_dct_size(void)
{
	 return 64;
}

void connectib_hw_qpc_r_dct_dump(const union connectib_hw_qpc_r_dct *ptr_struct, FILE* file)
{
	connectib_hw_qpc_r_dct_print(ptr_struct, file, 0);
}

void connectib_hw_s_qpc_pack(const union connectib_hw_s_qpc *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_s_qpc_packet_allowed_pack(&(ptr_struct->packet_allowed), ptr_buff);
}

void connectib_hw_s_qpc_unpack(union connectib_hw_s_qpc *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_s_qpc_packet_allowed_unpack(&(ptr_struct->packet_allowed), ptr_buff);
}

void connectib_hw_s_qpc_print(const union connectib_hw_s_qpc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_s_qpc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_unsent_psn:\n");
	connectib_hw_s_qpc_first_unsent_psn_print(&(ptr_struct->first_unsent_psn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "number_outstanding_reads:\n");
	connectib_hw_s_qpc_number_outstanding_reads_print(&(ptr_struct->number_outstanding_reads), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_psn_of_message_currently_completing:\n");
	connectib_hw_s_qpc_first_psn_of_message_currently_completing_print(&(ptr_struct->first_psn_of_message_currently_completing), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "currently_completing_ds:\n");
	connectib_hw_s_qpc_currently_completing_ds_print(&(ptr_struct->currently_completing_ds), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timer_enable:\n");
	connectib_hw_s_qpc_timer_enable_print(&(ptr_struct->timer_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl15:\n");
	connectib_hw_s_qpc_vl15_print(&(ptr_struct->vl15), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_grh:\n");
	connectib_hw_s_qpc_force_grh_print(&(ptr_struct->force_grh), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_access_key_31_0:\n");
	connectib_hw_s_qpc_dc_access_key_31_0_print(&(ptr_struct->dc_access_key_31_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_access_key_63_32:\n");
	connectib_hw_s_qpc_dc_access_key_63_32_print(&(ptr_struct->dc_access_key_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lsn:\n");
	connectib_hw_s_qpc_lsn_print(&(ptr_struct->lsn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_gather_commit:\n");
	connectib_hw_s_qpc_sx_gather_commit_print(&(ptr_struct->sx_gather_commit), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_my_gid_index:\n");
	connectib_hw_s_qpc_force_my_gid_index_print(&(ptr_struct->force_my_gid_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "to_sequence_error_retry_limit:\n");
	connectib_hw_s_qpc_to_sequence_error_retry_limit_print(&(ptr_struct->to_sequence_error_retry_limit), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_heartbeat:\n");
	connectib_hw_s_qpc_sx_heartbeat_print(&(ptr_struct->sx_heartbeat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts:\n");
	connectib_hw_s_qpc_ts_print(&(ptr_struct->ts), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_grh_tclass:\n");
	connectib_hw_s_qpc_force_grh_tclass_print(&(ptr_struct->force_grh_tclass), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reverse_sl:\n");
	connectib_hw_s_qpc_reverse_sl_print(&(ptr_struct->reverse_sl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "to_sequence_error_retry_counter:\n");
	connectib_hw_s_qpc_to_sequence_error_retry_counter_print(&(ptr_struct->to_sequence_error_retry_counter), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ssn:\n");
	connectib_hw_s_qpc_ssn_print(&(ptr_struct->ssn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "counter_index:\n");
	connectib_hw_s_qpc_counter_index_print(&(ptr_struct->counter_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flush_syndrom:\n");
	connectib_hw_s_qpc_flush_syndrom_print(&(ptr_struct->flush_syndrom), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_to_send_psn:\n");
	connectib_hw_s_qpc_next_to_send_psn_print(&(ptr_struct->next_to_send_psn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_number:\n");
	connectib_hw_s_qpc_cq_number_print(&(ptr_struct->cq_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "responder_session_id:\n");
	connectib_hw_s_qpc_responder_session_id_print(&(ptr_struct->responder_session_id), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "deth_sqpn:\n");
	connectib_hw_s_qpc_deth_sqpn_print(&(ptr_struct->deth_sqpn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "oldest_non_acked_psn:\n");
	connectib_hw_s_qpc_oldest_non_acked_psn_print(&(ptr_struct->oldest_non_acked_psn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "psn_since_ackreq:\n");
	connectib_hw_s_qpc_psn_since_ackreq_print(&(ptr_struct->psn_since_ackreq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_state:\n");
	connectib_hw_s_qpc_qp_state_print(&(ptr_struct->qp_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swqe_consumer_index:\n");
	connectib_hw_s_qpc_swqe_consumer_index_print(&(ptr_struct->swqe_consumer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "currently_completing_wqe_index:\n");
	connectib_hw_s_qpc_currently_completing_wqe_index_print(&(ptr_struct->currently_completing_wqe_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_state:\n");
	connectib_hw_s_qpc_dc_state_print(&(ptr_struct->dc_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "limited_qp:\n");
	connectib_hw_s_qpc_limited_qp_print(&(ptr_struct->limited_qp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ack_req_frequency:\n");
	connectib_hw_s_qpc_ack_req_frequency_print(&(ptr_struct->ack_req_frequency), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcu_trap:\n");
	connectib_hw_s_qpc_tcu_trap_print(&(ptr_struct->tcu_trap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "can_use_reserved_lkey:\n");
	connectib_hw_s_qpc_can_use_reserved_lkey_print(&(ptr_struct->can_use_reserved_lkey), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcu_drop:\n");
	connectib_hw_s_qpc_tcu_drop_print(&(ptr_struct->tcu_drop), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "umr_enable:\n");
	connectib_hw_s_qpc_umr_enable_print(&(ptr_struct->umr_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "frwr_enable:\n");
	connectib_hw_s_qpc_frwr_enable_print(&(ptr_struct->frwr_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_heartbeat:\n");
	connectib_hw_s_qpc_rx_heartbeat_print(&(ptr_struct->rx_heartbeat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ldb_int:\n");
	connectib_hw_s_qpc_ldb_int_print(&(ptr_struct->ldb_int), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inline_scatter:\n");
	connectib_hw_s_qpc_inline_scatter_print(&(ptr_struct->inline_scatter), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rnr_nak_retry_limit:\n");
	connectib_hw_s_qpc_rnr_nak_retry_limit_print(&(ptr_struct->rnr_nak_retry_limit), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fast_path:\n");
	connectib_hw_s_qpc_fast_path_print(&(ptr_struct->fast_path), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rnr_nak_retry_counter:\n");
	connectib_hw_s_qpc_rnr_nak_retry_counter_print(&(ptr_struct->rnr_nak_retry_counter), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ignore_pkey_check:\n");
	connectib_hw_s_qpc_ignore_pkey_check_print(&(ptr_struct->ignore_pkey_check), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_atomic:\n");
	connectib_hw_s_qpc_extended_atomic_print(&(ptr_struct->extended_atomic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_read_atomic:\n");
	connectib_hw_s_qpc_max_read_atomic_print(&(ptr_struct->max_read_atomic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "umr_permission_select:\n");
	connectib_hw_s_qpc_umr_permission_select_print(&(ptr_struct->umr_permission_select), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_direct_access:\n");
	connectib_hw_s_qpc_no_direct_access_print(&(ptr_struct->no_direct_access), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_credits_enabled:\n");
	connectib_hw_s_qpc_e2e_credits_enabled_print(&(ptr_struct->e2e_credits_enabled), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_full_handshake:\n");
	connectib_hw_s_qpc_dc_full_handshake_print(&(ptr_struct->dc_full_handshake), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uncompleted_non_wire_wqe:\n");
	connectib_hw_s_qpc_uncompleted_non_wire_wqe_print(&(ptr_struct->uncompleted_non_wire_wqe), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_valid:\n");
	connectib_hw_s_qpc_qp_valid_print(&(ptr_struct->qp_valid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_allowed:\n");
	connectib_hw_s_qpc_packet_allowed_print(&(ptr_struct->packet_allowed), file, indent_level + 1);

}

int connectib_hw_s_qpc_size(void)
{
	 return 64;
}

void connectib_hw_s_qpc_dump(const union connectib_hw_s_qpc *ptr_struct, FILE* file)
{
	connectib_hw_s_qpc_print(ptr_struct, file, 0);
}

void connectib_hw_c_qpc_pack(const union connectib_hw_c_qpc *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_c_qpc_common_valid_pack(&(ptr_struct->common_valid), ptr_buff);
}

void connectib_hw_c_qpc_unpack(union connectib_hw_c_qpc *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_c_qpc_common_valid_unpack(&(ptr_struct->common_valid), ptr_buff);
}

void connectib_hw_c_qpc_print(const union connectib_hw_c_qpc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_qpc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "peer_gid_31_0:\n");
	connectib_hw_c_qpc_peer_gid_31_0_print(&(ptr_struct->peer_gid_31_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "peer_gid_63_32:\n");
	connectib_hw_c_qpc_peer_gid_63_32_print(&(ptr_struct->peer_gid_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "peer_gid_95_64:\n");
	connectib_hw_c_qpc_peer_gid_95_64_print(&(ptr_struct->peer_gid_95_64), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "peer_gid_127_96:\n");
	connectib_hw_c_qpc_peer_gid_127_96_print(&(ptr_struct->peer_gid_127_96), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "peer_lid2:\n");
	connectib_hw_c_qpc_peer_lid2_print(&(ptr_struct->peer_lid2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "peer_qp2:\n");
	connectib_hw_c_qpc_peer_qp2_print(&(ptr_struct->peer_qp2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pd:\n");
	connectib_hw_c_qpc_pd_print(&(ptr_struct->pd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_4_0:\n");
	connectib_hw_c_qpc_mtt_pointer_4_0_print(&(ptr_struct->mtt_pointer_4_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_36_5:\n");
	connectib_hw_c_qpc_mtt_pointer_36_5_print(&(ptr_struct->mtt_pointer_36_5), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_39_37:\n");
	connectib_hw_c_qpc_mtt_pointer_39_37_print(&(ptr_struct->mtt_pointer_39_37), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl:\n");
	connectib_hw_c_qpc_sl_print(&(ptr_struct->sl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh_bit:\n");
	connectib_hw_c_qpc_grh_bit_print(&(ptr_struct->grh_bit), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number:\n");
	connectib_hw_c_qpc_port_number_print(&(ptr_struct->port_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "adaptive_routing:\n");
	connectib_hw_c_qpc_adaptive_routing_print(&(ptr_struct->adaptive_routing), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swqe_buffer_size:\n");
	connectib_hw_c_qpc_swqe_buffer_size_print(&(ptr_struct->swqe_buffer_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "peer_lid:\n");
	connectib_hw_c_qpc_peer_lid_print(&(ptr_struct->peer_lid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "my_lid:\n");
	connectib_hw_c_qpc_my_lid_print(&(ptr_struct->my_lid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "my_gid_mac_index:\n");
	connectib_hw_c_qpc_my_gid_mac_index_print(&(ptr_struct->my_gid_mac_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_message_size:\n");
	connectib_hw_c_qpc_max_message_size_print(&(ptr_struct->max_message_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtu:\n");
	connectib_hw_c_qpc_mtu_print(&(ptr_struct->mtu), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qkey:\n");
	connectib_hw_c_qpc_qkey_print(&(ptr_struct->qkey), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_label:\n");
	connectib_hw_c_qpc_flow_label_print(&(ptr_struct->flow_label), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey_table_index:\n");
	connectib_hw_c_qpc_pkey_table_index_print(&(ptr_struct->pkey_table_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "context_number:\n");
	connectib_hw_c_qpc_context_number_print(&(ptr_struct->context_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_page_offset:\n");
	connectib_hw_c_qpc_wqe_page_offset_print(&(ptr_struct->wqe_page_offset), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_migreq:\n");
	connectib_hw_c_qpc_sx_migreq_print(&(ptr_struct->sx_migreq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_migreq:\n");
	connectib_hw_c_qpc_rx_migreq_print(&(ptr_struct->rx_migreq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "peer_qp:\n");
	connectib_hw_c_qpc_peer_qp_print(&(ptr_struct->peer_qp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hop_limit:\n");
	connectib_hw_c_qpc_hop_limit_print(&(ptr_struct->hop_limit), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tclass:\n");
	connectib_hw_c_qpc_tclass_print(&(ptr_struct->tclass), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_buffer_size:\n");
	connectib_hw_c_qpc_rwqe_buffer_size_print(&(ptr_struct->rwqe_buffer_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rcyclic_size:\n");
	connectib_hw_c_qpc_rcyclic_size_print(&(ptr_struct->rcyclic_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_stride:\n");
	connectib_hw_c_qpc_rwqe_stride_print(&(ptr_struct->rwqe_stride), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ipd:\n");
	connectib_hw_c_qpc_ipd_print(&(ptr_struct->ipd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_loopback:\n");
	connectib_hw_c_qpc_force_loopback_print(&(ptr_struct->force_loopback), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwq_type:\n");
	connectib_hw_c_qpc_rwq_type_print(&(ptr_struct->rwq_type), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_page_size:\n");
	connectib_hw_c_qpc_log_page_size_print(&(ptr_struct->log_page_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dual_write:\n");
	connectib_hw_c_qpc_dual_write_print(&(ptr_struct->dual_write), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wq_signature:\n");
	connectib_hw_c_qpc_wq_signature_print(&(ptr_struct->wq_signature), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "common_valid:\n");
	connectib_hw_c_qpc_common_valid_print(&(ptr_struct->common_valid), file, indent_level + 1);

}

int connectib_hw_c_qpc_size(void)
{
	 return 64;
}

void connectib_hw_c_qpc_dump(const union connectib_hw_c_qpc *ptr_struct, FILE* file)
{
	connectib_hw_c_qpc_print(ptr_struct, file, 0);
}

void connectib_hw_c_srq_pack(const union connectib_hw_c_srq *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_e2e_pa_33_2_pack(&(ptr_struct->e2e_pa_33_2), ptr_buff);
}

void connectib_hw_c_srq_unpack(union connectib_hw_c_srq *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_e2e_pa_33_2_unpack(&(ptr_struct->e2e_pa_33_2), ptr_buff);
}

void connectib_hw_c_srq_print(const union connectib_hw_c_srq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_srq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_consumer_index:\n");
	connectib_e2e_consumer_index_print(&(ptr_struct->e2e_consumer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_fw:\n");
	connectib_srq_fw_print(&(ptr_struct->srq_fw), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_consumer_index:\n");
	connectib_rwqe_consumer_index_print(&(ptr_struct->rwqe_consumer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_producer_index:\n");
	connectib_e2e_producer_index_print(&(ptr_struct->e2e_producer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xrc_domain:\n");
	connectib_xrc_domain_print(&(ptr_struct->xrc_domain), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pd:\n");
	connectib_pd_print(&(ptr_struct->pd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_credits_prefetch_threshold:\n");
	connectib_e2e_credits_prefetch_threshold_print(&(ptr_struct->e2e_credits_prefetch_threshold), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_number:\n");
	connectib_cq_number_print(&(ptr_struct->cq_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_buffer_size:\n");
	connectib_rwqe_buffer_size_print(&(ptr_struct->rwqe_buffer_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_stride:\n");
	connectib_rwqe_stride_print(&(ptr_struct->rwqe_stride), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "context_number:\n");
	connectib_context_number_print(&(ptr_struct->context_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_page_size:\n");
	connectib_log_page_size_print(&(ptr_struct->log_page_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcu_drop:\n");
	connectib_tcu_drop_print(&(ptr_struct->tcu_drop), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "can_use_reserved_lkey:\n");
	connectib_can_use_reserved_lkey_print(&(ptr_struct->can_use_reserved_lkey), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcu_trap:\n");
	connectib_tcu_trap_print(&(ptr_struct->tcu_trap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_39_32:\n");
	connectib_mtt_pointer_39_32_print(&(ptr_struct->mtt_pointer_39_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_page_offset:\n");
	connectib_wqe_page_offset_print(&(ptr_struct->wqe_page_offset), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_heartbeat:\n");
	connectib_rx_heartbeat_print(&(ptr_struct->rx_heartbeat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_fetch_enable:\n");
	connectib_e2e_fetch_enable_print(&(ptr_struct->e2e_fetch_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_limit_event:\n");
	connectib_rwqe_limit_event_print(&(ptr_struct->rwqe_limit_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_31_0:\n");
	connectib_mtt_pointer_31_0_print(&(ptr_struct->mtt_pointer_31_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_pa_63_34:\n");
	connectib_e2e_pa_63_34_print(&(ptr_struct->e2e_pa_63_34), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_valid:\n");
	connectib_srq_valid_print(&(ptr_struct->srq_valid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wq_signature:\n");
	connectib_wq_signature_print(&(ptr_struct->wq_signature), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "e2e_pa_33_2:\n");
	connectib_e2e_pa_33_2_print(&(ptr_struct->e2e_pa_33_2), file, indent_level + 1);

}

int connectib_hw_c_srq_size(void)
{
	 return 64;
}

void connectib_hw_c_srq_dump(const union connectib_hw_c_srq *ptr_struct, FILE* file)
{
	connectib_hw_c_srq_print(ptr_struct, file, 0);
}

void connectib_hw_toc_pack(const struct connectib_hw_toc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->eqn_offset);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->msix_offset);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->flr_state);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->cmdq_phy_addr);

	offset = 128;
	connectib_hw_init_info_pack(&(ptr_struct->init_info), ptr_buff + offset/8);

	offset = 160;
	connectib_pf_vf_cfg_space_pack(&(ptr_struct->pci_cfg_space), ptr_buff + offset/8);

	offset = 408;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sw_cmd_eq);

	offset = 406;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->csum);

	offset = 405;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ext_mod);

	offset = 404;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->teardown_event);

	offset = 403;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pre_icm_driver_refused);

	offset = 402;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->initializing);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->issi);

	offset = 440;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hca_state);

	offset = 438;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->nic_interface_type);

	offset = 436;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->nic_interface_type_sw);

	offset = 435;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->non_pci_flr);

	offset = 434;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->icmc_error_px);

	offset = 433;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->doorbell_recovery_en);

	offset = 429;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->nodnic_idx);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->doorbell_record);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->capi_direct_pasid);

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->capi_pasid);

}

void connectib_hw_toc_unpack(struct connectib_hw_toc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->eqn_offset = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->msix_offset = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 62;
	ptr_struct->flr_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 64;
	ptr_struct->cmdq_phy_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 128;
	connectib_hw_init_info_unpack(&(ptr_struct->init_info), ptr_buff + offset/8);

	offset = 160;
	connectib_pf_vf_cfg_space_unpack(&(ptr_struct->pci_cfg_space), ptr_buff + offset/8);

	offset = 408;
	ptr_struct->sw_cmd_eq = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 406;
	ptr_struct->csum = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 405;
	ptr_struct->ext_mod = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 404;
	ptr_struct->teardown_event = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 403;
	ptr_struct->pre_icm_driver_refused = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 402;
	ptr_struct->initializing = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 384;
	ptr_struct->issi = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 440;
	ptr_struct->hca_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 438;
	ptr_struct->nic_interface_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 436;
	ptr_struct->nic_interface_type_sw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 435;
	ptr_struct->non_pci_flr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 434;
	ptr_struct->icmc_error_px = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 433;
	ptr_struct->doorbell_recovery_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 429;
	ptr_struct->nodnic_idx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 448;
	ptr_struct->doorbell_record = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 496;
	ptr_struct->capi_direct_pasid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 480;
	ptr_struct->capi_pasid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_toc_print(const struct connectib_hw_toc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_toc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eqn_offset           : " UH_FMT "\n", ptr_struct->eqn_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msix_offset          : " UH_FMT "\n", ptr_struct->msix_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flr_state            : " UH_FMT "\n", ptr_struct->flr_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmdq_phy_addr        : " U64H_FMT "\n", ptr_struct->cmdq_phy_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "init_info:\n");
	connectib_hw_init_info_print(&(ptr_struct->init_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_cfg_space:\n");
	connectib_pf_vf_cfg_space_print(&(ptr_struct->pci_cfg_space), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_cmd_eq            : " UH_FMT "\n", ptr_struct->sw_cmd_eq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "csum                 : " UH_FMT "\n", ptr_struct->csum);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ext_mod              : " UH_FMT "\n", ptr_struct->ext_mod);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "teardown_event       : " UH_FMT "\n", ptr_struct->teardown_event);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pre_icm_driver_refused : " UH_FMT "\n", ptr_struct->pre_icm_driver_refused);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "initializing         : " UH_FMT "\n", ptr_struct->initializing);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "issi                 : " UH_FMT "\n", ptr_struct->issi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hca_state            : " UH_FMT "\n", ptr_struct->hca_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nic_interface_type   : " UH_FMT "\n", ptr_struct->nic_interface_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nic_interface_type_sw : " UH_FMT "\n", ptr_struct->nic_interface_type_sw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "non_pci_flr          : " UH_FMT "\n", ptr_struct->non_pci_flr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icmc_error_px        : " UH_FMT "\n", ptr_struct->icmc_error_px);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "doorbell_recovery_en : " UH_FMT "\n", ptr_struct->doorbell_recovery_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nodnic_idx           : " UH_FMT "\n", ptr_struct->nodnic_idx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "doorbell_record      : " U32H_FMT "\n", ptr_struct->doorbell_record);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capi_direct_pasid    : " UH_FMT "\n", ptr_struct->capi_direct_pasid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capi_pasid           : " UH_FMT "\n", ptr_struct->capi_pasid);

}

int connectib_hw_toc_size(void)
{
	 return 64;
}

void connectib_hw_toc_dump(const struct connectib_hw_toc *ptr_struct, FILE* file)
{
	connectib_hw_toc_print(ptr_struct, file, 0);
}

void connectib_hw_mkc_pack(const struct connectib_hw_mkc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->access_mode);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lr);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lw);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rr);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->a);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->umr_en);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->small_fence_on_rdma_read_response);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->goto_pci);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->relaxed_ordering);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->no_snoop);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->tlp_processing_hints);

	offset = 9;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->tlp_processing_hints_en);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->free);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->mem_key);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->pd);

	offset = 103;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->en_rinval);

	offset = 101;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->num_of_sigerr);

	offset = 99;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->trast_a_en);

	offset = 98;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->trast_b_en);

	offset = 97;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bsf_enabled);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->length_64);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->start_address);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->length);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->bsf_octoword_count);

	offset = 305;
	adb2c_push_bits_to_buff(ptr_buff, offset, 15, (u_int32_t)ptr_struct->bsf_first_index_46_32);

	offset = 320;
	adb2c_push_bits_to_buff(ptr_buff, offset, 28, (u_int32_t)ptr_struct->bsf_first_index_31_4);

	offset = 369;
	adb2c_push_bits_to_buff(ptr_buff, offset, 15, (u_int32_t)ptr_struct->mtt_first_index_46_32);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 29, (u_int32_t)ptr_struct->mtt_first_index_31_3);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mtt_octoword_count);

	offset = 475;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->entity_size);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->generation_counter);

}

void connectib_hw_mkc_unpack(struct connectib_hw_mkc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 22;
	ptr_struct->access_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 21;
	ptr_struct->lr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->lw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->rr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->a = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->umr_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 15;
	ptr_struct->small_fence_on_rdma_read_response = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->goto_pci = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->relaxed_ordering = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->no_snoop = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->tlp_processing_hints = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 9;
	ptr_struct->tlp_processing_hints_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->free = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 56;
	ptr_struct->mem_key = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 104;
	ptr_struct->pd = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 103;
	ptr_struct->en_rinval = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 101;
	ptr_struct->num_of_sigerr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 99;
	ptr_struct->trast_a_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 98;
	ptr_struct->trast_b_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 97;
	ptr_struct->bsf_enabled = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->length_64 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 128;
	ptr_struct->start_address = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 192;
	ptr_struct->length = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 256;
	ptr_struct->bsf_octoword_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 305;
	ptr_struct->bsf_first_index_46_32 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 15);

	offset = 320;
	ptr_struct->bsf_first_index_31_4 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 28);

	offset = 369;
	ptr_struct->mtt_first_index_46_32 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 15);

	offset = 384;
	ptr_struct->mtt_first_index_31_3 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 29);

	offset = 416;
	ptr_struct->mtt_octoword_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 475;
	ptr_struct->entity_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 496;
	ptr_struct->generation_counter = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_hw_mkc_print(const struct connectib_hw_mkc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_mkc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_mode          : " UH_FMT "\n", ptr_struct->access_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lr                   : " UH_FMT "\n", ptr_struct->lr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lw                   : " UH_FMT "\n", ptr_struct->lw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rr                   : " UH_FMT "\n", ptr_struct->rr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a                    : " UH_FMT "\n", ptr_struct->a);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "umr_en               : " UH_FMT "\n", ptr_struct->umr_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "small_fence_on_rdma_read_response : " UH_FMT "\n", ptr_struct->small_fence_on_rdma_read_response);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "goto_pci             : " UH_FMT "\n", ptr_struct->goto_pci);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "relaxed_ordering     : " UH_FMT "\n", ptr_struct->relaxed_ordering);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_snoop             : " UH_FMT "\n", ptr_struct->no_snoop);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tlp_processing_hints : " UH_FMT "\n", ptr_struct->tlp_processing_hints);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tlp_processing_hints_en : " UH_FMT "\n", ptr_struct->tlp_processing_hints_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "free                 : " UH_FMT "\n", ptr_struct->free);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mem_key              : " UH_FMT "\n", ptr_struct->mem_key);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pd                   : " UH_FMT "\n", ptr_struct->pd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "en_rinval            : " UH_FMT "\n", ptr_struct->en_rinval);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_sigerr        : " UH_FMT "\n", ptr_struct->num_of_sigerr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trast_a_en           : " UH_FMT "\n", ptr_struct->trast_a_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trast_b_en           : " UH_FMT "\n", ptr_struct->trast_b_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bsf_enabled          : " UH_FMT "\n", ptr_struct->bsf_enabled);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "length_64            : " UH_FMT "\n", ptr_struct->length_64);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "start_address        : " U64H_FMT "\n", ptr_struct->start_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "length               : " U64H_FMT "\n", ptr_struct->length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bsf_octoword_count   : " U32H_FMT "\n", ptr_struct->bsf_octoword_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bsf_first_index_46_32 : " UH_FMT "\n", ptr_struct->bsf_first_index_46_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bsf_first_index_31_4 : " UH_FMT "\n", ptr_struct->bsf_first_index_31_4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_first_index_46_32 : " UH_FMT "\n", ptr_struct->mtt_first_index_46_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_first_index_31_3 : " UH_FMT "\n", ptr_struct->mtt_first_index_31_3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_octoword_count   : " U32H_FMT "\n", ptr_struct->mtt_octoword_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "entity_size          : " UH_FMT "\n", ptr_struct->entity_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "generation_counter   : " UH_FMT "\n", ptr_struct->generation_counter);

}

int connectib_hw_mkc_size(void)
{
	 return 64;
}

void connectib_hw_mkc_dump(const struct connectib_hw_mkc *ptr_struct, FILE* file)
{
	connectib_hw_mkc_print(ptr_struct, file, 0);
}

void connectib_hw_c_eqc_pack(const union connectib_hw_c_eqc *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_cqc_moderation_timestamp_7_0_pack(&(ptr_struct->moderation_timestamp_7_0), ptr_buff);
}

void connectib_hw_c_eqc_unpack(union connectib_hw_c_eqc *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_cqc_moderation_timestamp_7_0_unpack(&(ptr_struct->moderation_timestamp_7_0), ptr_buff);
}

void connectib_hw_c_eqc_print(const union connectib_hw_c_eqc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_eqc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lookahead_size_23_8:\n");
	connectib_hw_cqc_lookahead_size_23_8_print(&(ptr_struct->lookahead_size_23_8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqc_valid:\n");
	connectib_hw_cqc_cqc_valid_print(&(ptr_struct->cqc_valid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_128b_size:\n");
	connectib_hw_cqc_cqe_128b_size_print(&(ptr_struct->cqe_128b_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_coalesce_checksum_msix:\n");
	connectib_hw_cqc_cqe_coalesce_checksum_msix_print(&(ptr_struct->cqe_coalesce_checksum_msix), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa_error:\n");
	connectib_hw_cqc_pa_error_print(&(ptr_struct->pa_error), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_page_size:\n");
	connectib_hw_cqc_log_page_size_print(&(ptr_struct->log_page_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "poll_cq_cqe_with_error_generated_on_this_queue:\n");
	connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue_print(&(ptr_struct->poll_cq_cqe_with_error_generated_on_this_queue), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "poll_cq_queue_in_error_state:\n");
	connectib_hw_cqc_poll_cq_queue_in_error_state_print(&(ptr_struct->poll_cq_queue_in_error_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_notified_check_for_arm_en:\n");
	connectib_hw_cqc_last_notified_check_for_arm_en_print(&(ptr_struct->last_notified_check_for_arm_en), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_solicited_index:\n");
	connectib_hw_cqc_last_solicited_index_print(&(ptr_struct->last_solicited_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lookahead_size_7_0:\n");
	connectib_hw_cqc_lookahead_size_7_0_print(&(ptr_struct->lookahead_size_7_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa0_63_32:\n");
	connectib_hw_cqc_pa0_63_32_print(&(ptr_struct->pa0_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_page_offset:\n");
	connectib_hw_cqc_cq_page_offset_print(&(ptr_struct->cq_page_offset), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_recovery_enable:\n");
	connectib_hw_cqc_db_recovery_enable_print(&(ptr_struct->db_recovery_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_heartbeat:\n");
	connectib_hw_cqc_cqe_heartbeat_print(&(ptr_struct->cqe_heartbeat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa_mode:\n");
	connectib_hw_cqc_pa_mode_print(&(ptr_struct->pa_mode), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "shrink:\n");
	connectib_hw_cqc_shrink_print(&(ptr_struct->shrink), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_trap:\n");
	connectib_hw_cqc_cq_trap_print(&(ptr_struct->cq_trap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa0_31_12:\n");
	connectib_hw_cqc_pa0_31_12_print(&(ptr_struct->pa0_31_12), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_cnt_limit_15_8:\n");
	connectib_hw_cqc_moderation_cnt_limit_15_8_print(&(ptr_struct->moderation_cnt_limit_15_8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_notified_index:\n");
	connectib_hw_cqc_last_notified_index_print(&(ptr_struct->last_notified_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "producer_index:\n");
	connectib_hw_cqc_producer_index_print(&(ptr_struct->producer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_cnt_limit_7_0:\n");
	connectib_hw_cqc_moderation_cnt_limit_7_0_print(&(ptr_struct->moderation_cnt_limit_7_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eqn_msix:\n");
	connectib_hw_cqc_eqn_msix_print(&(ptr_struct->eqn_msix), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_min_wait_time:\n");
	connectib_hw_cqc_moderation_min_wait_time_print(&(ptr_struct->moderation_min_wait_time), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_ci:\n");
	connectib_hw_cqc_db_ci_print(&(ptr_struct->db_ci), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_size:\n");
	connectib_hw_cqc_log2_size_print(&(ptr_struct->log2_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_heartbeat:\n");
	connectib_hw_cqc_db_heartbeat_print(&(ptr_struct->db_heartbeat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "number_of_events_generated:\n");
	connectib_hw_cqc_number_of_events_generated_print(&(ptr_struct->number_of_events_generated), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cipa_63_32:\n");
	connectib_hw_cqc_cipa_63_32_print(&(ptr_struct->cipa_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_39_37:\n");
	connectib_hw_cqc_mtt_pointer_39_37_print(&(ptr_struct->mtt_pointer_39_37), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cipa_31_3:\n");
	connectib_hw_cqc_cipa_31_3_print(&(ptr_struct->cipa_31_3), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_36_5:\n");
	connectib_hw_cqc_mtt_pointer_36_5_print(&(ptr_struct->mtt_pointer_36_5), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "consumer_index:\n");
	connectib_hw_cqc_consumer_index_print(&(ptr_struct->consumer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_state:\n");
	connectib_hw_cqc_cq_state_print(&(ptr_struct->cq_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "overrun_ignore:\n");
	connectib_hw_cqc_overrun_ignore_print(&(ptr_struct->overrun_ignore), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_4_0:\n");
	connectib_hw_cqc_mtt_pointer_4_0_print(&(ptr_struct->mtt_pointer_4_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa1_63_32:\n");
	connectib_hw_cqc_pa1_63_32_print(&(ptr_struct->pa1_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_timestamp_51_40:\n");
	connectib_hw_cqc_moderation_timestamp_51_40_print(&(ptr_struct->moderation_timestamp_51_40), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa1_31_12:\n");
	connectib_hw_cqc_pa1_31_12_print(&(ptr_struct->pa1_31_12), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_timestamp_39_8:\n");
	connectib_hw_cqc_moderation_timestamp_39_8_print(&(ptr_struct->moderation_timestamp_39_8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uar:\n");
	connectib_hw_cqc_uar_print(&(ptr_struct->uar), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_timestamp_7_0:\n");
	connectib_hw_cqc_moderation_timestamp_7_0_print(&(ptr_struct->moderation_timestamp_7_0), file, indent_level + 1);

}

int connectib_hw_c_eqc_size(void)
{
	 return 64;
}

void connectib_hw_c_eqc_dump(const union connectib_hw_c_eqc *ptr_struct, FILE* file)
{
	connectib_hw_c_eqc_print(ptr_struct, file, 0);
}

void connectib_hw_c_cqc_pack(const union connectib_hw_c_cqc *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_cqc_moderation_timestamp_7_0_pack(&(ptr_struct->moderation_timestamp_7_0), ptr_buff);
}

void connectib_hw_c_cqc_unpack(union connectib_hw_c_cqc *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_cqc_moderation_timestamp_7_0_unpack(&(ptr_struct->moderation_timestamp_7_0), ptr_buff);
}

void connectib_hw_c_cqc_print(const union connectib_hw_c_cqc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_c_cqc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lookahead_size_23_8:\n");
	connectib_hw_cqc_lookahead_size_23_8_print(&(ptr_struct->lookahead_size_23_8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqc_valid:\n");
	connectib_hw_cqc_cqc_valid_print(&(ptr_struct->cqc_valid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_128b_size:\n");
	connectib_hw_cqc_cqe_128b_size_print(&(ptr_struct->cqe_128b_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_coalesce_checksum_msix:\n");
	connectib_hw_cqc_cqe_coalesce_checksum_msix_print(&(ptr_struct->cqe_coalesce_checksum_msix), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa_error:\n");
	connectib_hw_cqc_pa_error_print(&(ptr_struct->pa_error), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_page_size:\n");
	connectib_hw_cqc_log_page_size_print(&(ptr_struct->log_page_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "poll_cq_cqe_with_error_generated_on_this_queue:\n");
	connectib_hw_cqc_poll_cq_cqe_with_error_generated_on_this_queue_print(&(ptr_struct->poll_cq_cqe_with_error_generated_on_this_queue), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "poll_cq_queue_in_error_state:\n");
	connectib_hw_cqc_poll_cq_queue_in_error_state_print(&(ptr_struct->poll_cq_queue_in_error_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_notified_check_for_arm_en:\n");
	connectib_hw_cqc_last_notified_check_for_arm_en_print(&(ptr_struct->last_notified_check_for_arm_en), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_solicited_index:\n");
	connectib_hw_cqc_last_solicited_index_print(&(ptr_struct->last_solicited_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lookahead_size_7_0:\n");
	connectib_hw_cqc_lookahead_size_7_0_print(&(ptr_struct->lookahead_size_7_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa0_63_32:\n");
	connectib_hw_cqc_pa0_63_32_print(&(ptr_struct->pa0_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_page_offset:\n");
	connectib_hw_cqc_cq_page_offset_print(&(ptr_struct->cq_page_offset), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_recovery_enable:\n");
	connectib_hw_cqc_db_recovery_enable_print(&(ptr_struct->db_recovery_enable), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_heartbeat:\n");
	connectib_hw_cqc_cqe_heartbeat_print(&(ptr_struct->cqe_heartbeat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa_mode:\n");
	connectib_hw_cqc_pa_mode_print(&(ptr_struct->pa_mode), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "shrink:\n");
	connectib_hw_cqc_shrink_print(&(ptr_struct->shrink), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_trap:\n");
	connectib_hw_cqc_cq_trap_print(&(ptr_struct->cq_trap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa0_31_12:\n");
	connectib_hw_cqc_pa0_31_12_print(&(ptr_struct->pa0_31_12), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_cnt_limit_15_8:\n");
	connectib_hw_cqc_moderation_cnt_limit_15_8_print(&(ptr_struct->moderation_cnt_limit_15_8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_notified_index:\n");
	connectib_hw_cqc_last_notified_index_print(&(ptr_struct->last_notified_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "producer_index:\n");
	connectib_hw_cqc_producer_index_print(&(ptr_struct->producer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_cnt_limit_7_0:\n");
	connectib_hw_cqc_moderation_cnt_limit_7_0_print(&(ptr_struct->moderation_cnt_limit_7_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eqn_msix:\n");
	connectib_hw_cqc_eqn_msix_print(&(ptr_struct->eqn_msix), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_min_wait_time:\n");
	connectib_hw_cqc_moderation_min_wait_time_print(&(ptr_struct->moderation_min_wait_time), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_ci:\n");
	connectib_hw_cqc_db_ci_print(&(ptr_struct->db_ci), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_size:\n");
	connectib_hw_cqc_log2_size_print(&(ptr_struct->log2_size), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_heartbeat:\n");
	connectib_hw_cqc_db_heartbeat_print(&(ptr_struct->db_heartbeat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "number_of_events_generated:\n");
	connectib_hw_cqc_number_of_events_generated_print(&(ptr_struct->number_of_events_generated), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cipa_63_32:\n");
	connectib_hw_cqc_cipa_63_32_print(&(ptr_struct->cipa_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_39_37:\n");
	connectib_hw_cqc_mtt_pointer_39_37_print(&(ptr_struct->mtt_pointer_39_37), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cipa_31_3:\n");
	connectib_hw_cqc_cipa_31_3_print(&(ptr_struct->cipa_31_3), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_36_5:\n");
	connectib_hw_cqc_mtt_pointer_36_5_print(&(ptr_struct->mtt_pointer_36_5), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "consumer_index:\n");
	connectib_hw_cqc_consumer_index_print(&(ptr_struct->consumer_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_state:\n");
	connectib_hw_cqc_cq_state_print(&(ptr_struct->cq_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "overrun_ignore:\n");
	connectib_hw_cqc_overrun_ignore_print(&(ptr_struct->overrun_ignore), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt_pointer_4_0:\n");
	connectib_hw_cqc_mtt_pointer_4_0_print(&(ptr_struct->mtt_pointer_4_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa1_63_32:\n");
	connectib_hw_cqc_pa1_63_32_print(&(ptr_struct->pa1_63_32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_timestamp_51_40:\n");
	connectib_hw_cqc_moderation_timestamp_51_40_print(&(ptr_struct->moderation_timestamp_51_40), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pa1_31_12:\n");
	connectib_hw_cqc_pa1_31_12_print(&(ptr_struct->pa1_31_12), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_timestamp_39_8:\n");
	connectib_hw_cqc_moderation_timestamp_39_8_print(&(ptr_struct->moderation_timestamp_39_8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uar:\n");
	connectib_hw_cqc_uar_print(&(ptr_struct->uar), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "moderation_timestamp_7_0:\n");
	connectib_hw_cqc_moderation_timestamp_7_0_print(&(ptr_struct->moderation_timestamp_7_0), file, indent_level + 1);

}

int connectib_hw_c_cqc_size(void)
{
	 return 64;
}

void connectib_hw_c_cqc_dump(const union connectib_hw_c_cqc *ptr_struct, FILE* file)
{
	connectib_hw_c_cqc_print(ptr_struct, file, 0);
}

void connectib_ffe_tap_set_pack(const struct connectib_ffe_tap_set *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap0);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap1);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap2);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap3);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ffe_tap4);

}

void connectib_ffe_tap_set_unpack(struct connectib_ffe_tap_set *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->ffe_tap0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->ffe_tap1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->ffe_tap2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->ffe_tap3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 56;
	ptr_struct->ffe_tap4 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_ffe_tap_set_print(const struct connectib_ffe_tap_set *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ffe_tap_set ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap0             : " UH_FMT "\n", ptr_struct->ffe_tap0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap1             : " UH_FMT "\n", ptr_struct->ffe_tap1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap2             : " UH_FMT "\n", ptr_struct->ffe_tap2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap3             : " UH_FMT "\n", ptr_struct->ffe_tap3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap4             : " UH_FMT "\n", ptr_struct->ffe_tap4);

}

int connectib_ffe_tap_set_size(void)
{
	 return 8;
}

void connectib_ffe_tap_set_dump(const struct connectib_ffe_tap_set *ptr_struct, FILE* file)
{
	connectib_ffe_tap_set_print(ptr_struct, file, 0);
}

void connectib_GEN3_TRANSMITTER_SETTING_pack(const struct connectib_GEN3_TRANSMITTER_SETTING *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->post);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reject_coeff);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->parity);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->main);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->pre);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->ec);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reset_eieos);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->preset);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->use_preset);

}

void connectib_GEN3_TRANSMITTER_SETTING_unpack(struct connectib_GEN3_TRANSMITTER_SETTING *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->post = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 25;
	ptr_struct->reject_coeff = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->parity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->main = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 10;
	ptr_struct->pre = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 6;
	ptr_struct->ec = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 5;
	ptr_struct->reset_eieos = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->preset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->use_preset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_GEN3_TRANSMITTER_SETTING_print(const struct connectib_GEN3_TRANSMITTER_SETTING *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_GEN3_TRANSMITTER_SETTING ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "post                 : " UH_FMT "\n", ptr_struct->post);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reject_coeff         : " UH_FMT "\n", ptr_struct->reject_coeff);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parity               : " UH_FMT "\n", ptr_struct->parity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "main                 : " UH_FMT "\n", ptr_struct->main);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pre                  : " UH_FMT "\n", ptr_struct->pre);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ec                   : " UH_FMT "\n", ptr_struct->ec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reset_eieos          : " UH_FMT "\n", ptr_struct->reset_eieos);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "preset               : " UH_FMT "\n", ptr_struct->preset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "use_preset           : " UH_FMT "\n", ptr_struct->use_preset);

}

int connectib_GEN3_TRANSMITTER_SETTING_size(void)
{
	 return 4;
}

void connectib_GEN3_TRANSMITTER_SETTING_dump(const struct connectib_GEN3_TRANSMITTER_SETTING *ptr_struct, FILE* file)
{
	connectib_GEN3_TRANSMITTER_SETTING_print(ptr_struct, file, 0);
}

void connectib_gpio_config_pack(const struct connectib_gpio_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mode0);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mode1);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->def_val);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pull_up);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pull_en);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->masked);

}

void connectib_gpio_config_unpack(struct connectib_gpio_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 7;
	ptr_struct->mode0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->mode1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->def_val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->pull_up = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->pull_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->masked = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_gpio_config_print(const struct connectib_gpio_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_gpio_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mode0                : " UH_FMT "\n", ptr_struct->mode0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mode1                : " UH_FMT "\n", ptr_struct->mode1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "def_val              : " UH_FMT "\n", ptr_struct->def_val);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pull_up              : " UH_FMT "\n", ptr_struct->pull_up);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pull_en              : " UH_FMT "\n", ptr_struct->pull_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "masked               : " UH_FMT "\n", ptr_struct->masked);

}

int connectib_gpio_config_size(void)
{
	 return 1;
}

void connectib_gpio_config_dump(const struct connectib_gpio_config *ptr_struct, FILE* file)
{
	connectib_gpio_config_print(ptr_struct, file, 0);
}

void connectib_gpio_function_pack(const struct connectib_gpio_function *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->mapping);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reverse_polarity);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->masked);

}

void connectib_gpio_function_unpack(struct connectib_gpio_function *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	ptr_struct->mapping = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 1;
	ptr_struct->reverse_polarity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->masked = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_gpio_function_print(const struct connectib_gpio_function *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_gpio_function ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mapping              : " UH_FMT "\n", ptr_struct->mapping);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reverse_polarity     : " UH_FMT "\n", ptr_struct->reverse_polarity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "masked               : " UH_FMT "\n", ptr_struct->masked);

}

int connectib_gpio_function_size(void)
{
	 return 1;
}

void connectib_gpio_function_dump(const struct connectib_gpio_function *ptr_struct, FILE* file)
{
	connectib_gpio_function_print(ptr_struct, file, 0);
}

void connectib_mlpn_enhanced_database_ini_pack(const struct connectib_mlpn_enhanced_database_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mlnx_enhanced_equal_info_ini_pack(&(ptr_struct->as_bits), ptr_buff + offset/8);

}

void connectib_mlpn_enhanced_database_ini_unpack(struct connectib_mlpn_enhanced_database_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mlnx_enhanced_equal_info_ini_unpack(&(ptr_struct->as_bits), ptr_buff + offset/8);

}

void connectib_mlpn_enhanced_database_ini_print(const struct connectib_mlpn_enhanced_database_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mlpn_enhanced_database_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "as_bits:\n");
	connectib_mlnx_enhanced_equal_info_ini_print(&(ptr_struct->as_bits), file, indent_level + 1);

}

int connectib_mlpn_enhanced_database_ini_size(void)
{
	 return 4;
}

void connectib_mlpn_enhanced_database_ini_dump(const struct connectib_mlpn_enhanced_database_ini *ptr_struct, FILE* file)
{
	connectib_mlpn_enhanced_database_ini_print(ptr_struct, file, 0);
}

void connectib_mlpn_database_all_protocol_ini_pack(const struct connectib_mlpn_database_all_protocol_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mlpn_database_all_protocol_bits_ini_pack(&(ptr_struct->as_bits), ptr_buff + offset/8);

}

void connectib_mlpn_database_all_protocol_ini_unpack(struct connectib_mlpn_database_all_protocol_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mlpn_database_all_protocol_bits_ini_unpack(&(ptr_struct->as_bits), ptr_buff + offset/8);

}

void connectib_mlpn_database_all_protocol_ini_print(const struct connectib_mlpn_database_all_protocol_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mlpn_database_all_protocol_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "as_bits:\n");
	connectib_mlpn_database_all_protocol_bits_ini_print(&(ptr_struct->as_bits), file, indent_level + 1);

}

int connectib_mlpn_database_all_protocol_ini_size(void)
{
	 return 4;
}

void connectib_mlpn_database_all_protocol_ini_dump(const struct connectib_mlpn_database_all_protocol_ini *ptr_struct, FILE* file)
{
	connectib_mlpn_database_all_protocol_ini_print(ptr_struct, file, 0);
}

void connectib_tx_rx_logic_serdes_mapping_node_ini_pack(const struct connectib_tx_rx_logic_serdes_mapping_node_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->logic_to_serdes[i]);
	}

}

void connectib_tx_rx_logic_serdes_mapping_node_ini_unpack(struct connectib_tx_rx_logic_serdes_mapping_node_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	ptr_struct->logic_to_serdes[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_tx_rx_logic_serdes_mapping_node_ini_print(const struct connectib_tx_rx_logic_serdes_mapping_node_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_tx_rx_logic_serdes_mapping_node_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "logic_to_serdes_%03d : " U32H_FMT "\n", i, ptr_struct->logic_to_serdes[i]);
	}

}

int connectib_tx_rx_logic_serdes_mapping_node_ini_size(void)
{
	 return 32;
}

void connectib_tx_rx_logic_serdes_mapping_node_ini_dump(const struct connectib_tx_rx_logic_serdes_mapping_node_ini *ptr_struct, FILE* file)
{
	connectib_tx_rx_logic_serdes_mapping_node_ini_print(ptr_struct, file, 0);
}

void connectib_serdes_conf_ini_pack(const struct connectib_serdes_conf_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_autoneg_sets_ini_pack(&(ptr_struct->nego), ptr_buff + offset/8);

	offset = 3072;
	connectib_serdes_aba_sets_ini_pack(&(ptr_struct->aba), ptr_buff + offset/8);

	offset = 6144;
	connectib_serdes_force_sets_ini_pack(&(ptr_struct->force), ptr_buff + offset/8);

	offset = 7104;
	connectib_serdes_active_sets_ini_pack(&(ptr_struct->active), ptr_buff + offset/8);

	offset = 7488;
	connectib_serdes_preset_sets_ini_pack(&(ptr_struct->preset_tx_fdr), ptr_buff + offset/8);

	offset = 8000;
	connectib_serdes_preset_sets_ini_pack(&(ptr_struct->preset_tx_qdr), ptr_buff + offset/8);

}

void connectib_serdes_conf_ini_unpack(struct connectib_serdes_conf_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_autoneg_sets_ini_unpack(&(ptr_struct->nego), ptr_buff + offset/8);

	offset = 3072;
	connectib_serdes_aba_sets_ini_unpack(&(ptr_struct->aba), ptr_buff + offset/8);

	offset = 6144;
	connectib_serdes_force_sets_ini_unpack(&(ptr_struct->force), ptr_buff + offset/8);

	offset = 7104;
	connectib_serdes_active_sets_ini_unpack(&(ptr_struct->active), ptr_buff + offset/8);

	offset = 7488;
	connectib_serdes_preset_sets_ini_unpack(&(ptr_struct->preset_tx_fdr), ptr_buff + offset/8);

	offset = 8000;
	connectib_serdes_preset_sets_ini_unpack(&(ptr_struct->preset_tx_qdr), ptr_buff + offset/8);

}

void connectib_serdes_conf_ini_print(const struct connectib_serdes_conf_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_serdes_conf_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nego:\n");
	connectib_serdes_autoneg_sets_ini_print(&(ptr_struct->nego), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aba:\n");
	connectib_serdes_aba_sets_ini_print(&(ptr_struct->aba), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force:\n");
	connectib_serdes_force_sets_ini_print(&(ptr_struct->force), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "active:\n");
	connectib_serdes_active_sets_ini_print(&(ptr_struct->active), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "preset_tx_fdr:\n");
	connectib_serdes_preset_sets_ini_print(&(ptr_struct->preset_tx_fdr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "preset_tx_qdr:\n");
	connectib_serdes_preset_sets_ini_print(&(ptr_struct->preset_tx_qdr), file, indent_level + 1);

}

int connectib_serdes_conf_ini_size(void)
{
	 return 1088;
}

void connectib_serdes_conf_ini_dump(const struct connectib_serdes_conf_ini *ptr_struct, FILE* file)
{
	connectib_serdes_conf_ini_print(ptr_struct, file, 0);
}

void connectib_rx_adapt_ini_pack(const struct connectib_rx_adapt_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->max_num_ffe_search);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->num_of_ffe_taps);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->num_of_ffe_hw_steps);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_hw_ffe_before_open_eye);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->skip_ffe_srch);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->skip_rx_srch);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->offset_eye_opener_range);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mixer_offset_grade_margin);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->jump_size_ffe_level1);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->jump_size_ffe_level2);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->jump_size_ffe_level3);

	offset = 47;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->delta_from_edges_phase);

	offset = 41;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->delta_from_edges_offst);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->offset_penalty_en);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->offset_penalty_weight);

	offset = 95;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->center_mix90phase);

	offset = 94;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_calib_in_QDR_rev0);

	offset = 93;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->center_mixer_offset);

	offset = 92;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->grade_mixer_margin_en);

	offset = 87;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->eye_open_machine_measure_time);

	offset = 82;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->eye_open_machine_stab_time);

	offset = 74;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->best_grade_th);

	offset = 69;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->eye_open_machine_stab_time_extended);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->eye_phase_diff_min);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->eye_offset_diff_min);

	offset = 111;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->min_eye_en);

	offset = 110;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->grade_eye_from_center);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->offset_zero_grade_threshold);

	offset = 99;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->eye_open_machine_measure_time_in_stab_extend);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->offset_max_grade);

	offset = 147;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->phase_limit);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->rx_adaptation_reattempts);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->phase_weight);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->offset_weight);

	offset = 184;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->rx_taps_norm_val);

}

void connectib_rx_adapt_ini_unpack(struct connectib_rx_adapt_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->max_num_ffe_search = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 23;
	ptr_struct->num_of_ffe_taps = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 17;
	ptr_struct->num_of_ffe_hw_steps = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 13;
	ptr_struct->num_hw_ffe_before_open_eye = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 12;
	ptr_struct->skip_ffe_srch = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->skip_rx_srch = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->offset_eye_opener_range = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 0;
	ptr_struct->mixer_offset_grade_margin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 60;
	ptr_struct->jump_size_ffe_level1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 56;
	ptr_struct->jump_size_ffe_level2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 52;
	ptr_struct->jump_size_ffe_level3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 47;
	ptr_struct->delta_from_edges_phase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 41;
	ptr_struct->delta_from_edges_offst = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 40;
	ptr_struct->offset_penalty_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->offset_penalty_weight = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 95;
	ptr_struct->center_mix90phase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 94;
	ptr_struct->disable_calib_in_QDR_rev0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 93;
	ptr_struct->center_mixer_offset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 92;
	ptr_struct->grade_mixer_margin_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 87;
	ptr_struct->eye_open_machine_measure_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 82;
	ptr_struct->eye_open_machine_stab_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 74;
	ptr_struct->best_grade_th = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 69;
	ptr_struct->eye_open_machine_stab_time_extended = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 120;
	ptr_struct->eye_phase_diff_min = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 112;
	ptr_struct->eye_offset_diff_min = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 111;
	ptr_struct->min_eye_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 110;
	ptr_struct->grade_eye_from_center = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 104;
	ptr_struct->offset_zero_grade_threshold = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 99;
	ptr_struct->eye_open_machine_measure_time_in_stab_extend = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 152;
	ptr_struct->offset_max_grade = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 147;
	ptr_struct->phase_limit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 144;
	ptr_struct->rx_adaptation_reattempts = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 136;
	ptr_struct->phase_weight = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 128;
	ptr_struct->offset_weight = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 184;
	ptr_struct->rx_taps_norm_val = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_rx_adapt_ini_print(const struct connectib_rx_adapt_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rx_adapt_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_ffe_search   : " UH_FMT "\n", ptr_struct->max_num_ffe_search);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_ffe_taps      : " UH_FMT "\n", ptr_struct->num_of_ffe_taps);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_ffe_hw_steps  : " UH_FMT "\n", ptr_struct->num_of_ffe_hw_steps);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_hw_ffe_before_open_eye : " UH_FMT "\n", ptr_struct->num_hw_ffe_before_open_eye);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "skip_ffe_srch        : " UH_FMT "\n", ptr_struct->skip_ffe_srch);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "skip_rx_srch         : " UH_FMT "\n", ptr_struct->skip_rx_srch);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset_eye_opener_range : " UH_FMT "\n", ptr_struct->offset_eye_opener_range);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mixer_offset_grade_margin : " UH_FMT "\n", ptr_struct->mixer_offset_grade_margin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "jump_size_ffe_level1 : " UH_FMT "\n", ptr_struct->jump_size_ffe_level1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "jump_size_ffe_level2 : " UH_FMT "\n", ptr_struct->jump_size_ffe_level2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "jump_size_ffe_level3 : " UH_FMT "\n", ptr_struct->jump_size_ffe_level3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "delta_from_edges_phase : " UH_FMT "\n", ptr_struct->delta_from_edges_phase);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "delta_from_edges_offst : " UH_FMT "\n", ptr_struct->delta_from_edges_offst);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset_penalty_en    : " UH_FMT "\n", ptr_struct->offset_penalty_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset_penalty_weight : " UH_FMT "\n", ptr_struct->offset_penalty_weight);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "center_mix90phase    : " UH_FMT "\n", ptr_struct->center_mix90phase);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_calib_in_QDR_rev0 : " UH_FMT "\n", ptr_struct->disable_calib_in_QDR_rev0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "center_mixer_offset  : " UH_FMT "\n", ptr_struct->center_mixer_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grade_mixer_margin_en : " UH_FMT "\n", ptr_struct->grade_mixer_margin_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_open_machine_measure_time : " UH_FMT "\n", ptr_struct->eye_open_machine_measure_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_open_machine_stab_time : " UH_FMT "\n", ptr_struct->eye_open_machine_stab_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "best_grade_th        : " UH_FMT "\n", ptr_struct->best_grade_th);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_open_machine_stab_time_extended : " UH_FMT "\n", ptr_struct->eye_open_machine_stab_time_extended);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_phase_diff_min   : " UH_FMT "\n", ptr_struct->eye_phase_diff_min);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_offset_diff_min  : " UH_FMT "\n", ptr_struct->eye_offset_diff_min);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "min_eye_en           : " UH_FMT "\n", ptr_struct->min_eye_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grade_eye_from_center : " UH_FMT "\n", ptr_struct->grade_eye_from_center);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset_zero_grade_threshold : " UH_FMT "\n", ptr_struct->offset_zero_grade_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_open_machine_measure_time_in_stab_extend : " UH_FMT "\n", ptr_struct->eye_open_machine_measure_time_in_stab_extend);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset_max_grade     : " UH_FMT "\n", ptr_struct->offset_max_grade);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phase_limit          : " UH_FMT "\n", ptr_struct->phase_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_adaptation_reattempts : " UH_FMT "\n", ptr_struct->rx_adaptation_reattempts);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phase_weight         : " UH_FMT "\n", ptr_struct->phase_weight);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset_weight        : " UH_FMT "\n", ptr_struct->offset_weight);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_taps_norm_val     : " UH_FMT "\n", ptr_struct->rx_taps_norm_val);

}

int connectib_rx_adapt_ini_size(void)
{
	 return 24;
}

void connectib_rx_adapt_ini_dump(const struct connectib_rx_adapt_ini *ptr_struct, FILE* file)
{
	connectib_rx_adapt_ini_print(ptr_struct, file, 0);
}

void connectib_ib_ini_ini_pack(const struct connectib_ib_ini_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->llr_wait_time);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->spec1_3_ignore_peer_tx_req);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qdr_tx_aggressor_en);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->equal_req_en);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->ts3_fcu);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ts3_hbr);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ts3_sct);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ts3_tp);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ts3_mpr);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ts3_ft);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->min_ib_grade);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_polarity_port0);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_polarity_port1);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->Mloop_en_mask);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 768, 1);
	connectib_ib_ini_per_lcl_ini_pack(&(ptr_struct->lcl[i]), ptr_buff + offset/8);
	}

	offset = 121;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->retry_counter_threshold);

	offset = 113;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->time_2_clear_err_after_linkup);

	offset = 128;
	connectib_spec1_3_preset_ini_pack(&(ptr_struct->spec1_3_preset), ptr_buff + offset/8);

	offset = 767;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_llr_on_long_optic);

	offset = 760;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->llr_max_cable_length);

	offset = 759;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->terminate_pkt_timeout);

	offset = 758;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_rx_check_en);

	offset = 757;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable_fec_on_long_optic);

	offset = 750;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->fec_min_cable_length);

}

void connectib_ib_ini_ini_unpack(struct connectib_ib_ini_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->llr_wait_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 25;
	ptr_struct->spec1_3_ignore_peer_tx_req = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->qdr_tx_aggressor_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->equal_req_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->ts3_fcu = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 20;
	ptr_struct->ts3_hbr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->ts3_sct = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->ts3_tp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->ts3_mpr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->ts3_ft = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 11;
	ptr_struct->min_ib_grade = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 7;
	ptr_struct->tx_polarity_port0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 3;
	ptr_struct->tx_polarity_port1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 1;
	ptr_struct->Mloop_en_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 768, 1);
	connectib_ib_ini_per_lcl_ini_unpack(&(ptr_struct->lcl[i]), ptr_buff + offset/8);
	}

	offset = 121;
	ptr_struct->retry_counter_threshold = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 113;
	ptr_struct->time_2_clear_err_after_linkup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 128;
	connectib_spec1_3_preset_ini_unpack(&(ptr_struct->spec1_3_preset), ptr_buff + offset/8);

	offset = 767;
	ptr_struct->disable_llr_on_long_optic = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 760;
	ptr_struct->llr_max_cable_length = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 759;
	ptr_struct->terminate_pkt_timeout = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 758;
	ptr_struct->llr_rx_check_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 757;
	ptr_struct->enable_fec_on_long_optic = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 750;
	ptr_struct->fec_min_cable_length = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

}

void connectib_ib_ini_ini_print(const struct connectib_ib_ini_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ib_ini_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_wait_time        : " UH_FMT "\n", ptr_struct->llr_wait_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spec1_3_ignore_peer_tx_req : " UH_FMT "\n", ptr_struct->spec1_3_ignore_peer_tx_req);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qdr_tx_aggressor_en  : " UH_FMT "\n", ptr_struct->qdr_tx_aggressor_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "equal_req_en         : " UH_FMT "\n", ptr_struct->equal_req_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts3_fcu              : " UH_FMT "\n", ptr_struct->ts3_fcu);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts3_hbr              : " UH_FMT "\n", ptr_struct->ts3_hbr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts3_sct              : " UH_FMT "\n", ptr_struct->ts3_sct);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts3_tp               : " UH_FMT "\n", ptr_struct->ts3_tp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts3_mpr              : " UH_FMT "\n", ptr_struct->ts3_mpr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ts3_ft               : " UH_FMT "\n", ptr_struct->ts3_ft);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "min_ib_grade         : " UH_FMT "\n", ptr_struct->min_ib_grade);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_polarity_port0    : " UH_FMT "\n", ptr_struct->tx_polarity_port0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_polarity_port1    : " UH_FMT "\n", ptr_struct->tx_polarity_port1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Mloop_en_mask        : " UH_FMT "\n", ptr_struct->Mloop_en_mask);

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lcl_%03d:\n", i);
	connectib_ib_ini_per_lcl_ini_print(&(ptr_struct->lcl[i]), file, indent_level + 1);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "retry_counter_threshold : " UH_FMT "\n", ptr_struct->retry_counter_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "time_2_clear_err_after_linkup : " UH_FMT "\n", ptr_struct->time_2_clear_err_after_linkup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spec1_3_preset:\n");
	connectib_spec1_3_preset_ini_print(&(ptr_struct->spec1_3_preset), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_llr_on_long_optic : " UH_FMT "\n", ptr_struct->disable_llr_on_long_optic);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_max_cable_length : " UH_FMT "\n", ptr_struct->llr_max_cable_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "terminate_pkt_timeout : " UH_FMT "\n", ptr_struct->terminate_pkt_timeout);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_rx_check_en      : " UH_FMT "\n", ptr_struct->llr_rx_check_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable_fec_on_long_optic : " UH_FMT "\n", ptr_struct->enable_fec_on_long_optic);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fec_min_cable_length : " UH_FMT "\n", ptr_struct->fec_min_cable_length);

}

int connectib_ib_ini_ini_size(void)
{
	 return 96;
}

void connectib_ib_ini_ini_dump(const struct connectib_ib_ini_ini *ptr_struct, FILE* file)
{
	connectib_ib_ini_ini_print(ptr_struct, file, 0);
}

void connectib_PORT_PHY_ini_pack(const struct connectib_PORT_PHY_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_param_mask_ini_pack(&(ptr_struct->serdes_param_mask), ptr_buff + offset/8);

}

void connectib_PORT_PHY_ini_unpack(struct connectib_PORT_PHY_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_serdes_param_mask_ini_unpack(&(ptr_struct->serdes_param_mask), ptr_buff + offset/8);

}

void connectib_PORT_PHY_ini_print(const struct connectib_PORT_PHY_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_PORT_PHY_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serdes_param_mask:\n");
	connectib_serdes_param_mask_ini_print(&(ptr_struct->serdes_param_mask), file, indent_level + 1);

}

int connectib_PORT_PHY_ini_size(void)
{
	 return 16;
}

void connectib_PORT_PHY_ini_dump(const struct connectib_PORT_PHY_ini *ptr_struct, FILE* file)
{
	connectib_PORT_PHY_ini_print(ptr_struct, file, 0);
}

void connectib_nv_tlv_data_vpi_settings_pack(const struct connectib_nv_tlv_data_vpi_settings *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->network_link_type_ib);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->network_link_type_eth);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->default_link_type_ib);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->default_link_type_eth);

}

void connectib_nv_tlv_data_vpi_settings_unpack(struct connectib_nv_tlv_data_vpi_settings *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->network_link_type_ib = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->network_link_type_eth = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->default_link_type_ib = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->default_link_type_eth = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_nv_tlv_data_vpi_settings_print(const struct connectib_nv_tlv_data_vpi_settings *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_tlv_data_vpi_settings ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "network_link_type_ib : " UH_FMT "\n", ptr_struct->network_link_type_ib);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "network_link_type_eth : " UH_FMT "\n", ptr_struct->network_link_type_eth);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "default_link_type_ib : " UH_FMT "\n", ptr_struct->default_link_type_ib);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "default_link_type_eth : " UH_FMT "\n", ptr_struct->default_link_type_eth);

}

int connectib_nv_tlv_data_vpi_settings_size(void)
{
	 return 4;
}

void connectib_nv_tlv_data_vpi_settings_dump(const struct connectib_nv_tlv_data_vpi_settings *ptr_struct, FILE* file)
{
	connectib_nv_tlv_data_vpi_settings_print(ptr_struct, file, 0);
}

void connectib_nv_config_kdnet_data_pack(const struct connectib_nv_config_kdnet_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->kdnet_en);

}

void connectib_nv_config_kdnet_data_unpack(struct connectib_nv_config_kdnet_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->kdnet_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_nv_config_kdnet_data_print(const struct connectib_nv_config_kdnet_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_kdnet_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "kdnet_en             : " UH_FMT "\n", ptr_struct->kdnet_en);

}

int connectib_nv_config_kdnet_data_size(void)
{
	 return 4;
}

void connectib_nv_config_kdnet_data_dump(const struct connectib_nv_config_kdnet_data *ptr_struct, FILE* file)
{
	connectib_nv_config_kdnet_data_print(ptr_struct, file, 0);
}

void connectib_nv_config_tpt_pack(const struct connectib_nv_config_tpt *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_tpt_conf_pack(&(ptr_struct->settings), ptr_buff + offset/8);

	offset = 32;
	connectib_nv_tlv_data_tpt_capabilities_pack(&(ptr_struct->capabilities), ptr_buff + offset/8);

}

void connectib_nv_config_tpt_unpack(struct connectib_nv_config_tpt *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_tpt_conf_unpack(&(ptr_struct->settings), ptr_buff + offset/8);

	offset = 32;
	connectib_nv_tlv_data_tpt_capabilities_unpack(&(ptr_struct->capabilities), ptr_buff + offset/8);

}

void connectib_nv_config_tpt_print(const struct connectib_nv_config_tpt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_tpt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "settings:\n");
	connectib_nv_tpt_conf_print(&(ptr_struct->settings), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capabilities:\n");
	connectib_nv_tlv_data_tpt_capabilities_print(&(ptr_struct->capabilities), file, indent_level + 1);

}

int connectib_nv_config_tpt_size(void)
{
	 return 8;
}

void connectib_nv_config_tpt_dump(const struct connectib_nv_config_tpt *ptr_struct, FILE* file)
{
	connectib_nv_config_tpt_print(ptr_struct, file, 0);
}

void connectib_nv_config_pci_pack(const struct connectib_nv_config_pci *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_global_pci_conf_pack(&(ptr_struct->settings), ptr_buff + offset/8);

	offset = 96;
	connectib_nv_global_pci_cap_pack(&(ptr_struct->capabilities), ptr_buff + offset/8);

}

void connectib_nv_config_pci_unpack(struct connectib_nv_config_pci *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_global_pci_conf_unpack(&(ptr_struct->settings), ptr_buff + offset/8);

	offset = 96;
	connectib_nv_global_pci_cap_unpack(&(ptr_struct->capabilities), ptr_buff + offset/8);

}

void connectib_nv_config_pci_print(const struct connectib_nv_config_pci *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_pci ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "settings:\n");
	connectib_nv_global_pci_conf_print(&(ptr_struct->settings), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capabilities:\n");
	connectib_nv_global_pci_cap_print(&(ptr_struct->capabilities), file, indent_level + 1);

}

int connectib_nv_config_pci_size(void)
{
	 return 32;
}

void connectib_nv_config_pci_dump(const struct connectib_nv_config_pci *ptr_struct, FILE* file)
{
	connectib_nv_config_pci_print(ptr_struct, file, 0);
}

void connectib_acs_ext_cap_pack(const struct connectib_acs_ext_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->cap_id);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 32;
	connectib_pcie_acs_ctrl_fields_pack(&(ptr_struct->acs_ctrl_fields), ptr_buff + offset/8);

}

void connectib_acs_ext_cap_unpack(struct connectib_acs_ext_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->cap_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 12;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->nxt_ptr = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 32;
	connectib_pcie_acs_ctrl_fields_unpack(&(ptr_struct->acs_ctrl_fields), ptr_buff + offset/8);

}

void connectib_acs_ext_cap_print(const struct connectib_acs_ext_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_acs_ext_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_id               : " UH_FMT "\n", ptr_struct->cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs_ctrl_fields:\n");
	connectib_pcie_acs_ctrl_fields_print(&(ptr_struct->acs_ctrl_fields), file, indent_level + 1);

}

int connectib_acs_ext_cap_size(void)
{
	 return 8;
}

void connectib_acs_ext_cap_dump(const struct connectib_acs_ext_cap *ptr_struct, FILE* file)
{
	connectib_acs_ext_cap_print(ptr_struct, file, 0);
}

void connectib_advanced_error_reporting_pack(const struct connectib_advanced_error_reporting *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->cap_id);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 32;
	connectib_uncorrectable_error_status_pack(&(ptr_struct->uncorrectable_error_status), ptr_buff + offset/8);

	offset = 64;
	connectib_uncorrectable_error_mask_pack(&(ptr_struct->uncorrectable_error_mask), ptr_buff + offset/8);

	offset = 96;
	connectib_uncorrectable_error_severity_pack(&(ptr_struct->uncorrectable_error_severity), ptr_buff + offset/8);

	offset = 128;
	connectib_correctable_error_status_pack(&(ptr_struct->correctable_error_status), ptr_buff + offset/8);

	offset = 160;
	connectib_correctable_error_mask_pack(&(ptr_struct->correctable_error_mask), ptr_buff + offset/8);

	offset = 192;
	connectib_advanced_error_cap_ctl_pack(&(ptr_struct->advanced_error_cap_ctl), ptr_buff + offset/8);

	offset = 224;
	connectib_header_log_pack(&(ptr_struct->header_log), ptr_buff + offset/8);

}

void connectib_advanced_error_reporting_unpack(struct connectib_advanced_error_reporting *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->cap_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 12;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->nxt_ptr = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 32;
	connectib_uncorrectable_error_status_unpack(&(ptr_struct->uncorrectable_error_status), ptr_buff + offset/8);

	offset = 64;
	connectib_uncorrectable_error_mask_unpack(&(ptr_struct->uncorrectable_error_mask), ptr_buff + offset/8);

	offset = 96;
	connectib_uncorrectable_error_severity_unpack(&(ptr_struct->uncorrectable_error_severity), ptr_buff + offset/8);

	offset = 128;
	connectib_correctable_error_status_unpack(&(ptr_struct->correctable_error_status), ptr_buff + offset/8);

	offset = 160;
	connectib_correctable_error_mask_unpack(&(ptr_struct->correctable_error_mask), ptr_buff + offset/8);

	offset = 192;
	connectib_advanced_error_cap_ctl_unpack(&(ptr_struct->advanced_error_cap_ctl), ptr_buff + offset/8);

	offset = 224;
	connectib_header_log_unpack(&(ptr_struct->header_log), ptr_buff + offset/8);

}

void connectib_advanced_error_reporting_print(const struct connectib_advanced_error_reporting *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_advanced_error_reporting ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_id               : " UH_FMT "\n", ptr_struct->cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uncorrectable_error_status:\n");
	connectib_uncorrectable_error_status_print(&(ptr_struct->uncorrectable_error_status), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uncorrectable_error_mask:\n");
	connectib_uncorrectable_error_mask_print(&(ptr_struct->uncorrectable_error_mask), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uncorrectable_error_severity:\n");
	connectib_uncorrectable_error_severity_print(&(ptr_struct->uncorrectable_error_severity), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "correctable_error_status:\n");
	connectib_correctable_error_status_print(&(ptr_struct->correctable_error_status), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "correctable_error_mask:\n");
	connectib_correctable_error_mask_print(&(ptr_struct->correctable_error_mask), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "advanced_error_cap_ctl:\n");
	connectib_advanced_error_cap_ctl_print(&(ptr_struct->advanced_error_cap_ctl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "header_log:\n");
	connectib_header_log_print(&(ptr_struct->header_log), file, indent_level + 1);

}

int connectib_advanced_error_reporting_size(void)
{
	 return 56;
}

void connectib_advanced_error_reporting_dump(const struct connectib_advanced_error_reporting *ptr_struct, FILE* file)
{
	connectib_advanced_error_reporting_print(ptr_struct, file, 0);
}

void connectib_dev_serial_num_cap_pack(const struct connectib_dev_serial_num_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->cap_id);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->serial_number_31_0);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->serial_number_63_32);

}

void connectib_dev_serial_num_cap_unpack(struct connectib_dev_serial_num_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->cap_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 12;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->nxt_ptr = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 32;
	ptr_struct->serial_number_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->serial_number_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_dev_serial_num_cap_print(const struct connectib_dev_serial_num_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dev_serial_num_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_id               : " UH_FMT "\n", ptr_struct->cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serial_number_31_0   : " U32H_FMT "\n", ptr_struct->serial_number_31_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serial_number_63_32  : " U32H_FMT "\n", ptr_struct->serial_number_63_32);

}

int connectib_dev_serial_num_cap_size(void)
{
	 return 12;
}

void connectib_dev_serial_num_cap_dump(const struct connectib_dev_serial_num_cap *ptr_struct, FILE* file)
{
	connectib_dev_serial_num_cap_print(ptr_struct, file, 0);
}

void connectib_sriov_cap_pack(const struct connectib_sriov_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->cap_id);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 32;
	connectib_sriov_capabilities_pack(&(ptr_struct->capabilities), ptr_buff + offset/8);

	offset = 80;
	connectib_sriov_control_pack(&(ptr_struct->ctrl), ptr_buff + offset/8);

	offset = 64;
	connectib_sriov_status_pack(&(ptr_struct->status), ptr_buff + offset/8);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->initial_vfs);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->total_vfs);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->num_vfs);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->func_dependency_link);

	offset = 176;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->first_vf_offset);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vf_stride);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vf_device_id);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->supported_page_sizes);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->system_page_size);

	offset = 288;
	connectib_bar_low_pack(&(ptr_struct->vf_bar0), ptr_buff + offset/8);

	offset = 320;
	connectib_bar_high_pack(&(ptr_struct->vf_bar1), ptr_buff + offset/8);

	offset = 352;
	connectib_bar_low_pack(&(ptr_struct->vf_bar2), ptr_buff + offset/8);

	offset = 384;
	connectib_bar_high_pack(&(ptr_struct->vf_bar3), ptr_buff + offset/8);

	offset = 416;
	connectib_bar_low_pack(&(ptr_struct->vf_bar4), ptr_buff + offset/8);

	offset = 448;
	connectib_bar_high_pack(&(ptr_struct->vf_bar5), ptr_buff + offset/8);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vf_migstate_arrayofst);

}

void connectib_sriov_cap_unpack(struct connectib_sriov_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->cap_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 12;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->nxt_ptr = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 32;
	connectib_sriov_capabilities_unpack(&(ptr_struct->capabilities), ptr_buff + offset/8);

	offset = 80;
	connectib_sriov_control_unpack(&(ptr_struct->ctrl), ptr_buff + offset/8);

	offset = 64;
	connectib_sriov_status_unpack(&(ptr_struct->status), ptr_buff + offset/8);

	offset = 112;
	ptr_struct->initial_vfs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->total_vfs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 144;
	ptr_struct->num_vfs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 136;
	ptr_struct->func_dependency_link = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 176;
	ptr_struct->first_vf_offset = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 160;
	ptr_struct->vf_stride = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 192;
	ptr_struct->vf_device_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 224;
	ptr_struct->supported_page_sizes = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->system_page_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	connectib_bar_low_unpack(&(ptr_struct->vf_bar0), ptr_buff + offset/8);

	offset = 320;
	connectib_bar_high_unpack(&(ptr_struct->vf_bar1), ptr_buff + offset/8);

	offset = 352;
	connectib_bar_low_unpack(&(ptr_struct->vf_bar2), ptr_buff + offset/8);

	offset = 384;
	connectib_bar_high_unpack(&(ptr_struct->vf_bar3), ptr_buff + offset/8);

	offset = 416;
	connectib_bar_low_unpack(&(ptr_struct->vf_bar4), ptr_buff + offset/8);

	offset = 448;
	connectib_bar_high_unpack(&(ptr_struct->vf_bar5), ptr_buff + offset/8);

	offset = 480;
	ptr_struct->vf_migstate_arrayofst = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_sriov_cap_print(const struct connectib_sriov_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sriov_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_id               : " UH_FMT "\n", ptr_struct->cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capabilities:\n");
	connectib_sriov_capabilities_print(&(ptr_struct->capabilities), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctrl:\n");
	connectib_sriov_control_print(&(ptr_struct->ctrl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status:\n");
	connectib_sriov_status_print(&(ptr_struct->status), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "initial_vfs          : " UH_FMT "\n", ptr_struct->initial_vfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "total_vfs            : " UH_FMT "\n", ptr_struct->total_vfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_vfs              : " UH_FMT "\n", ptr_struct->num_vfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "func_dependency_link : " UH_FMT "\n", ptr_struct->func_dependency_link);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_vf_offset      : " UH_FMT "\n", ptr_struct->first_vf_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_stride            : " UH_FMT "\n", ptr_struct->vf_stride);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_device_id         : " UH_FMT "\n", ptr_struct->vf_device_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "supported_page_sizes : " U32H_FMT "\n", ptr_struct->supported_page_sizes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "system_page_size     : " U32H_FMT "\n", ptr_struct->system_page_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_bar0:\n");
	connectib_bar_low_print(&(ptr_struct->vf_bar0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_bar1:\n");
	connectib_bar_high_print(&(ptr_struct->vf_bar1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_bar2:\n");
	connectib_bar_low_print(&(ptr_struct->vf_bar2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_bar3:\n");
	connectib_bar_high_print(&(ptr_struct->vf_bar3), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_bar4:\n");
	connectib_bar_low_print(&(ptr_struct->vf_bar4), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_bar5:\n");
	connectib_bar_high_print(&(ptr_struct->vf_bar5), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_migstate_arrayofst : " U32H_FMT "\n", ptr_struct->vf_migstate_arrayofst);

}

int connectib_sriov_cap_size(void)
{
	 return 64;
}

void connectib_sriov_cap_dump(const struct connectib_sriov_cap *ptr_struct, FILE* file)
{
	connectib_sriov_cap_print(ptr_struct, file, 0);
}

void connectib_ari_cap_pack(const struct connectib_ari_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->cap_id);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 48;
	connectib_ari_capabilities_pack(&(ptr_struct->ari_capabilities), ptr_buff + offset/8);

	offset = 32;
	connectib_ari_control_pack(&(ptr_struct->ari_control), ptr_buff + offset/8);

}

void connectib_ari_cap_unpack(struct connectib_ari_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->cap_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 12;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->nxt_ptr = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 48;
	connectib_ari_capabilities_unpack(&(ptr_struct->ari_capabilities), ptr_buff + offset/8);

	offset = 32;
	connectib_ari_control_unpack(&(ptr_struct->ari_control), ptr_buff + offset/8);

}

void connectib_ari_cap_print(const struct connectib_ari_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ari_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_id               : " UH_FMT "\n", ptr_struct->cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ari_capabilities:\n");
	connectib_ari_capabilities_print(&(ptr_struct->ari_capabilities), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ari_control:\n");
	connectib_ari_control_print(&(ptr_struct->ari_control), file, indent_level + 1);

}

int connectib_ari_cap_size(void)
{
	 return 8;
}

void connectib_ari_cap_dump(const struct connectib_ari_cap *ptr_struct, FILE* file)
{
	connectib_ari_cap_print(ptr_struct, file, 0);
}

void connectib_secondary_pciex_ext_cap_pack(const struct connectib_secondary_pciex_ext_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->cap_id);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->version);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 32;
	connectib_pciex_link_ctl3_pack(&(ptr_struct->pciex_link_ctl3), ptr_buff + offset/8);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lane_err_bits);

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(96, 32, i, 352, 1);
	connectib_lane_eql_ctl_pack(&(ptr_struct->lane_eql_ctl[i]), ptr_buff + offset/8);
	}

}

void connectib_secondary_pciex_ext_cap_unpack(struct connectib_secondary_pciex_ext_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->cap_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 12;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->nxt_ptr = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 32;
	connectib_pciex_link_ctl3_unpack(&(ptr_struct->pciex_link_ctl3), ptr_buff + offset/8);

	offset = 80;
	ptr_struct->lane_err_bits = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(96, 32, i, 352, 1);
	connectib_lane_eql_ctl_unpack(&(ptr_struct->lane_eql_ctl[i]), ptr_buff + offset/8);
	}

}

void connectib_secondary_pciex_ext_cap_print(const struct connectib_secondary_pciex_ext_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_secondary_pciex_ext_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_id               : " UH_FMT "\n", ptr_struct->cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pciex_link_ctl3:\n");
	connectib_pciex_link_ctl3_print(&(ptr_struct->pciex_link_ctl3), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane_err_bits        : " UH_FMT "\n", ptr_struct->lane_err_bits);

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane_eql_ctl_%03d:\n", i);
	connectib_lane_eql_ctl_print(&(ptr_struct->lane_eql_ctl[i]), file, indent_level + 1);
	}

}

int connectib_secondary_pciex_ext_cap_size(void)
{
	 return 44;
}

void connectib_secondary_pciex_ext_cap_dump(const struct connectib_secondary_pciex_ext_cap *ptr_struct, FILE* file)
{
	connectib_secondary_pciex_ext_cap_print(ptr_struct, file, 0);
}

void connectib_pci_cfg_type1_header_pack(const struct connectib_pci_cfg_type1_header *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vendor_id);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->device_id);

	offset = 48;
	connectib_cmd_reg1_pack(&(ptr_struct->command), ptr_buff + offset/8);

	offset = 32;
	connectib_stat_reg1_pack(&(ptr_struct->status), ptr_buff + offset/8);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->revision_id);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->class_code);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cache_line);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->latency);

	offset = 104;
	connectib_header_type_pack(&(ptr_struct->header_type), ptr_buff + offset/8);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->bist);

	offset = 128;
	connectib_bar_low_pack(&(ptr_struct->bar_low), ptr_buff + offset/8);

	offset = 160;
	connectib_bar_high_pack(&(ptr_struct->bar_high), ptr_buff + offset/8);

	offset = 216;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->primary_bus);

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->secondary_bus);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->subordinate_bus);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->secondary_latency);

	offset = 248;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->io_base);

	offset = 240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->io_limit);

	offset = 224;
	connectib_secondary_status_pack(&(ptr_struct->secondary_status), ptr_buff + offset/8);

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->memory_base);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->memory_limit);

	offset = 304;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pre_memory_base);

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pre_memory_limit);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pre_base_upper_32);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pre_limit_upper_32);

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->io_base_upper_16);

	offset = 384;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->io_limit_upper_16);

	offset = 440;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cap_ptr);

	offset = 448;
	connectib_exp_rom_pack(&(ptr_struct->exp_rom_bar), ptr_buff + offset/8);

	offset = 504;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->interrupt_line);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->interrupt_pin);

	offset = 480;
	connectib_bridge_control_pack(&(ptr_struct->bridge_control), ptr_buff + offset/8);

}

void connectib_pci_cfg_type1_header_unpack(struct connectib_pci_cfg_type1_header *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->vendor_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->device_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	connectib_cmd_reg1_unpack(&(ptr_struct->command), ptr_buff + offset/8);

	offset = 32;
	connectib_stat_reg1_unpack(&(ptr_struct->status), ptr_buff + offset/8);

	offset = 88;
	ptr_struct->revision_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->class_code = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 120;
	ptr_struct->cache_line = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 112;
	ptr_struct->latency = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	connectib_header_type_unpack(&(ptr_struct->header_type), ptr_buff + offset/8);

	offset = 96;
	ptr_struct->bist = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 128;
	connectib_bar_low_unpack(&(ptr_struct->bar_low), ptr_buff + offset/8);

	offset = 160;
	connectib_bar_high_unpack(&(ptr_struct->bar_high), ptr_buff + offset/8);

	offset = 216;
	ptr_struct->primary_bus = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 208;
	ptr_struct->secondary_bus = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 200;
	ptr_struct->subordinate_bus = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 192;
	ptr_struct->secondary_latency = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 248;
	ptr_struct->io_base = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 240;
	ptr_struct->io_limit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 224;
	connectib_secondary_status_unpack(&(ptr_struct->secondary_status), ptr_buff + offset/8);

	offset = 272;
	ptr_struct->memory_base = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 256;
	ptr_struct->memory_limit = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 304;
	ptr_struct->pre_memory_base = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 288;
	ptr_struct->pre_memory_limit = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 320;
	ptr_struct->pre_base_upper_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->pre_limit_upper_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 400;
	ptr_struct->io_base_upper_16 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 384;
	ptr_struct->io_limit_upper_16 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 440;
	ptr_struct->cap_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 448;
	connectib_exp_rom_unpack(&(ptr_struct->exp_rom_bar), ptr_buff + offset/8);

	offset = 504;
	ptr_struct->interrupt_line = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 496;
	ptr_struct->interrupt_pin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 480;
	connectib_bridge_control_unpack(&(ptr_struct->bridge_control), ptr_buff + offset/8);

}

void connectib_pci_cfg_type1_header_print(const struct connectib_pci_cfg_type1_header *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pci_cfg_type1_header ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vendor_id            : " UH_FMT "\n", ptr_struct->vendor_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_id            : " UH_FMT "\n", ptr_struct->device_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "command:\n");
	connectib_cmd_reg1_print(&(ptr_struct->command), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status:\n");
	connectib_stat_reg1_print(&(ptr_struct->status), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "revision_id          : " UH_FMT "\n", ptr_struct->revision_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_code           : " UH_FMT "\n", ptr_struct->class_code);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cache_line           : " UH_FMT "\n", ptr_struct->cache_line);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "latency              : " UH_FMT "\n", ptr_struct->latency);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "header_type:\n");
	connectib_header_type_print(&(ptr_struct->header_type), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bist                 : " UH_FMT "\n", ptr_struct->bist);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bar_low:\n");
	connectib_bar_low_print(&(ptr_struct->bar_low), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bar_high:\n");
	connectib_bar_high_print(&(ptr_struct->bar_high), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "primary_bus          : " UH_FMT "\n", ptr_struct->primary_bus);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "secondary_bus        : " UH_FMT "\n", ptr_struct->secondary_bus);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "subordinate_bus      : " UH_FMT "\n", ptr_struct->subordinate_bus);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "secondary_latency    : " UH_FMT "\n", ptr_struct->secondary_latency);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "io_base              : " UH_FMT "\n", ptr_struct->io_base);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "io_limit             : " UH_FMT "\n", ptr_struct->io_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "secondary_status:\n");
	connectib_secondary_status_print(&(ptr_struct->secondary_status), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "memory_base          : " UH_FMT "\n", ptr_struct->memory_base);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "memory_limit         : " UH_FMT "\n", ptr_struct->memory_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pre_memory_base      : " UH_FMT "\n", ptr_struct->pre_memory_base);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pre_memory_limit     : " UH_FMT "\n", ptr_struct->pre_memory_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pre_base_upper_32    : " U32H_FMT "\n", ptr_struct->pre_base_upper_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pre_limit_upper_32   : " U32H_FMT "\n", ptr_struct->pre_limit_upper_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "io_base_upper_16     : " UH_FMT "\n", ptr_struct->io_base_upper_16);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "io_limit_upper_16    : " UH_FMT "\n", ptr_struct->io_limit_upper_16);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_ptr              : " UH_FMT "\n", ptr_struct->cap_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "exp_rom_bar:\n");
	connectib_exp_rom_print(&(ptr_struct->exp_rom_bar), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "interrupt_line       : " UH_FMT "\n", ptr_struct->interrupt_line);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "interrupt_pin        : " UH_FMT "\n", ptr_struct->interrupt_pin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bridge_control:\n");
	connectib_bridge_control_print(&(ptr_struct->bridge_control), file, indent_level + 1);

}

int connectib_pci_cfg_type1_header_size(void)
{
	 return 64;
}

void connectib_pci_cfg_type1_header_dump(const struct connectib_pci_cfg_type1_header *ptr_struct, FILE* file)
{
	connectib_pci_cfg_type1_header_print(ptr_struct, file, 0);
}

void connectib_msix_cap_pack(const struct connectib_msix_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->msix_cap_id);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 0;
	connectib_message_control_pack(&(ptr_struct->message_control), ptr_buff + offset/8);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->Table_BIR);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 29, (u_int32_t)ptr_struct->Table_Offset);

	offset = 93;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->PBA_BIR);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 29, (u_int32_t)ptr_struct->PBA_Offset);

}

void connectib_msix_cap_unpack(struct connectib_msix_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->msix_cap_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->nxt_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	connectib_message_control_unpack(&(ptr_struct->message_control), ptr_buff + offset/8);

	offset = 61;
	ptr_struct->Table_BIR = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 32;
	ptr_struct->Table_Offset = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 29);

	offset = 93;
	ptr_struct->PBA_BIR = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 64;
	ptr_struct->PBA_Offset = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 29);

}

void connectib_msix_cap_print(const struct connectib_msix_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_msix_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msix_cap_id          : " UH_FMT "\n", ptr_struct->msix_cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "message_control:\n");
	connectib_message_control_print(&(ptr_struct->message_control), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Table_BIR            : " UH_FMT "\n", ptr_struct->Table_BIR);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Table_Offset         : " UH_FMT "\n", ptr_struct->Table_Offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "PBA_BIR              : " UH_FMT "\n", ptr_struct->PBA_BIR);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "PBA_Offset           : " UH_FMT "\n", ptr_struct->PBA_Offset);

}

int connectib_msix_cap_size(void)
{
	 return 12;
}

void connectib_msix_cap_dump(const struct connectib_msix_cap *ptr_struct, FILE* file)
{
	connectib_msix_cap_print(ptr_struct, file, 0);
}

void connectib_pciex_cap_block_pack(const struct connectib_pciex_cap_block *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pciex_cap_id);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 0;
	connectib_pciex_cap_pack(&(ptr_struct->pciex_cap), ptr_buff + offset/8);

	offset = 32;
	connectib_device_caps_pack(&(ptr_struct->device_caps), ptr_buff + offset/8);

	offset = 80;
	connectib_device_ctl_pack(&(ptr_struct->device_ctl), ptr_buff + offset/8);

	offset = 64;
	connectib_device_stat_pack(&(ptr_struct->device_stat), ptr_buff + offset/8);

	offset = 96;
	connectib_link_caps_pack(&(ptr_struct->link_caps), ptr_buff + offset/8);

	offset = 144;
	connectib_link_ctl_pack(&(ptr_struct->link_ctl), ptr_buff + offset/8);

	offset = 128;
	connectib_link_stat_pack(&(ptr_struct->link_stat), ptr_buff + offset/8);

	offset = 160;
	connectib_slot_caps_pack(&(ptr_struct->slot_caps), ptr_buff + offset/8);

	offset = 288;
	connectib_device_caps2_pack(&(ptr_struct->device_caps2), ptr_buff + offset/8);

	offset = 336;
	connectib_device_ctl2_pack(&(ptr_struct->device_ctl2), ptr_buff + offset/8);

	offset = 320;
	connectib_device_stat2_pack(&(ptr_struct->device_stat2), ptr_buff + offset/8);

	offset = 352;
	connectib_link_caps2_pack(&(ptr_struct->link_caps2), ptr_buff + offset/8);

	offset = 400;
	connectib_link_ctl2_pack(&(ptr_struct->link_ctl2), ptr_buff + offset/8);

	offset = 384;
	connectib_link_stat2_pack(&(ptr_struct->link_stat2), ptr_buff + offset/8);

}

void connectib_pciex_cap_block_unpack(struct connectib_pciex_cap_block *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->pciex_cap_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->nxt_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	connectib_pciex_cap_unpack(&(ptr_struct->pciex_cap), ptr_buff + offset/8);

	offset = 32;
	connectib_device_caps_unpack(&(ptr_struct->device_caps), ptr_buff + offset/8);

	offset = 80;
	connectib_device_ctl_unpack(&(ptr_struct->device_ctl), ptr_buff + offset/8);

	offset = 64;
	connectib_device_stat_unpack(&(ptr_struct->device_stat), ptr_buff + offset/8);

	offset = 96;
	connectib_link_caps_unpack(&(ptr_struct->link_caps), ptr_buff + offset/8);

	offset = 144;
	connectib_link_ctl_unpack(&(ptr_struct->link_ctl), ptr_buff + offset/8);

	offset = 128;
	connectib_link_stat_unpack(&(ptr_struct->link_stat), ptr_buff + offset/8);

	offset = 160;
	connectib_slot_caps_unpack(&(ptr_struct->slot_caps), ptr_buff + offset/8);

	offset = 288;
	connectib_device_caps2_unpack(&(ptr_struct->device_caps2), ptr_buff + offset/8);

	offset = 336;
	connectib_device_ctl2_unpack(&(ptr_struct->device_ctl2), ptr_buff + offset/8);

	offset = 320;
	connectib_device_stat2_unpack(&(ptr_struct->device_stat2), ptr_buff + offset/8);

	offset = 352;
	connectib_link_caps2_unpack(&(ptr_struct->link_caps2), ptr_buff + offset/8);

	offset = 400;
	connectib_link_ctl2_unpack(&(ptr_struct->link_ctl2), ptr_buff + offset/8);

	offset = 384;
	connectib_link_stat2_unpack(&(ptr_struct->link_stat2), ptr_buff + offset/8);

}

void connectib_pciex_cap_block_print(const struct connectib_pciex_cap_block *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pciex_cap_block ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pciex_cap_id         : " UH_FMT "\n", ptr_struct->pciex_cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pciex_cap:\n");
	connectib_pciex_cap_print(&(ptr_struct->pciex_cap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_caps:\n");
	connectib_device_caps_print(&(ptr_struct->device_caps), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_ctl:\n");
	connectib_device_ctl_print(&(ptr_struct->device_ctl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_stat:\n");
	connectib_device_stat_print(&(ptr_struct->device_stat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_caps:\n");
	connectib_link_caps_print(&(ptr_struct->link_caps), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_ctl:\n");
	connectib_link_ctl_print(&(ptr_struct->link_ctl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_stat:\n");
	connectib_link_stat_print(&(ptr_struct->link_stat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slot_caps:\n");
	connectib_slot_caps_print(&(ptr_struct->slot_caps), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_caps2:\n");
	connectib_device_caps2_print(&(ptr_struct->device_caps2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_ctl2:\n");
	connectib_device_ctl2_print(&(ptr_struct->device_ctl2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_stat2:\n");
	connectib_device_stat2_print(&(ptr_struct->device_stat2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_caps2:\n");
	connectib_link_caps2_print(&(ptr_struct->link_caps2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_ctl2:\n");
	connectib_link_ctl2_print(&(ptr_struct->link_ctl2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_stat2:\n");
	connectib_link_stat2_print(&(ptr_struct->link_stat2), file, indent_level + 1);

}

int connectib_pciex_cap_block_size(void)
{
	 return 60;
}

void connectib_pciex_cap_block_dump(const struct connectib_pciex_cap_block *ptr_struct, FILE* file)
{
	connectib_pciex_cap_block_print(ptr_struct, file, 0);
}

void connectib_vpd_cap_pack(const struct connectib_vpd_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vpd_cap_id);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 15, (u_int32_t)ptr_struct->vpd_address);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->F);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vpd_data);

}

void connectib_vpd_cap_unpack(struct connectib_vpd_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->vpd_cap_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->nxt_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 1;
	ptr_struct->vpd_address = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 15);

	offset = 0;
	ptr_struct->F = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->vpd_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_vpd_cap_print(const struct connectib_vpd_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_vpd_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_cap_id           : " UH_FMT "\n", ptr_struct->vpd_cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_address          : " UH_FMT "\n", ptr_struct->vpd_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "F                    : " UH_FMT "\n", ptr_struct->F);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_data             : " U32H_FMT "\n", ptr_struct->vpd_data);

}

int connectib_vpd_cap_size(void)
{
	 return 8;
}

void connectib_vpd_cap_dump(const struct connectib_vpd_cap *ptr_struct, FILE* file)
{
	connectib_vpd_cap_print(ptr_struct, file, 0);
}

void connectib_pci_power_management_cap_pack(const struct connectib_pci_power_management_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cap_id);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->nxt_ptr);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->version);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pme_clock);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_specific);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->aux_current);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->d1_support);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->d2_support);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->pme_support);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->power_state);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->no_soft_reset);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pme_enable);

	offset = 51;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data_select);

	offset = 49;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->data_scale);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pme_status);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pmcsr_bse);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->data);

}

void connectib_pci_power_management_cap_unpack(struct connectib_pci_power_management_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->cap_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->nxt_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 13;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 12;
	ptr_struct->pme_clock = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 10;
	ptr_struct->device_specific = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 7;
	ptr_struct->aux_current = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 6;
	ptr_struct->d1_support = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->d2_support = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->pme_support = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 62;
	ptr_struct->power_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 60;
	ptr_struct->no_soft_reset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 55;
	ptr_struct->pme_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 51;
	ptr_struct->data_select = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 49;
	ptr_struct->data_scale = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 48;
	ptr_struct->pme_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 40;
	ptr_struct->pmcsr_bse = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->data = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_pci_power_management_cap_print(const struct connectib_pci_power_management_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pci_power_management_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_id               : " UH_FMT "\n", ptr_struct->cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nxt_ptr              : " UH_FMT "\n", ptr_struct->nxt_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pme_clock            : " UH_FMT "\n", ptr_struct->pme_clock);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_specific      : " UH_FMT "\n", ptr_struct->device_specific);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aux_current          : " UH_FMT "\n", ptr_struct->aux_current);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "d1_support           : " UH_FMT "\n", ptr_struct->d1_support);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "d2_support           : " UH_FMT "\n", ptr_struct->d2_support);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pme_support          : " UH_FMT "\n", ptr_struct->pme_support);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "power_state          : " UH_FMT "\n", ptr_struct->power_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_soft_reset        : " UH_FMT "\n", ptr_struct->no_soft_reset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pme_enable           : " UH_FMT "\n", ptr_struct->pme_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_select          : " UH_FMT "\n", ptr_struct->data_select);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_scale           : " UH_FMT "\n", ptr_struct->data_scale);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pme_status           : " UH_FMT "\n", ptr_struct->pme_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pmcsr_bse            : " UH_FMT "\n", ptr_struct->pmcsr_bse);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data                 : " UH_FMT "\n", ptr_struct->data);

}

int connectib_pci_power_management_cap_size(void)
{
	 return 8;
}

void connectib_pci_power_management_cap_dump(const struct connectib_pci_power_management_cap *ptr_struct, FILE* file)
{
	connectib_pci_power_management_cap_print(ptr_struct, file, 0);
}

void connectib_pci_cfg_type0_header_pack(const struct connectib_pci_cfg_type0_header *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vendor_id);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->device_id);

	offset = 48;
	connectib_cmd_reg1_pack(&(ptr_struct->cmd), ptr_buff + offset/8);

	offset = 32;
	connectib_stat_reg1_pack(&(ptr_struct->status), ptr_buff + offset/8);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->rev_id);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->class_code);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cache_line);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->latency);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hdr_type);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->bist);

	offset = 128;
	connectib_bar_low_pack(&(ptr_struct->reg4), ptr_buff + offset/8);

	offset = 160;
	connectib_bar_high_pack(&(ptr_struct->reg5), ptr_buff + offset/8);

	offset = 192;
	connectib_bar_low_pack(&(ptr_struct->reg6), ptr_buff + offset/8);

	offset = 224;
	connectib_bar_high_pack(&(ptr_struct->reg7), ptr_buff + offset/8);

	offset = 256;
	connectib_bar_low_pack(&(ptr_struct->reg8), ptr_buff + offset/8);

	offset = 288;
	connectib_bar_high_pack(&(ptr_struct->reg9), ptr_buff + offset/8);

	offset = 368;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->sub_vendor_id);

	offset = 352;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->subsystem_id);

	offset = 384;
	connectib_exp_rom_pack(&(ptr_struct->reg12), ptr_buff + offset/8);

	offset = 440;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cap_ptr);

	offset = 504;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->interrupt_line);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->interrupt_pin);

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->min_gnt_max_lat);

}

void connectib_pci_cfg_type0_header_unpack(struct connectib_pci_cfg_type0_header *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->vendor_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->device_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	connectib_cmd_reg1_unpack(&(ptr_struct->cmd), ptr_buff + offset/8);

	offset = 32;
	connectib_stat_reg1_unpack(&(ptr_struct->status), ptr_buff + offset/8);

	offset = 88;
	ptr_struct->rev_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->class_code = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 120;
	ptr_struct->cache_line = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 112;
	ptr_struct->latency = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->hdr_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->bist = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 128;
	connectib_bar_low_unpack(&(ptr_struct->reg4), ptr_buff + offset/8);

	offset = 160;
	connectib_bar_high_unpack(&(ptr_struct->reg5), ptr_buff + offset/8);

	offset = 192;
	connectib_bar_low_unpack(&(ptr_struct->reg6), ptr_buff + offset/8);

	offset = 224;
	connectib_bar_high_unpack(&(ptr_struct->reg7), ptr_buff + offset/8);

	offset = 256;
	connectib_bar_low_unpack(&(ptr_struct->reg8), ptr_buff + offset/8);

	offset = 288;
	connectib_bar_high_unpack(&(ptr_struct->reg9), ptr_buff + offset/8);

	offset = 368;
	ptr_struct->sub_vendor_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 352;
	ptr_struct->subsystem_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 384;
	connectib_exp_rom_unpack(&(ptr_struct->reg12), ptr_buff + offset/8);

	offset = 440;
	ptr_struct->cap_ptr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 504;
	ptr_struct->interrupt_line = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 496;
	ptr_struct->interrupt_pin = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 480;
	ptr_struct->min_gnt_max_lat = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_pci_cfg_type0_header_print(const struct connectib_pci_cfg_type0_header *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pci_cfg_type0_header ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vendor_id            : " UH_FMT "\n", ptr_struct->vendor_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_id            : " UH_FMT "\n", ptr_struct->device_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmd:\n");
	connectib_cmd_reg1_print(&(ptr_struct->cmd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status:\n");
	connectib_stat_reg1_print(&(ptr_struct->status), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rev_id               : " UH_FMT "\n", ptr_struct->rev_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_code           : " UH_FMT "\n", ptr_struct->class_code);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cache_line           : " UH_FMT "\n", ptr_struct->cache_line);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "latency              : " UH_FMT "\n", ptr_struct->latency);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hdr_type             : " UH_FMT "\n", ptr_struct->hdr_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bist                 : " UH_FMT "\n", ptr_struct->bist);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg4:\n");
	connectib_bar_low_print(&(ptr_struct->reg4), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg5:\n");
	connectib_bar_high_print(&(ptr_struct->reg5), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg6:\n");
	connectib_bar_low_print(&(ptr_struct->reg6), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg7:\n");
	connectib_bar_high_print(&(ptr_struct->reg7), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg8:\n");
	connectib_bar_low_print(&(ptr_struct->reg8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg9:\n");
	connectib_bar_high_print(&(ptr_struct->reg9), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sub_vendor_id        : " UH_FMT "\n", ptr_struct->sub_vendor_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "subsystem_id         : " UH_FMT "\n", ptr_struct->subsystem_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg12:\n");
	connectib_exp_rom_print(&(ptr_struct->reg12), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_ptr              : " UH_FMT "\n", ptr_struct->cap_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "interrupt_line       : " UH_FMT "\n", ptr_struct->interrupt_line);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "interrupt_pin        : " UH_FMT "\n", ptr_struct->interrupt_pin);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "min_gnt_max_lat      : " UH_FMT "\n", ptr_struct->min_gnt_max_lat);

}

int connectib_pci_cfg_type0_header_size(void)
{
	 return 64;
}

void connectib_pci_cfg_type0_header_dump(const struct connectib_pci_cfg_type0_header *ptr_struct, FILE* file)
{
	connectib_pci_cfg_type0_header_print(ptr_struct, file, 0);
}

void connectib_fw_block_behavior_fields_pack(const struct connectib_fw_block_behavior_fields *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_mfpa_mfba_mfbe);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_cr_space_flash_gw);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->en);

}

void connectib_fw_block_behavior_fields_unpack(struct connectib_fw_block_behavior_fields *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->disable_mfpa_mfba_mfbe = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->disable_cr_space_flash_gw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_fw_block_behavior_fields_print(const struct connectib_fw_block_behavior_fields *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_block_behavior_fields ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_mfpa_mfba_mfbe : " UH_FMT "\n", ptr_struct->disable_mfpa_mfba_mfbe);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_cr_space_flash_gw : " UH_FMT "\n", ptr_struct->disable_cr_space_flash_gw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "en                   : " UH_FMT "\n", ptr_struct->en);

}

int connectib_fw_block_behavior_fields_size(void)
{
	 return 4;
}

void connectib_fw_block_behavior_fields_dump(const struct connectib_fw_block_behavior_fields *ptr_struct, FILE* file)
{
	connectib_fw_block_behavior_fields_print(ptr_struct, file, 0);
}

void connectib_fw_block_behavior_dw_pack(const struct connectib_fw_block_behavior_dw *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dw);

}

void connectib_fw_block_behavior_dw_unpack(struct connectib_fw_block_behavior_dw *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->dw = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_fw_block_behavior_dw_print(const struct connectib_fw_block_behavior_dw *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_block_behavior_dw ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dw                   : " U32H_FMT "\n", ptr_struct->dw);

}

int connectib_fw_block_behavior_dw_size(void)
{
	 return 4;
}

void connectib_fw_block_behavior_dw_dump(const struct connectib_fw_block_behavior_dw *ptr_struct, FILE* file)
{
	connectib_fw_block_behavior_dw_print(ptr_struct, file, 0);
}

void connectib_nv_config_host_func_main_pack(const struct connectib_nv_config_host_func_main *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_external_port_pf_array_pack(&(ptr_struct->external_port), ptr_buff + offset/8);

}

void connectib_nv_config_host_func_main_unpack(struct connectib_nv_config_host_func_main *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_external_port_pf_array_unpack(&(ptr_struct->external_port), ptr_buff + offset/8);

}

void connectib_nv_config_host_func_main_print(const struct connectib_nv_config_host_func_main *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_host_func_main ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "external_port:\n");
	connectib_external_port_pf_array_print(&(ptr_struct->external_port), file, indent_level + 1);

}

int connectib_nv_config_host_func_main_size(void)
{
	 return 24;
}

void connectib_nv_config_host_func_main_dump(const struct connectib_nv_config_host_func_main *ptr_struct, FILE* file)
{
	connectib_nv_config_host_func_main_print(ptr_struct, file, 0);
}

void connectib_nv_config_global_main_pack(const struct connectib_nv_config_global_main *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_roce_1_5_conf_pack(&(ptr_struct->roce_1_5_next_prot), ptr_buff + offset/8);

	offset = 32;
	connectib_nv_sw_offload_conf_pack(&(ptr_struct->sw_offload_conf), ptr_buff + offset/8);

	offset = 96;
	connectib_nv_tlv_tweaks_pack(&(ptr_struct->tweaks), ptr_buff + offset/8);

	offset = 128;
	connectib_nv_config_host_2_bmc_pack(&(ptr_struct->host_2_bmc), ptr_buff + offset/8);

	offset = 160;
	connectib_nv_config_flex_nic_pack(&(ptr_struct->flex_nic), ptr_buff + offset/8);

	offset = 192;
	connectib_nv_tlv_data_curr_config_pack(&(ptr_struct->current_configurator), ptr_buff + offset/8);

}

void connectib_nv_config_global_main_unpack(struct connectib_nv_config_global_main *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_roce_1_5_conf_unpack(&(ptr_struct->roce_1_5_next_prot), ptr_buff + offset/8);

	offset = 32;
	connectib_nv_sw_offload_conf_unpack(&(ptr_struct->sw_offload_conf), ptr_buff + offset/8);

	offset = 96;
	connectib_nv_tlv_tweaks_unpack(&(ptr_struct->tweaks), ptr_buff + offset/8);

	offset = 128;
	connectib_nv_config_host_2_bmc_unpack(&(ptr_struct->host_2_bmc), ptr_buff + offset/8);

	offset = 160;
	connectib_nv_config_flex_nic_unpack(&(ptr_struct->flex_nic), ptr_buff + offset/8);

	offset = 192;
	connectib_nv_tlv_data_curr_config_unpack(&(ptr_struct->current_configurator), ptr_buff + offset/8);

}

void connectib_nv_config_global_main_print(const struct connectib_nv_config_global_main *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_global_main ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "roce_1_5_next_prot:\n");
	connectib_nv_roce_1_5_conf_print(&(ptr_struct->roce_1_5_next_prot), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_offload_conf:\n");
	connectib_nv_sw_offload_conf_print(&(ptr_struct->sw_offload_conf), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tweaks:\n");
	connectib_nv_tlv_tweaks_print(&(ptr_struct->tweaks), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_2_bmc:\n");
	connectib_nv_config_host_2_bmc_print(&(ptr_struct->host_2_bmc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flex_nic:\n");
	connectib_nv_config_flex_nic_print(&(ptr_struct->flex_nic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "current_configurator:\n");
	connectib_nv_tlv_data_curr_config_print(&(ptr_struct->current_configurator), file, indent_level + 1);

}

int connectib_nv_config_global_main_size(void)
{
	 return 64;
}

void connectib_nv_config_global_main_dump(const struct connectib_nv_config_global_main *ptr_struct, FILE* file)
{
	connectib_nv_config_global_main_print(ptr_struct, file, 0);
}

void connectib_port_settings_pack(const struct connectib_port_settings *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_roce_cc_pack(&(ptr_struct->roce_cc_ctrl), ptr_buff + offset/8);

	offset = 64;
	connectib_nv_roce_cc_ecn_pack(&(ptr_struct->roce_ecn), ptr_buff + offset/8);

	offset = 704;
	connectib_nv_keep_link_up_pack(&(ptr_struct->keep_link_up), ptr_buff + offset/8);

	offset = 768;
	connectib_nv_lldp_nb_cap_pack(&(ptr_struct->lldp_nb_cap), ptr_buff + offset/8);

	offset = 864;
	connectib_nv_lldp_nb_conf_pack(&(ptr_struct->lldp_nb_param), ptr_buff + offset/8);

	offset = 960;
	connectib_nv_lldp_nb_dcbx_pack(&(ptr_struct->dcbx_nb), ptr_buff + offset/8);

	offset = 1056;
	connectib_nv_qos_conf_pack(&(ptr_struct->qos_conf), ptr_buff + offset/8);

	offset = 1152;
	connectib_nv_qos_cap_pack(&(ptr_struct->qos_cap), ptr_buff + offset/8);

	offset = 1184;
	connectib_nv_qos_ets_tc_conf_pack(&(ptr_struct->qos_ets_tc_conf), ptr_buff + offset/8);

}

void connectib_port_settings_unpack(struct connectib_port_settings *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_roce_cc_unpack(&(ptr_struct->roce_cc_ctrl), ptr_buff + offset/8);

	offset = 64;
	connectib_nv_roce_cc_ecn_unpack(&(ptr_struct->roce_ecn), ptr_buff + offset/8);

	offset = 704;
	connectib_nv_keep_link_up_unpack(&(ptr_struct->keep_link_up), ptr_buff + offset/8);

	offset = 768;
	connectib_nv_lldp_nb_cap_unpack(&(ptr_struct->lldp_nb_cap), ptr_buff + offset/8);

	offset = 864;
	connectib_nv_lldp_nb_conf_unpack(&(ptr_struct->lldp_nb_param), ptr_buff + offset/8);

	offset = 960;
	connectib_nv_lldp_nb_dcbx_unpack(&(ptr_struct->dcbx_nb), ptr_buff + offset/8);

	offset = 1056;
	connectib_nv_qos_conf_unpack(&(ptr_struct->qos_conf), ptr_buff + offset/8);

	offset = 1152;
	connectib_nv_qos_cap_unpack(&(ptr_struct->qos_cap), ptr_buff + offset/8);

	offset = 1184;
	connectib_nv_qos_ets_tc_conf_unpack(&(ptr_struct->qos_ets_tc_conf), ptr_buff + offset/8);

}

void connectib_port_settings_print(const struct connectib_port_settings *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_port_settings ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "roce_cc_ctrl:\n");
	connectib_nv_roce_cc_print(&(ptr_struct->roce_cc_ctrl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "roce_ecn:\n");
	connectib_nv_roce_cc_ecn_print(&(ptr_struct->roce_ecn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "keep_link_up:\n");
	connectib_nv_keep_link_up_print(&(ptr_struct->keep_link_up), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_cap:\n");
	connectib_nv_lldp_nb_cap_print(&(ptr_struct->lldp_nb_cap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_nb_param:\n");
	connectib_nv_lldp_nb_conf_print(&(ptr_struct->lldp_nb_param), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcbx_nb:\n");
	connectib_nv_lldp_nb_dcbx_print(&(ptr_struct->dcbx_nb), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qos_conf:\n");
	connectib_nv_qos_conf_print(&(ptr_struct->qos_conf), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qos_cap:\n");
	connectib_nv_qos_cap_print(&(ptr_struct->qos_cap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qos_ets_tc_conf:\n");
	connectib_nv_qos_ets_tc_conf_print(&(ptr_struct->qos_ets_tc_conf), file, indent_level + 1);

}

int connectib_port_settings_size(void)
{
	 return 256;
}

void connectib_port_settings_dump(const struct connectib_port_settings *ptr_struct, FILE* file)
{
	connectib_port_settings_print(ptr_struct, file, 0);
}

void connectib_phy_lcl_ini_pack(const struct connectib_phy_lcl_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->PlaceHolder);

}

void connectib_phy_lcl_ini_unpack(struct connectib_phy_lcl_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->PlaceHolder = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_phy_lcl_ini_print(const struct connectib_phy_lcl_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_phy_lcl_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "PlaceHolder          : " U32H_FMT "\n", ptr_struct->PlaceHolder);

}

int connectib_phy_lcl_ini_size(void)
{
	 return 4;
}

void connectib_phy_lcl_ini_dump(const struct connectib_phy_lcl_ini *ptr_struct, FILE* file)
{
	connectib_phy_lcl_ini_print(ptr_struct, file, 0);
}

void connectib_port_cap_mask1_pack(const union connectib_port_cap_mask1 *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_dword_val_pack(&(ptr_struct->dword), ptr_buff);
}

void connectib_port_cap_mask1_unpack(union connectib_port_cap_mask1 *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_dword_val_unpack(&(ptr_struct->dword), ptr_buff);
}

void connectib_port_cap_mask1_print(const union connectib_port_cap_mask1 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_port_cap_mask1 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fields:\n");
	connectib_port_cap_mask1_fields_print(&(ptr_struct->fields), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword:\n");
	connectib_dword_val_print(&(ptr_struct->dword), file, indent_level + 1);

}

int connectib_port_cap_mask1_size(void)
{
	 return 4;
}

void connectib_port_cap_mask1_dump(const union connectib_port_cap_mask1 *ptr_struct, FILE* file)
{
	connectib_port_cap_mask1_print(ptr_struct, file, 0);
}

void connectib_reset_version_pack(const struct connectib_reset_version *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->minor);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->major);

}

void connectib_reset_version_unpack(struct connectib_reset_version *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->minor = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 0;
	ptr_struct->major = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_reset_version_print(const struct connectib_reset_version *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_reset_version ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "minor                : " UH_FMT "\n", ptr_struct->minor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "major                : " UH_FMT "\n", ptr_struct->major);

}

int connectib_reset_version_size(void)
{
	 return 4;
}

void connectib_reset_version_dump(const struct connectib_reset_version *ptr_struct, FILE* file)
{
	connectib_reset_version_print(ptr_struct, file, 0);
}

void connectib_dcr_odp_events_pack(const struct connectib_dcr_odp_events *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	connectib_dcr_odp_event_entry_pack(&(ptr_struct->event_entry[i]), ptr_buff + offset/8);
	}

}

void connectib_dcr_odp_events_unpack(struct connectib_dcr_odp_events *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	connectib_dcr_odp_event_entry_unpack(&(ptr_struct->event_entry[i]), ptr_buff + offset/8);
	}

}

void connectib_dcr_odp_events_print(const struct connectib_dcr_odp_events *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dcr_odp_events ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_entry_%03d:\n", i);
	connectib_dcr_odp_event_entry_print(&(ptr_struct->event_entry[i]), file, indent_level + 1);
	}

}

int connectib_dcr_odp_events_size(void)
{
	 return 64;
}

void connectib_dcr_odp_events_dump(const struct connectib_dcr_odp_events *ptr_struct, FILE* file)
{
	connectib_dcr_odp_events_print(ptr_struct, file, 0);
}

void connectib_disable_mask_value_pack(const struct connectib_disable_mask_value *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->value);

}

void connectib_disable_mask_value_unpack(struct connectib_disable_mask_value *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->value = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_disable_mask_value_print(const struct connectib_disable_mask_value *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_disable_mask_value ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "value                : " U32H_FMT "\n", ptr_struct->value);

}

int connectib_disable_mask_value_size(void)
{
	 return 4;
}

void connectib_disable_mask_value_dump(const struct connectib_disable_mask_value *ptr_struct, FILE* file)
{
	connectib_disable_mask_value_print(ptr_struct, file, 0);
}

void connectib_disable_mask_bits_pack(const struct connectib_disable_mask_bits *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->module_not_present);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->paos);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mad);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->non_mlnx_long_range_cable);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->verification);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ekey);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->high_power_not_allowed);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->unknown_identifier);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->paos_up_once_active);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->stamping_failure);

}

void connectib_disable_mask_bits_unpack(struct connectib_disable_mask_bits *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->module_not_present = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->paos = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->mad = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->non_mlnx_long_range_cable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->verification = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->ekey = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->high_power_not_allowed = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->unknown_identifier = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->paos_up_once_active = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->stamping_failure = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_disable_mask_bits_print(const struct connectib_disable_mask_bits *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_disable_mask_bits ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_not_present   : " UH_FMT "\n", ptr_struct->module_not_present);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "paos                 : " UH_FMT "\n", ptr_struct->paos);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mad                  : " UH_FMT "\n", ptr_struct->mad);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "non_mlnx_long_range_cable : " UH_FMT "\n", ptr_struct->non_mlnx_long_range_cable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "verification         : " UH_FMT "\n", ptr_struct->verification);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ekey                 : " UH_FMT "\n", ptr_struct->ekey);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "high_power_not_allowed : " UH_FMT "\n", ptr_struct->high_power_not_allowed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unknown_identifier   : " UH_FMT "\n", ptr_struct->unknown_identifier);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "paos_up_once_active  : " UH_FMT "\n", ptr_struct->paos_up_once_active);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "stamping_failure     : " UH_FMT "\n", ptr_struct->stamping_failure);

}

int connectib_disable_mask_bits_size(void)
{
	 return 4;
}

void connectib_disable_mask_bits_dump(const struct connectib_disable_mask_bits *ptr_struct, FILE* file)
{
	connectib_disable_mask_bits_print(ptr_struct, file, 0);
}

void connectib_dmfs_ft_type_info_pack(const struct connectib_dmfs_ft_type_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->root_ft);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ft_exist);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->level0_ft_exist);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_ft_root_set);

}

void connectib_dmfs_ft_type_info_unpack(struct connectib_dmfs_ft_type_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->root_ft = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 7;
	ptr_struct->ft_exist = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->level0_ft_exist = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->is_ft_root_set = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_dmfs_ft_type_info_print(const struct connectib_dmfs_ft_type_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dmfs_ft_type_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "root_ft              : " UH_FMT "\n", ptr_struct->root_ft);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_exist             : " UH_FMT "\n", ptr_struct->ft_exist);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "level0_ft_exist      : " UH_FMT "\n", ptr_struct->level0_ft_exist);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_ft_root_set       : " UH_FMT "\n", ptr_struct->is_ft_root_set);

}

int connectib_dmfs_ft_type_info_size(void)
{
	 return 4;
}

void connectib_dmfs_ft_type_info_dump(const struct connectib_dmfs_ft_type_info *ptr_struct, FILE* file)
{
	connectib_dmfs_ft_type_info_print(ptr_struct, file, 0);
}

void connectib_ocbb_fields_from_vpd_pack(const struct connectib_ocbb_fields_from_vpd *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 40; i++) {
	offset = adb2c_calc_array_field_address(24, 8, i, 544, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->product_name[i]);
	}

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(344, 8, i, 544, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->part_number[i]);
	}

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(440, 8, i, 544, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->serial_number[i]);
	}

	offset = 528;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->serial_number_size);

	offset = 520;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->part_number_size);

	offset = 512;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->product_name_size);

}

void connectib_ocbb_fields_from_vpd_unpack(struct connectib_ocbb_fields_from_vpd *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 40; i++) {
	offset = adb2c_calc_array_field_address(24, 8, i, 544, 1);
	ptr_struct->product_name[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(344, 8, i, 544, 1);
	ptr_struct->part_number[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(440, 8, i, 544, 1);
	ptr_struct->serial_number[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	offset = 528;
	ptr_struct->serial_number_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 520;
	ptr_struct->part_number_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 512;
	ptr_struct->product_name_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_ocbb_fields_from_vpd_print(const struct connectib_ocbb_fields_from_vpd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ocbb_fields_from_vpd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 40; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "product_name_%03d    : " UH_FMT "\n", i, ptr_struct->product_name[i]);
	}

	for (i = 0; i < 12; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "part_number_%03d     : " UH_FMT "\n", i, ptr_struct->part_number[i]);
	}

	for (i = 0; i < 12; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serial_number_%03d   : " UH_FMT "\n", i, ptr_struct->serial_number[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serial_number_size   : " UH_FMT "\n", ptr_struct->serial_number_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "part_number_size     : " UH_FMT "\n", ptr_struct->part_number_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "product_name_size    : " UH_FMT "\n", ptr_struct->product_name_size);

}

int connectib_ocbb_fields_from_vpd_size(void)
{
	 return 68;
}

void connectib_ocbb_fields_from_vpd_dump(const struct connectib_ocbb_fields_from_vpd *ptr_struct, FILE* file)
{
	connectib_ocbb_fields_from_vpd_print(ptr_struct, file, 0);
}

void connectib_fw_banner_pack(const struct connectib_fw_banner *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 3; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 96, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fw_banner_str[i]);
	}

}

void connectib_fw_banner_unpack(struct connectib_fw_banner *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 3; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 96, 1);
	ptr_struct->fw_banner_str[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_fw_banner_print(const struct connectib_fw_banner *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_banner ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 3; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_banner_str_%03d   : " U32H_FMT "\n", i, ptr_struct->fw_banner_str[i]);
	}

}

int connectib_fw_banner_size(void)
{
	 return 12;
}

void connectib_fw_banner_dump(const struct connectib_fw_banner *ptr_struct, FILE* file)
{
	connectib_fw_banner_print(ptr_struct, file, 0);
}

void connectib_resource_type_entry_pack(const struct connectib_resource_type_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->type);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->locked);

}

void connectib_resource_type_entry_unpack(struct connectib_resource_type_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->type = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->locked = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_resource_type_entry_print(const struct connectib_resource_type_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_resource_type_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " UH_FMT "\n", ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "locked               : " UH_FMT "\n", ptr_struct->locked);

}

int connectib_resource_type_entry_size(void)
{
	 return 4;
}

void connectib_resource_type_entry_dump(const struct connectib_resource_type_entry *ptr_struct, FILE* file)
{
	connectib_resource_type_entry_print(ptr_struct, file, 0);
}

void connectib_index_and_length_pack(const struct connectib_index_and_length *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->index);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->length);

}

void connectib_index_and_length_unpack(struct connectib_index_and_length *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->index = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->length = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_index_and_length_print(const struct connectib_index_and_length *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_index_and_length ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "index                : " U64H_FMT "\n", ptr_struct->index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "length               : " U32H_FMT "\n", ptr_struct->length);

}

int connectib_index_and_length_size(void)
{
	 return 16;
}

void connectib_index_and_length_dump(const struct connectib_index_and_length *ptr_struct, FILE* file)
{
	connectib_index_and_length_print(ptr_struct, file, 0);
}

void connectib_flow_list_pack(const union connectib_flow_list *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_flow_counter_list_pack(&(ptr_struct->cntr), ptr_buff);
}

void connectib_flow_list_unpack(union connectib_flow_list *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_flow_counter_list_unpack(&(ptr_struct->cntr), ptr_buff);
}

void connectib_flow_list_print(const union connectib_flow_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_flow_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dest:\n");
	connectib_dest_format_struct_print(&(ptr_struct->dest), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cntr:\n");
	connectib_flow_counter_list_print(&(ptr_struct->cntr), file, indent_level + 1);

}

int connectib_flow_list_size(void)
{
	 return 8;
}

void connectib_flow_list_dump(const union connectib_flow_list *ptr_struct, FILE* file)
{
	connectib_flow_list_print(ptr_struct, file, 0);
}

void connectib_flow_table_entry_match_set_lyr_2_4_pack(const struct connectib_flow_table_entry_match_set_lyr_2_4 *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->smac_47_16);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ethertype);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->smac_15_0);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dmac_47_16);

	offset = 116;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->first_vid);

	offset = 115;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->first_cfi);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->first_prio);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->dmac_15_0);

	offset = 151;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->tcp_flags);

	offset = 147;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ip_version);

	offset = 146;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->frag);

	offset = 145;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->svlan_tag);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cvlan_tag);

	offset = 142;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->ip_ecn);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->ip_dscp);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ip_protocol);

	offset = 176;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->tcp_dport);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->tcp_sport);

	offset = 240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->udp_dport);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->udp_sport);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->src_ip_127_96);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->src_ip_95_64);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->src_ip_63_32);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->src_ip_31_0);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dst_ip_127_96);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dst_ip_95_64);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dst_ip_63_32);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dst_ip_31_0);

}

void connectib_flow_table_entry_match_set_lyr_2_4_unpack(struct connectib_flow_table_entry_match_set_lyr_2_4 *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->smac_47_16 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 48;
	ptr_struct->ethertype = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->smac_15_0 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->dmac_47_16 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 116;
	ptr_struct->first_vid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 115;
	ptr_struct->first_cfi = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 112;
	ptr_struct->first_prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 96;
	ptr_struct->dmac_15_0 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 151;
	ptr_struct->tcp_flags = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 147;
	ptr_struct->ip_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 146;
	ptr_struct->frag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 145;
	ptr_struct->svlan_tag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 144;
	ptr_struct->cvlan_tag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 142;
	ptr_struct->ip_ecn = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 136;
	ptr_struct->ip_dscp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 128;
	ptr_struct->ip_protocol = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 176;
	ptr_struct->tcp_dport = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 160;
	ptr_struct->tcp_sport = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 240;
	ptr_struct->udp_dport = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 224;
	ptr_struct->udp_sport = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 256;
	ptr_struct->src_ip_127_96 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->src_ip_95_64 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->src_ip_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->src_ip_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->dst_ip_127_96 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->dst_ip_95_64 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->dst_ip_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->dst_ip_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_flow_table_entry_match_set_lyr_2_4_print(const struct connectib_flow_table_entry_match_set_lyr_2_4 *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_flow_table_entry_match_set_lyr_2_4 ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "smac_47_16           : " U32H_FMT "\n", ptr_struct->smac_47_16);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ethertype            : " UH_FMT "\n", ptr_struct->ethertype);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "smac_15_0            : " UH_FMT "\n", ptr_struct->smac_15_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmac_47_16           : " U32H_FMT "\n", ptr_struct->dmac_47_16);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_vid            : " UH_FMT "\n", ptr_struct->first_vid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_cfi            : " UH_FMT "\n", ptr_struct->first_cfi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_prio           : " UH_FMT "\n", ptr_struct->first_prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmac_15_0            : " UH_FMT "\n", ptr_struct->dmac_15_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcp_flags            : " UH_FMT "\n", ptr_struct->tcp_flags);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ip_version           : " UH_FMT "\n", ptr_struct->ip_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "frag                 : " UH_FMT "\n", ptr_struct->frag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "svlan_tag            : " UH_FMT "\n", ptr_struct->svlan_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cvlan_tag            : " UH_FMT "\n", ptr_struct->cvlan_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ip_ecn               : " UH_FMT "\n", ptr_struct->ip_ecn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ip_dscp              : " UH_FMT "\n", ptr_struct->ip_dscp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ip_protocol          : " UH_FMT "\n", ptr_struct->ip_protocol);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcp_dport            : " UH_FMT "\n", ptr_struct->tcp_dport);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tcp_sport            : " UH_FMT "\n", ptr_struct->tcp_sport);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "udp_dport            : " UH_FMT "\n", ptr_struct->udp_dport);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "udp_sport            : " UH_FMT "\n", ptr_struct->udp_sport);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "src_ip_127_96        : " U32H_FMT "\n", ptr_struct->src_ip_127_96);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "src_ip_95_64         : " U32H_FMT "\n", ptr_struct->src_ip_95_64);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "src_ip_63_32         : " U32H_FMT "\n", ptr_struct->src_ip_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "src_ip_31_0          : " U32H_FMT "\n", ptr_struct->src_ip_31_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dst_ip_127_96        : " U32H_FMT "\n", ptr_struct->dst_ip_127_96);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dst_ip_95_64         : " U32H_FMT "\n", ptr_struct->dst_ip_95_64);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dst_ip_63_32         : " U32H_FMT "\n", ptr_struct->dst_ip_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dst_ip_31_0          : " U32H_FMT "\n", ptr_struct->dst_ip_31_0);

}

int connectib_flow_table_entry_match_set_lyr_2_4_size(void)
{
	 return 64;
}

void connectib_flow_table_entry_match_set_lyr_2_4_dump(const struct connectib_flow_table_entry_match_set_lyr_2_4 *ptr_struct, FILE* file)
{
	connectib_flow_table_entry_match_set_lyr_2_4_print(ptr_struct, file, 0);
}

void connectib_flow_table_entry_match_set_misc_pack(const struct connectib_flow_table_entry_match_set_misc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->source_sqn);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->source_port);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->inner_second_vid);

	offset = 83;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->inner_second_cfi);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->inner_second_prio);

	offset = 68;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->outer_second_vid);

	offset = 67;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->outer_second_cfi);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->outer_second_prio);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gre_protocol);

	offset = 99;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->inner_second_svlan_tag);

	offset = 98;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->outer_second_svlan_tag);

	offset = 97;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->inner_second_cvlan_tag);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->outer_second_cvlan_tag);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->gre_key_l);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->gre_key_h);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->vxlan_vni);

	offset = 223;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->geneve_oam);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->geneve_vni);

	offset = 236;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->outer_ipv6_flow_label);

	offset = 268;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->inner_ipv6_flow_label);

	offset = 304;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->geneve_protocol_type);

	offset = 298;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->geneve_opt_len);

	offset = 328;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->bth_dst_qp);

}

void connectib_flow_table_entry_match_set_misc_unpack(struct connectib_flow_table_entry_match_set_misc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->source_sqn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 48;
	ptr_struct->source_port = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 84;
	ptr_struct->inner_second_vid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 83;
	ptr_struct->inner_second_cfi = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->inner_second_prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 68;
	ptr_struct->outer_second_vid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 67;
	ptr_struct->outer_second_cfi = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->outer_second_prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 112;
	ptr_struct->gre_protocol = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 99;
	ptr_struct->inner_second_svlan_tag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 98;
	ptr_struct->outer_second_svlan_tag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 97;
	ptr_struct->inner_second_cvlan_tag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->outer_second_cvlan_tag = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 152;
	ptr_struct->gre_key_l = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 128;
	ptr_struct->gre_key_h = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 160;
	ptr_struct->vxlan_vni = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 223;
	ptr_struct->geneve_oam = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 192;
	ptr_struct->geneve_vni = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 236;
	ptr_struct->outer_ipv6_flow_label = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

	offset = 268;
	ptr_struct->inner_ipv6_flow_label = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

	offset = 304;
	ptr_struct->geneve_protocol_type = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 298;
	ptr_struct->geneve_opt_len = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 328;
	ptr_struct->bth_dst_qp = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_flow_table_entry_match_set_misc_print(const struct connectib_flow_table_entry_match_set_misc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_flow_table_entry_match_set_misc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "source_sqn           : " UH_FMT "\n", ptr_struct->source_sqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "source_port          : " UH_FMT "\n", ptr_struct->source_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner_second_vid     : " UH_FMT "\n", ptr_struct->inner_second_vid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner_second_cfi     : " UH_FMT "\n", ptr_struct->inner_second_cfi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner_second_prio    : " UH_FMT "\n", ptr_struct->inner_second_prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "outer_second_vid     : " UH_FMT "\n", ptr_struct->outer_second_vid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "outer_second_cfi     : " UH_FMT "\n", ptr_struct->outer_second_cfi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "outer_second_prio    : " UH_FMT "\n", ptr_struct->outer_second_prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gre_protocol         : " UH_FMT "\n", ptr_struct->gre_protocol);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner_second_svlan_tag : " UH_FMT "\n", ptr_struct->inner_second_svlan_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "outer_second_svlan_tag : " UH_FMT "\n", ptr_struct->outer_second_svlan_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner_second_cvlan_tag : " UH_FMT "\n", ptr_struct->inner_second_cvlan_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "outer_second_cvlan_tag : " UH_FMT "\n", ptr_struct->outer_second_cvlan_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gre_key_l            : " UH_FMT "\n", ptr_struct->gre_key_l);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gre_key_h            : " UH_FMT "\n", ptr_struct->gre_key_h);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vxlan_vni            : " UH_FMT "\n", ptr_struct->vxlan_vni);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "geneve_oam           : " UH_FMT "\n", ptr_struct->geneve_oam);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "geneve_vni           : " UH_FMT "\n", ptr_struct->geneve_vni);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "outer_ipv6_flow_label : " UH_FMT "\n", ptr_struct->outer_ipv6_flow_label);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner_ipv6_flow_label : " UH_FMT "\n", ptr_struct->inner_ipv6_flow_label);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "geneve_protocol_type : " UH_FMT "\n", ptr_struct->geneve_protocol_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "geneve_opt_len       : " UH_FMT "\n", ptr_struct->geneve_opt_len);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bth_dst_qp           : " UH_FMT "\n", ptr_struct->bth_dst_qp);

}

int connectib_flow_table_entry_match_set_misc_size(void)
{
	 return 64;
}

void connectib_flow_table_entry_match_set_misc_dump(const struct connectib_flow_table_entry_match_set_misc *ptr_struct, FILE* file)
{
	connectib_flow_table_entry_match_set_misc_print(ptr_struct, file, 0);
}

void connectib_icmd_ver_data_one_qp_per_recovery_pack(const struct connectib_icmd_ver_data_one_qp_per_recovery *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->input);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->output);

}

void connectib_icmd_ver_data_one_qp_per_recovery_unpack(struct connectib_icmd_ver_data_one_qp_per_recovery *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->input = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->output = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_ver_data_one_qp_per_recovery_print(const struct connectib_icmd_ver_data_one_qp_per_recovery *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_ver_data_one_qp_per_recovery ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "input                : " UH_FMT "\n", ptr_struct->input);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "output               : " UH_FMT "\n", ptr_struct->output);

}

int connectib_icmd_ver_data_one_qp_per_recovery_size(void)
{
	 return 4;
}

void connectib_icmd_ver_data_one_qp_per_recovery_dump(const struct connectib_icmd_ver_data_one_qp_per_recovery *ptr_struct, FILE* file)
{
	connectib_icmd_ver_data_one_qp_per_recovery_print(ptr_struct, file, 0);
}

void connectib_icmd_ver_data_max_atomic_pack(const struct connectib_icmd_ver_data_max_atomic *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->log_max_atomic_size);

}

void connectib_icmd_ver_data_max_atomic_unpack(struct connectib_icmd_ver_data_max_atomic *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->log_max_atomic_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_ver_data_max_atomic_print(const struct connectib_icmd_ver_data_max_atomic *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_ver_data_max_atomic ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_atomic_size  : " UH_FMT "\n", ptr_struct->log_max_atomic_size);

}

int connectib_icmd_ver_data_max_atomic_size(void)
{
	 return 4;
}

void connectib_icmd_ver_data_max_atomic_dump(const struct connectib_icmd_ver_data_max_atomic *ptr_struct, FILE* file)
{
	connectib_icmd_ver_data_max_atomic_print(ptr_struct, file, 0);
}

void connectib_fw_vport_ctx_pack(const struct connectib_fw_vport_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->esw_state);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->nic_state);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->connected_2_steering);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->event_mask);

}

void connectib_fw_vport_ctx_unpack(struct connectib_fw_vport_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 14;
	ptr_struct->esw_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 11;
	ptr_struct->nic_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 8;
	ptr_struct->connected_2_steering = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->event_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_fw_vport_ctx_print(const struct connectib_fw_vport_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_vport_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "esw_state            : " UH_FMT "\n", ptr_struct->esw_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nic_state            : " UH_FMT "\n", ptr_struct->nic_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "connected_2_steering : " UH_FMT "\n", ptr_struct->connected_2_steering);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_mask           : " UH_FMT "\n", ptr_struct->event_mask);

}

int connectib_fw_vport_ctx_size(void)
{
	 return 2;
}

void connectib_fw_vport_ctx_dump(const struct connectib_fw_vport_ctx *ptr_struct, FILE* file)
{
	connectib_fw_vport_ctx_print(ptr_struct, file, 0);
}

void connectib_vf_pages_list_pack(const struct connectib_vf_pages_list *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->pages[i]);
	}

}

void connectib_vf_pages_list_unpack(struct connectib_vf_pages_list *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 512, 1);
	ptr_struct->pages[i] = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);
	}

}

void connectib_vf_pages_list_print(const struct connectib_vf_pages_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_vf_pages_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pages_%03d           : " U64H_FMT "\n", i, ptr_struct->pages[i]);
	}

}

int connectib_vf_pages_list_size(void)
{
	 return 64;
}

void connectib_vf_pages_list_dump(const struct connectib_vf_pages_list *ptr_struct, FILE* file)
{
	connectib_vf_pages_list_print(ptr_struct, file, 0);
}

void connectib_icm_page_manager_pack(const struct connectib_icm_page_manager *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->page_list);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->free_count);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->used_count);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->page_list_length);

	offset = 191;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lock);

	offset = 190;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->event_pending);

	offset = 189;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->driver_refused);

	offset = 187;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->state);

	offset = 186;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->num_vf_pages);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vf_page_ix);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->page_owner_pasid);

}

void connectib_icm_page_manager_unpack(struct connectib_icm_page_manager *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->page_list = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->free_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->used_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->page_list_length = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 191;
	ptr_struct->lock = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 190;
	ptr_struct->event_pending = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 189;
	ptr_struct->driver_refused = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 187;
	ptr_struct->state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 186;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->num_vf_pages = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 160;
	ptr_struct->vf_page_ix = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 224;
	ptr_struct->page_owner_pasid = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icm_page_manager_print(const struct connectib_icm_page_manager *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icm_page_manager ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "page_list            : " U64H_FMT "\n", ptr_struct->page_list);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "free_count           : " U32H_FMT "\n", ptr_struct->free_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "used_count           : " U32H_FMT "\n", ptr_struct->used_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "page_list_length     : " U32H_FMT "\n", ptr_struct->page_list_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lock                 : " UH_FMT "\n", ptr_struct->lock);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_pending        : " UH_FMT "\n", ptr_struct->event_pending);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "driver_refused       : " UH_FMT "\n", ptr_struct->driver_refused);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "state                : " UH_FMT "\n", ptr_struct->state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_vf_pages         : " UH_FMT "\n", ptr_struct->num_vf_pages);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_page_ix           : " UH_FMT "\n", ptr_struct->vf_page_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "page_owner_pasid     : " U32H_FMT "\n", ptr_struct->page_owner_pasid);

}

int connectib_icm_page_manager_size(void)
{
	 return 32;
}

void connectib_icm_page_manager_dump(const struct connectib_icm_page_manager *ptr_struct, FILE* file)
{
	connectib_icm_page_manager_print(ptr_struct, file, 0);
}

void connectib_hca_params_pack(const struct connectib_hca_params *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->partition_cap);

	offset = 67;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_max_qp);

	offset = 66;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->phy_port_owner);

	offset = 65;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->eswitch_owner);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dct);

	offset = 114;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log_max_ra_res_qp);

	offset = 98;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log_max_ra_req_qp);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->num_ports);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cache_line_128byte);

	offset = 176;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->log_uar_page_sz);

	offset = 170;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log_gid_table_size);

	offset = 169;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->uar_4k);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->named_qp);

	offset = 167;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->named_mkey);

	offset = 162;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->atomic_req_8b_endianess_mode);

	offset = 161;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->init_hca_disable_linkup);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->host_endianess);

	offset = 192;
	connectib_odp_hca_params_pack(&(ptr_struct->odp_caps), ptr_buff + offset/8);

}

void connectib_hca_params_unpack(struct connectib_hca_params *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 80;
	ptr_struct->partition_cap = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 67;
	ptr_struct->log_max_qp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 66;
	ptr_struct->phy_port_owner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 65;
	ptr_struct->eswitch_owner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->dct = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 114;
	ptr_struct->log_max_ra_res_qp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 98;
	ptr_struct->log_max_ra_req_qp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 144;
	ptr_struct->num_ports = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 128;
	ptr_struct->cache_line_128byte = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 176;
	ptr_struct->log_uar_page_sz = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 170;
	ptr_struct->log_gid_table_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 169;
	ptr_struct->uar_4k = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 168;
	ptr_struct->named_qp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 167;
	ptr_struct->named_mkey = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 162;
	ptr_struct->atomic_req_8b_endianess_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 161;
	ptr_struct->init_hca_disable_linkup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 160;
	ptr_struct->host_endianess = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 192;
	connectib_odp_hca_params_unpack(&(ptr_struct->odp_caps), ptr_buff + offset/8);

}

void connectib_hca_params_print(const struct connectib_hca_params *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hca_params ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "partition_cap        : " UH_FMT "\n", ptr_struct->partition_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_qp           : " UH_FMT "\n", ptr_struct->log_max_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_port_owner       : " UH_FMT "\n", ptr_struct->phy_port_owner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eswitch_owner        : " UH_FMT "\n", ptr_struct->eswitch_owner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dct                  : " UH_FMT "\n", ptr_struct->dct);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_ra_res_qp    : " UH_FMT "\n", ptr_struct->log_max_ra_res_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_ra_req_qp    : " UH_FMT "\n", ptr_struct->log_max_ra_req_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_ports            : " UH_FMT "\n", ptr_struct->num_ports);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cache_line_128byte   : " UH_FMT "\n", ptr_struct->cache_line_128byte);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_uar_page_sz      : " UH_FMT "\n", ptr_struct->log_uar_page_sz);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_gid_table_size   : " UH_FMT "\n", ptr_struct->log_gid_table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uar_4k               : " UH_FMT "\n", ptr_struct->uar_4k);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "named_qp             : " UH_FMT "\n", ptr_struct->named_qp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "named_mkey           : " UH_FMT "\n", ptr_struct->named_mkey);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "atomic_req_8b_endianess_mode : " UH_FMT "\n", ptr_struct->atomic_req_8b_endianess_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "init_hca_disable_linkup : " UH_FMT "\n", ptr_struct->init_hca_disable_linkup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_endianess       : " UH_FMT "\n", ptr_struct->host_endianess);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "odp_caps:\n");
	connectib_odp_hca_params_print(&(ptr_struct->odp_caps), file, indent_level + 1);

}

int connectib_hca_params_size(void)
{
	 return 32;
}

void connectib_hca_params_dump(const struct connectib_hca_params *ptr_struct, FILE* file)
{
	connectib_hca_params_print(ptr_struct, file, 0);
}

void connectib_dummy_addresses_one_cache_line_pack(const struct connectib_dummy_addresses_one_cache_line *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->ix[i]);
	}

}

void connectib_dummy_addresses_one_cache_line_unpack(struct connectib_dummy_addresses_one_cache_line *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 512, 1);
	ptr_struct->ix[i] = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);
	}

}

void connectib_dummy_addresses_one_cache_line_print(const struct connectib_dummy_addresses_one_cache_line *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dummy_addresses_one_cache_line ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ix_%03d              : " U64H_FMT "\n", i, ptr_struct->ix[i]);
	}

}

int connectib_dummy_addresses_one_cache_line_size(void)
{
	 return 64;
}

void connectib_dummy_addresses_one_cache_line_dump(const struct connectib_dummy_addresses_one_cache_line *ptr_struct, FILE* file)
{
	connectib_dummy_addresses_one_cache_line_print(ptr_struct, file, 0);
}

void connectib_cmdif_ctx_pack(const struct connectib_cmdif_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->done_missions);

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 25, (u_int32_t)ptr_struct->res_num);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cmd_valid);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->opcode);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->op_mod);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->allocated_pages);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->needed_pages);

	offset = 160;
	connectib_flush_type_pack(&(ptr_struct->flush_type), ptr_buff + offset/8);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->flush_qpn);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->flush_synd);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->nim_start_offset);

	offset = 256;
	connectib_cmdif_ctx_special_pack(&(ptr_struct->s), ptr_buff + offset/8);

}

void connectib_cmdif_ctx_unpack(struct connectib_cmdif_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->done_missions = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 39;
	ptr_struct->res_num = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 25);

	offset = 32;
	ptr_struct->cmd_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->opcode = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->op_mod = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->allocated_pages = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->needed_pages = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	connectib_flush_type_unpack(&(ptr_struct->flush_type), ptr_buff + offset/8);

	offset = 200;
	ptr_struct->flush_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 192;
	ptr_struct->flush_synd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 224;
	ptr_struct->nim_start_offset = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	connectib_cmdif_ctx_special_unpack(&(ptr_struct->s), ptr_buff + offset/8);

}

void connectib_cmdif_ctx_print(const struct connectib_cmdif_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "done_missions        : " U32H_FMT "\n", ptr_struct->done_missions);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "res_num              : " UH_FMT "\n", ptr_struct->res_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmd_valid            : " UH_FMT "\n", ptr_struct->cmd_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " UH_FMT "\n", ptr_struct->opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "op_mod               : " UH_FMT "\n", ptr_struct->op_mod);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allocated_pages      : " U32H_FMT "\n", ptr_struct->allocated_pages);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "needed_pages         : " U32H_FMT "\n", ptr_struct->needed_pages);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flush_type:\n");
	connectib_flush_type_print(&(ptr_struct->flush_type), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flush_qpn            : " UH_FMT "\n", ptr_struct->flush_qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flush_synd           : " UH_FMT "\n", ptr_struct->flush_synd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nim_start_offset     : " U32H_FMT "\n", ptr_struct->nim_start_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "s:\n");
	connectib_cmdif_ctx_special_print(&(ptr_struct->s), file, indent_level + 1);

}

int connectib_cmdif_ctx_size(void)
{
	 return 64;
}

void connectib_cmdif_ctx_dump(const struct connectib_cmdif_ctx *ptr_struct, FILE* file)
{
	connectib_cmdif_ctx_print(ptr_struct, file, 0);
}

void connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto_pack(const union connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_general_notification_delay_drop_event_pack(&(ptr_struct->general_notification_delay_drop_event), ptr_buff);
}

void connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto_unpack(union connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_general_notification_delay_drop_event_unpack(&(ptr_struct->general_notification_delay_drop_event), ptr_buff);
}

void connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto_print(const union connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flat_data:\n");
	connectib_sw_eqe_flat_data_print(&(ptr_struct->flat_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "comp_event:\n");
	connectib_comp_event_print(&(ptr_struct->comp_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dct_events:\n");
	connectib_dct_events_print(&(ptr_struct->dct_events), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_events:\n");
	connectib_qp_events_print(&(ptr_struct->qp_events), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_associated_page_fault_event:\n");
	connectib_wqe_associated_page_fault_event_print(&(ptr_struct->wqe_associated_page_fault_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdma_page_fault_event:\n");
	connectib_rdma_page_fault_event_print(&(ptr_struct->rdma_page_fault_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cq_error:\n");
	connectib_cq_error_print(&(ptr_struct->cq_error), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dropped_packet_logged:\n");
	connectib_dropped_packet_logged_print(&(ptr_struct->dropped_packet_logged), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_state_change_event:\n");
	connectib_port_state_change_event_print(&(ptr_struct->port_state_change_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpio_event:\n");
	connectib_gpio_event_print(&(ptr_struct->gpio_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_module_event:\n");
	connectib_port_module_event_print(&(ptr_struct->port_module_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcbx_change_event:\n");
	connectib_dcbx_change_event_print(&(ptr_struct->dcbx_change_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "temp_warning_event:\n");
	connectib_temp_warning_event_print(&(ptr_struct->temp_warning_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_bf_congestion_event:\n");
	connectib_db_bf_congestion_event_print(&(ptr_struct->db_bf_congestion_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "stall_vl_event:\n");
	connectib_stall_vl_event_print(&(ptr_struct->stall_vl_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmd_inter_comp_event:\n");
	connectib_cmd_inter_comp_event_print(&(ptr_struct->cmd_inter_comp_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pages_req_event:\n");
	connectib_pages_req_event_print(&(ptr_struct->pages_req_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nic_vport_change_event:\n");
	connectib_nic_vport_change_event_print(&(ptr_struct->nic_vport_change_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_error_event:\n");
	connectib_fpga_error_event_print(&(ptr_struct->fpga_error_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga_qp_error_event:\n");
	connectib_fpga_qp_error_event_print(&(ptr_struct->fpga_qp_error_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pps_event:\n");
	connectib_pps_event_print(&(ptr_struct->pps_event), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "general_notification_delay_drop_event:\n");
	connectib_general_notification_delay_drop_event_print(&(ptr_struct->general_notification_delay_drop_event), file, indent_level + 1);

}

int connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto_size(void)
{
	 return 28;
}

void connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto_dump(const union connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto *ptr_struct, FILE* file)
{
	connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto_print(ptr_struct, file, 0);
}

void connectib_icmd_generic_reg_access_reg_tlv_data_pack(const union connectib_icmd_generic_reg_access_reg_tlv_data *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_generic_flash_reg_access_data_pack(&(ptr_struct->flash_reg_access), ptr_buff);
}

void connectib_icmd_generic_reg_access_reg_tlv_data_unpack(union connectib_icmd_generic_reg_access_reg_tlv_data *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_generic_flash_reg_access_data_unpack(&(ptr_struct->flash_reg_access), ptr_buff);
}

void connectib_icmd_generic_reg_access_reg_tlv_data_print(const union connectib_icmd_generic_reg_access_reg_tlv_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_generic_reg_access_reg_tlv_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmdif_access_register:\n");
	connectib_access_register_data_layout_print(&(ptr_struct->cmdif_access_register), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_reg_access:\n");
	connectib_generic_flash_reg_access_data_print(&(ptr_struct->flash_reg_access), file, indent_level + 1);

}

int connectib_icmd_generic_reg_access_reg_tlv_data_size(void)
{
	 return 272;
}

void connectib_icmd_generic_reg_access_reg_tlv_data_dump(const union connectib_icmd_generic_reg_access_reg_tlv_data *ptr_struct, FILE* file)
{
	connectib_icmd_generic_reg_access_reg_tlv_data_print(ptr_struct, file, 0);
}

void connectib_reg_access_tlv_pack(const struct connectib_reg_access_tlv *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->len);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->type);

}

void connectib_reg_access_tlv_unpack(struct connectib_reg_access_tlv *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	ptr_struct->len = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

	offset = 0;
	ptr_struct->type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_reg_access_tlv_print(const struct connectib_reg_access_tlv *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_reg_access_tlv ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "len                  : " UH_FMT "\n", ptr_struct->len);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " UH_FMT "\n", ptr_struct->type);

}

int connectib_reg_access_tlv_size(void)
{
	 return 4;
}

void connectib_reg_access_tlv_dump(const struct connectib_reg_access_tlv *ptr_struct, FILE* file)
{
	connectib_reg_access_tlv_print(ptr_struct, file, 0);
}

void connectib_IB_TID_pack(const struct connectib_IB_TID *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->High);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->Low);

}

void connectib_IB_TID_unpack(struct connectib_IB_TID *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->High = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->Low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_IB_TID_print(const struct connectib_IB_TID *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_IB_TID ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "High                 : " U32H_FMT "\n", ptr_struct->High);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Low                  : " U32H_FMT "\n", ptr_struct->Low);

}

int connectib_IB_TID_size(void)
{
	 return 8;
}

void connectib_IB_TID_dump(const struct connectib_IB_TID *ptr_struct, FILE* file)
{
	connectib_IB_TID_print(ptr_struct, file, 0);
}

void connectib_icmd_kdnet_ctrl_out_pack(const struct connectib_icmd_kdnet_ctrl_out *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->kdnet_bdf);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->status);

}

void connectib_icmd_kdnet_ctrl_out_unpack(struct connectib_icmd_kdnet_ctrl_out *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->kdnet_bdf = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_kdnet_ctrl_out_print(const struct connectib_icmd_kdnet_ctrl_out *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_kdnet_ctrl_out ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "kdnet_bdf            : " UH_FMT "\n", ptr_struct->kdnet_bdf);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " UH_FMT "\n", ptr_struct->status);

}

int connectib_icmd_kdnet_ctrl_out_size(void)
{
	 return 4;
}

void connectib_icmd_kdnet_ctrl_out_dump(const struct connectib_icmd_kdnet_ctrl_out *ptr_struct, FILE* file)
{
	connectib_icmd_kdnet_ctrl_out_print(ptr_struct, file, 0);
}

void connectib_icmd_kdnet_ctrl_in_pack(const struct connectib_icmd_kdnet_ctrl_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->operation);

}

void connectib_icmd_kdnet_ctrl_in_unpack(struct connectib_icmd_kdnet_ctrl_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->operation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_kdnet_ctrl_in_print(const struct connectib_icmd_kdnet_ctrl_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_kdnet_ctrl_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "operation            : " UH_FMT "\n", ptr_struct->operation);

}

int connectib_icmd_kdnet_ctrl_in_size(void)
{
	 return 4;
}

void connectib_icmd_kdnet_ctrl_in_dump(const struct connectib_icmd_kdnet_ctrl_in *ptr_struct, FILE* file)
{
	connectib_icmd_kdnet_ctrl_in_print(ptr_struct, file, 0);
}

void connectib_itrace_ctrl_pack(const struct connectib_itrace_ctrl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->log_level);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->log_delay);

}

void connectib_itrace_ctrl_unpack(struct connectib_itrace_ctrl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->log_level = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->log_delay = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_itrace_ctrl_print(const struct connectib_itrace_ctrl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_itrace_ctrl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_level            : " UH_FMT "\n", ptr_struct->log_level);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_delay            : " UH_FMT "\n", ptr_struct->log_delay);

}

int connectib_itrace_ctrl_size(void)
{
	 return 4;
}

void connectib_itrace_ctrl_dump(const struct connectib_itrace_ctrl *ptr_struct, FILE* file)
{
	connectib_itrace_ctrl_print(ptr_struct, file, 0);
}

void connectib_output_inline_standart_pack(const struct connectib_output_inline_standart *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->auto_1);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->status);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->syndrome);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->ix);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->auto_2);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->auto_3);

}

void connectib_output_inline_standart_unpack(struct connectib_output_inline_standart *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->auto_1 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->syndrome = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->ix = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->auto_2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->auto_3 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_output_inline_standart_print(const struct connectib_output_inline_standart *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_output_inline_standart ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "auto_1               : " UH_FMT "\n", ptr_struct->auto_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " UH_FMT "\n", ptr_struct->status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "syndrome             : " U32H_FMT "\n", ptr_struct->syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ix                   : " UH_FMT "\n", ptr_struct->ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "auto_2               : " UH_FMT "\n", ptr_struct->auto_2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "auto_3               : " U32H_FMT "\n", ptr_struct->auto_3);

}

int connectib_output_inline_standart_size(void)
{
	 return 16;
}

void connectib_output_inline_standart_dump(const struct connectib_output_inline_standart *ptr_struct, FILE* file)
{
	connectib_output_inline_standart_print(ptr_struct, file, 0);
}

void connectib_input_inline_standart_pack(const struct connectib_input_inline_standart *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->reserved_0);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->opcode);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->opcode_mod);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->reserved_1);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->ix);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->reserved_2);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->reserved_3);

}

void connectib_input_inline_standart_unpack(struct connectib_input_inline_standart *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->reserved_0 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->opcode = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	ptr_struct->opcode_mod = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->reserved_1 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 72;
	ptr_struct->ix = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->reserved_2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->reserved_3 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_input_inline_standart_print(const struct connectib_input_inline_standart *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_input_inline_standart ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved_0           : " UH_FMT "\n", ptr_struct->reserved_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " UH_FMT "\n", ptr_struct->opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode_mod           : " UH_FMT "\n", ptr_struct->opcode_mod);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved_1           : " UH_FMT "\n", ptr_struct->reserved_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ix                   : " UH_FMT "\n", ptr_struct->ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved_2           : " UH_FMT "\n", ptr_struct->reserved_2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved_3           : " U32H_FMT "\n", ptr_struct->reserved_3);

}

int connectib_input_inline_standart_size(void)
{
	 return 16;
}

void connectib_input_inline_standart_dump(const struct connectib_input_inline_standart *ptr_struct, FILE* file)
{
	connectib_input_inline_standart_print(ptr_struct, file, 0);
}

void connectib_g_rse_decoders_as_dwords_pack(const struct connectib_g_rse_decoders_as_dwords *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2048, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword[i]);
	}

}

void connectib_g_rse_decoders_as_dwords_unpack(struct connectib_g_rse_decoders_as_dwords *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2048, 1);
	ptr_struct->dword[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_g_rse_decoders_as_dwords_print(const struct connectib_g_rse_decoders_as_dwords *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_g_rse_decoders_as_dwords ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_%03d           : " U32H_FMT "\n", i, ptr_struct->dword[i]);
	}

}

int connectib_g_rse_decoders_as_dwords_size(void)
{
	 return 256;
}

void connectib_g_rse_decoders_as_dwords_dump(const struct connectib_g_rse_decoders_as_dwords *ptr_struct, FILE* file)
{
	connectib_g_rse_decoders_as_dwords_print(ptr_struct, file, 0);
}

void connectib_rxs_rxt_desc_decoder_pack(const struct connectib_rxs_rxt_desc_decoder *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 50;
	adb2c_push_bits_to_buff(ptr_buff, offset, 14, (u_int32_t)ptr_struct->payload_end_offset);

	offset = 49;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mask_px_icmc_tpt_vld);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->need_invalidation);

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->extended_rdb_size);

	offset = 41;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->max_rdma_atomic);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->scatter_grh);

	offset = 37;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->packet_type);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->can_use_reserved_lkey);

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->use_dcr_gvmi_for_rdb);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 66;
	adb2c_push_bits_to_buff(ptr_buff, offset, 14, (u_int32_t)ptr_struct->packet_len);

	offset = 65;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->free_line);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->header_capture);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->rdb_write_pointer);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->psn);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->payload_offset);

	offset = 168;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->pd);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->grh_offset);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rkey);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dma_length);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->va_63_32);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->va_31_0);

}

void connectib_rxs_rxt_desc_decoder_unpack(struct connectib_rxs_rxt_desc_decoder *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 50;
	ptr_struct->payload_end_offset = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 14);

	offset = 49;
	ptr_struct->mask_px_icmc_tpt_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->need_invalidation = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 44;
	ptr_struct->extended_rdb_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 41;
	ptr_struct->max_rdma_atomic = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 40;
	ptr_struct->scatter_grh = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 37;
	ptr_struct->packet_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 36;
	ptr_struct->can_use_reserved_lkey = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 35;
	ptr_struct->use_dcr_gvmi_for_rdb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 66;
	ptr_struct->packet_len = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 14);

	offset = 65;
	ptr_struct->free_line = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->header_capture = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 104;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->rdb_write_pointer = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->psn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 128;
	ptr_struct->payload_offset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 168;
	ptr_struct->pd = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 160;
	ptr_struct->grh_offset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 192;
	ptr_struct->rkey = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->dma_length = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->va_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->va_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_rxs_rxt_desc_decoder_print(const struct connectib_rxs_rxt_desc_decoder *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rxs_rxt_desc_decoder ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "payload_end_offset   : " UH_FMT "\n", ptr_struct->payload_end_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mask_px_icmc_tpt_vld : " UH_FMT "\n", ptr_struct->mask_px_icmc_tpt_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "need_invalidation    : " UH_FMT "\n", ptr_struct->need_invalidation);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_rdb_size    : " UH_FMT "\n", ptr_struct->extended_rdb_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_rdma_atomic      : " UH_FMT "\n", ptr_struct->max_rdma_atomic);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "scatter_grh          : " UH_FMT "\n", ptr_struct->scatter_grh);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_type          : " UH_FMT "\n", ptr_struct->packet_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "can_use_reserved_lkey : " UH_FMT "\n", ptr_struct->can_use_reserved_lkey);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "use_dcr_gvmi_for_rdb : " UH_FMT "\n", ptr_struct->use_dcr_gvmi_for_rdb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_len           : " UH_FMT "\n", ptr_struct->packet_len);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "free_line            : " UH_FMT "\n", ptr_struct->free_line);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "header_capture       : " UH_FMT "\n", ptr_struct->header_capture);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdb_write_pointer    : " UH_FMT "\n", ptr_struct->rdb_write_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "psn                  : " UH_FMT "\n", ptr_struct->psn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "payload_offset       : " UH_FMT "\n", ptr_struct->payload_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pd                   : " UH_FMT "\n", ptr_struct->pd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh_offset           : " UH_FMT "\n", ptr_struct->grh_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rkey                 : " U32H_FMT "\n", ptr_struct->rkey);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dma_length           : " U32H_FMT "\n", ptr_struct->dma_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "va_63_32             : " U32H_FMT "\n", ptr_struct->va_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "va_31_0              : " U32H_FMT "\n", ptr_struct->va_31_0);

}

int connectib_rxs_rxt_desc_decoder_size(void)
{
	 return 64;
}

void connectib_rxs_rxt_desc_decoder_dump(const struct connectib_rxs_rxt_desc_decoder *ptr_struct, FILE* file)
{
	connectib_rxs_rxt_desc_decoder_print(ptr_struct, file, 0);
}

void connectib_rxt_context_fetch_qp_desc_decoder_pack(const struct connectib_rxt_context_fetch_qp_desc_decoder *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qpc_ts_3_3);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port_number);

	offset = 51;
	adb2c_push_bits_to_buff(ptr_buff, offset, 13, (u_int32_t)ptr_struct->srq_number_23_11);

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->qpc_ts_2_0);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->pkey_index);

	offset = 76;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->my_gid_index);

	offset = 75;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->srq_vld);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->srq_number_10_0);

}

void connectib_rxt_context_fetch_qp_desc_decoder_unpack(struct connectib_rxt_context_fetch_qp_desc_decoder *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->qpc_ts_3_3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->port_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 51;
	ptr_struct->srq_number_23_11 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 13);

	offset = 35;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->qpc_ts_2_0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 84;
	ptr_struct->pkey_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 76;
	ptr_struct->my_gid_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 75;
	ptr_struct->srq_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->srq_number_10_0 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

}

void connectib_rxt_context_fetch_qp_desc_decoder_print(const struct connectib_rxt_context_fetch_qp_desc_decoder *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rxt_context_fetch_qp_desc_decoder ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpc_ts_3_3           : " UH_FMT "\n", ptr_struct->qpc_ts_3_3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number          : " UH_FMT "\n", ptr_struct->port_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_number_23_11     : " UH_FMT "\n", ptr_struct->srq_number_23_11);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpc_ts_2_0           : " UH_FMT "\n", ptr_struct->qpc_ts_2_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey_index           : " UH_FMT "\n", ptr_struct->pkey_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "my_gid_index         : " UH_FMT "\n", ptr_struct->my_gid_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_vld              : " UH_FMT "\n", ptr_struct->srq_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_number_10_0      : " UH_FMT "\n", ptr_struct->srq_number_10_0);

}

int connectib_rxt_context_fetch_qp_desc_decoder_size(void)
{
	 return 16;
}

void connectib_rxt_context_fetch_qp_desc_decoder_dump(const struct connectib_rxt_context_fetch_qp_desc_decoder *ptr_struct, FILE* file)
{
	connectib_rxt_context_fetch_qp_desc_decoder_print(ptr_struct, file, 0);
}

void connectib_rxt_checks_steering_results_decoder_pack(const struct connectib_rxt_checks_steering_results_decoder *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vport_packet_counters_trigger);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->xqpn_40_32);

	offset = 54;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pport_counters_trigger_vld);

	offset = 53;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qpn_direct_access);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vport_counters_trigger_vld);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vport_byte_counters_trigger);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pport_byte_counters_trigger);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pport_packet_counters_trigger);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->xqpn_31_0);

}

void connectib_rxt_checks_steering_results_decoder_unpack(struct connectib_rxt_checks_steering_results_decoder *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->vport_packet_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 55;
	ptr_struct->xqpn_40_32 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 54;
	ptr_struct->pport_counters_trigger_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 53;
	ptr_struct->qpn_direct_access = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 52;
	ptr_struct->vport_counters_trigger_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->vport_byte_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 40;
	ptr_struct->pport_byte_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->pport_packet_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->xqpn_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_rxt_checks_steering_results_decoder_print(const struct connectib_rxt_checks_steering_results_decoder *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rxt_checks_steering_results_decoder ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_packet_counters_trigger : " UH_FMT "\n", ptr_struct->vport_packet_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xqpn_40_32           : " UH_FMT "\n", ptr_struct->xqpn_40_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pport_counters_trigger_vld : " UH_FMT "\n", ptr_struct->pport_counters_trigger_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn_direct_access    : " UH_FMT "\n", ptr_struct->qpn_direct_access);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_counters_trigger_vld : " UH_FMT "\n", ptr_struct->vport_counters_trigger_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_byte_counters_trigger : " UH_FMT "\n", ptr_struct->vport_byte_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pport_byte_counters_trigger : " UH_FMT "\n", ptr_struct->pport_byte_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pport_packet_counters_trigger : " UH_FMT "\n", ptr_struct->pport_packet_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xqpn_31_0            : " U32H_FMT "\n", ptr_struct->xqpn_31_0);

}

int connectib_rxt_checks_steering_results_decoder_size(void)
{
	 return 16;
}

void connectib_rxt_checks_steering_results_decoder_dump(const struct connectib_rxt_checks_steering_results_decoder *ptr_struct, FILE* file)
{
	connectib_rxt_checks_steering_results_decoder_print(ptr_struct, file, 0);
}

void connectib_rxt_steering_context_decoder_pack(const struct connectib_rxt_steering_context_decoder *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log2_hit_table_size);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->take_qp_from_packet);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vport_byte_counters_trigger);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vport_packet_counters_trigger);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pport_packet_counters_trigger);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->log2_hit_entry_size);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->first_line_of_steering_entry);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_sniffer_accomulated);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qp_list_index);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pport_byte_counters_trigger);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->member_count);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hit_lookup_type_hash_field_mask);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->hit_base_address_55_32);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hit_lookup_type);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hit_base_address_31_0);

}

void connectib_rxt_steering_context_decoder_unpack(struct connectib_rxt_steering_context_decoder *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->log2_hit_table_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 25;
	ptr_struct->take_qp_from_packet = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->vport_byte_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 16;
	ptr_struct->vport_packet_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->pport_packet_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 34;
	ptr_struct->log2_hit_entry_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 33;
	ptr_struct->first_line_of_steering_entry = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->sx_sniffer_accomulated = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->qp_list_index = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 64;
	ptr_struct->pport_byte_counters_trigger = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->member_count = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 96;
	ptr_struct->hit_lookup_type_hash_field_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->hit_base_address_55_32 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 128;
	ptr_struct->hit_lookup_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 160;
	ptr_struct->hit_base_address_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_rxt_steering_context_decoder_print(const struct connectib_rxt_steering_context_decoder *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rxt_steering_context_decoder ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_hit_table_size  : " UH_FMT "\n", ptr_struct->log2_hit_table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "take_qp_from_packet  : " UH_FMT "\n", ptr_struct->take_qp_from_packet);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_byte_counters_trigger : " UH_FMT "\n", ptr_struct->vport_byte_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_packet_counters_trigger : " UH_FMT "\n", ptr_struct->vport_packet_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pport_packet_counters_trigger : " UH_FMT "\n", ptr_struct->pport_packet_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_hit_entry_size  : " UH_FMT "\n", ptr_struct->log2_hit_entry_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_line_of_steering_entry : " UH_FMT "\n", ptr_struct->first_line_of_steering_entry);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_sniffer_accomulated : " UH_FMT "\n", ptr_struct->sx_sniffer_accomulated);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_list_index        : " UH_FMT "\n", ptr_struct->qp_list_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pport_byte_counters_trigger : " UH_FMT "\n", ptr_struct->pport_byte_counters_trigger);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "member_count         : " UH_FMT "\n", ptr_struct->member_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hit_lookup_type_hash_field_mask : " UH_FMT "\n", ptr_struct->hit_lookup_type_hash_field_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hit_base_address_55_32 : " UH_FMT "\n", ptr_struct->hit_base_address_55_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hit_lookup_type      : " UH_FMT "\n", ptr_struct->hit_lookup_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hit_base_address_31_0 : " U32H_FMT "\n", ptr_struct->hit_base_address_31_0);

}

int connectib_rxt_steering_context_decoder_size(void)
{
	 return 32;
}

void connectib_rxt_steering_context_decoder_dump(const struct connectib_rxt_steering_context_decoder *ptr_struct, FILE* file)
{
	connectib_rxt_steering_context_decoder_print(ptr_struct, file, 0);
}

void connectib_rxt_checks_packet_descriptor_decoder_pack(const struct connectib_rxt_checks_packet_descriptor_decoder *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->timestamp_version);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_loopback);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->functional_loopback);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->inner_crc_ok);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mad_baseversion_error);

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mad_classversion_error);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mad_send2host);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->outer_crc_ok);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port_error);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_sniffer);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->timestamp_vld);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->payload_offset);

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->custom_dw0_vld);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->custom_dw1_vld);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->custom_dw2_vld);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->custom_dw3_vld);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->custom_dw3);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->custom_dw2);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->custom_dw1);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->custom_dw0);

	offset = 215;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->source_xqpn_40_32);

	offset = 214;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dummy_packet);

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->mad_qp1_class);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->packet_end_offset);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->source_xqpn_31_0);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->timestamp_63_32);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->timestamp_31_0);

}

void connectib_rxt_checks_packet_descriptor_decoder_unpack(struct connectib_rxt_checks_packet_descriptor_decoder *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	ptr_struct->timestamp_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 29;
	ptr_struct->force_loopback = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->functional_loopback = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 27;
	ptr_struct->inner_crc_ok = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->mad_baseversion_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 25;
	ptr_struct->mad_classversion_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->mad_send2host = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->outer_crc_ok = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->port_error = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->sx_sniffer = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->timestamp_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->payload_offset = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 44;
	ptr_struct->custom_dw0_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 40;
	ptr_struct->custom_dw1_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 36;
	ptr_struct->custom_dw2_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->custom_dw3_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->custom_dw3 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->custom_dw2 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->custom_dw1 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->custom_dw0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 215;
	ptr_struct->source_xqpn_40_32 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 214;
	ptr_struct->dummy_packet = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 208;
	ptr_struct->mad_qp1_class = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 192;
	ptr_struct->packet_end_offset = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 224;
	ptr_struct->source_xqpn_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->timestamp_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->timestamp_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_rxt_checks_packet_descriptor_decoder_print(const struct connectib_rxt_checks_packet_descriptor_decoder *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rxt_checks_packet_descriptor_decoder ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_version    : " UH_FMT "\n", ptr_struct->timestamp_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_loopback       : " UH_FMT "\n", ptr_struct->force_loopback);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "functional_loopback  : " UH_FMT "\n", ptr_struct->functional_loopback);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner_crc_ok         : " UH_FMT "\n", ptr_struct->inner_crc_ok);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mad_baseversion_error : " UH_FMT "\n", ptr_struct->mad_baseversion_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mad_classversion_error : " UH_FMT "\n", ptr_struct->mad_classversion_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mad_send2host        : " UH_FMT "\n", ptr_struct->mad_send2host);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "outer_crc_ok         : " UH_FMT "\n", ptr_struct->outer_crc_ok);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_error           : " UH_FMT "\n", ptr_struct->port_error);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_sniffer           : " UH_FMT "\n", ptr_struct->sx_sniffer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_vld        : " UH_FMT "\n", ptr_struct->timestamp_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "payload_offset       : " UH_FMT "\n", ptr_struct->payload_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "custom_dw0_vld       : " UH_FMT "\n", ptr_struct->custom_dw0_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "custom_dw1_vld       : " UH_FMT "\n", ptr_struct->custom_dw1_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "custom_dw2_vld       : " UH_FMT "\n", ptr_struct->custom_dw2_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "custom_dw3_vld       : " UH_FMT "\n", ptr_struct->custom_dw3_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "custom_dw3           : " U32H_FMT "\n", ptr_struct->custom_dw3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "custom_dw2           : " U32H_FMT "\n", ptr_struct->custom_dw2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "custom_dw1           : " U32H_FMT "\n", ptr_struct->custom_dw1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "custom_dw0           : " U32H_FMT "\n", ptr_struct->custom_dw0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "source_xqpn_40_32    : " UH_FMT "\n", ptr_struct->source_xqpn_40_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dummy_packet         : " UH_FMT "\n", ptr_struct->dummy_packet);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mad_qp1_class        : " UH_FMT "\n", ptr_struct->mad_qp1_class);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_end_offset    : " UH_FMT "\n", ptr_struct->packet_end_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "source_xqpn_31_0     : " U32H_FMT "\n", ptr_struct->source_xqpn_31_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_63_32      : " U32H_FMT "\n", ptr_struct->timestamp_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_31_0       : " U32H_FMT "\n", ptr_struct->timestamp_31_0);

}

int connectib_rxt_checks_packet_descriptor_decoder_size(void)
{
	 return 64;
}

void connectib_rxt_checks_packet_descriptor_decoder_dump(const struct connectib_rxt_checks_packet_descriptor_decoder *ptr_struct, FILE* file)
{
	connectib_rxt_checks_packet_descriptor_decoder_print(ptr_struct, file, 0);
}

void connectib_ctx_as_dwords_pack(const struct connectib_ctx_as_dwords *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dwords[i]);
	}

}

void connectib_ctx_as_dwords_unpack(struct connectib_ctx_as_dwords *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	ptr_struct->dwords[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_ctx_as_dwords_print(const struct connectib_ctx_as_dwords *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ctx_as_dwords ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dwords_%03d          : " U32H_FMT "\n", i, ptr_struct->dwords[i]);
	}

}

int connectib_ctx_as_dwords_size(void)
{
	 return 64;
}

void connectib_ctx_as_dwords_dump(const struct connectib_ctx_as_dwords *ptr_struct, FILE* file)
{
	connectib_ctx_as_dwords_print(ptr_struct, file, 0);
}

void connectib_fw_resource_context_pack(const union connectib_fw_resource_context *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_scheduling_context_pack(&(ptr_struct->sched_element_ctx), ptr_buff);
}

void connectib_fw_resource_context_unpack(union connectib_fw_resource_context *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_scheduling_context_unpack(&(ptr_struct->sched_element_ctx), ptr_buff);
}

void connectib_fw_resource_context_print(const union connectib_fw_resource_context *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_resource_context ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_eqc:\n");
	connectib_fw_eqc_print(&(ptr_struct->fw_eqc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_alt_path:\n");
	connectib_fw_alt_path_print(&(ptr_struct->fw_alt_path), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_basic:\n");
	connectib_fw_basic_res_ctx_print(&(ptr_struct->fw_basic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_cre:\n");
	connectib_fw_cre_print(&(ptr_struct->fw_cre), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_mkc:\n");
	connectib_fw_mkc_print(&(ptr_struct->fw_mkc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_psv:\n");
	connectib_fw_psv_print(&(ptr_struct->fw_psv), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_cqc:\n");
	connectib_fw_cqc_print(&(ptr_struct->fw_cqc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_rqt:\n");
	connectib_fw_rqt_print(&(ptr_struct->fw_rqt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "list_element:\n");
	connectib_list_element_print(&(ptr_struct->list_element), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmfs_resources:\n");
	connectib_dmfs_resources_print(&(ptr_struct->dmfs_resources), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_control_packet_desc:\n");
	connectib_dc_control_packet_desc_print(&(ptr_struct->dc_control_packet_desc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vqos_arb_ctx:\n");
	connectib_vqos_arb_ctx_print(&(ptr_struct->vqos_arb_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_transport_domain:\n");
	connectib_fw_transport_domain_print(&(ptr_struct->fw_transport_domain), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_q_counters:\n");
	connectib_fw_q_counters_print(&(ptr_struct->fw_q_counters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ipoib_meta:\n");
	connectib_fw_ipoib_meta_print(&(ptr_struct->fw_ipoib_meta), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sched_element_ctx:\n");
	connectib_scheduling_context_print(&(ptr_struct->sched_element_ctx), file, indent_level + 1);

}

int connectib_fw_resource_context_size(void)
{
	 return 64;
}

void connectib_fw_resource_context_dump(const union connectib_fw_resource_context *ptr_struct, FILE* file)
{
	connectib_fw_resource_context_print(ptr_struct, file, 0);
}

void connectib_hw_hca_contexts_pack(const union connectib_hw_hca_contexts *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_hw_qp_counter_pack(&(ptr_struct->qp_counter), ptr_buff);
}

void connectib_hw_hca_contexts_unpack(union connectib_hw_hca_contexts *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_hw_qp_counter_unpack(&(ptr_struct->qp_counter), ptr_buff);
}

void connectib_hw_hca_contexts_print(const union connectib_hw_hca_contexts *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_hca_contexts ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_c_cqc:\n");
	connectib_hw_c_cqc_print(&(ptr_struct->hw_c_cqc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_c_eqc:\n");
	connectib_hw_c_eqc_print(&(ptr_struct->hw_c_eqc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mkc:\n");
	connectib_hw_mkc_print(&(ptr_struct->mkc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "toc:\n");
	connectib_hw_toc_print(&(ptr_struct->toc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_c_srq:\n");
	connectib_hw_c_srq_print(&(ptr_struct->hw_c_srq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dct:\n");
	connectib_hw_dct_print(&(ptr_struct->dct), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "c_qpc:\n");
	connectib_hw_c_qpc_print(&(ptr_struct->c_qpc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "s_qpc:\n");
	connectib_hw_s_qpc_print(&(ptr_struct->s_qpc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "r_qpc:\n");
	connectib_hw_qpc_r_dct_print(&(ptr_struct->r_qpc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_info:\n");
	connectib_hw_port_info_print(&(ptr_struct->port_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bsf_entry:\n");
	connectib_hw_bsf_entry_print(&(ptr_struct->bsf_entry), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "base_psv:\n");
	connectib_hw_base_psv_print(&(ptr_struct->base_psv), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trastb:\n");
	connectib_hw_trastb_print(&(ptr_struct->trastb), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trasta:\n");
	connectib_hw_trasta_print(&(ptr_struct->trasta), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq_pointers:\n");
	connectib_hw_sq_pointers_print(&(ptr_struct->sq_pointers), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_tokens:\n");
	connectib_hw_gvmi_tokens_print(&(ptr_struct->gvmi_tokens), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq_tokens:\n");
	connectib_hw_sq_tokens_print(&(ptr_struct->sq_tokens), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "klm_entry:\n");
	connectib_hw_klm_entry_print(&(ptr_struct->klm_entry), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sxdc:\n");
	connectib_hw_sxdc_print(&(ptr_struct->sxdc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pipa:\n");
	connectib_hw_pipa_print(&(ptr_struct->pipa), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq_qp_list:\n");
	connectib_hw_sq_qp_list_print(&(ptr_struct->sq_qp_list), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timer:\n");
	connectib_hw_timer_32_print(&(ptr_struct->timer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkey:\n");
	connectib_hw_pkey_print(&(ptr_struct->pkey), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_ste:\n");
	connectib_hw_ste_print(&(ptr_struct->hw_ste), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_entry_icmc:\n");
	connectib_steering_entry_icmc_print(&(ptr_struct->steering_entry_icmc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtt:\n");
	connectib_icm_mtt_print(&(ptr_struct->mtt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdb:\n");
	connectib_hw_rdb_print(&(ptr_struct->rdb), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msix:\n");
	connectib_hw_msix_print(&(ptr_struct->msix), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp_counter:\n");
	connectib_hw_qp_counter_print(&(ptr_struct->qp_counter), file, indent_level + 1);

}

int connectib_hw_hca_contexts_size(void)
{
	 return 64;
}

void connectib_hw_hca_contexts_dump(const union connectib_hw_hca_contexts *ptr_struct, FILE* file)
{
	connectib_hw_hca_contexts_print(ptr_struct, file, 0);
}

void connectib_field_boundary_pack(const struct connectib_field_boundary *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->field_boundary);

}

void connectib_field_boundary_unpack(struct connectib_field_boundary *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->field_boundary = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_field_boundary_print(const struct connectib_field_boundary *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_field_boundary ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "field_boundary       : " U32H_FMT "\n", ptr_struct->field_boundary);

}

int connectib_field_boundary_size(void)
{
	 return 4;
}

void connectib_field_boundary_dump(const struct connectib_field_boundary *ptr_struct, FILE* file)
{
	connectib_field_boundary_print(ptr_struct, file, 0);
}

void connectib_add_data_pack(const struct connectib_add_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->add_data);

}

void connectib_add_data_unpack(struct connectib_add_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->add_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_add_data_print(const struct connectib_add_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_add_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "add_data             : " U32H_FMT "\n", ptr_struct->add_data);

}

int connectib_add_data_size(void)
{
	 return 4;
}

void connectib_add_data_dump(const struct connectib_add_data *ptr_struct, FILE* file)
{
	connectib_add_data_print(ptr_struct, file, 0);
}

void connectib_compare_mask_data_pack(const struct connectib_compare_mask_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->compare_mask_data);

}

void connectib_compare_mask_data_unpack(struct connectib_compare_mask_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->compare_mask_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_compare_mask_data_print(const struct connectib_compare_mask_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_compare_mask_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_mask_data    : " U32H_FMT "\n", ptr_struct->compare_mask_data);

}

int connectib_compare_mask_data_size(void)
{
	 return 4;
}

void connectib_compare_mask_data_dump(const struct connectib_compare_mask_data *ptr_struct, FILE* file)
{
	connectib_compare_mask_data_print(ptr_struct, file, 0);
}

void connectib_swap_mask_data_pack(const struct connectib_swap_mask_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->swap_mask_data);

}

void connectib_swap_mask_data_unpack(struct connectib_swap_mask_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->swap_mask_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_swap_mask_data_print(const struct connectib_swap_mask_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_swap_mask_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_mask_data       : " U32H_FMT "\n", ptr_struct->swap_mask_data);

}

int connectib_swap_mask_data_size(void)
{
	 return 4;
}

void connectib_swap_mask_data_dump(const struct connectib_swap_mask_data *ptr_struct, FILE* file)
{
	connectib_swap_mask_data_print(ptr_struct, file, 0);
}

void connectib_compare_data_pack(const struct connectib_compare_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->compare_data);

}

void connectib_compare_data_unpack(struct connectib_compare_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->compare_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_compare_data_print(const struct connectib_compare_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_compare_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data         : " U32H_FMT "\n", ptr_struct->compare_data);

}

int connectib_compare_data_size(void)
{
	 return 4;
}

void connectib_compare_data_dump(const struct connectib_compare_data *ptr_struct, FILE* file)
{
	connectib_compare_data_print(ptr_struct, file, 0);
}

void connectib_swap_data_pack(const struct connectib_swap_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->swap_data);

}

void connectib_swap_data_unpack(struct connectib_swap_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->swap_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_swap_data_print(const struct connectib_swap_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_swap_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data            : " U32H_FMT "\n", ptr_struct->swap_data);

}

int connectib_swap_data_size(void)
{
	 return 4;
}

void connectib_swap_data_dump(const struct connectib_swap_data *ptr_struct, FILE* file)
{
	connectib_swap_data_print(ptr_struct, file, 0);
}

void connectib_component_authentication_configuration_pack(const struct connectib_component_authentication_configuration *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->auth_type);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->frc_en);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mlnx_nvconfig_en);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vendor_nvconfig_en);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cs_token_en);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fw_en);

}

void connectib_component_authentication_configuration_unpack(struct connectib_component_authentication_configuration *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->auth_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 4;
	ptr_struct->frc_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->mlnx_nvconfig_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->vendor_nvconfig_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->cs_token_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->fw_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_component_authentication_configuration_print(const struct connectib_component_authentication_configuration *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_component_authentication_configuration ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "auth_type            : " UH_FMT "\n", ptr_struct->auth_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "frc_en               : " UH_FMT "\n", ptr_struct->frc_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlnx_nvconfig_en     : " UH_FMT "\n", ptr_struct->mlnx_nvconfig_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vendor_nvconfig_en   : " UH_FMT "\n", ptr_struct->vendor_nvconfig_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cs_token_en          : " UH_FMT "\n", ptr_struct->cs_token_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_en                : " UH_FMT "\n", ptr_struct->fw_en);

}

int connectib_component_authentication_configuration_size(void)
{
	 return 4;
}

void connectib_component_authentication_configuration_dump(const struct connectib_component_authentication_configuration *ptr_struct, FILE* file)
{
	connectib_component_authentication_configuration_print(ptr_struct, file, 0);
}

void connectib_nv_data_pointer_pack(const struct connectib_nv_data_pointer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->offset_in_64_bytes);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->size_in_64_bytes);

}

void connectib_nv_data_pointer_unpack(struct connectib_nv_data_pointer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->offset_in_64_bytes = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 6;
	ptr_struct->size_in_64_bytes = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

}

void connectib_nv_data_pointer_print(const struct connectib_nv_data_pointer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_data_pointer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset_in_64_bytes   : " UH_FMT "\n", ptr_struct->offset_in_64_bytes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size_in_64_bytes     : " UH_FMT "\n", ptr_struct->size_in_64_bytes);

}

int connectib_nv_data_pointer_size(void)
{
	 return 4;
}

void connectib_nv_data_pointer_dump(const struct connectib_nv_data_pointer *ptr_struct, FILE* file)
{
	connectib_nv_data_pointer_print(ptr_struct, file, 0);
}

void connectib_jump_addresses_pack(const struct connectib_jump_addresses *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->plastic);

}

void connectib_jump_addresses_unpack(struct connectib_jump_addresses *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->plastic = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_jump_addresses_print(const struct connectib_jump_addresses *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_jump_addresses ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "plastic              : " UH_FMT "\n", ptr_struct->plastic);

}

int connectib_jump_addresses_size(void)
{
	 return 28;
}

void connectib_jump_addresses_dump(const struct connectib_jump_addresses *ptr_struct, FILE* file)
{
	connectib_jump_addresses_print(ptr_struct, file, 0);
}

void connectib_pcie_rx_adaptation_cfg_pack(const struct connectib_pcie_rx_adaptation_cfg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->actual_ffe_tap_config_num);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ffe_tap_value_lifting_en);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->redo_calibration);

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(32, 64, i, 544, 1);
	connectib_ffe_tap_set_pack(&(ptr_struct->ffe_tap_config[i]), ptr_buff + offset/8);
	}

}

void connectib_pcie_rx_adaptation_cfg_unpack(struct connectib_pcie_rx_adaptation_cfg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->actual_ffe_tap_config_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 27;
	ptr_struct->ffe_tap_value_lifting_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->redo_calibration = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(32, 64, i, 544, 1);
	connectib_ffe_tap_set_unpack(&(ptr_struct->ffe_tap_config[i]), ptr_buff + offset/8);
	}

}

void connectib_pcie_rx_adaptation_cfg_print(const struct connectib_pcie_rx_adaptation_cfg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_rx_adaptation_cfg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "actual_ffe_tap_config_num : " UH_FMT "\n", ptr_struct->actual_ffe_tap_config_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap_value_lifting_en : " UH_FMT "\n", ptr_struct->ffe_tap_value_lifting_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "redo_calibration     : " UH_FMT "\n", ptr_struct->redo_calibration);

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffe_tap_config_%03d:\n", i);
	connectib_ffe_tap_set_print(&(ptr_struct->ffe_tap_config[i]), file, indent_level + 1);
	}

}

int connectib_pcie_rx_adaptation_cfg_size(void)
{
	 return 68;
}

void connectib_pcie_rx_adaptation_cfg_dump(const struct connectib_pcie_rx_adaptation_cfg *ptr_struct, FILE* file)
{
	connectib_pcie_rx_adaptation_cfg_print(ptr_struct, file, 0);
}

void connectib_pcie_tx_adaptation_cfg_pack(const struct connectib_pcie_tx_adaptation_cfg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_adaptation_config_num);

	for (i = 0; i < 3; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 128, 1);
	connectib_GEN3_TRANSMITTER_SETTING_pack(&(ptr_struct->tx_adaptation_ts_param[i]), ptr_buff + offset/8);
	}

}

void connectib_pcie_tx_adaptation_cfg_unpack(struct connectib_pcie_tx_adaptation_cfg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->tx_adaptation_config_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 3; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 128, 1);
	connectib_GEN3_TRANSMITTER_SETTING_unpack(&(ptr_struct->tx_adaptation_ts_param[i]), ptr_buff + offset/8);
	}

}

void connectib_pcie_tx_adaptation_cfg_print(const struct connectib_pcie_tx_adaptation_cfg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_tx_adaptation_cfg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_adaptation_config_num : " UH_FMT "\n", ptr_struct->tx_adaptation_config_num);

	for (i = 0; i < 3; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_adaptation_ts_param_%03d:\n", i);
	connectib_GEN3_TRANSMITTER_SETTING_print(&(ptr_struct->tx_adaptation_ts_param[i]), file, indent_level + 1);
	}

}

int connectib_pcie_tx_adaptation_cfg_size(void)
{
	 return 16;
}

void connectib_pcie_tx_adaptation_cfg_dump(const struct connectib_pcie_tx_adaptation_cfg *ptr_struct, FILE* file)
{
	connectib_pcie_tx_adaptation_cfg_print(ptr_struct, file, 0);
}

void connectib_pcie_debug_cap_pack(const struct connectib_pcie_debug_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 13, (u_int32_t)ptr_struct->internal_cause_mask);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->width_debug_en);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->speed_debug_en);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->internal_cause_en);

}

void connectib_pcie_debug_cap_unpack(struct connectib_pcie_debug_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 19;
	ptr_struct->internal_cause_mask = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 13);

	offset = 2;
	ptr_struct->width_debug_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->speed_debug_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->internal_cause_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_pcie_debug_cap_print(const struct connectib_pcie_debug_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_debug_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "internal_cause_mask  : " UH_FMT "\n", ptr_struct->internal_cause_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "width_debug_en       : " UH_FMT "\n", ptr_struct->width_debug_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "speed_debug_en       : " UH_FMT "\n", ptr_struct->speed_debug_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "internal_cause_en    : " UH_FMT "\n", ptr_struct->internal_cause_en);

}

int connectib_pcie_debug_cap_size(void)
{
	 return 4;
}

void connectib_pcie_debug_cap_dump(const struct connectib_pcie_debug_cap *ptr_struct, FILE* file)
{
	connectib_pcie_debug_cap_print(ptr_struct, file, 0);
}

void connectib_pcie_equalization_and_eye_pack(const struct connectib_pcie_equalization_and_eye *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->local_fs);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->local_lf);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->preset);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_preset);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->phase0_timeout);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->phase2_timeout);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fast_x1_eql_for_compliance);

	for (i = 0; i < 11; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 416, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gen3_preset[i]);
	}

	offset = 411;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->eye_center_mixer_offset);

	offset = 410;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->eye_center_mix90_en);

	offset = 409;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->eye_center_offset_en);

	offset = 408;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->first_offset_phase_second);

	offset = 400;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->phase_err_wait_time);

	offset = 396;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->phase_stabilize_time);

}

void connectib_pcie_equalization_and_eye_unpack(struct connectib_pcie_equalization_and_eye *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 26;
	ptr_struct->local_fs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 19;
	ptr_struct->local_lf = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 12;
	ptr_struct->preset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 11;
	ptr_struct->force_preset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->phase0_timeout = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 1;
	ptr_struct->phase2_timeout = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 0;
	ptr_struct->fast_x1_eql_for_compliance = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	for (i = 0; i < 11; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 416, 1);
	ptr_struct->gen3_preset[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 411;
	ptr_struct->eye_center_mixer_offset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 410;
	ptr_struct->eye_center_mix90_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 409;
	ptr_struct->eye_center_offset_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 408;
	ptr_struct->first_offset_phase_second = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 400;
	ptr_struct->phase_err_wait_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 396;
	ptr_struct->phase_stabilize_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_pcie_equalization_and_eye_print(const struct connectib_pcie_equalization_and_eye *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_equalization_and_eye ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_fs             : " UH_FMT "\n", ptr_struct->local_fs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_lf             : " UH_FMT "\n", ptr_struct->local_lf);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "preset               : " UH_FMT "\n", ptr_struct->preset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_preset         : " UH_FMT "\n", ptr_struct->force_preset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phase0_timeout       : " UH_FMT "\n", ptr_struct->phase0_timeout);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phase2_timeout       : " UH_FMT "\n", ptr_struct->phase2_timeout);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fast_x1_eql_for_compliance : " UH_FMT "\n", ptr_struct->fast_x1_eql_for_compliance);

	for (i = 0; i < 11; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gen3_preset_%03d     : " U32H_FMT "\n", i, ptr_struct->gen3_preset[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_center_mixer_offset : " UH_FMT "\n", ptr_struct->eye_center_mixer_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_center_mix90_en  : " UH_FMT "\n", ptr_struct->eye_center_mix90_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_center_offset_en : " UH_FMT "\n", ptr_struct->eye_center_offset_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_offset_phase_second : " UH_FMT "\n", ptr_struct->first_offset_phase_second);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phase_err_wait_time  : " UH_FMT "\n", ptr_struct->phase_err_wait_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phase_stabilize_time : " UH_FMT "\n", ptr_struct->phase_stabilize_time);

}

int connectib_pcie_equalization_and_eye_size(void)
{
	 return 52;
}

void connectib_pcie_equalization_and_eye_dump(const struct connectib_pcie_equalization_and_eye *ptr_struct, FILE* file)
{
	connectib_pcie_equalization_and_eye_print(ptr_struct, file, 0);
}

void connectib_pcie_serdes_cfg_pack(const struct connectib_pcie_serdes_cfg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->sdr_preemp);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sdr_preemp_pre);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sdr_preemp_post);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sdr_preemp_main);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->ddr_6_db_preemp);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ddr_6_db_preemp_pre);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ddr_6_db_preemp_post);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ddr_6_db_preemp_main);

	offset = 93;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->ddr_3_dot_5_db_preemp);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ddr_3_dot_5_db_preemp_pre);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ddr_3_dot_5_db_preemp_post);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ddr_3_dot_5_db_preemp_main);

	offset = 96;
	connectib_GEN3_TRANSMITTER_SETTING_pack(&(ptr_struct->gen3_def_tx_preset), ptr_buff + offset/8);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->slicer_ind_en);

	offset = 148;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->slicer1_enable);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->slicer2_enable);

	offset = 185;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->signal_detect_threshold);

	offset = 184;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->signal_detect_en);

	offset = 180;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ob_bias);

	offset = 192;
	connectib_ffe_tap_set_pack(&(ptr_struct->default_ffe_tap_set), ptr_buff + offset/8);

}

void connectib_pcie_serdes_cfg_unpack(struct connectib_pcie_serdes_cfg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 29;
	ptr_struct->sdr_preemp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 16;
	ptr_struct->sdr_preemp_pre = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->sdr_preemp_post = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->sdr_preemp_main = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 61;
	ptr_struct->ddr_6_db_preemp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 48;
	ptr_struct->ddr_6_db_preemp_pre = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 40;
	ptr_struct->ddr_6_db_preemp_post = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->ddr_6_db_preemp_main = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 93;
	ptr_struct->ddr_3_dot_5_db_preemp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 80;
	ptr_struct->ddr_3_dot_5_db_preemp_pre = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 72;
	ptr_struct->ddr_3_dot_5_db_preemp_post = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->ddr_3_dot_5_db_preemp_main = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	connectib_GEN3_TRANSMITTER_SETTING_unpack(&(ptr_struct->gen3_def_tx_preset), ptr_buff + offset/8);

	offset = 152;
	ptr_struct->slicer_ind_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 148;
	ptr_struct->slicer1_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 144;
	ptr_struct->slicer2_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 185;
	ptr_struct->signal_detect_threshold = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 184;
	ptr_struct->signal_detect_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 180;
	ptr_struct->ob_bias = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 192;
	connectib_ffe_tap_set_unpack(&(ptr_struct->default_ffe_tap_set), ptr_buff + offset/8);

}

void connectib_pcie_serdes_cfg_print(const struct connectib_pcie_serdes_cfg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_serdes_cfg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sdr_preemp           : " UH_FMT "\n", ptr_struct->sdr_preemp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sdr_preemp_pre       : " UH_FMT "\n", ptr_struct->sdr_preemp_pre);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sdr_preemp_post      : " UH_FMT "\n", ptr_struct->sdr_preemp_post);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sdr_preemp_main      : " UH_FMT "\n", ptr_struct->sdr_preemp_main);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ddr_6_db_preemp      : " UH_FMT "\n", ptr_struct->ddr_6_db_preemp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ddr_6_db_preemp_pre  : " UH_FMT "\n", ptr_struct->ddr_6_db_preemp_pre);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ddr_6_db_preemp_post : " UH_FMT "\n", ptr_struct->ddr_6_db_preemp_post);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ddr_6_db_preemp_main : " UH_FMT "\n", ptr_struct->ddr_6_db_preemp_main);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ddr_3_dot_5_db_preemp : " UH_FMT "\n", ptr_struct->ddr_3_dot_5_db_preemp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ddr_3_dot_5_db_preemp_pre : " UH_FMT "\n", ptr_struct->ddr_3_dot_5_db_preemp_pre);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ddr_3_dot_5_db_preemp_post : " UH_FMT "\n", ptr_struct->ddr_3_dot_5_db_preemp_post);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ddr_3_dot_5_db_preemp_main : " UH_FMT "\n", ptr_struct->ddr_3_dot_5_db_preemp_main);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gen3_def_tx_preset:\n");
	connectib_GEN3_TRANSMITTER_SETTING_print(&(ptr_struct->gen3_def_tx_preset), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slicer_ind_en        : " UH_FMT "\n", ptr_struct->slicer_ind_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slicer1_enable       : " UH_FMT "\n", ptr_struct->slicer1_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slicer2_enable       : " UH_FMT "\n", ptr_struct->slicer2_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signal_detect_threshold : " UH_FMT "\n", ptr_struct->signal_detect_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signal_detect_en     : " UH_FMT "\n", ptr_struct->signal_detect_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ob_bias              : " UH_FMT "\n", ptr_struct->ob_bias);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "default_ffe_tap_set:\n");
	connectib_ffe_tap_set_print(&(ptr_struct->default_ffe_tap_set), file, indent_level + 1);

}

int connectib_pcie_serdes_cfg_size(void)
{
	 return 32;
}

void connectib_pcie_serdes_cfg_dump(const struct connectib_pcie_serdes_cfg *ptr_struct, FILE* file)
{
	connectib_pcie_serdes_cfg_print(ptr_struct, file, 0);
}

void connectib_gpio_configurations_pack(const struct connectib_gpio_configurations *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 48; i++) {
	offset = adb2c_calc_array_field_address(24, 8, i, 384, 1);
	connectib_gpio_config_pack(&(ptr_struct->gpio[i]), ptr_buff + offset/8);
	}

}

void connectib_gpio_configurations_unpack(struct connectib_gpio_configurations *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 48; i++) {
	offset = adb2c_calc_array_field_address(24, 8, i, 384, 1);
	connectib_gpio_config_unpack(&(ptr_struct->gpio[i]), ptr_buff + offset/8);
	}

}

void connectib_gpio_configurations_print(const struct connectib_gpio_configurations *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_gpio_configurations ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 48; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpio_%03d:\n", i);
	connectib_gpio_config_print(&(ptr_struct->gpio[i]), file, indent_level + 1);
	}

}

int connectib_gpio_configurations_size(void)
{
	 return 48;
}

void connectib_gpio_configurations_dump(const struct connectib_gpio_configurations *ptr_struct, FILE* file)
{
	connectib_gpio_configurations_print(ptr_struct, file, 0);
}

void connectib_gpio_functions_pack(const struct connectib_gpio_functions *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	connectib_gpio_function_pack(&(ptr_struct->physical_led_p1), ptr_buff + offset/8);

	offset = 16;
	connectib_gpio_function_pack(&(ptr_struct->physical_led_p2), ptr_buff + offset/8);

	offset = 8;
	connectib_gpio_function_pack(&(ptr_struct->logical_led_p1), ptr_buff + offset/8);

	offset = 0;
	connectib_gpio_function_pack(&(ptr_struct->logical_led_p2), ptr_buff + offset/8);

	offset = 56;
	connectib_gpio_function_pack(&(ptr_struct->gp_lv), ptr_buff + offset/8);

	offset = 48;
	connectib_gpio_function_pack(&(ptr_struct->int_p1_l), ptr_buff + offset/8);

	offset = 40;
	connectib_gpio_function_pack(&(ptr_struct->int_p2_l), ptr_buff + offset/8);

	offset = 32;
	connectib_gpio_function_pack(&(ptr_struct->modprs_p1_l), ptr_buff + offset/8);

	offset = 88;
	connectib_gpio_function_pack(&(ptr_struct->modprs_p2_l), ptr_buff + offset/8);

	offset = 80;
	connectib_gpio_function_pack(&(ptr_struct->modsel_p1_l), ptr_buff + offset/8);

	offset = 72;
	connectib_gpio_function_pack(&(ptr_struct->modsel_p2_l), ptr_buff + offset/8);

	offset = 64;
	connectib_gpio_function_pack(&(ptr_struct->ncsi_arb_in), ptr_buff + offset/8);

	offset = 120;
	connectib_gpio_function_pack(&(ptr_struct->ncsi_arb_out), ptr_buff + offset/8);

	offset = 112;
	connectib_gpio_function_pack(&(ptr_struct->ncsi_crs_dv), ptr_buff + offset/8);

	offset = 104;
	connectib_gpio_function_pack(&(ptr_struct->ncsi_rxd0), ptr_buff + offset/8);

	offset = 96;
	connectib_gpio_function_pack(&(ptr_struct->ncsi_rxd1), ptr_buff + offset/8);

	offset = 152;
	connectib_gpio_function_pack(&(ptr_struct->ncsi_txd0), ptr_buff + offset/8);

	offset = 144;
	connectib_gpio_function_pack(&(ptr_struct->ncsi_txd1), ptr_buff + offset/8);

	offset = 136;
	connectib_gpio_function_pack(&(ptr_struct->ncsi_ref_clk), ptr_buff + offset/8);

	offset = 128;
	connectib_gpio_function_pack(&(ptr_struct->ncsi_tx_en), ptr_buff + offset/8);

	offset = 184;
	connectib_gpio_function_pack(&(ptr_struct->oc_int_l), ptr_buff + offset/8);

	offset = 176;
	connectib_gpio_function_pack(&(ptr_struct->ovt_wrng), ptr_buff + offset/8);

	offset = 168;
	connectib_gpio_function_pack(&(ptr_struct->ovt_shtdn), ptr_buff + offset/8);

	offset = 160;
	connectib_gpio_function_pack(&(ptr_struct->pe_rst_l), ptr_buff + offset/8);

	offset = 216;
	connectib_gpio_function_pack(&(ptr_struct->gpio_func_reserved), ptr_buff + offset/8);

	offset = 208;
	connectib_gpio_function_pack(&(ptr_struct->reset_p1_l), ptr_buff + offset/8);

	offset = 200;
	connectib_gpio_function_pack(&(ptr_struct->reset_p2_l), ptr_buff + offset/8);

	offset = 192;
	connectib_gpio_function_pack(&(ptr_struct->spi_clk), ptr_buff + offset/8);

	offset = 248;
	connectib_gpio_function_pack(&(ptr_struct->spi_cs0_l), ptr_buff + offset/8);

	offset = 240;
	connectib_gpio_function_pack(&(ptr_struct->spi_cs1_l), ptr_buff + offset/8);

	offset = 232;
	connectib_gpio_function_pack(&(ptr_struct->spi_io0), ptr_buff + offset/8);

	offset = 224;
	connectib_gpio_function_pack(&(ptr_struct->spi_io1), ptr_buff + offset/8);

	offset = 280;
	connectib_gpio_function_pack(&(ptr_struct->spi_io2), ptr_buff + offset/8);

	offset = 272;
	connectib_gpio_function_pack(&(ptr_struct->spi_io3), ptr_buff + offset/8);

	offset = 264;
	connectib_gpio_function_pack(&(ptr_struct->i2c_scl1), ptr_buff + offset/8);

	offset = 256;
	connectib_gpio_function_pack(&(ptr_struct->i2c_scl2), ptr_buff + offset/8);

	offset = 312;
	connectib_gpio_function_pack(&(ptr_struct->i2c_scl3), ptr_buff + offset/8);

	offset = 304;
	connectib_gpio_function_pack(&(ptr_struct->i2c_sda1), ptr_buff + offset/8);

	offset = 296;
	connectib_gpio_function_pack(&(ptr_struct->i2c_sda2), ptr_buff + offset/8);

	offset = 288;
	connectib_gpio_function_pack(&(ptr_struct->i2c_sda3), ptr_buff + offset/8);

	offset = 344;
	connectib_gpio_function_pack(&(ptr_struct->i2c_int_l), ptr_buff + offset/8);

	offset = 336;
	connectib_gpio_function_pack(&(ptr_struct->i2c_rst_l), ptr_buff + offset/8);

	offset = 328;
	connectib_gpio_function_pack(&(ptr_struct->vdd_mod), ptr_buff + offset/8);

	offset = 320;
	connectib_gpio_function_pack(&(ptr_struct->wake_l), ptr_buff + offset/8);

	offset = 376;
	connectib_gpio_function_pack(&(ptr_struct->ibm_mezz_fault_l), ptr_buff + offset/8);

	offset = 368;
	connectib_gpio_function_pack(&(ptr_struct->ibm_ite_all_pgood), ptr_buff + offset/8);

	offset = 360;
	connectib_gpio_function_pack(&(ptr_struct->ibm_t_control_l), ptr_buff + offset/8);

	offset = 352;
	connectib_gpio_function_pack(&(ptr_struct->ibm_t_warning_l), ptr_buff + offset/8);

	offset = 408;
	connectib_gpio_function_pack(&(ptr_struct->ibm_t_critical_l), ptr_buff + offset/8);

	offset = 400;
	connectib_gpio_function_pack(&(ptr_struct->ibm_cib_slot_id_0), ptr_buff + offset/8);

	offset = 392;
	connectib_gpio_function_pack(&(ptr_struct->ibm_cib_slot_id_1), ptr_buff + offset/8);

	offset = 384;
	connectib_gpio_function_pack(&(ptr_struct->ibm_vpd_update_l), ptr_buff + offset/8);

	offset = 440;
	connectib_gpio_function_pack(&(ptr_struct->ibm_vpd_cache_comp), ptr_buff + offset/8);

	offset = 432;
	connectib_gpio_function_pack(&(ptr_struct->ibm_vpd_ctrl), ptr_buff + offset/8);

	offset = 424;
	connectib_gpio_function_pack(&(ptr_struct->gpio_port_swap), ptr_buff + offset/8);

	offset = 416;
	connectib_gpio_function_pack(&(ptr_struct->gpio_func_reserved_1), ptr_buff + offset/8);

}

void connectib_gpio_functions_unpack(struct connectib_gpio_functions *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	connectib_gpio_function_unpack(&(ptr_struct->physical_led_p1), ptr_buff + offset/8);

	offset = 16;
	connectib_gpio_function_unpack(&(ptr_struct->physical_led_p2), ptr_buff + offset/8);

	offset = 8;
	connectib_gpio_function_unpack(&(ptr_struct->logical_led_p1), ptr_buff + offset/8);

	offset = 0;
	connectib_gpio_function_unpack(&(ptr_struct->logical_led_p2), ptr_buff + offset/8);

	offset = 56;
	connectib_gpio_function_unpack(&(ptr_struct->gp_lv), ptr_buff + offset/8);

	offset = 48;
	connectib_gpio_function_unpack(&(ptr_struct->int_p1_l), ptr_buff + offset/8);

	offset = 40;
	connectib_gpio_function_unpack(&(ptr_struct->int_p2_l), ptr_buff + offset/8);

	offset = 32;
	connectib_gpio_function_unpack(&(ptr_struct->modprs_p1_l), ptr_buff + offset/8);

	offset = 88;
	connectib_gpio_function_unpack(&(ptr_struct->modprs_p2_l), ptr_buff + offset/8);

	offset = 80;
	connectib_gpio_function_unpack(&(ptr_struct->modsel_p1_l), ptr_buff + offset/8);

	offset = 72;
	connectib_gpio_function_unpack(&(ptr_struct->modsel_p2_l), ptr_buff + offset/8);

	offset = 64;
	connectib_gpio_function_unpack(&(ptr_struct->ncsi_arb_in), ptr_buff + offset/8);

	offset = 120;
	connectib_gpio_function_unpack(&(ptr_struct->ncsi_arb_out), ptr_buff + offset/8);

	offset = 112;
	connectib_gpio_function_unpack(&(ptr_struct->ncsi_crs_dv), ptr_buff + offset/8);

	offset = 104;
	connectib_gpio_function_unpack(&(ptr_struct->ncsi_rxd0), ptr_buff + offset/8);

	offset = 96;
	connectib_gpio_function_unpack(&(ptr_struct->ncsi_rxd1), ptr_buff + offset/8);

	offset = 152;
	connectib_gpio_function_unpack(&(ptr_struct->ncsi_txd0), ptr_buff + offset/8);

	offset = 144;
	connectib_gpio_function_unpack(&(ptr_struct->ncsi_txd1), ptr_buff + offset/8);

	offset = 136;
	connectib_gpio_function_unpack(&(ptr_struct->ncsi_ref_clk), ptr_buff + offset/8);

	offset = 128;
	connectib_gpio_function_unpack(&(ptr_struct->ncsi_tx_en), ptr_buff + offset/8);

	offset = 184;
	connectib_gpio_function_unpack(&(ptr_struct->oc_int_l), ptr_buff + offset/8);

	offset = 176;
	connectib_gpio_function_unpack(&(ptr_struct->ovt_wrng), ptr_buff + offset/8);

	offset = 168;
	connectib_gpio_function_unpack(&(ptr_struct->ovt_shtdn), ptr_buff + offset/8);

	offset = 160;
	connectib_gpio_function_unpack(&(ptr_struct->pe_rst_l), ptr_buff + offset/8);

	offset = 216;
	connectib_gpio_function_unpack(&(ptr_struct->gpio_func_reserved), ptr_buff + offset/8);

	offset = 208;
	connectib_gpio_function_unpack(&(ptr_struct->reset_p1_l), ptr_buff + offset/8);

	offset = 200;
	connectib_gpio_function_unpack(&(ptr_struct->reset_p2_l), ptr_buff + offset/8);

	offset = 192;
	connectib_gpio_function_unpack(&(ptr_struct->spi_clk), ptr_buff + offset/8);

	offset = 248;
	connectib_gpio_function_unpack(&(ptr_struct->spi_cs0_l), ptr_buff + offset/8);

	offset = 240;
	connectib_gpio_function_unpack(&(ptr_struct->spi_cs1_l), ptr_buff + offset/8);

	offset = 232;
	connectib_gpio_function_unpack(&(ptr_struct->spi_io0), ptr_buff + offset/8);

	offset = 224;
	connectib_gpio_function_unpack(&(ptr_struct->spi_io1), ptr_buff + offset/8);

	offset = 280;
	connectib_gpio_function_unpack(&(ptr_struct->spi_io2), ptr_buff + offset/8);

	offset = 272;
	connectib_gpio_function_unpack(&(ptr_struct->spi_io3), ptr_buff + offset/8);

	offset = 264;
	connectib_gpio_function_unpack(&(ptr_struct->i2c_scl1), ptr_buff + offset/8);

	offset = 256;
	connectib_gpio_function_unpack(&(ptr_struct->i2c_scl2), ptr_buff + offset/8);

	offset = 312;
	connectib_gpio_function_unpack(&(ptr_struct->i2c_scl3), ptr_buff + offset/8);

	offset = 304;
	connectib_gpio_function_unpack(&(ptr_struct->i2c_sda1), ptr_buff + offset/8);

	offset = 296;
	connectib_gpio_function_unpack(&(ptr_struct->i2c_sda2), ptr_buff + offset/8);

	offset = 288;
	connectib_gpio_function_unpack(&(ptr_struct->i2c_sda3), ptr_buff + offset/8);

	offset = 344;
	connectib_gpio_function_unpack(&(ptr_struct->i2c_int_l), ptr_buff + offset/8);

	offset = 336;
	connectib_gpio_function_unpack(&(ptr_struct->i2c_rst_l), ptr_buff + offset/8);

	offset = 328;
	connectib_gpio_function_unpack(&(ptr_struct->vdd_mod), ptr_buff + offset/8);

	offset = 320;
	connectib_gpio_function_unpack(&(ptr_struct->wake_l), ptr_buff + offset/8);

	offset = 376;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_mezz_fault_l), ptr_buff + offset/8);

	offset = 368;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_ite_all_pgood), ptr_buff + offset/8);

	offset = 360;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_t_control_l), ptr_buff + offset/8);

	offset = 352;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_t_warning_l), ptr_buff + offset/8);

	offset = 408;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_t_critical_l), ptr_buff + offset/8);

	offset = 400;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_cib_slot_id_0), ptr_buff + offset/8);

	offset = 392;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_cib_slot_id_1), ptr_buff + offset/8);

	offset = 384;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_vpd_update_l), ptr_buff + offset/8);

	offset = 440;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_vpd_cache_comp), ptr_buff + offset/8);

	offset = 432;
	connectib_gpio_function_unpack(&(ptr_struct->ibm_vpd_ctrl), ptr_buff + offset/8);

	offset = 424;
	connectib_gpio_function_unpack(&(ptr_struct->gpio_port_swap), ptr_buff + offset/8);

	offset = 416;
	connectib_gpio_function_unpack(&(ptr_struct->gpio_func_reserved_1), ptr_buff + offset/8);

}

void connectib_gpio_functions_print(const struct connectib_gpio_functions *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_gpio_functions ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "physical_led_p1:\n");
	connectib_gpio_function_print(&(ptr_struct->physical_led_p1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "physical_led_p2:\n");
	connectib_gpio_function_print(&(ptr_struct->physical_led_p2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "logical_led_p1:\n");
	connectib_gpio_function_print(&(ptr_struct->logical_led_p1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "logical_led_p2:\n");
	connectib_gpio_function_print(&(ptr_struct->logical_led_p2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gp_lv:\n");
	connectib_gpio_function_print(&(ptr_struct->gp_lv), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "int_p1_l:\n");
	connectib_gpio_function_print(&(ptr_struct->int_p1_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "int_p2_l:\n");
	connectib_gpio_function_print(&(ptr_struct->int_p2_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modprs_p1_l:\n");
	connectib_gpio_function_print(&(ptr_struct->modprs_p1_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modprs_p2_l:\n");
	connectib_gpio_function_print(&(ptr_struct->modprs_p2_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modsel_p1_l:\n");
	connectib_gpio_function_print(&(ptr_struct->modsel_p1_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "modsel_p2_l:\n");
	connectib_gpio_function_print(&(ptr_struct->modsel_p2_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncsi_arb_in:\n");
	connectib_gpio_function_print(&(ptr_struct->ncsi_arb_in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncsi_arb_out:\n");
	connectib_gpio_function_print(&(ptr_struct->ncsi_arb_out), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncsi_crs_dv:\n");
	connectib_gpio_function_print(&(ptr_struct->ncsi_crs_dv), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncsi_rxd0:\n");
	connectib_gpio_function_print(&(ptr_struct->ncsi_rxd0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncsi_rxd1:\n");
	connectib_gpio_function_print(&(ptr_struct->ncsi_rxd1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncsi_txd0:\n");
	connectib_gpio_function_print(&(ptr_struct->ncsi_txd0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncsi_txd1:\n");
	connectib_gpio_function_print(&(ptr_struct->ncsi_txd1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncsi_ref_clk:\n");
	connectib_gpio_function_print(&(ptr_struct->ncsi_ref_clk), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ncsi_tx_en:\n");
	connectib_gpio_function_print(&(ptr_struct->ncsi_tx_en), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "oc_int_l:\n");
	connectib_gpio_function_print(&(ptr_struct->oc_int_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ovt_wrng:\n");
	connectib_gpio_function_print(&(ptr_struct->ovt_wrng), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ovt_shtdn:\n");
	connectib_gpio_function_print(&(ptr_struct->ovt_shtdn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pe_rst_l:\n");
	connectib_gpio_function_print(&(ptr_struct->pe_rst_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpio_func_reserved:\n");
	connectib_gpio_function_print(&(ptr_struct->gpio_func_reserved), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reset_p1_l:\n");
	connectib_gpio_function_print(&(ptr_struct->reset_p1_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reset_p2_l:\n");
	connectib_gpio_function_print(&(ptr_struct->reset_p2_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spi_clk:\n");
	connectib_gpio_function_print(&(ptr_struct->spi_clk), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spi_cs0_l:\n");
	connectib_gpio_function_print(&(ptr_struct->spi_cs0_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spi_cs1_l:\n");
	connectib_gpio_function_print(&(ptr_struct->spi_cs1_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spi_io0:\n");
	connectib_gpio_function_print(&(ptr_struct->spi_io0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spi_io1:\n");
	connectib_gpio_function_print(&(ptr_struct->spi_io1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spi_io2:\n");
	connectib_gpio_function_print(&(ptr_struct->spi_io2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spi_io3:\n");
	connectib_gpio_function_print(&(ptr_struct->spi_io3), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_scl1:\n");
	connectib_gpio_function_print(&(ptr_struct->i2c_scl1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_scl2:\n");
	connectib_gpio_function_print(&(ptr_struct->i2c_scl2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_scl3:\n");
	connectib_gpio_function_print(&(ptr_struct->i2c_scl3), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_sda1:\n");
	connectib_gpio_function_print(&(ptr_struct->i2c_sda1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_sda2:\n");
	connectib_gpio_function_print(&(ptr_struct->i2c_sda2), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_sda3:\n");
	connectib_gpio_function_print(&(ptr_struct->i2c_sda3), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_int_l:\n");
	connectib_gpio_function_print(&(ptr_struct->i2c_int_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_rst_l:\n");
	connectib_gpio_function_print(&(ptr_struct->i2c_rst_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vdd_mod:\n");
	connectib_gpio_function_print(&(ptr_struct->vdd_mod), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wake_l:\n");
	connectib_gpio_function_print(&(ptr_struct->wake_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_mezz_fault_l:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_mezz_fault_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_ite_all_pgood:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_ite_all_pgood), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_t_control_l:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_t_control_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_t_warning_l:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_t_warning_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_t_critical_l:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_t_critical_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_cib_slot_id_0:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_cib_slot_id_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_cib_slot_id_1:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_cib_slot_id_1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_vpd_update_l:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_vpd_update_l), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_vpd_cache_comp:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_vpd_cache_comp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_vpd_ctrl:\n");
	connectib_gpio_function_print(&(ptr_struct->ibm_vpd_ctrl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpio_port_swap:\n");
	connectib_gpio_function_print(&(ptr_struct->gpio_port_swap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpio_func_reserved_1:\n");
	connectib_gpio_function_print(&(ptr_struct->gpio_func_reserved_1), file, indent_level + 1);

}

int connectib_gpio_functions_size(void)
{
	 return 84;
}

void connectib_gpio_functions_dump(const struct connectib_gpio_functions *ptr_struct, FILE* file)
{
	connectib_gpio_functions_print(ptr_struct, file, 0);
}

void connectib_i2c_device_entry_pack(const struct connectib_i2c_device_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->slave_addr);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->bus_id);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->addr_size);

}

void connectib_i2c_device_entry_unpack(struct connectib_i2c_device_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	ptr_struct->slave_addr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 22;
	ptr_struct->bus_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 13;
	ptr_struct->addr_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_i2c_device_entry_print(const struct connectib_i2c_device_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_i2c_device_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slave_addr           : " UH_FMT "\n", ptr_struct->slave_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bus_id               : " UH_FMT "\n", ptr_struct->bus_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr_size            : " UH_FMT "\n", ptr_struct->addr_size);

}

int connectib_i2c_device_entry_size(void)
{
	 return 4;
}

void connectib_i2c_device_entry_dump(const struct connectib_i2c_device_entry *ptr_struct, FILE* file)
{
	connectib_i2c_device_entry_print(ptr_struct, file, 0);
}

void connectib_module_config_st_pack(const struct connectib_module_config_st *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->module_power_level_supported);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->module_mlnx_sub_power_supported);

}

void connectib_module_config_st_unpack(struct connectib_module_config_st *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	ptr_struct->module_power_level_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 24;
	ptr_struct->module_mlnx_sub_power_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

}

void connectib_module_config_st_print(const struct connectib_module_config_st *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_module_config_st ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_power_level_supported : " UH_FMT "\n", ptr_struct->module_power_level_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_mlnx_sub_power_supported : " UH_FMT "\n", ptr_struct->module_mlnx_sub_power_supported);

}

int connectib_module_config_st_size(void)
{
	 return 4;
}

void connectib_module_config_st_dump(const struct connectib_module_config_st *ptr_struct, FILE* file)
{
	connectib_module_config_st_print(ptr_struct, file, 0);
}

void connectib_iopx_power_pack(const struct connectib_iopx_power *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ref_clk_right_left_);

}

void connectib_iopx_power_unpack(struct connectib_iopx_power *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->ref_clk_right_left_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_iopx_power_print(const struct connectib_iopx_power *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_iopx_power ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ref_clk_right_left_  : " UH_FMT "\n", ptr_struct->ref_clk_right_left_);

}

int connectib_iopx_power_size(void)
{
	 return 4;
}

void connectib_iopx_power_dump(const struct connectib_iopx_power *ptr_struct, FILE* file)
{
	connectib_iopx_power_print(ptr_struct, file, 0);
}

void connectib_iopl_power_pack(const struct connectib_iopl_power *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ref_clk_right_left_);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dynamic_pll_manage_en);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dynamic_serdes_manage_en);

}

void connectib_iopl_power_unpack(struct connectib_iopl_power *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->ref_clk_right_left_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->dynamic_pll_manage_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->dynamic_serdes_manage_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_iopl_power_print(const struct connectib_iopl_power *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_iopl_power ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ref_clk_right_left_  : " UH_FMT "\n", ptr_struct->ref_clk_right_left_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dynamic_pll_manage_en : " UH_FMT "\n", ptr_struct->dynamic_pll_manage_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dynamic_serdes_manage_en : " UH_FMT "\n", ptr_struct->dynamic_serdes_manage_en);

}

int connectib_iopl_power_size(void)
{
	 return 4;
}

void connectib_iopl_power_dump(const struct connectib_iopl_power *ptr_struct, FILE* file)
{
	connectib_iopl_power_print(ptr_struct, file, 0);
}

void connectib_port_mlpn_db_pack(const struct connectib_port_mlpn_db *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mlpn_database_all_protocol_ini_pack(&(ptr_struct->mlpn_supported), ptr_buff + offset/8);

	offset = 32;
	connectib_mlpn_database_all_protocol_ini_pack(&(ptr_struct->mlpn_request), ptr_buff + offset/8);

	offset = 64;
	connectib_mlpn_enhanced_database_ini_pack(&(ptr_struct->mlpn_enhanced_database), ptr_buff + offset/8);

}

void connectib_port_mlpn_db_unpack(struct connectib_port_mlpn_db *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_mlpn_database_all_protocol_ini_unpack(&(ptr_struct->mlpn_supported), ptr_buff + offset/8);

	offset = 32;
	connectib_mlpn_database_all_protocol_ini_unpack(&(ptr_struct->mlpn_request), ptr_buff + offset/8);

	offset = 64;
	connectib_mlpn_enhanced_database_ini_unpack(&(ptr_struct->mlpn_enhanced_database), ptr_buff + offset/8);

}

void connectib_port_mlpn_db_print(const struct connectib_port_mlpn_db *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_port_mlpn_db ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_supported:\n");
	connectib_mlpn_database_all_protocol_ini_print(&(ptr_struct->mlpn_supported), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_request:\n");
	connectib_mlpn_database_all_protocol_ini_print(&(ptr_struct->mlpn_request), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_enhanced_database:\n");
	connectib_mlpn_enhanced_database_ini_print(&(ptr_struct->mlpn_enhanced_database), file, indent_level + 1);

}

int connectib_port_mlpn_db_size(void)
{
	 return 16;
}

void connectib_port_mlpn_db_dump(const struct connectib_port_mlpn_db *ptr_struct, FILE* file)
{
	connectib_port_mlpn_db_print(ptr_struct, file, 0);
}

void connectib_pll_params_pack(const struct connectib_pll_params *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pll_fctrl_th);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable_phy_varactor_calibration);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 288, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pll_fctrl_var[i]);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(160, 32, i, 288, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pll_fctrl_val[i]);
	}

}

void connectib_pll_params_unpack(struct connectib_pll_params *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->pll_fctrl_th = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->enable_phy_varactor_calibration = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 288, 1);
	ptr_struct->pll_fctrl_var[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(160, 32, i, 288, 1);
	ptr_struct->pll_fctrl_val[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_pll_params_print(const struct connectib_pll_params *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pll_params ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pll_fctrl_th         : " UH_FMT "\n", ptr_struct->pll_fctrl_th);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable_phy_varactor_calibration : " UH_FMT "\n", ptr_struct->enable_phy_varactor_calibration);

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pll_fctrl_var_%03d   : " U32H_FMT "\n", i, ptr_struct->pll_fctrl_var[i]);
	}

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pll_fctrl_val_%03d   : " U32H_FMT "\n", i, ptr_struct->pll_fctrl_val[i]);
	}

}

int connectib_pll_params_size(void)
{
	 return 36;
}

void connectib_pll_params_dump(const struct connectib_pll_params *ptr_struct, FILE* file)
{
	connectib_pll_params_print(ptr_struct, file, 0);
}

void connectib_logic_serdes_mapping_node_ini_pack(const struct connectib_logic_serdes_mapping_node_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_tx_rx_logic_serdes_mapping_node_ini_pack(&(ptr_struct->tx), ptr_buff + offset/8);

	offset = 256;
	connectib_tx_rx_logic_serdes_mapping_node_ini_pack(&(ptr_struct->rx), ptr_buff + offset/8);

}

void connectib_logic_serdes_mapping_node_ini_unpack(struct connectib_logic_serdes_mapping_node_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_tx_rx_logic_serdes_mapping_node_ini_unpack(&(ptr_struct->tx), ptr_buff + offset/8);

	offset = 256;
	connectib_tx_rx_logic_serdes_mapping_node_ini_unpack(&(ptr_struct->rx), ptr_buff + offset/8);

}

void connectib_logic_serdes_mapping_node_ini_print(const struct connectib_logic_serdes_mapping_node_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_logic_serdes_mapping_node_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx:\n");
	connectib_tx_rx_logic_serdes_mapping_node_ini_print(&(ptr_struct->tx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx:\n");
	connectib_tx_rx_logic_serdes_mapping_node_ini_print(&(ptr_struct->rx), file, indent_level + 1);

}

int connectib_logic_serdes_mapping_node_ini_size(void)
{
	 return 64;
}

void connectib_logic_serdes_mapping_node_ini_dump(const struct connectib_logic_serdes_mapping_node_ini *ptr_struct, FILE* file)
{
	connectib_logic_serdes_mapping_node_ini_print(ptr_struct, file, 0);
}

void connectib_phy_hw_main_config_ip_pack(const struct connectib_phy_hw_main_config_ip *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 128, i, 11776, 1);
	connectib_PORT_PHY_ini_pack(&(ptr_struct->port[i]), ptr_buff + offset/8);
	}

	offset = 256;
	connectib_ib_ini_ini_pack(&(ptr_struct->ib), ptr_buff + offset/8);

	offset = 1024;
	connectib_rx_adapt_ini_pack(&(ptr_struct->rx_adapt_ini), ptr_buff + offset/8);

	offset = 1280;
	connectib_serdes_conf_ini_pack(&(ptr_struct->serdes_conf), ptr_buff + offset/8);

}

void connectib_phy_hw_main_config_ip_unpack(struct connectib_phy_hw_main_config_ip *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 128, i, 11776, 1);
	connectib_PORT_PHY_ini_unpack(&(ptr_struct->port[i]), ptr_buff + offset/8);
	}

	offset = 256;
	connectib_ib_ini_ini_unpack(&(ptr_struct->ib), ptr_buff + offset/8);

	offset = 1024;
	connectib_rx_adapt_ini_unpack(&(ptr_struct->rx_adapt_ini), ptr_buff + offset/8);

	offset = 1280;
	connectib_serdes_conf_ini_unpack(&(ptr_struct->serdes_conf), ptr_buff + offset/8);

}

void connectib_phy_hw_main_config_ip_print(const struct connectib_phy_hw_main_config_ip *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_phy_hw_main_config_ip ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_%03d:\n", i);
	connectib_PORT_PHY_ini_print(&(ptr_struct->port[i]), file, indent_level + 1);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib:\n");
	connectib_ib_ini_ini_print(&(ptr_struct->ib), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_adapt_ini:\n");
	connectib_rx_adapt_ini_print(&(ptr_struct->rx_adapt_ini), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serdes_conf:\n");
	connectib_serdes_conf_ini_print(&(ptr_struct->serdes_conf), file, indent_level + 1);

}

int connectib_phy_hw_main_config_ip_size(void)
{
	 return 1472;
}

void connectib_phy_hw_main_config_ip_dump(const struct connectib_phy_hw_main_config_ip *ptr_struct, FILE* file)
{
	connectib_phy_hw_main_config_ip_print(ptr_struct, file, 0);
}

void connectib_nv_config_boot_port_settings_pack(const struct connectib_nv_config_boot_port_settings *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_tlv_data_vpi_settings_pack(&(ptr_struct->vpi), ptr_buff + offset/8);

}

void connectib_nv_config_boot_port_settings_unpack(struct connectib_nv_config_boot_port_settings *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_tlv_data_vpi_settings_unpack(&(ptr_struct->vpi), ptr_buff + offset/8);

}

void connectib_nv_config_boot_port_settings_print(const struct connectib_nv_config_boot_port_settings *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_boot_port_settings ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpi:\n");
	connectib_nv_tlv_data_vpi_settings_print(&(ptr_struct->vpi), file, indent_level + 1);

}

int connectib_nv_config_boot_port_settings_size(void)
{
	 return 4;
}

void connectib_nv_config_boot_port_settings_dump(const struct connectib_nv_config_boot_port_settings *ptr_struct, FILE* file)
{
	connectib_nv_config_boot_port_settings_print(ptr_struct, file, 0);
}

void connectib_nv_config_boot_global_pack(const struct connectib_nv_config_boot_global *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_config_pci_pack(&(ptr_struct->pci), ptr_buff + offset/8);

	offset = 256;
	connectib_nv_config_tpt_pack(&(ptr_struct->tpt), ptr_buff + offset/8);

	offset = 352;
	connectib_nv_config_kdnet_data_pack(&(ptr_struct->kdnet_data), ptr_buff + offset/8);

}

void connectib_nv_config_boot_global_unpack(struct connectib_nv_config_boot_global *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_config_pci_unpack(&(ptr_struct->pci), ptr_buff + offset/8);

	offset = 256;
	connectib_nv_config_tpt_unpack(&(ptr_struct->tpt), ptr_buff + offset/8);

	offset = 352;
	connectib_nv_config_kdnet_data_unpack(&(ptr_struct->kdnet_data), ptr_buff + offset/8);

}

void connectib_nv_config_boot_global_print(const struct connectib_nv_config_boot_global *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_boot_global ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci:\n");
	connectib_nv_config_pci_print(&(ptr_struct->pci), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tpt:\n");
	connectib_nv_config_tpt_print(&(ptr_struct->tpt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "kdnet_data:\n");
	connectib_nv_config_kdnet_data_print(&(ptr_struct->kdnet_data), file, indent_level + 1);

}

int connectib_nv_config_boot_global_size(void)
{
	 return 48;
}

void connectib_nv_config_boot_global_dump(const struct connectib_nv_config_boot_global *ptr_struct, FILE* file)
{
	connectib_nv_config_boot_global_print(ptr_struct, file, 0);
}

void connectib_pciex_ext_cfg_space_pack(const struct connectib_pciex_ext_cfg_space *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_secondary_pciex_ext_cap_pack(&(ptr_struct->secondary_pciex_ext_cap), ptr_buff + offset/8);

	offset = 352;
	connectib_ari_cap_pack(&(ptr_struct->ari), ptr_buff + offset/8);

	offset = 416;
	connectib_sriov_cap_pack(&(ptr_struct->sriov), ptr_buff + offset/8);

	offset = 928;
	connectib_dev_serial_num_cap_pack(&(ptr_struct->device_serial_number), ptr_buff + offset/8);

	offset = 1024;
	connectib_advanced_error_reporting_pack(&(ptr_struct->advanced_error_reporting), ptr_buff + offset/8);

	offset = 1536;
	connectib_acs_ext_cap_pack(&(ptr_struct->acs), ptr_buff + offset/8);

}

void connectib_pciex_ext_cfg_space_unpack(struct connectib_pciex_ext_cfg_space *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_secondary_pciex_ext_cap_unpack(&(ptr_struct->secondary_pciex_ext_cap), ptr_buff + offset/8);

	offset = 352;
	connectib_ari_cap_unpack(&(ptr_struct->ari), ptr_buff + offset/8);

	offset = 416;
	connectib_sriov_cap_unpack(&(ptr_struct->sriov), ptr_buff + offset/8);

	offset = 928;
	connectib_dev_serial_num_cap_unpack(&(ptr_struct->device_serial_number), ptr_buff + offset/8);

	offset = 1024;
	connectib_advanced_error_reporting_unpack(&(ptr_struct->advanced_error_reporting), ptr_buff + offset/8);

	offset = 1536;
	connectib_acs_ext_cap_unpack(&(ptr_struct->acs), ptr_buff + offset/8);

}

void connectib_pciex_ext_cfg_space_print(const struct connectib_pciex_ext_cfg_space *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pciex_ext_cfg_space ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "secondary_pciex_ext_cap:\n");
	connectib_secondary_pciex_ext_cap_print(&(ptr_struct->secondary_pciex_ext_cap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ari:\n");
	connectib_ari_cap_print(&(ptr_struct->ari), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sriov:\n");
	connectib_sriov_cap_print(&(ptr_struct->sriov), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_serial_number:\n");
	connectib_dev_serial_num_cap_print(&(ptr_struct->device_serial_number), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "advanced_error_reporting:\n");
	connectib_advanced_error_reporting_print(&(ptr_struct->advanced_error_reporting), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "acs:\n");
	connectib_acs_ext_cap_print(&(ptr_struct->acs), file, indent_level + 1);

}

int connectib_pciex_ext_cfg_space_size(void)
{
	 return 256;
}

void connectib_pciex_ext_cfg_space_dump(const struct connectib_pciex_ext_cfg_space *ptr_struct, FILE* file)
{
	connectib_pciex_ext_cfg_space_print(ptr_struct, file, 0);
}

void connectib_pci_cfg_space_pack(const struct connectib_pci_cfg_space *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_pci_cfg_type0_header_pack(&(ptr_struct->cfg_hdr), ptr_buff + offset/8);

	offset = 512;
	connectib_pci_power_management_cap_pack(&(ptr_struct->power_management), ptr_buff + offset/8);

	offset = 576;
	connectib_vpd_cap_pack(&(ptr_struct->vpd), ptr_buff + offset/8);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->image_status);

	offset = 704;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cr_addr);

	offset = 736;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cr_data);

	offset = 768;
	connectib_pciex_cap_block_pack(&(ptr_struct->pciex_cap_block), ptr_buff + offset/8);

	offset = 1248;
	connectib_msix_cap_pack(&(ptr_struct->msix), ptr_buff + offset/8);

	offset = 1344;
	connectib_pci_cfg_type1_header_pack(&(ptr_struct->type1_cfg_hdr), ptr_buff + offset/8);

}

void connectib_pci_cfg_space_unpack(struct connectib_pci_cfg_space *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_pci_cfg_type0_header_unpack(&(ptr_struct->cfg_hdr), ptr_buff + offset/8);

	offset = 512;
	connectib_pci_power_management_cap_unpack(&(ptr_struct->power_management), ptr_buff + offset/8);

	offset = 576;
	connectib_vpd_cap_unpack(&(ptr_struct->vpd), ptr_buff + offset/8);

	offset = 640;
	ptr_struct->image_status = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 704;
	ptr_struct->cr_addr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 736;
	ptr_struct->cr_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 768;
	connectib_pciex_cap_block_unpack(&(ptr_struct->pciex_cap_block), ptr_buff + offset/8);

	offset = 1248;
	connectib_msix_cap_unpack(&(ptr_struct->msix), ptr_buff + offset/8);

	offset = 1344;
	connectib_pci_cfg_type1_header_unpack(&(ptr_struct->type1_cfg_hdr), ptr_buff + offset/8);

}

void connectib_pci_cfg_space_print(const struct connectib_pci_cfg_space *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pci_cfg_space ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cfg_hdr:\n");
	connectib_pci_cfg_type0_header_print(&(ptr_struct->cfg_hdr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "power_management:\n");
	connectib_pci_power_management_cap_print(&(ptr_struct->power_management), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd:\n");
	connectib_vpd_cap_print(&(ptr_struct->vpd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "image_status         : " U32H_FMT "\n", ptr_struct->image_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cr_addr              : " U32H_FMT "\n", ptr_struct->cr_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cr_data              : " U32H_FMT "\n", ptr_struct->cr_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pciex_cap_block:\n");
	connectib_pciex_cap_block_print(&(ptr_struct->pciex_cap_block), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "msix:\n");
	connectib_msix_cap_print(&(ptr_struct->msix), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type1_cfg_hdr:\n");
	connectib_pci_cfg_type1_header_print(&(ptr_struct->type1_cfg_hdr), file, indent_level + 1);

}

int connectib_pci_cfg_space_size(void)
{
	 return 256;
}

void connectib_pci_cfg_space_dump(const struct connectib_pci_cfg_space *ptr_struct, FILE* file)
{
	connectib_pci_cfg_space_print(ptr_struct, file, 0);
}

void connectib_logger_init_info_pack(const struct connectib_logger_init_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 15, (u_int32_t)ptr_struct->cfg_request_mask);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->splix_mask);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->writes_amount);

}

void connectib_logger_init_info_unpack(struct connectib_logger_init_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 17;
	ptr_struct->cfg_request_mask = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 15);

	offset = 7;
	ptr_struct->splix_mask = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 1;
	ptr_struct->writes_amount = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

}

void connectib_logger_init_info_print(const struct connectib_logger_init_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_logger_init_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cfg_request_mask     : " UH_FMT "\n", ptr_struct->cfg_request_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "splix_mask           : " UH_FMT "\n", ptr_struct->splix_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "writes_amount        : " UH_FMT "\n", ptr_struct->writes_amount);

}

int connectib_logger_init_info_size(void)
{
	 return 4;
}

void connectib_logger_init_info_dump(const struct connectib_logger_init_info *ptr_struct, FILE* file)
{
	connectib_logger_init_info_print(ptr_struct, file, 0);
}

void connectib_fw_block_behavior_pack(const union connectib_fw_block_behavior *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_fw_block_behavior_fields_pack(&(ptr_struct->fields), ptr_buff);
}

void connectib_fw_block_behavior_unpack(union connectib_fw_block_behavior *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_fw_block_behavior_fields_unpack(&(ptr_struct->fields), ptr_buff);
}

void connectib_fw_block_behavior_print(const union connectib_fw_block_behavior *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_block_behavior ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dw:\n");
	connectib_fw_block_behavior_dw_print(&(ptr_struct->dw), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fields:\n");
	connectib_fw_block_behavior_fields_print(&(ptr_struct->fields), file, indent_level + 1);

}

int connectib_fw_block_behavior_size(void)
{
	 return 4;
}

void connectib_fw_block_behavior_dump(const union connectib_fw_block_behavior *ptr_struct, FILE* file)
{
	connectib_fw_block_behavior_print(ptr_struct, file, 0);
}

void connectib_nv_config_main_fw_data_pack(const struct connectib_nv_config_main_fw_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 2048, i, 6144, 1);
	connectib_port_settings_pack(&(ptr_struct->port[i]), ptr_buff + offset/8);
	}

	offset = 4096;
	connectib_nv_config_global_main_pack(&(ptr_struct->global), ptr_buff + offset/8);

	offset = 5120;
	connectib_nv_config_host_func_main_pack(&(ptr_struct->host_func_pf_order), ptr_buff + offset/8);

}

void connectib_nv_config_main_fw_data_unpack(struct connectib_nv_config_main_fw_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 2048, i, 6144, 1);
	connectib_port_settings_unpack(&(ptr_struct->port[i]), ptr_buff + offset/8);
	}

	offset = 4096;
	connectib_nv_config_global_main_unpack(&(ptr_struct->global), ptr_buff + offset/8);

	offset = 5120;
	connectib_nv_config_host_func_main_unpack(&(ptr_struct->host_func_pf_order), ptr_buff + offset/8);

}

void connectib_nv_config_main_fw_data_print(const struct connectib_nv_config_main_fw_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_main_fw_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_%03d:\n", i);
	connectib_port_settings_print(&(ptr_struct->port[i]), file, indent_level + 1);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "global:\n");
	connectib_nv_config_global_main_print(&(ptr_struct->global), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_func_pf_order:\n");
	connectib_nv_config_host_func_main_print(&(ptr_struct->host_func_pf_order), file, indent_level + 1);

}

int connectib_nv_config_main_fw_data_size(void)
{
	 return 768;
}

void connectib_nv_config_main_fw_data_dump(const struct connectib_nv_config_main_fw_data *ptr_struct, FILE* file)
{
	connectib_nv_config_main_fw_data_print(ptr_struct, file, 0);
}

void connectib_phy_fw_main_config_ip_pack(const struct connectib_phy_fw_main_config_ip *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->PlaceHolder);

}

void connectib_phy_fw_main_config_ip_unpack(struct connectib_phy_fw_main_config_ip *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->PlaceHolder = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_phy_fw_main_config_ip_print(const struct connectib_phy_fw_main_config_ip *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_phy_fw_main_config_ip ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "PlaceHolder          : " U32H_FMT "\n", ptr_struct->PlaceHolder);

}

int connectib_phy_fw_main_config_ip_size(void)
{
	 return 384;
}

void connectib_phy_fw_main_config_ip_dump(const struct connectib_phy_fw_main_config_ip *ptr_struct, FILE* file)
{
	connectib_phy_fw_main_config_ip_print(ptr_struct, file, 0);
}

void connectib_phy_fw_main_config_golan_pack(const struct connectib_phy_fw_main_config_golan *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1856, 1);
	connectib_phy_lcl_ini_pack(&(ptr_struct->lcl[i]), ptr_buff + offset/8);
	}

}

void connectib_phy_fw_main_config_golan_unpack(struct connectib_phy_fw_main_config_golan *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1856, 1);
	connectib_phy_lcl_ini_unpack(&(ptr_struct->lcl[i]), ptr_buff + offset/8);
	}

}

void connectib_phy_fw_main_config_golan_print(const struct connectib_phy_fw_main_config_golan *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_phy_fw_main_config_golan ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lcl_%03d:\n", i);
	connectib_phy_lcl_ini_print(&(ptr_struct->lcl[i]), file, indent_level + 1);
	}

}

int connectib_phy_fw_main_config_golan_size(void)
{
	 return 232;
}

void connectib_phy_fw_main_config_golan_dump(const struct connectib_phy_fw_main_config_golan *ptr_struct, FILE* file)
{
	connectib_phy_fw_main_config_golan_print(ptr_struct, file, 0);
}

void connectib_wol_cap_host_port_pack(const struct connectib_wol_cap_host_port *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->magic_packet);

}

void connectib_wol_cap_host_port_unpack(struct connectib_wol_cap_host_port *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->magic_packet = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_wol_cap_host_port_print(const struct connectib_wol_cap_host_port *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wol_cap_host_port ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "magic_packet         : " UH_FMT "\n", ptr_struct->magic_packet);

}

int connectib_wol_cap_host_port_size(void)
{
	 return 4;
}

void connectib_wol_cap_host_port_dump(const struct connectib_wol_cap_host_port *ptr_struct, FILE* file)
{
	connectib_wol_cap_host_port_print(ptr_struct, file, 0);
}

void connectib_ibm_thermal_config_pack(const struct connectib_ibm_thermal_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->thermal_threshold_tcontrol);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->thermal_threshold_tcontrol_low);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->thermal_threshold_twarning);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->thermal_threshold_twarning_low);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->thermal_threshold_tcritical);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->thermal_threshold_tcritical_low);

}

void connectib_ibm_thermal_config_unpack(struct connectib_ibm_thermal_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->thermal_threshold_tcontrol = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->thermal_threshold_tcontrol_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->thermal_threshold_twarning = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->thermal_threshold_twarning_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->thermal_threshold_tcritical = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->thermal_threshold_tcritical_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_ibm_thermal_config_print(const struct connectib_ibm_thermal_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ibm_thermal_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable               : " UH_FMT "\n", ptr_struct->enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "thermal_threshold_tcontrol : " U32H_FMT "\n", ptr_struct->thermal_threshold_tcontrol);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "thermal_threshold_tcontrol_low : " U32H_FMT "\n", ptr_struct->thermal_threshold_tcontrol_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "thermal_threshold_twarning : " U32H_FMT "\n", ptr_struct->thermal_threshold_twarning);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "thermal_threshold_twarning_low : " U32H_FMT "\n", ptr_struct->thermal_threshold_twarning_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "thermal_threshold_tcritical : " U32H_FMT "\n", ptr_struct->thermal_threshold_tcritical);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "thermal_threshold_tcritical_low : " U32H_FMT "\n", ptr_struct->thermal_threshold_tcritical_low);

}

int connectib_ibm_thermal_config_size(void)
{
	 return 28;
}

void connectib_ibm_thermal_config_dump(const struct connectib_ibm_thermal_config *ptr_struct, FILE* file)
{
	connectib_ibm_thermal_config_print(ptr_struct, file, 0);
}

void connectib_ocsd_config_pack(const struct connectib_ocsd_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->internal_caution_threshold);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->extrenal_caution_threshold);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->internal_max_cont_threshold);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->external_max_cont_threshold);

}

void connectib_ocsd_config_unpack(struct connectib_ocsd_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->internal_caution_threshold = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->extrenal_caution_threshold = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->internal_max_cont_threshold = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->external_max_cont_threshold = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_ocsd_config_print(const struct connectib_ocsd_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ocsd_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable               : " UH_FMT "\n", ptr_struct->enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "internal_caution_threshold : " U32H_FMT "\n", ptr_struct->internal_caution_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extrenal_caution_threshold : " U32H_FMT "\n", ptr_struct->extrenal_caution_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "internal_max_cont_threshold : " U32H_FMT "\n", ptr_struct->internal_max_cont_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "external_max_cont_threshold : " U32H_FMT "\n", ptr_struct->external_max_cont_threshold);

}

int connectib_ocsd_config_size(void)
{
	 return 20;
}

void connectib_ocsd_config_dump(const struct connectib_ocsd_config *ptr_struct, FILE* file)
{
	connectib_ocsd_config_print(ptr_struct, file, 0);
}

void connectib_fw_vport0_ini_pack(const struct connectib_fw_vport0_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->partition_cap);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->log_guid_cap);

}

void connectib_fw_vport0_ini_unpack(struct connectib_fw_vport0_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->partition_cap = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->log_guid_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_fw_vport0_ini_print(const struct connectib_fw_vport0_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_vport0_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "partition_cap        : " UH_FMT "\n", ptr_struct->partition_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_guid_cap         : " UH_FMT "\n", ptr_struct->log_guid_cap);

}

int connectib_fw_vport0_ini_size(void)
{
	 return 4;
}

void connectib_fw_vport0_ini_dump(const struct connectib_fw_vport0_ini *ptr_struct, FILE* file)
{
	connectib_fw_vport0_ini_print(ptr_struct, file, 0);
}

void connectib_extended_port_info_ini_pack(const struct connectib_extended_port_info_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->link_speed_supported);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->link_speed_enabled);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_64_byte_cell_size);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->llr_128_byte_cell_size);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->link_up_after_boot);

}

void connectib_extended_port_info_ini_unpack(struct connectib_extended_port_info_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->link_speed_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->link_speed_enabled = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 15;
	ptr_struct->llr_64_byte_cell_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->llr_128_byte_cell_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->link_up_after_boot = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_extended_port_info_ini_print(const struct connectib_extended_port_info_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_extended_port_info_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_speed_supported : " UH_FMT "\n", ptr_struct->link_speed_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_speed_enabled   : " UH_FMT "\n", ptr_struct->link_speed_enabled);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_64_byte_cell_size : " UH_FMT "\n", ptr_struct->llr_64_byte_cell_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "llr_128_byte_cell_size : " UH_FMT "\n", ptr_struct->llr_128_byte_cell_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_up_after_boot   : " UH_FMT "\n", ptr_struct->link_up_after_boot);

}

int connectib_extended_port_info_ini_size(void)
{
	 return 4;
}

void connectib_extended_port_info_ini_dump(const struct connectib_extended_port_info_ini *ptr_struct, FILE* file)
{
	connectib_extended_port_info_ini_print(ptr_struct, file, 0);
}

void connectib_ib_port_info_ini_pack(const struct connectib_ib_port_info_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->cap_mask2);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->diag_code);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->link_roundtrip_lat);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_credit_hint);

	offset = 96;
	connectib_port_cap_mask1_pack(&(ptr_struct->cap_mask), ptr_buff + offset/8);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->link_width_sup);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->link_width_ena);

	offset = 140;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->link_speed_ena);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->down_def_state);

	offset = 132;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->link_speed_sup);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->init_type);

	offset = 184;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vl_arb_high_cap);

	offset = 176;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vl_high_limit);

	offset = 164;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vl_arb_low_cap);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->operational_vl);

	offset = 220;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->neighbor_mtu);

	offset = 216;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mtu_cap);

	offset = 214;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->mc_pkey_trap_suppress_en);

	offset = 206;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->guid_cap);

	offset = 201;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->resp_time_value);

	offset = 196;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->link_speed_ext_ena);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->link_speed_ext_sup);

}

void connectib_ib_port_info_ini_unpack(struct connectib_ib_port_info_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->cap_mask2 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->diag_code = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->link_roundtrip_lat = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 80;
	ptr_struct->max_credit_hint = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	connectib_port_cap_mask1_unpack(&(ptr_struct->cap_mask), ptr_buff + offset/8);

	offset = 152;
	ptr_struct->link_width_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 144;
	ptr_struct->link_width_ena = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 140;
	ptr_struct->link_speed_ena = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 136;
	ptr_struct->down_def_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 132;
	ptr_struct->link_speed_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->init_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 184;
	ptr_struct->vl_arb_high_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 176;
	ptr_struct->vl_high_limit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 164;
	ptr_struct->vl_arb_low_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 160;
	ptr_struct->operational_vl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 220;
	ptr_struct->neighbor_mtu = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 216;
	ptr_struct->mtu_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 214;
	ptr_struct->mc_pkey_trap_suppress_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 206;
	ptr_struct->guid_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 201;
	ptr_struct->resp_time_value = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 196;
	ptr_struct->link_speed_ext_ena = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 192;
	ptr_struct->link_speed_ext_sup = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_ib_port_info_ini_print(const struct connectib_ib_port_info_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ib_port_info_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_mask2            : " UH_FMT "\n", ptr_struct->cap_mask2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "diag_code            : " UH_FMT "\n", ptr_struct->diag_code);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_roundtrip_lat   : " UH_FMT "\n", ptr_struct->link_roundtrip_lat);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_credit_hint      : " UH_FMT "\n", ptr_struct->max_credit_hint);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_mask:\n");
	connectib_port_cap_mask1_print(&(ptr_struct->cap_mask), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_width_sup       : " UH_FMT "\n", ptr_struct->link_width_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_width_ena       : " UH_FMT "\n", ptr_struct->link_width_ena);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_speed_ena       : " UH_FMT "\n", ptr_struct->link_speed_ena);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "down_def_state       : " UH_FMT "\n", ptr_struct->down_def_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_speed_sup       : " UH_FMT "\n", ptr_struct->link_speed_sup);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "init_type            : " UH_FMT "\n", ptr_struct->init_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl_arb_high_cap      : " UH_FMT "\n", ptr_struct->vl_arb_high_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl_high_limit        : " UH_FMT "\n", ptr_struct->vl_high_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl_arb_low_cap       : " UH_FMT "\n", ptr_struct->vl_arb_low_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "operational_vl       : " UH_FMT "\n", ptr_struct->operational_vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "neighbor_mtu         : " UH_FMT "\n", ptr_struct->neighbor_mtu);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtu_cap              : " UH_FMT "\n", ptr_struct->mtu_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mc_pkey_trap_suppress_en : " UH_FMT "\n", ptr_struct->mc_pkey_trap_suppress_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "guid_cap             : " UH_FMT "\n", ptr_struct->guid_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "resp_time_value      : " UH_FMT "\n", ptr_struct->resp_time_value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_speed_ext_ena   : " UH_FMT "\n", ptr_struct->link_speed_ext_ena);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_speed_ext_sup   : " UH_FMT "\n", ptr_struct->link_speed_ext_sup);

}

int connectib_ib_port_info_ini_size(void)
{
	 return 28;
}

void connectib_ib_port_info_ini_dump(const struct connectib_ib_port_info_ini *ptr_struct, FILE* file)
{
	connectib_ib_port_info_ini_print(ptr_struct, file, 0);
}

void connectib_node_info_ini_pack(const struct connectib_node_info_ini *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->node_type);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->class_version);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->base_version);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->system_image_guid_hi);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->system_image_guid_lo);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->node_guid_hi);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->node_guid_lo);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_guid_hi);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_guid_lo);

	offset = 240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->device_id);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->partition_cap);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->revision);

	offset = 296;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->vendor_id);

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port_num);

}

void connectib_node_info_ini_unpack(struct connectib_node_info_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->node_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->class_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->base_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->system_image_guid_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->system_image_guid_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->node_guid_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->node_guid_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->port_guid_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->port_guid_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 240;
	ptr_struct->device_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 224;
	ptr_struct->partition_cap = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 256;
	ptr_struct->revision = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 296;
	ptr_struct->vendor_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 288;
	ptr_struct->local_port_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_node_info_ini_print(const struct connectib_node_info_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_node_info_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "node_type            : " UH_FMT "\n", ptr_struct->node_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_version        : " UH_FMT "\n", ptr_struct->class_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "base_version         : " UH_FMT "\n", ptr_struct->base_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "system_image_guid_hi : " U32H_FMT "\n", ptr_struct->system_image_guid_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "system_image_guid_lo : " U32H_FMT "\n", ptr_struct->system_image_guid_lo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "node_guid_hi         : " U32H_FMT "\n", ptr_struct->node_guid_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "node_guid_lo         : " U32H_FMT "\n", ptr_struct->node_guid_lo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_guid_hi         : " U32H_FMT "\n", ptr_struct->port_guid_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_guid_lo         : " U32H_FMT "\n", ptr_struct->port_guid_lo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_id            : " UH_FMT "\n", ptr_struct->device_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "partition_cap        : " UH_FMT "\n", ptr_struct->partition_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "revision             : " U32H_FMT "\n", ptr_struct->revision);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vendor_id            : " UH_FMT "\n", ptr_struct->vendor_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port_num       : " UH_FMT "\n", ptr_struct->local_port_num);

}

int connectib_node_info_ini_size(void)
{
	 return 64;
}

void connectib_node_info_ini_dump(const struct connectib_node_info_ini *ptr_struct, FILE* file)
{
	connectib_node_info_ini_print(ptr_struct, file, 0);
}

void connectib_led_port_blink_pattern_pack(const struct connectib_led_port_blink_pattern *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->phy_pattern);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->log_pattern);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->length);

}

void connectib_led_port_blink_pattern_unpack(struct connectib_led_port_blink_pattern *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->phy_pattern = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->log_pattern = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 56;
	ptr_struct->length = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_led_port_blink_pattern_print(const struct connectib_led_port_blink_pattern *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_led_port_blink_pattern ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_pattern          : " UH_FMT "\n", ptr_struct->phy_pattern);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_pattern          : " UH_FMT "\n", ptr_struct->log_pattern);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "length               : " UH_FMT "\n", ptr_struct->length);

}

int connectib_led_port_blink_pattern_size(void)
{
	 return 8;
}

void connectib_led_port_blink_pattern_dump(const struct connectib_led_port_blink_pattern *ptr_struct, FILE* file)
{
	connectib_led_port_blink_pattern_print(ptr_struct, file, 0);
}

void connectib_uid_entry_pack(const struct connectib_uid_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->num_allocated);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->step);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->uid);

}

void connectib_uid_entry_unpack(struct connectib_uid_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->num_allocated = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->step = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->uid = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_uid_entry_print(const struct connectib_uid_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_uid_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_allocated        : " UH_FMT "\n", ptr_struct->num_allocated);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "step                 : " UH_FMT "\n", ptr_struct->step);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uid                  : " U64H_FMT "\n", ptr_struct->uid);

}

int connectib_uid_entry_size(void)
{
	 return 16;
}

void connectib_uid_entry_dump(const struct connectib_uid_entry *ptr_struct, FILE* file)
{
	connectib_uid_entry_print(ptr_struct, file, 0);
}

void connectib_module_version_pack(const struct connectib_module_version *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->branch);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->minor);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->major);

}

void connectib_module_version_unpack(struct connectib_module_version *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->branch = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 12;
	ptr_struct->minor = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 0;
	ptr_struct->major = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

}

void connectib_module_version_print(const struct connectib_module_version *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_module_version ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "branch               : " UH_FMT "\n", ptr_struct->branch);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "minor                : " UH_FMT "\n", ptr_struct->minor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "major                : " UH_FMT "\n", ptr_struct->major);

}

int connectib_module_version_size(void)
{
	 return 4;
}

void connectib_module_version_dump(const struct connectib_module_version *ptr_struct, FILE* file)
{
	connectib_module_version_print(ptr_struct, file, 0);
}

void connectib_reset_level_info_pack(const struct connectib_reset_level_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reset_version_pack(&(ptr_struct->ver), ptr_buff + offset/8);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ini_sig);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->reset_type);

	offset = 87;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ini_sig_v);

	offset = 86;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ini_sig_ignore);

}

void connectib_reset_level_info_unpack(struct connectib_reset_level_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reset_version_unpack(&(ptr_struct->ver), ptr_buff + offset/8);

	offset = 32;
	ptr_struct->ini_sig = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 88;
	ptr_struct->reset_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 87;
	ptr_struct->ini_sig_v = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 86;
	ptr_struct->ini_sig_ignore = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_reset_level_info_print(const struct connectib_reset_level_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_reset_level_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ver:\n");
	connectib_reset_version_print(&(ptr_struct->ver), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ini_sig              : " U32H_FMT "\n", ptr_struct->ini_sig);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reset_type           : " UH_FMT "\n", ptr_struct->reset_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ini_sig_v            : " UH_FMT "\n", ptr_struct->ini_sig_v);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ini_sig_ignore       : " UH_FMT "\n", ptr_struct->ini_sig_ignore);

}

int connectib_reset_level_info_size(void)
{
	 return 16;
}

void connectib_reset_level_info_dump(const struct connectib_reset_level_info *ptr_struct, FILE* file)
{
	connectib_reset_level_info_print(ptr_struct, file, 0);
}

void connectib_sysport_packet_data_pack(const struct connectib_sysport_packet_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 183; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 5856, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

}

void connectib_sysport_packet_data_unpack(struct connectib_sysport_packet_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 183; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 5856, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_sysport_packet_data_print(const struct connectib_sysport_packet_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sysport_packet_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 183; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

}

int connectib_sysport_packet_data_size(void)
{
	 return 732;
}

void connectib_sysport_packet_data_dump(const struct connectib_sysport_packet_data *ptr_struct, FILE* file)
{
	connectib_sysport_packet_data_print(ptr_struct, file, 0);
}

void connectib_sysport_packet_properties_pack(const struct connectib_sysport_packet_properties *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->portid);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->prio);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->func_loopback);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->force_loopback);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sx_sniffer);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->packet_is_roce);

}

void connectib_sysport_packet_properties_unpack(struct connectib_sysport_packet_properties *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->gvmi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->portid = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->vl = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->prio = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->func_loopback = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->force_loopback = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->sx_sniffer = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->packet_is_roce = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_sysport_packet_properties_print(const struct connectib_sysport_packet_properties *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sysport_packet_properties ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " U32H_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "portid               : " U32H_FMT "\n", ptr_struct->portid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl                   : " U32H_FMT "\n", ptr_struct->vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prio                 : " U32H_FMT "\n", ptr_struct->prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "func_loopback        : " U32H_FMT "\n", ptr_struct->func_loopback);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_loopback       : " U32H_FMT "\n", ptr_struct->force_loopback);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_sniffer           : " U32H_FMT "\n", ptr_struct->sx_sniffer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_is_roce       : " U32H_FMT "\n", ptr_struct->packet_is_roce);

}

int connectib_sysport_packet_properties_size(void)
{
	 return 32;
}

void connectib_sysport_packet_properties_dump(const struct connectib_sysport_packet_properties *ptr_struct, FILE* file)
{
	connectib_sysport_packet_properties_print(ptr_struct, file, 0);
}

void connectib_rw_gvmi_fw_ctx_sub_structs_pack(const union connectib_rw_gvmi_fw_ctx_sub_structs *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_dcr_odp_events_pack(&(ptr_struct->dcr_odp_event), ptr_buff);
}

void connectib_rw_gvmi_fw_ctx_sub_structs_unpack(union connectib_rw_gvmi_fw_ctx_sub_structs *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_dcr_odp_events_unpack(&(ptr_struct->dcr_odp_event), ptr_buff);
}

void connectib_rw_gvmi_fw_ctx_sub_structs_print(const union connectib_rw_gvmi_fw_ctx_sub_structs *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rw_gvmi_fw_ctx_sub_structs ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcr_odp_event:\n");
	connectib_dcr_odp_events_print(&(ptr_struct->dcr_odp_event), file, indent_level + 1);

}

int connectib_rw_gvmi_fw_ctx_sub_structs_size(void)
{
	 return 64;
}

void connectib_rw_gvmi_fw_ctx_sub_structs_dump(const union connectib_rw_gvmi_fw_ctx_sub_structs *ptr_struct, FILE* file)
{
	connectib_rw_gvmi_fw_ctx_sub_structs_print(ptr_struct, file, 0);
}

void connectib_disable_mask_pack(const union connectib_disable_mask *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_disable_mask_value_pack(&(ptr_struct->value), ptr_buff);
}

void connectib_disable_mask_unpack(union connectib_disable_mask *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_disable_mask_value_unpack(&(ptr_struct->value), ptr_buff);
}

void connectib_disable_mask_print(const union connectib_disable_mask *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_disable_mask ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bits:\n");
	connectib_disable_mask_bits_print(&(ptr_struct->bits), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "value:\n");
	connectib_disable_mask_value_print(&(ptr_struct->value), file, indent_level + 1);

}

int connectib_disable_mask_size(void)
{
	 return 4;
}

void connectib_disable_mask_dump(const union connectib_disable_mask *ptr_struct, FILE* file)
{
	connectib_disable_mask_print(ptr_struct, file, 0);
}

void connectib_drivers_mask_arr_pack(const struct connectib_drivers_mask_arr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 64, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mask[i]);
	}

}

void connectib_drivers_mask_arr_unpack(struct connectib_drivers_mask_arr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 64, 1);
	ptr_struct->mask[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_drivers_mask_arr_print(const struct connectib_drivers_mask_arr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_drivers_mask_arr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mask_%03d            : " U32H_FMT "\n", i, ptr_struct->mask[i]);
	}

}

int connectib_drivers_mask_arr_size(void)
{
	 return 8;
}

void connectib_drivers_mask_arr_dump(const struct connectib_drivers_mask_arr *ptr_struct, FILE* file)
{
	connectib_drivers_mask_arr_print(ptr_struct, file, 0);
}

void connectib_vport_info_pack(const struct connectib_vport_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->port_guid);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->node_guid);

	offset = 128;
	connectib_mac_address_layout_pack(&(ptr_struct->permanent_address), ptr_buff + offset/8);

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->cap_mask);

	offset = 196;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->allowed_uc_list_size);

	offset = 195;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_roce_disable_shadow);

	offset = 194;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_roce_disable);

	offset = 193;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->esw_ctx_dirty);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->roce_en);

	offset = 240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->p_key_violations);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->q_key_violations);

	offset = 276;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->allowed_mc_list_size);

	offset = 275;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->event_on_uc_address_change);

	offset = 274;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->event_on_mc_address_change);

	offset = 273;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->event_on_vlan_change);

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->event_on_promisc_change);

	offset = 260;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->allowed_vlan_list_size);

	offset = 259;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->arm_change_event);

	offset = 258;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->promisc_all);

	offset = 257;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->promisc_mc);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->promisc_uc);

	offset = 297;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_mc_local_lb);

	offset = 296;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_uc_local_lb);

	offset = 293;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->min_wqe_inline_mode);

	offset = 292;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->uc_list_offset_selector);

	offset = 291;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mc_list_offset_selector);

	offset = 290;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vlan_list_offset_selector);

	offset = 289;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mtu_vld);

	offset = 288;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->event_on_mtu_change);

	offset = 336;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->mtu);

	offset = 320;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->flow_counter_head);

	offset = 352;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->esw_egress_info), ptr_buff + offset/8);

	offset = 384;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->esw_ingress_info), ptr_buff + offset/8);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->extension_event_mask);

}

void connectib_vport_info_unpack(struct connectib_vport_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->port_guid = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->node_guid = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 128;
	connectib_mac_address_layout_unpack(&(ptr_struct->permanent_address), ptr_buff + offset/8);

	offset = 208;
	ptr_struct->cap_mask = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 196;
	ptr_struct->allowed_uc_list_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 195;
	ptr_struct->force_roce_disable_shadow = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 194;
	ptr_struct->force_roce_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 193;
	ptr_struct->esw_ctx_dirty = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 192;
	ptr_struct->roce_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 240;
	ptr_struct->p_key_violations = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 224;
	ptr_struct->q_key_violations = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 276;
	ptr_struct->allowed_mc_list_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 275;
	ptr_struct->event_on_uc_address_change = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 274;
	ptr_struct->event_on_mc_address_change = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 273;
	ptr_struct->event_on_vlan_change = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 272;
	ptr_struct->event_on_promisc_change = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 260;
	ptr_struct->allowed_vlan_list_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 259;
	ptr_struct->arm_change_event = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 258;
	ptr_struct->promisc_all = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 257;
	ptr_struct->promisc_mc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 256;
	ptr_struct->promisc_uc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 297;
	ptr_struct->disable_mc_local_lb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 296;
	ptr_struct->disable_uc_local_lb = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 293;
	ptr_struct->min_wqe_inline_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 292;
	ptr_struct->uc_list_offset_selector = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 291;
	ptr_struct->mc_list_offset_selector = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 290;
	ptr_struct->vlan_list_offset_selector = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 289;
	ptr_struct->mtu_vld = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 288;
	ptr_struct->event_on_mtu_change = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 336;
	ptr_struct->mtu = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 320;
	ptr_struct->flow_counter_head = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 352;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->esw_egress_info), ptr_buff + offset/8);

	offset = 384;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->esw_ingress_info), ptr_buff + offset/8);

	offset = 416;
	ptr_struct->extension_event_mask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_vport_info_print(const struct connectib_vport_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_vport_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_guid            : " U64H_FMT "\n", ptr_struct->port_guid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "node_guid            : " U64H_FMT "\n", ptr_struct->node_guid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "permanent_address:\n");
	connectib_mac_address_layout_print(&(ptr_struct->permanent_address), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_mask             : " UH_FMT "\n", ptr_struct->cap_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allowed_uc_list_size : " UH_FMT "\n", ptr_struct->allowed_uc_list_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_roce_disable_shadow : " UH_FMT "\n", ptr_struct->force_roce_disable_shadow);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_roce_disable   : " UH_FMT "\n", ptr_struct->force_roce_disable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "esw_ctx_dirty        : " UH_FMT "\n", ptr_struct->esw_ctx_dirty);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "roce_en              : " UH_FMT "\n", ptr_struct->roce_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "p_key_violations     : " UH_FMT "\n", ptr_struct->p_key_violations);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "q_key_violations     : " UH_FMT "\n", ptr_struct->q_key_violations);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allowed_mc_list_size : " UH_FMT "\n", ptr_struct->allowed_mc_list_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_on_uc_address_change : " UH_FMT "\n", ptr_struct->event_on_uc_address_change);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_on_mc_address_change : " UH_FMT "\n", ptr_struct->event_on_mc_address_change);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_on_vlan_change : " UH_FMT "\n", ptr_struct->event_on_vlan_change);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_on_promisc_change : " UH_FMT "\n", ptr_struct->event_on_promisc_change);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allowed_vlan_list_size : " UH_FMT "\n", ptr_struct->allowed_vlan_list_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "arm_change_event     : " UH_FMT "\n", ptr_struct->arm_change_event);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "promisc_all          : " UH_FMT "\n", ptr_struct->promisc_all);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "promisc_mc           : " UH_FMT "\n", ptr_struct->promisc_mc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "promisc_uc           : " UH_FMT "\n", ptr_struct->promisc_uc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_mc_local_lb  : " UH_FMT "\n", ptr_struct->disable_mc_local_lb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_uc_local_lb  : " UH_FMT "\n", ptr_struct->disable_uc_local_lb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "min_wqe_inline_mode  : " UH_FMT "\n", ptr_struct->min_wqe_inline_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "uc_list_offset_selector : " UH_FMT "\n", ptr_struct->uc_list_offset_selector);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mc_list_offset_selector : " UH_FMT "\n", ptr_struct->mc_list_offset_selector);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vlan_list_offset_selector : " UH_FMT "\n", ptr_struct->vlan_list_offset_selector);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtu_vld              : " UH_FMT "\n", ptr_struct->mtu_vld);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_on_mtu_change  : " UH_FMT "\n", ptr_struct->event_on_mtu_change);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtu                  : " UH_FMT "\n", ptr_struct->mtu);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_counter_head    : " UH_FMT "\n", ptr_struct->flow_counter_head);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "esw_egress_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->esw_egress_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "esw_ingress_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->esw_ingress_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extension_event_mask : " U32H_FMT "\n", ptr_struct->extension_event_mask);

}

int connectib_vport_info_size(void)
{
	 return 64;
}

void connectib_vport_info_dump(const struct connectib_vport_info *ptr_struct, FILE* file)
{
	connectib_vport_info_print(ptr_struct, file, 0);
}

void connectib_vport_shadow_pack(const struct connectib_vport_shadow *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->port_guid);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->node_guid);

	offset = 128;
	connectib_mac_address_layout_pack(&(ptr_struct->permanent_address), ptr_buff + offset/8);

}

void connectib_vport_shadow_unpack(struct connectib_vport_shadow *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->port_guid = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->node_guid = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 128;
	connectib_mac_address_layout_unpack(&(ptr_struct->permanent_address), ptr_buff + offset/8);

}

void connectib_vport_shadow_print(const struct connectib_vport_shadow *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_vport_shadow ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_guid            : " U64H_FMT "\n", ptr_struct->port_guid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "node_guid            : " U64H_FMT "\n", ptr_struct->node_guid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "permanent_address:\n");
	connectib_mac_address_layout_print(&(ptr_struct->permanent_address), file, indent_level + 1);

}

int connectib_vport_shadow_size(void)
{
	 return 32;
}

void connectib_vport_shadow_dump(const struct connectib_vport_shadow *ptr_struct, FILE* file)
{
	connectib_vport_shadow_print(ptr_struct, file, 0);
}

void connectib_ocbb_pci_data_pack(const struct connectib_ocbb_pci_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pci_vendor_id);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pci_device_id);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pci_subsys_vendor_id);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pci_subsys_device_id);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->num_of_pcie_func_per_port);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pci_bus_number);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->pci_class);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pci_bus_rx_utilization);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pci_bus_tx_utilization);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pci_function_num);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pci_errors_cap_id);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pci_errors_uncor_stat);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pci_errors_uncor_mask);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pci_errors_corr_stat);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pci_errors_corr_mask);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pci_errors_cap_and_ctrl);

}

void connectib_ocbb_pci_data_unpack(struct connectib_ocbb_pci_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->pci_vendor_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->pci_device_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	ptr_struct->pci_subsys_vendor_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->pci_subsys_device_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 80;
	ptr_struct->num_of_pcie_func_per_port = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->pci_bus_number = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 104;
	ptr_struct->pci_class = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 152;
	ptr_struct->pci_bus_rx_utilization = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 144;
	ptr_struct->pci_bus_tx_utilization = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 136;
	ptr_struct->pci_function_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 160;
	ptr_struct->pci_errors_cap_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->pci_errors_uncor_stat = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->pci_errors_uncor_mask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->pci_errors_corr_stat = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->pci_errors_corr_mask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->pci_errors_cap_and_ctrl = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_ocbb_pci_data_print(const struct connectib_ocbb_pci_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ocbb_pci_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_vendor_id        : " UH_FMT "\n", ptr_struct->pci_vendor_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_device_id        : " UH_FMT "\n", ptr_struct->pci_device_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_subsys_vendor_id : " UH_FMT "\n", ptr_struct->pci_subsys_vendor_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_subsys_device_id : " UH_FMT "\n", ptr_struct->pci_subsys_device_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_pcie_func_per_port : " UH_FMT "\n", ptr_struct->num_of_pcie_func_per_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_bus_number       : " UH_FMT "\n", ptr_struct->pci_bus_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_class            : " UH_FMT "\n", ptr_struct->pci_class);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_bus_rx_utilization : " UH_FMT "\n", ptr_struct->pci_bus_rx_utilization);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_bus_tx_utilization : " UH_FMT "\n", ptr_struct->pci_bus_tx_utilization);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_function_num     : " UH_FMT "\n", ptr_struct->pci_function_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_errors_cap_id    : " U32H_FMT "\n", ptr_struct->pci_errors_cap_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_errors_uncor_stat : " U32H_FMT "\n", ptr_struct->pci_errors_uncor_stat);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_errors_uncor_mask : " U32H_FMT "\n", ptr_struct->pci_errors_uncor_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_errors_corr_stat : " U32H_FMT "\n", ptr_struct->pci_errors_corr_stat);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_errors_corr_mask : " U32H_FMT "\n", ptr_struct->pci_errors_corr_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_errors_cap_and_ctrl : " U32H_FMT "\n", ptr_struct->pci_errors_cap_and_ctrl);

}

int connectib_ocbb_pci_data_size(void)
{
	 return 44;
}

void connectib_ocbb_pci_data_dump(const struct connectib_ocbb_pci_data *ptr_struct, FILE* file)
{
	connectib_ocbb_pci_data_print(ptr_struct, file, 0);
}

void connectib_ocbb_fields_pack(const struct connectib_ocbb_fields *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_banner_pack(&(ptr_struct->fw_banner_str), ptr_buff + offset/8);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->hca_cap_num_ports);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->sriov_en);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_num_vmq_per_port);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_num_vmq_per_pf);

	offset = 176;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lowest_port_recv);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(248, 8, i, 4096, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->lldp_rx_get_chassis_id_tlv[i]);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(344, 8, i, 4096, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->lldp_rx_get_port_id_tlv[i]);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(440, 8, i, 4096, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->core_utilization[i]);
	}

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(472, 8, i, 4096, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->device_bios_version[i]);
	}

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(568, 8, i, 4096, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->device_hw_version[i]);
	}

	offset = 664;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_type);

	offset = 656;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->flow_control_indicator);

	offset = 648;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->flex_ten_mode_en);

	offset = 672;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_speed);

	offset = 728;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_phy_link_status);

	offset = 736;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_speed_supported);

	offset = 792;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->function_type);

	offset = 816;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->mtu_size);

	offset = 800;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->feature_flags);

	offset = 848;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_offload_size);

	offset = 832;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lso_min_seg_count);

	offset = 888;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->promiscuous_mode);

	offset = 864;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->tx_descr_size);

	offset = 912;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rx_descr_size);

	offset = 896;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->iov_offload);

	offset = 944;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->num_of_netq);

	offset = 960;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcp_prio_map_table);

	offset = 1016;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->max_num_pci_vfs_per_pfs);

	offset = 1008;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->os_driver_state);

	offset = 1024;
	connectib_ocbb_fields_from_vpd_pack(&(ptr_struct->vpd_data), ptr_buff + offset/8);

	offset = 1592;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->device_power_reset_state);

	offset = 1568;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_num_of_vfs);

	offset = 2048;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_discards);

	offset = 2080;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rx_errors);

	offset = 2112;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_errors);

	offset = 2144;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->tx_discards);

	offset = 2176;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->rx_frames_received);

	offset = 2240;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->rx_bytes_received);

	offset = 2304;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->tx_frames_sent);

	offset = 2368;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->tx_bytes_sent);

	offset = 2432;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->pfc_frames_sent);

	offset = 2496;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->pfc_frames_received);

	offset = 2560;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->prog_max_addr_1);

	offset = 2624;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->prog_max_addr_2);

	offset = 2704;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_mtu);

	offset = 2688;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_num_of_pci_func_per_port);

}

void connectib_ocbb_fields_unpack(struct connectib_ocbb_fields *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_banner_unpack(&(ptr_struct->fw_banner_str), ptr_buff + offset/8);

	offset = 112;
	ptr_struct->hca_cap_num_ports = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 104;
	ptr_struct->sriov_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 144;
	ptr_struct->max_num_vmq_per_port = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 128;
	ptr_struct->max_num_vmq_per_pf = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 176;
	ptr_struct->lowest_port_recv = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(248, 8, i, 4096, 1);
	ptr_struct->lldp_rx_get_chassis_id_tlv[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(344, 8, i, 4096, 1);
	ptr_struct->lldp_rx_get_port_id_tlv[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(440, 8, i, 4096, 1);
	ptr_struct->core_utilization[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(472, 8, i, 4096, 1);
	ptr_struct->device_bios_version[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(568, 8, i, 4096, 1);
	ptr_struct->device_hw_version[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	offset = 664;
	ptr_struct->port_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 656;
	ptr_struct->flow_control_indicator = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 648;
	ptr_struct->flex_ten_mode_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 672;
	ptr_struct->port_speed = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 728;
	ptr_struct->port_phy_link_status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 736;
	ptr_struct->max_speed_supported = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 792;
	ptr_struct->function_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 816;
	ptr_struct->mtu_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 800;
	ptr_struct->feature_flags = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 848;
	ptr_struct->max_offload_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 832;
	ptr_struct->lso_min_seg_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 888;
	ptr_struct->promiscuous_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 864;
	ptr_struct->tx_descr_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 912;
	ptr_struct->rx_descr_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 896;
	ptr_struct->iov_offload = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 944;
	ptr_struct->num_of_netq = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 960;
	ptr_struct->pcp_prio_map_table = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 1016;
	ptr_struct->max_num_pci_vfs_per_pfs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 1008;
	ptr_struct->os_driver_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 1024;
	connectib_ocbb_fields_from_vpd_unpack(&(ptr_struct->vpd_data), ptr_buff + offset/8);

	offset = 1592;
	ptr_struct->device_power_reset_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 1568;
	ptr_struct->max_num_of_vfs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 2048;
	ptr_struct->rx_discards = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 2080;
	ptr_struct->rx_errors = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 2112;
	ptr_struct->tx_errors = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 2144;
	ptr_struct->tx_discards = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 2176;
	ptr_struct->rx_frames_received = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2240;
	ptr_struct->rx_bytes_received = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2304;
	ptr_struct->tx_frames_sent = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2368;
	ptr_struct->tx_bytes_sent = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2432;
	ptr_struct->pfc_frames_sent = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2496;
	ptr_struct->pfc_frames_received = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2560;
	ptr_struct->prog_max_addr_1 = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2624;
	ptr_struct->prog_max_addr_2 = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2704;
	ptr_struct->max_mtu = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 2688;
	ptr_struct->max_num_of_pci_func_per_port = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_ocbb_fields_print(const struct connectib_ocbb_fields *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ocbb_fields ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_banner_str:\n");
	connectib_fw_banner_print(&(ptr_struct->fw_banner_str), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hca_cap_num_ports    : " UH_FMT "\n", ptr_struct->hca_cap_num_ports);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sriov_en             : " UH_FMT "\n", ptr_struct->sriov_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_vmq_per_port : " UH_FMT "\n", ptr_struct->max_num_vmq_per_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_vmq_per_pf   : " UH_FMT "\n", ptr_struct->max_num_vmq_per_pf);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lowest_port_recv     : " UH_FMT "\n", ptr_struct->lowest_port_recv);

	for (i = 0; i < 12; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_rx_get_chassis_id_tlv_%03d : " UH_FMT "\n", i, ptr_struct->lldp_rx_get_chassis_id_tlv[i]);
	}

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lldp_rx_get_port_id_tlv_%03d : " UH_FMT "\n", i, ptr_struct->lldp_rx_get_port_id_tlv[i]);
	}

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_utilization_%03d : " UH_FMT "\n", i, ptr_struct->core_utilization[i]);
	}

	for (i = 0; i < 12; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_bios_version_%03d : " UH_FMT "\n", i, ptr_struct->device_bios_version[i]);
	}

	for (i = 0; i < 12; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_hw_version_%03d : " UH_FMT "\n", i, ptr_struct->device_hw_version[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_type            : " UH_FMT "\n", ptr_struct->port_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_control_indicator : " UH_FMT "\n", ptr_struct->flow_control_indicator);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flex_ten_mode_en     : " UH_FMT "\n", ptr_struct->flex_ten_mode_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_speed           : " U32H_FMT "\n", ptr_struct->port_speed);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_phy_link_status : " UH_FMT "\n", ptr_struct->port_phy_link_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_speed_supported  : " U32H_FMT "\n", ptr_struct->max_speed_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "function_type        : " UH_FMT "\n", ptr_struct->function_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mtu_size             : " UH_FMT "\n", ptr_struct->mtu_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "feature_flags        : " UH_FMT "\n", ptr_struct->feature_flags);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_offload_size     : " UH_FMT "\n", ptr_struct->max_offload_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lso_min_seg_count    : " UH_FMT "\n", ptr_struct->lso_min_seg_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "promiscuous_mode     : " UH_FMT "\n", ptr_struct->promiscuous_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_descr_size        : " UH_FMT "\n", ptr_struct->tx_descr_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_descr_size        : " UH_FMT "\n", ptr_struct->rx_descr_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iov_offload          : " UH_FMT "\n", ptr_struct->iov_offload);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_netq          : " UH_FMT "\n", ptr_struct->num_of_netq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcp_prio_map_table   : " U32H_FMT "\n", ptr_struct->pcp_prio_map_table);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_pci_vfs_per_pfs : " UH_FMT "\n", ptr_struct->max_num_pci_vfs_per_pfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "os_driver_state      : " UH_FMT "\n", ptr_struct->os_driver_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_data:\n");
	connectib_ocbb_fields_from_vpd_print(&(ptr_struct->vpd_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_power_reset_state : " UH_FMT "\n", ptr_struct->device_power_reset_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_of_vfs       : " UH_FMT "\n", ptr_struct->max_num_of_vfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_discards          : " U32H_FMT "\n", ptr_struct->rx_discards);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_errors            : " U32H_FMT "\n", ptr_struct->rx_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_errors            : " U32H_FMT "\n", ptr_struct->tx_errors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_discards          : " U32H_FMT "\n", ptr_struct->tx_discards);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_frames_received   : " U64H_FMT "\n", ptr_struct->rx_frames_received);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_bytes_received    : " U64H_FMT "\n", ptr_struct->rx_bytes_received);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_frames_sent       : " U64H_FMT "\n", ptr_struct->tx_frames_sent);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_bytes_sent        : " U64H_FMT "\n", ptr_struct->tx_bytes_sent);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pfc_frames_sent      : " U64H_FMT "\n", ptr_struct->pfc_frames_sent);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pfc_frames_received  : " U64H_FMT "\n", ptr_struct->pfc_frames_received);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prog_max_addr_1      : " U64H_FMT "\n", ptr_struct->prog_max_addr_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prog_max_addr_2      : " U64H_FMT "\n", ptr_struct->prog_max_addr_2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_mtu              : " UH_FMT "\n", ptr_struct->max_mtu);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_num_of_pci_func_per_port : " UH_FMT "\n", ptr_struct->max_num_of_pci_func_per_port);

}

int connectib_ocbb_fields_size(void)
{
	 return 512;
}

void connectib_ocbb_fields_dump(const struct connectib_ocbb_fields *ptr_struct, FILE* file)
{
	connectib_ocbb_fields_print(ptr_struct, file, 0);
}

void connectib_module_db_pre_define_st_pack(const struct connectib_module_db_pre_define_st *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_module_db_pre_define_st_unpack(struct connectib_module_db_pre_define_st *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

}

void connectib_module_db_pre_define_st_print(const struct connectib_module_db_pre_define_st *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_module_db_pre_define_st ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

}

int connectib_module_db_pre_define_st_size(void)
{
	 return 56;
}

void connectib_module_db_pre_define_st_dump(const struct connectib_module_db_pre_define_st *ptr_struct, FILE* file)
{
	connectib_module_db_pre_define_st_print(ptr_struct, file, 0);
}

void connectib_fw_driver_version_pack(const struct connectib_fw_driver_version *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

}

void connectib_fw_driver_version_unpack(struct connectib_fw_driver_version *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_fw_driver_version_print(const struct connectib_fw_driver_version *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_driver_version ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

}

int connectib_fw_driver_version_size(void)
{
	 return 64;
}

void connectib_fw_driver_version_dump(const struct connectib_fw_driver_version *ptr_struct, FILE* file)
{
	connectib_fw_driver_version_print(ptr_struct, file, 0);
}

void connectib_iterator_result_pack(const struct connectib_iterator_result *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->port_0_1);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pp);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->req_res_);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_fw_sq);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sqn);

}

void connectib_iterator_result_unpack(struct connectib_iterator_result *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 14;
	ptr_struct->port_0_1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 10;
	ptr_struct->vl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 6;
	ptr_struct->sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 2;
	ptr_struct->pp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 1;
	ptr_struct->req_res_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->is_fw_sq = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->sqn = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_iterator_result_print(const struct connectib_iterator_result *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_iterator_result ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_0_1             : " UH_FMT "\n", ptr_struct->port_0_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl                   : " UH_FMT "\n", ptr_struct->vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl                   : " UH_FMT "\n", ptr_struct->sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pp                   : " UH_FMT "\n", ptr_struct->pp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "req_res_             : " UH_FMT "\n", ptr_struct->req_res_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_fw_sq             : " UH_FMT "\n", ptr_struct->is_fw_sq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sqn                  : " U32H_FMT "\n", ptr_struct->sqn);

}

int connectib_iterator_result_size(void)
{
	 return 8;
}

void connectib_iterator_result_dump(const struct connectib_iterator_result *ptr_struct, FILE* file)
{
	connectib_iterator_result_print(ptr_struct, file, 0);
}

void connectib_general_fault_injector_pair_pack(const struct connectib_general_fault_injector_pair *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->freq);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->duration);

}

void connectib_general_fault_injector_pair_unpack(struct connectib_general_fault_injector_pair *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->freq = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->duration = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_general_fault_injector_pair_print(const struct connectib_general_fault_injector_pair *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_general_fault_injector_pair ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "freq                 : " UH_FMT "\n", ptr_struct->freq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "duration             : " UH_FMT "\n", ptr_struct->duration);

}

int connectib_general_fault_injector_pair_size(void)
{
	 return 4;
}

void connectib_general_fault_injector_pair_dump(const struct connectib_general_fault_injector_pair *ptr_struct, FILE* file)
{
	connectib_general_fault_injector_pair_print(ptr_struct, file, 0);
}

void connectib_dmfs_gvmi_ctx_pack(const struct connectib_dmfs_gvmi_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->rx_nic_info), ptr_buff + offset/8);

	offset = 32;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->sx_nic_info), ptr_buff + offset/8);

	offset = 64;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->esw_fdb_info), ptr_buff + offset/8);

	offset = 96;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->rx_sniffer_info), ptr_buff + offset/8);

	offset = 128;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->sx_sniffer_info), ptr_buff + offset/8);

	offset = 160;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->rx_rdma_info), ptr_buff + offset/8);

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->sx_sniffer_pf_ref_count);

	offset = 196;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->active_vfs_counter);

	offset = 195;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_gvmi_init_hca_set);

	offset = 194;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vport_lag_enable);

	offset = 193;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cvlan_stripping_enable);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->gvmi_sx_sniffer_enable);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mpfs_cache_pf_mac_guid_counter);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mpfs_cache_pf_mac_vlan_counter);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ft_link_list_head);

	offset = 336;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->nic_flow_counters_cntr);

	offset = 328;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->load_balance_mac_ref_count);

	offset = 376;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->encap_en_ref_count);

	offset = 352;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->ipoib_enhanced_qps_counter);

}

void connectib_dmfs_gvmi_ctx_unpack(struct connectib_dmfs_gvmi_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->rx_nic_info), ptr_buff + offset/8);

	offset = 32;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->sx_nic_info), ptr_buff + offset/8);

	offset = 64;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->esw_fdb_info), ptr_buff + offset/8);

	offset = 96;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->rx_sniffer_info), ptr_buff + offset/8);

	offset = 128;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->sx_sniffer_info), ptr_buff + offset/8);

	offset = 160;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->rx_rdma_info), ptr_buff + offset/8);

	offset = 208;
	ptr_struct->sx_sniffer_pf_ref_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 196;
	ptr_struct->active_vfs_counter = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 195;
	ptr_struct->is_gvmi_init_hca_set = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 194;
	ptr_struct->vport_lag_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 193;
	ptr_struct->cvlan_stripping_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 192;
	ptr_struct->gvmi_sx_sniffer_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 224;
	ptr_struct->mpfs_cache_pf_mac_guid_counter = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->mpfs_cache_pf_mac_vlan_counter = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->ft_link_list_head = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 336;
	ptr_struct->nic_flow_counters_cntr = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 328;
	ptr_struct->load_balance_mac_ref_count = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 376;
	ptr_struct->encap_en_ref_count = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 352;
	ptr_struct->ipoib_enhanced_qps_counter = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_dmfs_gvmi_ctx_print(const struct connectib_dmfs_gvmi_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dmfs_gvmi_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_nic_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->rx_nic_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_nic_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->sx_nic_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "esw_fdb_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->esw_fdb_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_sniffer_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->rx_sniffer_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_sniffer_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->sx_sniffer_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_rdma_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->rx_rdma_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_sniffer_pf_ref_count : " UH_FMT "\n", ptr_struct->sx_sniffer_pf_ref_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "active_vfs_counter   : " UH_FMT "\n", ptr_struct->active_vfs_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_gvmi_init_hca_set : " UH_FMT "\n", ptr_struct->is_gvmi_init_hca_set);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_lag_enable     : " UH_FMT "\n", ptr_struct->vport_lag_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cvlan_stripping_enable : " UH_FMT "\n", ptr_struct->cvlan_stripping_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_sx_sniffer_enable : " UH_FMT "\n", ptr_struct->gvmi_sx_sniffer_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mpfs_cache_pf_mac_guid_counter : " U32H_FMT "\n", ptr_struct->mpfs_cache_pf_mac_guid_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mpfs_cache_pf_mac_vlan_counter : " U32H_FMT "\n", ptr_struct->mpfs_cache_pf_mac_vlan_counter);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ft_link_list_head    : " U32H_FMT "\n", ptr_struct->ft_link_list_head);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nic_flow_counters_cntr : " UH_FMT "\n", ptr_struct->nic_flow_counters_cntr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "load_balance_mac_ref_count : " UH_FMT "\n", ptr_struct->load_balance_mac_ref_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encap_en_ref_count   : " UH_FMT "\n", ptr_struct->encap_en_ref_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ipoib_enhanced_qps_counter : " UH_FMT "\n", ptr_struct->ipoib_enhanced_qps_counter);

}

int connectib_dmfs_gvmi_ctx_size(void)
{
	 return 64;
}

void connectib_dmfs_gvmi_ctx_dump(const struct connectib_dmfs_gvmi_ctx *ptr_struct, FILE* file)
{
	connectib_dmfs_gvmi_ctx_print(ptr_struct, file, 0);
}

void connectib_dmfs_data_pack(const struct connectib_dmfs_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(1, 1, i, 256, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mpfs_cache_enable[i]);
	}

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->steering_loopback_mode);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mpfs_cache_chicken);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(24, 8, i, 256, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->bmc_member_count[i]);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(48, 16, i, 256, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->sx_sniffer_ref_counter[i]);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 256, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mpfs_cache_mac_guid_addrs_counter[i]);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 256, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mpfs_cache_mac_vlan_addrs_counter[i]);
	}

}

void connectib_dmfs_data_unpack(struct connectib_dmfs_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(1, 1, i, 256, 1);
	ptr_struct->mpfs_cache_enable[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);
	}

	offset = 29;
	ptr_struct->steering_loopback_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->mpfs_cache_chicken = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(24, 8, i, 256, 1);
	ptr_struct->bmc_member_count[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(48, 16, i, 256, 1);
	ptr_struct->sx_sniffer_ref_counter[i] = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 256, 1);
	ptr_struct->mpfs_cache_mac_guid_addrs_counter[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 256, 1);
	ptr_struct->mpfs_cache_mac_vlan_addrs_counter[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_dmfs_data_print(const struct connectib_dmfs_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dmfs_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mpfs_cache_enable_%03d : " UH_FMT "\n", i, ptr_struct->mpfs_cache_enable[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_loopback_mode : " UH_FMT "\n", ptr_struct->steering_loopback_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mpfs_cache_chicken   : " UH_FMT "\n", ptr_struct->mpfs_cache_chicken);

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bmc_member_count_%03d : " UH_FMT "\n", i, ptr_struct->bmc_member_count[i]);
	}

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_sniffer_ref_counter_%03d : " UH_FMT "\n", i, ptr_struct->sx_sniffer_ref_counter[i]);
	}

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mpfs_cache_mac_guid_addrs_counter_%03d : " U32H_FMT "\n", i, ptr_struct->mpfs_cache_mac_guid_addrs_counter[i]);
	}

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mpfs_cache_mac_vlan_addrs_counter_%03d : " U32H_FMT "\n", i, ptr_struct->mpfs_cache_mac_vlan_addrs_counter[i]);
	}

}

int connectib_dmfs_data_size(void)
{
	 return 32;
}

void connectib_dmfs_data_dump(const struct connectib_dmfs_data *ptr_struct, FILE* file)
{
	connectib_dmfs_data_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ste_resources_list_out_pack(const struct connectib_icmd_get_ste_resources_list_out *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->resources_count);

	for (i = 0; i < 63; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 2048, 1);
	connectib_resource_type_entry_pack(&(ptr_struct->resource_type[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_get_ste_resources_list_out_unpack(struct connectib_icmd_get_ste_resources_list_out *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->resources_count = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	for (i = 0; i < 63; i++) {
	offset = adb2c_calc_array_field_address(32, 32, i, 2048, 1);
	connectib_resource_type_entry_unpack(&(ptr_struct->resource_type[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_get_ste_resources_list_out_print(const struct connectib_icmd_get_ste_resources_list_out *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ste_resources_list_out ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "resources_count      : " UH_FMT "\n", ptr_struct->resources_count);

	for (i = 0; i < 63; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "resource_type_%03d:\n", i);
	connectib_resource_type_entry_print(&(ptr_struct->resource_type[i]), file, indent_level + 1);
	}

}

int connectib_icmd_get_ste_resources_list_out_size(void)
{
	 return 256;
}

void connectib_icmd_get_ste_resources_list_out_dump(const struct connectib_icmd_get_ste_resources_list_out *ptr_struct, FILE* file)
{
	connectib_icmd_get_ste_resources_list_out_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ste_resources_list_in_pack(const struct connectib_icmd_get_ste_resources_list_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

}

void connectib_icmd_get_ste_resources_list_in_unpack(struct connectib_icmd_get_ste_resources_list_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_get_ste_resources_list_in_print(const struct connectib_icmd_get_ste_resources_list_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ste_resources_list_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

}

int connectib_icmd_get_ste_resources_list_in_size(void)
{
	 return 16;
}

void connectib_icmd_get_ste_resources_list_in_dump(const struct connectib_icmd_get_ste_resources_list_in *ptr_struct, FILE* file)
{
	connectib_icmd_get_ste_resources_list_in_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ste_open_resources_out_pack(const struct connectib_icmd_get_ste_open_resources_out *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->more);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->output_size);

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(64, 128, i, 6144, 1);
	connectib_index_and_length_pack(&(ptr_struct->opened_resource[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_get_ste_open_resources_out_unpack(struct connectib_icmd_get_ste_open_resources_out *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->more = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->output_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(64, 128, i, 6144, 1);
	connectib_index_and_length_unpack(&(ptr_struct->opened_resource[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_get_ste_open_resources_out_print(const struct connectib_icmd_get_ste_open_resources_out *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ste_open_resources_out ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "more                 : " UH_FMT "\n", ptr_struct->more);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "output_size          : " U32H_FMT "\n", ptr_struct->output_size);

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opened_resource_%03d:\n", i);
	connectib_index_and_length_print(&(ptr_struct->opened_resource[i]), file, indent_level + 1);
	}

}

int connectib_icmd_get_ste_open_resources_out_size(void)
{
	 return 768;
}

void connectib_icmd_get_ste_open_resources_out_dump(const struct connectib_icmd_get_ste_open_resources_out *ptr_struct, FILE* file)
{
	connectib_icmd_get_ste_open_resources_out_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ste_open_resources_in_pack(const struct connectib_icmd_get_ste_open_resources_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->icm_resource_type);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->start_index);

}

void connectib_icmd_get_ste_open_resources_in_unpack(struct connectib_icmd_get_ste_open_resources_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->icm_resource_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->start_index = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_get_ste_open_resources_in_print(const struct connectib_icmd_get_ste_open_resources_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ste_open_resources_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icm_resource_type    : " U32H_FMT "\n", ptr_struct->icm_resource_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "start_index          : " U64H_FMT "\n", ptr_struct->start_index);

}

int connectib_icmd_get_ste_open_resources_in_size(void)
{
	 return 16;
}

void connectib_icmd_get_ste_open_resources_in_dump(const struct connectib_icmd_get_ste_open_resources_in *ptr_struct, FILE* file)
{
	connectib_icmd_get_ste_open_resources_in_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fte_out_pack(const struct connectib_icmd_get_fte_out *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->action);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->group_id);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->flow_tag);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->destination_list_size);

	offset = 136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->flow_counter_list_size);

	offset = 512;
	connectib_flow_table_entry_match_set_lyr_2_4_pack(&(ptr_struct->outer_headers), ptr_buff + offset/8);

	offset = 1024;
	connectib_flow_table_entry_match_set_misc_pack(&(ptr_struct->misc_parameters), ptr_buff + offset/8);

	offset = 1536;
	connectib_flow_table_entry_match_set_lyr_2_4_pack(&(ptr_struct->inner_headers), ptr_buff + offset/8);

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(2048, 64, i, 6144, 1);
	connectib_flow_list_pack(&(ptr_struct->destination[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_get_fte_out_unpack(struct connectib_icmd_get_fte_out *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->action = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->group_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->flow_tag = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 104;
	ptr_struct->destination_list_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 136;
	ptr_struct->flow_counter_list_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 512;
	connectib_flow_table_entry_match_set_lyr_2_4_unpack(&(ptr_struct->outer_headers), ptr_buff + offset/8);

	offset = 1024;
	connectib_flow_table_entry_match_set_misc_unpack(&(ptr_struct->misc_parameters), ptr_buff + offset/8);

	offset = 1536;
	connectib_flow_table_entry_match_set_lyr_2_4_unpack(&(ptr_struct->inner_headers), ptr_buff + offset/8);

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(2048, 64, i, 6144, 1);
	connectib_flow_list_unpack(&(ptr_struct->destination[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_get_fte_out_print(const struct connectib_icmd_get_fte_out *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fte_out ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "action               : " UH_FMT "\n", ptr_struct->action);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "group_id             : " U32H_FMT "\n", ptr_struct->group_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_tag             : " UH_FMT "\n", ptr_struct->flow_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "destination_list_size : " UH_FMT "\n", ptr_struct->destination_list_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_counter_list_size : " UH_FMT "\n", ptr_struct->flow_counter_list_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "outer_headers:\n");
	connectib_flow_table_entry_match_set_lyr_2_4_print(&(ptr_struct->outer_headers), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "misc_parameters:\n");
	connectib_flow_table_entry_match_set_misc_print(&(ptr_struct->misc_parameters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner_headers:\n");
	connectib_flow_table_entry_match_set_lyr_2_4_print(&(ptr_struct->inner_headers), file, indent_level + 1);

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "destination_%03d:\n", i);
	connectib_flow_list_print(&(ptr_struct->destination[i]), file, indent_level + 1);
	}

}

int connectib_icmd_get_fte_out_size(void)
{
	 return 768;
}

void connectib_icmd_get_fte_out_dump(const struct connectib_icmd_get_fte_out *ptr_struct, FILE* file)
{
	connectib_icmd_get_fte_out_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fte_in_pack(const struct connectib_icmd_get_fte_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->flow_index);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->requested_fte_partition);

}

void connectib_icmd_get_fte_in_unpack(struct connectib_icmd_get_fte_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->flow_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 112;
	ptr_struct->requested_fte_partition = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_get_fte_in_print(const struct connectib_icmd_get_fte_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fte_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_index           : " U32H_FMT "\n", ptr_struct->flow_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "requested_fte_partition : " UH_FMT "\n", ptr_struct->requested_fte_partition);

}

int connectib_icmd_get_fte_in_size(void)
{
	 return 16;
}

void connectib_icmd_get_fte_in_dump(const struct connectib_icmd_get_fte_in *ptr_struct, FILE* file)
{
	connectib_icmd_get_fte_in_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fte_list_out_pack(const struct connectib_icmd_get_fte_list_out *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->last);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->first);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->flow_index);

}

void connectib_icmd_get_fte_list_out_unpack(struct connectib_icmd_get_fte_list_out *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->last = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->first = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->flow_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_get_fte_list_out_print(const struct connectib_icmd_get_fte_list_out *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fte_list_out ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last                 : " UH_FMT "\n", ptr_struct->last);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first                : " UH_FMT "\n", ptr_struct->first);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_index           : " U32H_FMT "\n", ptr_struct->flow_index);

}

int connectib_icmd_get_fte_list_out_size(void)
{
	 return 16;
}

void connectib_icmd_get_fte_list_out_dump(const struct connectib_icmd_get_fte_list_out *ptr_struct, FILE* file)
{
	connectib_icmd_get_fte_list_out_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fte_list_in_pack(const struct connectib_icmd_get_fte_list_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->info);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->flow_index);

}

void connectib_icmd_get_fte_list_in_unpack(struct connectib_icmd_get_fte_list_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->info = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->flow_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_get_fte_list_in_print(const struct connectib_icmd_get_fte_list_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fte_list_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "info                 : " UH_FMT "\n", ptr_struct->info);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_index           : " U32H_FMT "\n", ptr_struct->flow_index);

}

int connectib_icmd_get_fte_list_in_size(void)
{
	 return 16;
}

void connectib_icmd_get_fte_list_in_dump(const struct connectib_icmd_get_fte_list_in *ptr_struct, FILE* file)
{
	connectib_icmd_get_fte_list_in_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fg_out_pack(const struct connectib_icmd_get_fg_out *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->start_flow_index);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->end_flow_index);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->match_criteria_enable);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 512;
	connectib_flow_table_entry_match_set_lyr_2_4_pack(&(ptr_struct->outer_headers), ptr_buff + offset/8);

	offset = 1024;
	connectib_flow_table_entry_match_set_misc_pack(&(ptr_struct->misc_parameters), ptr_buff + offset/8);

	offset = 1536;
	connectib_flow_table_entry_match_set_lyr_2_4_pack(&(ptr_struct->inner_headers), ptr_buff + offset/8);

}

void connectib_icmd_get_fg_out_unpack(struct connectib_icmd_get_fg_out *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->start_flow_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->end_flow_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 88;
	ptr_struct->match_criteria_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 512;
	connectib_flow_table_entry_match_set_lyr_2_4_unpack(&(ptr_struct->outer_headers), ptr_buff + offset/8);

	offset = 1024;
	connectib_flow_table_entry_match_set_misc_unpack(&(ptr_struct->misc_parameters), ptr_buff + offset/8);

	offset = 1536;
	connectib_flow_table_entry_match_set_lyr_2_4_unpack(&(ptr_struct->inner_headers), ptr_buff + offset/8);

}

void connectib_icmd_get_fg_out_print(const struct connectib_icmd_get_fg_out *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fg_out ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "start_flow_index     : " U32H_FMT "\n", ptr_struct->start_flow_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "end_flow_index       : " U32H_FMT "\n", ptr_struct->end_flow_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "match_criteria_enable : " UH_FMT "\n", ptr_struct->match_criteria_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "outer_headers:\n");
	connectib_flow_table_entry_match_set_lyr_2_4_print(&(ptr_struct->outer_headers), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "misc_parameters:\n");
	connectib_flow_table_entry_match_set_misc_print(&(ptr_struct->misc_parameters), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inner_headers:\n");
	connectib_flow_table_entry_match_set_lyr_2_4_print(&(ptr_struct->inner_headers), file, indent_level + 1);

}

int connectib_icmd_get_fg_out_size(void)
{
	 return 256;
}

void connectib_icmd_get_fg_out_dump(const struct connectib_icmd_get_fg_out *ptr_struct, FILE* file)
{
	connectib_icmd_get_fg_out_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fg_in_pack(const struct connectib_icmd_get_fg_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->group_id);

}

void connectib_icmd_get_fg_in_unpack(struct connectib_icmd_get_fg_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->group_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_get_fg_in_print(const struct connectib_icmd_get_fg_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fg_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "group_id             : " U32H_FMT "\n", ptr_struct->group_id);

}

int connectib_icmd_get_fg_in_size(void)
{
	 return 16;
}

void connectib_icmd_get_fg_in_dump(const struct connectib_icmd_get_fg_in *ptr_struct, FILE* file)
{
	connectib_icmd_get_fg_in_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fg_list_out_pack(const struct connectib_icmd_get_fg_list_out *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->last);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->first);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->group_id);

}

void connectib_icmd_get_fg_list_out_unpack(struct connectib_icmd_get_fg_list_out *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 3;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->last = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->first = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->group_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_get_fg_list_out_print(const struct connectib_icmd_get_fg_list_out *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fg_list_out ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last                 : " UH_FMT "\n", ptr_struct->last);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first                : " UH_FMT "\n", ptr_struct->first);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "group_id             : " U32H_FMT "\n", ptr_struct->group_id);

}

int connectib_icmd_get_fg_list_out_size(void)
{
	 return 16;
}

void connectib_icmd_get_fg_list_out_dump(const struct connectib_icmd_get_fg_list_out *ptr_struct, FILE* file)
{
	connectib_icmd_get_fg_list_out_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fg_list_in_pack(const struct connectib_icmd_get_fg_list_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->info);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->group_id);

}

void connectib_icmd_get_fg_list_in_unpack(struct connectib_icmd_get_fg_list_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->info = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->group_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_get_fg_list_in_print(const struct connectib_icmd_get_fg_list_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fg_list_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "info                 : " UH_FMT "\n", ptr_struct->info);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "group_id             : " U32H_FMT "\n", ptr_struct->group_id);

}

int connectib_icmd_get_fg_list_in_size(void)
{
	 return 16;
}

void connectib_icmd_get_fg_list_in_dump(const struct connectib_icmd_get_fg_list_in *ptr_struct, FILE* file)
{
	connectib_icmd_get_fg_list_in_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ft_info_out_pack(const struct connectib_icmd_get_ft_info_out *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->log_size);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->level);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->table_type);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_miss_id);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->table_miss_mode);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->encap_en);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->decap_en);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->lag_master_next_table_id);

}

void connectib_icmd_get_ft_info_out_unpack(struct connectib_icmd_get_ft_info_out *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->log_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->level = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->table_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 40;
	ptr_struct->table_miss_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 36;
	ptr_struct->table_miss_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 33;
	ptr_struct->encap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->decap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 72;
	ptr_struct->lag_master_next_table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_icmd_get_ft_info_out_print(const struct connectib_icmd_get_ft_info_out *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ft_info_out ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_size             : " UH_FMT "\n", ptr_struct->log_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "level                : " UH_FMT "\n", ptr_struct->level);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_type           : " UH_FMT "\n", ptr_struct->table_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_miss_id        : " UH_FMT "\n", ptr_struct->table_miss_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_miss_mode      : " UH_FMT "\n", ptr_struct->table_miss_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "encap_en             : " UH_FMT "\n", ptr_struct->encap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "decap_en             : " UH_FMT "\n", ptr_struct->decap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lag_master_next_table_id : " UH_FMT "\n", ptr_struct->lag_master_next_table_id);

}

int connectib_icmd_get_ft_info_out_size(void)
{
	 return 16;
}

void connectib_icmd_get_ft_info_out_dump(const struct connectib_icmd_get_ft_info_out *ptr_struct, FILE* file)
{
	connectib_icmd_get_ft_info_out_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ft_info_in_pack(const struct connectib_icmd_get_ft_info_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

}

void connectib_icmd_get_ft_info_in_unpack(struct connectib_icmd_get_ft_info_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_get_ft_info_in_print(const struct connectib_icmd_get_ft_info_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ft_info_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

}

int connectib_icmd_get_ft_info_in_size(void)
{
	 return 16;
}

void connectib_icmd_get_ft_info_in_dump(const struct connectib_icmd_get_ft_info_in *ptr_struct, FILE* file)
{
	connectib_icmd_get_ft_info_in_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ft_list_out_pack(const struct connectib_icmd_get_ft_list_out *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->last);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->first);

}

void connectib_icmd_get_ft_list_out_unpack(struct connectib_icmd_get_ft_list_out *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 3;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->last = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->first = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_get_ft_list_out_print(const struct connectib_icmd_get_ft_list_out *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ft_list_out ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last                 : " UH_FMT "\n", ptr_struct->last);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first                : " UH_FMT "\n", ptr_struct->first);

}

int connectib_icmd_get_ft_list_out_size(void)
{
	 return 16;
}

void connectib_icmd_get_ft_list_out_dump(const struct connectib_icmd_get_ft_list_out *ptr_struct, FILE* file)
{
	connectib_icmd_get_ft_list_out_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ft_list_in_pack(const struct connectib_icmd_get_ft_list_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->table_id);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->info);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

}

void connectib_icmd_get_ft_list_in_unpack(struct connectib_icmd_get_ft_list_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->info = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_get_ft_list_in_print(const struct connectib_icmd_get_ft_list_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ft_list_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " UH_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "info                 : " UH_FMT "\n", ptr_struct->info);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

}

int connectib_icmd_get_ft_list_in_size(void)
{
	 return 16;
}

void connectib_icmd_get_ft_list_in_dump(const struct connectib_icmd_get_ft_list_in *ptr_struct, FILE* file)
{
	connectib_icmd_get_ft_list_in_print(ptr_struct, file, 0);
}

void connectib_led_control_data_pack(const struct connectib_led_control_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_led_pair_bank[i]);
	}

}

void connectib_led_control_data_unpack(struct connectib_led_control_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	ptr_struct->port_led_pair_bank[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_led_control_data_print(const struct connectib_led_control_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_led_control_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_led_pair_bank_%03d : " U32H_FMT "\n", i, ptr_struct->port_led_pair_bank[i]);
	}

}

int connectib_led_control_data_size(void)
{
	 return 64;
}

void connectib_led_control_data_dump(const struct connectib_led_control_data *ptr_struct, FILE* file)
{
	connectib_led_control_data_print(ptr_struct, file, 0);
}

void connectib_steering_table_info_pack(const struct connectib_steering_table_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->base_index);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->default_ste_ix);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->entry_count);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->descendant_count);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ste_lu_type);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->log2_table_size);

	offset = 108;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log2_entry_size);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sent_to_gc);

	offset = 103;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pipe);

	offset = 100;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->resize_state);

	offset = 98;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->match_mode);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_rx_);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hash_fieldwise_mask);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->table_id);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->group_id);

	offset = 336;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->bucket_count);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->new_hash_meta_ix);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->next_bucket_to_copy);

	offset = 432;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->needed_pages);

	offset = 416;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->allocated_pages);

}

void connectib_steering_table_info_unpack(struct connectib_steering_table_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->base_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->default_ste_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 80;
	ptr_struct->entry_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->descendant_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 120;
	ptr_struct->ste_lu_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 112;
	ptr_struct->log2_table_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 108;
	ptr_struct->log2_entry_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 104;
	ptr_struct->sent_to_gc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 103;
	ptr_struct->pipe = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 100;
	ptr_struct->resize_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 98;
	ptr_struct->match_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 96;
	ptr_struct->sx_rx_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 152;
	ptr_struct->hash_fieldwise_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 256;
	ptr_struct->table_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->group_id = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 336;
	ptr_struct->bucket_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 352;
	ptr_struct->new_hash_meta_ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->next_bucket_to_copy = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 432;
	ptr_struct->needed_pages = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 416;
	ptr_struct->allocated_pages = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_steering_table_info_print(const struct connectib_steering_table_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_steering_table_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "base_index           : " U32H_FMT "\n", ptr_struct->base_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "default_ste_ix       : " U32H_FMT "\n", ptr_struct->default_ste_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "entry_count          : " UH_FMT "\n", ptr_struct->entry_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "descendant_count     : " UH_FMT "\n", ptr_struct->descendant_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ste_lu_type          : " UH_FMT "\n", ptr_struct->ste_lu_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_table_size      : " UH_FMT "\n", ptr_struct->log2_table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_entry_size      : " UH_FMT "\n", ptr_struct->log2_entry_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sent_to_gc           : " UH_FMT "\n", ptr_struct->sent_to_gc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pipe                 : " UH_FMT "\n", ptr_struct->pipe);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "resize_state         : " UH_FMT "\n", ptr_struct->resize_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "match_mode           : " UH_FMT "\n", ptr_struct->match_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_rx_               : " UH_FMT "\n", ptr_struct->sx_rx_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hash_fieldwise_mask  : " UH_FMT "\n", ptr_struct->hash_fieldwise_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_id             : " U32H_FMT "\n", ptr_struct->table_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "group_id             : " U32H_FMT "\n", ptr_struct->group_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bucket_count         : " UH_FMT "\n", ptr_struct->bucket_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "new_hash_meta_ix     : " U32H_FMT "\n", ptr_struct->new_hash_meta_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_bucket_to_copy  : " U32H_FMT "\n", ptr_struct->next_bucket_to_copy);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "needed_pages         : " UH_FMT "\n", ptr_struct->needed_pages);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allocated_pages      : " UH_FMT "\n", ptr_struct->allocated_pages);

}

int connectib_steering_table_info_size(void)
{
	 return 64;
}

void connectib_steering_table_info_dump(const struct connectib_steering_table_info *ptr_struct, FILE* file)
{
	connectib_steering_table_info_print(ptr_struct, file, 0);
}

void connectib_dcr_info_pack(const struct connectib_dcr_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->next_dcri);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->valid);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->connected);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->trapped);

}

void connectib_dcr_info_unpack(struct connectib_dcr_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->next_dcri = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 7;
	ptr_struct->valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->connected = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->trapped = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_dcr_info_print(const struct connectib_dcr_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dcr_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_dcri            : " UH_FMT "\n", ptr_struct->next_dcri);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid                : " UH_FMT "\n", ptr_struct->valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "connected            : " UH_FMT "\n", ptr_struct->connected);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trapped              : " UH_FMT "\n", ptr_struct->trapped);

}

int connectib_dcr_info_size(void)
{
	 return 4;
}

void connectib_dcr_info_dump(const struct connectib_dcr_info *ptr_struct, FILE* file)
{
	connectib_dcr_info_print(ptr_struct, file, 0);
}

void connectib_icmd_ver_data_cmd_specific_pack(const union connectib_icmd_ver_data_cmd_specific *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_ver_data_one_qp_per_recovery_pack(&(ptr_struct->one_qp_per_recovery), ptr_buff);
}

void connectib_icmd_ver_data_cmd_specific_unpack(union connectib_icmd_ver_data_cmd_specific *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_ver_data_one_qp_per_recovery_unpack(&(ptr_struct->one_qp_per_recovery), ptr_buff);
}

void connectib_icmd_ver_data_cmd_specific_print(const union connectib_icmd_ver_data_cmd_specific *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_ver_data_cmd_specific ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_atomic:\n");
	connectib_icmd_ver_data_max_atomic_print(&(ptr_struct->max_atomic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "one_qp_per_recovery:\n");
	connectib_icmd_ver_data_one_qp_per_recovery_print(&(ptr_struct->one_qp_per_recovery), file, indent_level + 1);

}

int connectib_icmd_ver_data_cmd_specific_size(void)
{
	 return 64;
}

void connectib_icmd_ver_data_cmd_specific_dump(const union connectib_icmd_ver_data_cmd_specific *ptr_struct, FILE* file)
{
	connectib_icmd_ver_data_cmd_specific_print(ptr_struct, file, 0);
}

void connectib_icmd_pcie_ver_frequency_pack(const struct connectib_icmd_pcie_ver_frequency *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->core_f);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->core_clocks_per_usec);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->core_r);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cx3_count_in_core_clocks);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->usecs_in_high_tick);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vdd_mod_mapping);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->flash_div);

	offset = 10;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->core_od);

}

void connectib_icmd_pcie_ver_frequency_unpack(struct connectib_icmd_pcie_ver_frequency *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 25;
	ptr_struct->core_f = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 24;
	ptr_struct->core_clocks_per_usec = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->core_r = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 18;
	ptr_struct->cx3_count_in_core_clocks = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->usecs_in_high_tick = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->vdd_mod_mapping = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->flash_div = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 10;
	ptr_struct->core_od = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_icmd_pcie_ver_frequency_print(const struct connectib_icmd_pcie_ver_frequency *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_pcie_ver_frequency ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_f               : " UH_FMT "\n", ptr_struct->core_f);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_clocks_per_usec : " UH_FMT "\n", ptr_struct->core_clocks_per_usec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_r               : " UH_FMT "\n", ptr_struct->core_r);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cx3_count_in_core_clocks : " UH_FMT "\n", ptr_struct->cx3_count_in_core_clocks);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "usecs_in_high_tick   : " UH_FMT "\n", ptr_struct->usecs_in_high_tick);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vdd_mod_mapping      : " UH_FMT "\n", ptr_struct->vdd_mod_mapping);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_div            : " UH_FMT "\n", ptr_struct->flash_div);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_od              : " UH_FMT "\n", ptr_struct->core_od);

}

int connectib_icmd_pcie_ver_frequency_size(void)
{
	 return 16;
}

void connectib_icmd_pcie_ver_frequency_dump(const struct connectib_icmd_pcie_ver_frequency *ptr_struct, FILE* file)
{
	connectib_icmd_pcie_ver_frequency_print(ptr_struct, file, 0);
}

void connectib_icmd_pcie_ver_cables_pack(const struct connectib_icmd_pcie_ver_cables *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->identifier);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->extended_identifier);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->device_tech);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->attenuation_5g);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->vendor_oui);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->m_fsm_state);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mlnx_cable);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->eeprom_present);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->link_length_supported);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->eth_com_codes);

	offset = 71;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->tech);

	offset = 121;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->ib_10ge_com_codes);

}

void connectib_icmd_pcie_ver_cables_unpack(struct connectib_icmd_pcie_ver_cables *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->identifier = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->extended_identifier = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->device_tech = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->attenuation_5g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 40;
	ptr_struct->vendor_oui = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 34;
	ptr_struct->m_fsm_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 33;
	ptr_struct->mlnx_cable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->eeprom_present = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 88;
	ptr_struct->link_length_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 80;
	ptr_struct->eth_com_codes = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 71;
	ptr_struct->tech = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 121;
	ptr_struct->ib_10ge_com_codes = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

}

void connectib_icmd_pcie_ver_cables_print(const struct connectib_icmd_pcie_ver_cables *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_pcie_ver_cables ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "identifier           : " UH_FMT "\n", ptr_struct->identifier);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_identifier  : " UH_FMT "\n", ptr_struct->extended_identifier);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_tech          : " UH_FMT "\n", ptr_struct->device_tech);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "attenuation_5g       : " UH_FMT "\n", ptr_struct->attenuation_5g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vendor_oui           : " UH_FMT "\n", ptr_struct->vendor_oui);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "m_fsm_state          : " UH_FMT "\n", ptr_struct->m_fsm_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlnx_cable           : " UH_FMT "\n", ptr_struct->mlnx_cable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eeprom_present       : " UH_FMT "\n", ptr_struct->eeprom_present);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "link_length_supported : " UH_FMT "\n", ptr_struct->link_length_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_com_codes        : " UH_FMT "\n", ptr_struct->eth_com_codes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tech                 : " UH_FMT "\n", ptr_struct->tech);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_10ge_com_codes    : " UH_FMT "\n", ptr_struct->ib_10ge_com_codes);

}

int connectib_icmd_pcie_ver_cables_size(void)
{
	 return 16;
}

void connectib_icmd_pcie_ver_cables_dump(const struct connectib_icmd_pcie_ver_cables *ptr_struct, FILE* file)
{
	connectib_icmd_pcie_ver_cables_print(ptr_struct, file, 0);
}

void connectib_icmd_pcie_ver_device_pack(const struct connectib_icmd_pcie_ver_device *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->devid);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->revid);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->strap_eprom_present);

}

void connectib_icmd_pcie_ver_device_unpack(struct connectib_icmd_pcie_ver_device *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->devid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 12;
	ptr_struct->revid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 11;
	ptr_struct->strap_eprom_present = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_pcie_ver_device_print(const struct connectib_icmd_pcie_ver_device *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_pcie_ver_device ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "devid                : " UH_FMT "\n", ptr_struct->devid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "revid                : " UH_FMT "\n", ptr_struct->revid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "strap_eprom_present  : " UH_FMT "\n", ptr_struct->strap_eprom_present);

}

int connectib_icmd_pcie_ver_device_size(void)
{
	 return 16;
}

void connectib_icmd_pcie_ver_device_dump(const struct connectib_icmd_pcie_ver_device *ptr_struct, FILE* file)
{
	connectib_icmd_pcie_ver_device_print(ptr_struct, file, 0);
}

void connectib_code_coverage_control_pack(const struct connectib_code_coverage_control *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->window);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_window_size);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->irisc_bitmask);

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->bp);

	offset = 38;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->all_ips);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->trace_mode);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable);

}

void connectib_code_coverage_control_unpack(struct connectib_code_coverage_control *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->window = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 3;
	ptr_struct->log_window_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 48;
	ptr_struct->irisc_bitmask = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 39;
	ptr_struct->bp = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 38;
	ptr_struct->all_ips = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 36;
	ptr_struct->trace_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 32;
	ptr_struct->enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_code_coverage_control_print(const struct connectib_code_coverage_control *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_code_coverage_control ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "window               : " UH_FMT "\n", ptr_struct->window);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_window_size      : " UH_FMT "\n", ptr_struct->log_window_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "irisc_bitmask        : " UH_FMT "\n", ptr_struct->irisc_bitmask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bp                   : " UH_FMT "\n", ptr_struct->bp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "all_ips              : " UH_FMT "\n", ptr_struct->all_ips);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trace_mode           : " UH_FMT "\n", ptr_struct->trace_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable               : " UH_FMT "\n", ptr_struct->enable);

}

int connectib_code_coverage_control_size(void)
{
	 return 8;
}

void connectib_code_coverage_control_dump(const struct connectib_code_coverage_control *ptr_struct, FILE* file)
{
	connectib_code_coverage_control_print(ptr_struct, file, 0);
}

void connectib_vport_real_state_pack(const struct connectib_vport_real_state *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 256; i++) {
	offset = adb2c_calc_array_field_address(28, 4, i, 1024, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->real_state[i]);
	}

}

void connectib_vport_real_state_unpack(struct connectib_vport_real_state *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 256; i++) {
	offset = adb2c_calc_array_field_address(28, 4, i, 1024, 1);
	ptr_struct->real_state[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_vport_real_state_print(const struct connectib_vport_real_state *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_vport_real_state ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 256; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "real_state_%03d      : " UH_FMT "\n", i, ptr_struct->real_state[i]);
	}

}

int connectib_vport_real_state_size(void)
{
	 return 128;
}

void connectib_vport_real_state_dump(const struct connectib_vport_real_state *ptr_struct, FILE* file)
{
	connectib_vport_real_state_print(ptr_struct, file, 0);
}

void connectib_fw_esw_pack(const struct connectib_fw_esw *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 256; i++) {
	offset = adb2c_calc_array_field_address(16, 16, i, 4096, 1);
	connectib_fw_vport_ctx_pack(&(ptr_struct->vport_ctx[i]), ptr_buff + offset/8);
	}

}

void connectib_fw_esw_unpack(struct connectib_fw_esw *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 256; i++) {
	offset = adb2c_calc_array_field_address(16, 16, i, 4096, 1);
	connectib_fw_vport_ctx_unpack(&(ptr_struct->vport_ctx[i]), ptr_buff + offset/8);
	}

}

void connectib_fw_esw_print(const struct connectib_fw_esw *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_esw ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 256; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_ctx_%03d:\n", i);
	connectib_fw_vport_ctx_print(&(ptr_struct->vport_ctx[i]), file, indent_level + 1);
	}

}

int connectib_fw_esw_size(void)
{
	 return 512;
}

void connectib_fw_esw_dump(const struct connectib_fw_esw *ptr_struct, FILE* file)
{
	connectib_fw_esw_print(ptr_struct, file, 0);
}

void connectib_gvmix_sub_structs_pack(const union connectib_gvmix_sub_structs *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_vf_pages_list_pack(&(ptr_struct->vf_pages_list), ptr_buff);
}

void connectib_gvmix_sub_structs_unpack(union connectib_gvmix_sub_structs *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_vf_pages_list_unpack(&(ptr_struct->vf_pages_list), ptr_buff);
}

void connectib_gvmix_sub_structs_print(const union connectib_gvmix_sub_structs *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_gvmix_sub_structs ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmdif_ctx:\n");
	connectib_cmdif_ctx_print(&(ptr_struct->cmdif_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dummy_addresses:\n");
	connectib_dummy_addresses_one_cache_line_print(&(ptr_struct->dummy_addresses), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hca_params:\n");
	connectib_hca_params_print(&(ptr_struct->hca_params), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icm_page_manager:\n");
	connectib_icm_page_manager_print(&(ptr_struct->icm_page_manager), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_pages_list:\n");
	connectib_vf_pages_list_print(&(ptr_struct->vf_pages_list), file, indent_level + 1);

}

int connectib_gvmix_sub_structs_size(void)
{
	 return 64;
}

void connectib_gvmix_sub_structs_dump(const union connectib_gvmix_sub_structs *ptr_struct, FILE* file)
{
	connectib_gvmix_sub_structs_print(ptr_struct, file, 0);
}

void connectib_sw_eqe_pack(const struct connectib_sw_eqe *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->event_sub_type);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->event_type);

	offset = 256;
	connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto_pack(&(ptr_struct->event_data), ptr_buff + offset/8);

	offset = 511;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->owner);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->signature);

}

void connectib_sw_eqe_unpack(struct connectib_sw_eqe *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->event_sub_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->event_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 256;
	connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto_unpack(&(ptr_struct->event_data), ptr_buff + offset/8);

	offset = 511;
	ptr_struct->owner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 496;
	ptr_struct->signature = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_sw_eqe_print(const struct connectib_sw_eqe *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_eqe ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_sub_type       : " UH_FMT "\n", ptr_struct->event_sub_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_type           : " UH_FMT "\n", ptr_struct->event_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_data:\n");
	connectib_comp_event_dct_events_qp_events_wqe_associated_page_fault_event_rdma_page_fault_event_cq_error_dropped_packet_logged_port_state_change_event_gpio_event_port_module_event_dcbx_change_event_temp_warning_event_db_bf_congestion_event_stall_vl_event_cmd_inter_comp_event_pages_req_event_nic_vport_change_event_fpga_error_event_fpga_qp_error_event_general_notification_delay_drop_event_auto_print(&(ptr_struct->event_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "owner                : " UH_FMT "\n", ptr_struct->owner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature            : " UH_FMT "\n", ptr_struct->signature);

}

int connectib_sw_eqe_size(void)
{
	 return 64;
}

void connectib_sw_eqe_dump(const struct connectib_sw_eqe *ptr_struct, FILE* file)
{
	connectib_sw_eqe_print(ptr_struct, file, 0);
}

void connectib_lock_tag_pack(const struct connectib_lock_tag *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->resource_id);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->resource_type);

}

void connectib_lock_tag_unpack(struct connectib_lock_tag *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->resource_id = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->resource_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_lock_tag_print(const struct connectib_lock_tag *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_lock_tag ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "resource_id          : " UH_FMT "\n", ptr_struct->resource_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "resource_type        : " UH_FMT "\n", ptr_struct->resource_type);

}

int connectib_lock_tag_size(void)
{
	 return 4;
}

void connectib_lock_tag_dump(const struct connectib_lock_tag *ptr_struct, FILE* file)
{
	connectib_lock_tag_print(ptr_struct, file, 0);
}

void connectib_icmd_generic_reg_access_reg_tlv_pack(const struct connectib_icmd_generic_reg_access_reg_tlv *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reg_access_tlv_pack(&(ptr_struct->hdr), ptr_buff + offset/8);

	offset = 32;
	connectib_icmd_generic_reg_access_reg_tlv_data_pack(&(ptr_struct->reg_tlv_data), ptr_buff + offset/8);

}

void connectib_icmd_generic_reg_access_reg_tlv_unpack(struct connectib_icmd_generic_reg_access_reg_tlv *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reg_access_tlv_unpack(&(ptr_struct->hdr), ptr_buff + offset/8);

	offset = 32;
	connectib_icmd_generic_reg_access_reg_tlv_data_unpack(&(ptr_struct->reg_tlv_data), ptr_buff + offset/8);

}

void connectib_icmd_generic_reg_access_reg_tlv_print(const struct connectib_icmd_generic_reg_access_reg_tlv *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_generic_reg_access_reg_tlv ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hdr:\n");
	connectib_reg_access_tlv_print(&(ptr_struct->hdr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reg_tlv_data:\n");
	connectib_icmd_generic_reg_access_reg_tlv_data_print(&(ptr_struct->reg_tlv_data), file, indent_level + 1);

}

int connectib_icmd_generic_reg_access_reg_tlv_size(void)
{
	 return 276;
}

void connectib_icmd_generic_reg_access_reg_tlv_dump(const struct connectib_icmd_generic_reg_access_reg_tlv *ptr_struct, FILE* file)
{
	connectib_icmd_generic_reg_access_reg_tlv_print(ptr_struct, file, 0);
}

void connectib_reg_access_oper_tlv_pack(const struct connectib_reg_access_oper_tlv *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->Status);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dr);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->len);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->type);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->class_);

	offset = 49;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->Method);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->r);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->register_id);

	offset = 64;
	connectib_IB_TID_pack(&(ptr_struct->TID), ptr_buff + offset/8);

}

void connectib_reg_access_oper_tlv_unpack(struct connectib_reg_access_oper_tlv *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 17;
	ptr_struct->Status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 16;
	ptr_struct->dr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->len = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

	offset = 0;
	ptr_struct->type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 56;
	ptr_struct->class_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 49;
	ptr_struct->Method = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 48;
	ptr_struct->r = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->register_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	connectib_IB_TID_unpack(&(ptr_struct->TID), ptr_buff + offset/8);

}

void connectib_reg_access_oper_tlv_print(const struct connectib_reg_access_oper_tlv *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_reg_access_oper_tlv ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Status               : " UH_FMT "\n", ptr_struct->Status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dr                   : " UH_FMT "\n", ptr_struct->dr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "len                  : " UH_FMT "\n", ptr_struct->len);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " UH_FMT "\n", ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "class_               : " UH_FMT "\n", ptr_struct->class_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Method               : " UH_FMT "\n", ptr_struct->Method);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "r                    : " UH_FMT "\n", ptr_struct->r);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "register_id          : " UH_FMT "\n", ptr_struct->register_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "TID:\n");
	connectib_IB_TID_print(&(ptr_struct->TID), file, indent_level + 1);

}

int connectib_reg_access_oper_tlv_size(void)
{
	 return 16;
}

void connectib_reg_access_oper_tlv_dump(const struct connectib_reg_access_oper_tlv *ptr_struct, FILE* file)
{
	connectib_reg_access_oper_tlv_print(ptr_struct, file, 0);
}

void connectib_icmd_kdnet_ctrl_io_pack(const union connectib_icmd_kdnet_ctrl_io *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_kdnet_ctrl_out_pack(&(ptr_struct->ctrl_out), ptr_buff);
}

void connectib_icmd_kdnet_ctrl_io_unpack(union connectib_icmd_kdnet_ctrl_io *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_kdnet_ctrl_out_unpack(&(ptr_struct->ctrl_out), ptr_buff);
}

void connectib_icmd_kdnet_ctrl_io_print(const union connectib_icmd_kdnet_ctrl_io *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_kdnet_ctrl_io ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctrl_in:\n");
	connectib_icmd_kdnet_ctrl_in_print(&(ptr_struct->ctrl_in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctrl_out:\n");
	connectib_icmd_kdnet_ctrl_out_print(&(ptr_struct->ctrl_out), file, indent_level + 1);

}

int connectib_icmd_kdnet_ctrl_io_size(void)
{
	 return 4;
}

void connectib_icmd_kdnet_ctrl_io_dump(const union connectib_icmd_kdnet_ctrl_io *ptr_struct, FILE* file)
{
	connectib_icmd_kdnet_ctrl_io_print(ptr_struct, file, 0);
}

void connectib_icmd_query_cap_general_pack(const struct connectib_icmd_query_cap_general *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fw_ctrl_update_icmd);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->kdnet_ctrl);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mh_sync);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->allow_icmd_access_reg_on_all_registers);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fw_info_psid);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->nv_access);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wol_p);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wol_u);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wol_m);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wol_b);

	offset = 59;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wol_a);

	offset = 58;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wol_g);

	offset = 57;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wol_s);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rol_g);

	offset = 54;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rol_s);

	offset = 53;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fpga);

}

void connectib_icmd_query_cap_general_unpack(struct connectib_icmd_query_cap_general *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 5;
	ptr_struct->fw_ctrl_update_icmd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->kdnet_ctrl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->mh_sync = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->allow_icmd_access_reg_on_all_registers = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->fw_info_psid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->nv_access = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 63;
	ptr_struct->wol_p = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 62;
	ptr_struct->wol_u = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 61;
	ptr_struct->wol_m = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 60;
	ptr_struct->wol_b = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 59;
	ptr_struct->wol_a = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 58;
	ptr_struct->wol_g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 57;
	ptr_struct->wol_s = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 55;
	ptr_struct->rol_g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 54;
	ptr_struct->rol_s = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 53;
	ptr_struct->fpga = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_query_cap_general_print(const struct connectib_icmd_query_cap_general *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_query_cap_general ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ctrl_update_icmd  : " UH_FMT "\n", ptr_struct->fw_ctrl_update_icmd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "kdnet_ctrl           : " UH_FMT "\n", ptr_struct->kdnet_ctrl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mh_sync              : " UH_FMT "\n", ptr_struct->mh_sync);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allow_icmd_access_reg_on_all_registers : " UH_FMT "\n", ptr_struct->allow_icmd_access_reg_on_all_registers);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_info_psid         : " UH_FMT "\n", ptr_struct->fw_info_psid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nv_access            : " UH_FMT "\n", ptr_struct->nv_access);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wol_p                : " UH_FMT "\n", ptr_struct->wol_p);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wol_u                : " UH_FMT "\n", ptr_struct->wol_u);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wol_m                : " UH_FMT "\n", ptr_struct->wol_m);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wol_b                : " UH_FMT "\n", ptr_struct->wol_b);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wol_a                : " UH_FMT "\n", ptr_struct->wol_a);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wol_g                : " UH_FMT "\n", ptr_struct->wol_g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wol_s                : " UH_FMT "\n", ptr_struct->wol_s);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rol_g                : " UH_FMT "\n", ptr_struct->rol_g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rol_s                : " UH_FMT "\n", ptr_struct->rol_s);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpga                 : " UH_FMT "\n", ptr_struct->fpga);

}

int connectib_icmd_query_cap_general_size(void)
{
	 return 8;
}

void connectib_icmd_query_cap_general_dump(const struct connectib_icmd_query_cap_general *ptr_struct, FILE* file)
{
	connectib_icmd_query_cap_general_print(ptr_struct, file, 0);
}

void connectib_icmd_query_cap_in_pack(const struct connectib_icmd_query_cap_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->capability_group);

}

void connectib_icmd_query_cap_in_unpack(struct connectib_icmd_query_cap_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->capability_group = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_query_cap_in_print(const struct connectib_icmd_query_cap_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_query_cap_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capability_group     : " UH_FMT "\n", ptr_struct->capability_group);

}

int connectib_icmd_query_cap_in_size(void)
{
	 return 4;
}

void connectib_icmd_query_cap_in_dump(const struct connectib_icmd_query_cap_in *ptr_struct, FILE* file)
{
	connectib_icmd_query_cap_in_print(ptr_struct, file, 0);
}

void connectib_cable_info_pack(const struct connectib_cable_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->address);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->page_number);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->device_address);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->size);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->passwd_clr);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->passwd_v);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->password);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword[i]);
	}

}

void connectib_cable_info_unpack(struct connectib_cable_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->address = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 8;
	ptr_struct->page_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->device_address = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 33;
	ptr_struct->passwd_clr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->passwd_v = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->password = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 512, 1);
	ptr_struct->dword[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_cable_info_print(const struct connectib_cable_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cable_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address              : " UH_FMT "\n", ptr_struct->address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "page_number          : " UH_FMT "\n", ptr_struct->page_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_address       : " UH_FMT "\n", ptr_struct->device_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "passwd_clr           : " UH_FMT "\n", ptr_struct->passwd_clr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "passwd_v             : " UH_FMT "\n", ptr_struct->passwd_v);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "password             : " U32H_FMT "\n", ptr_struct->password);

	for (i = 0; i < 12; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_%03d           : " U32H_FMT "\n", i, ptr_struct->dword[i]);
	}

}

int connectib_cable_info_size(void)
{
	 return 64;
}

void connectib_cable_info_dump(const struct connectib_cable_info *ptr_struct, FILE* file)
{
	connectib_cable_info_print(ptr_struct, file, 0);
}

void connectib_grepper_dwords_pack(const struct connectib_grepper_dwords *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dwords[i]);
	}

}

void connectib_grepper_dwords_unpack(struct connectib_grepper_dwords *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	ptr_struct->dwords[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_grepper_dwords_print(const struct connectib_grepper_dwords *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_grepper_dwords ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dwords_%03d          : " U32H_FMT "\n", i, ptr_struct->dwords[i]);
	}

}

int connectib_grepper_dwords_size(void)
{
	 return 64;
}

void connectib_grepper_dwords_dump(const struct connectib_grepper_dwords *ptr_struct, FILE* file)
{
	connectib_grepper_dwords_print(ptr_struct, file, 0);
}

void connectib_itrace_pack(const struct connectib_itrace *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->unit_mask);

	offset = 32;
	connectib_itrace_ctrl_pack(&(ptr_struct->ctrl), ptr_buff + offset/8);

}

void connectib_itrace_unpack(struct connectib_itrace *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->unit_mask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	connectib_itrace_ctrl_unpack(&(ptr_struct->ctrl), ptr_buff + offset/8);

}

void connectib_itrace_print(const struct connectib_itrace *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_itrace ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "unit_mask            : " U32H_FMT "\n", ptr_struct->unit_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctrl:\n");
	connectib_itrace_ctrl_print(&(ptr_struct->ctrl), file, indent_level + 1);

}

int connectib_itrace_size(void)
{
	 return 16;
}

void connectib_itrace_dump(const struct connectib_itrace *ptr_struct, FILE* file)
{
	connectib_itrace_print(ptr_struct, file, 0);
}

void connectib_cc_table_entry_pack(const struct connectib_cc_table_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 14, (u_int32_t)ptr_struct->cct_multiplier);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->cct_shift);

}

void connectib_cc_table_entry_unpack(struct connectib_cc_table_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 2;
	ptr_struct->cct_multiplier = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 14);

	offset = 0;
	ptr_struct->cct_shift = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_cc_table_entry_print(const struct connectib_cc_table_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cc_table_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cct_multiplier       : " UH_FMT "\n", ptr_struct->cct_multiplier);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cct_shift            : " UH_FMT "\n", ptr_struct->cct_shift);

}

int connectib_cc_table_entry_size(void)
{
	 return 2;
}

void connectib_cc_table_entry_dump(const struct connectib_cc_table_entry *ptr_struct, FILE* file)
{
	connectib_cc_table_entry_print(ptr_struct, file, 0);
}

void connectib_ca_congestion_entry_pack(const struct connectib_ca_congestion_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->trigger_threshold);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ccti_increase);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ccti_timer);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->ccti_min);

}

void connectib_ca_congestion_entry_unpack(struct connectib_ca_congestion_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->trigger_threshold = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->ccti_increase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->ccti_timer = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->ccti_min = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_ca_congestion_entry_print(const struct connectib_ca_congestion_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ca_congestion_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trigger_threshold    : " UH_FMT "\n", ptr_struct->trigger_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ccti_increase        : " UH_FMT "\n", ptr_struct->ccti_increase);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ccti_timer           : " UH_FMT "\n", ptr_struct->ccti_timer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ccti_min             : " UH_FMT "\n", ptr_struct->ccti_min);

}

int connectib_ca_congestion_entry_size(void)
{
	 return 8;
}

void connectib_ca_congestion_entry_dump(const struct connectib_ca_congestion_entry *ptr_struct, FILE* file)
{
	connectib_ca_congestion_entry_print(ptr_struct, file, 0);
}

void connectib_congestion_log_event_entry_pack(const struct connectib_congestion_log_event_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->service_type);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->local_qpn);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->remote_qpn);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->remote_lid);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->timestamp);

}

void connectib_congestion_log_event_entry_unpack(struct connectib_congestion_log_event_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->service_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 24;
	ptr_struct->sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->local_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 32;
	ptr_struct->remote_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 80;
	ptr_struct->remote_lid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->timestamp = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_congestion_log_event_entry_print(const struct connectib_congestion_log_event_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_congestion_log_event_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "service_type         : " UH_FMT "\n", ptr_struct->service_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl                   : " UH_FMT "\n", ptr_struct->sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_qpn            : " UH_FMT "\n", ptr_struct->local_qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "remote_qpn           : " UH_FMT "\n", ptr_struct->remote_qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "remote_lid           : " UH_FMT "\n", ptr_struct->remote_lid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp            : " U32H_FMT "\n", ptr_struct->timestamp);

}

int connectib_congestion_log_event_entry_size(void)
{
	 return 16;
}

void connectib_congestion_log_event_entry_dump(const struct connectib_congestion_log_event_entry *ptr_struct, FILE* file)
{
	connectib_congestion_log_event_entry_print(ptr_struct, file, 0);
}

void connectib_special_qps_pack(const struct connectib_special_qps *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->qp0);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->qp1);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->raw);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ipv6);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->reserved);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ptp1588);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sync_umr);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cnak);

}

void connectib_special_qps_unpack(struct connectib_special_qps *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->qp0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->qp1 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->raw = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->ipv6 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->reserved = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->ptp1588 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->sync_umr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->cnak = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_special_qps_print(const struct connectib_special_qps *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_special_qps ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp0                  : " U32H_FMT "\n", ptr_struct->qp0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qp1                  : " U32H_FMT "\n", ptr_struct->qp1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw                  : " U32H_FMT "\n", ptr_struct->raw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ipv6                 : " U32H_FMT "\n", ptr_struct->ipv6);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved             : " U32H_FMT "\n", ptr_struct->reserved);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ptp1588              : " U32H_FMT "\n", ptr_struct->ptp1588);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sync_umr             : " U32H_FMT "\n", ptr_struct->sync_umr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cnak                 : " U32H_FMT "\n", ptr_struct->cnak);

}

int connectib_special_qps_size(void)
{
	 return 32;
}

void connectib_special_qps_dump(const struct connectib_special_qps *ptr_struct, FILE* file)
{
	connectib_special_qps_print(ptr_struct, file, 0);
}

void connectib_cmdif_hdr_pack(const struct connectib_cmdif_hdr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->reserved0);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->type);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->input_length);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->ibox_pointer);

	offset = 128;
	connectib_input_inline_standart_pack(&(ptr_struct->input_inline), ptr_buff + offset/8);

	offset = 256;
	connectib_output_inline_standart_pack(&(ptr_struct->output_inline), ptr_buff + offset/8);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->obox_pointer);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->output_length);

	offset = 511;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ownership);

	offset = 504;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->status);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->reserved1);

	offset = 488;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->signature);

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->token);

}

void connectib_cmdif_hdr_unpack(struct connectib_cmdif_hdr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->reserved0 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->input_length = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->ibox_pointer = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 128;
	connectib_input_inline_standart_unpack(&(ptr_struct->input_inline), ptr_buff + offset/8);

	offset = 256;
	connectib_output_inline_standart_unpack(&(ptr_struct->output_inline), ptr_buff + offset/8);

	offset = 384;
	ptr_struct->obox_pointer = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 448;
	ptr_struct->output_length = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 511;
	ptr_struct->ownership = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 504;
	ptr_struct->status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 496;
	ptr_struct->reserved1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 488;
	ptr_struct->signature = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 480;
	ptr_struct->token = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_cmdif_hdr_print(const struct connectib_cmdif_hdr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_cmdif_hdr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved0            : " UH_FMT "\n", ptr_struct->reserved0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " UH_FMT "\n", ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "input_length         : " U32H_FMT "\n", ptr_struct->input_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibox_pointer         : " U64H_FMT "\n", ptr_struct->ibox_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "input_inline:\n");
	connectib_input_inline_standart_print(&(ptr_struct->input_inline), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "output_inline:\n");
	connectib_output_inline_standart_print(&(ptr_struct->output_inline), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "obox_pointer         : " U64H_FMT "\n", ptr_struct->obox_pointer);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "output_length        : " U32H_FMT "\n", ptr_struct->output_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ownership            : " UH_FMT "\n", ptr_struct->ownership);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " UH_FMT "\n", ptr_struct->status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved1            : " UH_FMT "\n", ptr_struct->reserved1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature            : " UH_FMT "\n", ptr_struct->signature);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "token                : " UH_FMT "\n", ptr_struct->token);

}

int connectib_cmdif_hdr_size(void)
{
	 return 64;
}

void connectib_cmdif_hdr_dump(const struct connectib_cmdif_hdr *ptr_struct, FILE* file)
{
	connectib_cmdif_hdr_print(ptr_struct, file, 0);
}

void connectib_icmd_header_capture_checks_pack(const struct connectib_icmd_header_capture_checks *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->word[i]);
	}

}

void connectib_icmd_header_capture_checks_unpack(struct connectib_icmd_header_capture_checks *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	ptr_struct->word[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_icmd_header_capture_checks_print(const struct connectib_icmd_header_capture_checks *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_header_capture_checks ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "word_%03d            : " U32H_FMT "\n", i, ptr_struct->word[i]);
	}

}

int connectib_icmd_header_capture_checks_size(void)
{
	 return 64;
}

void connectib_icmd_header_capture_checks_dump(const struct connectib_icmd_header_capture_checks *ptr_struct, FILE* file)
{
	connectib_icmd_header_capture_checks_print(ptr_struct, file, 0);
}

void connectib_icmd_header_capture_headers_pack(const struct connectib_icmd_header_capture_headers *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->word[i]);
	}

}

void connectib_icmd_header_capture_headers_unpack(struct connectib_icmd_header_capture_headers *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	ptr_struct->word[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_icmd_header_capture_headers_print(const struct connectib_icmd_header_capture_headers *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_header_capture_headers ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "word_%03d            : " U32H_FMT "\n", i, ptr_struct->word[i]);
	}

}

int connectib_icmd_header_capture_headers_size(void)
{
	 return 128;
}

void connectib_icmd_header_capture_headers_dump(const struct connectib_icmd_header_capture_headers *ptr_struct, FILE* file)
{
	connectib_icmd_header_capture_headers_print(ptr_struct, file, 0);
}

void connectib_g_rse_hw_decoders_pack(const union connectib_g_rse_hw_decoders *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_g_rse_decoders_as_dwords_pack(&(ptr_struct->raw_dwords), ptr_buff);
}

void connectib_g_rse_hw_decoders_unpack(union connectib_g_rse_hw_decoders *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_g_rse_decoders_as_dwords_unpack(&(ptr_struct->raw_dwords), ptr_buff);
}

void connectib_g_rse_hw_decoders_print(const union connectib_g_rse_hw_decoders *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_g_rse_hw_decoders ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rxt_checks_packet_descriptor_decoder:\n");
	connectib_rxt_checks_packet_descriptor_decoder_print(&(ptr_struct->rxt_checks_packet_descriptor_decoder), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rxt_steering_context_decoder:\n");
	connectib_rxt_steering_context_decoder_print(&(ptr_struct->rxt_steering_context_decoder), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rxt_checks_steering_results_decoder:\n");
	connectib_rxt_checks_steering_results_decoder_print(&(ptr_struct->rxt_checks_steering_results_decoder), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rxt_context_fetch_qp_desc_decoder:\n");
	connectib_rxt_context_fetch_qp_desc_decoder_print(&(ptr_struct->rxt_context_fetch_qp_desc_decoder), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rxs_rxt_desc_decoder:\n");
	connectib_rxs_rxt_desc_decoder_print(&(ptr_struct->rxs_rxt_desc_decoder), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "raw_dwords:\n");
	connectib_g_rse_decoders_as_dwords_print(&(ptr_struct->raw_dwords), file, indent_level + 1);

}

int connectib_g_rse_hw_decoders_size(void)
{
	 return 256;
}

void connectib_g_rse_hw_decoders_dump(const union connectib_g_rse_hw_decoders *ptr_struct, FILE* file)
{
	connectib_g_rse_hw_decoders_print(ptr_struct, file, 0);
}

void connectib_FW_VERSION_pack(const struct connectib_FW_VERSION *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->MAJOR);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->SUBMINOR);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->MINOR);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->Hour);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->Minutes);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->Seconds);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->Day);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->Month);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->Year);

}

void connectib_FW_VERSION_unpack(struct connectib_FW_VERSION *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->MAJOR = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	ptr_struct->SUBMINOR = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->MINOR = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 80;
	ptr_struct->Hour = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 72;
	ptr_struct->Minutes = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->Seconds = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 120;
	ptr_struct->Day = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 112;
	ptr_struct->Month = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->Year = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_FW_VERSION_print(const struct connectib_FW_VERSION *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_FW_VERSION ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "MAJOR                : " UH_FMT "\n", ptr_struct->MAJOR);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "SUBMINOR             : " UH_FMT "\n", ptr_struct->SUBMINOR);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "MINOR                : " UH_FMT "\n", ptr_struct->MINOR);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Hour                 : " UH_FMT "\n", ptr_struct->Hour);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Minutes              : " UH_FMT "\n", ptr_struct->Minutes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Seconds              : " UH_FMT "\n", ptr_struct->Seconds);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Day                  : " UH_FMT "\n", ptr_struct->Day);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Month                : " UH_FMT "\n", ptr_struct->Month);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "Year                 : " UH_FMT "\n", ptr_struct->Year);

}

int connectib_FW_VERSION_size(void)
{
	 return 16;
}

void connectib_FW_VERSION_dump(const struct connectib_FW_VERSION *ptr_struct, FILE* file)
{
	connectib_FW_VERSION_print(ptr_struct, file, 0);
}

void connectib_golan_hw_fw_ctx_pack(const union connectib_golan_hw_fw_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_ctx_as_dwords_pack(&(ptr_struct->as_dwords), ptr_buff);
}

void connectib_golan_hw_fw_ctx_unpack(union connectib_golan_hw_fw_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_ctx_as_dwords_unpack(&(ptr_struct->as_dwords), ptr_buff);
}

void connectib_golan_hw_fw_ctx_print(const union connectib_golan_hw_fw_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_golan_hw_fw_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_ctx:\n");
	connectib_hw_hca_contexts_print(&(ptr_struct->hw_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_ctx:\n");
	connectib_fw_resource_context_print(&(ptr_struct->fw_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "as_dwords:\n");
	connectib_ctx_as_dwords_print(&(ptr_struct->as_dwords), file, indent_level + 1);

}

int connectib_golan_hw_fw_ctx_size(void)
{
	 return 64;
}

void connectib_golan_hw_fw_ctx_dump(const union connectib_golan_hw_fw_ctx *ptr_struct, FILE* file)
{
	connectib_golan_hw_fw_ctx_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_fetch_add_128byte_pack(const struct connectib_wqe_extended_atomic_fetch_add_128byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2048, 1);
	connectib_add_data_pack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(1024, 32, i, 2048, 1);
	connectib_field_boundary_pack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_128byte_unpack(struct connectib_wqe_extended_atomic_fetch_add_128byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2048, 1);
	connectib_add_data_unpack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(1024, 32, i, 2048, 1);
	connectib_field_boundary_unpack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_128byte_print(const struct connectib_wqe_extended_atomic_fetch_add_128byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_fetch_add_128byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "add_data_%03d:\n", i);
	connectib_add_data_print(&(ptr_struct->add_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "field_boundary_%03d:\n", i);
	connectib_field_boundary_print(&(ptr_struct->field_boundary[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_fetch_add_128byte_size(void)
{
	 return 256;
}

void connectib_wqe_extended_atomic_fetch_add_128byte_dump(const struct connectib_wqe_extended_atomic_fetch_add_128byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_fetch_add_128byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_fetch_add_64byte_pack(const struct connectib_wqe_extended_atomic_fetch_add_64byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	connectib_add_data_pack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(512, 32, i, 1024, 1);
	connectib_field_boundary_pack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_64byte_unpack(struct connectib_wqe_extended_atomic_fetch_add_64byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	connectib_add_data_unpack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(512, 32, i, 1024, 1);
	connectib_field_boundary_unpack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_64byte_print(const struct connectib_wqe_extended_atomic_fetch_add_64byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_fetch_add_64byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "add_data_%03d:\n", i);
	connectib_add_data_print(&(ptr_struct->add_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "field_boundary_%03d:\n", i);
	connectib_field_boundary_print(&(ptr_struct->field_boundary[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_fetch_add_64byte_size(void)
{
	 return 128;
}

void connectib_wqe_extended_atomic_fetch_add_64byte_dump(const struct connectib_wqe_extended_atomic_fetch_add_64byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_fetch_add_64byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_fetch_add_32byte_pack(const struct connectib_wqe_extended_atomic_fetch_add_32byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	connectib_add_data_pack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 512, 1);
	connectib_field_boundary_pack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_32byte_unpack(struct connectib_wqe_extended_atomic_fetch_add_32byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	connectib_add_data_unpack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 512, 1);
	connectib_field_boundary_unpack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_32byte_print(const struct connectib_wqe_extended_atomic_fetch_add_32byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_fetch_add_32byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "add_data_%03d:\n", i);
	connectib_add_data_print(&(ptr_struct->add_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "field_boundary_%03d:\n", i);
	connectib_field_boundary_print(&(ptr_struct->field_boundary[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_fetch_add_32byte_size(void)
{
	 return 64;
}

void connectib_wqe_extended_atomic_fetch_add_32byte_dump(const struct connectib_wqe_extended_atomic_fetch_add_32byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_fetch_add_32byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_fetch_add_16byte_pack(const struct connectib_wqe_extended_atomic_fetch_add_16byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	connectib_add_data_pack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 256, 1);
	connectib_field_boundary_pack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_16byte_unpack(struct connectib_wqe_extended_atomic_fetch_add_16byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	connectib_add_data_unpack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 256, 1);
	connectib_field_boundary_unpack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_16byte_print(const struct connectib_wqe_extended_atomic_fetch_add_16byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_fetch_add_16byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "add_data_%03d:\n", i);
	connectib_add_data_print(&(ptr_struct->add_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "field_boundary_%03d:\n", i);
	connectib_field_boundary_print(&(ptr_struct->field_boundary[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_fetch_add_16byte_size(void)
{
	 return 32;
}

void connectib_wqe_extended_atomic_fetch_add_16byte_dump(const struct connectib_wqe_extended_atomic_fetch_add_16byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_fetch_add_16byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_fetch_add_8byte_pack(const struct connectib_wqe_extended_atomic_fetch_add_8byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	connectib_add_data_pack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 128, 1);
	connectib_field_boundary_pack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_8byte_unpack(struct connectib_wqe_extended_atomic_fetch_add_8byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	connectib_add_data_unpack(&(ptr_struct->add_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 128, 1);
	connectib_field_boundary_unpack(&(ptr_struct->field_boundary[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_fetch_add_8byte_print(const struct connectib_wqe_extended_atomic_fetch_add_8byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_fetch_add_8byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "add_data_%03d:\n", i);
	connectib_add_data_print(&(ptr_struct->add_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "field_boundary_%03d:\n", i);
	connectib_field_boundary_print(&(ptr_struct->field_boundary[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_fetch_add_8byte_size(void)
{
	 return 16;
}

void connectib_wqe_extended_atomic_fetch_add_8byte_dump(const struct connectib_wqe_extended_atomic_fetch_add_8byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_fetch_add_8byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_fetch_add_4byte_pack(const struct connectib_wqe_extended_atomic_fetch_add_4byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->add_data);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->field_boundary);

}

void connectib_wqe_extended_atomic_fetch_add_4byte_unpack(struct connectib_wqe_extended_atomic_fetch_add_4byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->add_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->field_boundary = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_extended_atomic_fetch_add_4byte_print(const struct connectib_wqe_extended_atomic_fetch_add_4byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_fetch_add_4byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "add_data             : " U32H_FMT "\n", ptr_struct->add_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "field_boundary       : " U32H_FMT "\n", ptr_struct->field_boundary);

}

int connectib_wqe_extended_atomic_fetch_add_4byte_size(void)
{
	 return 16;
}

void connectib_wqe_extended_atomic_fetch_add_4byte_dump(const struct connectib_wqe_extended_atomic_fetch_add_4byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_fetch_add_4byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_cmp_swap_128byte_pack(const struct connectib_wqe_extended_atomic_cmp_swap_128byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 4096, 1);
	connectib_swap_data_pack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(1024, 32, i, 4096, 1);
	connectib_compare_data_pack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(2048, 32, i, 4096, 1);
	connectib_swap_mask_data_pack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(3072, 32, i, 4096, 1);
	connectib_compare_mask_data_pack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_128byte_unpack(struct connectib_wqe_extended_atomic_cmp_swap_128byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 4096, 1);
	connectib_swap_data_unpack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(1024, 32, i, 4096, 1);
	connectib_compare_data_unpack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(2048, 32, i, 4096, 1);
	connectib_swap_mask_data_unpack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(3072, 32, i, 4096, 1);
	connectib_compare_mask_data_unpack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_128byte_print(const struct connectib_wqe_extended_atomic_cmp_swap_128byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_cmp_swap_128byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data_%03d:\n", i);
	connectib_swap_data_print(&(ptr_struct->swap_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data_%03d:\n", i);
	connectib_compare_data_print(&(ptr_struct->compare_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_mask_data_%03d:\n", i);
	connectib_swap_mask_data_print(&(ptr_struct->swap_mask_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_mask_data_%03d:\n", i);
	connectib_compare_mask_data_print(&(ptr_struct->compare_mask_data[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_cmp_swap_128byte_size(void)
{
	 return 512;
}

void connectib_wqe_extended_atomic_cmp_swap_128byte_dump(const struct connectib_wqe_extended_atomic_cmp_swap_128byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_cmp_swap_128byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_cmp_swap_64byte_pack(const struct connectib_wqe_extended_atomic_cmp_swap_64byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2048, 1);
	connectib_swap_data_pack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(512, 32, i, 2048, 1);
	connectib_compare_data_pack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(1024, 32, i, 2048, 1);
	connectib_swap_mask_data_pack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(1536, 32, i, 2048, 1);
	connectib_compare_mask_data_pack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_64byte_unpack(struct connectib_wqe_extended_atomic_cmp_swap_64byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2048, 1);
	connectib_swap_data_unpack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(512, 32, i, 2048, 1);
	connectib_compare_data_unpack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(1024, 32, i, 2048, 1);
	connectib_swap_mask_data_unpack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(1536, 32, i, 2048, 1);
	connectib_compare_mask_data_unpack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_64byte_print(const struct connectib_wqe_extended_atomic_cmp_swap_64byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_cmp_swap_64byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data_%03d:\n", i);
	connectib_swap_data_print(&(ptr_struct->swap_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data_%03d:\n", i);
	connectib_compare_data_print(&(ptr_struct->compare_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_mask_data_%03d:\n", i);
	connectib_swap_mask_data_print(&(ptr_struct->swap_mask_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_mask_data_%03d:\n", i);
	connectib_compare_mask_data_print(&(ptr_struct->compare_mask_data[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_cmp_swap_64byte_size(void)
{
	 return 256;
}

void connectib_wqe_extended_atomic_cmp_swap_64byte_dump(const struct connectib_wqe_extended_atomic_cmp_swap_64byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_cmp_swap_64byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_cmp_swap_32byte_pack(const struct connectib_wqe_extended_atomic_cmp_swap_32byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	connectib_swap_data_pack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 1024, 1);
	connectib_compare_data_pack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(512, 32, i, 1024, 1);
	connectib_swap_mask_data_pack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(768, 32, i, 1024, 1);
	connectib_compare_mask_data_pack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_32byte_unpack(struct connectib_wqe_extended_atomic_cmp_swap_32byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 1024, 1);
	connectib_swap_data_unpack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 1024, 1);
	connectib_compare_data_unpack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(512, 32, i, 1024, 1);
	connectib_swap_mask_data_unpack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(768, 32, i, 1024, 1);
	connectib_compare_mask_data_unpack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_32byte_print(const struct connectib_wqe_extended_atomic_cmp_swap_32byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_cmp_swap_32byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data_%03d:\n", i);
	connectib_swap_data_print(&(ptr_struct->swap_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data_%03d:\n", i);
	connectib_compare_data_print(&(ptr_struct->compare_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_mask_data_%03d:\n", i);
	connectib_swap_mask_data_print(&(ptr_struct->swap_mask_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_mask_data_%03d:\n", i);
	connectib_compare_mask_data_print(&(ptr_struct->compare_mask_data[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_cmp_swap_32byte_size(void)
{
	 return 128;
}

void connectib_wqe_extended_atomic_cmp_swap_32byte_dump(const struct connectib_wqe_extended_atomic_cmp_swap_32byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_cmp_swap_32byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_cmp_swap_16byte_pack(const struct connectib_wqe_extended_atomic_cmp_swap_16byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	connectib_swap_data_pack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 512, 1);
	connectib_compare_data_pack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 512, 1);
	connectib_swap_mask_data_pack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(384, 32, i, 512, 1);
	connectib_compare_mask_data_pack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_16byte_unpack(struct connectib_wqe_extended_atomic_cmp_swap_16byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	connectib_swap_data_unpack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 512, 1);
	connectib_compare_data_unpack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 512, 1);
	connectib_swap_mask_data_unpack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(384, 32, i, 512, 1);
	connectib_compare_mask_data_unpack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_16byte_print(const struct connectib_wqe_extended_atomic_cmp_swap_16byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_cmp_swap_16byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data_%03d:\n", i);
	connectib_swap_data_print(&(ptr_struct->swap_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data_%03d:\n", i);
	connectib_compare_data_print(&(ptr_struct->compare_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_mask_data_%03d:\n", i);
	connectib_swap_mask_data_print(&(ptr_struct->swap_mask_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_mask_data_%03d:\n", i);
	connectib_compare_mask_data_print(&(ptr_struct->compare_mask_data[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_cmp_swap_16byte_size(void)
{
	 return 64;
}

void connectib_wqe_extended_atomic_cmp_swap_16byte_dump(const struct connectib_wqe_extended_atomic_cmp_swap_16byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_cmp_swap_16byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_cmp_swap_8byte_pack(const struct connectib_wqe_extended_atomic_cmp_swap_8byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	connectib_swap_data_pack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 256, 1);
	connectib_compare_data_pack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 256, 1);
	connectib_swap_mask_data_pack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(192, 32, i, 256, 1);
	connectib_compare_mask_data_pack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_8byte_unpack(struct connectib_wqe_extended_atomic_cmp_swap_8byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 256, 1);
	connectib_swap_data_unpack(&(ptr_struct->swap_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(64, 32, i, 256, 1);
	connectib_compare_data_unpack(&(ptr_struct->compare_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 256, 1);
	connectib_swap_mask_data_unpack(&(ptr_struct->swap_mask_data[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(192, 32, i, 256, 1);
	connectib_compare_mask_data_unpack(&(ptr_struct->compare_mask_data[i]), ptr_buff + offset/8);
	}

}

void connectib_wqe_extended_atomic_cmp_swap_8byte_print(const struct connectib_wqe_extended_atomic_cmp_swap_8byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_cmp_swap_8byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data_%03d:\n", i);
	connectib_swap_data_print(&(ptr_struct->swap_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data_%03d:\n", i);
	connectib_compare_data_print(&(ptr_struct->compare_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_mask_data_%03d:\n", i);
	connectib_swap_mask_data_print(&(ptr_struct->swap_mask_data[i]), file, indent_level + 1);
	}

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_mask_data_%03d:\n", i);
	connectib_compare_mask_data_print(&(ptr_struct->compare_mask_data[i]), file, indent_level + 1);
	}

}

int connectib_wqe_extended_atomic_cmp_swap_8byte_size(void)
{
	 return 32;
}

void connectib_wqe_extended_atomic_cmp_swap_8byte_dump(const struct connectib_wqe_extended_atomic_cmp_swap_8byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_cmp_swap_8byte_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_cmp_swap_4byte_pack(const struct connectib_wqe_extended_atomic_cmp_swap_4byte *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->swap_data);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->compare_data);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->swap_mask_data);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->compare_mask_data);

}

void connectib_wqe_extended_atomic_cmp_swap_4byte_unpack(struct connectib_wqe_extended_atomic_cmp_swap_4byte *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->swap_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->compare_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->swap_mask_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->compare_mask_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_extended_atomic_cmp_swap_4byte_print(const struct connectib_wqe_extended_atomic_cmp_swap_4byte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_cmp_swap_4byte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data            : " U32H_FMT "\n", ptr_struct->swap_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data         : " U32H_FMT "\n", ptr_struct->compare_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_mask_data       : " U32H_FMT "\n", ptr_struct->swap_mask_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_mask_data    : " U32H_FMT "\n", ptr_struct->compare_mask_data);

}

int connectib_wqe_extended_atomic_cmp_swap_4byte_size(void)
{
	 return 16;
}

void connectib_wqe_extended_atomic_cmp_swap_4byte_dump(const struct connectib_wqe_extended_atomic_cmp_swap_4byte *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_cmp_swap_4byte_print(ptr_struct, file, 0);
}

void connectib_file_public_keys_pack(const struct connectib_file_public_keys *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_component_authentication_configuration_pack(&(ptr_struct->component_authentication_configuration), ptr_buff + offset/8);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->keypair_exp);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 2304, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->keypair_uuid[i]);
	}

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 2304, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->key[i]);
	}

}

void connectib_file_public_keys_unpack(struct connectib_file_public_keys *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_component_authentication_configuration_unpack(&(ptr_struct->component_authentication_configuration), ptr_buff + offset/8);

	offset = 96;
	ptr_struct->keypair_exp = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 2304, 1);
	ptr_struct->keypair_uuid[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 2304, 1);
	ptr_struct->key[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_file_public_keys_print(const struct connectib_file_public_keys *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_file_public_keys ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "component_authentication_configuration:\n");
	connectib_component_authentication_configuration_print(&(ptr_struct->component_authentication_configuration), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "keypair_exp          : " U32H_FMT "\n", ptr_struct->keypair_exp);

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "keypair_uuid_%03d    : " U32H_FMT "\n", i, ptr_struct->keypair_uuid[i]);
	}

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "key_%03d             : " U32H_FMT "\n", i, ptr_struct->key[i]);
	}

}

int connectib_file_public_keys_size(void)
{
	 return 288;
}

void connectib_file_public_keys_dump(const struct connectib_file_public_keys *ptr_struct, FILE* file)
{
	connectib_file_public_keys_print(ptr_struct, file, 0);
}

void connectib_nv_config_header_pack(const struct connectib_nv_config_header *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->version_minor);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->version_major);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->num_of_init_cfg_hdr);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_swaps);

	for (i = 0; i < 6; i++) {
	offset = adb2c_calc_array_field_address(96, 32, i, 512, 1);
	connectib_nv_data_pointer_pack(&(ptr_struct->list_pointers[i]), ptr_buff + offset/8);
	}

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->crc16);

}

void connectib_nv_config_header_unpack(struct connectib_nv_config_header *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->signature = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 56;
	ptr_struct->version_minor = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->version_major = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->num_of_init_cfg_hdr = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->num_of_swaps = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 6; i++) {
	offset = adb2c_calc_array_field_address(96, 32, i, 512, 1);
	connectib_nv_data_pointer_unpack(&(ptr_struct->list_pointers[i]), ptr_buff + offset/8);
	}

	offset = 496;
	ptr_struct->crc16 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_nv_config_header_print(const struct connectib_nv_config_header *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_header ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature            : " U32H_FMT "\n", ptr_struct->signature);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version_minor        : " UH_FMT "\n", ptr_struct->version_minor);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version_major        : " UH_FMT "\n", ptr_struct->version_major);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_init_cfg_hdr  : " UH_FMT "\n", ptr_struct->num_of_init_cfg_hdr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_swaps         : " U32H_FMT "\n", ptr_struct->num_of_swaps);

	for (i = 0; i < 6; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "list_pointers_%03d:\n", i);
	connectib_nv_data_pointer_print(&(ptr_struct->list_pointers[i]), file, indent_level + 1);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "crc16                : " UH_FMT "\n", ptr_struct->crc16);

}

int connectib_nv_config_header_size(void)
{
	 return 64;
}

void connectib_nv_config_header_dump(const struct connectib_nv_config_header *ptr_struct, FILE* file)
{
	connectib_nv_config_header_print(ptr_struct, file, 0);
}

void connectib_fw_image_pack(const struct connectib_fw_image *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->itoc_ptr);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->is_palladium);

	offset = 32;
	connectib_jump_addresses_pack(&(ptr_struct->jump_addresses), ptr_buff + offset/8);

}

void connectib_fw_image_unpack(struct connectib_fw_image *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->itoc_ptr = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 7;
	ptr_struct->is_palladium = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_jump_addresses_unpack(&(ptr_struct->jump_addresses), ptr_buff + offset/8);

}

void connectib_fw_image_print(const struct connectib_fw_image *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_image ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "itoc_ptr             : " UH_FMT "\n", ptr_struct->itoc_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_palladium         : " UH_FMT "\n", ptr_struct->is_palladium);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "jump_addresses:\n");
	connectib_jump_addresses_print(&(ptr_struct->jump_addresses), file, indent_level + 1);

}

int connectib_fw_image_size(void)
{
	 return 32;
}

void connectib_fw_image_dump(const struct connectib_fw_image *ptr_struct, FILE* file)
{
	connectib_fw_image_print(ptr_struct, file, 0);
}

void connectib_pcie_cfg_golan_pack(const struct connectib_pcie_cfg_golan *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_max_speed_supported);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->pcie_max_width_supported);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->drop_loopback_slave_en);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_automatic_reversal);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->upconfigure_capable);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->px_tx_reversal);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->polling_compliance_en);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pcie_center_phase_optimization_en);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->handle_bar_miss_error_by_cr_handler_en);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dc_ballance_en);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->receiver_detect_en);

	offset = 3;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->receiver_detect_time);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->slot_clock_cfg);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dynamic_pll_manage_en);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->static_width_reduction_en);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->do_mixer0_calibration_optimized);

	offset = 57;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->do_mixer0_calibration_delta);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->first_iteration_calibration_mask);

	offset = 64;
	connectib_pcie_serdes_cfg_pack(&(ptr_struct->serdes_cfg), ptr_buff + offset/8);

	offset = 320;
	connectib_pcie_equalization_and_eye_pack(&(ptr_struct->pcie_equalization_and_eye), ptr_buff + offset/8);

	offset = 736;
	connectib_pcie_debug_cap_pack(&(ptr_struct->pcie_debug_cap), ptr_buff + offset/8);

	offset = 768;
	connectib_pcie_tx_adaptation_cfg_pack(&(ptr_struct->tx_adaptation_cfg), ptr_buff + offset/8);

	offset = 896;
	connectib_pcie_rx_adaptation_cfg_pack(&(ptr_struct->rx_adaptation_cfg), ptr_buff + offset/8);

	offset = 1456;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pcie_tx_polarity);

}

void connectib_pcie_cfg_golan_unpack(struct connectib_pcie_cfg_golan *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->pcie_max_speed_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 23;
	ptr_struct->pcie_max_width_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 22;
	ptr_struct->drop_loopback_slave_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->disable_automatic_reversal = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->upconfigure_capable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->px_tx_reversal = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->polling_compliance_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->pcie_center_phase_optimization_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->handle_bar_miss_error_by_cr_handler_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->dc_ballance_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->receiver_detect_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 3;
	ptr_struct->receiver_detect_time = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 2;
	ptr_struct->slot_clock_cfg = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1;
	ptr_struct->dynamic_pll_manage_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->static_width_reduction_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 63;
	ptr_struct->do_mixer0_calibration_optimized = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 57;
	ptr_struct->do_mixer0_calibration_delta = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 32;
	ptr_struct->first_iteration_calibration_mask = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	connectib_pcie_serdes_cfg_unpack(&(ptr_struct->serdes_cfg), ptr_buff + offset/8);

	offset = 320;
	connectib_pcie_equalization_and_eye_unpack(&(ptr_struct->pcie_equalization_and_eye), ptr_buff + offset/8);

	offset = 736;
	connectib_pcie_debug_cap_unpack(&(ptr_struct->pcie_debug_cap), ptr_buff + offset/8);

	offset = 768;
	connectib_pcie_tx_adaptation_cfg_unpack(&(ptr_struct->tx_adaptation_cfg), ptr_buff + offset/8);

	offset = 896;
	connectib_pcie_rx_adaptation_cfg_unpack(&(ptr_struct->rx_adaptation_cfg), ptr_buff + offset/8);

	offset = 1456;
	ptr_struct->pcie_tx_polarity = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_pcie_cfg_golan_print(const struct connectib_pcie_cfg_golan *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_cfg_golan ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_max_speed_supported : " UH_FMT "\n", ptr_struct->pcie_max_speed_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_max_width_supported : " UH_FMT "\n", ptr_struct->pcie_max_width_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "drop_loopback_slave_en : " UH_FMT "\n", ptr_struct->drop_loopback_slave_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_automatic_reversal : " UH_FMT "\n", ptr_struct->disable_automatic_reversal);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "upconfigure_capable  : " UH_FMT "\n", ptr_struct->upconfigure_capable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "px_tx_reversal       : " UH_FMT "\n", ptr_struct->px_tx_reversal);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "polling_compliance_en : " UH_FMT "\n", ptr_struct->polling_compliance_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_center_phase_optimization_en : " UH_FMT "\n", ptr_struct->pcie_center_phase_optimization_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "handle_bar_miss_error_by_cr_handler_en : " UH_FMT "\n", ptr_struct->handle_bar_miss_error_by_cr_handler_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_ballance_en       : " UH_FMT "\n", ptr_struct->dc_ballance_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "receiver_detect_en   : " UH_FMT "\n", ptr_struct->receiver_detect_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "receiver_detect_time : " UH_FMT "\n", ptr_struct->receiver_detect_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slot_clock_cfg       : " UH_FMT "\n", ptr_struct->slot_clock_cfg);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dynamic_pll_manage_en : " UH_FMT "\n", ptr_struct->dynamic_pll_manage_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "static_width_reduction_en : " UH_FMT "\n", ptr_struct->static_width_reduction_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "do_mixer0_calibration_optimized : " UH_FMT "\n", ptr_struct->do_mixer0_calibration_optimized);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "do_mixer0_calibration_delta : " UH_FMT "\n", ptr_struct->do_mixer0_calibration_delta);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_iteration_calibration_mask : " UH_FMT "\n", ptr_struct->first_iteration_calibration_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "serdes_cfg:\n");
	connectib_pcie_serdes_cfg_print(&(ptr_struct->serdes_cfg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_equalization_and_eye:\n");
	connectib_pcie_equalization_and_eye_print(&(ptr_struct->pcie_equalization_and_eye), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_debug_cap:\n");
	connectib_pcie_debug_cap_print(&(ptr_struct->pcie_debug_cap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_adaptation_cfg:\n");
	connectib_pcie_tx_adaptation_cfg_print(&(ptr_struct->tx_adaptation_cfg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_adaptation_cfg:\n");
	connectib_pcie_rx_adaptation_cfg_print(&(ptr_struct->rx_adaptation_cfg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_tx_polarity     : " UH_FMT "\n", ptr_struct->pcie_tx_polarity);

}

int connectib_pcie_cfg_golan_size(void)
{
	 return 192;
}

void connectib_pcie_cfg_golan_dump(const struct connectib_pcie_cfg_golan *ptr_struct, FILE* file)
{
	connectib_pcie_cfg_golan_print(ptr_struct, file, 0);
}

void connectib_vpd_eeprom_info_pack(const struct connectib_vpd_eeprom_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vpd_size);

	offset = 11;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log2_vpd_eeprom_size);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->vpd_num_eeproms);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vpd_eeprom);

}

void connectib_vpd_eeprom_info_unpack(struct connectib_vpd_eeprom_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->vpd_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 11;
	ptr_struct->log2_vpd_eeprom_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 5;
	ptr_struct->vpd_num_eeproms = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 4;
	ptr_struct->vpd_eeprom = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_vpd_eeprom_info_print(const struct connectib_vpd_eeprom_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_vpd_eeprom_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_size             : " UH_FMT "\n", ptr_struct->vpd_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_vpd_eeprom_size : " UH_FMT "\n", ptr_struct->log2_vpd_eeprom_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_num_eeproms      : " UH_FMT "\n", ptr_struct->vpd_num_eeproms);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_eeprom           : " UH_FMT "\n", ptr_struct->vpd_eeprom);

}

int connectib_vpd_eeprom_info_size(void)
{
	 return 8;
}

void connectib_vpd_eeprom_info_dump(const struct connectib_vpd_eeprom_info *ptr_struct, FILE* file)
{
	connectib_vpd_eeprom_info_print(ptr_struct, file, 0);
}

void connectib_ini_gpios_golan_pack(const struct connectib_ini_gpios_golan *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_gpio_functions_pack(&(ptr_struct->functions), ptr_buff + offset/8);

	offset = 672;
	connectib_gpio_configurations_pack(&(ptr_struct->configurations), ptr_buff + offset/8);

}

void connectib_ini_gpios_golan_unpack(struct connectib_ini_gpios_golan *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_gpio_functions_unpack(&(ptr_struct->functions), ptr_buff + offset/8);

	offset = 672;
	connectib_gpio_configurations_unpack(&(ptr_struct->configurations), ptr_buff + offset/8);

}

void connectib_ini_gpios_golan_print(const struct connectib_ini_gpios_golan *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ini_gpios_golan ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "functions:\n");
	connectib_gpio_functions_print(&(ptr_struct->functions), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "configurations:\n");
	connectib_gpio_configurations_print(&(ptr_struct->configurations), file, indent_level + 1);

}

int connectib_ini_gpios_golan_size(void)
{
	 return 132;
}

void connectib_ini_gpios_golan_dump(const struct connectib_ini_gpios_golan *ptr_struct, FILE* file)
{
	connectib_ini_gpios_golan_print(ptr_struct, file, 0);
}

void connectib_i2c_devices_db_pack(const struct connectib_i2c_devices_db *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 640, 1);
	connectib_i2c_device_entry_pack(&(ptr_struct->device[i]), ptr_buff + offset/8);
	}

}

void connectib_i2c_devices_db_unpack(struct connectib_i2c_devices_db *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 640, 1);
	connectib_i2c_device_entry_unpack(&(ptr_struct->device[i]), ptr_buff + offset/8);
	}

}

void connectib_i2c_devices_db_print(const struct connectib_i2c_devices_db *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_i2c_devices_db ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_%03d:\n", i);
	connectib_i2c_device_entry_print(&(ptr_struct->device[i]), file, indent_level + 1);
	}

}

int connectib_i2c_devices_db_size(void)
{
	 return 80;
}

void connectib_i2c_devices_db_dump(const struct connectib_i2c_devices_db *ptr_struct, FILE* file)
{
	connectib_i2c_devices_db_print(ptr_struct, file, 0);
}

void connectib_module_config_pack(const struct connectib_module_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 64, 1);
	connectib_module_config_st_pack(&(ptr_struct->module[i]), ptr_buff + offset/8);
	}

}

void connectib_module_config_unpack(struct connectib_module_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 64, 1);
	connectib_module_config_st_unpack(&(ptr_struct->module[i]), ptr_buff + offset/8);
	}

}

void connectib_module_config_print(const struct connectib_module_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_module_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_%03d:\n", i);
	connectib_module_config_st_print(&(ptr_struct->module[i]), file, indent_level + 1);
	}

}

int connectib_module_config_size(void)
{
	 return 8;
}

void connectib_module_config_dump(const struct connectib_module_config *ptr_struct, FILE* file)
{
	connectib_module_config_print(ptr_struct, file, 0);
}

void connectib_thermal_config_golan_pack(const struct connectib_thermal_config_golan *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->warm2cold_threshold);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cold2warm_threshold);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->fw_shut_down_threshold);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fw_shut_down_en);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->gpio_enable);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->trap_supported);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->trap_enable);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fatal_enable);

	offset = 52;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->active_ext_sensors);

	offset = 45;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->a2d_compare_mask);

	offset = 42;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->a2d_measure_mask);

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->a2d_compare_and_measure_mask);

}

void connectib_thermal_config_golan_unpack(struct connectib_thermal_config_golan *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->warm2cold_threshold = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 16;
	ptr_struct->cold2warm_threshold = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->fw_shut_down_threshold = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 1;
	ptr_struct->fw_shut_down_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->gpio_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 63;
	ptr_struct->trap_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 62;
	ptr_struct->trap_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 61;
	ptr_struct->fatal_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 52;
	ptr_struct->active_ext_sensors = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 45;
	ptr_struct->a2d_compare_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 42;
	ptr_struct->a2d_measure_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 39;
	ptr_struct->a2d_compare_and_measure_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

}

void connectib_thermal_config_golan_print(const struct connectib_thermal_config_golan *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_thermal_config_golan ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "warm2cold_threshold  : " UH_FMT "\n", ptr_struct->warm2cold_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cold2warm_threshold  : " UH_FMT "\n", ptr_struct->cold2warm_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_shut_down_threshold : " UH_FMT "\n", ptr_struct->fw_shut_down_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_shut_down_en      : " UH_FMT "\n", ptr_struct->fw_shut_down_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpio_enable          : " UH_FMT "\n", ptr_struct->gpio_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trap_supported       : " UH_FMT "\n", ptr_struct->trap_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "trap_enable          : " UH_FMT "\n", ptr_struct->trap_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fatal_enable         : " UH_FMT "\n", ptr_struct->fatal_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "active_ext_sensors   : " UH_FMT "\n", ptr_struct->active_ext_sensors);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a2d_compare_mask     : %s (" UH_FMT ")\n", (ptr_struct->a2d_compare_mask == 0 ? ("NONE") : ((ptr_struct->a2d_compare_mask == 1 ? ("IOPX") : ((ptr_struct->a2d_compare_mask == 2 ? ("IOPL") : ((ptr_struct->a2d_compare_mask == 4 ? ("YU") : ("unknown")))))))), ptr_struct->a2d_compare_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a2d_measure_mask     : " UH_FMT "\n", ptr_struct->a2d_measure_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a2d_compare_and_measure_mask : " UH_FMT "\n", ptr_struct->a2d_compare_and_measure_mask);

}

int connectib_thermal_config_golan_size(void)
{
	 return 8;
}

void connectib_thermal_config_golan_dump(const struct connectib_thermal_config_golan *ptr_struct, FILE* file)
{
	connectib_thermal_config_golan_print(ptr_struct, file, 0);
}

void connectib_system_mng_pack(const struct connectib_system_mng *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port_swap);

}

void connectib_system_mng_unpack(struct connectib_system_mng *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->port_swap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_system_mng_print(const struct connectib_system_mng *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_system_mng ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_swap            : " UH_FMT "\n", ptr_struct->port_swap);

}

int connectib_system_mng_size(void)
{
	 return 4;
}

void connectib_system_mng_dump(const struct connectib_system_mng *ptr_struct, FILE* file)
{
	connectib_system_mng_print(ptr_struct, file, 0);
}

void connectib_power_pack(const struct connectib_power *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->fuse_dvfs);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fuse_dvfs_force);

	offset = 32;
	connectib_iopl_power_pack(&(ptr_struct->iopl_power), ptr_buff + offset/8);

	offset = 64;
	connectib_iopx_power_pack(&(ptr_struct->iopx_power), ptr_buff + offset/8);

}

void connectib_power_unpack(struct connectib_power *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->fuse_dvfs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 0;
	ptr_struct->fuse_dvfs_force = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_iopl_power_unpack(&(ptr_struct->iopl_power), ptr_buff + offset/8);

	offset = 64;
	connectib_iopx_power_unpack(&(ptr_struct->iopx_power), ptr_buff + offset/8);

}

void connectib_power_print(const struct connectib_power *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_power ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fuse_dvfs            : " UH_FMT "\n", ptr_struct->fuse_dvfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fuse_dvfs_force      : " UH_FMT "\n", ptr_struct->fuse_dvfs_force);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iopl_power:\n");
	connectib_iopl_power_print(&(ptr_struct->iopl_power), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iopx_power:\n");
	connectib_iopx_power_print(&(ptr_struct->iopx_power), file, indent_level + 1);

}

int connectib_power_size(void)
{
	 return 16;
}

void connectib_power_dump(const struct connectib_power *ptr_struct, FILE* file)
{
	connectib_power_print(ptr_struct, file, 0);
}

void connectib_port_serdes_pack(const struct connectib_port_serdes *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_phy_hw_main_config_ip_pack(&(ptr_struct->phy_hw_main_config_ip), ptr_buff + offset/8);

	offset = 11904;
	connectib_logic_serdes_mapping_node_ini_pack(&(ptr_struct->logic_serdes_mapping_node_ini), ptr_buff + offset/8);

	offset = 12448;
	connectib_pll_params_pack(&(ptr_struct->pll_params), ptr_buff + offset/8);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(12736, 128, i, 14336, 1);
	connectib_port_mlpn_db_pack(&(ptr_struct->mlpn_port[i]), ptr_buff + offset/8);
	}

}

void connectib_port_serdes_unpack(struct connectib_port_serdes *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_phy_hw_main_config_ip_unpack(&(ptr_struct->phy_hw_main_config_ip), ptr_buff + offset/8);

	offset = 11904;
	connectib_logic_serdes_mapping_node_ini_unpack(&(ptr_struct->logic_serdes_mapping_node_ini), ptr_buff + offset/8);

	offset = 12448;
	connectib_pll_params_unpack(&(ptr_struct->pll_params), ptr_buff + offset/8);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(12736, 128, i, 14336, 1);
	connectib_port_mlpn_db_unpack(&(ptr_struct->mlpn_port[i]), ptr_buff + offset/8);
	}

}

void connectib_port_serdes_print(const struct connectib_port_serdes *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_port_serdes ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_hw_main_config_ip:\n");
	connectib_phy_hw_main_config_ip_print(&(ptr_struct->phy_hw_main_config_ip), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "logic_serdes_mapping_node_ini:\n");
	connectib_logic_serdes_mapping_node_ini_print(&(ptr_struct->logic_serdes_mapping_node_ini), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pll_params:\n");
	connectib_pll_params_print(&(ptr_struct->pll_params), file, indent_level + 1);

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlpn_port_%03d:\n", i);
	connectib_port_mlpn_db_print(&(ptr_struct->mlpn_port[i]), file, indent_level + 1);
	}

}

int connectib_port_serdes_size(void)
{
	 return 1792;
}

void connectib_port_serdes_dump(const struct connectib_port_serdes *ptr_struct, FILE* file)
{
	connectib_port_serdes_print(ptr_struct, file, 0);
}

void connectib_nv_config_boot_pack(const struct connectib_nv_config_boot *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_config_boot_global_pack(&(ptr_struct->global), ptr_buff + offset/8);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(384, 32, i, 512, 1);
	connectib_nv_config_boot_port_settings_pack(&(ptr_struct->port[i]), ptr_buff + offset/8);
	}

}

void connectib_nv_config_boot_unpack(struct connectib_nv_config_boot *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_config_boot_global_unpack(&(ptr_struct->global), ptr_buff + offset/8);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(384, 32, i, 512, 1);
	connectib_nv_config_boot_port_settings_unpack(&(ptr_struct->port[i]), ptr_buff + offset/8);
	}

}

void connectib_nv_config_boot_print(const struct connectib_nv_config_boot *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_boot ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "global:\n");
	connectib_nv_config_boot_global_print(&(ptr_struct->global), file, indent_level + 1);

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_%03d:\n", i);
	connectib_nv_config_boot_port_settings_print(&(ptr_struct->port[i]), file, indent_level + 1);
	}

}

int connectib_nv_config_boot_size(void)
{
	 return 64;
}

void connectib_nv_config_boot_dump(const struct connectib_nv_config_boot *ptr_struct, FILE* file)
{
	connectib_nv_config_boot_print(ptr_struct, file, 0);
}

void connectib_pcie_power_management_pack(const struct connectib_pcie_power_management *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->delay_in_recovery_lock);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pcie_l1_en);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->close_icmc_on_standby);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->l1_on_gen1_gen2_en);

}

void connectib_pcie_power_management_unpack(struct connectib_pcie_power_management *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->delay_in_recovery_lock = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 15;
	ptr_struct->pcie_l1_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->close_icmc_on_standby = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->l1_on_gen1_gen2_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_pcie_power_management_print(const struct connectib_pcie_power_management *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_power_management ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "delay_in_recovery_lock : " UH_FMT "\n", ptr_struct->delay_in_recovery_lock);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_l1_en           : " UH_FMT "\n", ptr_struct->pcie_l1_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "close_icmc_on_standby : " UH_FMT "\n", ptr_struct->close_icmc_on_standby);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l1_on_gen1_gen2_en   : " UH_FMT "\n", ptr_struct->l1_on_gen1_gen2_en);

}

int connectib_pcie_power_management_size(void)
{
	 return 4;
}

void connectib_pcie_power_management_dump(const struct connectib_pcie_power_management *ptr_struct, FILE* file)
{
	connectib_pcie_power_management_print(ptr_struct, file, 0);
}

void connectib_pci_cfg_pack(const struct connectib_pci_cfg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_pci_cfg_space_pack(&(ptr_struct->pci_cfg_space), ptr_buff + offset/8);

	offset = 2048;
	connectib_pciex_ext_cfg_space_pack(&(ptr_struct->pciex_ext_cfg_space), ptr_buff + offset/8);

}

void connectib_pci_cfg_unpack(struct connectib_pci_cfg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_pci_cfg_space_unpack(&(ptr_struct->pci_cfg_space), ptr_buff + offset/8);

	offset = 2048;
	connectib_pciex_ext_cfg_space_unpack(&(ptr_struct->pciex_ext_cfg_space), ptr_buff + offset/8);

}

void connectib_pci_cfg_print(const struct connectib_pci_cfg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pci_cfg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_cfg_space:\n");
	connectib_pci_cfg_space_print(&(ptr_struct->pci_cfg_space), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pciex_ext_cfg_space:\n");
	connectib_pciex_ext_cfg_space_print(&(ptr_struct->pciex_ext_cfg_space), file, indent_level + 1);

}

int connectib_pci_cfg_size(void)
{
	 return 512;
}

void connectib_pci_cfg_dump(const struct connectib_pci_cfg *ptr_struct, FILE* file)
{
	connectib_pci_cfg_print(ptr_struct, file, 0);
}

void connectib_pcie_transaction_pack(const struct connectib_pcie_transaction *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->global_read_task_limit);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->function_level_reset_capability);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aer_anf_by_cfg_type_1_en);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aer_anf_by_bad_func_en);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->advisory_non_fatal_en);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aer_cap_en);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ari_cap_en);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sriov_cap_en);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vpd_cap_en);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pci_crspace_access_en);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pci_vsec_cap_en);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pcie_vsec_cap_en);

	offset = 32;
	connectib_logger_init_info_pack(&(ptr_struct->logger_init_info), ptr_buff + offset/8);

}

void connectib_pcie_transaction_unpack(struct connectib_pcie_transaction *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 23;
	ptr_struct->global_read_task_limit = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 22;
	ptr_struct->function_level_reset_capability = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->aer_anf_by_cfg_type_1_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->aer_anf_by_bad_func_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->advisory_non_fatal_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->aer_cap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->ari_cap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->sriov_cap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 15;
	ptr_struct->vpd_cap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->pci_crspace_access_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->pci_vsec_cap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->pcie_vsec_cap_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_logger_init_info_unpack(&(ptr_struct->logger_init_info), ptr_buff + offset/8);

}

void connectib_pcie_transaction_print(const struct connectib_pcie_transaction *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_transaction ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "global_read_task_limit : " UH_FMT "\n", ptr_struct->global_read_task_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "function_level_reset_capability : " UH_FMT "\n", ptr_struct->function_level_reset_capability);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aer_anf_by_cfg_type_1_en : " UH_FMT "\n", ptr_struct->aer_anf_by_cfg_type_1_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aer_anf_by_bad_func_en : " UH_FMT "\n", ptr_struct->aer_anf_by_bad_func_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "advisory_non_fatal_en : " UH_FMT "\n", ptr_struct->advisory_non_fatal_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aer_cap_en           : " UH_FMT "\n", ptr_struct->aer_cap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ari_cap_en           : " UH_FMT "\n", ptr_struct->ari_cap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sriov_cap_en         : " UH_FMT "\n", ptr_struct->sriov_cap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_cap_en           : " UH_FMT "\n", ptr_struct->vpd_cap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_crspace_access_en : " UH_FMT "\n", ptr_struct->pci_crspace_access_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pci_vsec_cap_en      : " UH_FMT "\n", ptr_struct->pci_vsec_cap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_vsec_cap_en     : " UH_FMT "\n", ptr_struct->pcie_vsec_cap_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "logger_init_info:\n");
	connectib_logger_init_info_print(&(ptr_struct->logger_init_info), file, indent_level + 1);

}

int connectib_pcie_transaction_size(void)
{
	 return 8;
}

void connectib_pcie_transaction_dump(const struct connectib_pcie_transaction *ptr_struct, FILE* file)
{
	connectib_pcie_transaction_print(ptr_struct, file, 0);
}

void connectib_multi_function_pack(const struct connectib_multi_function *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->vport0_hix);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lid_based);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->num_hosts);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->num_of_ports);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pf_msix_table_size);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vf_msix_table_size);

	offset = 89;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->pf_bar_type);

	offset = 81;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_minibar_type);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vf_minibar_prefetchable);

	offset = 76;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cr_bar_func);

	offset = 75;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cr_bar_type);

	offset = 74;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cr_bar_en);

	offset = 73;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->credits_blockage_fsm_en);

	offset = 68;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log_max_num_of_eqs);

}

void connectib_multi_function_unpack(struct connectib_multi_function *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	ptr_struct->vport0_hix = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 29;
	ptr_struct->lid_based = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->num_hosts = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 21;
	ptr_struct->num_of_ports = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 8;
	ptr_struct->pf_msix_table_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->vf_msix_table_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 89;
	ptr_struct->pf_bar_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 81;
	ptr_struct->vf_minibar_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->vf_minibar_prefetchable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 76;
	ptr_struct->cr_bar_func = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 75;
	ptr_struct->cr_bar_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 74;
	ptr_struct->cr_bar_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 73;
	ptr_struct->credits_blockage_fsm_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 68;
	ptr_struct->log_max_num_of_eqs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_multi_function_print(const struct connectib_multi_function *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_multi_function ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport0_hix           : " UH_FMT "\n", ptr_struct->vport0_hix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lid_based            : " UH_FMT "\n", ptr_struct->lid_based);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_hosts            : " UH_FMT "\n", ptr_struct->num_hosts);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_ports         : " UH_FMT "\n", ptr_struct->num_of_ports);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pf_msix_table_size   : " UH_FMT "\n", ptr_struct->pf_msix_table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_msix_table_size   : " UH_FMT "\n", ptr_struct->vf_msix_table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pf_bar_type          : " UH_FMT "\n", ptr_struct->pf_bar_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_minibar_type      : " UH_FMT "\n", ptr_struct->vf_minibar_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vf_minibar_prefetchable : " UH_FMT "\n", ptr_struct->vf_minibar_prefetchable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cr_bar_func          : " UH_FMT "\n", ptr_struct->cr_bar_func);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cr_bar_type          : " UH_FMT "\n", ptr_struct->cr_bar_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cr_bar_en            : " UH_FMT "\n", ptr_struct->cr_bar_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "credits_blockage_fsm_en : " UH_FMT "\n", ptr_struct->credits_blockage_fsm_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_num_of_eqs   : " UH_FMT "\n", ptr_struct->log_max_num_of_eqs);

}

int connectib_multi_function_size(void)
{
	 return 12;
}

void connectib_multi_function_dump(const struct connectib_multi_function *ptr_struct, FILE* file)
{
	connectib_multi_function_print(ptr_struct, file, 0);
}

void connectib_secure_fw_pack(const struct connectib_secure_fw *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	connectib_fw_block_behavior_pack(&(ptr_struct->fw_block_behavior), ptr_buff + offset/8);

}

void connectib_secure_fw_unpack(struct connectib_secure_fw *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	connectib_fw_block_behavior_unpack(&(ptr_struct->fw_block_behavior), ptr_buff + offset/8);

}

void connectib_secure_fw_print(const struct connectib_secure_fw *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_secure_fw ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_block_behavior:\n");
	connectib_fw_block_behavior_print(&(ptr_struct->fw_block_behavior), file, indent_level + 1);

}

int connectib_secure_fw_size(void)
{
	 return 8;
}

void connectib_secure_fw_dump(const struct connectib_secure_fw *ptr_struct, FILE* file)
{
	connectib_secure_fw_print(ptr_struct, file, 0);
}

void connectib_nv_config_main_pack(const struct connectib_nv_config_main *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_config_main_fw_data_pack(&(ptr_struct->fw_config), ptr_buff + offset/8);

}

void connectib_nv_config_main_unpack(struct connectib_nv_config_main *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_config_main_fw_data_unpack(&(ptr_struct->fw_config), ptr_buff + offset/8);

}

void connectib_nv_config_main_print(const struct connectib_nv_config_main *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_config_main ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_config:\n");
	connectib_nv_config_main_fw_data_print(&(ptr_struct->fw_config), file, indent_level + 1);

}

int connectib_nv_config_main_size(void)
{
	 return 768;
}

void connectib_nv_config_main_dump(const struct connectib_nv_config_main *ptr_struct, FILE* file)
{
	connectib_nv_config_main_print(ptr_struct, file, 0);
}

void connectib_phy_fw_main_config_pack(const struct connectib_phy_fw_main_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_phy_fw_main_config_golan_pack(&(ptr_struct->phy_golan), ptr_buff + offset/8);

	offset = 2048;
	connectib_phy_fw_main_config_ip_pack(&(ptr_struct->phy_ip), ptr_buff + offset/8);

}

void connectib_phy_fw_main_config_unpack(struct connectib_phy_fw_main_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_phy_fw_main_config_golan_unpack(&(ptr_struct->phy_golan), ptr_buff + offset/8);

	offset = 2048;
	connectib_phy_fw_main_config_ip_unpack(&(ptr_struct->phy_ip), ptr_buff + offset/8);

}

void connectib_phy_fw_main_config_print(const struct connectib_phy_fw_main_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_phy_fw_main_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_golan:\n");
	connectib_phy_fw_main_config_golan_print(&(ptr_struct->phy_golan), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_ip:\n");
	connectib_phy_fw_main_config_ip_print(&(ptr_struct->phy_ip), file, indent_level + 1);

}

int connectib_phy_fw_main_config_size(void)
{
	 return 720;
}

void connectib_phy_fw_main_config_dump(const struct connectib_phy_fw_main_config *ptr_struct, FILE* file)
{
	connectib_phy_fw_main_config_print(ptr_struct, file, 0);
}

void connectib_host_management_pack(const struct connectib_host_management *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 128;
	connectib_ocsd_config_pack(&(ptr_struct->ocsd_config), ptr_buff + offset/8);

	offset = 288;
	connectib_ibm_thermal_config_pack(&(ptr_struct->ibm_thermal_config), ptr_buff + offset/8);

	offset = 608;
	connectib_wol_cap_host_port_pack(&(ptr_struct->wol_cap), ptr_buff + offset/8);

}

void connectib_host_management_unpack(struct connectib_host_management *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 128;
	connectib_ocsd_config_unpack(&(ptr_struct->ocsd_config), ptr_buff + offset/8);

	offset = 288;
	connectib_ibm_thermal_config_unpack(&(ptr_struct->ibm_thermal_config), ptr_buff + offset/8);

	offset = 608;
	connectib_wol_cap_host_port_unpack(&(ptr_struct->wol_cap), ptr_buff + offset/8);

}

void connectib_host_management_print(const struct connectib_host_management *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_host_management ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ocsd_config:\n");
	connectib_ocsd_config_print(&(ptr_struct->ocsd_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ibm_thermal_config:\n");
	connectib_ibm_thermal_config_print(&(ptr_struct->ibm_thermal_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wol_cap:\n");
	connectib_wol_cap_host_port_print(&(ptr_struct->wol_cap), file, indent_level + 1);

}

int connectib_host_management_size(void)
{
	 return 128;
}

void connectib_host_management_dump(const struct connectib_host_management *ptr_struct, FILE* file)
{
	connectib_host_management_print(ptr_struct, file, 0);
}

void connectib_management_pack(const struct connectib_management *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_node_info_ini_pack(&(ptr_struct->node_info), ptr_buff + offset/8);

	offset = 512;
	connectib_ib_port_info_ini_pack(&(ptr_struct->port_info), ptr_buff + offset/8);

	offset = 1567;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->eeprom_read_disable_p1);

	offset = 1566;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->eeprom_read_disable_p2);

	offset = 1565;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fw_stamping_override);

	offset = 1564;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cable_speed_override);

	offset = 1568;
	connectib_extended_port_info_ini_pack(&(ptr_struct->extended_port_info), ptr_buff + offset/8);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(1624, 8, i, 3072, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->valid_i2c_address[i]);
	}

	offset = 2048;
	connectib_fw_vport0_ini_pack(&(ptr_struct->fw_vport0), ptr_buff + offset/8);

}

void connectib_management_unpack(struct connectib_management *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_node_info_ini_unpack(&(ptr_struct->node_info), ptr_buff + offset/8);

	offset = 512;
	connectib_ib_port_info_ini_unpack(&(ptr_struct->port_info), ptr_buff + offset/8);

	offset = 1567;
	ptr_struct->eeprom_read_disable_p1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1566;
	ptr_struct->eeprom_read_disable_p2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1565;
	ptr_struct->fw_stamping_override = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1564;
	ptr_struct->cable_speed_override = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 1568;
	connectib_extended_port_info_ini_unpack(&(ptr_struct->extended_port_info), ptr_buff + offset/8);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(1624, 8, i, 3072, 1);
	ptr_struct->valid_i2c_address[i] = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	offset = 2048;
	connectib_fw_vport0_ini_unpack(&(ptr_struct->fw_vport0), ptr_buff + offset/8);

}

void connectib_management_print(const struct connectib_management *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_management ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "node_info:\n");
	connectib_node_info_ini_print(&(ptr_struct->node_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_info:\n");
	connectib_ib_port_info_ini_print(&(ptr_struct->port_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eeprom_read_disable_p1 : " UH_FMT "\n", ptr_struct->eeprom_read_disable_p1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eeprom_read_disable_p2 : " UH_FMT "\n", ptr_struct->eeprom_read_disable_p2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_stamping_override : " UH_FMT "\n", ptr_struct->fw_stamping_override);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cable_speed_override : " UH_FMT "\n", ptr_struct->cable_speed_override);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "extended_port_info:\n");
	connectib_extended_port_info_ini_print(&(ptr_struct->extended_port_info), file, indent_level + 1);

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "valid_i2c_address_%03d : " UH_FMT "\n", i, ptr_struct->valid_i2c_address[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_vport0:\n");
	connectib_fw_vport0_ini_print(&(ptr_struct->fw_vport0), file, indent_level + 1);

}

int connectib_management_size(void)
{
	 return 384;
}

void connectib_management_dump(const struct connectib_management *ptr_struct, FILE* file)
{
	connectib_management_print(ptr_struct, file, 0);
}

void connectib_customization_pack(const struct connectib_customization *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->active_amp_range_p1);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->active_amp_range_p2);

}

void connectib_customization_unpack(struct connectib_customization *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->active_amp_range_p1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 24;
	ptr_struct->active_amp_range_p2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_customization_print(const struct connectib_customization *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_customization ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "active_amp_range_p1  : " UH_FMT "\n", ptr_struct->active_amp_range_p1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "active_amp_range_p2  : " UH_FMT "\n", ptr_struct->active_amp_range_p2);

}

int connectib_customization_size(void)
{
	 return 64;
}

void connectib_customization_dump(const struct connectib_customization *ptr_struct, FILE* file)
{
	connectib_customization_print(ptr_struct, file, 0);
}

void connectib_leds_blink_patterns_pack(const struct connectib_leds_blink_patterns *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_led_port_blink_pattern_pack(&(ptr_struct->no_phy_no_log_link), ptr_buff + offset/8);

	offset = 64;
	connectib_led_port_blink_pattern_pack(&(ptr_struct->phy_no_log_link), ptr_buff + offset/8);

	offset = 128;
	connectib_led_port_blink_pattern_pack(&(ptr_struct->phy_log_link), ptr_buff + offset/8);

	offset = 192;
	connectib_led_port_blink_pattern_pack(&(ptr_struct->phy_log_link_traffic), ptr_buff + offset/8);

	offset = 256;
	connectib_led_port_blink_pattern_pack(&(ptr_struct->error), ptr_buff + offset/8);

	for (i = 0; i < 15; i++) {
	offset = adb2c_calc_array_field_address(320, 64, i, 1280, 1);
	connectib_led_port_blink_pattern_pack(&(ptr_struct->reserved_pattern[i]), ptr_buff + offset/8);
	}

}

void connectib_leds_blink_patterns_unpack(struct connectib_leds_blink_patterns *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_led_port_blink_pattern_unpack(&(ptr_struct->no_phy_no_log_link), ptr_buff + offset/8);

	offset = 64;
	connectib_led_port_blink_pattern_unpack(&(ptr_struct->phy_no_log_link), ptr_buff + offset/8);

	offset = 128;
	connectib_led_port_blink_pattern_unpack(&(ptr_struct->phy_log_link), ptr_buff + offset/8);

	offset = 192;
	connectib_led_port_blink_pattern_unpack(&(ptr_struct->phy_log_link_traffic), ptr_buff + offset/8);

	offset = 256;
	connectib_led_port_blink_pattern_unpack(&(ptr_struct->error), ptr_buff + offset/8);

	for (i = 0; i < 15; i++) {
	offset = adb2c_calc_array_field_address(320, 64, i, 1280, 1);
	connectib_led_port_blink_pattern_unpack(&(ptr_struct->reserved_pattern[i]), ptr_buff + offset/8);
	}

}

void connectib_leds_blink_patterns_print(const struct connectib_leds_blink_patterns *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_leds_blink_patterns ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_phy_no_log_link:\n");
	connectib_led_port_blink_pattern_print(&(ptr_struct->no_phy_no_log_link), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_no_log_link:\n");
	connectib_led_port_blink_pattern_print(&(ptr_struct->phy_no_log_link), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_log_link:\n");
	connectib_led_port_blink_pattern_print(&(ptr_struct->phy_log_link), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_log_link_traffic:\n");
	connectib_led_port_blink_pattern_print(&(ptr_struct->phy_log_link_traffic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error:\n");
	connectib_led_port_blink_pattern_print(&(ptr_struct->error), file, indent_level + 1);

	for (i = 0; i < 15; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reserved_pattern_%03d:\n", i);
	connectib_led_port_blink_pattern_print(&(ptr_struct->reserved_pattern[i]), file, indent_level + 1);
	}

}

int connectib_leds_blink_patterns_size(void)
{
	 return 160;
}

void connectib_leds_blink_patterns_dump(const struct connectib_leds_blink_patterns *ptr_struct, FILE* file)
{
	connectib_leds_blink_patterns_print(ptr_struct, file, 0);
}

void connectib_static_config_pack(const struct connectib_static_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->dwrr_en);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ifarm_en);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->xrc_use_ib_spec_opcode);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->allowed_madifc_on_all_pfs);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ffser_en);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->prio_g_0_ix_mask_no_ifarm);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->prio_g_1_ix_mask_no_ifarm);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi_rl_custom_tokens_mantissa);

	offset = 76;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gvmi_rl_custom_tokens_exponent);

	offset = 73;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->gvmi_rl_mode);

	offset = 67;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ocp_bmc_mac_plus_two);

	offset = 65;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->global_pause_en);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->allow_flash_64k_erase);

}

void connectib_static_config_unpack(struct connectib_static_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->dwrr_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->ifarm_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->xrc_use_ib_spec_opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->allowed_madifc_on_all_pfs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 23;
	ptr_struct->ffser_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 56;
	ptr_struct->prio_g_0_ix_mask_no_ifarm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->prio_g_1_ix_mask_no_ifarm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 80;
	ptr_struct->gvmi_rl_custom_tokens_mantissa = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 76;
	ptr_struct->gvmi_rl_custom_tokens_exponent = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 73;
	ptr_struct->gvmi_rl_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 67;
	ptr_struct->ocp_bmc_mac_plus_two = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 65;
	ptr_struct->global_pause_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 64;
	ptr_struct->allow_flash_64k_erase = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_static_config_print(const struct connectib_static_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_static_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dwrr_en              : " UH_FMT "\n", ptr_struct->dwrr_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ifarm_en             : " UH_FMT "\n", ptr_struct->ifarm_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xrc_use_ib_spec_opcode : " UH_FMT "\n", ptr_struct->xrc_use_ib_spec_opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allowed_madifc_on_all_pfs : " UH_FMT "\n", ptr_struct->allowed_madifc_on_all_pfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ffser_en             : " UH_FMT "\n", ptr_struct->ffser_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prio_g_0_ix_mask_no_ifarm : " UH_FMT "\n", ptr_struct->prio_g_0_ix_mask_no_ifarm);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prio_g_1_ix_mask_no_ifarm : " UH_FMT "\n", ptr_struct->prio_g_1_ix_mask_no_ifarm);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_rl_custom_tokens_mantissa : " UH_FMT "\n", ptr_struct->gvmi_rl_custom_tokens_mantissa);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_rl_custom_tokens_exponent : " UH_FMT "\n", ptr_struct->gvmi_rl_custom_tokens_exponent);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_rl_mode         : " UH_FMT "\n", ptr_struct->gvmi_rl_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ocp_bmc_mac_plus_two : " UH_FMT "\n", ptr_struct->ocp_bmc_mac_plus_two);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "global_pause_en      : " UH_FMT "\n", ptr_struct->global_pause_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allow_flash_64k_erase : " UH_FMT "\n", ptr_struct->allow_flash_64k_erase);

}

int connectib_static_config_size(void)
{
	 return 16;
}

void connectib_static_config_dump(const struct connectib_static_config *ptr_struct, FILE* file)
{
	connectib_static_config_print(ptr_struct, file, 0);
}

void connectib_guids_pack(const struct connectib_guids *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 128, i, 512, 1);
	connectib_uid_entry_pack(&(ptr_struct->guids[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(256, 128, i, 512, 1);
	connectib_uid_entry_pack(&(ptr_struct->macs[i]), ptr_buff + offset/8);
	}

}

void connectib_guids_unpack(struct connectib_guids *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(0, 128, i, 512, 1);
	connectib_uid_entry_unpack(&(ptr_struct->guids[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(256, 128, i, 512, 1);
	connectib_uid_entry_unpack(&(ptr_struct->macs[i]), ptr_buff + offset/8);
	}

}

void connectib_guids_print(const struct connectib_guids *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_guids ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "guids_%03d:\n", i);
	connectib_uid_entry_print(&(ptr_struct->guids[i]), file, indent_level + 1);
	}

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "macs_%03d:\n", i);
	connectib_uid_entry_print(&(ptr_struct->macs[i]), file, indent_level + 1);
	}

}

int connectib_guids_size(void)
{
	 return 64;
}

void connectib_guids_dump(const struct connectib_guids *ptr_struct, FILE* file)
{
	connectib_guids_print(ptr_struct, file, 0);
}

void connectib_module_versions_pack(const struct connectib_module_versions *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_module_version_pack(&(ptr_struct->core), ptr_buff + offset/8);

	offset = 32;
	connectib_module_version_pack(&(ptr_struct->phy), ptr_buff + offset/8);

	offset = 64;
	connectib_module_version_pack(&(ptr_struct->kernel), ptr_buff + offset/8);

	offset = 96;
	connectib_module_version_pack(&(ptr_struct->iron_image), ptr_buff + offset/8);

	offset = 128;
	connectib_module_version_pack(&(ptr_struct->host_management), ptr_buff + offset/8);

	offset = 160;
	connectib_module_version_pack(&(ptr_struct->mad), ptr_buff + offset/8);

}

void connectib_module_versions_unpack(struct connectib_module_versions *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_module_version_unpack(&(ptr_struct->core), ptr_buff + offset/8);

	offset = 32;
	connectib_module_version_unpack(&(ptr_struct->phy), ptr_buff + offset/8);

	offset = 64;
	connectib_module_version_unpack(&(ptr_struct->kernel), ptr_buff + offset/8);

	offset = 96;
	connectib_module_version_unpack(&(ptr_struct->iron_image), ptr_buff + offset/8);

	offset = 128;
	connectib_module_version_unpack(&(ptr_struct->host_management), ptr_buff + offset/8);

	offset = 160;
	connectib_module_version_unpack(&(ptr_struct->mad), ptr_buff + offset/8);

}

void connectib_module_versions_print(const struct connectib_module_versions *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_module_versions ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core:\n");
	connectib_module_version_print(&(ptr_struct->core), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy:\n");
	connectib_module_version_print(&(ptr_struct->phy), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "kernel:\n");
	connectib_module_version_print(&(ptr_struct->kernel), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iron_image:\n");
	connectib_module_version_print(&(ptr_struct->iron_image), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_management:\n");
	connectib_module_version_print(&(ptr_struct->host_management), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mad:\n");
	connectib_module_version_print(&(ptr_struct->mad), file, indent_level + 1);

}

int connectib_module_versions_size(void)
{
	 return 64;
}

void connectib_module_versions_dump(const struct connectib_module_versions *ptr_struct, FILE* file)
{
	connectib_module_versions_print(ptr_struct, file, 0);
}

void connectib_image_size_pack(const struct connectib_image_size *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->log_step);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->max_size);

}

void connectib_image_size_unpack(struct connectib_image_size *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->log_step = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->max_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_image_size_print(const struct connectib_image_size *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_image_size ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_step             : " UH_FMT "\n", ptr_struct->log_step);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_size             : " U32H_FMT "\n", ptr_struct->max_size);

}

int connectib_image_size_size(void)
{
	 return 8;
}

void connectib_image_size_dump(const struct connectib_image_size *ptr_struct, FILE* file)
{
	connectib_image_size_print(ptr_struct, file, 0);
}

void connectib_TRIPPLE_VERSION_pack(const struct connectib_TRIPPLE_VERSION *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->MAJOR);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->SUBMINOR);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->MINOR);

}

void connectib_TRIPPLE_VERSION_unpack(struct connectib_TRIPPLE_VERSION *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->MAJOR = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	ptr_struct->SUBMINOR = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->MINOR = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_TRIPPLE_VERSION_print(const struct connectib_TRIPPLE_VERSION *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_TRIPPLE_VERSION ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "MAJOR                : " UH_FMT "\n", ptr_struct->MAJOR);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "SUBMINOR             : " UH_FMT "\n", ptr_struct->SUBMINOR);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "MINOR                : " UH_FMT "\n", ptr_struct->MINOR);

}

int connectib_TRIPPLE_VERSION_size(void)
{
	 return 8;
}

void connectib_TRIPPLE_VERSION_dump(const struct connectib_TRIPPLE_VERSION *ptr_struct, FILE* file)
{
	connectib_TRIPPLE_VERSION_print(ptr_struct, file, 0);
}

void connectib_operation_key_pack(const struct connectib_operation_key *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->key_modifier);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->key);

}

void connectib_operation_key_unpack(struct connectib_operation_key *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->key_modifier = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->key = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_operation_key_print(const struct connectib_operation_key *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_operation_key ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "key_modifier         : " UH_FMT "\n", ptr_struct->key_modifier);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "key                  : " U64H_FMT "\n", ptr_struct->key);

}

int connectib_operation_key_size(void)
{
	 return 16;
}

void connectib_operation_key_dump(const struct connectib_operation_key *ptr_struct, FILE* file)
{
	connectib_operation_key_print(ptr_struct, file, 0);
}

void connectib_toc_data_pack(const struct connectib_toc_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->address);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->size);

}

void connectib_toc_data_unpack(struct connectib_toc_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->address = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_toc_data_print(const struct connectib_toc_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_toc_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address              : " U32H_FMT "\n", ptr_struct->address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " U32H_FMT "\n", ptr_struct->size);

}

int connectib_toc_data_size(void)
{
	 return 8;
}

void connectib_toc_data_dump(const struct connectib_toc_data *ptr_struct, FILE* file)
{
	connectib_toc_data_print(ptr_struct, file, 0);
}

void connectib_rst_api_ver_pack(const struct connectib_rst_api_ver *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 128, i, 1024, 1);
	connectib_reset_level_info_pack(&(ptr_struct->reset[i]), ptr_buff + offset/8);
	}

}

void connectib_rst_api_ver_unpack(struct connectib_rst_api_ver *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 128, i, 1024, 1);
	connectib_reset_level_info_unpack(&(ptr_struct->reset[i]), ptr_buff + offset/8);
	}

}

void connectib_rst_api_ver_print(const struct connectib_rst_api_ver *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_rst_api_ver ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reset_%03d:\n", i);
	connectib_reset_level_info_print(&(ptr_struct->reset[i]), file, indent_level + 1);
	}

}

int connectib_rst_api_ver_size(void)
{
	 return 128;
}

void connectib_rst_api_ver_dump(const struct connectib_rst_api_ver *ptr_struct, FILE* file)
{
	connectib_rst_api_ver_print(ptr_struct, file, 0);
}

void connectib_reset_capabilities_pack(const struct connectib_reset_capabilities *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reset_ver_en);

}

void connectib_reset_capabilities_unpack(struct connectib_reset_capabilities *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->reset_ver_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_reset_capabilities_print(const struct connectib_reset_capabilities *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_reset_capabilities ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reset_ver_en         : " UH_FMT "\n", ptr_struct->reset_ver_en);

}

int connectib_reset_capabilities_size(void)
{
	 return 4;
}

void connectib_reset_capabilities_dump(const struct connectib_reset_capabilities *ptr_struct, FILE* file)
{
	connectib_reset_capabilities_print(ptr_struct, file, 0);
}

void connectib_icmd_get_gvmi_rl_ctx_ix_pack(const struct connectib_icmd_get_gvmi_rl_ctx_ix *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->real_gvmi);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi_rl_ctx_ix);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi_tag);

}

void connectib_icmd_get_gvmi_rl_ctx_ix_unpack(struct connectib_icmd_get_gvmi_rl_ctx_ix *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->real_gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 12;
	ptr_struct->vl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->gvmi_rl_ctx_ix = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->gvmi_tag = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_get_gvmi_rl_ctx_ix_print(const struct connectib_icmd_get_gvmi_rl_ctx_ix *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_gvmi_rl_ctx_ix ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "real_gvmi            : " UH_FMT "\n", ptr_struct->real_gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl                   : " UH_FMT "\n", ptr_struct->vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_rl_ctx_ix       : " UH_FMT "\n", ptr_struct->gvmi_rl_ctx_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_tag             : " UH_FMT "\n", ptr_struct->gvmi_tag);

}

int connectib_icmd_get_gvmi_rl_ctx_ix_size(void)
{
	 return 8;
}

void connectib_icmd_get_gvmi_rl_ctx_ix_dump(const struct connectib_icmd_get_gvmi_rl_ctx_ix *ptr_struct, FILE* file)
{
	connectib_icmd_get_gvmi_rl_ctx_ix_print(ptr_struct, file, 0);
}

void connectib_icmd_sysport_pack(const struct connectib_icmd_sysport *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_sysport_packet_properties_pack(&(ptr_struct->pkt_prop), ptr_buff + offset/8);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pkt_length);

	offset = 288;
	connectib_sysport_packet_data_pack(&(ptr_struct->pkt_data), ptr_buff + offset/8);

}

void connectib_icmd_sysport_unpack(struct connectib_icmd_sysport *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_sysport_packet_properties_unpack(&(ptr_struct->pkt_prop), ptr_buff + offset/8);

	offset = 256;
	ptr_struct->pkt_length = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	connectib_sysport_packet_data_unpack(&(ptr_struct->pkt_data), ptr_buff + offset/8);

}

void connectib_icmd_sysport_print(const struct connectib_icmd_sysport *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_sysport ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkt_prop:\n");
	connectib_sysport_packet_properties_print(&(ptr_struct->pkt_prop), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkt_length           : " U32H_FMT "\n", ptr_struct->pkt_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pkt_data:\n");
	connectib_sysport_packet_data_print(&(ptr_struct->pkt_data), file, indent_level + 1);

}

int connectib_icmd_sysport_size(void)
{
	 return 768;
}

void connectib_icmd_sysport_dump(const struct connectib_icmd_sysport *ptr_struct, FILE* file)
{
	connectib_icmd_sysport_print(ptr_struct, file, 0);
}

void connectib_icmd_drop_counter_read_pack(const struct connectib_icmd_drop_counter_read *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sq_num_to);

}

void connectib_icmd_drop_counter_read_unpack(struct connectib_icmd_drop_counter_read *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->sq_num_to = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_drop_counter_read_print(const struct connectib_icmd_drop_counter_read *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_drop_counter_read ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq_num_to            : " U32H_FMT "\n", ptr_struct->sq_num_to);

}

int connectib_icmd_drop_counter_read_size(void)
{
	 return 4;
}

void connectib_icmd_drop_counter_read_dump(const struct connectib_icmd_drop_counter_read *ptr_struct, FILE* file)
{
	connectib_icmd_drop_counter_read_print(ptr_struct, file, 0);
}

void connectib_icmd_rw_gvmi_fw_ctx_pack(const struct connectib_icmd_rw_gvmi_fw_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_rw_gvmi_fw_ctx_sub_structs_pack(&(ptr_struct->sub_structs), ptr_buff + offset/8);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gvmi);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->node_type);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ix);

}

void connectib_icmd_rw_gvmi_fw_ctx_unpack(struct connectib_icmd_rw_gvmi_fw_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_rw_gvmi_fw_ctx_sub_structs_unpack(&(ptr_struct->sub_structs), ptr_buff + offset/8);

	offset = 512;
	ptr_struct->gvmi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->node_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_rw_gvmi_fw_ctx_print(const struct connectib_icmd_rw_gvmi_fw_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_rw_gvmi_fw_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sub_structs:\n");
	connectib_rw_gvmi_fw_ctx_sub_structs_print(&(ptr_struct->sub_structs), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " U32H_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "node_type            : " U32H_FMT "\n", ptr_struct->node_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ix                   : " U32H_FMT "\n", ptr_struct->ix);

}

int connectib_icmd_rw_gvmi_fw_ctx_size(void)
{
	 return 128;
}

void connectib_icmd_rw_gvmi_fw_ctx_dump(const struct connectib_icmd_rw_gvmi_fw_ctx *ptr_struct, FILE* file)
{
	connectib_icmd_rw_gvmi_fw_ctx_print(ptr_struct, file, 0);
}

void connectib_icmd_mh_sync_pack(const struct connectib_icmd_mh_sync *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->input_state);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->input_sync_type);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->input_ignore_inactive_hosts);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fsm_state);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->fsm_sync_type);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fsm_ignore_inactive_hosts);

	offset = 92;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fsm_host_ready);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->fsm_start_uptime);

}

void connectib_icmd_mh_sync_unpack(struct connectib_icmd_mh_sync *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->input_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 16;
	ptr_struct->input_sync_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->input_ignore_inactive_hosts = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 60;
	ptr_struct->fsm_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 48;
	ptr_struct->fsm_sync_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->fsm_ignore_inactive_hosts = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 92;
	ptr_struct->fsm_host_ready = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->fsm_start_uptime = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_mh_sync_print(const struct connectib_icmd_mh_sync *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_mh_sync ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "input_state          : " UH_FMT "\n", ptr_struct->input_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "input_sync_type      : " UH_FMT "\n", ptr_struct->input_sync_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "input_ignore_inactive_hosts : " UH_FMT "\n", ptr_struct->input_ignore_inactive_hosts);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fsm_state            : " UH_FMT "\n", ptr_struct->fsm_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fsm_sync_type        : " UH_FMT "\n", ptr_struct->fsm_sync_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fsm_ignore_inactive_hosts : " UH_FMT "\n", ptr_struct->fsm_ignore_inactive_hosts);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fsm_host_ready       : " UH_FMT "\n", ptr_struct->fsm_host_ready);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fsm_start_uptime     : " U32H_FMT "\n", ptr_struct->fsm_start_uptime);

}

int connectib_icmd_mh_sync_size(void)
{
	 return 16;
}

void connectib_icmd_mh_sync_dump(const struct connectib_icmd_mh_sync *ptr_struct, FILE* file)
{
	connectib_icmd_mh_sync_print(ptr_struct, file, 0);
}

void connectib_icmd_asn1_force_port_state_pack(const struct connectib_icmd_asn1_force_port_state *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port0_1);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_enable_disable);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->state_enable_disable);

}

void connectib_icmd_asn1_force_port_state_unpack(struct connectib_icmd_asn1_force_port_state *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->port0_1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 23;
	ptr_struct->force_enable_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->state_enable_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_asn1_force_port_state_print(const struct connectib_icmd_asn1_force_port_state *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_asn1_force_port_state ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port0_1              : " UH_FMT "\n", ptr_struct->port0_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_enable_disable : " UH_FMT "\n", ptr_struct->force_enable_disable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "state_enable_disable : " UH_FMT "\n", ptr_struct->state_enable_disable);

}

int connectib_icmd_asn1_force_port_state_size(void)
{
	 return 4;
}

void connectib_icmd_asn1_force_port_state_dump(const struct connectib_icmd_asn1_force_port_state *ptr_struct, FILE* file)
{
	connectib_icmd_asn1_force_port_state_print(ptr_struct, file, 0);
}

void connectib_icmd_port_admin_state_mng_pack(const struct connectib_icmd_port_admin_state_mng *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->set_get_);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->portid);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->mgmt_update_mask);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->mgmt_update_data);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->klu_update_mask);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->klu_update_data);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->klu_state);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reneg_on_change_set_mask);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->reneg_on_change_set_data);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->reneg_on_change_set_gvmi);

	offset = 96;
	connectib_pf_array_default_ones_pack(&(ptr_struct->reneg_on_change), ptr_buff + offset/8);

	offset = 160;
	connectib_drivers_mask_arr_pack(&(ptr_struct->drivers_mask), ptr_buff + offset/8);

	offset = 224;
	connectib_disable_mask_pack(&(ptr_struct->disable_mask), ptr_buff + offset/8);

}

void connectib_icmd_port_admin_state_mng_unpack(struct connectib_icmd_port_admin_state_mng *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->set_get_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->portid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->mgmt_update_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 22;
	ptr_struct->mgmt_update_data = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 16;
	ptr_struct->klu_update_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 8;
	ptr_struct->klu_update_data = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 2;
	ptr_struct->klu_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 63;
	ptr_struct->reneg_on_change_set_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 55;
	ptr_struct->reneg_on_change_set_data = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->reneg_on_change_set_gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	connectib_pf_array_default_ones_unpack(&(ptr_struct->reneg_on_change), ptr_buff + offset/8);

	offset = 160;
	connectib_drivers_mask_arr_unpack(&(ptr_struct->drivers_mask), ptr_buff + offset/8);

	offset = 224;
	connectib_disable_mask_unpack(&(ptr_struct->disable_mask), ptr_buff + offset/8);

}

void connectib_icmd_port_admin_state_mng_print(const struct connectib_icmd_port_admin_state_mng *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_port_admin_state_mng ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_get_             : " UH_FMT "\n", ptr_struct->set_get_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "portid               : " UH_FMT "\n", ptr_struct->portid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mgmt_update_mask     : " UH_FMT "\n", ptr_struct->mgmt_update_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mgmt_update_data     : " UH_FMT "\n", ptr_struct->mgmt_update_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "klu_update_mask      : " UH_FMT "\n", ptr_struct->klu_update_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "klu_update_data      : " UH_FMT "\n", ptr_struct->klu_update_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "klu_state            : " UH_FMT "\n", ptr_struct->klu_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reneg_on_change_set_mask : " UH_FMT "\n", ptr_struct->reneg_on_change_set_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reneg_on_change_set_data : " UH_FMT "\n", ptr_struct->reneg_on_change_set_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reneg_on_change_set_gvmi : " UH_FMT "\n", ptr_struct->reneg_on_change_set_gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reneg_on_change:\n");
	connectib_pf_array_default_ones_print(&(ptr_struct->reneg_on_change), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "drivers_mask:\n");
	connectib_drivers_mask_arr_print(&(ptr_struct->drivers_mask), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_mask:\n");
	connectib_disable_mask_print(&(ptr_struct->disable_mask), file, indent_level + 1);

}

int connectib_icmd_port_admin_state_mng_size(void)
{
	 return 32;
}

void connectib_icmd_port_admin_state_mng_dump(const struct connectib_icmd_port_admin_state_mng *ptr_struct, FILE* file)
{
	connectib_icmd_port_admin_state_mng_print(ptr_struct, file, 0);
}

void connectib_icmd_get_packet_pacing_debug_info_pack(const struct connectib_icmd_get_packet_pacing_debug_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->rate_limit_index);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rate_limit_active);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rate_kbits);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->burst_upper_bound_bytes);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->burst_bytes);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log2qp_rate_limiter_time);

	for (i = 0; i < 20; i++) {
	offset = adb2c_calc_array_field_address(160, 32, i, 800, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sq_index[i]);
	}

}

void connectib_icmd_get_packet_pacing_debug_info_unpack(struct connectib_icmd_get_packet_pacing_debug_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 8;
	ptr_struct->rate_limit_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->rate_limit_active = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->rate_kbits = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->burst_upper_bound_bytes = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->burst_bytes = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->log2qp_rate_limiter_time = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 20; i++) {
	offset = adb2c_calc_array_field_address(160, 32, i, 800, 1);
	ptr_struct->sq_index[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_icmd_get_packet_pacing_debug_info_print(const struct connectib_icmd_get_packet_pacing_debug_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_packet_pacing_debug_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rate_limit_index     : " UH_FMT "\n", ptr_struct->rate_limit_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rate_limit_active    : " UH_FMT "\n", ptr_struct->rate_limit_active);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rate_kbits           : " U32H_FMT "\n", ptr_struct->rate_kbits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "burst_upper_bound_bytes : " U32H_FMT "\n", ptr_struct->burst_upper_bound_bytes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "burst_bytes          : " U32H_FMT "\n", ptr_struct->burst_bytes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2qp_rate_limiter_time : " U32H_FMT "\n", ptr_struct->log2qp_rate_limiter_time);

	for (i = 0; i < 20; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sq_index_%03d        : " U32H_FMT "\n", i, ptr_struct->sq_index[i]);
	}

}

int connectib_icmd_get_packet_pacing_debug_info_size(void)
{
	 return 100;
}

void connectib_icmd_get_packet_pacing_debug_info_dump(const struct connectib_icmd_get_packet_pacing_debug_info *ptr_struct, FILE* file)
{
	connectib_icmd_get_packet_pacing_debug_info_print(ptr_struct, file, 0);
}

void connectib_icmd_map_event_eq_pack(const struct connectib_icmd_map_event_eq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->set_get_);

	offset = 14;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->map_unmap_);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->eqn);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->event_bitmask);

}

void connectib_icmd_map_event_eq_unpack(struct connectib_icmd_map_event_eq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 15;
	ptr_struct->set_get_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 14;
	ptr_struct->map_unmap_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->eqn = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->event_bitmask = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_map_event_eq_print(const struct connectib_icmd_map_event_eq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_map_event_eq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_get_             : " UH_FMT "\n", ptr_struct->set_get_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "map_unmap_           : " UH_FMT "\n", ptr_struct->map_unmap_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eqn                  : " UH_FMT "\n", ptr_struct->eqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "event_bitmask        : " U64H_FMT "\n", ptr_struct->event_bitmask);

}

int connectib_icmd_map_event_eq_size(void)
{
	 return 16;
}

void connectib_icmd_map_event_eq_dump(const struct connectib_icmd_map_event_eq *ptr_struct, FILE* file)
{
	connectib_icmd_map_event_eq_print(ptr_struct, file, 0);
}

void connectib_icmd_internal_query_cap_pack(const struct connectib_icmd_internal_query_cap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->opcode);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->opcode_exists_out);

}

void connectib_icmd_internal_query_cap_unpack(struct connectib_icmd_internal_query_cap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->opcode = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->opcode_exists_out = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_internal_query_cap_print(const struct connectib_icmd_internal_query_cap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_internal_query_cap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " U32H_FMT "\n", ptr_struct->opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode_exists_out    : " U32H_FMT "\n", ptr_struct->opcode_exists_out);

}

int connectib_icmd_internal_query_cap_size(void)
{
	 return 8;
}

void connectib_icmd_internal_query_cap_dump(const struct connectib_icmd_internal_query_cap *ptr_struct, FILE* file)
{
	connectib_icmd_internal_query_cap_print(ptr_struct, file, 0);
}

void connectib_icmd_detect_gvmi_in_qpc_and_icmc_pack(const struct connectib_icmd_detect_gvmi_in_qpc_and_icmc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpi);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->qpc_detection_valid);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->icmc_detection_valid);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->va_63_32);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->va_31_0);

}

void connectib_icmd_detect_gvmi_in_qpc_and_icmc_unpack(struct connectib_icmd_detect_gvmi_in_qpc_and_icmc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->qpi = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 32;
	ptr_struct->qpc_detection_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->icmc_detection_valid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->va_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->va_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_detect_gvmi_in_qpc_and_icmc_print(const struct connectib_icmd_detect_gvmi_in_qpc_and_icmc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_detect_gvmi_in_qpc_and_icmc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpi                  : " UH_FMT "\n", ptr_struct->qpi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpc_detection_valid  : " UH_FMT "\n", ptr_struct->qpc_detection_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icmc_detection_valid : " UH_FMT "\n", ptr_struct->icmc_detection_valid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "va_63_32             : " U32H_FMT "\n", ptr_struct->va_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "va_31_0              : " U32H_FMT "\n", ptr_struct->va_31_0);

}

int connectib_icmd_detect_gvmi_in_qpc_and_icmc_size(void)
{
	 return 32;
}

void connectib_icmd_detect_gvmi_in_qpc_and_icmc_dump(const struct connectib_icmd_detect_gvmi_in_qpc_and_icmc *ptr_struct, FILE* file)
{
	connectib_icmd_detect_gvmi_in_qpc_and_icmc_print(ptr_struct, file, 0);
}

void connectib_icmd_temp_warn_gen_event_pack(const struct connectib_icmd_temp_warn_gen_event *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sensors_arr[i]);
	}

}

void connectib_icmd_temp_warn_gen_event_unpack(struct connectib_icmd_temp_warn_gen_event *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 128, 1);
	ptr_struct->sensors_arr[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_icmd_temp_warn_gen_event_print(const struct connectib_icmd_temp_warn_gen_event *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_temp_warn_gen_event ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sensors_arr_%03d     : " U32H_FMT "\n", i, ptr_struct->sensors_arr[i]);
	}

}

int connectib_icmd_temp_warn_gen_event_size(void)
{
	 return 16;
}

void connectib_icmd_temp_warn_gen_event_dump(const struct connectib_icmd_temp_warn_gen_event *ptr_struct, FILE* file)
{
	connectib_icmd_temp_warn_gen_event_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_nodnic_queue_info_pack(const struct connectib_icmd_access_register_nodnic_queue_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->nodnic_idx);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->send0_qpi);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->send1_qpi);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rcv0_qpi);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rcv1_qpi);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cqi);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->eqi);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->drop_qpi);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->status);

}

void connectib_icmd_access_register_nodnic_queue_info_unpack(struct connectib_icmd_access_register_nodnic_queue_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->nodnic_idx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->send0_qpi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->send1_qpi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->rcv0_qpi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->rcv1_qpi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->cqi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->eqi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->drop_qpi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->status = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_access_register_nodnic_queue_info_print(const struct connectib_icmd_access_register_nodnic_queue_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_nodnic_queue_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nodnic_idx           : " UH_FMT "\n", ptr_struct->nodnic_idx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "send0_qpi            : " U32H_FMT "\n", ptr_struct->send0_qpi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "send1_qpi            : " U32H_FMT "\n", ptr_struct->send1_qpi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rcv0_qpi             : " U32H_FMT "\n", ptr_struct->rcv0_qpi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rcv1_qpi             : " U32H_FMT "\n", ptr_struct->rcv1_qpi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqi                  : " U32H_FMT "\n", ptr_struct->cqi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eqi                  : " U32H_FMT "\n", ptr_struct->eqi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "drop_qpi             : " U32H_FMT "\n", ptr_struct->drop_qpi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " U32H_FMT "\n", ptr_struct->status);

}

int connectib_icmd_access_register_nodnic_queue_info_size(void)
{
	 return 64;
}

void connectib_icmd_access_register_nodnic_queue_info_dump(const struct connectib_icmd_access_register_nodnic_queue_info *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_nodnic_queue_info_print(ptr_struct, file, 0);
}

void connectib_icmd_set_get_debug_pf_mac_pack(const struct connectib_icmd_set_get_debug_pf_mac *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_id);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->host_id);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mac_high);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mac_low);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->pf_idx_in_host_port);

}

void connectib_icmd_set_get_debug_pf_mac_unpack(struct connectib_icmd_set_get_debug_pf_mac *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 56;
	ptr_struct->port_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->host_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->mac_high = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->mac_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 152;
	ptr_struct->pf_idx_in_host_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_set_get_debug_pf_mac_print(const struct connectib_icmd_set_get_debug_pf_mac *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_set_get_debug_pf_mac ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_id              : " UH_FMT "\n", ptr_struct->port_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_id              : " UH_FMT "\n", ptr_struct->host_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mac_high             : " U32H_FMT "\n", ptr_struct->mac_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mac_low              : " U32H_FMT "\n", ptr_struct->mac_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pf_idx_in_host_port  : " UH_FMT "\n", ptr_struct->pf_idx_in_host_port);

}

int connectib_icmd_set_get_debug_pf_mac_size(void)
{
	 return 32;
}

void connectib_icmd_set_get_debug_pf_mac_dump(const struct connectib_icmd_set_get_debug_pf_mac *ptr_struct, FILE* file)
{
	connectib_icmd_set_get_debug_pf_mac_print(ptr_struct, file, 0);
}

void connectib_icmd_get_pf_ctx_pack(const struct connectib_icmd_get_pf_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->portid);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vport_num);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pf_gvmi);

	offset = 256;
	connectib_vport_shadow_pack(&(ptr_struct->vport_shadow), ptr_buff + offset/8);

	offset = 512;
	connectib_vport_info_pack(&(ptr_struct->vport_info), ptr_buff + offset/8);

}

void connectib_icmd_get_pf_ctx_unpack(struct connectib_icmd_get_pf_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->portid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->vport_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 48;
	ptr_struct->pf_gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 256;
	connectib_vport_shadow_unpack(&(ptr_struct->vport_shadow), ptr_buff + offset/8);

	offset = 512;
	connectib_vport_info_unpack(&(ptr_struct->vport_info), ptr_buff + offset/8);

}

void connectib_icmd_get_pf_ctx_print(const struct connectib_icmd_get_pf_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_pf_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "portid               : " UH_FMT "\n", ptr_struct->portid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_num            : " UH_FMT "\n", ptr_struct->vport_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pf_gvmi              : " UH_FMT "\n", ptr_struct->pf_gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_shadow:\n");
	connectib_vport_shadow_print(&(ptr_struct->vport_shadow), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_info:\n");
	connectib_vport_info_print(&(ptr_struct->vport_info), file, indent_level + 1);

}

int connectib_icmd_get_pf_ctx_size(void)
{
	 return 128;
}

void connectib_icmd_get_pf_ctx_dump(const struct connectib_icmd_get_pf_ctx *ptr_struct, FILE* file)
{
	connectib_icmd_get_pf_ctx_print(ptr_struct, file, 0);
}

void connectib_icmd_ocbb_get_pci_api_pack(const struct connectib_icmd_ocbb_get_pci_api *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->func_index);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->host);

	offset = 32;
	connectib_ocbb_pci_data_pack(&(ptr_struct->output_data), ptr_buff + offset/8);

}

void connectib_icmd_ocbb_get_pci_api_unpack(struct connectib_icmd_ocbb_get_pci_api *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->func_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->host = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	connectib_ocbb_pci_data_unpack(&(ptr_struct->output_data), ptr_buff + offset/8);

}

void connectib_icmd_ocbb_get_pci_api_print(const struct connectib_icmd_ocbb_get_pci_api *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_ocbb_get_pci_api ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "func_index           : " UH_FMT "\n", ptr_struct->func_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host                 : " UH_FMT "\n", ptr_struct->host);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "output_data:\n");
	connectib_ocbb_pci_data_print(&(ptr_struct->output_data), file, indent_level + 1);

}

int connectib_icmd_ocbb_get_pci_api_size(void)
{
	 return 48;
}

void connectib_icmd_ocbb_get_pci_api_dump(const struct connectib_icmd_ocbb_get_pci_api *ptr_struct, FILE* file)
{
	connectib_icmd_ocbb_get_pci_api_print(ptr_struct, file, 0);
}

void connectib_icmd_ocbb_rndc_get_core_api_pack(const struct connectib_icmd_ocbb_rndc_get_core_api *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_ocbb_fields_pack(&(ptr_struct->output_data), ptr_buff + offset/8);

	offset = 4120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->portid);

	offset = 4096;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->host_ix);

}

void connectib_icmd_ocbb_rndc_get_core_api_unpack(struct connectib_icmd_ocbb_rndc_get_core_api *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_ocbb_fields_unpack(&(ptr_struct->output_data), ptr_buff + offset/8);

	offset = 4120;
	ptr_struct->portid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 4096;
	ptr_struct->host_ix = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_ocbb_rndc_get_core_api_print(const struct connectib_icmd_ocbb_rndc_get_core_api *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_ocbb_rndc_get_core_api ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "output_data:\n");
	connectib_ocbb_fields_print(&(ptr_struct->output_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "portid               : " UH_FMT "\n", ptr_struct->portid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_ix              : " UH_FMT "\n", ptr_struct->host_ix);

}

int connectib_icmd_ocbb_rndc_get_core_api_size(void)
{
	 return 768;
}

void connectib_icmd_ocbb_rndc_get_core_api_dump(const struct connectib_icmd_ocbb_rndc_get_core_api *ptr_struct, FILE* file)
{
	connectib_icmd_ocbb_rndc_get_core_api_print(ptr_struct, file, 0);
}

void connectib_icmd_pre_define_module_db_pack(const struct connectib_icmd_pre_define_module_db *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->cable_type);

	offset = 32;
	connectib_module_db_pre_define_st_pack(&(ptr_struct->module_database), ptr_buff + offset/8);

}

void connectib_icmd_pre_define_module_db_unpack(struct connectib_icmd_pre_define_module_db *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->module = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 21;
	ptr_struct->cable_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 32;
	connectib_module_db_pre_define_st_unpack(&(ptr_struct->module_database), ptr_buff + offset/8);

}

void connectib_icmd_pre_define_module_db_print(const struct connectib_icmd_pre_define_module_db *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_pre_define_module_db ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module               : " UH_FMT "\n", ptr_struct->module);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cable_type           : " UH_FMT "\n", ptr_struct->cable_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_database:\n");
	connectib_module_db_pre_define_st_print(&(ptr_struct->module_database), file, indent_level + 1);

}

int connectib_icmd_pre_define_module_db_size(void)
{
	 return 60;
}

void connectib_icmd_pre_define_module_db_dump(const struct connectib_icmd_pre_define_module_db *ptr_struct, FILE* file)
{
	connectib_icmd_pre_define_module_db_print(ptr_struct, file, 0);
}

void connectib_icmd_pmlp_query_pack(const struct connectib_icmd_pmlp_query *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->width);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->local_port);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module0);

	offset = 46;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lane0);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module1);

	offset = 78;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lane1);

	offset = 120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module2);

	offset = 110;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lane2);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module3);

	offset = 142;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->lane3);

}

void connectib_icmd_pmlp_query_unpack(struct connectib_icmd_pmlp_query *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->width = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 56;
	ptr_struct->module0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 46;
	ptr_struct->lane0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 88;
	ptr_struct->module1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 78;
	ptr_struct->lane1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 120;
	ptr_struct->module2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 110;
	ptr_struct->lane2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 152;
	ptr_struct->module3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 142;
	ptr_struct->lane3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_icmd_pmlp_query_print(const struct connectib_icmd_pmlp_query *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_pmlp_query ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "width                : " UH_FMT "\n", ptr_struct->width);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module0              : " UH_FMT "\n", ptr_struct->module0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane0                : " UH_FMT "\n", ptr_struct->lane0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module1              : " UH_FMT "\n", ptr_struct->module1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane1                : " UH_FMT "\n", ptr_struct->lane1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module2              : " UH_FMT "\n", ptr_struct->module2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane2                : " UH_FMT "\n", ptr_struct->lane2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module3              : " UH_FMT "\n", ptr_struct->module3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane3                : " UH_FMT "\n", ptr_struct->lane3);

}

int connectib_icmd_pmlp_query_size(void)
{
	 return 36;
}

void connectib_icmd_pmlp_query_dump(const struct connectib_icmd_pmlp_query *ptr_struct, FILE* file)
{
	connectib_icmd_pmlp_query_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fw_driver_version_pack(const struct connectib_icmd_get_fw_driver_version *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_driver_version_pack(&(ptr_struct->fw_driver_version), ptr_buff + offset/8);

	offset = 543;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->use_hix_func_num);

	offset = 512;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 568;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->func_num);

	offset = 560;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->host_ix);

}

void connectib_icmd_get_fw_driver_version_unpack(struct connectib_icmd_get_fw_driver_version *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_driver_version_unpack(&(ptr_struct->fw_driver_version), ptr_buff + offset/8);

	offset = 543;
	ptr_struct->use_hix_func_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 512;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 568;
	ptr_struct->func_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 560;
	ptr_struct->host_ix = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_get_fw_driver_version_print(const struct connectib_icmd_get_fw_driver_version *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fw_driver_version ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_driver_version:\n");
	connectib_fw_driver_version_print(&(ptr_struct->fw_driver_version), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "use_hix_func_num     : " UH_FMT "\n", ptr_struct->use_hix_func_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "func_num             : " UH_FMT "\n", ptr_struct->func_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_ix              : " UH_FMT "\n", ptr_struct->host_ix);

}

int connectib_icmd_get_fw_driver_version_size(void)
{
	 return 80;
}

void connectib_icmd_get_fw_driver_version_dump(const struct connectib_icmd_get_fw_driver_version *ptr_struct, FILE* file)
{
	connectib_icmd_get_fw_driver_version_print(ptr_struct, file, 0);
}

void connectib_icmd_set_oc_int_pack(const struct connectib_icmd_set_oc_int *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->module);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_gpio_down);

}

void connectib_icmd_set_oc_int_unpack(struct connectib_icmd_set_oc_int *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->module = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->force_gpio_down = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_set_oc_int_print(const struct connectib_icmd_set_oc_int *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_set_oc_int ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module               : " UH_FMT "\n", ptr_struct->module);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_gpio_down      : " UH_FMT "\n", ptr_struct->force_gpio_down);

}

int connectib_icmd_set_oc_int_size(void)
{
	 return 4;
}

void connectib_icmd_set_oc_int_dump(const struct connectib_icmd_set_oc_int *ptr_struct, FILE* file)
{
	connectib_icmd_set_oc_int_print(ptr_struct, file, 0);
}

void connectib_icmd_gen_assert_pack(const struct connectib_icmd_gen_assert *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->value);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->type);

}

void connectib_icmd_gen_assert_unpack(struct connectib_icmd_gen_assert *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->value = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_gen_assert_print(const struct connectib_icmd_gen_assert *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_gen_assert ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "value                : " U32H_FMT "\n", ptr_struct->value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " U32H_FMT "\n", ptr_struct->type);

}

int connectib_icmd_gen_assert_size(void)
{
	 return 8;
}

void connectib_icmd_gen_assert_dump(const struct connectib_icmd_gen_assert *ptr_struct, FILE* file)
{
	connectib_icmd_gen_assert_print(ptr_struct, file, 0);
}

void connectib_icmd_read_write_varb_ctx_pack(const struct connectib_icmd_read_write_varb_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->arb_id);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	connectib_vqos_arb_ctx_pack(&(ptr_struct->data), ptr_buff + offset/8);

	offset = 288;
	connectib_vqos_arb_ctx_pack(&(ptr_struct->mask), ptr_buff + offset/8);

}

void connectib_icmd_read_write_varb_ctx_unpack(struct connectib_icmd_read_write_varb_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->arb_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 7;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 32;
	connectib_vqos_arb_ctx_unpack(&(ptr_struct->data), ptr_buff + offset/8);

	offset = 288;
	connectib_vqos_arb_ctx_unpack(&(ptr_struct->mask), ptr_buff + offset/8);

}

void connectib_icmd_read_write_varb_ctx_print(const struct connectib_icmd_read_write_varb_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_read_write_varb_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "arb_id               : " UH_FMT "\n", ptr_struct->arb_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data:\n");
	connectib_vqos_arb_ctx_print(&(ptr_struct->data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mask:\n");
	connectib_vqos_arb_ctx_print(&(ptr_struct->mask), file, indent_level + 1);

}

int connectib_icmd_read_write_varb_ctx_size(void)
{
	 return 68;
}

void connectib_icmd_read_write_varb_ctx_dump(const struct connectib_icmd_read_write_varb_ctx *ptr_struct, FILE* file)
{
	connectib_icmd_read_write_varb_ctx_print(ptr_struct, file, 0);
}

void connectib_icmd_debug_calc_sqn_pack(const struct connectib_icmd_debug_calc_sqn *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_0_1);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->vl);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pp);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->req_res_);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->is_fw_sq);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sqn);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->calc_sqn_iterator_);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->iterations);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->skip_vl_15);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->include_sl2vl_mismatch);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->iterator_lowest_iter);

	for (i = 0; i < 89; i++) {
	offset = adb2c_calc_array_field_address(448, 64, i, 6144, 1);
	connectib_iterator_result_pack(&(ptr_struct->iterator_res[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_debug_calc_sqn_unpack(struct connectib_icmd_debug_calc_sqn *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->gvmi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->port_0_1 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->vl = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->sl = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->pp = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->req_res_ = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->is_fw_sq = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->sqn = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->calc_sqn_iterator_ = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->iterations = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->skip_vl_15 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->include_sl2vl_mismatch = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->iterator_lowest_iter = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 89; i++) {
	offset = adb2c_calc_array_field_address(448, 64, i, 6144, 1);
	connectib_iterator_result_unpack(&(ptr_struct->iterator_res[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_debug_calc_sqn_print(const struct connectib_icmd_debug_calc_sqn *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_debug_calc_sqn ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " U32H_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_0_1             : " U32H_FMT "\n", ptr_struct->port_0_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl                   : " U32H_FMT "\n", ptr_struct->vl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl                   : " U32H_FMT "\n", ptr_struct->sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pp                   : " U32H_FMT "\n", ptr_struct->pp);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "req_res_             : " U32H_FMT "\n", ptr_struct->req_res_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "is_fw_sq             : " U32H_FMT "\n", ptr_struct->is_fw_sq);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sqn                  : " U32H_FMT "\n", ptr_struct->sqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "calc_sqn_iterator_   : " U32H_FMT "\n", ptr_struct->calc_sqn_iterator_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iterations           : " U32H_FMT "\n", ptr_struct->iterations);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "skip_vl_15           : " U32H_FMT "\n", ptr_struct->skip_vl_15);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "include_sl2vl_mismatch : " U32H_FMT "\n", ptr_struct->include_sl2vl_mismatch);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iterator_lowest_iter : " U32H_FMT "\n", ptr_struct->iterator_lowest_iter);

	for (i = 0; i < 89; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iterator_res_%03d:\n", i);
	connectib_iterator_result_print(&(ptr_struct->iterator_res[i]), file, indent_level + 1);
	}

}

int connectib_icmd_debug_calc_sqn_size(void)
{
	 return 768;
}

void connectib_icmd_debug_calc_sqn_dump(const struct connectib_icmd_debug_calc_sqn *ptr_struct, FILE* file)
{
	connectib_icmd_debug_calc_sqn_print(ptr_struct, file, 0);
}

void connectib_icmd_mini_flow_activator_pack(const struct connectib_icmd_mini_flow_activator *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->type);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->freq);

}

void connectib_icmd_mini_flow_activator_unpack(struct connectib_icmd_mini_flow_activator *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->freq = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_mini_flow_activator_print(const struct connectib_icmd_mini_flow_activator *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_mini_flow_activator ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " U32H_FMT "\n", ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "freq                 : " U32H_FMT "\n", ptr_struct->freq);

}

int connectib_icmd_mini_flow_activator_size(void)
{
	 return 8;
}

void connectib_icmd_mini_flow_activator_dump(const struct connectib_icmd_mini_flow_activator *ptr_struct, FILE* file)
{
	connectib_icmd_mini_flow_activator_print(ptr_struct, file, 0);
}

void connectib_icmd_general_fault_injector_pack(const struct connectib_icmd_general_fault_injector *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->type);

	offset = 32;
	connectib_general_fault_injector_pair_pack(&(ptr_struct->config), ptr_buff + offset/8);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->param);

}

void connectib_icmd_general_fault_injector_unpack(struct connectib_icmd_general_fault_injector *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	connectib_general_fault_injector_pair_unpack(&(ptr_struct->config), ptr_buff + offset/8);

	offset = 80;
	ptr_struct->param = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_general_fault_injector_print(const struct connectib_icmd_general_fault_injector *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_general_fault_injector ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " U32H_FMT "\n", ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "config:\n");
	connectib_general_fault_injector_pair_print(&(ptr_struct->config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "param                : " UH_FMT "\n", ptr_struct->param);

}

int connectib_icmd_general_fault_injector_size(void)
{
	 return 16;
}

void connectib_icmd_general_fault_injector_dump(const struct connectib_icmd_general_fault_injector *ptr_struct, FILE* file)
{
	connectib_icmd_general_fault_injector_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_pvlc_pack(const struct connectib_icmd_access_register_pvlc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_pvlc_reg_pack(&(ptr_struct->pvlc), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pvlc_unpack(struct connectib_icmd_access_register_pvlc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_pvlc_reg_unpack(&(ptr_struct->pvlc), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pvlc_print(const struct connectib_icmd_access_register_pvlc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_pvlc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pvlc:\n");
	connectib_pvlc_reg_print(&(ptr_struct->pvlc), file, indent_level + 1);

}

int connectib_icmd_access_register_pvlc_size(void)
{
	 return 20;
}

void connectib_icmd_access_register_pvlc_dump(const struct connectib_icmd_access_register_pvlc *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_pvlc_print(ptr_struct, file, 0);
}

void connectib_icmd_parse_icm_address_pack(const struct connectib_icmd_parse_icm_address *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->addr);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gvmi);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->res_type);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->res_num);

}

void connectib_icmd_parse_icm_address_unpack(struct connectib_icmd_parse_icm_address *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->gvmi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->res_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->res_num = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_parse_icm_address_print(const struct connectib_icmd_parse_icm_address *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_parse_icm_address ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr                 : " U64H_FMT "\n", ptr_struct->addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " U32H_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "res_type             : " U32H_FMT "\n", ptr_struct->res_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "res_num              : " U32H_FMT "\n", ptr_struct->res_num);

}

int connectib_icmd_parse_icm_address_size(void)
{
	 return 64;
}

void connectib_icmd_parse_icm_address_dump(const struct connectib_icmd_parse_icm_address *ptr_struct, FILE* file)
{
	connectib_icmd_parse_icm_address_print(ptr_struct, file, 0);
}

void connectib_icmd_get_steering_info_pack(const struct connectib_icmd_get_steering_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->gvmi_active);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fpp_enable);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sriov_enable);

	offset = 59;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->portid);

	offset = 58;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ib_eth_);

	offset = 57;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->roce_en);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->gvmi_steering_enabled);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pf_gvmi);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->port_pf_gvmi);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vport_num);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->allowed_uc_list_size);

	offset = 158;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->nic_state);

	offset = 156;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->esw_state);

	offset = 155;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->connected_2_steering);

	offset = 148;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_hosts);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_pfs);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->total_vfs);

	offset = 160;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->esw_egress_info), ptr_buff + offset/8);

	offset = 192;
	connectib_dmfs_ft_type_info_pack(&(ptr_struct->esw_ingress_info), ptr_buff + offset/8);

	offset = 256;
	connectib_dmfs_data_pack(&(ptr_struct->dmfs_data), ptr_buff + offset/8);

	offset = 512;
	connectib_dmfs_gvmi_ctx_pack(&(ptr_struct->dmfs_gvmi_ctx), ptr_buff + offset/8);

}

void connectib_icmd_get_steering_info_unpack(struct connectib_icmd_get_steering_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 63;
	ptr_struct->gvmi_active = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 61;
	ptr_struct->fpp_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 60;
	ptr_struct->sriov_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 59;
	ptr_struct->portid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 58;
	ptr_struct->ib_eth_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 57;
	ptr_struct->roce_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 56;
	ptr_struct->gvmi_steering_enabled = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->pf_gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->port_pf_gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 112;
	ptr_struct->vport_num = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->allowed_uc_list_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 158;
	ptr_struct->nic_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 156;
	ptr_struct->esw_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 155;
	ptr_struct->connected_2_steering = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 148;
	ptr_struct->num_of_hosts = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 144;
	ptr_struct->num_pfs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->total_vfs = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 160;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->esw_egress_info), ptr_buff + offset/8);

	offset = 192;
	connectib_dmfs_ft_type_info_unpack(&(ptr_struct->esw_ingress_info), ptr_buff + offset/8);

	offset = 256;
	connectib_dmfs_data_unpack(&(ptr_struct->dmfs_data), ptr_buff + offset/8);

	offset = 512;
	connectib_dmfs_gvmi_ctx_unpack(&(ptr_struct->dmfs_gvmi_ctx), ptr_buff + offset/8);

}

void connectib_icmd_get_steering_info_print(const struct connectib_icmd_get_steering_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_steering_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_active          : " UH_FMT "\n", ptr_struct->gvmi_active);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fpp_enable           : " UH_FMT "\n", ptr_struct->fpp_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sriov_enable         : " UH_FMT "\n", ptr_struct->sriov_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "portid               : " UH_FMT "\n", ptr_struct->portid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ib_eth_              : " UH_FMT "\n", ptr_struct->ib_eth_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "roce_en              : " UH_FMT "\n", ptr_struct->roce_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi_steering_enabled : " UH_FMT "\n", ptr_struct->gvmi_steering_enabled);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pf_gvmi              : " UH_FMT "\n", ptr_struct->pf_gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_pf_gvmi         : " UH_FMT "\n", ptr_struct->port_pf_gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_num            : " UH_FMT "\n", ptr_struct->vport_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allowed_uc_list_size : " UH_FMT "\n", ptr_struct->allowed_uc_list_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nic_state            : " UH_FMT "\n", ptr_struct->nic_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "esw_state            : " UH_FMT "\n", ptr_struct->esw_state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "connected_2_steering : " UH_FMT "\n", ptr_struct->connected_2_steering);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_hosts         : " UH_FMT "\n", ptr_struct->num_of_hosts);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_pfs              : " UH_FMT "\n", ptr_struct->num_pfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "total_vfs            : " UH_FMT "\n", ptr_struct->total_vfs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "esw_egress_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->esw_egress_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "esw_ingress_info:\n");
	connectib_dmfs_ft_type_info_print(&(ptr_struct->esw_ingress_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmfs_data:\n");
	connectib_dmfs_data_print(&(ptr_struct->dmfs_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmfs_gvmi_ctx:\n");
	connectib_dmfs_gvmi_ctx_print(&(ptr_struct->dmfs_gvmi_ctx), file, indent_level + 1);

}

int connectib_icmd_get_steering_info_size(void)
{
	 return 128;
}

void connectib_icmd_get_steering_info_dump(const struct connectib_icmd_get_steering_info *ptr_struct, FILE* file)
{
	connectib_icmd_get_steering_info_print(ptr_struct, file, 0);
}

void connectib_icmd_read_ste_pack(const struct connectib_icmd_read_ste *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_hw_ste_pack(&(ptr_struct->data), ptr_buff + offset/8);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->index);

	offset = 560;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 544;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->locked_in_icmc);

}

void connectib_icmd_read_ste_unpack(struct connectib_icmd_read_ste *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_hw_ste_unpack(&(ptr_struct->data), ptr_buff + offset/8);

	offset = 512;
	ptr_struct->index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 560;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 544;
	ptr_struct->locked_in_icmc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_read_ste_print(const struct connectib_icmd_read_ste *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_read_ste ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data:\n");
	connectib_hw_ste_print(&(ptr_struct->data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "index                : " U32H_FMT "\n", ptr_struct->index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "locked_in_icmc       : " UH_FMT "\n", ptr_struct->locked_in_icmc);

}

int connectib_icmd_read_ste_size(void)
{
	 return 80;
}

void connectib_icmd_read_ste_dump(const struct connectib_icmd_read_ste *ptr_struct, FILE* file)
{
	connectib_icmd_read_ste_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ste_resources_list_pack(const union connectib_icmd_get_ste_resources_list *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_get_ste_resources_list_out_pack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_ste_resources_list_unpack(union connectib_icmd_get_ste_resources_list *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_get_ste_resources_list_out_unpack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_ste_resources_list_print(const union connectib_icmd_get_ste_resources_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ste_resources_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in:\n");
	connectib_icmd_get_ste_resources_list_in_print(&(ptr_struct->in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out:\n");
	connectib_icmd_get_ste_resources_list_out_print(&(ptr_struct->out), file, indent_level + 1);

}

int connectib_icmd_get_ste_resources_list_size(void)
{
	 return 256;
}

void connectib_icmd_get_ste_resources_list_dump(const union connectib_icmd_get_ste_resources_list *ptr_struct, FILE* file)
{
	connectib_icmd_get_ste_resources_list_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ste_open_resources_pack(const union connectib_icmd_get_ste_open_resources *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_get_ste_open_resources_out_pack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_ste_open_resources_unpack(union connectib_icmd_get_ste_open_resources *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_get_ste_open_resources_out_unpack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_ste_open_resources_print(const union connectib_icmd_get_ste_open_resources *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ste_open_resources ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in:\n");
	connectib_icmd_get_ste_open_resources_in_print(&(ptr_struct->in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out:\n");
	connectib_icmd_get_ste_open_resources_out_print(&(ptr_struct->out), file, indent_level + 1);

}

int connectib_icmd_get_ste_open_resources_size(void)
{
	 return 768;
}

void connectib_icmd_get_ste_open_resources_dump(const union connectib_icmd_get_ste_open_resources *ptr_struct, FILE* file)
{
	connectib_icmd_get_ste_open_resources_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fte_pack(const union connectib_icmd_get_fte *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_get_fte_out_pack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_fte_unpack(union connectib_icmd_get_fte *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_get_fte_out_unpack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_fte_print(const union connectib_icmd_get_fte *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fte ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in:\n");
	connectib_icmd_get_fte_in_print(&(ptr_struct->in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out:\n");
	connectib_icmd_get_fte_out_print(&(ptr_struct->out), file, indent_level + 1);

}

int connectib_icmd_get_fte_size(void)
{
	 return 768;
}

void connectib_icmd_get_fte_dump(const union connectib_icmd_get_fte *ptr_struct, FILE* file)
{
	connectib_icmd_get_fte_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fte_list_pack(const union connectib_icmd_get_fte_list *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_get_fte_list_out_pack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_fte_list_unpack(union connectib_icmd_get_fte_list *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_get_fte_list_out_unpack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_fte_list_print(const union connectib_icmd_get_fte_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fte_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in:\n");
	connectib_icmd_get_fte_list_in_print(&(ptr_struct->in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out:\n");
	connectib_icmd_get_fte_list_out_print(&(ptr_struct->out), file, indent_level + 1);

}

int connectib_icmd_get_fte_list_size(void)
{
	 return 16;
}

void connectib_icmd_get_fte_list_dump(const union connectib_icmd_get_fte_list *ptr_struct, FILE* file)
{
	connectib_icmd_get_fte_list_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fg_pack(const union connectib_icmd_get_fg *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_get_fg_out_pack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_fg_unpack(union connectib_icmd_get_fg *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_get_fg_out_unpack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_fg_print(const union connectib_icmd_get_fg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in:\n");
	connectib_icmd_get_fg_in_print(&(ptr_struct->in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out:\n");
	connectib_icmd_get_fg_out_print(&(ptr_struct->out), file, indent_level + 1);

}

int connectib_icmd_get_fg_size(void)
{
	 return 256;
}

void connectib_icmd_get_fg_dump(const union connectib_icmd_get_fg *ptr_struct, FILE* file)
{
	connectib_icmd_get_fg_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fg_list_pack(const union connectib_icmd_get_fg_list *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_get_fg_list_out_pack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_fg_list_unpack(union connectib_icmd_get_fg_list *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_get_fg_list_out_unpack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_fg_list_print(const union connectib_icmd_get_fg_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fg_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in:\n");
	connectib_icmd_get_fg_list_in_print(&(ptr_struct->in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out:\n");
	connectib_icmd_get_fg_list_out_print(&(ptr_struct->out), file, indent_level + 1);

}

int connectib_icmd_get_fg_list_size(void)
{
	 return 16;
}

void connectib_icmd_get_fg_list_dump(const union connectib_icmd_get_fg_list *ptr_struct, FILE* file)
{
	connectib_icmd_get_fg_list_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ft_info_pack(const union connectib_icmd_get_ft_info *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_get_ft_info_out_pack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_ft_info_unpack(union connectib_icmd_get_ft_info *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_get_ft_info_out_unpack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_ft_info_print(const union connectib_icmd_get_ft_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ft_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in:\n");
	connectib_icmd_get_ft_info_in_print(&(ptr_struct->in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out:\n");
	connectib_icmd_get_ft_info_out_print(&(ptr_struct->out), file, indent_level + 1);

}

int connectib_icmd_get_ft_info_size(void)
{
	 return 16;
}

void connectib_icmd_get_ft_info_dump(const union connectib_icmd_get_ft_info *ptr_struct, FILE* file)
{
	connectib_icmd_get_ft_info_print(ptr_struct, file, 0);
}

void connectib_icmd_get_ft_list_pack(const union connectib_icmd_get_ft_list *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_get_ft_list_out_pack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_ft_list_unpack(union connectib_icmd_get_ft_list *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_get_ft_list_out_unpack(&(ptr_struct->out), ptr_buff);
}

void connectib_icmd_get_ft_list_print(const union connectib_icmd_get_ft_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_ft_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in:\n");
	connectib_icmd_get_ft_list_in_print(&(ptr_struct->in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "out:\n");
	connectib_icmd_get_ft_list_out_print(&(ptr_struct->out), file, indent_level + 1);

}

int connectib_icmd_get_ft_list_size(void)
{
	 return 16;
}

void connectib_icmd_get_ft_list_dump(const union connectib_icmd_get_ft_list *ptr_struct, FILE* file)
{
	connectib_icmd_get_ft_list_print(ptr_struct, file, 0);
}

void connectib_icmd_led_control_mad_pack(const struct connectib_icmd_led_control_mad *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->method);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->attribute_mod);

	offset = 64;
	connectib_led_control_data_pack(&(ptr_struct->data), ptr_buff + offset/8);

}

void connectib_icmd_led_control_mad_unpack(struct connectib_icmd_led_control_mad *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->method = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->attribute_mod = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	connectib_led_control_data_unpack(&(ptr_struct->data), ptr_buff + offset/8);

}

void connectib_icmd_led_control_mad_print(const struct connectib_icmd_led_control_mad *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_led_control_mad ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "method               : " UH_FMT "\n", ptr_struct->method);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "attribute_mod        : " U32H_FMT "\n", ptr_struct->attribute_mod);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data:\n");
	connectib_led_control_data_print(&(ptr_struct->data), file, indent_level + 1);

}

int connectib_icmd_led_control_mad_size(void)
{
	 return 72;
}

void connectib_icmd_led_control_mad_dump(const struct connectib_icmd_led_control_mad *ptr_struct, FILE* file)
{
	connectib_icmd_led_control_mad_print(ptr_struct, file, 0);
}

void connectib_icmd_led_cmd_pack(const struct connectib_icmd_led_cmd *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->index);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->led_type);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->pattern);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->en);

}

void connectib_icmd_led_cmd_unpack(struct connectib_icmd_led_cmd *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 20;
	ptr_struct->led_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 0;
	ptr_struct->pattern = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 33;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_led_cmd_print(const struct connectib_icmd_led_cmd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_led_cmd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "index                : " UH_FMT "\n", ptr_struct->index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "led_type             : " UH_FMT "\n", ptr_struct->led_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pattern              : " UH_FMT "\n", ptr_struct->pattern);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "en                   : " UH_FMT "\n", ptr_struct->en);

}

int connectib_icmd_led_cmd_size(void)
{
	 return 8;
}

void connectib_icmd_led_cmd_dump(const struct connectib_icmd_led_cmd *ptr_struct, FILE* file)
{
	connectib_icmd_led_cmd_print(ptr_struct, file, 0);
}

void connectib_icmd_set_get_port_cntr_pack(const struct connectib_icmd_set_get_port_cntr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->data);

	offset = 95;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port_0_1);

	offset = 94;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->set_get_);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->cntr_ix);

}

void connectib_icmd_set_get_port_cntr_unpack(struct connectib_icmd_set_get_port_cntr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->data = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 95;
	ptr_struct->port_0_1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 94;
	ptr_struct->set_get_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->cntr_ix = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_set_get_port_cntr_print(const struct connectib_icmd_set_get_port_cntr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_set_get_port_cntr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data                 : " U64H_FMT "\n", ptr_struct->data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_0_1             : " UH_FMT "\n", ptr_struct->port_0_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_get_             : " UH_FMT "\n", ptr_struct->set_get_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cntr_ix              : " UH_FMT "\n", ptr_struct->cntr_ix);

}

int connectib_icmd_set_get_port_cntr_size(void)
{
	 return 24;
}

void connectib_icmd_set_get_port_cntr_dump(const struct connectib_icmd_set_get_port_cntr *ptr_struct, FILE* file)
{
	connectib_icmd_set_get_port_cntr_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_ppcnt_pack(const struct connectib_icmd_access_register_ppcnt *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_ppcnt_reg_pack(&(ptr_struct->ppcnt), ptr_buff + offset/8);

}

void connectib_icmd_access_register_ppcnt_unpack(struct connectib_icmd_access_register_ppcnt *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_ppcnt_reg_unpack(&(ptr_struct->ppcnt), ptr_buff + offset/8);

}

void connectib_icmd_access_register_ppcnt_print(const struct connectib_icmd_access_register_ppcnt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_ppcnt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ppcnt:\n");
	connectib_ppcnt_reg_print(&(ptr_struct->ppcnt), file, indent_level + 1);

}

int connectib_icmd_access_register_ppcnt_size(void)
{
	 return 268;
}

void connectib_icmd_access_register_ppcnt_dump(const struct connectib_icmd_access_register_ppcnt *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_ppcnt_print(ptr_struct, file, 0);
}

void connectib_icmd_measure_frequency_pack(const struct connectib_icmd_measure_frequency *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cl);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->pll);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->measured_div_quotient);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->measured_div_fraction);

}

void connectib_icmd_measure_frequency_unpack(struct connectib_icmd_measure_frequency *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->cl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 22;
	ptr_struct->pll = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 48;
	ptr_struct->measured_div_quotient = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->measured_div_fraction = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_measure_frequency_print(const struct connectib_icmd_measure_frequency *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_measure_frequency ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cl                   : " UH_FMT "\n", ptr_struct->cl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pll                  : " UH_FMT "\n", ptr_struct->pll);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measured_div_quotient : " UH_FMT "\n", ptr_struct->measured_div_quotient);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measured_div_fraction : " UH_FMT "\n", ptr_struct->measured_div_fraction);

}

int connectib_icmd_measure_frequency_size(void)
{
	 return 12;
}

void connectib_icmd_measure_frequency_dump(const struct connectib_icmd_measure_frequency *ptr_struct, FILE* file)
{
	connectib_icmd_measure_frequency_print(ptr_struct, file, 0);
}

void connectib_restore_phy_registers_pack(const struct connectib_restore_phy_registers *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->local_port);

}

void connectib_restore_phy_registers_unpack(struct connectib_restore_phy_registers *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 27;
	ptr_struct->local_port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_restore_phy_registers_print(const struct connectib_restore_phy_registers *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_restore_phy_registers ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_port           : " UH_FMT "\n", ptr_struct->local_port);

}

int connectib_restore_phy_registers_size(void)
{
	 return 32;
}

void connectib_restore_phy_registers_dump(const struct connectib_restore_phy_registers *ptr_struct, FILE* file)
{
	connectib_restore_phy_registers_print(ptr_struct, file, 0);
}

void connectib_icmd_access_reg_ptys_pack(const struct connectib_icmd_access_reg_ptys *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_ptys_reg_pack(&(ptr_struct->ptys), ptr_buff + offset/8);

}

void connectib_icmd_access_reg_ptys_unpack(struct connectib_icmd_access_reg_ptys *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_ptys_reg_unpack(&(ptr_struct->ptys), ptr_buff + offset/8);

}

void connectib_icmd_access_reg_ptys_print(const struct connectib_icmd_access_reg_ptys *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_reg_ptys ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ptys:\n");
	connectib_ptys_reg_print(&(ptr_struct->ptys), file, indent_level + 1);

}

int connectib_icmd_access_reg_ptys_size(void)
{
	 return 68;
}

void connectib_icmd_access_reg_ptys_dump(const struct connectib_icmd_access_reg_ptys *ptr_struct, FILE* file)
{
	connectib_icmd_access_reg_ptys_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_pmtu_pack(const struct connectib_icmd_access_register_pmtu *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_pmtu_reg_pack(&(ptr_struct->pmtu), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pmtu_unpack(struct connectib_icmd_access_register_pmtu *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_pmtu_reg_unpack(&(ptr_struct->pmtu), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pmtu_print(const struct connectib_icmd_access_register_pmtu *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_pmtu ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pmtu:\n");
	connectib_pmtu_reg_print(&(ptr_struct->pmtu), file, indent_level + 1);

}

int connectib_icmd_access_register_pmtu_size(void)
{
	 return 20;
}

void connectib_icmd_access_register_pmtu_dump(const struct connectib_icmd_access_register_pmtu *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_pmtu_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_mlcr_pack(const struct connectib_icmd_access_register_mlcr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_mlcr_pack(&(ptr_struct->mlcr), ptr_buff + offset/8);

}

void connectib_icmd_access_register_mlcr_unpack(struct connectib_icmd_access_register_mlcr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_mlcr_unpack(&(ptr_struct->mlcr), ptr_buff + offset/8);

}

void connectib_icmd_access_register_mlcr_print(const struct connectib_icmd_access_register_mlcr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_mlcr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlcr:\n");
	connectib_mlcr_print(&(ptr_struct->mlcr), file, indent_level + 1);

}

int connectib_icmd_access_register_mlcr_size(void)
{
	 return 16;
}

void connectib_icmd_access_register_mlcr_dump(const struct connectib_icmd_access_register_mlcr *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_mlcr_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_plpc_pack(const struct connectib_icmd_access_register_plpc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_plpc_reg_pack(&(ptr_struct->plpc), ptr_buff + offset/8);

}

void connectib_icmd_access_register_plpc_unpack(struct connectib_icmd_access_register_plpc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_plpc_reg_unpack(&(ptr_struct->plpc), ptr_buff + offset/8);

}

void connectib_icmd_access_register_plpc_print(const struct connectib_icmd_access_register_plpc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_plpc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "plpc:\n");
	connectib_plpc_reg_print(&(ptr_struct->plpc), file, indent_level + 1);

}

int connectib_icmd_access_register_plpc_size(void)
{
	 return 48;
}

void connectib_icmd_access_register_plpc_dump(const struct connectib_icmd_access_register_plpc *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_plpc_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_pplm_pack(const struct connectib_icmd_access_register_pplm *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_pplm_reg_pack(&(ptr_struct->pplm), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pplm_unpack(struct connectib_icmd_access_register_pplm *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_pplm_reg_unpack(&(ptr_struct->pplm), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pplm_print(const struct connectib_icmd_access_register_pplm *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_pplm ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pplm:\n");
	connectib_pplm_reg_print(&(ptr_struct->pplm), file, indent_level + 1);

}

int connectib_icmd_access_register_pplm_size(void)
{
	 return 48;
}

void connectib_icmd_access_register_pplm_dump(const struct connectib_icmd_access_register_pplm *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_pplm_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_pplr_pack(const struct connectib_icmd_access_register_pplr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_pplr_reg_pack(&(ptr_struct->pplr), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pplr_unpack(struct connectib_icmd_access_register_pplr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_pplr_reg_unpack(&(ptr_struct->pplr), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pplr_print(const struct connectib_icmd_access_register_pplr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_pplr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pplr:\n");
	connectib_pplr_reg_print(&(ptr_struct->pplr), file, indent_level + 1);

}

int connectib_icmd_access_register_pplr_size(void)
{
	 return 48;
}

void connectib_icmd_access_register_pplr_dump(const struct connectib_icmd_access_register_pplr *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_pplr_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_ppamp_pack(const struct connectib_icmd_access_register_ppamp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_pamp_reg_pack(&(ptr_struct->ppamp), ptr_buff + offset/8);

}

void connectib_icmd_access_register_ppamp_unpack(struct connectib_icmd_access_register_ppamp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_pamp_reg_unpack(&(ptr_struct->ppamp), ptr_buff + offset/8);

}

void connectib_icmd_access_register_ppamp_print(const struct connectib_icmd_access_register_ppamp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_ppamp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ppamp:\n");
	connectib_pamp_reg_print(&(ptr_struct->ppamp), file, indent_level + 1);

}

int connectib_icmd_access_register_ppamp_size(void)
{
	 return 48;
}

void connectib_icmd_access_register_ppamp_dump(const struct connectib_icmd_access_register_ppamp *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_ppamp_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_ptas_pack(const struct connectib_icmd_access_register_ptas *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_ptas_reg_pack(&(ptr_struct->ptas), ptr_buff + offset/8);

}

void connectib_icmd_access_register_ptas_unpack(struct connectib_icmd_access_register_ptas *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_ptas_reg_unpack(&(ptr_struct->ptas), ptr_buff + offset/8);

}

void connectib_icmd_access_register_ptas_print(const struct connectib_icmd_access_register_ptas *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_ptas ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ptas:\n");
	connectib_ptas_reg_print(&(ptr_struct->ptas), file, indent_level + 1);

}

int connectib_icmd_access_register_ptas_size(void)
{
	 return 48;
}

void connectib_icmd_access_register_ptas_dump(const struct connectib_icmd_access_register_ptas *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_ptas_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_ppaos_pack(const struct connectib_icmd_access_register_ppaos *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_ppaos_reg_pack(&(ptr_struct->ppaos), ptr_buff + offset/8);

}

void connectib_icmd_access_register_ppaos_unpack(struct connectib_icmd_access_register_ppaos *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_ppaos_reg_unpack(&(ptr_struct->ppaos), ptr_buff + offset/8);

}

void connectib_icmd_access_register_ppaos_print(const struct connectib_icmd_access_register_ppaos *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_ppaos ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ppaos:\n");
	connectib_ppaos_reg_print(&(ptr_struct->ppaos), file, indent_level + 1);

}

int connectib_icmd_access_register_ppaos_size(void)
{
	 return 20;
}

void connectib_icmd_access_register_ppaos_dump(const struct connectib_icmd_access_register_ppaos *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_ppaos_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_pprt_pack(const struct connectib_icmd_access_register_pprt *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_pprt_reg_pack(&(ptr_struct->pprt), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pprt_unpack(struct connectib_icmd_access_register_pprt *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_pprt_reg_unpack(&(ptr_struct->pprt), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pprt_print(const struct connectib_icmd_access_register_pprt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_pprt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pprt:\n");
	connectib_pprt_reg_print(&(ptr_struct->pprt), file, indent_level + 1);

}

int connectib_icmd_access_register_pprt_size(void)
{
	 return 44;
}

void connectib_icmd_access_register_pprt_dump(const struct connectib_icmd_access_register_pprt *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_pprt_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_pptt_pack(const struct connectib_icmd_access_register_pptt *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_pptt_reg_pack(&(ptr_struct->pptt), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pptt_unpack(struct connectib_icmd_access_register_pptt *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_pptt_reg_unpack(&(ptr_struct->pptt), ptr_buff + offset/8);

}

void connectib_icmd_access_register_pptt_print(const struct connectib_icmd_access_register_pptt *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_pptt ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pptt:\n");
	connectib_pptt_reg_print(&(ptr_struct->pptt), file, indent_level + 1);

}

int connectib_icmd_access_register_pptt_size(void)
{
	 return 32;
}

void connectib_icmd_access_register_pptt_dump(const struct connectib_icmd_access_register_pptt *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_pptt_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_slrg_pack(const struct connectib_icmd_access_register_slrg *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_slrg_reg_pack(&(ptr_struct->slrg), ptr_buff + offset/8);

}

void connectib_icmd_access_register_slrg_unpack(struct connectib_icmd_access_register_slrg *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_slrg_reg_unpack(&(ptr_struct->slrg), ptr_buff + offset/8);

}

void connectib_icmd_access_register_slrg_print(const struct connectib_icmd_access_register_slrg *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_slrg ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slrg:\n");
	connectib_slrg_reg_print(&(ptr_struct->slrg), file, indent_level + 1);

}

int connectib_icmd_access_register_slrg_size(void)
{
	 return 44;
}

void connectib_icmd_access_register_slrg_dump(const struct connectib_icmd_access_register_slrg *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_slrg_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_sltp_pack(const struct connectib_icmd_access_register_sltp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_sltp_reg_pack(&(ptr_struct->sltp), ptr_buff + offset/8);

}

void connectib_icmd_access_register_sltp_unpack(struct connectib_icmd_access_register_sltp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_sltp_reg_unpack(&(ptr_struct->sltp), ptr_buff + offset/8);

}

void connectib_icmd_access_register_sltp_print(const struct connectib_icmd_access_register_sltp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_sltp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sltp:\n");
	connectib_sltp_reg_print(&(ptr_struct->sltp), file, indent_level + 1);

}

int connectib_icmd_access_register_sltp_size(void)
{
	 return 24;
}

void connectib_icmd_access_register_sltp_dump(const struct connectib_icmd_access_register_sltp *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_sltp_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_slrp_pack(const struct connectib_icmd_access_register_slrp *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_slrp_reg_pack(&(ptr_struct->slrp), ptr_buff + offset/8);

}

void connectib_icmd_access_register_slrp_unpack(struct connectib_icmd_access_register_slrp *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_slrp_reg_unpack(&(ptr_struct->slrp), ptr_buff + offset/8);

}

void connectib_icmd_access_register_slrp_print(const struct connectib_icmd_access_register_slrp *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_slrp ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slrp:\n");
	connectib_slrp_reg_print(&(ptr_struct->slrp), file, indent_level + 1);

}

int connectib_icmd_access_register_slrp_size(void)
{
	 return 48;
}

void connectib_icmd_access_register_slrp_dump(const struct connectib_icmd_access_register_slrp *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_slrp_print(ptr_struct, file, 0);
}

void connectib_icmd_access_register_paos_pack(const struct connectib_icmd_access_register_paos *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw);

	offset = 32;
	connectib_paos_reg_pack(&(ptr_struct->paos), ptr_buff + offset/8);

}

void connectib_icmd_access_register_paos_unpack(struct connectib_icmd_access_register_paos *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_paos_reg_unpack(&(ptr_struct->paos), ptr_buff + offset/8);

}

void connectib_icmd_access_register_paos_print(const struct connectib_icmd_access_register_paos *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_register_paos ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw                   : " UH_FMT "\n", ptr_struct->rw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "paos:\n");
	connectib_paos_reg_print(&(ptr_struct->paos), file, indent_level + 1);

}

int connectib_icmd_access_register_paos_size(void)
{
	 return 20;
}

void connectib_icmd_access_register_paos_dump(const struct connectib_icmd_access_register_paos *ptr_struct, FILE* file)
{
	connectib_icmd_access_register_paos_print(ptr_struct, file, 0);
}

void connectib_icmd_enable_tools_pf_pack(const struct connectib_icmd_enable_tools_pf *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->disable_enable_);

}

void connectib_icmd_enable_tools_pf_unpack(struct connectib_icmd_enable_tools_pf *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->disable_enable_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_enable_tools_pf_print(const struct connectib_icmd_enable_tools_pf *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_enable_tools_pf ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "disable_enable_      : " UH_FMT "\n", ptr_struct->disable_enable_);

}

int connectib_icmd_enable_tools_pf_size(void)
{
	 return 4;
}

void connectib_icmd_enable_tools_pf_dump(const struct connectib_icmd_enable_tools_pf *ptr_struct, FILE* file)
{
	connectib_icmd_enable_tools_pf_print(ptr_struct, file, 0);
}

void connectib_icmd_set_toolpf_tracer_pack(const struct connectib_icmd_set_toolpf_tracer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->PA);

}

void connectib_icmd_set_toolpf_tracer_unpack(struct connectib_icmd_set_toolpf_tracer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->PA = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_set_toolpf_tracer_print(const struct connectib_icmd_set_toolpf_tracer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_set_toolpf_tracer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "PA                   : " U64H_FMT "\n", ptr_struct->PA);

}

int connectib_icmd_set_toolpf_tracer_size(void)
{
	 return 64;
}

void connectib_icmd_set_toolpf_tracer_dump(const struct connectib_icmd_set_toolpf_tracer *ptr_struct, FILE* file)
{
	connectib_icmd_set_toolpf_tracer_print(ptr_struct, file, 0);
}

void connectib_icmd_nvcfg_rw_tlv_pack(const struct connectib_icmd_nvcfg_rw_tlv *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->writer_id);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->apply);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->shadow);

	offset = 5;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ovr_en);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rd_en);

	offset = 2;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->prio);

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->read_defaults);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 32;
	connectib_nv_tlv_type_pack(&(ptr_struct->type), ptr_buff + offset/8);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data_len_bytes);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->status);

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 2176, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

}

void connectib_icmd_nvcfg_rw_tlv_unpack(struct connectib_icmd_nvcfg_rw_tlv *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->writer_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 7;
	ptr_struct->apply = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->shadow = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 5;
	ptr_struct->ovr_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 4;
	ptr_struct->rd_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2;
	ptr_struct->prio = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 1;
	ptr_struct->read_defaults = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_nv_tlv_type_unpack(&(ptr_struct->type), ptr_buff + offset/8);

	offset = 64;
	ptr_struct->data_len_bytes = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->status = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 2176, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_icmd_nvcfg_rw_tlv_print(const struct connectib_icmd_nvcfg_rw_tlv *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_nvcfg_rw_tlv ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "writer_id            : " UH_FMT "\n", ptr_struct->writer_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "apply                : " UH_FMT "\n", ptr_struct->apply);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "shadow               : " UH_FMT "\n", ptr_struct->shadow);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ovr_en               : " UH_FMT "\n", ptr_struct->ovr_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rd_en                : " UH_FMT "\n", ptr_struct->rd_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "prio                 : " UH_FMT "\n", ptr_struct->prio);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_defaults        : " UH_FMT "\n", ptr_struct->read_defaults);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type:\n");
	connectib_nv_tlv_type_print(&(ptr_struct->type), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_len_bytes       : " U32H_FMT "\n", ptr_struct->data_len_bytes);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " U32H_FMT "\n", ptr_struct->status);

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

}

int connectib_icmd_nvcfg_rw_tlv_size(void)
{
	 return 272;
}

void connectib_icmd_nvcfg_rw_tlv_dump(const struct connectib_icmd_nvcfg_rw_tlv *ptr_struct, FILE* file)
{
	connectib_icmd_nvcfg_rw_tlv_print(ptr_struct, file, 0);
}

void connectib_icmd_xport_xonfig_pack(const struct connectib_icmd_xport_xonfig *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->bitmask);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->ack_timeout);

}

void connectib_icmd_xport_xonfig_unpack(struct connectib_icmd_xport_xonfig *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->bitmask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 63;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 123;
	ptr_struct->ack_timeout = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_icmd_xport_xonfig_print(const struct connectib_icmd_xport_xonfig *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_xport_xonfig ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bitmask              : " U32H_FMT "\n", ptr_struct->bitmask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ack_timeout          : " UH_FMT "\n", ptr_struct->ack_timeout);

}

int connectib_icmd_xport_xonfig_size(void)
{
	 return 16;
}

void connectib_icmd_xport_xonfig_dump(const struct connectib_icmd_xport_xonfig *ptr_struct, FILE* file)
{
	connectib_icmd_xport_xonfig_print(ptr_struct, file, 0);
}

void connectib_icmd_get_gvmi_via_vsec_pack(const struct connectib_icmd_get_gvmi_via_vsec *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hix);

	offset = 4;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pfs_per_host);

	offset = 60;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_physical_ports);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port_pfs_per_host);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->device_id);

}

void connectib_icmd_get_gvmi_via_vsec_unpack(struct connectib_icmd_get_gvmi_via_vsec *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 8;
	ptr_struct->hix = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 4;
	ptr_struct->pfs_per_host = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 60;
	ptr_struct->num_physical_ports = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 56;
	ptr_struct->port_pfs_per_host = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->device_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_get_gvmi_via_vsec_print(const struct connectib_icmd_get_gvmi_via_vsec *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_gvmi_via_vsec ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hix                  : " UH_FMT "\n", ptr_struct->hix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pfs_per_host         : " UH_FMT "\n", ptr_struct->pfs_per_host);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_physical_ports   : " UH_FMT "\n", ptr_struct->num_physical_ports);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_pfs_per_host    : " UH_FMT "\n", ptr_struct->port_pfs_per_host);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_id            : " UH_FMT "\n", ptr_struct->device_id);

}

int connectib_icmd_get_gvmi_via_vsec_size(void)
{
	 return 16;
}

void connectib_icmd_get_gvmi_via_vsec_dump(const struct connectib_icmd_get_gvmi_via_vsec *ptr_struct, FILE* file)
{
	connectib_icmd_get_gvmi_via_vsec_print(ptr_struct, file, 0);
}

void connectib_icmd_steering_table_info_pack(const struct connectib_icmd_steering_table_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_steering_table_info_pack(&(ptr_struct->table_info), ptr_buff + offset/8);

	offset = 520;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->steering_table);

	offset = 512;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->portid);

}

void connectib_icmd_steering_table_info_unpack(struct connectib_icmd_steering_table_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_steering_table_info_unpack(&(ptr_struct->table_info), ptr_buff + offset/8);

	offset = 520;
	ptr_struct->steering_table = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 512;
	ptr_struct->portid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_steering_table_info_print(const struct connectib_icmd_steering_table_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_steering_table_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_info:\n");
	connectib_steering_table_info_print(&(ptr_struct->table_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_table       : " UH_FMT "\n", ptr_struct->steering_table);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "portid               : " UH_FMT "\n", ptr_struct->portid);

}

int connectib_icmd_steering_table_info_size(void)
{
	 return 80;
}

void connectib_icmd_steering_table_info_dump(const struct connectib_icmd_steering_table_info *ptr_struct, FILE* file)
{
	connectib_icmd_steering_table_info_print(ptr_struct, file, 0);
}

void connectib_icmd_access_steering_root_pack(const struct connectib_icmd_access_steering_root *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_hw_ste_pack(&(ptr_struct->data), ptr_buff + offset/8);

	offset = 543;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_rx_);

	offset = 542;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->portid);

	offset = 541;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mc_uc_);

	offset = 540;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->special_root);

	offset = 539;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable_disable_);

	offset = 512;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

}

void connectib_icmd_access_steering_root_unpack(struct connectib_icmd_access_steering_root *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_hw_ste_unpack(&(ptr_struct->data), ptr_buff + offset/8);

	offset = 543;
	ptr_struct->sx_rx_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 542;
	ptr_struct->portid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 541;
	ptr_struct->mc_uc_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 540;
	ptr_struct->special_root = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 539;
	ptr_struct->enable_disable_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 512;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_access_steering_root_print(const struct connectib_icmd_access_steering_root *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_steering_root ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data:\n");
	connectib_hw_ste_print(&(ptr_struct->data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_rx_               : " UH_FMT "\n", ptr_struct->sx_rx_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "portid               : " UH_FMT "\n", ptr_struct->portid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mc_uc_               : " UH_FMT "\n", ptr_struct->mc_uc_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "special_root         : " UH_FMT "\n", ptr_struct->special_root);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable_disable_      : " UH_FMT "\n", ptr_struct->enable_disable_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

}

int connectib_icmd_access_steering_root_size(void)
{
	 return 68;
}

void connectib_icmd_access_steering_root_dump(const struct connectib_icmd_access_steering_root *ptr_struct, FILE* file)
{
	connectib_icmd_access_steering_root_print(ptr_struct, file, 0);
}

void connectib_icmd_dcr_ll_next_dcr_list_pack(const struct connectib_icmd_dcr_ll_next_dcr_list *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->start_dcri);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->list_length);

	for (i = 0; i < 128; i++) {
	offset = adb2c_calc_array_field_address(2048, 32, i, 6144, 1);
	connectib_dcr_info_pack(&(ptr_struct->dcr_info[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_dcr_ll_next_dcr_list_unpack(struct connectib_icmd_dcr_ll_next_dcr_list *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->start_dcri = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 32;
	ptr_struct->list_length = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 128; i++) {
	offset = adb2c_calc_array_field_address(2048, 32, i, 6144, 1);
	connectib_dcr_info_unpack(&(ptr_struct->dcr_info[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_dcr_ll_next_dcr_list_print(const struct connectib_icmd_dcr_ll_next_dcr_list *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_dcr_ll_next_dcr_list ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "start_dcri           : " UH_FMT "\n", ptr_struct->start_dcri);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "list_length          : " U32H_FMT "\n", ptr_struct->list_length);

	for (i = 0; i < 128; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcr_info_%03d:\n", i);
	connectib_dcr_info_print(&(ptr_struct->dcr_info[i]), file, indent_level + 1);
	}

}

int connectib_icmd_dcr_ll_next_dcr_list_size(void)
{
	 return 768;
}

void connectib_icmd_dcr_ll_next_dcr_list_dump(const struct connectib_icmd_dcr_ll_next_dcr_list *ptr_struct, FILE* file)
{
	connectib_icmd_dcr_ll_next_dcr_list_print(ptr_struct, file, 0);
}

void connectib_icmd_dcr_ll_trvrs_pack(const struct connectib_icmd_dcr_ll_trvrs *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->start_dcri);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->list_length);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->last_dcri);

}

void connectib_icmd_dcr_ll_trvrs_unpack(struct connectib_icmd_dcr_ll_trvrs *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->start_dcri = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 32;
	ptr_struct->list_length = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->last_dcri = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_icmd_dcr_ll_trvrs_print(const struct connectib_icmd_dcr_ll_trvrs *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_dcr_ll_trvrs ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "start_dcri           : " UH_FMT "\n", ptr_struct->start_dcri);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "list_length          : " U32H_FMT "\n", ptr_struct->list_length);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_dcri            : " UH_FMT "\n", ptr_struct->last_dcri);

}

int connectib_icmd_dcr_ll_trvrs_size(void)
{
	 return 16;
}

void connectib_icmd_dcr_ll_trvrs_dump(const struct connectib_icmd_dcr_ll_trvrs *ptr_struct, FILE* file)
{
	connectib_icmd_dcr_ll_trvrs_print(ptr_struct, file, 0);
}

void connectib_icmd_flr_debug_pack(const struct connectib_icmd_flr_debug *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->flow_id);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->flow_data_0);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->flow_data_1);

}

void connectib_icmd_flr_debug_unpack(struct connectib_icmd_flr_debug *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->flow_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->flow_data_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->flow_data_1 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_flr_debug_print(const struct connectib_icmd_flr_debug *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_flr_debug ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_id              : " UH_FMT "\n", ptr_struct->flow_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_data_0          : " U32H_FMT "\n", ptr_struct->flow_data_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_data_1          : " U32H_FMT "\n", ptr_struct->flow_data_1);

}

int connectib_icmd_flr_debug_size(void)
{
	 return 32;
}

void connectib_icmd_flr_debug_dump(const struct connectib_icmd_flr_debug *ptr_struct, FILE* file)
{
	connectib_icmd_flr_debug_print(ptr_struct, file, 0);
}

void connectib_icmd_push_db_pack(const struct connectib_icmd_push_db *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 39;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->req_res_);

}

void connectib_icmd_push_db_unpack(struct connectib_icmd_push_db *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 40;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 39;
	ptr_struct->req_res_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_push_db_print(const struct connectib_icmd_push_db *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_push_db ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "req_res_             : " UH_FMT "\n", ptr_struct->req_res_);

}

int connectib_icmd_push_db_size(void)
{
	 return 8;
}

void connectib_icmd_push_db_dump(const struct connectib_icmd_push_db *ptr_struct, FILE* file)
{
	connectib_icmd_push_db_print(ptr_struct, file, 0);
}

void connectib_icmd_ver_data_pack(const struct connectib_icmd_ver_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->opcode);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->user_data);

	offset = 128;
	connectib_icmd_ver_data_cmd_specific_pack(&(ptr_struct->cmd_specific), ptr_buff + offset/8);

}

void connectib_icmd_ver_data_unpack(struct connectib_icmd_ver_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 56;
	ptr_struct->opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->user_data = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	connectib_icmd_ver_data_cmd_specific_unpack(&(ptr_struct->cmd_specific), ptr_buff + offset/8);

}

void connectib_icmd_ver_data_print(const struct connectib_icmd_ver_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_ver_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " UH_FMT "\n", ptr_struct->opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "user_data            : " U32H_FMT "\n", ptr_struct->user_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmd_specific:\n");
	connectib_icmd_ver_data_cmd_specific_print(&(ptr_struct->cmd_specific), file, indent_level + 1);

}

int connectib_icmd_ver_data_size(void)
{
	 return 80;
}

void connectib_icmd_ver_data_dump(const struct connectib_icmd_ver_data *ptr_struct, FILE* file)
{
	connectib_icmd_ver_data_print(ptr_struct, file, 0);
}

void connectib_icmd_pcie_ver_data_pack(const struct connectib_icmd_pcie_ver_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_icmd_pcie_ver_device_pack(&(ptr_struct->device), ptr_buff + offset/8);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(128, 128, i, 512, 1);
	connectib_icmd_pcie_ver_cables_pack(&(ptr_struct->cables[i]), ptr_buff + offset/8);
	}

	offset = 384;
	connectib_icmd_pcie_ver_frequency_pack(&(ptr_struct->frequency), ptr_buff + offset/8);

}

void connectib_icmd_pcie_ver_data_unpack(struct connectib_icmd_pcie_ver_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_icmd_pcie_ver_device_unpack(&(ptr_struct->device), ptr_buff + offset/8);

	for (i = 0; i < 2; i++) {
	offset = adb2c_calc_array_field_address(128, 128, i, 512, 1);
	connectib_icmd_pcie_ver_cables_unpack(&(ptr_struct->cables[i]), ptr_buff + offset/8);
	}

	offset = 384;
	connectib_icmd_pcie_ver_frequency_unpack(&(ptr_struct->frequency), ptr_buff + offset/8);

}

void connectib_icmd_pcie_ver_data_print(const struct connectib_icmd_pcie_ver_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_pcie_ver_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device:\n");
	connectib_icmd_pcie_ver_device_print(&(ptr_struct->device), file, indent_level + 1);

	for (i = 0; i < 2; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cables_%03d:\n", i);
	connectib_icmd_pcie_ver_cables_print(&(ptr_struct->cables[i]), file, indent_level + 1);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "frequency:\n");
	connectib_icmd_pcie_ver_frequency_print(&(ptr_struct->frequency), file, indent_level + 1);

}

int connectib_icmd_pcie_ver_data_size(void)
{
	 return 64;
}

void connectib_icmd_pcie_ver_data_dump(const struct connectib_icmd_pcie_ver_data *ptr_struct, FILE* file)
{
	connectib_icmd_pcie_ver_data_print(ptr_struct, file, 0);
}

void connectib_icmd_error_flow_trigger_pack(const struct connectib_icmd_error_flow_trigger *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->type);

}

void connectib_icmd_error_flow_trigger_unpack(struct connectib_icmd_error_flow_trigger *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_error_flow_trigger_print(const struct connectib_icmd_error_flow_trigger *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_error_flow_trigger ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " UH_FMT "\n", ptr_struct->type);

}

int connectib_icmd_error_flow_trigger_size(void)
{
	 return 256;
}

void connectib_icmd_error_flow_trigger_dump(const struct connectib_icmd_error_flow_trigger *ptr_struct, FILE* file)
{
	connectib_icmd_error_flow_trigger_print(ptr_struct, file, 0);
}

void connectib_icmd_stop_toggler_pack(const struct connectib_icmd_stop_toggler *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->type);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mostly_working);

	offset = 91;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_duty_cycle);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_stress);

	offset = 279;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->xmit_hold_vl_mask);

	offset = 312;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->xmit_hold_port_num);

}

void connectib_icmd_stop_toggler_unpack(struct connectib_icmd_stop_toggler *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 63;
	ptr_struct->mostly_working = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 91;
	ptr_struct->log_duty_cycle = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 123;
	ptr_struct->log_stress = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 279;
	ptr_struct->xmit_hold_vl_mask = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 312;
	ptr_struct->xmit_hold_port_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_stop_toggler_print(const struct connectib_icmd_stop_toggler *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_stop_toggler ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " UH_FMT "\n", ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mostly_working       : " UH_FMT "\n", ptr_struct->mostly_working);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_duty_cycle       : " UH_FMT "\n", ptr_struct->log_duty_cycle);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_stress           : " UH_FMT "\n", ptr_struct->log_stress);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xmit_hold_vl_mask    : " UH_FMT "\n", ptr_struct->xmit_hold_vl_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xmit_hold_port_num   : " UH_FMT "\n", ptr_struct->xmit_hold_port_num);

}

int connectib_icmd_stop_toggler_size(void)
{
	 return 256;
}

void connectib_icmd_stop_toggler_dump(const struct connectib_icmd_stop_toggler *ptr_struct, FILE* file)
{
	connectib_icmd_stop_toggler_print(ptr_struct, file, 0);
}

void connectib_icmd_code_coverage_read_pack(const struct connectib_icmd_code_coverage_read *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2112, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

	offset = 2064;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->start_dw);

	offset = 2048;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->num_dw);

}

void connectib_icmd_code_coverage_read_unpack(struct connectib_icmd_code_coverage_read *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2112, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 2064;
	ptr_struct->start_dw = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 2048;
	ptr_struct->num_dw = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_code_coverage_read_print(const struct connectib_icmd_code_coverage_read *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_code_coverage_read ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "start_dw             : " UH_FMT "\n", ptr_struct->start_dw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_dw               : " UH_FMT "\n", ptr_struct->num_dw);

}

int connectib_icmd_code_coverage_read_size(void)
{
	 return 264;
}

void connectib_icmd_code_coverage_read_dump(const struct connectib_icmd_code_coverage_read *ptr_struct, FILE* file)
{
	connectib_icmd_code_coverage_read_print(ptr_struct, file, 0);
}

void connectib_icmd_code_coverage_control_pack(const struct connectib_icmd_code_coverage_control *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 32;
	connectib_code_coverage_control_pack(&(ptr_struct->code_coverage_control), ptr_buff + offset/8);

}

void connectib_icmd_code_coverage_control_unpack(struct connectib_icmd_code_coverage_control *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	connectib_code_coverage_control_unpack(&(ptr_struct->code_coverage_control), ptr_buff + offset/8);

}

void connectib_icmd_code_coverage_control_print(const struct connectib_icmd_code_coverage_control *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_code_coverage_control ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "code_coverage_control:\n");
	connectib_code_coverage_control_print(&(ptr_struct->code_coverage_control), file, indent_level + 1);

}

int connectib_icmd_code_coverage_control_size(void)
{
	 return 16;
}

void connectib_icmd_code_coverage_control_dump(const struct connectib_icmd_code_coverage_control *ptr_struct, FILE* file)
{
	connectib_icmd_code_coverage_control_print(ptr_struct, file, 0);
}

void connectib_icmd_esw_pack(const struct connectib_icmd_esw *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_esw_pack(&(ptr_struct->fw_esw), ptr_buff + offset/8);

	offset = 4096;
	connectib_vport_real_state_pack(&(ptr_struct->vport_real_state), ptr_buff + offset/8);

	offset = 6050;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port_is_ib);

	offset = 6049;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 6048;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->portid);

	offset = 6080;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gvmi);

	offset = 6112;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_vport);

}

void connectib_icmd_esw_unpack(struct connectib_icmd_esw *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_fw_esw_unpack(&(ptr_struct->fw_esw), ptr_buff + offset/8);

	offset = 4096;
	connectib_vport_real_state_unpack(&(ptr_struct->vport_real_state), ptr_buff + offset/8);

	offset = 6050;
	ptr_struct->port_is_ib = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6049;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6048;
	ptr_struct->portid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6080;
	ptr_struct->gvmi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 6112;
	ptr_struct->num_of_vport = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_esw_print(const struct connectib_icmd_esw *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_esw ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_esw:\n");
	connectib_fw_esw_print(&(ptr_struct->fw_esw), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vport_real_state:\n");
	connectib_vport_real_state_print(&(ptr_struct->vport_real_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_is_ib           : " UH_FMT "\n", ptr_struct->port_is_ib);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "portid               : " UH_FMT "\n", ptr_struct->portid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " U32H_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_vport         : " U32H_FMT "\n", ptr_struct->num_of_vport);

}

int connectib_icmd_esw_size(void)
{
	 return 768;
}

void connectib_icmd_esw_dump(const struct connectib_icmd_esw *ptr_struct, FILE* file)
{
	connectib_icmd_esw_print(ptr_struct, file, 0);
}

void connectib_icmd_rw_gvmix_pack(const struct connectib_icmd_rw_gvmix *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_gvmix_sub_structs_pack(&(ptr_struct->sub_structs), ptr_buff + offset/8);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gvmi);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->node_type);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ix);

}

void connectib_icmd_rw_gvmix_unpack(struct connectib_icmd_rw_gvmix *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_gvmix_sub_structs_unpack(&(ptr_struct->sub_structs), ptr_buff + offset/8);

	offset = 512;
	ptr_struct->gvmi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->node_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 576;
	ptr_struct->ix = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_rw_gvmix_print(const struct connectib_icmd_rw_gvmix *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_rw_gvmix ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sub_structs:\n");
	connectib_gvmix_sub_structs_print(&(ptr_struct->sub_structs), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " U32H_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "node_type            : " U32H_FMT "\n", ptr_struct->node_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ix                   : " U32H_FMT "\n", ptr_struct->ix);

}

int connectib_icmd_rw_gvmix_size(void)
{
	 return 128;
}

void connectib_icmd_rw_gvmix_dump(const struct connectib_icmd_rw_gvmix *ptr_struct, FILE* file)
{
	connectib_icmd_rw_gvmix_print(ptr_struct, file, 0);
}

void connectib_icmd_gen_eqe_pack(const struct connectib_icmd_gen_eqe *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->eqn);

	offset = 128;
	connectib_sw_eqe_pack(&(ptr_struct->eqe), ptr_buff + offset/8);

}

void connectib_icmd_gen_eqe_unpack(struct connectib_icmd_gen_eqe *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 56;
	ptr_struct->eqn = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 128;
	connectib_sw_eqe_unpack(&(ptr_struct->eqe), ptr_buff + offset/8);

}

void connectib_icmd_gen_eqe_print(const struct connectib_icmd_gen_eqe *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_gen_eqe ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eqn                  : " UH_FMT "\n", ptr_struct->eqn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eqe:\n");
	connectib_sw_eqe_print(&(ptr_struct->eqe), file, indent_level + 1);

}

int connectib_icmd_gen_eqe_size(void)
{
	 return 80;
}

void connectib_icmd_gen_eqe_dump(const struct connectib_icmd_gen_eqe *ptr_struct, FILE* file)
{
	connectib_icmd_gen_eqe_print(ptr_struct, file, 0);
}

void connectib_pcie_tools_pointers_pack(const struct connectib_pcie_tools_pointers *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_endpoint_data_ptr);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_timestamps_ptr);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_transaction_ptr);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_debug_equalization_debug_ptr);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_debug_ts_debug_ptr);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_debug_internal_error_debug_ptr);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_phy_gen3_equalization_ptr);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_phy_pcie_eye_centering_ptr);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_phy_first_calibration_results_ptr);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_phy_pcie_rx_adaptation_ptr);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_phy_pcie_tx_adaptation_ptr);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_phy_tracer_ptr);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pcie_pfs_ptr);

}

void connectib_pcie_tools_pointers_unpack(struct connectib_pcie_tools_pointers *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->pcie_endpoint_data_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->pcie_timestamps_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->pcie_transaction_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->pcie_debug_equalization_debug_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->pcie_debug_ts_debug_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->pcie_debug_internal_error_debug_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->pcie_phy_gen3_equalization_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->pcie_phy_pcie_eye_centering_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->pcie_phy_first_calibration_results_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->pcie_phy_pcie_rx_adaptation_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->pcie_phy_pcie_tx_adaptation_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->pcie_phy_tracer_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->pcie_pfs_ptr = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_pcie_tools_pointers_print(const struct connectib_pcie_tools_pointers *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_pcie_tools_pointers ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_endpoint_data_ptr : " U32H_FMT "\n", ptr_struct->pcie_endpoint_data_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_timestamps_ptr  : " U32H_FMT "\n", ptr_struct->pcie_timestamps_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_transaction_ptr : " U32H_FMT "\n", ptr_struct->pcie_transaction_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_debug_equalization_debug_ptr : " U32H_FMT "\n", ptr_struct->pcie_debug_equalization_debug_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_debug_ts_debug_ptr : " U32H_FMT "\n", ptr_struct->pcie_debug_ts_debug_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_debug_internal_error_debug_ptr : " U32H_FMT "\n", ptr_struct->pcie_debug_internal_error_debug_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_phy_gen3_equalization_ptr : " U32H_FMT "\n", ptr_struct->pcie_phy_gen3_equalization_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_phy_pcie_eye_centering_ptr : " U32H_FMT "\n", ptr_struct->pcie_phy_pcie_eye_centering_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_phy_first_calibration_results_ptr : " U32H_FMT "\n", ptr_struct->pcie_phy_first_calibration_results_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_phy_pcie_rx_adaptation_ptr : " U32H_FMT "\n", ptr_struct->pcie_phy_pcie_rx_adaptation_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_phy_pcie_tx_adaptation_ptr : " U32H_FMT "\n", ptr_struct->pcie_phy_pcie_tx_adaptation_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_phy_tracer_ptr  : " U32H_FMT "\n", ptr_struct->pcie_phy_tracer_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_pfs_ptr         : " U32H_FMT "\n", ptr_struct->pcie_pfs_ptr);

}

int connectib_pcie_tools_pointers_size(void)
{
	 return 64;
}

void connectib_pcie_tools_pointers_dump(const struct connectib_pcie_tools_pointers *ptr_struct, FILE* file)
{
	connectib_pcie_tools_pointers_print(ptr_struct, file, 0);
}

void connectib_icmd_phy_ver_overwrite_pack(const struct connectib_icmd_phy_ver_overwrite *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port_id);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->field_id);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->field_value);

}

void connectib_icmd_phy_ver_overwrite_unpack(struct connectib_icmd_phy_ver_overwrite *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->port_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->field_id = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->field_value = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_phy_ver_overwrite_print(const struct connectib_icmd_phy_ver_overwrite *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_phy_ver_overwrite ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_id              : " UH_FMT "\n", ptr_struct->port_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "field_id             : " UH_FMT "\n", ptr_struct->field_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "field_value          : " U32H_FMT "\n", ptr_struct->field_value);

}

int connectib_icmd_phy_ver_overwrite_size(void)
{
	 return 8;
}

void connectib_icmd_phy_ver_overwrite_dump(const struct connectib_icmd_phy_ver_overwrite *ptr_struct, FILE* file)
{
	connectib_icmd_phy_ver_overwrite_print(ptr_struct, file, 0);
}

void connectib_icmd_init_ocsd_pack(const struct connectib_icmd_init_ocsd *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->addr_hi);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->addr_low);

}

void connectib_icmd_init_ocsd_unpack(struct connectib_icmd_init_ocsd *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	ptr_struct->addr_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->addr_low = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_init_ocsd_print(const struct connectib_icmd_init_ocsd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_init_ocsd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr_hi              : " U32H_FMT "\n", ptr_struct->addr_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr_low             : " U32H_FMT "\n", ptr_struct->addr_low);

}

int connectib_icmd_init_ocsd_size(void)
{
	 return 12;
}

void connectib_icmd_init_ocsd_dump(const struct connectib_icmd_init_ocsd *ptr_struct, FILE* file)
{
	connectib_icmd_init_ocsd_print(ptr_struct, file, 0);
}

void connectib_init_ncsi_pack(const struct connectib_init_ncsi *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_bmc);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_channels);

}

void connectib_init_ncsi_unpack(struct connectib_init_ncsi *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->num_of_bmc = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->num_of_channels = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_init_ncsi_print(const struct connectib_init_ncsi *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_init_ncsi ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_bmc           : " U32H_FMT "\n", ptr_struct->num_of_bmc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_channels      : " U32H_FMT "\n", ptr_struct->num_of_channels);

}

int connectib_init_ncsi_size(void)
{
	 return 8;
}

void connectib_init_ncsi_dump(const struct connectib_init_ncsi *ptr_struct, FILE* file)
{
	connectib_init_ncsi_print(ptr_struct, file, 0);
}

void connectib_icmd_dc_config_pack(const struct connectib_icmd_dc_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->bitmask);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_dcr_lifo_size);

	offset = 155;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_dcr_hash_size);

	offset = 191;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->full_handshake_enable);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->available_lifo_size);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->force_garbage_collection_fail_threshold);

	offset = 480;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->number_of_active_ghost_busters);

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cleanup_drop_n_connects);

	offset = 544;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->cleanup_drop_n_disconnects);

	offset = 607;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ghost_buster_enable);

	offset = 624;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->max_cnak_buffer_size);

	offset = 648;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->cnak_qp_on_threshold);

	offset = 680;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->cnak_qp_off_threshold);

	offset = 712;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->dcr_lifo_size);

}

void connectib_icmd_dc_config_unpack(struct connectib_icmd_dc_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->bitmask = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 63;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 123;
	ptr_struct->log_dcr_lifo_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 155;
	ptr_struct->log_dcr_hash_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 191;
	ptr_struct->full_handshake_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 384;
	ptr_struct->available_lifo_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->force_garbage_collection_fail_threshold = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 480;
	ptr_struct->number_of_active_ghost_busters = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 512;
	ptr_struct->cleanup_drop_n_connects = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 544;
	ptr_struct->cleanup_drop_n_disconnects = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 607;
	ptr_struct->ghost_buster_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 624;
	ptr_struct->max_cnak_buffer_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 648;
	ptr_struct->cnak_qp_on_threshold = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 680;
	ptr_struct->cnak_qp_off_threshold = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 712;
	ptr_struct->dcr_lifo_size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_icmd_dc_config_print(const struct connectib_icmd_dc_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_dc_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "bitmask              : " U32H_FMT "\n", ptr_struct->bitmask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_dcr_lifo_size    : " UH_FMT "\n", ptr_struct->log_dcr_lifo_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_dcr_hash_size    : " UH_FMT "\n", ptr_struct->log_dcr_hash_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "full_handshake_enable : " UH_FMT "\n", ptr_struct->full_handshake_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "available_lifo_size  : " U32H_FMT "\n", ptr_struct->available_lifo_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_garbage_collection_fail_threshold : " U32H_FMT "\n", ptr_struct->force_garbage_collection_fail_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "number_of_active_ghost_busters : " U32H_FMT "\n", ptr_struct->number_of_active_ghost_busters);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cleanup_drop_n_connects : " U32H_FMT "\n", ptr_struct->cleanup_drop_n_connects);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cleanup_drop_n_disconnects : " U32H_FMT "\n", ptr_struct->cleanup_drop_n_disconnects);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ghost_buster_enable  : " UH_FMT "\n", ptr_struct->ghost_buster_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "max_cnak_buffer_size : " UH_FMT "\n", ptr_struct->max_cnak_buffer_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cnak_qp_on_threshold : " UH_FMT "\n", ptr_struct->cnak_qp_on_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cnak_qp_off_threshold : " UH_FMT "\n", ptr_struct->cnak_qp_off_threshold);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcr_lifo_size        : " UH_FMT "\n", ptr_struct->dcr_lifo_size);

}

int connectib_icmd_dc_config_size(void)
{
	 return 128;
}

void connectib_icmd_dc_config_dump(const struct connectib_icmd_dc_config *ptr_struct, FILE* file)
{
	connectib_icmd_dc_config_print(ptr_struct, file, 0);
}

void connectib_icmd_read_icmc_tags_and_info_pack(const struct connectib_icmd_read_icmc_tags_and_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 191; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 6144, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

	offset = 6132;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->first_set);

	offset = 6120;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->number_of_sets);

}

void connectib_icmd_read_icmc_tags_and_info_unpack(struct connectib_icmd_read_icmc_tags_and_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 191; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 6144, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 6132;
	ptr_struct->first_set = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 6120;
	ptr_struct->number_of_sets = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

}

void connectib_icmd_read_icmc_tags_and_info_print(const struct connectib_icmd_read_icmc_tags_and_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_read_icmc_tags_and_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 191; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_set            : " UH_FMT "\n", ptr_struct->first_set);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "number_of_sets       : " UH_FMT "\n", ptr_struct->number_of_sets);

}

int connectib_icmd_read_icmc_tags_and_info_size(void)
{
	 return 768;
}

void connectib_icmd_read_icmc_tags_and_info_dump(const struct connectib_icmd_read_icmc_tags_and_info *ptr_struct, FILE* file)
{
	connectib_icmd_read_icmc_tags_and_info_print(ptr_struct, file, 0);
}

void connectib_icmd_access_icm_line_pack(const struct connectib_icmd_access_icm_line *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 640, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

	offset = 512;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->addr);

}

void connectib_icmd_access_icm_line_unpack(struct connectib_icmd_access_icm_line *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 640, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 512;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 576;
	ptr_struct->addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_access_icm_line_print(const struct connectib_icmd_access_icm_line *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_icm_line ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr                 : " U64H_FMT "\n", ptr_struct->addr);

}

int connectib_icmd_access_icm_line_size(void)
{
	 return 80;
}

void connectib_icmd_access_icm_line_dump(const struct connectib_icmd_access_icm_line *ptr_struct, FILE* file)
{
	connectib_icmd_access_icm_line_print(ptr_struct, file, 0);
}

void connectib_icmd_check_sqpc_status_pack(const struct connectib_icmd_check_sqpc_status *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->result_type);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->result_diff);

}

void connectib_icmd_check_sqpc_status_unpack(struct connectib_icmd_check_sqpc_status *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->result_type = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->result_diff = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_check_sqpc_status_print(const struct connectib_icmd_check_sqpc_status *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_check_sqpc_status ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "result_type          : " UH_FMT "\n", ptr_struct->result_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "result_diff          : " U32H_FMT "\n", ptr_struct->result_diff);

}

int connectib_icmd_check_sqpc_status_size(void)
{
	 return 16;
}

void connectib_icmd_check_sqpc_status_dump(const struct connectib_icmd_check_sqpc_status *ptr_struct, FILE* file)
{
	connectib_icmd_check_sqpc_status_print(ptr_struct, file, 0);
}

void connectib_icmd_lock_unlock_resource_pack(const struct connectib_icmd_lock_unlock_resource *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_lock_tag_pack(&(ptr_struct->lock_tag), ptr_buff + offset/8);

	offset = 127;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lock_unlock_);

	offset = 117;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->irisc_ix);

	offset = 116;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->spy_en);

}

void connectib_icmd_lock_unlock_resource_unpack(struct connectib_icmd_lock_unlock_resource *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_lock_tag_unpack(&(ptr_struct->lock_tag), ptr_buff + offset/8);

	offset = 127;
	ptr_struct->lock_unlock_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 117;
	ptr_struct->irisc_ix = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 116;
	ptr_struct->spy_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_lock_unlock_resource_print(const struct connectib_icmd_lock_unlock_resource *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_lock_unlock_resource ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lock_tag:\n");
	connectib_lock_tag_print(&(ptr_struct->lock_tag), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lock_unlock_         : " UH_FMT "\n", ptr_struct->lock_unlock_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "irisc_ix             : " UH_FMT "\n", ptr_struct->irisc_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spy_en               : " UH_FMT "\n", ptr_struct->spy_en);

}

int connectib_icmd_lock_unlock_resource_size(void)
{
	 return 16;
}

void connectib_icmd_lock_unlock_resource_dump(const struct connectib_icmd_lock_unlock_resource *ptr_struct, FILE* file)
{
	connectib_icmd_lock_unlock_resource_print(ptr_struct, file, 0);
}

void connectib_icmd_generic_access_register_pack(const struct connectib_icmd_generic_access_register *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reg_access_oper_tlv_pack(&(ptr_struct->oper_tlv), ptr_buff + offset/8);

	offset = 128;
	connectib_icmd_generic_reg_access_reg_tlv_pack(&(ptr_struct->generic_reg_tlv), ptr_buff + offset/8);

}

void connectib_icmd_generic_access_register_unpack(struct connectib_icmd_generic_access_register *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reg_access_oper_tlv_unpack(&(ptr_struct->oper_tlv), ptr_buff + offset/8);

	offset = 128;
	connectib_icmd_generic_reg_access_reg_tlv_unpack(&(ptr_struct->generic_reg_tlv), ptr_buff + offset/8);

}

void connectib_icmd_generic_access_register_print(const struct connectib_icmd_generic_access_register *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_generic_access_register ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "oper_tlv:\n");
	connectib_reg_access_oper_tlv_print(&(ptr_struct->oper_tlv), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "generic_reg_tlv:\n");
	connectib_icmd_generic_reg_access_reg_tlv_print(&(ptr_struct->generic_reg_tlv), file, indent_level + 1);

}

int connectib_icmd_generic_access_register_size(void)
{
	 return 296;
}

void connectib_icmd_generic_access_register_dump(const struct connectib_icmd_generic_access_register *ptr_struct, FILE* file)
{
	connectib_icmd_generic_access_register_print(ptr_struct, file, 0);
}

void connectib_icmd_kdnet_ctrl_pack(const struct connectib_icmd_kdnet_ctrl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_icmd_kdnet_ctrl_io_pack(&(ptr_struct->io), ptr_buff + offset/8);

	offset = 32;
	connectib_nv_config_kdnet_data_pack(&(ptr_struct->data), ptr_buff + offset/8);

}

void connectib_icmd_kdnet_ctrl_unpack(struct connectib_icmd_kdnet_ctrl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_icmd_kdnet_ctrl_io_unpack(&(ptr_struct->io), ptr_buff + offset/8);

	offset = 32;
	connectib_nv_config_kdnet_data_unpack(&(ptr_struct->data), ptr_buff + offset/8);

}

void connectib_icmd_kdnet_ctrl_print(const struct connectib_icmd_kdnet_ctrl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_kdnet_ctrl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "io:\n");
	connectib_icmd_kdnet_ctrl_io_print(&(ptr_struct->io), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data:\n");
	connectib_nv_config_kdnet_data_print(&(ptr_struct->data), file, indent_level + 1);

}

int connectib_icmd_kdnet_ctrl_size(void)
{
	 return 8;
}

void connectib_icmd_kdnet_ctrl_dump(const struct connectib_icmd_kdnet_ctrl *ptr_struct, FILE* file)
{
	connectib_icmd_kdnet_ctrl_print(ptr_struct, file, 0);
}

void connectib_icmd_query_capabilities_pack(const union connectib_icmd_query_capabilities *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_query_cap_general_pack(&(ptr_struct->cap_general), ptr_buff);
}

void connectib_icmd_query_capabilities_unpack(union connectib_icmd_query_capabilities *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_query_cap_general_unpack(&(ptr_struct->cap_general), ptr_buff);
}

void connectib_icmd_query_capabilities_print(const union connectib_icmd_query_capabilities *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_query_capabilities ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_in:\n");
	connectib_icmd_query_cap_in_print(&(ptr_struct->cap_in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cap_general:\n");
	connectib_icmd_query_cap_general_print(&(ptr_struct->cap_general), file, indent_level + 1);

}

int connectib_icmd_query_capabilities_size(void)
{
	 return 8;
}

void connectib_icmd_query_capabilities_dump(const union connectib_icmd_query_capabilities *ptr_struct, FILE* file)
{
	connectib_icmd_query_capabilities_print(ptr_struct, file, 0);
}

void connectib_icmd_get_icm_ctx_addr_pack(const struct connectib_icmd_get_icm_ctx_addr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->icm_res_type);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ctx_num_hi);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ctx_num_lo);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->icm_addr_hi);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->icm_addr_lo);

}

void connectib_icmd_get_icm_ctx_addr_unpack(struct connectib_icmd_get_icm_ctx_addr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->icm_res_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->ctx_num_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->ctx_num_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->icm_addr_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->icm_addr_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_get_icm_ctx_addr_print(const struct connectib_icmd_get_icm_ctx_addr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_icm_ctx_addr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icm_res_type         : " U32H_FMT "\n", ptr_struct->icm_res_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctx_num_hi           : " U32H_FMT "\n", ptr_struct->ctx_num_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctx_num_lo           : " U32H_FMT "\n", ptr_struct->ctx_num_lo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icm_addr_hi          : " U32H_FMT "\n", ptr_struct->icm_addr_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icm_addr_lo          : " U32H_FMT "\n", ptr_struct->icm_addr_lo);

}

int connectib_icmd_get_icm_ctx_addr_size(void)
{
	 return 24;
}

void connectib_icmd_get_icm_ctx_addr_dump(const struct connectib_icmd_get_icm_ctx_addr *ptr_struct, FILE* file)
{
	connectib_icmd_get_icm_ctx_addr_print(ptr_struct, file, 0);
}

void connectib_phy_nego_ctrl_st_pack(const struct connectib_phy_nego_ctrl_st *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_link_up_port0);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->halt_fw_phy_fsm_port0);

	offset = 29;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->force_link_up_port1);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->halt_fw_phy_fsm_port1);

}

void connectib_phy_nego_ctrl_st_unpack(struct connectib_phy_nego_ctrl_st *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->force_link_up_port0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->halt_fw_phy_fsm_port0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 29;
	ptr_struct->force_link_up_port1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->halt_fw_phy_fsm_port1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_phy_nego_ctrl_st_print(const struct connectib_phy_nego_ctrl_st *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_phy_nego_ctrl_st ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_link_up_port0  : " UH_FMT "\n", ptr_struct->force_link_up_port0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "halt_fw_phy_fsm_port0 : " UH_FMT "\n", ptr_struct->halt_fw_phy_fsm_port0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "force_link_up_port1  : " UH_FMT "\n", ptr_struct->force_link_up_port1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "halt_fw_phy_fsm_port1 : " UH_FMT "\n", ptr_struct->halt_fw_phy_fsm_port1);

}

int connectib_phy_nego_ctrl_st_size(void)
{
	 return 4;
}

void connectib_phy_nego_ctrl_st_dump(const struct connectib_phy_nego_ctrl_st *ptr_struct, FILE* file)
{
	connectib_phy_nego_ctrl_st_print(ptr_struct, file, 0);
}

void connectib_eye_opening_read_st_pack(const struct connectib_eye_opening_read_st *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_num);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->lane_num);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->amplitude_high);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->amplitude_low);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->phase_high);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->phase_low);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->tx_set_chosen);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rx_set_chosen);

}

void connectib_eye_opening_read_st_unpack(struct connectib_eye_opening_read_st *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->port_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->lane_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 48;
	ptr_struct->amplitude_high = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->amplitude_low = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 80;
	ptr_struct->phase_high = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->phase_low = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 112;
	ptr_struct->tx_set_chosen = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->rx_set_chosen = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_eye_opening_read_st_print(const struct connectib_eye_opening_read_st *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eye_opening_read_st ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_num             : " UH_FMT "\n", ptr_struct->port_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lane_num             : " UH_FMT "\n", ptr_struct->lane_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "amplitude_high       : " UH_FMT "\n", ptr_struct->amplitude_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "amplitude_low        : " UH_FMT "\n", ptr_struct->amplitude_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phase_high           : " UH_FMT "\n", ptr_struct->phase_high);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phase_low            : " UH_FMT "\n", ptr_struct->phase_low);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tx_set_chosen        : " UH_FMT "\n", ptr_struct->tx_set_chosen);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_set_chosen        : " UH_FMT "\n", ptr_struct->rx_set_chosen);

}

int connectib_eye_opening_read_st_size(void)
{
	 return 16;
}

void connectib_eye_opening_read_st_dump(const struct connectib_eye_opening_read_st *ptr_struct, FILE* file)
{
	connectib_eye_opening_read_st_print(ptr_struct, file, 0);
}

void connectib_icmd_alloc_dealloc_resource_pack(const struct connectib_icmd_alloc_dealloc_resource *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->alloc_dealloc);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->icm_res_type);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->res_num_hi);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->res_num_lo);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->return_status);

}

void connectib_icmd_alloc_dealloc_resource_unpack(struct connectib_icmd_alloc_dealloc_resource *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 15;
	ptr_struct->alloc_dealloc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->icm_res_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->res_num_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->res_num_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->return_status = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_alloc_dealloc_resource_print(const struct connectib_icmd_alloc_dealloc_resource *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_alloc_dealloc_resource ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "alloc_dealloc        : " UH_FMT "\n", ptr_struct->alloc_dealloc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icm_res_type         : " U32H_FMT "\n", ptr_struct->icm_res_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "res_num_hi           : " U32H_FMT "\n", ptr_struct->res_num_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "res_num_lo           : " U32H_FMT "\n", ptr_struct->res_num_lo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "return_status        : " U32H_FMT "\n", ptr_struct->return_status);

}

int connectib_icmd_alloc_dealloc_resource_size(void)
{
	 return 24;
}

void connectib_icmd_alloc_dealloc_resource_dump(const struct connectib_icmd_alloc_dealloc_resource *ptr_struct, FILE* file)
{
	connectib_icmd_alloc_dealloc_resource_print(ptr_struct, file, 0);
}

void connectib_icmd_get_icm_res_prop_pack(const struct connectib_icmd_get_icm_res_prop *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->input_icm_type);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->table_base_hi);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->table_base_lo);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->offset);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log2_stride);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log2_size);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log2_table_size);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dummy_idx);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sw_type);

}

void connectib_icmd_get_icm_res_prop_unpack(struct connectib_icmd_get_icm_res_prop *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->input_icm_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->table_base_hi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->table_base_lo = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->offset = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->log2_stride = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->log2_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->log2_table_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->dummy_idx = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->sw_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_get_icm_res_prop_print(const struct connectib_icmd_get_icm_res_prop *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_icm_res_prop ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "input_icm_type       : " U32H_FMT "\n", ptr_struct->input_icm_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_base_hi        : " U32H_FMT "\n", ptr_struct->table_base_hi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "table_base_lo        : " U32H_FMT "\n", ptr_struct->table_base_lo);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset               : " U32H_FMT "\n", ptr_struct->offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_stride          : " U32H_FMT "\n", ptr_struct->log2_stride);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_size            : " U32H_FMT "\n", ptr_struct->log2_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log2_table_size      : " U32H_FMT "\n", ptr_struct->log2_table_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dummy_idx            : " U32H_FMT "\n", ptr_struct->dummy_idx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_type              : " U32H_FMT "\n", ptr_struct->sw_type);

}

int connectib_icmd_get_icm_res_prop_size(void)
{
	 return 36;
}

void connectib_icmd_get_icm_res_prop_dump(const struct connectib_icmd_get_icm_res_prop *ptr_struct, FILE* file)
{
	connectib_icmd_get_icm_res_prop_print(ptr_struct, file, 0);
}

void connectib_icmd_emad_mcia_pack(const struct connectib_icmd_emad_mcia *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->write_query_);

	offset = 12;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sanity_checks_sts);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->status);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->l);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->device_address);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->page_number);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->i2c_device_address);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->size);

	offset = 98;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->passwd_cap);

	offset = 97;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->passwd_clr);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->passwd_v);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->password);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(160, 32, i, 576, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword[i]);
	}

	offset = 575;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->internal_state);

}

void connectib_icmd_emad_mcia_unpack(struct connectib_icmd_emad_mcia *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->write_query_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 12;
	ptr_struct->sanity_checks_sts = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 56;
	ptr_struct->status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 40;
	ptr_struct->module = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	ptr_struct->l = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->device_address = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 72;
	ptr_struct->page_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->i2c_device_address = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 112;
	ptr_struct->size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 98;
	ptr_struct->passwd_cap = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 97;
	ptr_struct->passwd_clr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->passwd_v = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 128;
	ptr_struct->password = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(160, 32, i, 576, 1);
	ptr_struct->dword[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 575;
	ptr_struct->internal_state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_emad_mcia_print(const struct connectib_icmd_emad_mcia *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_emad_mcia ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_query_         : " UH_FMT "\n", ptr_struct->write_query_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sanity_checks_sts    : " UH_FMT "\n", ptr_struct->sanity_checks_sts);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " UH_FMT "\n", ptr_struct->status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module               : " UH_FMT "\n", ptr_struct->module);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l                    : " UH_FMT "\n", ptr_struct->l);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_address       : " UH_FMT "\n", ptr_struct->device_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "page_number          : " UH_FMT "\n", ptr_struct->page_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_device_address   : " UH_FMT "\n", ptr_struct->i2c_device_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "passwd_cap           : " UH_FMT "\n", ptr_struct->passwd_cap);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "passwd_clr           : " UH_FMT "\n", ptr_struct->passwd_clr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "passwd_v             : " UH_FMT "\n", ptr_struct->passwd_v);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "password             : " U32H_FMT "\n", ptr_struct->password);

	for (i = 0; i < 12; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_%03d           : " U32H_FMT "\n", i, ptr_struct->dword[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "internal_state       : " UH_FMT "\n", ptr_struct->internal_state);

}

int connectib_icmd_emad_mcia_size(void)
{
	 return 72;
}

void connectib_icmd_emad_mcia_dump(const struct connectib_icmd_emad_mcia *ptr_struct, FILE* file)
{
	connectib_icmd_emad_mcia_print(ptr_struct, file, 0);
}

void connectib_icmd_cable_info_pack(const struct connectib_icmd_cable_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->method);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 15, (u_int32_t)ptr_struct->status);

	offset = 184;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->module);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lock_bit);

	offset = 512;
	connectib_cable_info_pack(&(ptr_struct->data), ptr_buff + offset/8);

}

void connectib_icmd_cable_info_unpack(struct connectib_icmd_cable_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->method = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 33;
	ptr_struct->status = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 15);

	offset = 184;
	ptr_struct->module = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 160;
	ptr_struct->lock_bit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 512;
	connectib_cable_info_unpack(&(ptr_struct->data), ptr_buff + offset/8);

}

void connectib_icmd_cable_info_print(const struct connectib_icmd_cable_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_cable_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "method               : " UH_FMT "\n", ptr_struct->method);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " UH_FMT "\n", ptr_struct->status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module               : " UH_FMT "\n", ptr_struct->module);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lock_bit             : " UH_FMT "\n", ptr_struct->lock_bit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data:\n");
	connectib_cable_info_print(&(ptr_struct->data), file, indent_level + 1);

}

int connectib_icmd_cable_info_size(void)
{
	 return 128;
}

void connectib_icmd_cable_info_dump(const struct connectib_icmd_cable_info *ptr_struct, FILE* file)
{
	connectib_icmd_cable_info_print(ptr_struct, file, 0);
}

void connectib_icmd_cable_plug_out_in_pack(const struct connectib_icmd_cable_plug_out_in *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->port);

	offset = 23;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->plug_out_in);

}

void connectib_icmd_cable_plug_out_in_unpack(struct connectib_icmd_cable_plug_out_in *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 23;
	ptr_struct->plug_out_in = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_cable_plug_out_in_print(const struct connectib_icmd_cable_plug_out_in *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_cable_plug_out_in ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "plug_out_in          : " UH_FMT "\n", ptr_struct->plug_out_in);

}

int connectib_icmd_cable_plug_out_in_size(void)
{
	 return 4;
}

void connectib_icmd_cable_plug_out_in_dump(const struct connectib_icmd_cable_plug_out_in *ptr_struct, FILE* file)
{
	connectib_icmd_cable_plug_out_in_print(ptr_struct, file, 0);
}

void connectib_icmd_standby_pack(const struct connectib_icmd_standby *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->exit_enter_);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->in_standby_mode);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->last_iron_down_time_usec);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->flow_id);

}

void connectib_icmd_standby_unpack(struct connectib_icmd_standby *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 63;
	ptr_struct->exit_enter_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 88;
	ptr_struct->in_standby_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->last_iron_down_time_usec = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 144;
	ptr_struct->flow_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_standby_print(const struct connectib_icmd_standby *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_standby ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "exit_enter_          : " UH_FMT "\n", ptr_struct->exit_enter_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "in_standby_mode      : " UH_FMT "\n", ptr_struct->in_standby_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "last_iron_down_time_usec : " U32H_FMT "\n", ptr_struct->last_iron_down_time_usec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_id              : " UH_FMT "\n", ptr_struct->flow_id);

}

int connectib_icmd_standby_size(void)
{
	 return 32;
}

void connectib_icmd_standby_dump(const struct connectib_icmd_standby *ptr_struct, FILE* file)
{
	connectib_icmd_standby_print(ptr_struct, file, 0);
}

void connectib_icmd_grepper_pack(const struct connectib_icmd_grepper *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->icmd_grepper_flow);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->read_addr);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->write_addr);

	offset = 219;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_read_stride);

	offset = 214;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->log_write_stride);

	offset = 230;
	adb2c_push_bits_to_buff(ptr_buff, offset, 26, (u_int32_t)ptr_struct->line_start_offset);

	offset = 262;
	adb2c_push_bits_to_buff(ptr_buff, offset, 26, (u_int32_t)ptr_struct->iterations);

	offset = 294;
	adb2c_push_bits_to_buff(ptr_buff, offset, 26, (u_int32_t)ptr_struct->first_line_hit);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->number_of_hits);

	offset = 352;
	connectib_grepper_dwords_pack(&(ptr_struct->write_data), ptr_buff + offset/8);

	offset = 864;
	connectib_grepper_dwords_pack(&(ptr_struct->write_mask), ptr_buff + offset/8);

	offset = 1376;
	connectib_grepper_dwords_pack(&(ptr_struct->compare_data), ptr_buff + offset/8);

	offset = 2400;
	connectib_grepper_dwords_pack(&(ptr_struct->compare_mask), ptr_buff + offset/8);

}

void connectib_icmd_grepper_unpack(struct connectib_icmd_grepper *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->icmd_grepper_flow = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->read_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 128;
	ptr_struct->write_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 219;
	ptr_struct->log_read_stride = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 214;
	ptr_struct->log_write_stride = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 230;
	ptr_struct->line_start_offset = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 26);

	offset = 262;
	ptr_struct->iterations = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 26);

	offset = 294;
	ptr_struct->first_line_hit = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 26);

	offset = 320;
	ptr_struct->number_of_hits = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	connectib_grepper_dwords_unpack(&(ptr_struct->write_data), ptr_buff + offset/8);

	offset = 864;
	connectib_grepper_dwords_unpack(&(ptr_struct->write_mask), ptr_buff + offset/8);

	offset = 1376;
	connectib_grepper_dwords_unpack(&(ptr_struct->compare_data), ptr_buff + offset/8);

	offset = 2400;
	connectib_grepper_dwords_unpack(&(ptr_struct->compare_mask), ptr_buff + offset/8);

}

void connectib_icmd_grepper_print(const struct connectib_icmd_grepper *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_grepper ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icmd_grepper_flow    : " U32H_FMT "\n", ptr_struct->icmd_grepper_flow);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_addr            : " U64H_FMT "\n", ptr_struct->read_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_addr           : " U64H_FMT "\n", ptr_struct->write_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_read_stride      : " UH_FMT "\n", ptr_struct->log_read_stride);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_write_stride     : " UH_FMT "\n", ptr_struct->log_write_stride);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "line_start_offset    : " UH_FMT "\n", ptr_struct->line_start_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iterations           : " UH_FMT "\n", ptr_struct->iterations);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "first_line_hit       : " UH_FMT "\n", ptr_struct->first_line_hit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "number_of_hits       : " U32H_FMT "\n", ptr_struct->number_of_hits);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_data:\n");
	connectib_grepper_dwords_print(&(ptr_struct->write_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_mask:\n");
	connectib_grepper_dwords_print(&(ptr_struct->write_mask), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data:\n");
	connectib_grepper_dwords_print(&(ptr_struct->compare_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_mask:\n");
	connectib_grepper_dwords_print(&(ptr_struct->compare_mask), file, indent_level + 1);

}

int connectib_icmd_grepper_size(void)
{
	 return 512;
}

void connectib_icmd_grepper_dump(const struct connectib_icmd_grepper *ptr_struct, FILE* file)
{
	connectib_icmd_grepper_print(ptr_struct, file, 0);
}

void connectib_icmd_execute_fw_log_pack(const struct connectib_icmd_execute_fw_log *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_itrace_pack(&(ptr_struct->itrace), ptr_buff + offset/8);

	offset = 156;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->opcode);

	for (i = 0; i < 7; i++) {
	offset = adb2c_calc_array_field_address(160, 32, i, 384, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log_var[i]);
	}

}

void connectib_icmd_execute_fw_log_unpack(struct connectib_icmd_execute_fw_log *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_itrace_unpack(&(ptr_struct->itrace), ptr_buff + offset/8);

	offset = 156;
	ptr_struct->opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 7; i++) {
	offset = adb2c_calc_array_field_address(160, 32, i, 384, 1);
	ptr_struct->log_var[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_icmd_execute_fw_log_print(const struct connectib_icmd_execute_fw_log *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_execute_fw_log ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "itrace:\n");
	connectib_itrace_print(&(ptr_struct->itrace), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " UH_FMT "\n", ptr_struct->opcode);

	for (i = 0; i < 7; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_var_%03d         : " U32H_FMT "\n", i, ptr_struct->log_var[i]);
	}

}

int connectib_icmd_execute_fw_log_size(void)
{
	 return 48;
}

void connectib_icmd_execute_fw_log_dump(const struct connectib_icmd_execute_fw_log *ptr_struct, FILE* file)
{
	connectib_icmd_execute_fw_log_print(ptr_struct, file, 0);
}

void connectib_icmd_debug_fw_tracer_pack(const struct connectib_icmd_debug_fw_tracer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 384, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->arg[i]);
	}

	offset = 284;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_args);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_of_iterations);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->iter_delay);

}

void connectib_icmd_debug_fw_tracer_unpack(struct connectib_icmd_debug_fw_tracer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 384, 1);
	ptr_struct->arg[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 284;
	ptr_struct->num_of_args = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->num_of_iterations = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->iter_delay = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_debug_fw_tracer_print(const struct connectib_icmd_debug_fw_tracer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_debug_fw_tracer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "arg_%03d             : " U32H_FMT "\n", i, ptr_struct->arg[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_args          : " UH_FMT "\n", ptr_struct->num_of_args);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_of_iterations    : " U32H_FMT "\n", ptr_struct->num_of_iterations);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "iter_delay           : " U32H_FMT "\n", ptr_struct->iter_delay);

}

int connectib_icmd_debug_fw_tracer_size(void)
{
	 return 48;
}

void connectib_icmd_debug_fw_tracer_dump(const struct connectib_icmd_debug_fw_tracer *ptr_struct, FILE* file)
{
	connectib_icmd_debug_fw_tracer_print(ptr_struct, file, 0);
}

void connectib_icmd_get_highest_index_pack(const struct connectib_icmd_get_highest_index *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->icm_res_type);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->highest_internal_index);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->highest_icm_res_num);

}

void connectib_icmd_get_highest_index_unpack(struct connectib_icmd_get_highest_index *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->icm_res_type = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->highest_internal_index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->highest_icm_res_num = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_get_highest_index_print(const struct connectib_icmd_get_highest_index *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_highest_index ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icm_res_type         : " UH_FMT "\n", ptr_struct->icm_res_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "highest_internal_index : " U32H_FMT "\n", ptr_struct->highest_internal_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "highest_icm_res_num  : " U64H_FMT "\n", ptr_struct->highest_icm_res_num);

}

int connectib_icmd_get_highest_index_size(void)
{
	 return 32;
}

void connectib_icmd_get_highest_index_dump(const struct connectib_icmd_get_highest_index *ptr_struct, FILE* file)
{
	connectib_icmd_get_highest_index_print(ptr_struct, file, 0);
}

void connectib_icmd_get_open_resources_pack(const struct connectib_icmd_get_open_resources *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->icm_res_type);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->start_index);

	offset = 159;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->more);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->output_size);

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(256, 128, i, 6144, 1);
	connectib_index_and_length_pack(&(ptr_struct->open_resources[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_get_open_resources_unpack(struct connectib_icmd_get_open_resources *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->icm_res_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->start_index = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 159;
	ptr_struct->more = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 160;
	ptr_struct->output_size = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 32; i++) {
	offset = adb2c_calc_array_field_address(256, 128, i, 6144, 1);
	connectib_index_and_length_unpack(&(ptr_struct->open_resources[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_get_open_resources_print(const struct connectib_icmd_get_open_resources *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_open_resources ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icm_res_type         : " U32H_FMT "\n", ptr_struct->icm_res_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "start_index          : " U64H_FMT "\n", ptr_struct->start_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "more                 : " UH_FMT "\n", ptr_struct->more);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "output_size          : " U32H_FMT "\n", ptr_struct->output_size);

	for (i = 0; i < 32; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "open_resources_%03d:\n", i);
	connectib_index_and_length_print(&(ptr_struct->open_resources[i]), file, indent_level + 1);
	}

}

int connectib_icmd_get_open_resources_size(void)
{
	 return 768;
}

void connectib_icmd_get_open_resources_dump(const struct connectib_icmd_get_open_resources *ptr_struct, FILE* file)
{
	connectib_icmd_get_open_resources_print(ptr_struct, file, 0);
}

void connectib_icmd_read_icm_dummy_addr_pack(const struct connectib_icmd_read_icm_dummy_addr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->dummy_idx);

	offset = 93;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->hop_num);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->dummy_addr);

}

void connectib_icmd_read_icm_dummy_addr_unpack(struct connectib_icmd_read_icm_dummy_addr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 56;
	ptr_struct->dummy_idx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 93;
	ptr_struct->hop_num = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 128;
	ptr_struct->dummy_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_read_icm_dummy_addr_print(const struct connectib_icmd_read_icm_dummy_addr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_read_icm_dummy_addr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dummy_idx            : " UH_FMT "\n", ptr_struct->dummy_idx);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hop_num              : " UH_FMT "\n", ptr_struct->hop_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dummy_addr           : " U64H_FMT "\n", ptr_struct->dummy_addr);

}

int connectib_icmd_read_icm_dummy_addr_size(void)
{
	 return 32;
}

void connectib_icmd_read_icm_dummy_addr_dump(const struct connectib_icmd_read_icm_dummy_addr *ptr_struct, FILE* file)
{
	connectib_icmd_read_icm_dummy_addr_print(ptr_struct, file, 0);
}

void connectib_icmd_read_icm_tlb_line_pack(const struct connectib_icmd_read_icm_tlb_line *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 640, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->data[i]);
	}

	offset = 512;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->tlb_addr);

	offset = 604;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hop_number);

}

void connectib_icmd_read_icm_tlb_line_unpack(struct connectib_icmd_read_icm_tlb_line *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 64, i, 640, 1);
	ptr_struct->data[i] = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);
	}

	offset = 512;
	ptr_struct->tlb_addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 604;
	ptr_struct->hop_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_read_icm_tlb_line_print(const struct connectib_icmd_read_icm_tlb_line *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_read_icm_tlb_line ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U64H_FMT "\n", i, ptr_struct->data[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tlb_addr             : " U64H_FMT "\n", ptr_struct->tlb_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hop_number           : " UH_FMT "\n", ptr_struct->hop_number);

}

int connectib_icmd_read_icm_tlb_line_size(void)
{
	 return 80;
}

void connectib_icmd_read_icm_tlb_line_dump(const struct connectib_icmd_read_icm_tlb_line *ptr_struct, FILE* file)
{
	connectib_icmd_read_icm_tlb_line_print(ptr_struct, file, 0);
}

void connectib_icmd_config_cc_pack(const struct connectib_icmd_config_cc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->opcode);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->portid);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->cc_mode);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->sl_map);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->base);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->num_entries);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(2048, 64, i, 6144, 1);
	connectib_ca_congestion_entry_pack(&(ptr_struct->ca_congestion_entry_list[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(4112, 16, i, 6144, 1);
	connectib_cc_table_entry_pack(&(ptr_struct->ccti_entry_list[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_config_cc_unpack(struct connectib_icmd_config_cc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 30;
	ptr_struct->opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 27;
	ptr_struct->portid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 22;
	ptr_struct->cc_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 0;
	ptr_struct->sl_map = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 56;
	ptr_struct->base = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 40;
	ptr_struct->num_entries = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(2048, 64, i, 6144, 1);
	connectib_ca_congestion_entry_unpack(&(ptr_struct->ca_congestion_entry_list[i]), ptr_buff + offset/8);
	}

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(4112, 16, i, 6144, 1);
	connectib_cc_table_entry_unpack(&(ptr_struct->ccti_entry_list[i]), ptr_buff + offset/8);
	}

}

void connectib_icmd_config_cc_print(const struct connectib_icmd_config_cc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_config_cc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " UH_FMT "\n", ptr_struct->opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "portid               : " UH_FMT "\n", ptr_struct->portid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cc_mode              : " UH_FMT "\n", ptr_struct->cc_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl_map               : " UH_FMT "\n", ptr_struct->sl_map);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "base                 : " UH_FMT "\n", ptr_struct->base);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_entries          : " UH_FMT "\n", ptr_struct->num_entries);

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ca_congestion_entry_list_%03d:\n", i);
	connectib_ca_congestion_entry_print(&(ptr_struct->ca_congestion_entry_list[i]), file, indent_level + 1);
	}

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ccti_entry_list_%03d:\n", i);
	connectib_cc_table_entry_print(&(ptr_struct->ccti_entry_list[i]), file, indent_level + 1);
	}

}

int connectib_icmd_config_cc_size(void)
{
	 return 768;
}

void connectib_icmd_config_cc_dump(const struct connectib_icmd_config_cc *ptr_struct, FILE* file)
{
	connectib_icmd_config_cc_print(ptr_struct, file, 0);
}

void connectib_icmd_ud_to_raw_pack(const struct connectib_icmd_ud_to_raw *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vl15);

}

void connectib_icmd_ud_to_raw_unpack(struct connectib_icmd_ud_to_raw *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 0;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 63;
	ptr_struct->vl15 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_ud_to_raw_print(const struct connectib_icmd_ud_to_raw *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_ud_to_raw ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl15                 : " UH_FMT "\n", ptr_struct->vl15);

}

int connectib_icmd_ud_to_raw_size(void)
{
	 return 64;
}

void connectib_icmd_ud_to_raw_dump(const struct connectib_icmd_ud_to_raw *ptr_struct, FILE* file)
{
	connectib_icmd_ud_to_raw_print(ptr_struct, file, 0);
}

void connectib_icmd_update_rq_ci_pack(const struct connectib_icmd_update_rq_ci *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->context_type);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->context_num);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->new_ci);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->new_next_index);

}

void connectib_icmd_update_rq_ci_unpack(struct connectib_icmd_update_rq_ci *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->context_type = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->context_num = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 112;
	ptr_struct->new_ci = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 144;
	ptr_struct->new_next_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_update_rq_ci_print(const struct connectib_icmd_update_rq_ci *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_update_rq_ci ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "context_type         : " U32H_FMT "\n", ptr_struct->context_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "context_num          : " U32H_FMT "\n", ptr_struct->context_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "new_ci               : " UH_FMT "\n", ptr_struct->new_ci);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "new_next_index       : " UH_FMT "\n", ptr_struct->new_next_index);

}

int connectib_icmd_update_rq_ci_size(void)
{
	 return 64;
}

void connectib_icmd_update_rq_ci_dump(const struct connectib_icmd_update_rq_ci *ptr_struct, FILE* file)
{
	connectib_icmd_update_rq_ci_print(ptr_struct, file, 0);
}

void connectib_icmd_write_dc_cnak_lid_pack(const struct connectib_icmd_write_dc_cnak_lid *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->gvmi);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->port);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lid);

}

void connectib_icmd_write_dc_cnak_lid_unpack(struct connectib_icmd_write_dc_cnak_lid *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->gvmi = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 62;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 80;
	ptr_struct->lid = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_write_dc_cnak_lid_print(const struct connectib_icmd_write_dc_cnak_lid *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_write_dc_cnak_lid ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " U32H_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lid                  : " UH_FMT "\n", ptr_struct->lid);

}

int connectib_icmd_write_dc_cnak_lid_size(void)
{
	 return 64;
}

void connectib_icmd_write_dc_cnak_lid_dump(const struct connectib_icmd_write_dc_cnak_lid *ptr_struct, FILE* file)
{
	connectib_icmd_write_dc_cnak_lid_print(ptr_struct, file, 0);
}

void connectib_icmd_congestion_log_pack(const struct connectib_icmd_congestion_log *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->port);

	offset = 128;
	connectib_congestion_log_event_entry_pack(&(ptr_struct->log_entry), ptr_buff + offset/8);

}

void connectib_icmd_congestion_log_unpack(struct connectib_icmd_congestion_log *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 62;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 128;
	connectib_congestion_log_event_entry_unpack(&(ptr_struct->log_entry), ptr_buff + offset/8);

}

void connectib_icmd_congestion_log_print(const struct connectib_icmd_congestion_log *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_congestion_log ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_entry:\n");
	connectib_congestion_log_event_entry_print(&(ptr_struct->log_entry), file, indent_level + 1);

}

int connectib_icmd_congestion_log_size(void)
{
	 return 64;
}

void connectib_icmd_congestion_log_dump(const struct connectib_icmd_congestion_log *ptr_struct, FILE* file)
{
	connectib_icmd_congestion_log_print(ptr_struct, file, 0);
}

void connectib_icmd_get_port_sniffer_pack(const struct connectib_icmd_get_port_sniffer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->sx_port1);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->sx_port2);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->rx_port1);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->rx_port2);

}

void connectib_icmd_get_port_sniffer_unpack(struct connectib_icmd_get_port_sniffer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->sx_port1 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 40;
	ptr_struct->sx_port2 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 72;
	ptr_struct->rx_port1 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 104;
	ptr_struct->rx_port2 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_icmd_get_port_sniffer_print(const struct connectib_icmd_get_port_sniffer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_port_sniffer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_port1             : " UH_FMT "\n", ptr_struct->sx_port1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_port2             : " UH_FMT "\n", ptr_struct->sx_port2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_port1             : " UH_FMT "\n", ptr_struct->rx_port1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rx_port2             : " UH_FMT "\n", ptr_struct->rx_port2);

}

int connectib_icmd_get_port_sniffer_size(void)
{
	 return 16;
}

void connectib_icmd_get_port_sniffer_dump(const struct connectib_icmd_get_port_sniffer *ptr_struct, FILE* file)
{
	connectib_icmd_get_port_sniffer_print(ptr_struct, file, 0);
}

void connectib_icmd_set_port_sniffer_pack(const struct connectib_icmd_set_port_sniffer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->sx_rx_);

	offset = 47;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->attach_detach_);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->sniffer_qpn);

}

void connectib_icmd_set_port_sniffer_unpack(struct connectib_icmd_set_port_sniffer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 63;
	ptr_struct->sx_rx_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 47;
	ptr_struct->attach_detach_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 104;
	ptr_struct->sniffer_qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_icmd_set_port_sniffer_print(const struct connectib_icmd_set_port_sniffer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_set_port_sniffer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sx_rx_               : " UH_FMT "\n", ptr_struct->sx_rx_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "attach_detach_       : " UH_FMT "\n", ptr_struct->attach_detach_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sniffer_qpn          : " UH_FMT "\n", ptr_struct->sniffer_qpn);

}

int connectib_icmd_set_port_sniffer_size(void)
{
	 return 16;
}

void connectib_icmd_set_port_sniffer_dump(const struct connectib_icmd_set_port_sniffer *ptr_struct, FILE* file)
{
	connectib_icmd_set_port_sniffer_print(ptr_struct, file, 0);
}

void connectib_icmd_special_qps_pack(const struct connectib_icmd_special_qps *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_special_qps_pack(&(ptr_struct->qps), ptr_buff + offset/8);

	offset = 1248;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->host_mng_qpn);

	offset = 2064;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 2063;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->port);

}

void connectib_icmd_special_qps_unpack(struct connectib_icmd_special_qps *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_special_qps_unpack(&(ptr_struct->qps), ptr_buff + offset/8);

	offset = 1248;
	ptr_struct->host_mng_qpn = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 2064;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 2063;
	ptr_struct->port = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_special_qps_print(const struct connectib_icmd_special_qps *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_special_qps ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qps:\n");
	connectib_special_qps_print(&(ptr_struct->qps), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_mng_qpn         : " U32H_FMT "\n", ptr_struct->host_mng_qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port                 : " UH_FMT "\n", ptr_struct->port);

}

int connectib_icmd_special_qps_size(void)
{
	 return 260;
}

void connectib_icmd_special_qps_dump(const struct connectib_icmd_special_qps *ptr_struct, FILE* file)
{
	connectib_icmd_special_qps_print(ptr_struct, file, 0);
}

void connectib_eqn4type_pack(const struct connectib_eqn4type *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2080, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->eqn[i]);
	}

	offset = 2064;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

}

void connectib_eqn4type_unpack(struct connectib_eqn4type *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2080, 1);
	ptr_struct->eqn[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 2064;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_eqn4type_print(const struct connectib_eqn4type *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_eqn4type ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eqn_%03d             : " U32H_FMT "\n", i, ptr_struct->eqn[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

}

int connectib_eqn4type_size(void)
{
	 return 260;
}

void connectib_eqn4type_dump(const struct connectib_eqn4type *ptr_struct, FILE* file)
{
	connectib_eqn4type_print(ptr_struct, file, 0);
}

void connectib_icmd_access_cmdq_pack(const struct connectib_icmd_access_cmdq *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_cmdif_hdr_pack(&(ptr_struct->cmdq), ptr_buff + offset/8);

	offset = 528;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 523;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->cmd_ix);

}

void connectib_icmd_access_cmdq_unpack(struct connectib_icmd_access_cmdq *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_cmdif_hdr_unpack(&(ptr_struct->cmdq), ptr_buff + offset/8);

	offset = 528;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 523;
	ptr_struct->cmd_ix = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_icmd_access_cmdq_print(const struct connectib_icmd_access_cmdq *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_cmdq ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmdq:\n");
	connectib_cmdif_hdr_print(&(ptr_struct->cmdq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmd_ix               : " UH_FMT "\n", ptr_struct->cmd_ix);

}

int connectib_icmd_access_cmdq_size(void)
{
	 return 72;
}

void connectib_icmd_access_cmdq_dump(const struct connectib_icmd_access_cmdq *ptr_struct, FILE* file)
{
	connectib_icmd_access_cmdq_print(ptr_struct, file, 0);
}

void connectib_icmd_read_sx_wq_buffer_pack(const struct connectib_icmd_read_sx_wq_buffer *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 576, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

	offset = 539;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->ec);

	offset = 571;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->wqe_idx);

}

void connectib_icmd_read_sx_wq_buffer_unpack(struct connectib_icmd_read_sx_wq_buffer *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 576, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 539;
	ptr_struct->ec = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 571;
	ptr_struct->wqe_idx = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

}

void connectib_icmd_read_sx_wq_buffer_print(const struct connectib_icmd_read_sx_wq_buffer *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_read_sx_wq_buffer ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ec                   : " UH_FMT "\n", ptr_struct->ec);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_idx              : " UH_FMT "\n", ptr_struct->wqe_idx);

}

int connectib_icmd_read_sx_wq_buffer_size(void)
{
	 return 72;
}

void connectib_icmd_read_sx_wq_buffer_dump(const struct connectib_icmd_read_sx_wq_buffer *ptr_struct, FILE* file)
{
	connectib_icmd_read_sx_wq_buffer_print(ptr_struct, file, 0);
}

void connectib_icmd_header_capture_pack(const struct connectib_icmd_header_capture *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_icmd_header_capture_headers_pack(&(ptr_struct->headers), ptr_buff + offset/8);

	offset = 1024;
	connectib_icmd_header_capture_checks_pack(&(ptr_struct->checks), ptr_buff + offset/8);

}

void connectib_icmd_header_capture_unpack(struct connectib_icmd_header_capture *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_icmd_header_capture_headers_unpack(&(ptr_struct->headers), ptr_buff + offset/8);

	offset = 1024;
	connectib_icmd_header_capture_checks_unpack(&(ptr_struct->checks), ptr_buff + offset/8);

}

void connectib_icmd_header_capture_print(const struct connectib_icmd_header_capture *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_header_capture ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "headers:\n");
	connectib_icmd_header_capture_headers_print(&(ptr_struct->headers), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "checks:\n");
	connectib_icmd_header_capture_checks_print(&(ptr_struct->checks), file, indent_level + 1);

}

int connectib_icmd_header_capture_size(void)
{
	 return 192;
}

void connectib_icmd_header_capture_dump(const struct connectib_icmd_header_capture *ptr_struct, FILE* file)
{
	connectib_icmd_header_capture_print(ptr_struct, file, 0);
}

void connectib_print_icm_pack(const struct connectib_print_icm *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable_specific_gvmi);

	offset = 30;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->enable_specific_icm_type);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->icm_res_type);

}

void connectib_print_icm_unpack(struct connectib_print_icm *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->enable_specific_gvmi = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 30;
	ptr_struct->enable_specific_icm_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->icm_res_type = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_print_icm_print(const struct connectib_print_icm *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_print_icm ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable_specific_gvmi : " UH_FMT "\n", ptr_struct->enable_specific_gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable_specific_icm_type : " UH_FMT "\n", ptr_struct->enable_specific_icm_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icm_res_type         : " UH_FMT "\n", ptr_struct->icm_res_type);

}

int connectib_print_icm_size(void)
{
	 return 8;
}

void connectib_print_icm_dump(const struct connectib_print_icm *ptr_struct, FILE* file)
{
	connectib_print_icm_print(ptr_struct, file, 0);
}

void connectib_icmd_allowed_madifc_on_all_pfs_pack(const struct connectib_icmd_allowed_madifc_on_all_pfs *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->protection);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->enable_disable);

}

void connectib_icmd_allowed_madifc_on_all_pfs_unpack(struct connectib_icmd_allowed_madifc_on_all_pfs *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->protection = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 62;
	ptr_struct->enable_disable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_icmd_allowed_madifc_on_all_pfs_print(const struct connectib_icmd_allowed_madifc_on_all_pfs *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_allowed_madifc_on_all_pfs ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "protection           : " U32H_FMT "\n", ptr_struct->protection);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable_disable       : " UH_FMT "\n", ptr_struct->enable_disable);

}

int connectib_icmd_allowed_madifc_on_all_pfs_size(void)
{
	 return 8;
}

void connectib_icmd_allowed_madifc_on_all_pfs_dump(const struct connectib_icmd_allowed_madifc_on_all_pfs *ptr_struct, FILE* file)
{
	connectib_icmd_allowed_madifc_on_all_pfs_print(ptr_struct, file, 0);
}

void connectib_icmd_fw_reset_pack(const struct connectib_icmd_fw_reset *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->protection);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->reset_mode);

}

void connectib_icmd_fw_reset_unpack(struct connectib_icmd_fw_reset *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->protection = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 62;
	ptr_struct->reset_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

}

void connectib_icmd_fw_reset_print(const struct connectib_icmd_fw_reset *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_fw_reset ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "protection           : " U32H_FMT "\n", ptr_struct->protection);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reset_mode           : " UH_FMT "\n", ptr_struct->reset_mode);

}

int connectib_icmd_fw_reset_size(void)
{
	 return 8;
}

void connectib_icmd_fw_reset_dump(const struct connectib_icmd_fw_reset *ptr_struct, FILE* file)
{
	connectib_icmd_fw_reset_print(ptr_struct, file, 0);
}

void connectib_icmd_read_g_rse_slice_desc_pack(const struct connectib_icmd_read_g_rse_slice_desc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_g_rse_hw_decoders_pack(&(ptr_struct->desc), ptr_buff + offset/8);

	offset = 2075;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->slice);

	offset = 2068;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->array_index);

	offset = 2104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->desc_type);

}

void connectib_icmd_read_g_rse_slice_desc_unpack(struct connectib_icmd_read_g_rse_slice_desc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_g_rse_hw_decoders_unpack(&(ptr_struct->desc), ptr_buff + offset/8);

	offset = 2075;
	ptr_struct->slice = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 2068;
	ptr_struct->array_index = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 2104;
	ptr_struct->desc_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_read_g_rse_slice_desc_print(const struct connectib_icmd_read_g_rse_slice_desc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_read_g_rse_slice_desc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "desc:\n");
	connectib_g_rse_hw_decoders_print(&(ptr_struct->desc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slice                : " UH_FMT "\n", ptr_struct->slice);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "array_index          : " UH_FMT "\n", ptr_struct->array_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "desc_type            : " UH_FMT "\n", ptr_struct->desc_type);

}

int connectib_icmd_read_g_rse_slice_desc_size(void)
{
	 return 264;
}

void connectib_icmd_read_g_rse_slice_desc_dump(const struct connectib_icmd_read_g_rse_slice_desc *ptr_struct, FILE* file)
{
	connectib_icmd_read_g_rse_slice_desc_print(ptr_struct, file, 0);
}

void connectib_icmd_read_rx_slice_packet_pack(const struct connectib_icmd_read_rx_slice_packet *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 128; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 4192, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

	offset = 4116;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->packet_size);

	offset = 4155;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->slice);

	offset = 4180;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->size);

	offset = 4164;
	adb2c_push_bits_to_buff(ptr_buff, offset, 12, (u_int32_t)ptr_struct->offset_dw);

}

void connectib_icmd_read_rx_slice_packet_unpack(struct connectib_icmd_read_rx_slice_packet *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 128; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 4192, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 4116;
	ptr_struct->packet_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 4155;
	ptr_struct->slice = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 4180;
	ptr_struct->size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

	offset = 4164;
	ptr_struct->offset_dw = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 12);

}

void connectib_icmd_read_rx_slice_packet_print(const struct connectib_icmd_read_rx_slice_packet *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_read_rx_slice_packet ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 128; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "packet_size          : " UH_FMT "\n", ptr_struct->packet_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slice                : " UH_FMT "\n", ptr_struct->slice);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "offset_dw            : " UH_FMT "\n", ptr_struct->offset_dw);

}

int connectib_icmd_read_rx_slice_packet_size(void)
{
	 return 524;
}

void connectib_icmd_read_rx_slice_packet_dump(const struct connectib_icmd_read_rx_slice_packet *ptr_struct, FILE* file)
{
	connectib_icmd_read_rx_slice_packet_print(ptr_struct, file, 0);
}

void connectib_icmd_get_fw_info_pack(const struct connectib_icmd_get_fw_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_FW_VERSION_pack(&(ptr_struct->fw_version), ptr_buff + offset/8);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->hash_signature);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(184, 8, i, 288, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->psid[i]);
	}

}

void connectib_icmd_get_fw_info_unpack(struct connectib_icmd_get_fw_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_FW_VERSION_unpack(&(ptr_struct->fw_version), ptr_buff + offset/8);

	offset = 144;
	ptr_struct->hash_signature = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(184, 8, i, 288, 1);
	ptr_struct->psid[i] = (char)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	ptr_struct->psid[16] = '\0';
}

void connectib_icmd_get_fw_info_print(const struct connectib_icmd_get_fw_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_fw_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_version:\n");
	connectib_FW_VERSION_print(&(ptr_struct->fw_version), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hash_signature       : " UH_FMT "\n", ptr_struct->hash_signature);

	fprintf(file, "psid                 : \"%s\"\n", ptr_struct->psid);
}

int connectib_icmd_get_fw_info_size(void)
{
	 return 36;
}

void connectib_icmd_get_fw_info_dump(const struct connectib_icmd_get_fw_info *ptr_struct, FILE* file)
{
	connectib_icmd_get_fw_info_print(ptr_struct, file, 0);
}

void connectib_icmd_get_link_leds_pack(const struct connectib_icmd_get_link_leds *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->port_number);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->physical_link);

	offset = 55;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->logical_link);

}

void connectib_icmd_get_link_leds_unpack(struct connectib_icmd_get_link_leds *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->port_number = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 63;
	ptr_struct->physical_link = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 55;
	ptr_struct->logical_link = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_icmd_get_link_leds_print(const struct connectib_icmd_get_link_leds *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_link_leds ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_number          : " UH_FMT "\n", ptr_struct->port_number);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "physical_link        : " UH_FMT "\n", ptr_struct->physical_link);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "logical_link         : " UH_FMT "\n", ptr_struct->logical_link);

}

int connectib_icmd_get_link_leds_size(void)
{
	 return 8;
}

void connectib_icmd_get_link_leds_dump(const struct connectib_icmd_get_link_leds *ptr_struct, FILE* file)
{
	connectib_icmd_get_link_leds_print(ptr_struct, file, 0);
}

void connectib_icmd_get_boot_stage_pack(const struct connectib_icmd_get_boot_stage *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->boot_stage);

}

void connectib_icmd_get_boot_stage_unpack(struct connectib_icmd_get_boot_stage *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->boot_stage = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_get_boot_stage_print(const struct connectib_icmd_get_boot_stage *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_boot_stage ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "boot_stage           : " UH_FMT "\n", ptr_struct->boot_stage);

}

int connectib_icmd_get_boot_stage_size(void)
{
	 return 4;
}

void connectib_icmd_get_boot_stage_dump(const struct connectib_icmd_get_boot_stage *ptr_struct, FILE* file)
{
	connectib_icmd_get_boot_stage_print(ptr_struct, file, 0);
}

void connectib_icmd_get_irisc_heart_beat_pack(const struct connectib_icmd_get_irisc_heart_beat *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->irisc_heart_beat);

}

void connectib_icmd_get_irisc_heart_beat_unpack(struct connectib_icmd_get_irisc_heart_beat *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->irisc_heart_beat = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_get_irisc_heart_beat_print(const struct connectib_icmd_get_irisc_heart_beat *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_irisc_heart_beat ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "irisc_heart_beat     : " UH_FMT "\n", ptr_struct->irisc_heart_beat);

}

int connectib_icmd_get_irisc_heart_beat_size(void)
{
	 return 4;
}

void connectib_icmd_get_irisc_heart_beat_dump(const struct connectib_icmd_get_irisc_heart_beat *ptr_struct, FILE* file)
{
	connectib_icmd_get_irisc_heart_beat_print(ptr_struct, file, 0);
}

void connectib_icmd_read_q_entry_pack(const struct connectib_icmd_read_q_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 128; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 4192, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

	offset = 4112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 4136;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->q_num);

	offset = 4128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->q_type);

	offset = 4160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->index);

}

void connectib_icmd_read_q_entry_unpack(struct connectib_icmd_read_q_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 128; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 4192, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 4112;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 4136;
	ptr_struct->q_num = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 4128;
	ptr_struct->q_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 4160;
	ptr_struct->index = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_read_q_entry_print(const struct connectib_icmd_read_q_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_read_q_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 128; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "q_num                : " UH_FMT "\n", ptr_struct->q_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "q_type               : " UH_FMT "\n", ptr_struct->q_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "index                : " U32H_FMT "\n", ptr_struct->index);

}

int connectib_icmd_read_q_entry_size(void)
{
	 return 524;
}

void connectib_icmd_read_q_entry_dump(const struct connectib_icmd_read_q_entry *ptr_struct, FILE* file)
{
	connectib_icmd_read_q_entry_print(ptr_struct, file, 0);
}

void connectib_icmd_access_host_mem_pack(const struct connectib_icmd_access_host_mem *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2368, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->data[i]);
	}

	offset = 2064;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 2048;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 2080;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->num_dwords);

	offset = 2112;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->addr);

	offset = 2176;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->key_or_ptr);

	offset = 2268;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->addr_type);

	offset = 2240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->l2_block_s);

	offset = 2272;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->capi_pasid);

	offset = 2312;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->cmas_ix);

	offset = 2304;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->cmas_type);

}

void connectib_icmd_access_host_mem_unpack(struct connectib_icmd_access_host_mem *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2368, 1);
	ptr_struct->data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 2064;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 2048;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 2080;
	ptr_struct->num_dwords = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 2112;
	ptr_struct->addr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2176;
	ptr_struct->key_or_ptr = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 2268;
	ptr_struct->addr_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 2240;
	ptr_struct->l2_block_s = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 2272;
	ptr_struct->capi_pasid = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 2312;
	ptr_struct->cmas_ix = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 2304;
	ptr_struct->cmas_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_icmd_access_host_mem_print(const struct connectib_icmd_access_host_mem *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_host_mem ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "data_%03d            : " U32H_FMT "\n", i, ptr_struct->data[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "num_dwords           : " U32H_FMT "\n", ptr_struct->num_dwords);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr                 : " U64H_FMT "\n", ptr_struct->addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "key_or_ptr           : " U64H_FMT "\n", ptr_struct->key_or_ptr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "addr_type            : " UH_FMT "\n", ptr_struct->addr_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l2_block_s           : " UH_FMT "\n", ptr_struct->l2_block_s);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "capi_pasid           : " U32H_FMT "\n", ptr_struct->capi_pasid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmas_ix              : " UH_FMT "\n", ptr_struct->cmas_ix);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmas_type            : " UH_FMT "\n", ptr_struct->cmas_type);

}

int connectib_icmd_access_host_mem_size(void)
{
	 return 296;
}

void connectib_icmd_access_host_mem_dump(const struct connectib_icmd_access_host_mem *ptr_struct, FILE* file)
{
	connectib_icmd_access_host_mem_print(ptr_struct, file, 0);
}

void connectib_icmd_get_pi_ci_pack(const struct connectib_icmd_get_pi_ci *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->pi_ci);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ctx_type);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ctx_num);

}

void connectib_icmd_get_pi_ci_unpack(struct connectib_icmd_get_pi_ci *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->pi_ci = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->ctx_type = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->ctx_num = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_get_pi_ci_print(const struct connectib_icmd_get_pi_ci *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_get_pi_ci ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pi_ci                : " U32H_FMT "\n", ptr_struct->pi_ci);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctx_type             : " UH_FMT "\n", ptr_struct->ctx_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctx_num              : " U32H_FMT "\n", ptr_struct->ctx_num);

}

int connectib_icmd_get_pi_ci_size(void)
{
	 return 12;
}

void connectib_icmd_get_pi_ci_dump(const struct connectib_icmd_get_pi_ci *ptr_struct, FILE* file)
{
	connectib_icmd_get_pi_ci_print(ptr_struct, file, 0);
}

void connectib_icmd_access_icm_ctx_pack(const struct connectib_icmd_access_icm_ctx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_golan_hw_fw_ctx_pack(&(ptr_struct->ctx), ptr_buff + offset/8);

	offset = 528;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 512;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rw_);

	offset = 560;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->ctx_type);

	offset = 576;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->ctx_num);

	offset = 640;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ctx_num2);

}

void connectib_icmd_access_icm_ctx_unpack(struct connectib_icmd_access_icm_ctx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_golan_hw_fw_ctx_unpack(&(ptr_struct->ctx), ptr_buff + offset/8);

	offset = 528;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 512;
	ptr_struct->rw_ = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 560;
	ptr_struct->ctx_type = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 576;
	ptr_struct->ctx_num = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 640;
	ptr_struct->ctx_num2 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_icmd_access_icm_ctx_print(const struct connectib_icmd_access_icm_ctx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_access_icm_ctx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctx:\n");
	connectib_golan_hw_fw_ctx_print(&(ptr_struct->ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_                  : " UH_FMT "\n", ptr_struct->rw_);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctx_type             : %s (" UH_FMT ")\n", (ptr_struct->ctx_type == 95 ? (" MTT") : ((ptr_struct->ctx_type == 1 ? ("SQ_QP_LIST_REQ") : ((ptr_struct->ctx_type == 2 ? (" SQ_QP_LIST_RES") : ((ptr_struct->ctx_type == 3 ? (" BSF") : ((ptr_struct->ctx_type == 4 ? (" RDB") : ((ptr_struct->ctx_type == 5 ? (" EXT_RDB") : ((ptr_struct->ctx_type == 6 ? ("ICM_CRC") : ((ptr_struct->ctx_type == 7 ? ("QP_COMMON") : ((ptr_struct->ctx_type == 8 ? ("REQUESTOR_QPC") : ((ptr_struct->ctx_type == 9 ? (" RESPONDER_QPC") : ((ptr_struct->ctx_type == 10 ? ("MKEY") : ((ptr_struct->ctx_type == 11 ? ("SRQ") : ((ptr_struct->ctx_type == 12 ? ("CQ") : ((ptr_struct->ctx_type == 13 ? ("TRANS_STATUS_BSF") : ((ptr_struct->ctx_type == 14 ? ("TRANS_STATUS_KLM") : ((ptr_struct->ctx_type == 15 ? ("PSV") : ((ptr_struct->ctx_type == 16 ? ("EXT_PSV") : ((ptr_struct->ctx_type == 17 ? ("SXDC") : ((ptr_struct->ctx_type == 18 ? ("PIPA") : ((ptr_struct->ctx_type == 19 ? (" TIMER") : ((ptr_struct->ctx_type == 20 ? ("FW_COUNTERS") : ((ptr_struct->ctx_type == 21 ? ("EQ") : ((ptr_struct->ctx_type == 22 ? ("MSIX") : ((ptr_struct->ctx_type == 23 ? ("SXD_GVMI_RATE_LIMITER") : ((ptr_struct->ctx_type == 24 ? ("PORT0_PKEY") : ((ptr_struct->ctx_type == 25 ? ("PORT0_GUID") : ((ptr_struct->ctx_type == 26 ? ("PORT0_INFO") : ((ptr_struct->ctx_type == 27 ? ("PORT0_COUNTERS_GVMI_RX") : ((ptr_struct->ctx_type == 28 ? (" PORT1_PKEY") : ((ptr_struct->ctx_type == 29 ? ("PORT1_GUID") : ((ptr_struct->ctx_type == 30 ? ("PORT1_INFO") : ((ptr_struct->ctx_type == 31 ? ("PORT1_COUNTERS_GVMI_RX") : ((ptr_struct->ctx_type == 32 ? ("STEERING") : ((ptr_struct->ctx_type == 33 ? ("LDB_CACHE") : ((ptr_struct->ctx_type == 34 ? ("REQ_SL_CACHE") : ((ptr_struct->ctx_type == 35 ? ("IRISC") : ((ptr_struct->ctx_type == 36 ? (" SCRATCHPAD") : ((ptr_struct->ctx_type == 37 ? ("SQ_POINTERS") : ((ptr_struct->ctx_type == 38 ? ("SQ_TOKENS") : ((ptr_struct->ctx_type == 39 ? ("TOC") : ((ptr_struct->ctx_type == 40 ? ("FW_GVMI_CTX") : ((ptr_struct->ctx_type == 41 ? ("FW_QPC") : ((ptr_struct->ctx_type == 42 ? ("FW_MALLOC") : ((ptr_struct->ctx_type == 43 ? ("FW_PD") : ((ptr_struct->ctx_type == 44 ? ("FW_UAR") : ((ptr_struct->ctx_type == 45 ? ("FW_EQ") : ((ptr_struct->ctx_type == 46 ? (" FW_CQ") : ((ptr_struct->ctx_type == 47 ? ("FW_MKEY") : ((ptr_struct->ctx_type == 48 ? ("FW_SRQ") : ((ptr_struct->ctx_type == 49 ? ("FW_GLOBAL") : ((ptr_struct->ctx_type == 50 ? ("FW_SQ") : ((ptr_struct->ctx_type == 51 ? ("GLOBAL_FW_GVMI_CTX") : ((ptr_struct->ctx_type == 52 ? ("FW_ALT_PATH") : ((ptr_struct->ctx_type == 53 ? ("COUNTERS_QP_RX") : ((ptr_struct->ctx_type == 54 ? (" PORT0_COUNTERS_GVMI_SX") : ((ptr_struct->ctx_type == 55 ? ("PORT1_COUNTERS_GVMI_SX") : ((ptr_struct->ctx_type == 56 ? ("FW_LL_QP_REQ") : ((ptr_struct->ctx_type == 57 ? ("FW_LL_QP_RES") : ((ptr_struct->ctx_type == 58 ? ("FW_LL_CQ_OVERRUN") : ((ptr_struct->ctx_type == 59 ? ("FW_LL_EQ_OVERRUN_CQ") : ((ptr_struct->ctx_type == 60 ? (" FW_LL_EQ_OVERRUN_EQ") : ((ptr_struct->ctx_type == 61 ? ("EQ_OVERRUN_LIST_META") : ((ptr_struct->ctx_type == 62 ? ("ICM_RES_FW_XRCD") : ((ptr_struct->ctx_type == 63 ? ("ICM_RES_FW_FREE_LIST") : ((ptr_struct->ctx_type == 78 ? ("COUNTERS_QP_SX") : ((ptr_struct->ctx_type == 94 ? ("FW_PORT_INFO") : ("unknown")))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))), ptr_struct->ctx_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctx_num              : " U64H_FMT "\n", ptr_struct->ctx_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctx_num2             : " U32H_FMT "\n", ptr_struct->ctx_num2);

}

int connectib_icmd_access_icm_ctx_size(void)
{
	 return 96;
}

void connectib_icmd_access_icm_ctx_dump(const struct connectib_icmd_access_icm_ctx *ptr_struct, FILE* file)
{
	connectib_icmd_access_icm_ctx_print(ptr_struct, file, 0);
}

void connectib_icmd_cmd_as_dwords_pack(const struct connectib_icmd_cmd_as_dwords *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 192; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 6144, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dword[i]);
	}

}

void connectib_icmd_cmd_as_dwords_unpack(struct connectib_icmd_cmd_as_dwords *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 192; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 6144, 1);
	ptr_struct->dword[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_icmd_cmd_as_dwords_print(const struct connectib_icmd_cmd_as_dwords *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_cmd_as_dwords ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 192; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dword_%03d           : " U32H_FMT "\n", i, ptr_struct->dword[i]);
	}

}

int connectib_icmd_cmd_as_dwords_size(void)
{
	 return 768;
}

void connectib_icmd_cmd_as_dwords_dump(const struct connectib_icmd_cmd_as_dwords *ptr_struct, FILE* file)
{
	connectib_icmd_cmd_as_dwords_print(ptr_struct, file, 0);
}

void connectib_sw_cqe_64b_pack(const struct connectib_sw_cqe_64b *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_5);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_4);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->checksum);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rsvd_3);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ip_status);

	offset = 144;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->slid_smac_47_32);

	offset = 128;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vlan_ml_path);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->slid_smac_31_0);

	offset = 200;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->rqpn);

	offset = 196;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl_vlan_type);

	offset = 194;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->g);

	offset = 193;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fl);

	offset = 192;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->rsvd_bit);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rss_val);

	offset = 264;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->srq_num);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->rsvd_1);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->imm_inv_key);

	offset = 336;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->application_info);

	offset = 328;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->application_opcode);

	offset = 320;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->application);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->byte_count);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->timestamp_h);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->timestamp_l);

	offset = 456;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 448;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->swqe_opcode_drop_cntr);

	offset = 511;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->owner);

	offset = 510;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->se);

	offset = 508;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->cqe_format);

	offset = 504;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rwqe_opcode);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->signature);

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->wqe_counter);

}

void connectib_sw_cqe_64b_unpack(struct connectib_sw_cqe_64b *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->rsvd_5 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->rsvd_4 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 80;
	ptr_struct->checksum = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->rsvd_3 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->ip_status = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 144;
	ptr_struct->slid_smac_47_32 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 128;
	ptr_struct->vlan_ml_path = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 160;
	ptr_struct->slid_smac_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 200;
	ptr_struct->rqpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 196;
	ptr_struct->sl_vlan_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 194;
	ptr_struct->g = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 193;
	ptr_struct->fl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 192;
	ptr_struct->rsvd_bit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 224;
	ptr_struct->rss_val = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 264;
	ptr_struct->srq_num = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 256;
	ptr_struct->rsvd_1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 288;
	ptr_struct->imm_inv_key = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 336;
	ptr_struct->application_info = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 328;
	ptr_struct->application_opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 320;
	ptr_struct->application = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 352;
	ptr_struct->byte_count = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->timestamp_h = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->timestamp_l = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 456;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 448;
	ptr_struct->swqe_opcode_drop_cntr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 511;
	ptr_struct->owner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 510;
	ptr_struct->se = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 508;
	ptr_struct->cqe_format = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 504;
	ptr_struct->rwqe_opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 496;
	ptr_struct->signature = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 480;
	ptr_struct->wqe_counter = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_sw_cqe_64b_print(const struct connectib_sw_cqe_64b *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_cqe_64b ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_5               : " U32H_FMT "\n", ptr_struct->rsvd_5);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_4               : " U32H_FMT "\n", ptr_struct->rsvd_4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "checksum             : " UH_FMT "\n", ptr_struct->checksum);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_3               : " UH_FMT "\n", ptr_struct->rsvd_3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ip_status            : " U32H_FMT "\n", ptr_struct->ip_status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slid_smac_47_32      : " UH_FMT "\n", ptr_struct->slid_smac_47_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vlan_ml_path         : " UH_FMT "\n", ptr_struct->vlan_ml_path);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slid_smac_31_0       : " U32H_FMT "\n", ptr_struct->slid_smac_31_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rqpn                 : " UH_FMT "\n", ptr_struct->rqpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl_vlan_type         : " UH_FMT "\n", ptr_struct->sl_vlan_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "g                    : " UH_FMT "\n", ptr_struct->g);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fl                   : " UH_FMT "\n", ptr_struct->fl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_bit             : " UH_FMT "\n", ptr_struct->rsvd_bit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rss_val              : " U32H_FMT "\n", ptr_struct->rss_val);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_num              : " UH_FMT "\n", ptr_struct->srq_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_1               : " UH_FMT "\n", ptr_struct->rsvd_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "imm_inv_key          : " U32H_FMT "\n", ptr_struct->imm_inv_key);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "application_info     : " UH_FMT "\n", ptr_struct->application_info);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "application_opcode   : " UH_FMT "\n", ptr_struct->application_opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "application          : " UH_FMT "\n", ptr_struct->application);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_count           : " U32H_FMT "\n", ptr_struct->byte_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_h          : " U32H_FMT "\n", ptr_struct->timestamp_h);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_l          : " U32H_FMT "\n", ptr_struct->timestamp_l);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swqe_opcode_drop_cntr : " UH_FMT "\n", ptr_struct->swqe_opcode_drop_cntr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "owner                : " UH_FMT "\n", ptr_struct->owner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "se                   : " UH_FMT "\n", ptr_struct->se);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_format           : " UH_FMT "\n", ptr_struct->cqe_format);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_opcode          : " UH_FMT "\n", ptr_struct->rwqe_opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature            : " UH_FMT "\n", ptr_struct->signature);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_counter          : " UH_FMT "\n", ptr_struct->wqe_counter);

}

int connectib_sw_cqe_64b_size(void)
{
	 return 64;
}

void connectib_sw_cqe_64b_dump(const struct connectib_sw_cqe_64b *ptr_struct, FILE* file)
{
	connectib_sw_cqe_64b_print(ptr_struct, file, 0);
}

void connectib_grh_inline_data_pack(const struct connectib_grh_inline_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->grh_inline_data[i]);
	}

}

void connectib_grh_inline_data_unpack(struct connectib_grh_inline_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 512, 1);
	ptr_struct->grh_inline_data[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_grh_inline_data_print(const struct connectib_grh_inline_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_grh_inline_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 16; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh_inline_data_%03d : " U32H_FMT "\n", i, ptr_struct->grh_inline_data[i]);
	}

}

int connectib_grh_inline_data_size(void)
{
	 return 64;
}

void connectib_grh_inline_data_dump(const struct connectib_grh_inline_data *ptr_struct, FILE* file)
{
	connectib_grh_inline_data_print(ptr_struct, file, 0);
}

void connectib_wqe_eth_segment_pack(const struct connectib_wqe_eth_segment *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 50;
	adb2c_push_bits_to_buff(ptr_buff, offset, 14, (u_int32_t)ptr_struct->mss);

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->l3cs_inner);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->l4cs_inner);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->l3cs);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->l4cs);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->inline_headers);

	offset = 102;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->inline_header_size);

}

void connectib_wqe_eth_segment_unpack(struct connectib_wqe_eth_segment *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 50;
	ptr_struct->mss = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 14);

	offset = 35;
	ptr_struct->l3cs_inner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 34;
	ptr_struct->l4cs_inner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 33;
	ptr_struct->l3cs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->l4cs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 112;
	ptr_struct->inline_headers = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 102;
	ptr_struct->inline_header_size = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

}

void connectib_wqe_eth_segment_print(const struct connectib_wqe_eth_segment *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_eth_segment ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mss                  : " UH_FMT "\n", ptr_struct->mss);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l3cs_inner           : " UH_FMT "\n", ptr_struct->l3cs_inner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l4cs_inner           : " UH_FMT "\n", ptr_struct->l4cs_inner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l3cs                 : " UH_FMT "\n", ptr_struct->l3cs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "l4cs                 : " UH_FMT "\n", ptr_struct->l4cs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inline_headers       : " UH_FMT "\n", ptr_struct->inline_headers);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inline_header_size   : " UH_FMT "\n", ptr_struct->inline_header_size);

}

int connectib_wqe_eth_segment_size(void)
{
	 return 16;
}

void connectib_wqe_eth_segment_dump(const struct connectib_wqe_eth_segment *ptr_struct, FILE* file)
{
	connectib_wqe_eth_segment_print(ptr_struct, file, 0);
}

void connectib_wqe_inline_header_pack(const struct connectib_wqe_inline_header *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 10, (u_int32_t)ptr_struct->byte_count);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wqe_inline);

}

void connectib_wqe_inline_header_unpack(struct connectib_wqe_inline_header *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	ptr_struct->byte_count = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 10);

	offset = 0;
	ptr_struct->wqe_inline = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_wqe_inline_header_print(const struct connectib_wqe_inline_header *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_inline_header ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_count           : " UH_FMT "\n", ptr_struct->byte_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_inline           : " UH_FMT "\n", ptr_struct->wqe_inline);

}

int connectib_wqe_inline_header_size(void)
{
	 return 4;
}

void connectib_wqe_inline_header_dump(const struct connectib_wqe_inline_header *ptr_struct, FILE* file)
{
	connectib_wqe_inline_header_print(ptr_struct, file, 0);
}

void connectib_wqe_gather_scatter_entry_pack(const struct connectib_wqe_gather_scatter_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	adb2c_push_bits_to_buff(ptr_buff, offset, 31, (u_int32_t)ptr_struct->byte_count);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wqe_inline);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->local_key);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->local_address_63_32);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->local_address_31_0);

}

void connectib_wqe_gather_scatter_entry_unpack(struct connectib_wqe_gather_scatter_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 1;
	ptr_struct->byte_count = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 31);

	offset = 0;
	ptr_struct->wqe_inline = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->local_key = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->local_address_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->local_address_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_gather_scatter_entry_print(const struct connectib_wqe_gather_scatter_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_gather_scatter_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_count           : " UH_FMT "\n", ptr_struct->byte_count);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_inline           : " UH_FMT "\n", ptr_struct->wqe_inline);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_key            : " U32H_FMT "\n", ptr_struct->local_key);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_address_63_32  : " U32H_FMT "\n", ptr_struct->local_address_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "local_address_31_0   : " U32H_FMT "\n", ptr_struct->local_address_31_0);

}

int connectib_wqe_gather_scatter_entry_size(void)
{
	 return 16;
}

void connectib_wqe_gather_scatter_entry_dump(const struct connectib_wqe_gather_scatter_entry *ptr_struct, FILE* file)
{
	connectib_wqe_gather_scatter_entry_print(ptr_struct, file, 0);
}

void connectib_wqe_srq_next_pack(const struct connectib_wqe_srq_next *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->next_wqe_index);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->signature);

}

void connectib_wqe_srq_next_unpack(struct connectib_wqe_srq_next *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->next_wqe_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->signature = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_wqe_srq_next_print(const struct connectib_wqe_srq_next *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_srq_next ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "next_wqe_index       : " UH_FMT "\n", ptr_struct->next_wqe_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature            : " UH_FMT "\n", ptr_struct->signature);

}

int connectib_wqe_srq_next_size(void)
{
	 return 16;
}

void connectib_wqe_srq_next_dump(const struct connectib_wqe_srq_next *ptr_struct, FILE* file)
{
	connectib_wqe_srq_next_print(ptr_struct, file, 0);
}

void connectib_wqe_signature_pack(const struct connectib_wqe_signature *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->signature);

}

void connectib_wqe_signature_unpack(struct connectib_wqe_signature *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 32;
	ptr_struct->signature = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

}

void connectib_wqe_signature_print(const struct connectib_wqe_signature *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_signature ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature            : " UH_FMT "\n", ptr_struct->signature);

}

int connectib_wqe_signature_size(void)
{
	 return 16;
}

void connectib_wqe_signature_dump(const struct connectib_wqe_signature *ptr_struct, FILE* file)
{
	connectib_wqe_signature_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_fetch_add_pack(const union connectib_wqe_extended_atomic_fetch_add *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_wqe_extended_atomic_fetch_add_128byte_pack(&(ptr_struct->byte128), ptr_buff);
}

void connectib_wqe_extended_atomic_fetch_add_unpack(union connectib_wqe_extended_atomic_fetch_add *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_wqe_extended_atomic_fetch_add_128byte_unpack(&(ptr_struct->byte128), ptr_buff);
}

void connectib_wqe_extended_atomic_fetch_add_print(const union connectib_wqe_extended_atomic_fetch_add *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_fetch_add ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte4:\n");
	connectib_wqe_extended_atomic_fetch_add_4byte_print(&(ptr_struct->byte4), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte8:\n");
	connectib_wqe_extended_atomic_fetch_add_8byte_print(&(ptr_struct->byte8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte16:\n");
	connectib_wqe_extended_atomic_fetch_add_16byte_print(&(ptr_struct->byte16), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte32:\n");
	connectib_wqe_extended_atomic_fetch_add_32byte_print(&(ptr_struct->byte32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte64:\n");
	connectib_wqe_extended_atomic_fetch_add_64byte_print(&(ptr_struct->byte64), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte128:\n");
	connectib_wqe_extended_atomic_fetch_add_128byte_print(&(ptr_struct->byte128), file, indent_level + 1);

}

int connectib_wqe_extended_atomic_fetch_add_size(void)
{
	 return 256;
}

void connectib_wqe_extended_atomic_fetch_add_dump(const union connectib_wqe_extended_atomic_fetch_add *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_fetch_add_print(ptr_struct, file, 0);
}

void connectib_wqe_atomic_fetch_add_pack(const struct connectib_wqe_atomic_fetch_add *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->add_data_63_32);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->add_data_31_0);

}

void connectib_wqe_atomic_fetch_add_unpack(struct connectib_wqe_atomic_fetch_add *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->add_data_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->add_data_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_atomic_fetch_add_print(const struct connectib_wqe_atomic_fetch_add *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_atomic_fetch_add ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "add_data_63_32       : " U32H_FMT "\n", ptr_struct->add_data_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "add_data_31_0        : " U32H_FMT "\n", ptr_struct->add_data_31_0);

}

int connectib_wqe_atomic_fetch_add_size(void)
{
	 return 16;
}

void connectib_wqe_atomic_fetch_add_dump(const struct connectib_wqe_atomic_fetch_add *ptr_struct, FILE* file)
{
	connectib_wqe_atomic_fetch_add_print(ptr_struct, file, 0);
}

void connectib_wqe_extended_atomic_cmp_swap_pack(const union connectib_wqe_extended_atomic_cmp_swap *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_wqe_extended_atomic_cmp_swap_128byte_pack(&(ptr_struct->byte126), ptr_buff);
}

void connectib_wqe_extended_atomic_cmp_swap_unpack(union connectib_wqe_extended_atomic_cmp_swap *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_wqe_extended_atomic_cmp_swap_128byte_unpack(&(ptr_struct->byte126), ptr_buff);
}

void connectib_wqe_extended_atomic_cmp_swap_print(const union connectib_wqe_extended_atomic_cmp_swap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_extended_atomic_cmp_swap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte4:\n");
	connectib_wqe_extended_atomic_cmp_swap_4byte_print(&(ptr_struct->byte4), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte8:\n");
	connectib_wqe_extended_atomic_cmp_swap_8byte_print(&(ptr_struct->byte8), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte16:\n");
	connectib_wqe_extended_atomic_cmp_swap_16byte_print(&(ptr_struct->byte16), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte32:\n");
	connectib_wqe_extended_atomic_cmp_swap_32byte_print(&(ptr_struct->byte32), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte64:\n");
	connectib_wqe_extended_atomic_cmp_swap_64byte_print(&(ptr_struct->byte64), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte126:\n");
	connectib_wqe_extended_atomic_cmp_swap_128byte_print(&(ptr_struct->byte126), file, indent_level + 1);

}

int connectib_wqe_extended_atomic_cmp_swap_size(void)
{
	 return 512;
}

void connectib_wqe_extended_atomic_cmp_swap_dump(const union connectib_wqe_extended_atomic_cmp_swap *ptr_struct, FILE* file)
{
	connectib_wqe_extended_atomic_cmp_swap_print(ptr_struct, file, 0);
}

void connectib_wqe_atomic_cmp_swap_pack(const struct connectib_wqe_atomic_cmp_swap *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->swap_data_63_32);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->swap_data_31_0);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->compare_data_63_32);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->compare_data_31_0);

}

void connectib_wqe_atomic_cmp_swap_unpack(struct connectib_wqe_atomic_cmp_swap *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->swap_data_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->swap_data_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->compare_data_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->compare_data_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_atomic_cmp_swap_print(const struct connectib_wqe_atomic_cmp_swap *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_atomic_cmp_swap ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data_63_32      : " U32H_FMT "\n", ptr_struct->swap_data_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swap_data_31_0       : " U32H_FMT "\n", ptr_struct->swap_data_31_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data_63_32   : " U32H_FMT "\n", ptr_struct->compare_data_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "compare_data_31_0    : " U32H_FMT "\n", ptr_struct->compare_data_31_0);

}

int connectib_wqe_atomic_cmp_swap_size(void)
{
	 return 16;
}

void connectib_wqe_atomic_cmp_swap_dump(const struct connectib_wqe_atomic_cmp_swap *ptr_struct, FILE* file)
{
	connectib_wqe_atomic_cmp_swap_print(ptr_struct, file, 0);
}

void connectib_wqe_rdma_read_pack(const struct connectib_wqe_rdma_read *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 8, ptr_struct->remote_va);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->remote_key);

}

void connectib_wqe_rdma_read_unpack(struct connectib_wqe_rdma_read *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->remote_va = adb2c_pop_integer_from_buff(ptr_buff, offset, 8);

	offset = 64;
	ptr_struct->remote_key = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_rdma_read_print(const struct connectib_wqe_rdma_read *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_rdma_read ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "remote_va            : " U64H_FMT "\n", ptr_struct->remote_va);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "remote_key           : " U32H_FMT "\n", ptr_struct->remote_key);

}

int connectib_wqe_rdma_read_size(void)
{
	 return 16;
}

void connectib_wqe_rdma_read_dump(const struct connectib_wqe_rdma_read *ptr_struct, FILE* file)
{
	connectib_wqe_rdma_read_print(ptr_struct, file, 0);
}

void connectib_wqe_address_vector_ext_pack(const struct connectib_wqe_address_vector_ext *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->auto_1);

	offset = 63;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->auto_2);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->hop_limit);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->tclass);

	offset = 108;
	adb2c_push_bits_to_buff(ptr_buff, offset, 20, (u_int32_t)ptr_struct->flow_label);

	offset = 100;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->my_gid_index1);

	offset = 97;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->grh);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rgid_127_96);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rgid_95_64);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rgid_63_32);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rgid_31_0);

}

void connectib_wqe_address_vector_ext_unpack(struct connectib_wqe_address_vector_ext *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->auto_1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 63;
	ptr_struct->auto_2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 88;
	ptr_struct->hop_limit = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 80;
	ptr_struct->tclass = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 108;
	ptr_struct->flow_label = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 20);

	offset = 100;
	ptr_struct->my_gid_index1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 97;
	ptr_struct->grh = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 128;
	ptr_struct->rgid_127_96 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->rgid_95_64 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->rgid_63_32 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->rgid_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_address_vector_ext_print(const struct connectib_wqe_address_vector_ext *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_address_vector_ext ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "auto_1               : " UH_FMT "\n", ptr_struct->auto_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "auto_2               : " UH_FMT "\n", ptr_struct->auto_2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hop_limit            : " UH_FMT "\n", ptr_struct->hop_limit);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "tclass               : " UH_FMT "\n", ptr_struct->tclass);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flow_label           : " UH_FMT "\n", ptr_struct->flow_label);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "my_gid_index1        : " UH_FMT "\n", ptr_struct->my_gid_index1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh                  : " UH_FMT "\n", ptr_struct->grh);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rgid_127_96          : " U32H_FMT "\n", ptr_struct->rgid_127_96);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rgid_95_64           : " U32H_FMT "\n", ptr_struct->rgid_95_64);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rgid_63_32           : " U32H_FMT "\n", ptr_struct->rgid_63_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rgid_31_0            : " U32H_FMT "\n", ptr_struct->rgid_31_0);

}

int connectib_wqe_address_vector_ext_size(void)
{
	 return 32;
}

void connectib_wqe_address_vector_ext_dump(const struct connectib_wqe_address_vector_ext *ptr_struct, FILE* file)
{
	connectib_wqe_address_vector_ext_print(ptr_struct, file, 0);
}

void connectib_wqe_address_vector_basic_pack(const struct connectib_wqe_address_vector_basic *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dca_key_63_32_qkey);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dca_key_31_0);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->destination_qp_dct);

	offset = 68;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->reverse_sl);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ext);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rlid_15_0);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->slid);

	offset = 100;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->static_rate);

}

void connectib_wqe_address_vector_basic_unpack(struct connectib_wqe_address_vector_basic *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->dca_key_63_32_qkey = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->dca_key_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 72;
	ptr_struct->destination_qp_dct = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 68;
	ptr_struct->reverse_sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->ext = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 112;
	ptr_struct->rlid_15_0 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 104;
	ptr_struct->slid = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 100;
	ptr_struct->sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->static_rate = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_address_vector_basic_print(const struct connectib_wqe_address_vector_basic *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_address_vector_basic ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dca_key_63_32_qkey   : " U32H_FMT "\n", ptr_struct->dca_key_63_32_qkey);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dca_key_31_0         : " U32H_FMT "\n", ptr_struct->dca_key_31_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "destination_qp_dct   : " UH_FMT "\n", ptr_struct->destination_qp_dct);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reverse_sl           : " UH_FMT "\n", ptr_struct->reverse_sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ext                  : " UH_FMT "\n", ptr_struct->ext);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rlid_15_0            : " UH_FMT "\n", ptr_struct->rlid_15_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slid                 : " UH_FMT "\n", ptr_struct->slid);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl                   : " UH_FMT "\n", ptr_struct->sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "static_rate          : " UH_FMT "\n", ptr_struct->static_rate);

}

int connectib_wqe_address_vector_basic_size(void)
{
	 return 16;
}

void connectib_wqe_address_vector_basic_dump(const struct connectib_wqe_address_vector_basic *ptr_struct, FILE* file)
{
	connectib_wqe_address_vector_basic_print(ptr_struct, file, 0);
}

void connectib_wqe_xrc_pack(const struct connectib_wqe_xrc *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->xrc);

}

void connectib_wqe_xrc_unpack(struct connectib_wqe_xrc *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->xrc = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

}

void connectib_wqe_xrc_print(const struct connectib_wqe_xrc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_xrc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xrc                  : " UH_FMT "\n", ptr_struct->xrc);

}

int connectib_wqe_xrc_size(void)
{
	 return 16;
}

void connectib_wqe_xrc_dump(const struct connectib_wqe_xrc *ptr_struct, FILE* file)
{
	connectib_wqe_xrc_print(ptr_struct, file, 0);
}

void connectib_wqe_mlx_pack(const struct connectib_wqe_mlx *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 44;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sl);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->static_rate);

	offset = 37;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fl);

	offset = 36;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->ic);

	offset = 35;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fc);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->slr);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->vl15);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->v);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->dmac_dlid_47_32);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vlan);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->dmac_dlid_31_0);

}

void connectib_wqe_mlx_unpack(struct connectib_wqe_mlx *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 44;
	ptr_struct->sl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 40;
	ptr_struct->static_rate = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 37;
	ptr_struct->fl = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 36;
	ptr_struct->ic = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 35;
	ptr_struct->fc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 34;
	ptr_struct->slr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 33;
	ptr_struct->vl15 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->v = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->dmac_dlid_47_32 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 64;
	ptr_struct->vlan = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 96;
	ptr_struct->dmac_dlid_31_0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_mlx_print(const struct connectib_wqe_mlx *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_mlx ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sl                   : " UH_FMT "\n", ptr_struct->sl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "static_rate          : " UH_FMT "\n", ptr_struct->static_rate);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fl                   : " UH_FMT "\n", ptr_struct->fl);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ic                   : " UH_FMT "\n", ptr_struct->ic);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fc                   : " UH_FMT "\n", ptr_struct->fc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "slr                  : " UH_FMT "\n", ptr_struct->slr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vl15                 : " UH_FMT "\n", ptr_struct->vl15);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "v                    : " UH_FMT "\n", ptr_struct->v);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmac_dlid_47_32      : " UH_FMT "\n", ptr_struct->dmac_dlid_47_32);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vlan                 : " UH_FMT "\n", ptr_struct->vlan);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dmac_dlid_31_0       : " U32H_FMT "\n", ptr_struct->dmac_dlid_31_0);

}

int connectib_wqe_mlx_size(void)
{
	 return 16;
}

void connectib_wqe_mlx_dump(const struct connectib_wqe_mlx *ptr_struct, FILE* file)
{
	connectib_wqe_mlx_print(ptr_struct, file, 0);
}

void connectib_wqe_control_pack(const struct connectib_wqe_control *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->opcode);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->wqe_index);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->opc_mod);

	offset = 58;
	adb2c_push_bits_to_buff(ptr_buff, offset, 6, (u_int32_t)ptr_struct->ds);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 95;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->interrupt);

	offset = 94;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->se);

	offset = 92;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->ce);

	offset = 91;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->nop);

	offset = 88;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->fm);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->immediate_invalidation_key);

}

void connectib_wqe_control_unpack(struct connectib_wqe_control *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 24;
	ptr_struct->opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 8;
	ptr_struct->wqe_index = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->opc_mod = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 58;
	ptr_struct->ds = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 6);

	offset = 32;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 95;
	ptr_struct->interrupt = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 94;
	ptr_struct->se = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 92;
	ptr_struct->ce = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 91;
	ptr_struct->nop = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 88;
	ptr_struct->fm = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 96;
	ptr_struct->immediate_invalidation_key = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

}

void connectib_wqe_control_print(const struct connectib_wqe_control *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_wqe_control ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " UH_FMT "\n", ptr_struct->opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_index            : " UH_FMT "\n", ptr_struct->wqe_index);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opc_mod              : " UH_FMT "\n", ptr_struct->opc_mod);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ds                   : " UH_FMT "\n", ptr_struct->ds);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "interrupt            : " UH_FMT "\n", ptr_struct->interrupt);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "se                   : " UH_FMT "\n", ptr_struct->se);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ce                   : " UH_FMT "\n", ptr_struct->ce);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nop                  : " UH_FMT "\n", ptr_struct->nop);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fm                   : " UH_FMT "\n", ptr_struct->fm);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "immediate_invalidation_key : " U32H_FMT "\n", ptr_struct->immediate_invalidation_key);

}

int connectib_wqe_control_size(void)
{
	 return 16;
}

void connectib_wqe_control_dump(const struct connectib_wqe_control *ptr_struct, FILE* file)
{
	connectib_wqe_control_print(ptr_struct, file, 0);
}

void connectib_public_keys_pack(const struct connectib_public_keys *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 2304, i, 18432, 1);
	connectib_file_public_keys_pack(&(ptr_struct->file_public_keys[i]), ptr_buff + offset/8);
	}

}

void connectib_public_keys_unpack(struct connectib_public_keys *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 8; i++) {
	offset = adb2c_calc_array_field_address(0, 2304, i, 18432, 1);
	connectib_file_public_keys_unpack(&(ptr_struct->file_public_keys[i]), ptr_buff + offset/8);
	}

}

void connectib_public_keys_print(const struct connectib_public_keys *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_public_keys ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 8; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "file_public_keys_%03d:\n", i);
	connectib_file_public_keys_print(&(ptr_struct->file_public_keys[i]), file, indent_level + 1);
	}

}

int connectib_public_keys_size(void)
{
	 return 2304;
}

void connectib_public_keys_dump(const struct connectib_public_keys *ptr_struct, FILE* file)
{
	connectib_public_keys_print(ptr_struct, file, 0);
}

void connectib_image_signature_pack(const struct connectib_image_signature *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2560, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature_uuid[i]);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 2560, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->keypair_uuid[i]);
	}

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 2560, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature[i]);
	}

}

void connectib_image_signature_unpack(struct connectib_image_signature *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(0, 32, i, 2560, 1);
	ptr_struct->signature_uuid[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(128, 32, i, 2560, 1);
	ptr_struct->keypair_uuid[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	for (i = 0; i < 64; i++) {
	offset = adb2c_calc_array_field_address(256, 32, i, 2560, 1);
	ptr_struct->signature[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

}

void connectib_image_signature_print(const struct connectib_image_signature *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_image_signature ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature_uuid_%03d  : " U32H_FMT "\n", i, ptr_struct->signature_uuid[i]);
	}

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "keypair_uuid_%03d    : " U32H_FMT "\n", i, ptr_struct->keypair_uuid[i]);
	}

	for (i = 0; i < 64; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature_%03d       : " U32H_FMT "\n", i, ptr_struct->signature[i]);
	}

}

int connectib_image_signature_size(void)
{
	 return 320;
}

void connectib_image_signature_dump(const struct connectib_image_signature *ptr_struct, FILE* file)
{
	connectib_image_signature_print(ptr_struct, file, 0);
}

void connectib_nv_data_pack(const struct connectib_nv_data *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_config_header_pack(&(ptr_struct->header), ptr_buff + offset/8);

}

void connectib_nv_data_unpack(struct connectib_nv_data *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_nv_config_header_unpack(&(ptr_struct->header), ptr_buff + offset/8);

}

void connectib_nv_data_print(const struct connectib_nv_data *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_nv_data ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "header:\n");
	connectib_nv_config_header_print(&(ptr_struct->header), file, indent_level + 1);

}

int connectib_nv_data_size(void)
{
	 return 65536;
}

void connectib_nv_data_dump(const struct connectib_nv_data *ptr_struct, FILE* file)
{
	connectib_nv_data_print(ptr_struct, file, 0);
}

void connectib_hw_boot_config_pack(const struct connectib_hw_boot_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->manufacture_info_address);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->ro_manufacture_info_address);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->vpd_info_address_1);

	offset = 104;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->vpd_info_address_2);

	offset = 103;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->aux_power_indication_gpio_en);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->aux_power_indication_gpio_offset);

	offset = 128;
	connectib_vpd_eeprom_info_pack(&(ptr_struct->vpd_eeprom), ptr_buff + offset/8);

	offset = 192;
	connectib_code_coverage_control_pack(&(ptr_struct->code_coverage), ptr_buff + offset/8);

	offset = 256;
	connectib_pcie_cfg_golan_pack(&(ptr_struct->pcie_cfg), ptr_buff + offset/8);

	offset = 1792;
	connectib_fw_image_pack(&(ptr_struct->fw_image), ptr_buff + offset/8);

}

void connectib_hw_boot_config_unpack(struct connectib_hw_boot_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->manufacture_info_address = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 40;
	ptr_struct->ro_manufacture_info_address = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 72;
	ptr_struct->vpd_info_address_1 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 104;
	ptr_struct->vpd_info_address_2 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 103;
	ptr_struct->aux_power_indication_gpio_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->aux_power_indication_gpio_offset = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 128;
	connectib_vpd_eeprom_info_unpack(&(ptr_struct->vpd_eeprom), ptr_buff + offset/8);

	offset = 192;
	connectib_code_coverage_control_unpack(&(ptr_struct->code_coverage), ptr_buff + offset/8);

	offset = 256;
	connectib_pcie_cfg_golan_unpack(&(ptr_struct->pcie_cfg), ptr_buff + offset/8);

	offset = 1792;
	connectib_fw_image_unpack(&(ptr_struct->fw_image), ptr_buff + offset/8);

}

void connectib_hw_boot_config_print(const struct connectib_hw_boot_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_boot_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "manufacture_info_address : " UH_FMT "\n", ptr_struct->manufacture_info_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ro_manufacture_info_address : " UH_FMT "\n", ptr_struct->ro_manufacture_info_address);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_info_address_1   : " UH_FMT "\n", ptr_struct->vpd_info_address_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_info_address_2   : " UH_FMT "\n", ptr_struct->vpd_info_address_2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aux_power_indication_gpio_en : " UH_FMT "\n", ptr_struct->aux_power_indication_gpio_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "aux_power_indication_gpio_offset : " UH_FMT "\n", ptr_struct->aux_power_indication_gpio_offset);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vpd_eeprom:\n");
	connectib_vpd_eeprom_info_print(&(ptr_struct->vpd_eeprom), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "code_coverage:\n");
	connectib_code_coverage_control_print(&(ptr_struct->code_coverage), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_cfg:\n");
	connectib_pcie_cfg_golan_print(&(ptr_struct->pcie_cfg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_image:\n");
	connectib_fw_image_print(&(ptr_struct->fw_image), file, indent_level + 1);

}

int connectib_hw_boot_config_size(void)
{
	 return 320;
}

void connectib_hw_boot_config_dump(const struct connectib_hw_boot_config *ptr_struct, FILE* file)
{
	connectib_hw_boot_config_print(ptr_struct, file, 0);
}

void connectib_hw_main_config_pack(const struct connectib_hw_main_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_port_serdes_pack(&(ptr_struct->ports_serdes), ptr_buff + offset/8);

	offset = 14336;
	connectib_power_pack(&(ptr_struct->power), ptr_buff + offset/8);

	offset = 14464;
	connectib_system_mng_pack(&(ptr_struct->system_mng), ptr_buff + offset/8);

	offset = 14496;
	connectib_thermal_config_golan_pack(&(ptr_struct->thermal), ptr_buff + offset/8);

	offset = 15360;
	connectib_module_config_pack(&(ptr_struct->module), ptr_buff + offset/8);

	offset = 15424;
	connectib_i2c_devices_db_pack(&(ptr_struct->i2c), ptr_buff + offset/8);

	offset = 16064;
	connectib_ini_gpios_golan_pack(&(ptr_struct->gpios), ptr_buff + offset/8);

}

void connectib_hw_main_config_unpack(struct connectib_hw_main_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_port_serdes_unpack(&(ptr_struct->ports_serdes), ptr_buff + offset/8);

	offset = 14336;
	connectib_power_unpack(&(ptr_struct->power), ptr_buff + offset/8);

	offset = 14464;
	connectib_system_mng_unpack(&(ptr_struct->system_mng), ptr_buff + offset/8);

	offset = 14496;
	connectib_thermal_config_golan_unpack(&(ptr_struct->thermal), ptr_buff + offset/8);

	offset = 15360;
	connectib_module_config_unpack(&(ptr_struct->module), ptr_buff + offset/8);

	offset = 15424;
	connectib_i2c_devices_db_unpack(&(ptr_struct->i2c), ptr_buff + offset/8);

	offset = 16064;
	connectib_ini_gpios_golan_unpack(&(ptr_struct->gpios), ptr_buff + offset/8);

}

void connectib_hw_main_config_print(const struct connectib_hw_main_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_hw_main_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ports_serdes:\n");
	connectib_port_serdes_print(&(ptr_struct->ports_serdes), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "power:\n");
	connectib_power_print(&(ptr_struct->power), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "system_mng:\n");
	connectib_system_mng_print(&(ptr_struct->system_mng), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "thermal:\n");
	connectib_thermal_config_golan_print(&(ptr_struct->thermal), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module:\n");
	connectib_module_config_print(&(ptr_struct->module), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c:\n");
	connectib_i2c_devices_db_print(&(ptr_struct->i2c), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpios:\n");
	connectib_ini_gpios_golan_print(&(ptr_struct->gpios), file, indent_level + 1);

}

int connectib_hw_main_config_size(void)
{
	 return 2816;
}

void connectib_hw_main_config_dump(const struct connectib_hw_main_config *ptr_struct, FILE* file)
{
	connectib_hw_main_config_print(ptr_struct, file, 0);
}

void connectib_fw_boot_config_pack(const struct connectib_fw_boot_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_FW_VERSION_pack(&(ptr_struct->fw_version), ptr_buff + offset/8);

	offset = 128;
	connectib_multi_function_pack(&(ptr_struct->multi_function), ptr_buff + offset/8);

	offset = 224;
	connectib_pcie_transaction_pack(&(ptr_struct->pcie_transaction), ptr_buff + offset/8);

	offset = 288;
	connectib_pci_cfg_pack(&(ptr_struct->pcie_cfg_data), ptr_buff + offset/8);

	offset = 4384;
	connectib_pci_cfg_pack(&(ptr_struct->pcie_cfg_mask), ptr_buff + offset/8);

	offset = 8480;
	connectib_pf_vf_cfg_space_pack(&(ptr_struct->pf_vf_cfg_data), ptr_buff + offset/8);

	offset = 8704;
	connectib_pcie_power_management_pack(&(ptr_struct->pcie_power_management), ptr_buff + offset/8);

	offset = 8736;
	connectib_nv_config_boot_pack(&(ptr_struct->nv_config), ptr_buff + offset/8);

}

void connectib_fw_boot_config_unpack(struct connectib_fw_boot_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_FW_VERSION_unpack(&(ptr_struct->fw_version), ptr_buff + offset/8);

	offset = 128;
	connectib_multi_function_unpack(&(ptr_struct->multi_function), ptr_buff + offset/8);

	offset = 224;
	connectib_pcie_transaction_unpack(&(ptr_struct->pcie_transaction), ptr_buff + offset/8);

	offset = 288;
	connectib_pci_cfg_unpack(&(ptr_struct->pcie_cfg_data), ptr_buff + offset/8);

	offset = 4384;
	connectib_pci_cfg_unpack(&(ptr_struct->pcie_cfg_mask), ptr_buff + offset/8);

	offset = 8480;
	connectib_pf_vf_cfg_space_unpack(&(ptr_struct->pf_vf_cfg_data), ptr_buff + offset/8);

	offset = 8704;
	connectib_pcie_power_management_unpack(&(ptr_struct->pcie_power_management), ptr_buff + offset/8);

	offset = 8736;
	connectib_nv_config_boot_unpack(&(ptr_struct->nv_config), ptr_buff + offset/8);

}

void connectib_fw_boot_config_print(const struct connectib_fw_boot_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_boot_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_version:\n");
	connectib_FW_VERSION_print(&(ptr_struct->fw_version), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "multi_function:\n");
	connectib_multi_function_print(&(ptr_struct->multi_function), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_transaction:\n");
	connectib_pcie_transaction_print(&(ptr_struct->pcie_transaction), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_cfg_data:\n");
	connectib_pci_cfg_print(&(ptr_struct->pcie_cfg_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_cfg_mask:\n");
	connectib_pci_cfg_print(&(ptr_struct->pcie_cfg_mask), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pf_vf_cfg_data:\n");
	connectib_pf_vf_cfg_space_print(&(ptr_struct->pf_vf_cfg_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_power_management:\n");
	connectib_pcie_power_management_print(&(ptr_struct->pcie_power_management), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nv_config:\n");
	connectib_nv_config_boot_print(&(ptr_struct->nv_config), file, indent_level + 1);

}

int connectib_fw_boot_config_size(void)
{
	 return 1216;
}

void connectib_fw_boot_config_dump(const struct connectib_fw_boot_config *ptr_struct, FILE* file)
{
	connectib_fw_boot_config_print(ptr_struct, file, 0);
}

void connectib_fw_main_config_pack(const struct connectib_fw_main_config *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->log_max_num_rdbs);

	offset = 56;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->log_max_atomic_size);

	offset = 128;
	connectib_static_config_pack(&(ptr_struct->static_config), ptr_buff + offset/8);

	offset = 256;
	connectib_leds_blink_patterns_pack(&(ptr_struct->leds_blink_pattern), ptr_buff + offset/8);

	offset = 1536;
	connectib_customization_pack(&(ptr_struct->customization), ptr_buff + offset/8);

	offset = 2048;
	connectib_management_pack(&(ptr_struct->management), ptr_buff + offset/8);

	offset = 5120;
	connectib_host_management_pack(&(ptr_struct->host_management), ptr_buff + offset/8);

	offset = 6144;
	connectib_phy_fw_main_config_pack(&(ptr_struct->phy_fw_main_config), ptr_buff + offset/8);

	offset = 11904;
	connectib_nv_config_main_pack(&(ptr_struct->nv_config), ptr_buff + offset/8);

	offset = 20096;
	connectib_secure_fw_pack(&(ptr_struct->secure_fw), ptr_buff + offset/8);

}

void connectib_fw_main_config_unpack(struct connectib_fw_main_config *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 61;
	ptr_struct->log_max_num_rdbs = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 56;
	ptr_struct->log_max_atomic_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 128;
	connectib_static_config_unpack(&(ptr_struct->static_config), ptr_buff + offset/8);

	offset = 256;
	connectib_leds_blink_patterns_unpack(&(ptr_struct->leds_blink_pattern), ptr_buff + offset/8);

	offset = 1536;
	connectib_customization_unpack(&(ptr_struct->customization), ptr_buff + offset/8);

	offset = 2048;
	connectib_management_unpack(&(ptr_struct->management), ptr_buff + offset/8);

	offset = 5120;
	connectib_host_management_unpack(&(ptr_struct->host_management), ptr_buff + offset/8);

	offset = 6144;
	connectib_phy_fw_main_config_unpack(&(ptr_struct->phy_fw_main_config), ptr_buff + offset/8);

	offset = 11904;
	connectib_nv_config_main_unpack(&(ptr_struct->nv_config), ptr_buff + offset/8);

	offset = 20096;
	connectib_secure_fw_unpack(&(ptr_struct->secure_fw), ptr_buff + offset/8);

}

void connectib_fw_main_config_print(const struct connectib_fw_main_config *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_fw_main_config ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_num_rdbs     : " UH_FMT "\n", ptr_struct->log_max_num_rdbs);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "log_max_atomic_size  : " UH_FMT "\n", ptr_struct->log_max_atomic_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "static_config:\n");
	connectib_static_config_print(&(ptr_struct->static_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "leds_blink_pattern:\n");
	connectib_leds_blink_patterns_print(&(ptr_struct->leds_blink_pattern), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "customization:\n");
	connectib_customization_print(&(ptr_struct->customization), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "management:\n");
	connectib_management_print(&(ptr_struct->management), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "host_management:\n");
	connectib_host_management_print(&(ptr_struct->host_management), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_fw_main_config:\n");
	connectib_phy_fw_main_config_print(&(ptr_struct->phy_fw_main_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nv_config:\n");
	connectib_nv_config_main_print(&(ptr_struct->nv_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "secure_fw:\n");
	connectib_secure_fw_print(&(ptr_struct->secure_fw), file, indent_level + 1);

}

int connectib_fw_main_config_size(void)
{
	 return 4096;
}

void connectib_fw_main_config_dump(const struct connectib_fw_main_config *ptr_struct, FILE* file)
{
	connectib_fw_main_config_print(ptr_struct, file, 0);
}

void connectib_mfg_info_pack(const struct connectib_mfg_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(24, 8, i, 2560, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->psid[i]);
	}

	offset = 255;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->guids_override_en);

	offset = 232;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->minor_version);

	offset = 224;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->major_version);

	offset = 256;
	connectib_guids_pack(&(ptr_struct->guids), ptr_buff + offset/8);

}

void connectib_mfg_info_unpack(struct connectib_mfg_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(24, 8, i, 2560, 1);
	ptr_struct->psid[i] = (char)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	ptr_struct->psid[16] = '\0';
	offset = 255;
	ptr_struct->guids_override_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 232;
	ptr_struct->minor_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 224;
	ptr_struct->major_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 256;
	connectib_guids_unpack(&(ptr_struct->guids), ptr_buff + offset/8);

}

void connectib_mfg_info_print(const struct connectib_mfg_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_mfg_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	fprintf(file, "psid                 : \"%s\"\n", ptr_struct->psid);
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "guids_override_en    : " UH_FMT "\n", ptr_struct->guids_override_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "minor_version        : " UH_FMT "\n", ptr_struct->minor_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "major_version        : " UH_FMT "\n", ptr_struct->major_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "guids:\n");
	connectib_guids_print(&(ptr_struct->guids), file, indent_level + 1);

}

int connectib_mfg_info_size(void)
{
	 return 320;
}

void connectib_mfg_info_dump(const struct connectib_mfg_info *ptr_struct, FILE* file)
{
	connectib_mfg_info_print(ptr_struct, file, 0);
}

void connectib_image_info_pack(const struct connectib_image_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->signed_vendor_nvconfig_files);

	offset = 21;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->signed_mlnx_nvconfig_files);

	offset = 20;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->frc_supported);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cs_tokens_supported);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->debug_fw);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->signed_fw);

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->secured_fw);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->minor_version);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->major_version);

	offset = 32;
	connectib_FW_VERSION_pack(&(ptr_struct->FW_VERSION), ptr_buff + offset/8);

	offset = 160;
	connectib_TRIPPLE_VERSION_pack(&(ptr_struct->mic_version), ptr_buff + offset/8);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(312, 8, i, 8192, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->psid[i]);
	}

	offset = 432;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vsd_vendor_id);

	for (i = 0; i < 208; i++) {
	offset = adb2c_calc_array_field_address(472, 8, i, 8192, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vsd[i]);
	}

	offset = 2112;
	connectib_image_size_pack(&(ptr_struct->image_size), ptr_buff + offset/8);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(2240, 32, i, 8192, 1);
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->supported_hw_id[i]);
	}

	offset = 2368;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->ini_file_num);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(3608, 8, i, 8192, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->prod_ver[i]);
	}

	offset = 6144;
	connectib_module_versions_pack(&(ptr_struct->module_versions), ptr_buff + offset/8);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(6680, 8, i, 8192, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->spare_part[i]);
	}

	for (i = 0; i < 24; i++) {
	offset = adb2c_calc_array_field_address(6776, 8, i, 8192, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->product_id[i]);
	}

}

void connectib_image_info_unpack(struct connectib_image_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 22;
	ptr_struct->signed_vendor_nvconfig_files = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 21;
	ptr_struct->signed_mlnx_nvconfig_files = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 20;
	ptr_struct->frc_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 19;
	ptr_struct->cs_tokens_supported = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 18;
	ptr_struct->debug_fw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 17;
	ptr_struct->signed_fw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 16;
	ptr_struct->secured_fw = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 8;
	ptr_struct->minor_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 0;
	ptr_struct->major_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 32;
	connectib_FW_VERSION_unpack(&(ptr_struct->FW_VERSION), ptr_buff + offset/8);

	offset = 160;
	connectib_TRIPPLE_VERSION_unpack(&(ptr_struct->mic_version), ptr_buff + offset/8);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(312, 8, i, 8192, 1);
	ptr_struct->psid[i] = (char)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	ptr_struct->psid[16] = '\0';
	offset = 432;
	ptr_struct->vsd_vendor_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	for (i = 0; i < 208; i++) {
	offset = adb2c_calc_array_field_address(472, 8, i, 8192, 1);
	ptr_struct->vsd[i] = (char)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	ptr_struct->vsd[208] = '\0';
	offset = 2112;
	connectib_image_size_unpack(&(ptr_struct->image_size), ptr_buff + offset/8);

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(2240, 32, i, 8192, 1);
	ptr_struct->supported_hw_id[i] = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);
	}

	offset = 2368;
	ptr_struct->ini_file_num = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	for (i = 0; i < 16; i++) {
	offset = adb2c_calc_array_field_address(3608, 8, i, 8192, 1);
	ptr_struct->prod_ver[i] = (char)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	ptr_struct->prod_ver[16] = '\0';
	offset = 6144;
	connectib_module_versions_unpack(&(ptr_struct->module_versions), ptr_buff + offset/8);

	for (i = 0; i < 12; i++) {
	offset = adb2c_calc_array_field_address(6680, 8, i, 8192, 1);
	ptr_struct->spare_part[i] = (char)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	ptr_struct->spare_part[12] = '\0';
	for (i = 0; i < 24; i++) {
	offset = adb2c_calc_array_field_address(6776, 8, i, 8192, 1);
	ptr_struct->product_id[i] = (char)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	ptr_struct->product_id[24] = '\0';
}

void connectib_image_info_print(const struct connectib_image_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_image_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signed_vendor_nvconfig_files : " UH_FMT "\n", ptr_struct->signed_vendor_nvconfig_files);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signed_mlnx_nvconfig_files : " UH_FMT "\n", ptr_struct->signed_mlnx_nvconfig_files);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "frc_supported        : " UH_FMT "\n", ptr_struct->frc_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cs_tokens_supported  : " UH_FMT "\n", ptr_struct->cs_tokens_supported);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "debug_fw             : " UH_FMT "\n", ptr_struct->debug_fw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signed_fw            : " UH_FMT "\n", ptr_struct->signed_fw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "secured_fw           : " UH_FMT "\n", ptr_struct->secured_fw);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "minor_version        : " UH_FMT "\n", ptr_struct->minor_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "major_version        : " UH_FMT "\n", ptr_struct->major_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "FW_VERSION:\n");
	connectib_FW_VERSION_print(&(ptr_struct->FW_VERSION), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mic_version:\n");
	connectib_TRIPPLE_VERSION_print(&(ptr_struct->mic_version), file, indent_level + 1);

	fprintf(file, "psid                 : \"%s\"\n", ptr_struct->psid);
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vsd_vendor_id        : " UH_FMT "\n", ptr_struct->vsd_vendor_id);

	fprintf(file, "vsd                  : \"%s\"\n", ptr_struct->vsd);
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "image_size:\n");
	connectib_image_size_print(&(ptr_struct->image_size), file, indent_level + 1);

	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "supported_hw_id_%03d : " U32H_FMT "\n", i, ptr_struct->supported_hw_id[i]);
	}

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ini_file_num         : " U32H_FMT "\n", ptr_struct->ini_file_num);

	fprintf(file, "prod_ver             : \"%s\"\n", ptr_struct->prod_ver);
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "module_versions:\n");
	connectib_module_versions_print(&(ptr_struct->module_versions), file, indent_level + 1);

	fprintf(file, "spare_part           : \"%s\"\n", ptr_struct->spare_part);
	fprintf(file, "product_id           : \"%s\"\n", ptr_struct->product_id);
}

int connectib_image_info_size(void)
{
	 return 1024;
}

void connectib_image_info_dump(const struct connectib_image_info *ptr_struct, FILE* file)
{
	connectib_image_info_print(ptr_struct, file, 0);
}

void connectib_device_info_pack(const struct connectib_device_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature0);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature1);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature2);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature3);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->minor_version);

	offset = 143;
	adb2c_push_bits_to_buff(ptr_buff, offset, 9, (u_int32_t)ptr_struct->major_version);

	offset = 256;
	connectib_guids_pack(&(ptr_struct->guids), ptr_buff + offset/8);

	offset = 880;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->vsd_vendor_id);

	for (i = 0; i < 208; i++) {
	offset = adb2c_calc_array_field_address(920, 8, i, 4096, 1);
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vsd[i]);
	}

	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(2816, 128, i, 4096, 1);
	connectib_operation_key_pack(&(ptr_struct->keys[i]), ptr_buff + offset/8);
	}

}

void connectib_device_info_unpack(struct connectib_device_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->signature0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->signature1 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->signature2 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->signature3 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 152;
	ptr_struct->minor_version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 143;
	ptr_struct->major_version = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 9);

	offset = 256;
	connectib_guids_unpack(&(ptr_struct->guids), ptr_buff + offset/8);

	offset = 880;
	ptr_struct->vsd_vendor_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	for (i = 0; i < 208; i++) {
	offset = adb2c_calc_array_field_address(920, 8, i, 4096, 1);
	ptr_struct->vsd[i] = (char)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);
	}

	ptr_struct->vsd[208] = '\0';
	for (i = 0; i < 4; i++) {
	offset = adb2c_calc_array_field_address(2816, 128, i, 4096, 1);
	connectib_operation_key_unpack(&(ptr_struct->keys[i]), ptr_buff + offset/8);
	}

}

void connectib_device_info_print(const struct connectib_device_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_device_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature0           : " U32H_FMT "\n", ptr_struct->signature0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature1           : " U32H_FMT "\n", ptr_struct->signature1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature2           : " U32H_FMT "\n", ptr_struct->signature2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature3           : " U32H_FMT "\n", ptr_struct->signature3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "minor_version        : " UH_FMT "\n", ptr_struct->minor_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "major_version        : " UH_FMT "\n", ptr_struct->major_version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "guids:\n");
	connectib_guids_print(&(ptr_struct->guids), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vsd_vendor_id        : " UH_FMT "\n", ptr_struct->vsd_vendor_id);

	fprintf(file, "vsd                  : \"%s\"\n", ptr_struct->vsd);
	for (i = 0; i < 4; i++) {
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "keys_%03d:\n", i);
	connectib_operation_key_print(&(ptr_struct->keys[i]), file, indent_level + 1);
	}

}

int connectib_device_info_size(void)
{
	 return 512;
}

void connectib_device_info_dump(const struct connectib_device_info *ptr_struct, FILE* file)
{
	connectib_device_info_print(ptr_struct, file, 0);
}

void connectib_dtoc_pointers_pack(const union connectib_dtoc_pointers *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_toc_data_pack(&(ptr_struct->mfg_info), ptr_buff);
}

void connectib_dtoc_pointers_unpack(union connectib_dtoc_pointers *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_toc_data_unpack(&(ptr_struct->mfg_info), ptr_buff);
}

void connectib_dtoc_pointers_print(const union connectib_dtoc_pointers *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_dtoc_pointers ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_info_0:\n");
	connectib_toc_data_print(&(ptr_struct->device_info_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nv_data_0:\n");
	connectib_toc_data_print(&(ptr_struct->nv_data_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nv_data_1:\n");
	connectib_toc_data_print(&(ptr_struct->nv_data_1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_nv_log:\n");
	connectib_toc_data_print(&(ptr_struct->fw_nv_log), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfg_info:\n");
	connectib_toc_data_print(&(ptr_struct->mfg_info), file, indent_level + 1);

}

int connectib_dtoc_pointers_size(void)
{
	 return 256;
}

void connectib_dtoc_pointers_dump(const union connectib_dtoc_pointers *ptr_struct, FILE* file)
{
	connectib_dtoc_pointers_print(ptr_struct, file, 0);
}

void connectib_reset_info_pack(const struct connectib_reset_info *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reset_capabilities_pack(&(ptr_struct->rst_cap_mask), ptr_buff + offset/8);

	offset = 128;
	connectib_rst_api_ver_pack(&(ptr_struct->rst_api_ver), ptr_buff + offset/8);

}

void connectib_reset_info_unpack(struct connectib_reset_info *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_reset_capabilities_unpack(&(ptr_struct->rst_cap_mask), ptr_buff + offset/8);

	offset = 128;
	connectib_rst_api_ver_unpack(&(ptr_struct->rst_api_ver), ptr_buff + offset/8);

}

void connectib_reset_info_print(const struct connectib_reset_info *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_reset_info ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rst_cap_mask:\n");
	connectib_reset_capabilities_print(&(ptr_struct->rst_cap_mask), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rst_api_ver:\n");
	connectib_rst_api_ver_print(&(ptr_struct->rst_api_ver), file, indent_level + 1);

}

int connectib_reset_info_size(void)
{
	 return 256;
}

void connectib_reset_info_dump(const struct connectib_reset_info *ptr_struct, FILE* file)
{
	connectib_reset_info_print(ptr_struct, file, 0);
}

void connectib_boot_record_golan_pack(const struct connectib_boot_record_golan *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->flash_div);

	offset = 27;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->hot_reset_enable);

	offset = 26;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->wait_lock);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->perst_action);

	offset = 17;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->core_f);

	offset = 15;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->core_pd);

	offset = 13;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->core_od);

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->core_r);

	offset = 7;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->core_bypass);

	offset = 6;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->core_from_pex);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->parity_reserved0);

	offset = 47;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->temp_gpio_mask);

	offset = 43;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->temp_fatal_mask);

	offset = 40;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->gpio_pull_en);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->pll_debug);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->parity_reserved1);

	offset = 92;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mbist_group_size);

	offset = 89;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->mbist_unit_size);

	offset = 87;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->mbist_en);

	offset = 80;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->lbist_capture_delay);

	offset = 73;
	adb2c_push_bits_to_buff(ptr_buff, offset, 7, (u_int32_t)ptr_struct->i2c_slave_addr);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->i2c_slave_en);

	offset = 71;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->fuse_enable);

	offset = 65;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->pll_stabilize);

	offset = 64;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->parity_reserved2);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->lbist_pat_cnt_lsb);

	offset = 106;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->lbist_capture);

	offset = 105;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lbist_capt_2);

	offset = 102;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->lbist_shift_clk_divider);

	offset = 98;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->lbist_capture_last);

	offset = 97;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->lbist_en);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->parity_reserved3);

}

void connectib_boot_record_golan_unpack(struct connectib_boot_record_golan *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 28;
	ptr_struct->flash_div = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 27;
	ptr_struct->hot_reset_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 26;
	ptr_struct->wait_lock = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 24;
	ptr_struct->perst_action = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 17;
	ptr_struct->core_f = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 15;
	ptr_struct->core_pd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 13;
	ptr_struct->core_od = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 8;
	ptr_struct->core_r = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 7;
	ptr_struct->core_bypass = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 6;
	ptr_struct->core_from_pex = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 0;
	ptr_struct->parity_reserved0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 47;
	ptr_struct->temp_gpio_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 43;
	ptr_struct->temp_fatal_mask = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 40;
	ptr_struct->gpio_pull_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 33;
	ptr_struct->pll_debug = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 32;
	ptr_struct->parity_reserved1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 92;
	ptr_struct->mbist_group_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 89;
	ptr_struct->mbist_unit_size = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 87;
	ptr_struct->mbist_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 80;
	ptr_struct->lbist_capture_delay = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 73;
	ptr_struct->i2c_slave_addr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 7);

	offset = 72;
	ptr_struct->i2c_slave_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 71;
	ptr_struct->fuse_enable = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 65;
	ptr_struct->pll_stabilize = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 64;
	ptr_struct->parity_reserved2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 112;
	ptr_struct->lbist_pat_cnt_lsb = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 106;
	ptr_struct->lbist_capture = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 105;
	ptr_struct->lbist_capt_2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 102;
	ptr_struct->lbist_shift_clk_divider = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 98;
	ptr_struct->lbist_capture_last = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 97;
	ptr_struct->lbist_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 96;
	ptr_struct->parity_reserved3 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

}

void connectib_boot_record_golan_print(const struct connectib_boot_record_golan *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_boot_record_golan ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_div            : " UH_FMT "\n", ptr_struct->flash_div);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hot_reset_enable     : " UH_FMT "\n", ptr_struct->hot_reset_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wait_lock            : " UH_FMT "\n", ptr_struct->wait_lock);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "perst_action         : " UH_FMT "\n", ptr_struct->perst_action);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_f               : " UH_FMT "\n", ptr_struct->core_f);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_pd              : " UH_FMT "\n", ptr_struct->core_pd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_od              : " UH_FMT "\n", ptr_struct->core_od);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_r               : " UH_FMT "\n", ptr_struct->core_r);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_bypass          : " UH_FMT "\n", ptr_struct->core_bypass);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "core_from_pex        : " UH_FMT "\n", ptr_struct->core_from_pex);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parity_reserved0     : " UH_FMT "\n", ptr_struct->parity_reserved0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "temp_gpio_mask       : " UH_FMT "\n", ptr_struct->temp_gpio_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "temp_fatal_mask      : " UH_FMT "\n", ptr_struct->temp_fatal_mask);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gpio_pull_en         : " UH_FMT "\n", ptr_struct->gpio_pull_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pll_debug            : " UH_FMT "\n", ptr_struct->pll_debug);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parity_reserved1     : " UH_FMT "\n", ptr_struct->parity_reserved1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mbist_group_size     : " UH_FMT "\n", ptr_struct->mbist_group_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mbist_unit_size      : " UH_FMT "\n", ptr_struct->mbist_unit_size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mbist_en             : " UH_FMT "\n", ptr_struct->mbist_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lbist_capture_delay  : " UH_FMT "\n", ptr_struct->lbist_capture_delay);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_slave_addr       : " UH_FMT "\n", ptr_struct->i2c_slave_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "i2c_slave_en         : " UH_FMT "\n", ptr_struct->i2c_slave_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fuse_enable          : " UH_FMT "\n", ptr_struct->fuse_enable);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pll_stabilize        : " UH_FMT "\n", ptr_struct->pll_stabilize);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parity_reserved2     : " UH_FMT "\n", ptr_struct->parity_reserved2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lbist_pat_cnt_lsb    : " UH_FMT "\n", ptr_struct->lbist_pat_cnt_lsb);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lbist_capture        : " UH_FMT "\n", ptr_struct->lbist_capture);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lbist_capt_2         : " UH_FMT "\n", ptr_struct->lbist_capt_2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lbist_shift_clk_divider : " UH_FMT "\n", ptr_struct->lbist_shift_clk_divider);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lbist_capture_last   : " UH_FMT "\n", ptr_struct->lbist_capture_last);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lbist_en             : " UH_FMT "\n", ptr_struct->lbist_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parity_reserved3     : " UH_FMT "\n", ptr_struct->parity_reserved3);

}

int connectib_boot_record_golan_size(void)
{
	 return 16;
}

void connectib_boot_record_golan_dump(const struct connectib_boot_record_golan *ptr_struct, FILE* file)
{
	connectib_boot_record_golan_print(ptr_struct, file, 0);
}

void connectib_icmd_ctrl_pack(const struct connectib_icmd_ctrl *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->destructive_op_unlock);

	offset = 48;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->gvmi);

	offset = 47;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->public_icmd);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->state);

	offset = 72;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->syndrome);

	offset = 127;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->busy);

	offset = 112;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->status);

	offset = 96;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->opcode);

}

void connectib_icmd_ctrl_unpack(struct connectib_icmd_ctrl *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->destructive_op_unlock = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 48;
	ptr_struct->gvmi = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 47;
	ptr_struct->public_icmd = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->state = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 72;
	ptr_struct->syndrome = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 127;
	ptr_struct->busy = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 112;
	ptr_struct->status = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 96;
	ptr_struct->opcode = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_icmd_ctrl_print(const struct connectib_icmd_ctrl *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_ctrl ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "destructive_op_unlock : " U32H_FMT "\n", ptr_struct->destructive_op_unlock);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gvmi                 : " UH_FMT "\n", ptr_struct->gvmi);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "public_icmd          : " UH_FMT "\n", ptr_struct->public_icmd);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "state                : " UH_FMT "\n", ptr_struct->state);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "syndrome             : " UH_FMT "\n", ptr_struct->syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "busy                 : " UH_FMT "\n", ptr_struct->busy);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "status               : " UH_FMT "\n", ptr_struct->status);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : %s (" UH_FMT ")\n", (ptr_struct->opcode == 0 ? ("ICMD_NONE") : ((ptr_struct->opcode == 4096 ? (" OPCODE_MNG_MMINIT") : ((ptr_struct->opcode == 4097 ? (" ICMD_OPCODE_MNG_GPIO_FUNC") : ((ptr_struct->opcode == 0 ? (" ICMD_NONE") : ((ptr_struct->opcode == 4098 ? (" OPCODE_MNG_TEMP_HOOK") : ((ptr_struct->opcode == 8192 ? (" ICMD_OPCODE_PRE_DEFINE_MODULE_DB") : ((ptr_struct->opcode == 10240 ? (" ICMD_OPCODE_GET_FW_DRIVER_VERSION") : ((ptr_struct->opcode == 10241 ? (" ICMD_OPCODE_OCBB_RNDC_GET_CORE_API") : ((ptr_struct->opcode == 10242 ? (" ICMD_OPCODE_OCBB_GET_PCI_API") : ((ptr_struct->opcode == 10243 ? (" ICMD_OPCODE_READ_NCSI_RDMA_COUNTERS") : ((ptr_struct->opcode == 10244 ? (" ICMD_OPCODE_CABLE_ERROR_GEN_EVENT") : ((ptr_struct->opcode == 10245 ? (" ICMD_OPCODE_LED_CMD") : ((ptr_struct->opcode == 10246 ? (" READ_ICMC_TAGS_AND_INFO") : ((ptr_struct->opcode == 10247 ? (" ICMD_OPCODE_OVONEL_MNG_INITIAL_CMD") : ((ptr_struct->opcode == 10256 ? (" ICMD_OPCODE_LED_CONTROL_MAD") : ((ptr_struct->opcode == 10257 ? (" ICMD_OPCODE_TEMP_WARN_GEN_EVENT") : ((ptr_struct->opcode == 10258 ? (" ICMD_OPCODE_ENABLE_DISABLE_PORT") : ((ptr_struct->opcode == 10259 ? (" OPCODE_MNG_SSPL_HOOK") : ((ptr_struct->opcode == 10260 ? (" ICMD_OPCODE_ASN1_FORCE_PORT_STATE") : ((ptr_struct->opcode == 10261 ? (" ICMD_OPCODE_SET_STEERING_LOOPBACK") : ((ptr_struct->opcode == 10262 ? (" ICMD_OPCODE_DROP_COUNTER_READ") : ((ptr_struct->opcode == 10263 ? (" ICMD_OPCODE_PORT_ADMIN_STATE_MNG") : ((ptr_struct->opcode == 10272 ? (" ICMD_OPCODE_INTERNAL_QUERY_CAP") : ((ptr_struct->opcode == 10273 ? (" ALLOWED_PFS_MADIFC") : ((ptr_struct->opcode == 10274 ? (" ICMD_OPCODE_MAP_EVENT_EQ") : ((ptr_struct->opcode == 10275 ? (" ICMD_OPCODE_SYSPORT") : ((ptr_struct->opcode == 24577 ? (" ICMD_OPCODE_PHY_UC_DO_CONTINUE") : ((ptr_struct->opcode == 24578 ? (" ICMD_OPCODE_PHY_UC_DO_STEP") : ((ptr_struct->opcode == 24579 ? (" ICMD_OPCODE_PHY_UC_SET_DEL_BP") : ((ptr_struct->opcode == 24580 ? (" ICMD_OPCODE_PHY_UC_SET_GET_DATA") : ((ptr_struct->opcode == 24581 ? (" ICMD_OPCODE_PHY_UC_GET_ARRAY_PROP") : ((ptr_struct->opcode == 24582 ? (" ICMD_OPCODE_PHY_UC_GET_STATUS") : ((ptr_struct->opcode == 24583 ? (" ICMD_OPCODE_PHY_ACTIVATE_RX_ADAP") : ((ptr_struct->opcode == 24584 ? (" ICMD_OPCODE_PHY_UC_SET_GET_DATA_FIELD") : ((ptr_struct->opcode == 24585 ? (" ICMD_OPCODE_PHY_UC_ACTIVATE_UC") : ((ptr_struct->opcode == 24587 ? (" ICMD_OPCODE_PHY_GET_LINK_STATUS") : ((ptr_struct->opcode == 24588 ? (" ICMD_OPCODE_PHY_OPEN_CLOSE_PORT") : ((ptr_struct->opcode == 24589 ? (" ICMD_OPCODE_PHY_PLL_VARACTOR_CALIB") : ((ptr_struct->opcode == 24590 ? (" ICMD_OPCODE_PHY_HALT_FSM") : ((ptr_struct->opcode == 24591 ? (" ICMD_OPCODE_PHY_SET_GET_TX_SD") : ((ptr_struct->opcode == 24592 ? (" ICMD_OPCODE_PHY_SET_GET_RX_SD") : ((ptr_struct->opcode == 24593 ? (" ICMD_OPCODE_PHY_TRANS_OR_SYNC") : ((ptr_struct->opcode == 24595 ? (" ICMD_OPCODE_PHY_GET_CLEAR_COUNTERS") : ((ptr_struct->opcode == 24596 ? (" ICMD_OPCODE_PHY_UPDATE_CALIB_VALS") : ((ptr_struct->opcode == 24597 ? (" ICMD_OPCODE_PHY_GET_GRADE_INFO") : ((ptr_struct->opcode == 24598 ? (" ICMD_OPCODE_PHY_SET_GET_PTAS") : ((ptr_struct->opcode == 24599 ? (" ICMD_OPCODE_PHY_GET_PPAMP") : ((ptr_struct->opcode == 24600 ? (" ICMD_OPCODE_PHY_GET_PPLL") : ((ptr_struct->opcode == 24601 ? (" ICMD_OPCODE_PHY_GET_PDDR") : ((ptr_struct->opcode == 24602 ? (" ICMD_OPCODE_PHY_RUN_HYB_EYE_OPENER") : ((ptr_struct->opcode == 24603 ? (" ICMD_OPCODE_PHY_DESIGN_VER_IF") : ((ptr_struct->opcode == 24604 ? (" ICMD_OPCODE_RESTORE_PHY_REGISTERS") : ((ptr_struct->opcode == 24605 ? (" ICMD_OPCODE_PHY_MEASURE_FREQUENCY") : ((ptr_struct->opcode == 24606 ? (" ICMD_OPCODE_PHY_ACTIVATE_PLL") : ((ptr_struct->opcode == 32768 ? (" ACCESS_ICM_LINE") : ((ptr_struct->opcode == 32769 ? (" ACCESS_ICM_CTX") : ((ptr_struct->opcode == 32771 ? (" READ_Q_ENTRY") : ((ptr_struct->opcode == 32772 ? (" GET_IRISC_HEARTBEAT") : ((ptr_struct->opcode == 32773 ? (" GET_BOOT_STAGE") : ((ptr_struct->opcode == 32774 ? (" GET_LINK_LEDS") : ((ptr_struct->opcode == 32775 ? (" GET_FW_INFO") : ((ptr_struct->opcode == 32776 ? (" READ_RX_SLICE_PKT") : ((ptr_struct->opcode == 32777 ? (" READ_G_RSE_DESC") : ((ptr_struct->opcode == 32784 ? (" READ_SX_WQ_BUFFER") : ((ptr_struct->opcode == 32785 ? (" READ_ICM_TLB_LINE") : ((ptr_struct->opcode == 32786 ? (" READ_DUMMY_ADDRESS") : ((ptr_struct->opcode == 32787 ? (" DEBUG_FW_TRACER") : ((ptr_struct->opcode == 32788 ? (" CABLE_PLUG_OUT_IN") : ((ptr_struct->opcode == 32789 ? (" GET_ICM_RES_PROP") : ((ptr_struct->opcode == 32791 ? (" GET_ICM_CTX_ADDRESS") : ((ptr_struct->opcode == 32792 ? (" CHECK_SQPC_STATUS") : ((ptr_struct->opcode == 32793 ? (" ALLOC_DEALLOC_RESOURCE") : ((ptr_struct->opcode == 32794 ? (" LOCK_UNLOCK") : ((ptr_struct->opcode == 32795 ? (" DB_RECOVERY") : ((ptr_struct->opcode == 32796 ? (" READ_ICMC_TAG") : ((ptr_struct->opcode == 32798 ? (" ACCESS_HOST_MEM_FULL") : ((ptr_struct->opcode == 32800 ? (" RW_GVMIX") : ((ptr_struct->opcode == 32801 ? (" ICMC_LOCKED_HISTOGRAM") : ((ptr_struct->opcode == 32802 ? (" DC_CONFIG") : ((ptr_struct->opcode == 32803 ? (" PUSH_DB") : ((ptr_struct->opcode == 32804 ? (" GET_OPEN_RESOURCES") : ((ptr_struct->opcode == 32805 ? (" RW_ESW") : ((ptr_struct->opcode == 32806 ? (" ACCESS_REG_PMTU") : ((ptr_struct->opcode == 32807 ? (" ICMD_OPCODE_ACCESS_REGISTER_PAOS") : ((ptr_struct->opcode == 32808 ? (" ACCESS_REG_PPCNT") : ((ptr_struct->opcode == 32809 ? (" ACCESS_REG_PTYS") : ((ptr_struct->opcode == 32816 ? (" ICMD_OPCODE_ACCESS_REGISTER_SLRP") : ((ptr_struct->opcode == 32817 ? (" ICMD_OPCODE_ACCESS_REGISTER_SLTP") : ((ptr_struct->opcode == 32818 ? (" ICMD_OPCODE_ACCESS_REGISTER_SLRG") : ((ptr_struct->opcode == 32819 ? (" ICMD_OPCODE_ACCESS_REGISTER_PTAS") : ((ptr_struct->opcode == 32820 ? (" ICMD_OPCODE_ACCESS_REGISTER_PPAMP") : ((ptr_struct->opcode == 32821 ? (" ICMD_OPCODE_ACCESS_REGISTER_PPLR") : ((ptr_struct->opcode == 32822 ? (" ICMD_OPCODE_ACCESS_REGISTER_PLPC") : ((ptr_struct->opcode == 32823 ? (" ICMD_OPCODE_ACCESS_REGISTER_PPLM") : ((ptr_struct->opcode == 32824 ? (" ICMD_OPCODE_ACCESS_REGISTER_PPTT") : ((ptr_struct->opcode == 32825 ? (" ICMD_OPCODE_ACCESS_REGISTER_PPRT") : ((ptr_struct->opcode == 32832 ? (" ICMD_OPCODE_ACCESS_REGISTER_PPAOS") : ((ptr_struct->opcode == 32810 ? (" ACCESS_REG_PVLC") : ((ptr_struct->opcode == 33024 ? (" DEBUG_PRINT_ICM") : ((ptr_struct->opcode == 33025 ? (" GET_HEADER_CAPTURE") : ((ptr_struct->opcode == 33026 ? (" GEN_EQE") : ((ptr_struct->opcode == 33280 ? (" ICMD_OPCODE_SET_GET_PORT_CNTR") : ((ptr_struct->opcode == 33281 ? (" ICMD_OPCODE_GET_PF_CTX") : ((ptr_struct->opcode == 33282 ? (" GET_PI_CI") : ((ptr_struct->opcode == 33536 ? (" GET_FT_LIST") : ((ptr_struct->opcode == 33537 ? (" GET_FT_INFO") : ((ptr_struct->opcode == 33538 ? (" GET_FG_LIST") : ((ptr_struct->opcode == 33539 ? (" GET_FG") : ((ptr_struct->opcode == 33540 ? (" GET_FTE_LIST") : ((ptr_struct->opcode == 33541 ? (" GET_FTE") : ((ptr_struct->opcode == 33542 ? (" GET_STE_OPEN_RESOURCES") : ((ptr_struct->opcode == 33543 ? (" GET_STE_RESOURCES_LIST") : ((ptr_struct->opcode == 33544 ? (" READ_STE") : ((ptr_struct->opcode == 33545 ? (" STEERING_INFO") : ((ptr_struct->opcode == 33546 ? (" PARSE_ICM_ADDRESS") : ((ptr_struct->opcode == 33547 ? (" ICMD_OPCODE_DEBUG_LLDP") : ((ptr_struct->opcode == 33548 ? (" ICMD_OPCODE_GET_SET_LLDP_TLV") : ((ptr_struct->opcode == 33549 ? (" ICMD_OPCODE_DEBUG_MCTP_VND_PCI") : ((ptr_struct->opcode == 33550 ? (" EXECUTE_FW_LOG") : ((ptr_struct->opcode == 33551 ? (" PACKET_DROP") : ((ptr_struct->opcode == 33552 ? (" GET_HIGHEST_INDEX") : ((ptr_struct->opcode == 33792 ? (" ICMD_QUERY_CAPABILITIES") : ((ptr_struct->opcode == 33793 ? (" ICMD_KDNET_CTRL") : ((ptr_struct->opcode == 33794 ? (" ICMD_OPCODE_MH_SYNC") : ((ptr_struct->opcode == 33795 ? (" ICMD_OPCODE_MH_SYNC_STATUS") : ((ptr_struct->opcode == 36865 ? (" ICMD_ACCESS_REGISTER") : ((ptr_struct->opcode == 36866 ? (" ICMD_OPCODE_NVCONF_RW_TLV_DEBUG") : ((ptr_struct->opcode == 36867 ? (" ICMD_OPCODE_QUERY_VIRTUAL_MAC") : ((ptr_struct->opcode == 36868 ? (" ICMD_OPCODE_SET_VIRTUAL_MAC") : ((ptr_struct->opcode == 36869 ? (" ICMD_QUERY_WOL_ROL") : ((ptr_struct->opcode == 36870 ? (" ICMD_SET_WOL_ROL") : ((ptr_struct->opcode == 36871 ? (" ICMD_OCBB_INIT") : ((ptr_struct->opcode == 36872 ? (" ICMD_OCBB_QUERY_HEADER_STATUS") : ((ptr_struct->opcode == 36873 ? (" ICMD_OCBB_QUERY_ETOC_STATUS") : ((ptr_struct->opcode == 36874 ? (" ICMD_OCBB_SET_EVENT") : ((ptr_struct->opcode == 36907 ? (" ICMD_OPCODE_ACCESS_REGISTER_MLCR") : ((ptr_struct->opcode == 40961 ? (" FW_RESET") : ((ptr_struct->opcode == 40964 ? (" ICMD_OPCODE_PMAOS_WRITE") : ((ptr_struct->opcode == 40965 ? (" ICMD_OPCODE_PMAOS_QUERY") : ((ptr_struct->opcode == 40968 ? (" ICMD_OPCODE_MNG_EMAD_MCIA") : ((ptr_struct->opcode == 45056 ? (" READ_COMMAND") : ((ptr_struct->opcode == 49152 ? (" READ_EQ4TYPE") : ((ptr_struct->opcode == 49153 ? (" READ_SPECIAL_QPS") : ((ptr_struct->opcode == 49154 ? (" SET_PORT_SNIFFER") : ((ptr_struct->opcode == 49155 ? (" GET_PORT_SNIFFER") : ((ptr_struct->opcode == 49156 ? (" WRITE_CC_LOG") : ((ptr_struct->opcode == 49157 ? (" CONFIG_CC") : ((ptr_struct->opcode == 49158 ? (" WRITE_DC_CNAK_LID") : ((ptr_struct->opcode == 49159 ? (" UPDATE_RQ_CI") : ((ptr_struct->opcode == 49160 ? (" UD_TO_RAW") : ((ptr_struct->opcode == 61440 ? (" PHY_NEGO_CTRL") : ((ptr_struct->opcode == 61441 ? (" EYE_OPENER_READ") : ((ptr_struct->opcode == 61443 ? (" SET_ITRACE") : ((ptr_struct->opcode == 61444 ? (" INIT_OCSD") : ((ptr_struct->opcode == 61445 ? (" PHY_VER_OVERWRITE") : ((ptr_struct->opcode == 61446 ? (" PCIE_TOOLS_POINTERS") : ((ptr_struct->opcode == 61447 ? (" CODE_COVERAGE") : ((ptr_struct->opcode == 61448 ? (" CODE_COVERAGE_READ") : ((ptr_struct->opcode == 61449 ? (" PCIE_VER_DATA") : ((ptr_struct->opcode == 61450 ? (" VER_DATA") : ((ptr_struct->opcode == 61451 ? (" STOP_TOGGLER") : ((ptr_struct->opcode == 61452 ? (" ERROR_FLOW_TRIGGER") : ((ptr_struct->opcode == 61456 ? (" GREPPER") : ((ptr_struct->opcode == 61457 ? (" STANDBY") : ((ptr_struct->opcode == 61458 ? (" FLR_DEBUG") : ((ptr_struct->opcode == 61459 ? (" DCR_LL_TRVRS") : ((ptr_struct->opcode == 61460 ? (" DCR_LL_NEXT_DCR") : ((ptr_struct->opcode == 61461 ? (" ICMD_OPCODE_GENERAL_FAULT_INJECTOR") : ((ptr_struct->opcode == 61462 ? (" ACCESS_STEERING_ROOT") : ((ptr_struct->opcode == 61463 ? (" STEERING_TABLE_INFO") : ((ptr_struct->opcode == 61464 ? (" GET_GVMI_VIA_VSEC") : ((ptr_struct->opcode == 61465 ? (" ICMD_OPCODE_XPORT_CONFIG") : ((ptr_struct->opcode == 61466 ? (" ICMD_OPCODE_MINI_FLOW_ACTIVATOR") : ((ptr_struct->opcode == 61467 ? (" ICMD_OPCODE_ENABLE_TOOLPF") : ((ptr_struct->opcode == 61468 ? (" ICMD_OPCODE_SET_TOOLPF_TRACER") : ((ptr_struct->opcode == 61469 ? (" ICMD_OPCODE_RW_GVMI_FW_CONTEXT") : ((ptr_struct->opcode == 41216 ? (" SMBUS_MASTER_ACCESS") : ((ptr_struct->opcode == 41218 ? (" SMBUS_GW_FREQ") : ((ptr_struct->opcode == 41472 ? (" FPGA_FLEX_NIC_CTRL") : ((ptr_struct->opcode == 65369 ? (" ICMD_OPCODE_GVMI_RL_CTX_IX") : ((ptr_struct->opcode == 65376 ? (" ICMD_OPCODE_CABLE_INFO") : ((ptr_struct->opcode == 65377 ? (" ICMD_OPCODE_GEN_ASSERT") : ((ptr_struct->opcode == 65378 ? (" ICMD_OPCODE_DEBUG_PF_MAC") : ((ptr_struct->opcode == 65379 ? (" ICMD_OPCODE_SET_OC_INT_CAUSE") : ((ptr_struct->opcode == 65380 ? (" ICMD_OPCODE_GET_NODNIC_QUEUE_INFO") : ((ptr_struct->opcode == 65381 ? (" ICMD_OPCODE_PMLP_QUERY") : ((ptr_struct->opcode == 10248 ? (" ICMD_OPCODE_INIT_NCSI") : ((ptr_struct->opcode == 10249 ? (" ICMD_OPCODE_INJECT_NCSI") : ((ptr_struct->opcode == 65382 ? (" ICMD_OPCODE_READ_WRITE_VARB_CTX") : ((ptr_struct->opcode == 65383 ? (" ICMD_OPCODE_GET_PACKET_PACING_DEBUG_INFO") : ((ptr_struct->opcode == 65384 ? (" ICMD_OPCODE_DEBUG_CALC_SQN") : ((ptr_struct->opcode == 65385 ? (" ICMD_OPCODE_DETECT_GVMI") : ("unknown")))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))), ptr_struct->opcode);

}

int connectib_icmd_ctrl_size(void)
{
	 return 16;
}

void connectib_icmd_ctrl_dump(const struct connectib_icmd_ctrl *ptr_struct, FILE* file)
{
	connectib_icmd_ctrl_print(ptr_struct, file, 0);
}

void connectib_icmd_cmd_pack(const union connectib_icmd_cmd *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_icmd_get_gvmi_rl_ctx_ix_pack(&(ptr_struct->get_gvmi_rl_ctx_ix), ptr_buff);
}

void connectib_icmd_cmd_unpack(union connectib_icmd_cmd *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_icmd_get_gvmi_rl_ctx_ix_unpack(&(ptr_struct->get_gvmi_rl_ctx_ix), ptr_buff);
}

void connectib_icmd_cmd_print(const union connectib_icmd_cmd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd_cmd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "as_dwords:\n");
	connectib_icmd_cmd_as_dwords_print(&(ptr_struct->as_dwords), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_icm_ctx:\n");
	connectib_icmd_access_icm_ctx_print(&(ptr_struct->access_icm_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_pi_ci:\n");
	connectib_icmd_get_pi_ci_print(&(ptr_struct->get_pi_ci), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_icm_line:\n");
	connectib_icmd_access_icm_line_print(&(ptr_struct->access_icm_line), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_host_mem:\n");
	connectib_icmd_access_host_mem_print(&(ptr_struct->access_host_mem), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_q_entry:\n");
	connectib_icmd_read_q_entry_print(&(ptr_struct->read_q_entry), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_irisc_heart_beat:\n");
	connectib_icmd_get_irisc_heart_beat_print(&(ptr_struct->get_irisc_heart_beat), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_boot_stage:\n");
	connectib_icmd_get_boot_stage_print(&(ptr_struct->get_boot_stage), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_link_leds:\n");
	connectib_icmd_get_link_leds_print(&(ptr_struct->get_link_leds), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_fw_info:\n");
	connectib_icmd_get_fw_info_print(&(ptr_struct->get_fw_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_rx_slice_packet:\n");
	connectib_icmd_read_rx_slice_packet_print(&(ptr_struct->read_rx_slice_packet), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_g_rse_slice_desc:\n");
	connectib_icmd_read_g_rse_slice_desc_print(&(ptr_struct->read_g_rse_slice_desc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_reset:\n");
	connectib_icmd_fw_reset_print(&(ptr_struct->fw_reset), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "allowed_pfs_madifc:\n");
	connectib_icmd_allowed_madifc_on_all_pfs_print(&(ptr_struct->allowed_pfs_madifc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "debug_print_icm:\n");
	connectib_print_icm_print(&(ptr_struct->debug_print_icm), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "header_capture:\n");
	connectib_icmd_header_capture_print(&(ptr_struct->header_capture), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_sx_wq_buffer:\n");
	connectib_icmd_read_sx_wq_buffer_print(&(ptr_struct->read_sx_wq_buffer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icmd_access_cmdq:\n");
	connectib_icmd_access_cmdq_print(&(ptr_struct->icmd_access_cmdq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eqn4type:\n");
	connectib_eqn4type_print(&(ptr_struct->eqn4type), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "special_qps:\n");
	connectib_icmd_special_qps_print(&(ptr_struct->special_qps), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_port_sniffer:\n");
	connectib_icmd_set_port_sniffer_print(&(ptr_struct->set_port_sniffer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_port_sniffer:\n");
	connectib_icmd_get_port_sniffer_print(&(ptr_struct->get_port_sniffer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "congestion_log:\n");
	connectib_icmd_congestion_log_print(&(ptr_struct->congestion_log), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "write_dc_cnak_lid:\n");
	connectib_icmd_write_dc_cnak_lid_print(&(ptr_struct->write_dc_cnak_lid), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "update_rq_ci:\n");
	connectib_icmd_update_rq_ci_print(&(ptr_struct->update_rq_ci), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ud_to_raw:\n");
	connectib_icmd_ud_to_raw_print(&(ptr_struct->ud_to_raw), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "config_cc:\n");
	connectib_icmd_config_cc_print(&(ptr_struct->config_cc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_icm_tlb_line:\n");
	connectib_icmd_read_icm_tlb_line_print(&(ptr_struct->read_icm_tlb_line), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_icm_dummy_addr:\n");
	connectib_icmd_read_icm_dummy_addr_print(&(ptr_struct->read_icm_dummy_addr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_open_resources:\n");
	connectib_icmd_get_open_resources_print(&(ptr_struct->get_open_resources), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_highest_index:\n");
	connectib_icmd_get_highest_index_print(&(ptr_struct->get_highest_index), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "debug_fw_tracer:\n");
	connectib_icmd_debug_fw_tracer_print(&(ptr_struct->debug_fw_tracer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "execute_fw_log:\n");
	connectib_icmd_execute_fw_log_print(&(ptr_struct->execute_fw_log), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grepper:\n");
	connectib_icmd_grepper_print(&(ptr_struct->grepper), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "standby:\n");
	connectib_icmd_standby_print(&(ptr_struct->standby), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cable_plug_out_in:\n");
	connectib_icmd_cable_plug_out_in_print(&(ptr_struct->cable_plug_out_in), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cable_info:\n");
	connectib_icmd_cable_info_print(&(ptr_struct->cable_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "emad_mcia:\n");
	connectib_icmd_emad_mcia_print(&(ptr_struct->emad_mcia), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_icm_res_prop:\n");
	connectib_icmd_get_icm_res_prop_print(&(ptr_struct->get_icm_res_prop), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "alloc_dealloc_resource:\n");
	connectib_icmd_alloc_dealloc_resource_print(&(ptr_struct->alloc_dealloc_resource), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eye_opening_read:\n");
	connectib_eye_opening_read_st_print(&(ptr_struct->eye_opening_read), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_nego_ctrl:\n");
	connectib_phy_nego_ctrl_st_print(&(ptr_struct->phy_nego_ctrl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_icm_ctx_addr:\n");
	connectib_icmd_get_icm_ctx_addr_print(&(ptr_struct->get_icm_ctx_addr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "query_capabilities:\n");
	connectib_icmd_query_capabilities_print(&(ptr_struct->query_capabilities), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "kdnet_ctrl:\n");
	connectib_icmd_kdnet_ctrl_print(&(ptr_struct->kdnet_ctrl), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register:\n");
	connectib_icmd_generic_access_register_print(&(ptr_struct->access_register), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "check_sqpc_status:\n");
	connectib_icmd_check_sqpc_status_print(&(ptr_struct->check_sqpc_status), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "lock_unlock_resource:\n");
	connectib_icmd_lock_unlock_resource_print(&(ptr_struct->lock_unlock_resource), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "db_recovery:\n");
	connectib_icmd_check_sqpc_status_print(&(ptr_struct->db_recovery), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_icmc_tag:\n");
	connectib_icmd_access_icm_line_print(&(ptr_struct->read_icmc_tag), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_icmc_tags_and_info:\n");
	connectib_icmd_read_icmc_tags_and_info_print(&(ptr_struct->read_icmc_tags_and_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dc_config:\n");
	connectib_icmd_dc_config_print(&(ptr_struct->dc_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_itrace:\n");
	connectib_itrace_print(&(ptr_struct->set_itrace), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "init_ncsi:\n");
	connectib_init_ncsi_print(&(ptr_struct->init_ncsi), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "init_ocsd:\n");
	connectib_icmd_init_ocsd_print(&(ptr_struct->init_ocsd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "phy_ver_overwrite:\n");
	connectib_icmd_phy_ver_overwrite_print(&(ptr_struct->phy_ver_overwrite), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_tools_pointers:\n");
	connectib_pcie_tools_pointers_print(&(ptr_struct->pcie_tools_pointers), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gen_eqe:\n");
	connectib_icmd_gen_eqe_print(&(ptr_struct->gen_eqe), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_gvmix:\n");
	connectib_icmd_rw_gvmix_print(&(ptr_struct->rw_gvmix), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_esw:\n");
	connectib_icmd_esw_print(&(ptr_struct->rw_esw), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "code_coverage:\n");
	connectib_icmd_code_coverage_control_print(&(ptr_struct->code_coverage), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "code_coverage_read:\n");
	connectib_icmd_code_coverage_read_print(&(ptr_struct->code_coverage_read), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "stop_toggler:\n");
	connectib_icmd_stop_toggler_print(&(ptr_struct->stop_toggler), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "error_flow_trigger:\n");
	connectib_icmd_error_flow_trigger_print(&(ptr_struct->error_flow_trigger), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pcie_ver_data:\n");
	connectib_icmd_pcie_ver_data_print(&(ptr_struct->pcie_ver_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ver_data:\n");
	connectib_icmd_ver_data_print(&(ptr_struct->ver_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "push_db:\n");
	connectib_icmd_push_db_print(&(ptr_struct->push_db), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flr_debug:\n");
	connectib_icmd_flr_debug_print(&(ptr_struct->flr_debug), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcr_ll_trvrs:\n");
	connectib_icmd_dcr_ll_trvrs_print(&(ptr_struct->dcr_ll_trvrs), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dcr_ll_next_dcr_list:\n");
	connectib_icmd_dcr_ll_next_dcr_list_print(&(ptr_struct->dcr_ll_next_dcr_list), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_steering_root:\n");
	connectib_icmd_access_steering_root_print(&(ptr_struct->access_steering_root), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_steering_table_info:\n");
	connectib_icmd_steering_table_info_print(&(ptr_struct->get_steering_table_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_gvmi_via_vsec:\n");
	connectib_icmd_get_gvmi_via_vsec_print(&(ptr_struct->get_gvmi_via_vsec), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xport_config:\n");
	connectib_icmd_xport_xonfig_print(&(ptr_struct->xport_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nv_ram_rw_tlv:\n");
	connectib_icmd_nvcfg_rw_tlv_print(&(ptr_struct->nv_ram_rw_tlv), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_toolpf_tracer:\n");
	connectib_icmd_set_toolpf_tracer_print(&(ptr_struct->set_toolpf_tracer), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "enable_tools_pf:\n");
	connectib_icmd_enable_tools_pf_print(&(ptr_struct->enable_tools_pf), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_paos:\n");
	connectib_icmd_access_register_paos_print(&(ptr_struct->access_register_paos), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_slrp:\n");
	connectib_icmd_access_register_slrp_print(&(ptr_struct->access_register_slrp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_sltp:\n");
	connectib_icmd_access_register_sltp_print(&(ptr_struct->access_register_sltp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_slrg:\n");
	connectib_icmd_access_register_slrg_print(&(ptr_struct->access_register_slrg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pptt:\n");
	connectib_icmd_access_register_pptt_print(&(ptr_struct->access_register_pptt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pprt:\n");
	connectib_icmd_access_register_pprt_print(&(ptr_struct->access_register_pprt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ppaos:\n");
	connectib_icmd_access_register_ppaos_print(&(ptr_struct->access_register_ppaos), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ptas:\n");
	connectib_icmd_access_register_ptas_print(&(ptr_struct->access_register_ptas), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ppamp:\n");
	connectib_icmd_access_register_ppamp_print(&(ptr_struct->access_register_ppamp), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pplr:\n");
	connectib_icmd_access_register_pplr_print(&(ptr_struct->access_register_pplr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pplm:\n");
	connectib_icmd_access_register_pplm_print(&(ptr_struct->access_register_pplm), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_plpc:\n");
	connectib_icmd_access_register_plpc_print(&(ptr_struct->access_register_plpc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_mlcr:\n");
	connectib_icmd_access_register_mlcr_print(&(ptr_struct->access_register_mlcr), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pmtu:\n");
	connectib_icmd_access_register_pmtu_print(&(ptr_struct->access_register_pmtu), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ptys:\n");
	connectib_icmd_access_reg_ptys_print(&(ptr_struct->access_register_ptys), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "restore_phy_registers:\n");
	connectib_restore_phy_registers_print(&(ptr_struct->restore_phy_registers), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measure_frequency:\n");
	connectib_icmd_measure_frequency_print(&(ptr_struct->measure_frequency), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_ppcnt:\n");
	connectib_icmd_access_register_ppcnt_print(&(ptr_struct->access_register_ppcnt), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_get_port_counter:\n");
	connectib_icmd_set_get_port_cntr_print(&(ptr_struct->set_get_port_counter), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "led_cmd:\n");
	connectib_icmd_led_cmd_print(&(ptr_struct->led_cmd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "led_control_mad:\n");
	connectib_icmd_led_control_mad_print(&(ptr_struct->led_control_mad), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_ft_list:\n");
	connectib_icmd_get_ft_list_print(&(ptr_struct->get_ft_list), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_ft_info:\n");
	connectib_icmd_get_ft_info_print(&(ptr_struct->get_ft_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_fg_list:\n");
	connectib_icmd_get_fg_list_print(&(ptr_struct->get_fg_list), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_fg:\n");
	connectib_icmd_get_fg_print(&(ptr_struct->get_fg), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_fte_list:\n");
	connectib_icmd_get_fte_list_print(&(ptr_struct->get_fte_list), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_fte:\n");
	connectib_icmd_get_fte_print(&(ptr_struct->get_fte), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_ste_open_resources:\n");
	connectib_icmd_get_ste_open_resources_print(&(ptr_struct->get_ste_open_resources), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_ste_resources_list:\n");
	connectib_icmd_get_ste_resources_list_print(&(ptr_struct->get_ste_resources_list), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_ste:\n");
	connectib_icmd_read_ste_print(&(ptr_struct->read_ste), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "steering_info:\n");
	connectib_icmd_get_steering_info_print(&(ptr_struct->steering_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "parse_icm_address:\n");
	connectib_icmd_parse_icm_address_print(&(ptr_struct->parse_icm_address), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "access_register_pvlc:\n");
	connectib_icmd_access_register_pvlc_print(&(ptr_struct->access_register_pvlc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "general_fault_injector:\n");
	connectib_icmd_general_fault_injector_print(&(ptr_struct->general_fault_injector), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mini_flow_activator:\n");
	connectib_icmd_mini_flow_activator_print(&(ptr_struct->mini_flow_activator), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "calc_sqn:\n");
	connectib_icmd_debug_calc_sqn_print(&(ptr_struct->calc_sqn), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "read_write_varb_ctx:\n");
	connectib_icmd_read_write_varb_ctx_print(&(ptr_struct->read_write_varb_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gen_assert:\n");
	connectib_icmd_gen_assert_print(&(ptr_struct->gen_assert), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "set_oc_int:\n");
	connectib_icmd_set_oc_int_print(&(ptr_struct->set_oc_int), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_fw_driver_ver:\n");
	connectib_icmd_get_fw_driver_version_print(&(ptr_struct->get_fw_driver_ver), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pmlp_query:\n");
	connectib_icmd_pmlp_query_print(&(ptr_struct->pmlp_query), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pre_define_module_db:\n");
	connectib_icmd_pre_define_module_db_print(&(ptr_struct->pre_define_module_db), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ocbb_rndc_get_core_api:\n");
	connectib_icmd_ocbb_rndc_get_core_api_print(&(ptr_struct->ocbb_rndc_get_core_api), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ocbb_get_pci_api:\n");
	connectib_icmd_ocbb_get_pci_api_print(&(ptr_struct->ocbb_get_pci_api), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_pf_ctx:\n");
	connectib_icmd_get_pf_ctx_print(&(ptr_struct->get_pf_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "debug_pf_mac:\n");
	connectib_icmd_set_get_debug_pf_mac_print(&(ptr_struct->debug_pf_mac), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nodnic_queue_info:\n");
	connectib_icmd_access_register_nodnic_queue_info_print(&(ptr_struct->nodnic_queue_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "temp_warn_gen:\n");
	connectib_icmd_temp_warn_gen_event_print(&(ptr_struct->temp_warn_gen), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "detect_gvmi_in_hw:\n");
	connectib_icmd_detect_gvmi_in_qpc_and_icmc_print(&(ptr_struct->detect_gvmi_in_hw), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "internal_query_cap:\n");
	connectib_icmd_internal_query_cap_print(&(ptr_struct->internal_query_cap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "map_event_eq:\n");
	connectib_icmd_map_event_eq_print(&(ptr_struct->map_event_eq), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_packet_pacing_debug_info:\n");
	connectib_icmd_get_packet_pacing_debug_info_print(&(ptr_struct->get_packet_pacing_debug_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "port_admin_state_mng:\n");
	connectib_icmd_port_admin_state_mng_print(&(ptr_struct->port_admin_state_mng), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "asn1_force_port_state:\n");
	connectib_icmd_asn1_force_port_state_print(&(ptr_struct->asn1_force_port_state), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mh_sync:\n");
	connectib_icmd_mh_sync_print(&(ptr_struct->mh_sync), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rw_gvmi_fw_ctx:\n");
	connectib_icmd_rw_gvmi_fw_ctx_print(&(ptr_struct->rw_gvmi_fw_ctx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "drop_counter_read:\n");
	connectib_icmd_drop_counter_read_print(&(ptr_struct->drop_counter_read), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sysport:\n");
	connectib_icmd_sysport_print(&(ptr_struct->sysport), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "get_gvmi_rl_ctx_ix:\n");
	connectib_icmd_get_gvmi_rl_ctx_ix_print(&(ptr_struct->get_gvmi_rl_ctx_ix), file, indent_level + 1);

}

int connectib_icmd_cmd_size(void)
{
	 return 768;
}

void connectib_icmd_cmd_dump(const union connectib_icmd_cmd *ptr_struct, FILE* file)
{
	connectib_icmd_cmd_print(ptr_struct, file, 0);
}

void connectib_sw_cqe_sigerr_pack(const struct connectib_sw_cqe_sigerr *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_9);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_8);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_7);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_6);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->expected_trans_sig);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->actual_trans_sig);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->expected_ref_tag);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->actual_ref_tag);

	offset = 272;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rsvd_5);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->syndrome);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->mkey);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sig_err_byte_h);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->sig_err_byte_l);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_4);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_3);

	offset = 456;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 448;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->rsvd_2);

	offset = 511;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->owner);

	offset = 508;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->rsvd_0);

	offset = 504;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->opcode);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->signature);

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rsvd_1);

}

void connectib_sw_cqe_sigerr_unpack(struct connectib_sw_cqe_sigerr *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->rsvd_9 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->rsvd_8 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->rsvd_7 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->rsvd_6 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->expected_trans_sig = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->actual_trans_sig = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->expected_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->actual_ref_tag = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 272;
	ptr_struct->rsvd_5 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 256;
	ptr_struct->syndrome = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 288;
	ptr_struct->mkey = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->sig_err_byte_h = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->sig_err_byte_l = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->rsvd_4 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->rsvd_3 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 456;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 448;
	ptr_struct->rsvd_2 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 511;
	ptr_struct->owner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 508;
	ptr_struct->rsvd_0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 504;
	ptr_struct->opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 496;
	ptr_struct->signature = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 480;
	ptr_struct->rsvd_1 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_sw_cqe_sigerr_print(const struct connectib_sw_cqe_sigerr *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_cqe_sigerr ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_9               : " U32H_FMT "\n", ptr_struct->rsvd_9);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_8               : " U32H_FMT "\n", ptr_struct->rsvd_8);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_7               : " U32H_FMT "\n", ptr_struct->rsvd_7);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_6               : " U32H_FMT "\n", ptr_struct->rsvd_6);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "expected_trans_sig   : " U32H_FMT "\n", ptr_struct->expected_trans_sig);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "actual_trans_sig     : " U32H_FMT "\n", ptr_struct->actual_trans_sig);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "expected_ref_tag     : " U32H_FMT "\n", ptr_struct->expected_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "actual_ref_tag       : " U32H_FMT "\n", ptr_struct->actual_ref_tag);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_5               : " UH_FMT "\n", ptr_struct->rsvd_5);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "syndrome             : " UH_FMT "\n", ptr_struct->syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mkey                 : " U32H_FMT "\n", ptr_struct->mkey);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sig_err_byte_h       : " U32H_FMT "\n", ptr_struct->sig_err_byte_h);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sig_err_byte_l       : " U32H_FMT "\n", ptr_struct->sig_err_byte_l);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_4               : " U32H_FMT "\n", ptr_struct->rsvd_4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_3               : " U32H_FMT "\n", ptr_struct->rsvd_3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_2               : " UH_FMT "\n", ptr_struct->rsvd_2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "owner                : " UH_FMT "\n", ptr_struct->owner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_0               : " UH_FMT "\n", ptr_struct->rsvd_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " UH_FMT "\n", ptr_struct->opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature            : " UH_FMT "\n", ptr_struct->signature);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_1               : " UH_FMT "\n", ptr_struct->rsvd_1);

}

int connectib_sw_cqe_sigerr_size(void)
{
	 return 64;
}

void connectib_sw_cqe_sigerr_dump(const struct connectib_sw_cqe_sigerr *ptr_struct, FILE* file)
{
	connectib_sw_cqe_sigerr_print(ptr_struct, file, 0);
}

void connectib_sw_cqe_resize_pack(const struct connectib_sw_cqe_resize *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_14);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_13);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_12);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_11);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_10);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_9);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_8);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_7);

	offset = 256;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_6);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_5);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_4);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_3);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->timestamp_h);

	offset = 416;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->timestamp_l);

	offset = 448;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_2);

	offset = 511;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->owner);

	offset = 508;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->rsvd_1);

	offset = 504;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->opcode);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->signature);

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rsvd_0);

}

void connectib_sw_cqe_resize_unpack(struct connectib_sw_cqe_resize *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->rsvd_14 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->rsvd_13 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->rsvd_12 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->rsvd_11 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->rsvd_10 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->rsvd_9 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->rsvd_8 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->rsvd_7 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 256;
	ptr_struct->rsvd_6 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 288;
	ptr_struct->rsvd_5 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->rsvd_4 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->rsvd_3 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->timestamp_h = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->timestamp_l = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 448;
	ptr_struct->rsvd_2 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 511;
	ptr_struct->owner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 508;
	ptr_struct->rsvd_1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 504;
	ptr_struct->opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 496;
	ptr_struct->signature = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 480;
	ptr_struct->rsvd_0 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_sw_cqe_resize_print(const struct connectib_sw_cqe_resize *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_cqe_resize ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_14              : " U32H_FMT "\n", ptr_struct->rsvd_14);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_13              : " U32H_FMT "\n", ptr_struct->rsvd_13);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_12              : " U32H_FMT "\n", ptr_struct->rsvd_12);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_11              : " U32H_FMT "\n", ptr_struct->rsvd_11);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_10              : " U32H_FMT "\n", ptr_struct->rsvd_10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_9               : " U32H_FMT "\n", ptr_struct->rsvd_9);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_8               : " U32H_FMT "\n", ptr_struct->rsvd_8);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_7               : " U32H_FMT "\n", ptr_struct->rsvd_7);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_6               : " U32H_FMT "\n", ptr_struct->rsvd_6);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_5               : " U32H_FMT "\n", ptr_struct->rsvd_5);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_4               : " U32H_FMT "\n", ptr_struct->rsvd_4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_3               : " U32H_FMT "\n", ptr_struct->rsvd_3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_h          : " U32H_FMT "\n", ptr_struct->timestamp_h);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "timestamp_l          : " U32H_FMT "\n", ptr_struct->timestamp_l);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_2               : " U32H_FMT "\n", ptr_struct->rsvd_2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "owner                : " UH_FMT "\n", ptr_struct->owner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_1               : " UH_FMT "\n", ptr_struct->rsvd_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "opcode               : " UH_FMT "\n", ptr_struct->opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature            : " UH_FMT "\n", ptr_struct->signature);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_0               : " UH_FMT "\n", ptr_struct->rsvd_0);

}

int connectib_sw_cqe_resize_size(void)
{
	 return 64;
}

void connectib_sw_cqe_resize_dump(const struct connectib_sw_cqe_resize *ptr_struct, FILE* file)
{
	connectib_sw_cqe_resize_print(ptr_struct, file, 0);
}

void connectib_sw_cqe_error_pack(const struct connectib_sw_cqe_error *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->wqe_id);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->rsvd_14);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_13);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_12);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_11);

	offset = 128;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_10);

	offset = 160;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_9);

	offset = 192;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_8);

	offset = 224;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_7);

	offset = 264;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->srq_num);

	offset = 256;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->rsvd_6);

	offset = 288;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_5);

	offset = 320;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_4);

	offset = 352;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->byte_cnt);

	offset = 384;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_2);

	offset = 440;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->syndrome);

	offset = 432;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vendor_error_syndrome);

	offset = 428;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rsvd_1);

	offset = 424;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->hw_syndrome_type);

	offset = 416;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->vendor_hw_syndrome);

	offset = 456;
	adb2c_push_bits_to_buff(ptr_buff, offset, 24, (u_int32_t)ptr_struct->qpn);

	offset = 448;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->swqe_opcode_drop_cntr);

	offset = 511;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->owner);

	offset = 510;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->se);

	offset = 508;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->rsvd_0);

	offset = 504;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->rwqe_opcode);

	offset = 496;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->signature);

	offset = 480;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->wqe_counter);

}

void connectib_sw_cqe_error_unpack(struct connectib_sw_cqe_error *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 16;
	ptr_struct->wqe_id = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 0;
	ptr_struct->rsvd_14 = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 32;
	ptr_struct->rsvd_13 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->rsvd_12 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->rsvd_11 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 128;
	ptr_struct->rsvd_10 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 160;
	ptr_struct->rsvd_9 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 192;
	ptr_struct->rsvd_8 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 224;
	ptr_struct->rsvd_7 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 264;
	ptr_struct->srq_num = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 256;
	ptr_struct->rsvd_6 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 288;
	ptr_struct->rsvd_5 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 320;
	ptr_struct->rsvd_4 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 352;
	ptr_struct->byte_cnt = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 384;
	ptr_struct->rsvd_2 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 440;
	ptr_struct->syndrome = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 432;
	ptr_struct->vendor_error_syndrome = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 428;
	ptr_struct->rsvd_1 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 424;
	ptr_struct->hw_syndrome_type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 416;
	ptr_struct->vendor_hw_syndrome = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 456;
	ptr_struct->qpn = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 24);

	offset = 448;
	ptr_struct->swqe_opcode_drop_cntr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 511;
	ptr_struct->owner = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 510;
	ptr_struct->se = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 508;
	ptr_struct->rsvd_0 = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 504;
	ptr_struct->rwqe_opcode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 496;
	ptr_struct->signature = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 480;
	ptr_struct->wqe_counter = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_sw_cqe_error_print(const struct connectib_sw_cqe_error *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_cqe_error ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_id               : " UH_FMT "\n", ptr_struct->wqe_id);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_14              : " UH_FMT "\n", ptr_struct->rsvd_14);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_13              : " U32H_FMT "\n", ptr_struct->rsvd_13);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_12              : " U32H_FMT "\n", ptr_struct->rsvd_12);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_11              : " U32H_FMT "\n", ptr_struct->rsvd_11);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_10              : " U32H_FMT "\n", ptr_struct->rsvd_10);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_9               : " U32H_FMT "\n", ptr_struct->rsvd_9);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_8               : " U32H_FMT "\n", ptr_struct->rsvd_8);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_7               : " U32H_FMT "\n", ptr_struct->rsvd_7);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_num              : " UH_FMT "\n", ptr_struct->srq_num);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_6               : " UH_FMT "\n", ptr_struct->rsvd_6);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_5               : " U32H_FMT "\n", ptr_struct->rsvd_5);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_4               : " U32H_FMT "\n", ptr_struct->rsvd_4);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "byte_cnt             : " U32H_FMT "\n", ptr_struct->byte_cnt);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_2               : " U32H_FMT "\n", ptr_struct->rsvd_2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "syndrome             : " UH_FMT "\n", ptr_struct->syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vendor_error_syndrome : " UH_FMT "\n", ptr_struct->vendor_error_syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_1               : " UH_FMT "\n", ptr_struct->rsvd_1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_syndrome_type     : " UH_FMT "\n", ptr_struct->hw_syndrome_type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "vendor_hw_syndrome   : " UH_FMT "\n", ptr_struct->vendor_hw_syndrome);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "qpn                  : " UH_FMT "\n", ptr_struct->qpn);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "swqe_opcode_drop_cntr : " UH_FMT "\n", ptr_struct->swqe_opcode_drop_cntr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "owner                : " UH_FMT "\n", ptr_struct->owner);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "se                   : " UH_FMT "\n", ptr_struct->se);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rsvd_0               : " UH_FMT "\n", ptr_struct->rsvd_0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rwqe_opcode          : " UH_FMT "\n", ptr_struct->rwqe_opcode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature            : " UH_FMT "\n", ptr_struct->signature);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "wqe_counter          : " UH_FMT "\n", ptr_struct->wqe_counter);

}

int connectib_sw_cqe_error_size(void)
{
	 return 64;
}

void connectib_sw_cqe_error_dump(const struct connectib_sw_cqe_error *ptr_struct, FILE* file)
{
	connectib_sw_cqe_error_print(ptr_struct, file, 0);
}

void connectib_sw_cqe_128b_pack(const struct connectib_sw_cqe_128b *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_grh_inline_data_pack(&(ptr_struct->grh_inline_data), ptr_buff + offset/8);

	offset = 512;
	connectib_sw_cqe_64b_pack(&(ptr_struct->cqe_64b), ptr_buff + offset/8);

}

void connectib_sw_cqe_128b_unpack(struct connectib_sw_cqe_128b *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_grh_inline_data_unpack(&(ptr_struct->grh_inline_data), ptr_buff + offset/8);

	offset = 512;
	connectib_sw_cqe_64b_unpack(&(ptr_struct->cqe_64b), ptr_buff + offset/8);

}

void connectib_sw_cqe_128b_print(const struct connectib_sw_cqe_128b *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_cqe_128b ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "grh_inline_data:\n");
	connectib_grh_inline_data_print(&(ptr_struct->grh_inline_data), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_64b:\n");
	connectib_sw_cqe_64b_print(&(ptr_struct->cqe_64b), file, indent_level + 1);

}

int connectib_sw_cqe_128b_size(void)
{
	 return 128;
}

void connectib_sw_cqe_128b_dump(const struct connectib_sw_cqe_128b *ptr_struct, FILE* file)
{
	connectib_sw_cqe_128b_print(ptr_struct, file, 0);
}

void connectib_itoc_entry_pack(const struct connectib_itoc_entry *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	adb2c_push_bits_to_buff(ptr_buff, offset, 22, (u_int32_t)ptr_struct->size);

	offset = 0;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->type);

	offset = 34;
	adb2c_push_bits_to_buff(ptr_buff, offset, 30, (u_int32_t)ptr_struct->param0);

	offset = 33;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->cache_line_crc);

	offset = 32;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->zipped_image);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->param1);

	offset = 161;
	adb2c_push_bits_to_buff(ptr_buff, offset, 29, (u_int32_t)ptr_struct->flash_addr);

	offset = 160;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->relative_addr);

	offset = 208;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->section_crc);

	offset = 207;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->no_crc);

	offset = 206;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->device_data);

	offset = 240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->itoc_entry_crc);

}

void connectib_itoc_entry_unpack(struct connectib_itoc_entry *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 8;
	ptr_struct->size = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 22);

	offset = 0;
	ptr_struct->type = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 34;
	ptr_struct->param0 = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 30);

	offset = 33;
	ptr_struct->cache_line_crc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 32;
	ptr_struct->zipped_image = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 64;
	ptr_struct->param1 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 161;
	ptr_struct->flash_addr = (u_int32_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 29);

	offset = 160;
	ptr_struct->relative_addr = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 208;
	ptr_struct->section_crc = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

	offset = 207;
	ptr_struct->no_crc = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 206;
	ptr_struct->device_data = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 240;
	ptr_struct->itoc_entry_crc = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_itoc_entry_print(const struct connectib_itoc_entry *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_itoc_entry ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "size                 : " UH_FMT "\n", ptr_struct->size);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "type                 : " UH_FMT "\n", ptr_struct->type);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "param0               : " UH_FMT "\n", ptr_struct->param0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cache_line_crc       : " UH_FMT "\n", ptr_struct->cache_line_crc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "zipped_image         : " UH_FMT "\n", ptr_struct->zipped_image);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "param1               : " U32H_FMT "\n", ptr_struct->param1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "flash_addr           : " UH_FMT "\n", ptr_struct->flash_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "relative_addr        : " UH_FMT "\n", ptr_struct->relative_addr);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "section_crc          : " UH_FMT "\n", ptr_struct->section_crc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "no_crc               : " UH_FMT "\n", ptr_struct->no_crc);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_data          : " UH_FMT "\n", ptr_struct->device_data);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "itoc_entry_crc       : " UH_FMT "\n", ptr_struct->itoc_entry_crc);

}

int connectib_itoc_entry_size(void)
{
	 return 32;
}

void connectib_itoc_entry_dump(const struct connectib_itoc_entry *ptr_struct, FILE* file)
{
	connectib_itoc_entry_print(ptr_struct, file, 0);
}

void connectib_itoc_header_pack(const struct connectib_itoc_header *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature0);

	offset = 32;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature1);

	offset = 64;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature2);

	offset = 96;
	adb2c_push_integer_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->signature3);

	offset = 152;
	adb2c_push_bits_to_buff(ptr_buff, offset, 8, (u_int32_t)ptr_struct->version);

	offset = 240;
	adb2c_push_bits_to_buff(ptr_buff, offset, 16, (u_int32_t)ptr_struct->itoc_entry_crc);

}

void connectib_itoc_header_unpack(struct connectib_itoc_header *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	ptr_struct->signature0 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 32;
	ptr_struct->signature1 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 64;
	ptr_struct->signature2 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 96;
	ptr_struct->signature3 = (u_int32_t)adb2c_pop_integer_from_buff(ptr_buff, offset, 4);

	offset = 152;
	ptr_struct->version = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 8);

	offset = 240;
	ptr_struct->itoc_entry_crc = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 16);

}

void connectib_itoc_header_print(const struct connectib_itoc_header *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_itoc_header ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature0           : " U32H_FMT "\n", ptr_struct->signature0);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature1           : " U32H_FMT "\n", ptr_struct->signature1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature2           : " U32H_FMT "\n", ptr_struct->signature2);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature3           : " U32H_FMT "\n", ptr_struct->signature3);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "version              : " UH_FMT "\n", ptr_struct->version);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "itoc_entry_crc       : " UH_FMT "\n", ptr_struct->itoc_entry_crc);

}

int connectib_itoc_header_size(void)
{
	 return 32;
}

void connectib_itoc_header_dump(const struct connectib_itoc_header *ptr_struct, FILE* file)
{
	connectib_itoc_header_print(ptr_struct, file, 0);
}

void connectib_sw_wqe_pack(const struct connectib_sw_wqe *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_wqe_control_pack(&(ptr_struct->conrtol), ptr_buff + offset/8);

	offset = 128;
	connectib_wqe_mlx_pack(&(ptr_struct->mlx), ptr_buff + offset/8);

	offset = 256;
	connectib_wqe_xrc_pack(&(ptr_struct->xrc), ptr_buff + offset/8);

	offset = 384;
	connectib_wqe_address_vector_basic_pack(&(ptr_struct->address_vector_basic), ptr_buff + offset/8);

	offset = 512;
	connectib_wqe_address_vector_ext_pack(&(ptr_struct->address_vector_ext), ptr_buff + offset/8);

	offset = 768;
	connectib_wqe_rdma_read_pack(&(ptr_struct->rdma_read), ptr_buff + offset/8);

	offset = 896;
	connectib_wqe_atomic_cmp_swap_pack(&(ptr_struct->atomic_cmp_swap), ptr_buff + offset/8);

	offset = 1024;
	connectib_wqe_extended_atomic_cmp_swap_pack(&(ptr_struct->ext_atom_cmp_swap), ptr_buff + offset/8);

	offset = 5120;
	connectib_wqe_atomic_fetch_add_pack(&(ptr_struct->atomic_fetch_add), ptr_buff + offset/8);

	offset = 5248;
	connectib_wqe_extended_atomic_fetch_add_pack(&(ptr_struct->ext_atom_fetch_add), ptr_buff + offset/8);

	offset = 7296;
	connectib_wqe_signature_pack(&(ptr_struct->signature), ptr_buff + offset/8);

	offset = 7424;
	connectib_wqe_srq_next_pack(&(ptr_struct->srq_next), ptr_buff + offset/8);

	offset = 7552;
	connectib_wqe_gather_scatter_entry_pack(&(ptr_struct->gather_scatter_entry), ptr_buff + offset/8);

	offset = 7680;
	connectib_wqe_inline_header_pack(&(ptr_struct->inline_header), ptr_buff + offset/8);

	offset = 7808;
	connectib_wqe_eth_segment_pack(&(ptr_struct->eth_segment), ptr_buff + offset/8);

}

void connectib_sw_wqe_unpack(struct connectib_sw_wqe *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_wqe_control_unpack(&(ptr_struct->conrtol), ptr_buff + offset/8);

	offset = 128;
	connectib_wqe_mlx_unpack(&(ptr_struct->mlx), ptr_buff + offset/8);

	offset = 256;
	connectib_wqe_xrc_unpack(&(ptr_struct->xrc), ptr_buff + offset/8);

	offset = 384;
	connectib_wqe_address_vector_basic_unpack(&(ptr_struct->address_vector_basic), ptr_buff + offset/8);

	offset = 512;
	connectib_wqe_address_vector_ext_unpack(&(ptr_struct->address_vector_ext), ptr_buff + offset/8);

	offset = 768;
	connectib_wqe_rdma_read_unpack(&(ptr_struct->rdma_read), ptr_buff + offset/8);

	offset = 896;
	connectib_wqe_atomic_cmp_swap_unpack(&(ptr_struct->atomic_cmp_swap), ptr_buff + offset/8);

	offset = 1024;
	connectib_wqe_extended_atomic_cmp_swap_unpack(&(ptr_struct->ext_atom_cmp_swap), ptr_buff + offset/8);

	offset = 5120;
	connectib_wqe_atomic_fetch_add_unpack(&(ptr_struct->atomic_fetch_add), ptr_buff + offset/8);

	offset = 5248;
	connectib_wqe_extended_atomic_fetch_add_unpack(&(ptr_struct->ext_atom_fetch_add), ptr_buff + offset/8);

	offset = 7296;
	connectib_wqe_signature_unpack(&(ptr_struct->signature), ptr_buff + offset/8);

	offset = 7424;
	connectib_wqe_srq_next_unpack(&(ptr_struct->srq_next), ptr_buff + offset/8);

	offset = 7552;
	connectib_wqe_gather_scatter_entry_unpack(&(ptr_struct->gather_scatter_entry), ptr_buff + offset/8);

	offset = 7680;
	connectib_wqe_inline_header_unpack(&(ptr_struct->inline_header), ptr_buff + offset/8);

	offset = 7808;
	connectib_wqe_eth_segment_unpack(&(ptr_struct->eth_segment), ptr_buff + offset/8);

}

void connectib_sw_wqe_print(const struct connectib_sw_wqe *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_wqe ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "conrtol:\n");
	connectib_wqe_control_print(&(ptr_struct->conrtol), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mlx:\n");
	connectib_wqe_mlx_print(&(ptr_struct->mlx), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "xrc:\n");
	connectib_wqe_xrc_print(&(ptr_struct->xrc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address_vector_basic:\n");
	connectib_wqe_address_vector_basic_print(&(ptr_struct->address_vector_basic), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "address_vector_ext:\n");
	connectib_wqe_address_vector_ext_print(&(ptr_struct->address_vector_ext), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "rdma_read:\n");
	connectib_wqe_rdma_read_print(&(ptr_struct->rdma_read), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "atomic_cmp_swap:\n");
	connectib_wqe_atomic_cmp_swap_print(&(ptr_struct->atomic_cmp_swap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ext_atom_cmp_swap:\n");
	connectib_wqe_extended_atomic_cmp_swap_print(&(ptr_struct->ext_atom_cmp_swap), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "atomic_fetch_add:\n");
	connectib_wqe_atomic_fetch_add_print(&(ptr_struct->atomic_fetch_add), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ext_atom_fetch_add:\n");
	connectib_wqe_extended_atomic_fetch_add_print(&(ptr_struct->ext_atom_fetch_add), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "signature:\n");
	connectib_wqe_signature_print(&(ptr_struct->signature), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "srq_next:\n");
	connectib_wqe_srq_next_print(&(ptr_struct->srq_next), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "gather_scatter_entry:\n");
	connectib_wqe_gather_scatter_entry_print(&(ptr_struct->gather_scatter_entry), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "inline_header:\n");
	connectib_wqe_inline_header_print(&(ptr_struct->inline_header), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "eth_segment:\n");
	connectib_wqe_eth_segment_print(&(ptr_struct->eth_segment), file, indent_level + 1);

}

int connectib_sw_wqe_size(void)
{
	 return 1024;
}

void connectib_sw_wqe_dump(const struct connectib_sw_wqe *ptr_struct, FILE* file)
{
	connectib_sw_wqe_print(ptr_struct, file, 0);
}

void connectib_ini_pack(const union connectib_ini *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_public_keys_pack(&(ptr_struct->public_keys), ptr_buff);
}

void connectib_ini_unpack(union connectib_ini *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_public_keys_unpack(&(ptr_struct->public_keys), ptr_buff);
}

void connectib_ini_print(const union connectib_ini *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_ini ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "boot_record:\n");
	connectib_boot_record_golan_print(&(ptr_struct->boot_record), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "reset_info:\n");
	connectib_reset_info_print(&(ptr_struct->reset_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dtoc_pointers:\n");
	connectib_dtoc_pointers_print(&(ptr_struct->dtoc_pointers), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "device_info:\n");
	connectib_device_info_print(&(ptr_struct->device_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "image_info:\n");
	connectib_image_info_print(&(ptr_struct->image_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "mfg_info:\n");
	connectib_mfg_info_print(&(ptr_struct->mfg_info), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_main_config:\n");
	connectib_fw_main_config_print(&(ptr_struct->fw_main_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_boot_config:\n");
	connectib_fw_boot_config_print(&(ptr_struct->fw_boot_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_main_config:\n");
	connectib_hw_main_config_print(&(ptr_struct->hw_main_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_boot_config:\n");
	connectib_hw_boot_config_print(&(ptr_struct->hw_boot_config), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nv_data_0:\n");
	connectib_nv_data_print(&(ptr_struct->nv_data_0), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "nv_data_1:\n");
	connectib_nv_data_print(&(ptr_struct->nv_data_1), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "fw_nv_log:\n");
	connectib_nv_data_print(&(ptr_struct->fw_nv_log), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "image_signature:\n");
	connectib_image_signature_print(&(ptr_struct->image_signature), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "public_keys:\n");
	connectib_public_keys_print(&(ptr_struct->public_keys), file, indent_level + 1);

}

int connectib_ini_size(void)
{
	 return 65536;
}

void connectib_ini_dump(const union connectib_ini *ptr_struct, FILE* file)
{
	connectib_ini_print(ptr_struct, file, 0);
}

void connectib_a2d_pack(const struct connectib_a2d *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->power_up);

	offset = 28;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->clock_div);

	offset = 24;
	adb2c_push_bits_to_buff(ptr_buff, offset, 4, (u_int32_t)ptr_struct->probe_sel);

	offset = 22;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->measure_mode);

	offset = 19;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->clock_sel);

	offset = 18;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->measure_polarity);

	offset = 62;
	adb2c_push_bits_to_buff(ptr_buff, offset, 2, (u_int32_t)ptr_struct->pad_out_sel);

	offset = 61;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->spike_direction);

	offset = 57;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->dynamic_range);

	offset = 51;
	adb2c_push_bits_to_buff(ptr_buff, offset, 3, (u_int32_t)ptr_struct->measure_serdes_probe);

	offset = 50;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->thermal_protect_en);

	offset = 85;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->measure_value);

	offset = 84;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->circuit_binary_done);

	offset = 83;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->a2d_compare);

	offset = 82;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->spike_detect);

	offset = 123;
	adb2c_push_bits_to_buff(ptr_buff, offset, 5, (u_int32_t)ptr_struct->stabilization_time);

	offset = 149;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->measure_min);

	offset = 148;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->measure_compare);

	offset = 181;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->measure_max);

	offset = 213;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->protect_value);

	offset = 212;
	adb2c_push_bits_to_buff(ptr_buff, offset, 1, (u_int32_t)ptr_struct->protect_polarity);

	offset = 245;
	adb2c_push_bits_to_buff(ptr_buff, offset, 11, (u_int32_t)ptr_struct->calib_value);

}

void connectib_a2d_unpack(struct connectib_a2d *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 31;
	ptr_struct->power_up = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 28;
	ptr_struct->clock_div = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 24;
	ptr_struct->probe_sel = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 4);

	offset = 22;
	ptr_struct->measure_mode = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 19;
	ptr_struct->clock_sel = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 18;
	ptr_struct->measure_polarity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 62;
	ptr_struct->pad_out_sel = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 2);

	offset = 61;
	ptr_struct->spike_direction = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 57;
	ptr_struct->dynamic_range = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 51;
	ptr_struct->measure_serdes_probe = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 3);

	offset = 50;
	ptr_struct->thermal_protect_en = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 85;
	ptr_struct->measure_value = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

	offset = 84;
	ptr_struct->circuit_binary_done = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 83;
	ptr_struct->a2d_compare = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 82;
	ptr_struct->spike_detect = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 123;
	ptr_struct->stabilization_time = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 5);

	offset = 149;
	ptr_struct->measure_min = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

	offset = 148;
	ptr_struct->measure_compare = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 181;
	ptr_struct->measure_max = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

	offset = 213;
	ptr_struct->protect_value = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

	offset = 212;
	ptr_struct->protect_polarity = (u_int8_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 1);

	offset = 245;
	ptr_struct->calib_value = (u_int16_t)adb2c_pop_bits_from_buff(ptr_buff, offset, 11);

}

void connectib_a2d_print(const struct connectib_a2d *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_a2d ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "power_up             : " UH_FMT "\n", ptr_struct->power_up);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "clock_div            : " UH_FMT "\n", ptr_struct->clock_div);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "probe_sel            : " UH_FMT "\n", ptr_struct->probe_sel);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measure_mode         : " UH_FMT "\n", ptr_struct->measure_mode);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "clock_sel            : " UH_FMT "\n", ptr_struct->clock_sel);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measure_polarity     : " UH_FMT "\n", ptr_struct->measure_polarity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "pad_out_sel          : " UH_FMT "\n", ptr_struct->pad_out_sel);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spike_direction      : " UH_FMT "\n", ptr_struct->spike_direction);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "dynamic_range        : " UH_FMT "\n", ptr_struct->dynamic_range);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measure_serdes_probe : " UH_FMT "\n", ptr_struct->measure_serdes_probe);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "thermal_protect_en   : " UH_FMT "\n", ptr_struct->thermal_protect_en);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measure_value        : " UH_FMT "\n", ptr_struct->measure_value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "circuit_binary_done  : " UH_FMT "\n", ptr_struct->circuit_binary_done);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a2d_compare          : " UH_FMT "\n", ptr_struct->a2d_compare);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "spike_detect         : " UH_FMT "\n", ptr_struct->spike_detect);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "stabilization_time   : " UH_FMT "\n", ptr_struct->stabilization_time);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measure_min          : " UH_FMT "\n", ptr_struct->measure_min);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measure_compare      : " UH_FMT "\n", ptr_struct->measure_compare);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "measure_max          : " UH_FMT "\n", ptr_struct->measure_max);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "protect_value        : " UH_FMT "\n", ptr_struct->protect_value);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "protect_polarity     : " UH_FMT "\n", ptr_struct->protect_polarity);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "calib_value          : " UH_FMT "\n", ptr_struct->calib_value);

}

int connectib_a2d_size(void)
{
	 return 32;
}

void connectib_a2d_dump(const struct connectib_a2d *ptr_struct, FILE* file)
{
	connectib_a2d_print(ptr_struct, file, 0);
}

void connectib_icmd_pack(const struct connectib_icmd *ptr_struct, u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_icmd_cmd_pack(&(ptr_struct->cmd), ptr_buff + offset/8);

	offset = 8064;
	connectib_icmd_ctrl_pack(&(ptr_struct->ctrl), ptr_buff + offset/8);

}

void connectib_icmd_unpack(struct connectib_icmd *ptr_struct, const u_int8_t* ptr_buff)
{
	u_int32_t offset;
	int i = 0;
	(void)offset;
	(void)i;
	(void)ptr_struct;
	(void)ptr_buff;

	offset = 0;
	connectib_icmd_cmd_unpack(&(ptr_struct->cmd), ptr_buff + offset/8);

	offset = 8064;
	connectib_icmd_ctrl_unpack(&(ptr_struct->ctrl), ptr_buff + offset/8);

}

void connectib_icmd_print(const struct connectib_icmd *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_icmd ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cmd:\n");
	connectib_icmd_cmd_print(&(ptr_struct->cmd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ctrl:\n");
	connectib_icmd_ctrl_print(&(ptr_struct->ctrl), file, indent_level + 1);

}

int connectib_icmd_size(void)
{
	 return 1024;
}

void connectib_icmd_dump(const struct connectib_icmd *ptr_struct, FILE* file)
{
	connectib_icmd_print(ptr_struct, file, 0);
}

void connectib_sw_cqe_pack(const union connectib_sw_cqe *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_sw_cqe_sigerr_pack(&(ptr_struct->cqe_sigerr), ptr_buff);
}

void connectib_sw_cqe_unpack(union connectib_sw_cqe *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_sw_cqe_sigerr_unpack(&(ptr_struct->cqe_sigerr), ptr_buff);
}

void connectib_sw_cqe_print(const union connectib_sw_cqe *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_sw_cqe ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_64b:\n");
	connectib_sw_cqe_64b_print(&(ptr_struct->cqe_64b), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_128b:\n");
	connectib_sw_cqe_128b_print(&(ptr_struct->cqe_128b), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_error:\n");
	connectib_sw_cqe_error_print(&(ptr_struct->cqe_error), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_resize:\n");
	connectib_sw_cqe_resize_print(&(ptr_struct->cqe_resize), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "cqe_sigerr:\n");
	connectib_sw_cqe_sigerr_print(&(ptr_struct->cqe_sigerr), file, indent_level + 1);

}

int connectib_sw_cqe_size(void)
{
	 return 128;
}

void connectib_sw_cqe_dump(const union connectib_sw_cqe *ptr_struct, FILE* file)
{
	connectib_sw_cqe_print(ptr_struct, file, 0);
}

void connectib_itoc_pack(const union connectib_itoc *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_itoc_entry_pack(&(ptr_struct->entry), ptr_buff);
}

void connectib_itoc_unpack(union connectib_itoc *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_itoc_entry_unpack(&(ptr_struct->entry), ptr_buff);
}

void connectib_itoc_print(const union connectib_itoc *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_itoc ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "header:\n");
	connectib_itoc_header_print(&(ptr_struct->header), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "entry:\n");
	connectib_itoc_entry_print(&(ptr_struct->entry), file, indent_level + 1);

}

int connectib_itoc_size(void)
{
	 return 32;
}

void connectib_itoc_dump(const union connectib_itoc *ptr_struct, FILE* file)
{
	connectib_itoc_print(ptr_struct, file, 0);
}

void connectib_connectib_Nodes_pack(const union connectib_connectib_Nodes *ptr_struct, u_int8_t* ptr_buff)
{
	connectib_sw_eqe_pack(&(ptr_struct->sw_eqe), ptr_buff);
}

void connectib_connectib_Nodes_unpack(union connectib_connectib_Nodes *ptr_struct, const u_int8_t* ptr_buff)
{
	connectib_sw_eqe_unpack(&(ptr_struct->sw_eqe), ptr_buff);
}

void connectib_connectib_Nodes_print(const union connectib_connectib_Nodes *ptr_struct, FILE* file, int indent_level)
{
	adb2c_add_indentation(file, indent_level);
	fprintf(file, "======== connectib_connectib_Nodes ========\n");
	int i = 0;
	(void)i;
	(void)ptr_struct;
	(void)file;
	(void)indent_level;

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "itoc:\n");
	connectib_itoc_print(&(ptr_struct->itoc), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "hw_hca_contexts:\n");
	connectib_hw_hca_contexts_print(&(ptr_struct->hw_hca_contexts), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "guids:\n");
	connectib_guids_print(&(ptr_struct->guids), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_cqe:\n");
	connectib_sw_cqe_print(&(ptr_struct->sw_cqe), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "icmd:\n");
	connectib_icmd_print(&(ptr_struct->icmd), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "a2d:\n");
	connectib_a2d_print(&(ptr_struct->a2d), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "ini:\n");
	connectib_ini_print(&(ptr_struct->ini), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_wqe:\n");
	connectib_sw_wqe_print(&(ptr_struct->sw_wqe), file, indent_level + 1);

	adb2c_add_indentation(file, indent_level);
	fprintf(file, "sw_eqe:\n");
	connectib_sw_eqe_print(&(ptr_struct->sw_eqe), file, indent_level + 1);

}

int connectib_connectib_Nodes_size(void)
{
	 return 65536;
}

void connectib_connectib_Nodes_dump(const union connectib_connectib_Nodes *ptr_struct, FILE* file)
{
	connectib_connectib_Nodes_print(ptr_struct, file, 0);
}

